#!/bin/bash

 ##############################################################################
 # Filename: bash/license.sh
 #
 # This file is part of the JAS project.
 # Copyright 2003-2013 by Matthijs Schuurman
 #
 # The JAS project is free software under the terms of the
 # GNU Lesser General Public License version 3 or any later version.
 #
 # For information about the license please refer to:
 # LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 ##############################################################################


cd "`dirname $0`/../"
. ./bash/functions.sh

title "License updater" "0.2"


if [ "$1" != "" ]; then
 DIRS="$1"
else
 DIRS="license"
fi

if [ "$DIRS" == "jas" ]; then
 DIRS="./"
else
 DIRS="`getDirectories \"$DIRS\"`"
fi


CURDIR=`pwd`
for DIR in $DIRS; do
{
 if [ "$DIR" == "./" ]; then #JAS
  TITLE="JAS"
  LICENSEFILE="index.php"
 else #plugin
  TITLE=`echo $DIR | perl -pane 's!.*?\/([^/]+)/?$!\1!'`
  LICENSEFILE="${TITLE}.php"
 fi

 action "Finding $TITLE script files"
 if [ "$DIR" == "./" ]; then #JAS
  FILES=(`getFiles jas ".*\.\(php\|js\|css\|sh\)$"`)
 else
  FILES=(`getFiles "$DIR" ".*\.\(php\|js\|css\|sh\)$"`)
 fi

 if [ ${#FILES[@]} -eq 0 ]; then
  status failed "No files found"
  cd "$CURDIR"
  continue;
 fi
 status done


 cd "$DIR"

 action "Getting $TITLE license text" "$LICENSEFILE"
 LICENSE=`getLicense "$LICENSEFILE"`
 status $? "$LICENSE" #error hangling
 if [ $? == 1 ]; then
  cd "$CURDIR"
  continue;
 fi


 #update license
 TOTAL=${#FILES[@]}
 action "Updating license in $TITLE script files" "$TOTAL files"

 for ((C=0;C<$TOTAL;C++)) ; do
 {
  progress $C $TOTAL

  FILE=`echo "${FILES[$C]}" | perl -pane "s!$DIR/*!!"` #remove dir (./, ./project/jas/include/plugin/skeleton, etc) from file

  if [ "$FILE" == "$LICENSEFILE" ]; then continue ; fi #skip license file
  if [ "`echo $FILE | grep \"plugins/skeleton\"`" != "" ]; then continue ; fi #skip skeleton plugin files

  LICENSENEW=`echo "$LICENSE" | perl -pane "s!$LICENSEFILE!$FILE!"` #replace filename in license

  #update license in file
  setLicense "$FILE" "$LICENSENEW"
  if [ $? != 0 ]; then
   status failed "$OUTPUT" #error hangling
  fi
 }
 done

 progress $TOTAL $TOTAL
 echo -en "\b\b\b\b" #remove 100%
 status done
 echo

 cd "$CURDIR"
}
done

