<?

 /*****************************************************************************
 * Filename: image.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 if(!isset($_path["root"]))
  $_path["root"]="";

 require($_path["root"]."include.php");

 //hack for JAS project, switch to correct jses+path
 if($_jses["project"]=="jas"&&isset($_jses["currentProject"])&&$_jses["currentProject"]!="jas")
 {
  $_jasPath=$_path;
  require($_jasPath["project"]["include"]."switch.php"); //currentProject
 }

 ini_set("display_errors", 0); //don't show PHP errors in API

 require_once($_path["include"]."data.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."image.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."dt.php");

 $e=new Error();

 //select module
 if($module=strip_tags(request("m")))
  $d=new Data($module, "view", "item");
 else //use images module by default
  $d=new Data("images", "view", "item");

 //identifier
 $d->setValue("nr", request("nr"));

 //module field to display
 if($field=request('field'))
 {
  if(!$d->check("type", "image", $field)&&!$d->check("type", "localfile", $field))
   $e->add("Incorrect image field for module: \"".$module."\"", "hack");
 }
 else //autodetect
 {
  //get first image field
  foreach($d->check("type", "image") as $field)
   break;

  if(!$field)
  {
   //get first local file field
   foreach($d->check("type", "localfile") as $field)
    break;

   if(!$field)
   {
    $e->add("No image field for module: \"".$module."\"", "hack");
    $field=false;
   }
  }
 }

 //force width/height numbers
 $width=abs(request("width"));
 $height=abs(request("height"));

 //get rotate
 if($rotate=request('rotate'))
 {
  if(!preg_match("/^[0-9a-z]+$/i", $rotate)) //must be alphanumeric
  {
   $e->add("Illegal rotation", "hack");
   $rotate=false;
  }
 }

 if(isset($_GET["nocache"])||isset($_GET["redirect"])||(!$width&&!$height&&!$rotate)) //no caching or reason to cache
  $cache=false;
 else //caching
 {
  //get field and last updated (not the data)
  $d->setShowFields($field, "updated");

  if(!$d->get()||!$d->next()) //no image returned... prolly will error later on as well
   $cache=false;
  else //see if cached version exists
  {
   //cache file name
   $cachefile=$d->module."-nr-".$d->getValue("nr")."-".$field."-".$width."x".$height;

   if($rotate) //add (optional) rotation to cache file name
    $cachefile.="_".$rotate;

   if(preg_match("/(\.[a-z]+)$/i", $d->v[$field], $tmp)) //added extension
    $cachefile.=strtolower($tmp[1]);

   $cachetime=DT::dateConvert($d->v["updated"], $d->format[$d->dataformat]["datetimeformat"]);

   $cache=new Cache("file");
   if($cache->time($cachefile)==$cachetime) //cached, output and exit
   {
    print $cache->output($cachefile);
    exit;
   }
   else if($cache->exists($cachefile)) //cached version exist
    $cache->remove($cachefile); //remove so it can be updated
  }

  //no cache available

  $d->clearValues();
  $d->setValue("nr", request("nr"));
 }

 //failure?
 if($e->any("fatal"))
  imgerror($width, $height);

 //load image from data object
 $image=new Image();

 if(isset($_GET["redirect"]))
 {
  if(!$image->redirectData($d, $field)) //error
  {
   $e->add($image->e);
   imgerror($width, $height);
  }

  exit; //done
 }
 else
 {
  if(!$image->loadData($d, $field)) //error
  {
   $e->add($image->e);
   imgerror($width, $height);
  }
 }

 //resize
 if($width||$height)
 {
  if(!$image->resize($width, $height))
   $width=$height=0;
 }

 //rotate (after resize to speed it up)
 if($rotate)
 {
  if(!$image->rotate($rotate))
   $rotate=false;
  else if($width||$height) //resize again to correct size
  {
   if(!$image->resize($width, $height))
    $width=$height=0;
  }
 }

 //output
 if($content=$image->output()) //get output
 {
  if($cache&&($width||$height||$rotate)) //cache if necessary
   $cache->save($cachefile, $content, $cachetime);

  print $content; //output content and exit
  exit;
 }

 //error generating image
 $e->add($image->e);
 imgerror($width, $height);


 /*
  * error function, displays imagenotfound.png
  */
 function imgerror($width, $height)
 {
  global $_path;

  $image=new Image();
  $image->load($_path["images"]."imagenotfound.png");

  if($width||$height)
   $image->resize($width, $height);

  print $image->output();
  exit;
 }
?>
