<?

 /*****************************************************************************
 * Filename: include/archive.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 @include_once("Archive/Tar.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."fs.php");

 class Archive
 {
  function Archive()
  {
   global $_path;

   $this->e=new Error();
   $this->e->l->read("archive");

   $this->files=Array();
   $this->string="";

   $this->methods=Array("TAR", "TAR.GZ", "TGZ", "TAR.BZ2", "GZ", "BZ2", "RAR", "ZIP");
   $this->tmpfile=$_path["project"]["include"]."tmp.jas"; //better to use a global/project tmp dir
  }

  function _detectMethod($file)
  {
   if(preg_match("/\.([a-z0-9]+\.[a-z0-9]+)$/i", $file, $tmp)&&in_array(strtoupper($tmp[1]), $this->methods))
    return strtoupper($tmp[1]);

   if(preg_match("/\.([a-z0-9]+)$/i", $file, $tmp)&&in_array(strtoupper($tmp[1]), $this->methods))
    return strtoupper($tmp[1]);

   $this->e->add("Unabled to detect compression method for file: \"".$file."\"");
   return false;
  }

  function _checkSupport($method)
  {
   if(!in_array($method, $this->methods))
   {
    $this->e->add("Unknown compression method: \"".$method."\"");
    return false;
   }

   switch($method)
   {
    case "TAR":
    {
     if(!class_exists("Archive_Tar"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      $this->e->add("Tar support needs Pear Archive_Tar: http://pear.php.net/package/Archive_Tar/", "warning");
      return false;
     }

     break;
    }
    case "TGZ":
    case "TAR.GZ":
    {
     if(!$this->_checkSupport("TAR")||!$this->_checkSupport("GZ"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      return false;
     }
     break;
    }
    case "TAR.BZ2":
    {
     if(!$this->_checkSupport("TAR")||!$this->_checkSupport("BZ2"))
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      return false;
     }
     break;
    }
    case "GZ":
    {
     if(
      !function_exists("gzopen")||
      !function_exists("gzread")||
      !function_exists("gzwrite")||
      !function_exists("gzclose")
     )
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      return false;
     }

     break;
    }
    case "BZ2":
    {
     if(
      !function_exists("bzopen")||
      !function_exists("bzread")||
      !function_exists("bzwrite")||
      !function_exists("bzclose")
     )
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      return false;
     }

     break;
    }
    case "RAR":
    {
     if(
      !function_exists("rar_open")||
      !function_exists("rar_read")||
      !function_exists("rar_list")||
      !function_exists("rar_entry_get")||
      !function_exists("rar_close")
     ) 
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      return false;
     }
     break;
    }
    case "ZIP":
    {
     if(
      !function_exists("zip_open")||
      !function_exists("zip_read")||
      !function_exists("zip_entry_open")||
      !function_exists("zip_entry_read")||
      !function_exists("zip_entry_filesize")||
      !function_exists("zip_entry_name")||
      !function_exists("zip_entry_close")||
      !function_exists("zip_close")
     ) 
     {
      $this->e->add("PHP doesn't have support for compression method: \"".$method."\"");
      return false;
     }
     break;
    }
    default:
    {
     $this->e->add("Can't detect PHP compression support for compression method: \"".$method."\"");
     return false;
    }
   }

   return true;
  }

  function open($file, $method=false)
  {
   if($this->e->any("error"))
    return false;

   if(!file_exists($file))
   {
    $this->e->add("File doesn't exist: \"".$file."\"");
    return false;
   }

   if(!$method&&!($method=$this->_detectMethod($file)))
    return false;

   if(!$this->_checkSupport($method))
    return false;

   switch($method)
   {
    case "TAR":
    {
     $tar=new Archive_Tar($file);
     if(!($files=$tar->listContent()))
     {
      $this->e->add("No files in TAR archive: \"".$file."\"");
      return false;
     }
     
     foreach($files as $entry)
     {
      $archivefile=$entry["filename"];
      $this->_addFilePath($archivefile);
      
      $this->files[$archivefile]=Array();
      $this->files[$archivefile]["time"]=$entry["mtime"];
      $this->files[$archivefile]["size"]=$entry["size"];

      if($this->files[$archivefile]["size"]==0)
       $this->files[$archivefile]["type"]="directory";
      else
      {
       $this->files[$archivefile]["type"]="file";
       $this->files[$archivefile]["contents"]=$tar->extractInString($archivefile);
      }
     }

     break;    
    }
    case "TGZ":
    case "TAR.GZ":
    {
     if(!$this->open($file, "GZ"))
      return false;

     if(!($fp=@fopen($this->tmpfile, "w")))
     {
      $this->e->add("Unable to open temporary file: \"".$this->tmpfile."\" for compression method: \"".$method."\"");
      return false;
     }
     fwrite($fp, $this->string);
     fclose($fp);

     if(!$this->open($this->tmpfile, "TAR"))
     {
      $this->e->clear(); //make sure TAR errors are not shown
      $this->e->add("Unable to open archive: \"".$file."\" for compression method: \"".$method."\"");
      unlink($this->tmpfile);
      return false;
     }

     unlink($this->tmpfile);

     break;    
    }
    case "TAR.BZ2":
    {
     if(!$this->open($file, "BZ2"))
      return false;

     if(!($fp=@fopen($this->tmpfile, "w")))
     {
      $this->e->add("Unable to open temporary file: \"".$this->tmpfile."\" for compression method: \"".$method."\"");
      return false;
     }
     fwrite($fp, $this->string);
     fclose($fp);

     if(!$this->open($this->tmpfile, "TAR"))
     {
      $this->e->clear(); //make sure TAR errors are not shown
      $this->e->add("Unable to open archive: \"".$file."\" for compression method: \"".$method."\"");
      unlink($this->tmpfile);
      return false;
     }

     unlink($this->tmpfile);

     break;    
    }
    case "GZ":
    {
     if(!($gz=@gzopen($file, "rb")))
     {
      $this->e->add("Unable to open file: \"".$file."\", error: ".$archive);
      return false;
     }

     $contents=""; 
     while($tmp=gzread($gz, 1024))
      $contents.=$tmp;

     gzclose($gz);

     $this->setString($contents);

     break;
    }
    case "BZ2":
    {
     if(!($bz=@bzopen($file, "r")))
     {
      $this->e->add("Unable to open file: \"".$file."\", error: ".$archive);
      return false;
     }

     $contents=""; 
     while($tmp=bzread($bz, 1024))
      $contents.=$tmp;

     bzclose($bz);

     $this->setString($contents);

     break;
    }
    case "RAR":
    {
     if(!($archive=@rar_open($file)))
     {
      $this->e->add("Unable to open file: \"".$file."\"");
      return false;
     }

     foreach(rar_list($archive) as $entry)
     {
      $archivefile=$entry->getName();
      $this->_addFilePath($archivefile);

      $this->files[$archivefile]=Array();
      $this->files[$archivefile]["time"]=$entry->getFileTime();
      $this->files[$archivefile]["contents"]="";
      $this->files[$archivefile]["size"]=$entry->getUnpackedSize();

      if($this->files[$archivefile]["size"]==0)
       $this->files[$archivefile]["type"]="directory";
      else
      {
       $this->files[$archivefile]["type"]="file";

       if($entry->extra("./", $this->tmpfile))
       {
        $this->files[$archivefile]["contents"]=file_get_contents($this->tmpfile);
        unlink($this->tmpfile);
       }
      }
     }

     rar_close($archive);

     break;
    }
    case "ZIP":
    {
     $archive=@zip_open($file);

     if(is_int($archive))
     {
      $this->e->add("Unable to open file: \"".$file."\", error: ".$archive);
      return false;
     }

     while($entry=zip_read($archive))
     {
      if(!zip_entry_open($archive, $entry))
      {
       $this->e->add("Unable to open entry from file: \"".$file."\"");
       return false;
      }

      $archivefile=zip_entry_name($entry);
      $this->_addFilePath($archivefile);

      $this->files[$archivefile]=Array();
      $this->files[$archivefile]["time"]=time();
      $this->files[$archivefile]["contents"]="";
      $this->files[$archivefile]["size"]=zip_entry_filesize($entry);

      if($this->files[$archivefile]["size"]==0)
       $this->files[$archivefile]["type"]="directory";
      else
      {
       $this->files[$archivefile]["type"]="file";
       while($contents=zip_entry_read($entry))
        $this->files[$archivefile]["contents"].=$contents;
      }

      zip_entry_close($entry);
     }

     zip_close($archive);

     break;
    }
    default:
    {
     $this->e->add("Can't open archive: \"".$file."\" with compression method: \"".$method."\"");
     return false;
    }
   }

   return true;
  }

  function _addFilePath($file)
  {
   if(preg_match_all("!([^/]+/)!", $file, $tmp))
   {
    $dir="";
    foreach($tmp[1] as $tmp2)
    {
     $dir.=$tmp2;
     if(!isset($this->files[$dir]))
     {
      $this->files[$dir]=Array();
      $this->files[$dir]["time"]=time();
      $this->files[$dir]["size"]=0;
      $this->files[$dir]["type"]="directory";
     }
    }
   }
   unset($tmp, $dir);
  }

  function getFiles()
  {
   if($this->e->any("error"))
    return false;

   return array_keys($this->files);
  }

  function rerootFiles($oldroot, $newroot=false)
  {
   if($this->e->any("error"))
    return false;

   if(!$oldroot||$oldroot=="./")
    $oldroot="";
   else if(substr($oldroot, -1)!="/")
    $oldroot.="/";

   if(!$newroot)
    $newroot="";
   else if(substr($newroot, -1)!="/")
    $newroot.="/";

   $remove=Array();
   if(preg_match_all("!([^/]+/)!", $oldroot, $tmp))
   {
    $dir="";
    foreach($tmp[1] as $tmp2)
    {
     $dir.=$tmp2;
     $remove[]=$dir;
     if(!isset($this->files[$dir]))
     {
      $this->e->add("Unabled to re-root, root: \"".$dir."\" doesn't exist in archive");
      return false;
     }
    }
   }

   $oldrootregexp="!^".preg_quote($oldroot, "!")."!";

   foreach($this->getFiles() as $file)
   {
    if(!in_array($file, $remove)&&!preg_match($oldrootregexp, $file))
    {
     $this->e->add("Unabled to re-root file: \"".$file."\" outside of root: \"".$oldroot."\"");
     return false;
    }
   }

   foreach($remove as $dir)
    unset($this->files[$dir]);

   if($newroot)
   {
    $this->files[$newroot]=Array();
    $this->files[$newroot]["time"]=time();
    $this->files[$newroot]["size"]=0;
    $this->files[$newroot]["type"]="directory";
   }

   foreach($this->getFiles() as $file)
   {
    if($file==$newroot)
     continue;

    $newfile=$newroot.preg_replace($oldrootregexp, "", $file);
    $this->files[$newfile]=&$this->files[$file];
    unset($this->files[$file]);
   }

   return true;
  }

  function saveFiles($dir=false, $overwrite=false)
  {
   if($this->e->any("error"))
    return false;

   $return=true;
   foreach($this->getFiles() as $file)
    if(!$this->saveFile($file, $dir, $overwrite))
     $return=false;

   return $return;
  }

  function saveFile($file, $dir=false, $overwrite=false)
  {
   if($this->e->any("error"))
    return false;

   if(!$dir)
    $dir="./";
   else if(substr($dir, -1)!="/")
    $dir.="/";

   if(!file_exists($dir))
   {
    if(!@mkdir($dir, 0755, true))
    {
     $this->e->add("Unabled to create destination directory: \"".$dir."\"");
     return false;
    }
   }

   if(!isset($this->files[$file]))
   {
    $this->e->add("File: \"".$file."\" not in archive");
    return false;
   }

   if(!$overwrite&&file_exists($dir.$file))
   {
    $this->e->add("File: \"".$dir.$file."\" already exists");
    return false;
   }

   if($this->files[$file]["type"]=="directory") 
   {
    if(!file_exists($dir.$file)) //don't try to overwrite it
    {
     if(!@mkdir($dir.$file, 0755, true))
     {
      $this->e->add("Unabled to create archive directory: \"".$dir.$file."\"");
      return false;
     }
     else
      $this->e->add("Archive directory: \"".$dir.$file."\" created", "note");
    }
    else
     $this->e->add("Not creating archive directory: \"".$dir.$file."\"", "note");
   }
   else
   {
    if(file_exists($dir.$file)) 
     $overwritting=true;
    else
     $overwritting=false;

    if(@file_put_contents($dir.$file, $this->files[$file]["contents"])===false)
    {
     $this->e->add("Unabled to write archive file: \"".$dir.$file."\"");
     return false;
    } 

    if(!@touch($dir.$file, $this->files[$file]["time"]))
    {
     $this->e->add("Unabled to change file time for file: \"".$dir.$file."\"");
     return false;
    } 

    if(filesize($dir.$file)!=$this->files[$file]["size"])
    {
     $this->e->add("File size of file: \"".$dir.$file."\" (".filesize($dir.$file).") differs from original (".$this->files[$file]["size"].")");
     return false;
    } 

    if($overwritting)
     $this->e->add("File: \"".$dir.$file."\" overwritten", "note");
    else
     $this->e->add("File: \"".$dir.$file."\" saved", "note");
   }

   return true;
  }

  function addDir($dir=false, $strippath=false)
  {
   if($this->e->any("error"))
    return false;

   if(!$dir)
    $dir="./";
   else if(substr($dir, -1)!="/")
    $dir.="/";

   if($strippath===true)
    $strippath=$dir;

   if(!file_exists($dir))
   {
    $this->e->add("Directory: \"".$dir."\" doesn't exist");
    return false;
   }

   $return=true;
   foreach(FS::recurseDir($dir) as $file)
    if(!$this->addFile($dir.$file, $strippath))
     $return=false;

   return $return;
  }

  function addFile($file, $strippath=false)
  {
   if($this->e->any("error"))
    return false;

   if(is_dir($file)&&substr($file, -1)!="/")
    $file.="/";

   if($strippath)
    $archivefile=preg_replace("!".preg_quote($strippath, "!")."!", "", $file);
   else
    $archivefile=$file;

   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exists");
    return false;
   }

   if(isset($this->files[$archivefile]))
   {
    $this->e->add("File: \"".$archivefile."\" already in archive");
    return false;
   }

   if(preg_match("!^(.+)/.+$!", $archivefile, $tmp))
   {
    $dir=$tmp[1]."/";
    if(!isset($this->files[$dir]))
    {
     $this->addFile($strippath.$dir, $strippath);
    }
   }

   $this->files[$archivefile]=Array();
   if(is_dir($file)) //directory
   {
    $this->files[$archivefile]["size"]=0;
    $this->files[$archivefile]["contents"]="";
    $this->files[$archivefile]["type"]="directory";
   }
   else
   {
    $this->files[$archivefile]["size"]=filesize($file);
    $this->files[$archivefile]["contents"]=file_get_contents($file);
    $this->files[$archivefile]["type"]="file";
   }

   return true;
  }

  function setString($string)
  {
   if($this->e->any("error"))
    return false;

   $this->string=$string;

   return true;
  }

  function getString()
  {
   if($this->e->any("error"))
    return false;

   return $this->string;
  }

  function send($file, $method=false)
  {
   if($this->e->any("error"))
    return false;

   if(!$method&&!($method=$this->_detectMethod($file)))
    return false;

   if(!$this->_checkSupport($method))
    return false;

   switch($method)
   {
    case "TAR":
    {
     $type="application/x-tar";
     break;
    }
    case "GZ":
    case "TGZ":
    case "TAR.GZ":
    {
     $type="application/x-gzip";
     break;
    }
    case "BZ2":
    case "TAR.BZ2":
    {
     $type="application/bz2";
     break;
    }
    case "RAR":
    {
     $type="application/x-rar-compressed";
     break;
    }
    case "ZIP":
    {
     $type="application/zip";
     break;
    }
    default:
    {
     $this->e->add("Sending files is not support for compression method: \"".$method."\"");
     return false;
    }
   }

   if(!($contents=&$this->output($method)))
    return false;

   header("Cache-Control: no-cache, no-store, max-age=0, must-revalidate");
   header("Expires: Thu, 01 Jan 1970 00:00:00 GMT"); 
   header("Pragma: no-cache");

   header("Content-type: ".$type);
   header("Content-Disposition: attachment; filename=\"".$file."\""); 

   header("Content-Length: ".strlen($contents));

   print $contents;

   return true;
  }

  function save($file, $method=false)
  {
   if($this->e->any("error"))
    return false;

   if(!$method&&!($method=$this->_detectMethod($file)))
    return false;

   if(!$this->_checkSupport($method))
    return false;

   if(file_exists($file))
   {
    $this->e->add("File: \"".$file."\" already exists");
    return false;
   }

   if(!($contents=&$this->output($method)))
    return false;

   if(!@file_put_contents($file, $contents))
   {
    $this->e->add("Can't write compressed data to: \"".$file."\" because it already exists");
    return false;
   }

   return true;
  }

  function output($method)
  {
   if($this->e->any("error"))
    return false;

   if(!$this->_checkSupport($method))
    return false;

   switch($method)
   {
    case "TAR":
    {
     if(!$this->files)
     {
      $this->e->add("No files set for archiving");
      return false;
     }

     $tar=new Archive_Tar($this->tmpfile);
     foreach($this->getFiles() as $file)
     {
      if($this->files[$file]["type"]=="file")
      {
       if(!$tar->addString($file, $this->files[$file]["contents"]))
       {
        $this->e->add("Can't create: \".$file.\" in archive for compression method: \"".$method."\"");
        unlink($this->tmpfile);
        return false;
       }
      }
     }

     $output=file_get_contents($this->tmpfile);
     unlink($this->tmpfile);

     break;
    }
    case "TGZ":
    case "TAR.GZ":
    {
     $output=$this->output("TAR");
     $this->setString($output);
     $output=$this->output("GZ");

     break;
    }
    case "TAR.BZ2":
    {
     $output=$this->output("TAR");
     $this->setString($output);
     $output=$this->output("BZ2");

     break;
    }
    case "GZ":
    {
     if(!$this->string)
     {
      $this->e->add("No string set for archiving");
      return false;
     }

     if(!($gz=gzopen($this->tmpfile, "wb")))
     {
      $this->e->add("Unable to open temporary file: \"".$this->tmpfile."\" for compression method: \"".$method."\"");
      return false;
     }
     gzwrite($gz, $this->string);
     gzclose($gz);

     $output=file_get_contents($this->tmpfile);
     unlink($this->tmpfile);

     break;
    }
    case "BZ2":
    {
     if(!$this->string)
     {
      $this->e->add("No string set for archiving");
      return false;
     }

     if(!($bz=bzopen($this->tmpfile, "w")))
     {
      $this->e->add("Unable to open temporary file \"".$this->tmpfile."\" for compression method: \"".$method."\"");
      return false;
     }
     bzwrite($bz, $this->string);
     bzclose($bz);

     $output=file_get_contents($this->tmpfile);
     unlink($this->tmpfile);

     break;
    }
    default:
    {
     $this->e->add("Can't create output for compression method: \"".$method."\"");
     return false;
    }
   }

   return $output;
  }
 }
?>
