<?

 /*****************************************************************************
 * Filename: include/benchmark.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 class Benchmark
 {
  function Benchmark()
  {
   $this->c=0;
   $this->parent=false;
   $this->markers=Array();
   $this->precision=3;

   $this->start=microtime(true);
  }

  function start($name, $hilite=false)
  {
   $this->markers[$this->c]=Array();
   $this->markers[$this->c]["name"]=$name;
   $this->markers[$this->c]["start"]=microtime(true)-$this->start;
   $this->markers[$this->c]["parent"]=$this->parent;
   $this->markers[$this->c]["hilite"]=$hilite;

   $this->parent=$this->c;
   $this->c++;
  }

  function stop()
  {
   for($c=$this->c-1;$c>=0;$c--)
   {
    if(!isset($this->markers[$c]["time"]))
    {
     $this->markers[$c]["stop"]=microtime(true)-$this->start;
     $this->markers[$c]["time"]=$this->markers[$c]["stop"]-$this->markers[$c]["start"];
     $this->parent=$this->markers[$c]["parent"];
     return;
    }
   }
  }

  function show()
  {
   $this->stop=microtime(true);
   $this->time=$this->stop-$this->start;

   for($c=0;$c<$this->c;$c++)
   {
    if(!isset($this->markers[$c]["time"]))
    {
     $this->markers[$c]["stop"]=$this->stop-$this->start;
     $this->markers[$c]["time"]=$this->markers[$c]["stop"]-$this->markers[$c]["start"];
    }
   }

   $r="<div class=\"benchmark\">";
   $r.="<div>Benchmark</div>";
   $r.="<table>";
   $r.="<tr class=\"header\"><td class=\"marker\">Marker</td><td class=\"start\">Start</td><td class=\"stop\">Stop</td><td class=\"time\">Time</td><td class=\"percentage\">Percentage</td></tr>";

   for($c=0;$c<$this->c;$c++)
   {
    $r.="<tr".($this->markers[$c]["hilite"]?" style=\"background: black; color: white; font-weight: bold;\"":"").">";
    $r.="<td>".implode("->", $this->getTree($c))."</td>";
    $r.="<td>".number_format($this->markers[$c]["start"], $this->precision)."</td>";
    $r.="<td>".number_format($this->markers[$c]["stop"], $this->precision)."</td>";
    $r.="<td>".number_format($this->markers[$c]["time"], $this->precision)."</td>";
    $r.="<td>".number_format(($this->markers[$c]["time"]/$this->time)*100, 2)."%</td>";
    $r.="</tr>\n";
   }

   $r.="<tr class=\"total\">";
   $r.="<td>Total</td>";
   $r.="<td>".number_format(0, $this->precision)."</td>";
   $r.="<td>".number_format($this->time, $this->precision)."</td>";
   $r.="<td>".number_format($this->time, $this->precision)."</td>";
   $r.="<td>100.00%</td>";
   $r.="</tr>\n";

   $r.="</table>\n";
   $r.="</div>\n";

   return $r;
  }

  function getTree($id)
  {
   $tree=Array($this->markers[$id]["name"]);

   $parent=$id;
   while(($parent=$this->markers[$parent]["parent"])!==false)
    $tree[]=$this->markers[$parent]["name"];

   return array_reverse($tree);
  }
 }

?>
