<?

 /*****************************************************************************
 * Filename: include/cache.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."config.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."mime.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."fs.php");

 class Cache extends Config
 {
  function Cache($type, $config=false)
  {
   global $_path, $_config;

   Config::Config();
   $this->merge("Cache", $config);

   $this->moduleCaching=$this->config["module"];
   $this->moduleCleanup=$this->config["moduleCleanup"];

   $this->fileCaching=$this->config["file"];
   $this->filePublic=$this->config["filePublic"];
   $this->fileCleanup=$this->config["fileCleanup"];

   if(isset($this->config["memory"]))
   {
    $this->memoryCaching=$this->config["memory"];
    $this->memoryMethod=$this->config["memoryMethod"];
    $this->memoryConnection=$this->config["memoryConnection"];
    $this->memoryCleanup=$this->config["memoryCleanup"];
    $this->memoryObject=false;
   }
   else
    $this->memoryCaching=false;

   $this->type=false;

   $this->mime=false;

   $this->e->l->read("cache");

   if($type=="file")
   {
    if(!$this->fileCaching)
    {
     $this->e->add("File based caching not enable in configuration", "error");
     return false;
    }

    $this->fs=new FS();
    $this->e=&$this->fs->e; //just use FS error object

    if(!$this->fs->exists($_path["project"]["cache"]))
    {
     if(!$this->fs->mkdir($_path["project"]["cache"]))
      return false;
    }
    else if(!is_dir($_path["project"]["cache"]))
    {
     $this->e->add("Cache directory: \"".$_path["project"]["cache"]."\" isn't a directory");
     return false;
    }

    $this->type=$type;

    if($this->filePublic) //public cache folder
    {
     if($this->fs->exists($_path["project"]["cache"].".htaccess"))
      if(!$this->fs->delete($_path["project"]["cache"].".htaccess")) //allow web-access
       return false;
    }
    else //non public
    {
     if(!$this->fs->exists($_path["project"]["cache"].".htaccess"))
      if(!$this->fs->copy($_path["include"].".htaccess", $_path["project"]["cache"].".htaccess")) //deny web-access
       return false;
    }

    if($this->fileCleanup&&isset($_config["cache"]))
    {
     if(!isset($_config["cache"]["fileCleanupNext"])||$_config["cache"]["fileCleanupNext"]<=time()) //next auto clean up?
     {
      $this->cleanup($this->fileCleanup);  //clean up everything older the $fileCleanup seconds
      $_config["cache"]["fileCleanupNext"]=time()+$this->fileCleanup; //next auto clean up $fileCleanup seconds
     }
    }
   }
   else if($type=="module")
   {
    if(!$this->moduleCaching)
    {
     $this->e->add("Module based caching not enable in configuration", "error");
     return false;
    }

    $this->d=Data::cache("cache", "view", "item", Array("format" => "sql")); //use SQL data format: no convertion
    $this->d->clearValues();

    if($this->d->e->any("fatal"))
    {
     $this->e->add($this->d->e);
     return false;
    }

    $this->d2=false; //init at save / remove
    $this->type=$type;

    if($this->moduleCleanup&&isset($_config["cache"]))
    {
     if(!isset($_config["cache"]["moduleCleanupNext"])||$_config["cache"]["moduleCleanupNext"]<=time()) //next auto clean up?
     {
      $this->cleanup($this->moduleCleanup);  //clean up everything older the $moduleCleanup seconds
      $_config["cache"]["moduleCleanupNext"]=time()+$this->moduleCleanup; //next auto clean up $moduleCleanup seconds
     }
    }
   }
   else if($type=="memory")
   {
    if(!$this->memoryCaching)
    {
     $this->e->add("Memory based caching not enable in configuration", "error");
     return false;
    }

    if($this->memoryMethod=="Memcache")
    {
     if(preg_match("/^([a-z0-9_\-\.]+):(\d+)$/i", $this->memoryConnection, $tmp)) //domain:port
     {
      $server=$tmp[1];
      $port=$tmp[2];
     }
     else //socket filename
     {
      $server=$this->memoryConnection;
      $port=0;
     }

     if($this->memoryMethod=="Memcache")
     {
      $this->memoryObject=new Memcache();

      if(!$port) //local server
      {
       if(!@$this->memoryObject->pconnect("unix://".$server, $port)) //persistent connection, not closed when object is destroyed
       {
        $this->e->add("Unable to connect to local Memcached server via: \"".$server."\"", "error");
        $this->memoryObject=false;
        return false;
       }
      }
      else
      {
       if(!@$this->memoryObject->pconnect($server, $port)) //persistent connection, not closed when object is destroyed
       {
        $this->e->add("Unable to connect to Memcached server: \"".$server."\" on port: ".$port, "error");
        $this->memoryObject=false;
        return false;
       }
      }
     }
     else
     {
      $this->memoryObject=new Memcached("Cache"); //persistent connection

      d($this->memoryObject->getServerList());
      if(!@$this->memoryObject->addServer($server, $port)||!@$this->memoryObject->set("cacheTest", 1)) //add server and test connection
      {
       if(!$port) //local server
        $this->e->add("Unable to connect to local Memcached server via: \"".$server."\"", "error");
       else
        $this->e->add("Unable to connect to Memcached server: \"".$server."\" on port: ".$port, "error");

       $this->memoryObject->resetServerList();
       $this->memoryObject=false;
       return false;
      }

      $this->memoryObject->delete("cacheTest");
     }

     $this->type=$type;
    }
    else
    {
     $this->e->add("Memory caching method: \"".$this->memoryMethod."\" not supported", "error");
     return false;
    }
   }
   else
   {
    $this->e->add("Unknown cache type: \"".$type."\"");
    return false;
   }
  }


  function exists($key, $log=true)
  {
   global $_path;

   if(!$key)
   {
    $this->e->add("Cache key empty");
    return false;
   }

   if($key[0]==".")
   {
    $this->e->add("Cache key may not begin with a \".\"");
    return false;
   }

   if($this->type=="file")
   {
    if($this->fs->exists($_path["project"]["cache"].$key))
     return true;
   }
   else if($this->type=="module")
   {
    if($key==$this->d->getValue("key")) //already got full entry, don't bother getting again
     return true;

    $this->d->clearValues();
    $this->d->setValue("key", $key);

    if(!$this->d->get()) //only count entries
    {
     $this->e->add($this->d->e);
     return false;
    }

    if($this->d->vmax) //any entries?
     return true;
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     if(@$this->memoryObject->get($key)!==false&&@$this->memoryObject->get($key."_accessed")!==false)
      return true;
    }
   }
   else
   {
    $this->e->add("Unknown cache type: \"".$this->type."\"");
    return false;
   }

   if($log)
    $this->e->add("Cache key: \"".$key."\" doesn't exist");

   return false;
  }

  function save($key, $value, $time=false)
  {
   global $_path;

   if(!$key)
   {
    $this->e->add("Cache key empty");
    return false;
   }

   if($key[0]==".")
   {
    $this->e->add("Cache key may not begin with a \".\"");
    return false;
   }

   if($this->type=="file")
   {
    if(!is_scalar($value))
    {
     $this->e->add("Only scalar values allowed for ".$this->type." based caching for key: \"".$key."\"");
     return false;
    }

    if($this->fs->exists($_path["project"]["cache"].$key))
    {
     $this->e->add("Cache key: \"".$key."\" already exists");
     return false;
    }

    if(!$this->fs->write($_path["project"]["cache"].".".$key, $value)) //write temporary file
     return false;

    if(!$this->fs->rename($_path["project"]["cache"].".".$key, $_path["project"]["cache"].$key)) //rename temporary file to final destination
     return false;

    if($time!==false)
     if(!$this->fs->setTime($_path["project"]["cache"].$key, $time, "modified"))
      return false;
   }
   else if($this->type=="module")
   {
    if(!is_scalar($value))
    {
     $this->e->add("Only scalar values allowed for ".$this->type." based caching for key: \"".$key."\"");
     return false;
    }

    if(!$this->d2)
     $this->d2=$this->d->copy();

    $this->d2->setAction("add");
    $this->d2->setExtra("item");

    $this->d2->clearValues();
    $this->d2->setValue("key", $key);
    $this->d2->setValue("value", $value);

    if($time!==false)
     $this->d2->setValue("modified", date($this->d2->field["modified"]["format"][$this->d2->dataformat]["datetimeformat"], $time));

    if(!$this->d2->submit())
    {
     $this->e->add($this->d2->e);
     return false;
    }
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if(!$time)
     $time=time();

    if(!is_scalar($value))
    {
     $type=2; //serialized
     $value=serialize($value);
    }
    else
     $type=1;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     if( //store key, time and value
      !@$this->memoryObject->set($key, $type)||
      !@$this->memoryObject->set($key."_accessed", time())||
      !@$this->memoryObject->set($key."_modified", $time)||
      !@$this->memoryObject->set($key."_size", strlen($value))||
      !@$this->memoryObject->set($key."_value", $value)
     )
     {
      $this->e->add("Unable to save key: \"".$key."\" for memory method: \"".$this->memoryMethod."\"", "error");
      return false;
     }
    }
   }
   else
    return false;

   return true;
  }

  function read($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
    return $this->fs->read($_path["project"]["cache"].$key);
   else if($this->type=="module") //content already gathered in exists function
   {
    //update accessed time
    if(!$this->d2)
     $this->d2=$this->d->copy();

    $this->d2->setAction("edit");
    $this->d2->setExtra("item");

    $this->d2->clearValues();
    $this->d2->setValue("key", $key);
    $this->d2->setValue("accessed", date($this->d2->field["accessed"]["format"][$this->d2->dataformat]["datetimeformat"], time()));

    if(!$this->d2->submit())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    return $this->d->getValue("value");
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     @$this->memoryObject->set($key."_accessed", time()); //update accessed time

     if(@$this->memoryObject->get($key)==2) //serialized
      return unserialize(@$this->memoryObject->get($key."_value"));
     else
      return @$this->memoryObject->get($key."_value");
    }
   }

   return false;
  }

  function time($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
    return $this->fs->getTime($_path["project"]["cache"].$key, "modified");
   else if($this->type=="module") //content already gathered in exists function
    return DT::dateConvert($this->d->getValue("modified"), $this->d->field["modified"]["format"][$this->d->dataformat]["datetimeformat"]); //convert to unix timestamp
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
     return @$this->memoryObject->get($key."_modified");
   }

   return false;
  }

  function size($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
    return $this->fs->size($_path["project"]["cache"].$key);
   else if($this->type=="module") //content already gathered in exists function
    return strlen($this->d->getValue("value"));
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
     return @$this->memoryObject->get($key."_size");
   }

   return false;
  }

  function remove($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
   {
    if(!$this->fs->delete($_path["project"]["cache"].$key))
     return false;
   }
   else if($this->type=="module")
   {
    if(!$this->d2)
     $this->d2=$this->d->copy();

    $this->d2->setAction("delete");
    $this->d2->setExtra("item");

    $this->d2->clearValues();
    $this->d2->setValue("key", $key);

    if(!$this->d2->submit())
    {
     $this->e->add($this->d2->e);
     return false;
    }
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     if( //store key, time and value
      !@$this->memoryObject->delete($key)||
      !@$this->memoryObject->delete($key."_accessed")||
      !@$this->memoryObject->delete($key."_modified")||
      !@$this->memoryObject->delete($key."_size")||
      !@$this->memoryObject->delete($key."_value")
     )
     {
      $this->e->add("Unable to remove key: \"".$key."\" for memory method: \"".$this->memoryMethod."\"", "error");
      return false;
     }
    }
   }
   else
    return false;

   return true;
  }

  function cleanup($lifeTime)
  {
   global $_path;

   $failed=false;
   $lifeTime=time()-$lifeTime;

   if($this->type=="file")
   {
    foreach($this->fs->recurseDir($_path["project"]["cache"]) as $key)
     if($this->fs->getTime($_path["project"]["cache"].$key, "accessed")<$lifeTime) //remove all files not accessed longer then lifeTime ago
      if(!$this->remove($key))
       $failed=true;
   }
   else if($this->type=="module")
   {
    if(!$this->d2)
     $this->d2=$this->d->copy();

    $this->d2->setShowFields("key");
    $this->d2->setAction("view");
    $this->d2->setExtra("all");

    $this->d2->clearValues();
    $this->d2->setValue("accessed", date($this->d2->field["accessed"]["format"][$this->d2->dataformat]["datetimeformat"], $lifeTime));
    $this->d2->setOperator("accessed", "<");

    if(!$this->d2->get())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    $keys=Array();
    while($this->d2->next())
     $keys[]=$this->d2->v["key"];

    foreach($keys as $key)
     if(!$this->remove($key))
      $failed=true;
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     foreach($this->keys() as $key)
      if($this->memoryObject->get($key."_accessed")<$lifeTime)  //remove all keys not accessed longer then lifeTime ago
       if(!$this->remove($key))
        $failed=true;
    }
   }
   else
    return false;

   if($failed)
    return false;

   return true;
  }

  function search($search)
  {
   global $_path;

   if(!$search)
   {
    $this->e->add("Cache search empty");
    return false;
   }

   if($search[0]==".")
   {
    $this->e->add("Cache search may not begin with a \".\"");
    return false;
   }

   $matches=Array();
   if($this->type=="file")
   {
    foreach($this->fs->recurseDir($_path["project"]["cache"]) as $key)
     if(stripos($key, $search)!==false)
      $matches[]=$key;
   }
   else if($this->type=="module")
   {
    if(!$this->d2)
     $this->d2=$this->d->copy();

    $this->d2->setShowFields("key");
    $this->d2->setAction("search");
    $this->d2->setExtra("title");

    $this->d2->clearValues();
    $this->d2->setValue("key", $search);
    $this->d2->setOperator("key", "like");

    if(!$this->d2->get())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    while($this->d2->next())
     $matches[]=$this->d2->v["key"];
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     foreach($this->keys() as $key)
      if(stripos($key, $search)!==false)
       $matches[]=$key;
    }
   }
   else
    return false;

   return $matches;
  }

  function keys()
  {
   global $_path;

   $matches=Array();
   if($this->type=="file")
   {
    foreach($this->fs->recurseDir($_path["project"]["cache"]) as $key)
     $matches[]=$key;
   }
   else if($this->type=="module")
   {
    if(!$this->d2)
     $this->d2=$this->d->copy();

    $this->d2->setShowFields("key");
    $this->d2->setAction("view");
    $this->d2->setExtra("all");

    $this->d2->clearValues();

    if(!$this->d2->get())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    while($this->d2->next())
     $matches[]=$this->d2->v["key"];
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->memoryMethod=="Memcache"||$this->memoryMethod=="Memcached")
    {
     if($stats=@$this->memoryObject->getStats("items")) //get stats
      foreach(array_keys($stats["items"]) as $item)
       if($keys=@$this->memoryObject->getStats("cachedump", $item)) //get cache dump per item
        foreach(array_keys($keys) as $key) //validate each key
         if(@$this->memoryObject->get($key."_accessed")) //check if also $key_accessed exists
          $matches[]=$key;
    }
   }
   else
    return false;

   return $matches;
  }

  function output($key, $headers=true)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if(($value=$this->read($key))===false)
    return false;

   if($headers)
   {
    if(!$this->mime)
     $this->mime=new Mime();

    if(preg_match("/\.([a-z]+)$/i", $key, $tmp)) //use "extension"
     $ctype=$this->mime->extension($tmp[1]);
    else
     $ctype=$this->mime->content($value);

    header("Content-Type: ".$ctype);
    header("Content-Disposition: inline; filename=\"".$key."\"");
    header("Content-Length: ".strlen($value));
   }

   return $value;
  }

  function redirect($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
   {
    if(!$this->filePublic) //non public cache folder
    {
     $this->e->add("Redirect allowed, public cache folder not enable in configuration", "error");
     return false;
    }

    header("Location: ".$_path["project"]["cache"].$key);
   }
   else
   {
    $this->e->add("Redirect not supported for ".$this->type." based caching", "error");
    return false;
   }

   return true;
  }


  function lastUpdate($module)
  {
   $d=Data::cache($module, "view", "all");

   if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    return false;
   }

   if(!isset($d->field["updated"]))
   {
    $this->e->add("Data last update check required \"updated\" field", "error");
    return false;
   }

   $d->clearValues();
   $d->setValue("show_total", 1);
   $d->sortOrder("updated-");
   $d->setShowFields("updated");

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   return DT::dateConvert($d->v["updated"], $d->field["updated"]["format"][$d->dataformat]["datetimeformat"]);
  }
 }

?>
