<?

 /*****************************************************************************
 * Filename: include/captcha.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."config.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."mime.php");

 class Captcha extends Config
 {
  function Captcha($config=false)
  {
   global $_jses;

   Config::Config();
   $this->merge("Captcha", $config);

   $this->mime=false;

   $this->challenge=&$_jses["captcha"];

   $this->format($this->config["imageFormat"]);

   $this->length($this->config["length"]);
   $this->fontsize($this->config["fontSize"]);

   $this->size($this->config["imageWidth"], $this->config["imageHeight"]);
   $this->lines($this->config["lines"]);

   if($this->config["backgroundTransparent"])
    $this->colors(explode(",", $this->config["fontColor"]), explode(",", $this->config["linesColor"]), false);
   else
    $this->colors(explode(",", $this->config["fontColor"]), explode(",", $this->config["linesColor"]), explode(",", $this->config["backgroundColor"]));

   if(!$this->challenge||strlen($this->challenge)!=$this->length)
    $this->random();
  }

  function format($format)
  {
   if($format!="PNG"&&$format!="GIF"&&$format!="JPG")
   {
    $this->e->add("Unknown format: \"".$format."\", use PNG, GIF or JPG");
    return false;
   }

   $this->format=$format;
   return true;
  }

  function length($length)
  {
   if($length<1||20<$length)
   {
    $this->e->add("Length may not be less then 1 or more then 20");
    return false;
   }

   $this->length=$length;
   return true;
  }

  function fontsize($fontsize)
  {
   if($fontsize<1||100<$fontsize)
   {
    $this->e->add("Font size may not be less then 1 or more then 100");
    return false;
   }

   $this->fontsize=$fontsize;
   return true;
  }

  function size($width, $height)
  {
   if($width<20||2000<$width)
   {
    $this->e->add("Width may not be less then 20 and more then 2000");
    return false;
   }

   if($height<20||2000<$height)
   {
    $this->e->add("Height may not be less then 20 and more then 2000");
    return false;
   }

   $this->width=$width;
   $this->height=$height;
   return true;
  }

  function lines($lines)
  {
   if($lines<0||200<$lines)
   {
    $this->e->add("Lines may not be less then 0 or more then 200");
    return false;
   }

   $this->lines=$lines;
   return true;
  }

  function colors($font, $lines, $background)
  {
   if(!is_array($font)||sizeof($font)!=3||!is_numeric($font[0])||!is_numeric($font[1])||!is_numeric($font[2]))
   {
    $this->e->add("Illegal font color, example: Array(255,0,100)");
    return false;
   }

   if($lines)
   {
    if(!is_array($lines)||sizeof($lines)!=3||!is_numeric($lines[0])||!is_numeric($lines[1])||!is_numeric($lines[2]))
    {
     $this->e->add("Illegal line color, example: Array(255,0,100)");
     return false;
    }
   }

   if($background)
   {
    if(!is_array($background)||sizeof($background)!=3||!is_numeric($background[0])||!is_numeric($background[1])||!is_numeric($background[2]))
    {
     $this->e->add("Illegal background color, example: Array(255,0,100)");
     return false;
    }
   }

   $this->colors=Array(
    "font" => $font,
    "lines" => $lines,
    "background" => $background,
   );

   return true;
  }
  function check($challenge)
  {
   if($this->challenge!=$challenge)
   {
    $this->e->add("Captcha challenge doesn't match", "error");
    return false;
   }

   return true;
  }

  function get()
  {
   return $this->challenge;
  }

  function set($challenge)
  {
   if(strlen($challenge)<1||20<strlen($challenge))
   {
    $this->e->add("Challenge may not be less then 1 or more then 20 characters");
    return false;
   }

   $this->challenge=$challenge;

   return true;
  }

  function random()
  {
   $challenge="";

   for($c=0;$c<$this->length;$c++) 
   {
    if(rand(0,3)) //25% chance of numbers
     $challenge.=chr(rand(97, 122)); //lower case
    else
     $challenge.=chr(rand(48, 57)); //numbers
   }

   return $this->set($challenge);
  }

  function output($headers=true)
  {
   global $_path;

   if(!function_exists("gd_info"))
    $this->e->add("GD not found");

   if(!file_exists($_path["include"]."captcha.ttf"))
    $this->e->add("Font: \"".$_path["include"]."captcha.ttf\" not found");

   if($this->e->any())
    return $this->e->show();

   $image=imagecreatetruecolor($this->width, $this->height);

   $font=imagecolorallocate($image, $this->colors["font"][0], $this->colors["font"][1], $this->colors["font"][2]);

   if($this->colors["lines"])
    $lines=imagecolorallocate($image, $this->colors["lines"][0], $this->colors["lines"][1], $this->colors["lines"][2]);
   else 
    $lines=false;

   if($this->format=="PNG")
   {
    if($this->colors["background"])
    {
     $background=imagecolorallocate($image, $this->colors["background"][0], $this->colors["background"][1], $this->colors["background"][2]);
     imagefilledrectangle($image, 0, 0, $this->width, $this->height, $background);
    }
    else
    {
     imagealphablending($image, false);
     imagesavealpha($image, true);
     imagefilledrectangle($image, 0, 0, $this->width, $this->height, imagecolorallocatealpha($image, 0, 0, 0, 127));
    }
   }
   else if($this->format=="GIF")
   {
    if($this->colors["background"])
    {
     $background=imagecolorallocate($image, $this->colors["background"][0], $this->colors["background"][1], $this->colors["background"][2]);
     imagefilledrectangle($image, 0, 0, $this->width, $this->height, $background);
    }
    else
    {
     $alpha=Array(255, 255, 255);
     if($alpha[0]==$this->colors["font"][0])
      $alpha[0]--;
     if($alpha[0]==$this->colors["lines"][0])
      $alpha[0]--;

     imagealphablending($image, true);
     imagesavealpha($image, true);
     imagecolortransparent($image, imagecolorallocate($image, $alpha[0], $alpha[1], $alpha[2]));

     $alpha=imagecolorallocate($image, $alpha[0], $alpha[1], $alpha[2]);
     imagefilledrectangle($image, 0, 0, $this->width, $this->height, $alpha);
    }
   }
   else //JPG
   {
    $background=imagecolorallocate($image, $this->colors["background"][0], $this->colors["background"][1], $this->colors["background"][2]);
    imagefilledrectangle($image, 0, 0, $this->width, $this->height, $background);
   }

   $bbox=imagettfbbox($this->fontsize, rand(-5, 5), $_path["include"]."captcha.ttf", $this->challenge);
   $width=$bbox[2]-$bbox[0]; //lower right - lower left;
   $height=$bbox[1]-$bbox[5]; //lower left - upper right;

   imagettftext($image, $this->fontsize, rand(-5, 5), ($this->width-$width)/2, (($this->height+$height)/2), $font, $_path["include"]."captcha.ttf", $this->challenge);

   if($lines)
    for($c=0;$c<$this->lines;$c++)
     imageline($image, 0, rand(0, $this->height), $this->width, rand(0, $this->height), $lines);

   ob_start();
   switch($this->format)
   {
    case "GIF": imagegif($image); break;
    case "PNG": imagepng($image); break;
    case "JPG": imagejpeg($image); break;
   }
   $data=ob_get_contents();
   ob_end_clean();

   imagedestroy($image);

   //output
   if($headers)
   {
    if(!$this->mime)
     $this->mime=new Mime();

    header("Content-Type: ".$this->mime->extension($this->format));
    header("Content-Disposition: inline; filename=\"captcha.".strtolower($this->format)."\"");
    header("Content-Length: ".strlen($data));
   }

   return $data;
  }

 }

?>
