<?

 /*****************************************************************************
 * Filename: include/cms.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."config.php");
 require_once($_path["include"]."form.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."captcha.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."fs.php");

 class CMS extends Config
 {
  function CMS($config=false)
  {
   Config::Config();
   $this->merge("CMS", $config);

   $this->l=new Language("cms");

   $this->multiple=false;//multiple modules in upcoming post ? (change field names)
   $this->javascriptRights=false; //javascript rights function already printed?

   $this->referenceManipulation=true;

   $this->data=false;
   $this->form=false;

   $this->vars=Array();
   if(isset($this->config["defaultFormVars"]))
    foreach(explode("&amp;", $this->config["defaultFormVars"]) as $var)
     if(strstr($var, "=")&&list($k, $v)=explode("=", $var))
      $this->addVar($k, $v);
  }

  function setData(&$data)
  {
   $this->data=&$data;
  }

  function setForm(&$form)
  {
   $this->form=&$form;
  }

  function setDataForm(&$data, &$form)
  {
   $this->data=&$data;
   $this->form=&$form;
  }

  function setReturnURL($url)
  {
   $this->config["returnURL"]=strip_tags($url);
  }

  function addVar($name, $value)
  {
   if(isset($this->vars[$name]))
    $this->e->add("Overwriting variable: \"".$name."\"", "warning");

   $this->vars[$name]=$value;
  }



  function frontForm($cmsFile=false)
  {
   $r=$this->formStart($cmsFile);

   $r.="<div class=\"field\">";
   $r.="<div class=\"title\">".$this->l->t("Module")."</div>\n<div class=\"input\">";
   $r.=$this->form->selectStart(false, "m", "form", false, "onChange=\"this.form.submit();\"");
   $r.=$this->form->selectOption("");
   foreach(modules() as $m)
   {
    if($m==$this->data->module)
     $r.=$this->form->selectOption($m, false, "SELECTED");
    else
     $r.=$this->form->selectOption($m);
   }
   $r.=$this->form->selectEnd();

   $r.="</div>\n";
   $r.="</div>\n";

   $r.="<div class=\"field\">";
   $r.="<div class=\"title\">".$this->l->t("Quick selector")."</div>\n<div class=\"input\">";

   $r.=$this->form->dataSelect($this->data->module, "nr", $this->data->getValue("nr"), "form", false, "onChange=\"this.form.submit();\"")."\n";

   $r.="</div>\n";
   $r.="</div>\n";

   $r.="<div class=\"field\">";
   $r.="<div class=\"title\">".$this->l->t("Options")."</div>\n<div class=\"input\">";

   $r.=$this->form->selectStart(false, "a", "form", false, "onChange=\"this.form.submit();\"");
   $r.=$this->form->selectOption("");
   foreach($this->data->actionList as $m)
    $r.=$this->form->selectOption($m);
   $r.=$this->form->selectEnd()."</div>\n";

   foreach($this->vars as $k => $v)
    $r.=$this->form->input("hidden", $k, $v)."\n";

   $r.="</div>\n";
   $r.="</div>\n";

   $r.=$this->formEnd()."\n";

   return $r;
  }



  function simpleForm($cmsFile=false, $extra=false)
  {
   if(!($name=$this->data->getID()))
    return false;

   if(isset($_REQUEST[$name]))
    $values=$_REQUEST[$name];
   else
    $values=false;

   $r=$this->formStart($cmsFile);

   $r.="<div class=\"input\">";
   $r.=$this->form->select("multiple", $name, $this->data, $values, "form", false, $extra." style=\"height: 100px;\"");
   $r.="</div>\n";

   $r.="<div class=\"input\">";
   if($this->data->r("add"))
    $r.=$this->form->input("submit", "", $this->l->t("Add"), "submit", false, "onclick=\"this.form.a.value='add';\" style=\"width: 25%;\"")."\n";
   if($this->data->r("copy"))
    $r.=$this->form->input("submit", "", $this->l->t("Copy"), "submit", false, "onclick=\"this.form.a.value='copy';\" style=\"width: 25%;\"")."\n";
   if($this->data->r("edit"))
    $r.=$this->form->input("submit", "", $this->l->t("Edit"), "submit", false, "onclick=\"this.form.a.value='edit';\" style=\"width: 25%;\"")."\n";
   if($this->data->r("delete"))
    $r.=$this->form->input("submit", "", $this->l->t("Delete"), "submit", false, "onclick=\"this.form.a.value='delete';\" style=\"width: 25%;\"")."\n";
   $r.="</div>\n";

   foreach($this->vars as $k => $v)
    $r.=$this->form->input("hidden", $k, $v)."\n";

   $r.=$this->form->input("hidden", "m", $this->data->module)."\n";
   $r.=$this->form->input("hidden", "a", "view")."\n";
   $r.=$this->form->input("hidden", "e", "item")."\n";

   $r.=$this->formEnd()."\n";

   return $r;
  }



  function form($cmsFile=false)
  {
   $r="";

   if(!$this->formDataSetup())
    return false;

   if($this->config["showTitles"])
    $r.="<h1>".$this->data->title."</h1>\n";

   if($this->config["showText"])
    $r.=$this->formShowText();


   if($this->config["handleSubmit"])
   {
    if($this->formHandleSubmit()) //submit successful
    {
     if(!request("cmsContinue")) //no more cms action?
     {
      if(!isset($this->config["showErrors"])||$this->config["showErrors"]) //show notices
       $r.=$this->formShowErrors();

      if($this->config["reloadPage"]) //reload page
       $r.=$this->handleReloadPage();

      return $r;
     }
    }
   }


   if($this->referenceManipulation)
    $r.=$this->formReferenceManipulation();

   if(!isset($this->config["showErrors"])||$this->config["showErrors"])
    $r.=$this->formShowErrors();


   $r.=$this->formStart($cmsFile);

   if($this->config["showFields"])
    $r.=$this->formShowFields();

   if($this->config["showSubmit"])
    $r.=$this->formShowSubmit();

   if($this->config["showCancel"])
    $r.=$this->formShowCancel();


   $r.=$this->formEnd();

   return $r;
  }


  function formDataSetup()
  {
   if($this->data==false)
    $this->data=Data::cache(false, false, false, Array("format" => "form"));

   $this->data->dataformat="form";

   if(!$this->data->validAction())
   {
    $this->e->add($this->data->e);
    return false;
   }

   return true;
  }


  function formStart($cmsFile=false)
  {
   global $_path;

   if(!$cmsFile)
    $cmsFile=$_path["HTML"]["url"];

   if(!$this->form)
    $this->form=new Form();

   return $this->form->start($cmsFile)."\n\n";
  }


  function formShowText()
  {
   $r="";

   if(!request("s")) //only show first time
   {
    $r.="<div class=\"text\">\n";
    switch($this->data->action)
    {
     case "view":
     {
      $r.=$this->l->t("Viewing data, select an option below");
      break;
     }

     case "add":
     case "copy":
     {
      $r.=$this->l->t("Fill in the fields below and press \"Add\" to insert the new data");
      break;
     }

     case "edit":
     {
      $r.=$this->l->t("Modify the fields below and press \"Update\" to save the changes");
      break;
     }

     case "move":
     {
      $r.=$this->l->t("Select a category and press \"Move\" to save the new category");
      break;
     }

     case "delete":
     {
      if($this->data->module=="categories"&&isset($this->data->moduleObject->autoDeleteContent)&&($this->data->moduleObject->autoDeleteContent||$this->data->moduleObject->autoDeleteSubcategories))
      {
       if($this->data->moduleObject->autoDeleteContent&&$this->data->moduleObject->autoDeleteSubcategories)
        $r.=$this->l->t("Are you sure you wish to delete this \"".$this->data->title."\" and it's content and subcategories?");
       else if($this->data->moduleObject->autoDeleteContent)
        $r.=$this->l->t("Are you sure you wish to delete this \"".$this->data->title."\" and it's content?");
       else
        $r.=$this->l->t("Are you sure you wish to delete this \"".$this->data->title."\" and it's subcategories?");
      }
      else
       $r.=$this->l->t("Are you sure you wish to delete this data?");

      break;
     }

     case "undelete":
     {
      $r.=$this->l->t("Are you sure you wish to undelete this data?");
      break;
     }

     case "rights":
     {
      $r.=$this->l->t("Select the group / user, their new rights and press \"Update\" to save the new rights");
      break;
     }

     case "owner":
     {
      $r.=$this->l->t("Select an user and press \"Submit\" to save the new owner");
      break;
     }

     case "user":
     {
      switch($this->data->extra)
      {
       case "login":
       {
        $r.=$this->l->t("Please enter your username and password and press \"Login\"");
        break;
       }
       case "logout":
       {
        $r.=$this->l->t("Are you sure you wish to logout ?");
        break;
       }
       case "password":
       {
        $r.=$this->l->t("Fill in your new password and verify it, then press \"Update\" to change it");
        break;
       }
      }
      break;
     }

     default:
     {
      $r.=$this->l->t($this->data->action)." => ".$this->l->t($this->data->extra);

      break;
     }
    }
    $r.="</div>\n";
   }

   return $r;
  }

  function formHandleSubmit()
  {
   if(request("s"))
   {
    if($this->data->submit())
    {
     if(request("cmsContinue"))//oh you want more ?
     {
      if($this->data->action=="add")//Clean up, after adding
      {
       if(request("cmsContinue")=="add")
       {
        foreach($this->data->check("use", "form") as $k)
         if(!$this->data->check("reference", "categories", $k))
          $this->data->setDefaultValue($k);
       }
       else if(request("cmsContinue")=="edit")
       {
        $this->data->setAction("edit");
        $this->data->setValue("a", "edit");
       }
      }
     }

     return true;
    }
   }

   return false;
  }


  function handleReloadPage()
  {
   global $_jses;

   $hs=new History();

   if($hs->getTotal()&&($page=$hs->previous())) //history in use
    $returnURL=$page;
   else if($this->config["returnURL"])
    $returnURL=$this->config["returnURL"];
   else
    return "";

   return "<script type=\"text/javascript\">setTimeout(\"this.location='".$returnURL."'\", ".$this->config["reloadTimeout"].");</script>\n";
  }


  function formReferenceManipulation()
  {
   $r="";

   if($this->data->requestVar)
    $ref_name=request($this->data->requestVar, "_refmanip");
   else
    $ref_name=request("_refmanip");

   if(is_array($ref_name))
   {
    $ref_action=strtolower(reset($ref_name));
    $ref_name=array_keys($ref_name)[0];
   }
   else
   {
    $ref_name=false;
    $ref_action=false;
   }

   if($ref_action&&$ref_name) //reference manipulation
   {
    $name=$ref_name;
    $this->showReference[$name]=false;

    $subd=Data::cache($this->data->reference[$name]["module"], ($ref_action=="edit"?"edit":"add"), "item", Array("request" => $this->data->requestVar."SubCMS"));

    if(request($subd->requestVar, "s")) //submit
    {
     if($subd->submit())
     {
      $this->showReference[$name]=true;

      if($subd->action=="add")
      {
       if($this->data->reference[$name]["type"]=="0:*"||$this->data->reference[$name]["type"]=="1:*")
        $this->data->setValue($name, array_merge($this->data->getValue($name), Array($subd->getValue($subd->getID()))));
       else if($this->data->reference[$name]["type"]=="*:*")
        $this->data->setValue($name, array_merge($this->data->getValue($name), Array($subd->getValue($this->data->reference[$name]["to"]))));
       else
        $this->data->setValue($name, $subd->getValue($subd->getID()));
      }
     }
     else
      $r.=$subd->e->show();
    }
   }

   return $r;
  }


  function formShowErrors()
  {
   $r="";

   if($this->data->e->any())
   {
    $this->e->add($this->data->e);
    $this->data->e->clear();
   }

   if($this->e->any())
   {
    $r.=$this->e->show();
    $this->e->clear();
   }

   return $r;
  }


  function formShowFields()
  {
   global $_config;

   $r="";

   switch($this->data->action)
   {
    case "add":
    case "copy":
    case "edit":
    {
     if(($this->data->action=="copy"||$this->data->action=="edit")&&!request("s")&&!request("dontgetdata"))//get data the first time
     {
      if(!$this->data->get()||$this->data->e->any("warning"))
      {
       $this->e->add($this->data->e);
       $r.=$this->e->show();
       break;
      }

      if($this->data->action=="copy")
       foreach($this->data->check("properties", "identifier") as $k)
        $this->data->unsetValue($k);
     }

     if($this->data->getGroups()&&($this->data->action=="add"||$this->data->action=="copy")&&$this->data->check("properties", "rightsReference", "catnr")&&!$this->data->check("properties", "hidden", "catnr"))
      $this->data->group("Category", "catnr");

     if($groups=$this->data->getGroups())
     {
      $r.="<div class=\"groups ".$this->data->module."\">";
      foreach($groups as $key => $title)
      {
       if(!($groupFields=$this->data->getGroupFields($key)))
        continue;

       $visibleFields=false;
       foreach($groupFields as $name)
       {
        if($this->data->check("use", "form", $name)&&!$this->data->check("properties", "hidden", $name)&&!$this->data->check("properties", "owner", $name))
        {
         $visibleFields=true;
         break;
        }
       }

       if($visibleFields)
       {
        $r.="<div class=\"group ".preg_replace("/[^a-z]*/i", "", $title)."\">";
        $r.="<div class=\"groupHeader\">".$title."</div>";
        $r.="<div class=\"groupBody\">";
       }

       $names=Array();
       $boolean=false;
       $currentTitle=false;
       foreach($groupFields as $name)
       {
        if($this->data->check("properties", "control", $name))
         if(strpos($name, "aggregation_")===0||strpos($name, "show_")===0) //don't added aggregation and show control fields
          continue;

        if($this->data->check("use", "form", $name))
        {
         if(!$this->data->check("properties", "rightsReference", $name)&&
           (!$this->data->check("properties", "owner", $name)||$this->data->check("properties", "identifier", $name))
         )
         {
          if($currentTitle===false)
           $currentTitle=$this->data->title($name);

          if(
           (!$boolean&&$currentTitle!=$this->data->title($name))||
           ($boolean&&$this->data->field[$name]["type"]!="boolean")
          )
          {
           $r.=$this->field($names);
           $currentTitle=$this->data->title($name);
           $names=Array();
          }

          $names[]=$name;

          if($this->data->field[$name]["type"]=="boolean")
           $boolean=true;
          else
           $boolean=false;
         }
        }
       }

       if($names)
        $r.=$this->field($names);

       foreach($groupFields as $name)
       {
        if(($this->data->action=="add"||$this->data->action=="copy")&&$this->data->check("properties", "rightsReference", $name))
        {
         if(!$this->data->check("properties", "owner", $name))
          $r.=$this->field($name);
        }
       }

       if($visibleFields)
       {
        $r.="</div>";
        $r.="<div class=\"groupFooter\"></div>";
        $r.="</div>";
       }
      }
      $r.="</div>";
     }
     else
     {
      $names=Array();
      $boolean=false;
      $currentTitle=false;
      //Get all fields that should be used in the form
      foreach($this->data->check("use", "form") as $name)
      {
       if($this->data->check("properties", "control", $name))
        if(strpos($name, "aggregation_")===0||strpos($name, "show_")===0) //don't added aggregation and show control fields
         continue;

       if(!$this->data->check("properties", "rightsReference", $name)&&
         (!$this->data->check("properties", "owner", $name)||$this->data->check("properties", "identifier", $name))
       )
       {
        if($currentTitle===false)
         $currentTitle=$this->data->title($name);

        if(
         (!$boolean&&$currentTitle!=$this->data->title($name))||
         ($boolean&&$this->data->field[$name]["type"]!="boolean")
        )
        {
         $r.=$this->field($names);
         $currentTitle=$this->data->title($name);
         $names=Array();
        }

        $names[]=$name;

        if($this->data->field[$name]["type"]=="boolean")
         $boolean=true;
        else
         $boolean=false;
       }
      }

      if($names)
       $r.=$this->field($names);

      if($this->data->action=="add"||$this->data->action=="copy")
      {
       foreach($this->data->check("properties", "rightsReference") as $name)
       {
        if(!$this->data->check("properties", "owner", $name))
         $r.=$this->field($name);
       }
      }
     }

     break;
    }


    case "user":
    {
     switch($this->data->extra)
     {
      case "login":
      {
//Get all fields that should be used in the form
       foreach($this->data->check("use", "login") as $name)
       {
        $r.=$this->field($name);
       }
       foreach($this->data->check("use", "form") as $name)
       {
        if($this->data->check("properties", "hidden", $name))
         $r.=$this->field($name);
       }
       break;
      }
      case "password":
      {
       foreach($this->data->check("properties", "password") as $name)
       {
        $r.=$this->field($name);
       }
      }
      case "logout":
      {
 //Get all fields that should be used in the form
       foreach($this->data->check("use", "logout") as $name)
       {
        $r.=$this->field($name);
       }
       foreach($this->data->check("use", "form") as $name)
       {
        if($this->data->check("properties", "hidden", $name))
         $r.=$this->field($name);
       }
       break;
      }
     }
     break;
    }

    case "view":
    case "move":
    case "owner":
    case "delete":
    case "undelete":
    {
     if(!$this->data->get()||$this->data->e->any("warning"))
     {
      $this->e->add($this->data->e);
      $r.=$this->e->show();
      break;
     }
     else if($this->data->action=="move")
     {
      $this->data->setRequestValue("newcatnr");
      if($this->data->check("valuetype", "none", "newcatnr"))
       $this->data->setValue("newcatnr", $this->data->getValue("catnr"));

      if($this->data->getGroups())
       $this->data->group("Category", "newcatnr");
     }

     if($this->data->extra=="list")
      $this->data->field["catnr"]["properties"].=";hidden";

     if($groups=$this->data->getGroups())
     {
      foreach($groups as $key => $title)
      {
       if(!($groupFields=$this->data->getGroupFields($key)))
        continue;

       $r.="<div class=\"group ".preg_replace("/[^a-z]*/i", "", $title)."\">";
       $r.="<div class=\"groupHeader\">".$title."</div>";
       $r.="<div class=\"groupBody\">";

       $names=Array();
       $currentTitle=false;
       foreach($groupFields as $name)
       {
        if($this->data->check("properties", "control", $name))
        {
         if(strpos($name, "aggregation_")!==0&&strpos($name, "show_")!==0) //don't added aggregation and show control fields
          $r.=$this->field($name, "hidden");
        }
        else if(($this->data->action=="move"&&$name=="newcatnr")||($this->data->action=="owner"&&$this->data->check("properties", "owner", $name)))
        {
         $r.=$this->field($name, "show");
        }
        else if($this->data->check("properties", "hidden", $name)||$this->data->check("properties", "identifier", $name))
        {
         for($nr=0;$nr<$this->data->vmax;$nr++)
          $r.=$this->field($name, "hidden", $nr);
        }
        else if(($this->data->extra=="item"&&$this->data->check("use", "show", $name)))
        {
         if($currentTitle===false)
          $currentTitle=$this->data->title($name);

         if($currentTitle!=$this->data->title($name))
         {
          $r.=$this->field($names, "fake");
          $currentTitle=$this->data->title($name);
          $names=Array();
         }

         $names[]=$name;
        }
        else if($this->data->check("properties", "title", $name))
        {
         for($nr=0;$nr<$this->data->vmax;$nr++)
          $r.=$this->field($name, "fake", $nr);
        }
       }

       if($names)
        $r.=$this->field($names, "fake");

       $r.="</div>";
       $r.="<div class=\"groupFooter\"></div>";
       $r.="</div>";
      }
     }
     else
     {
      $names=Array();
      $currentTitle=false;
      foreach($this->data->check("use", "form") as $name)
      {
       if($this->data->check("properties", "control", $name))
       {
        if(strpos($name, "aggregation_")!==0&&strpos($name, "show_")!==0) //don't added aggregation and show control fields
         $r.=$this->field($name, "hidden");
       }
       else if(($this->data->action=="move"&&$name=="newcatnr")||($this->data->action=="owner"&&$this->data->check("properties", "owner", $name)))
       {
        $r.=$this->field($name, "show");
       }
       else if($this->data->check("properties", "hidden", $name)||$this->data->check("properties", "identifier", $name))
       {
        for($nr=0;$nr<$this->data->vmax;$nr++)
         $r.=$this->field($name, "hidden", $nr);
       }
       else if(($this->data->extra=="item"&&$this->data->check("use", "show", $name)))
       {
        if($currentTitle===false)
         $currentTitle=$this->data->title($name);

        if($currentTitle!=$this->data->title($name))
        {
         $r.=$this->field($names, "fake");
         $currentTitle=$this->data->title($name);
         $names=Array();
        }

        $names[]=$name;
       }
       else if($this->data->check("properties", "title", $name))
       {
        for($nr=0;$nr<$this->data->vmax;$nr++)
         $r.=$this->field($name, "fake", $nr);
       }
      }

      if($names)
       $r.=$this->field($names, "fake");
     }

     break;
    }


    case "rights":
    {
     $r.="<div class=\"field\">";
     $r.="<div class=\"title\">".$this->l->t("Category")."</div>\n";
     $r.="<div class=\"input\">";
     $r.=$this->form->dataSelect("categories", "nr", $this->data->getValue("nr"), false, false, "onchange=\"this.form.reload_trigger.value='nr'; this.form.submit();\" ")."\n";
     $r.="</div>\n";
     $r.="</div>\n";

     $c=1;
     $type="userRights";

     while($type)
     {
      $dr=Data::cache($type, "view", "list");
      $dr->setValue("catnr", $this->data->getValue("nr"));
      $dr->get(); //don't get references... seems to be trouble when getting rights

      $r.="<div class=\"field\">";
      $r.="<div class=\"title\">\n";
      $r.=$dr->title;
      $r.="</div><div class=\"input\">\n";

      while($dr->next())
      {
       if($type=="userRights")
        $dg=Data::cache($_config["usermodule"], "view", "item");
       else
        $dg=Data::cache("categories", "view", "item");

       $dg->setValue("nr", $dr->v["nr"]);
       $dg->get();

       if($dg->next())
        $r.=$dg->v("title");

       $r.=": ".$dr->v["rights"]."<br>\n";
       $c++;
      }

      $r.="</div>\n";
      $r.="</div>\n";

      if($type=="userRights")
      {
       $r.=$this->field("usernr");
       $r.=$this->field("userrights");
      }
      else
      {
       $r.=$this->field("groupnr");
       $r.=$this->field("grouprights");
      }

      if($type=="userRights"){$type="groupRights";}
      else{$type=NULL;}
     }

     foreach($this->data->check("use", "form") as $name)
     {
      if($name!="nr"&&
       ($this->data->check("properties", "control", $name)
        ||($this->data->check("properties", "hidden", $name))))
         $r.=$this->field($name);
     }


     break;
    }


    case "database":
    {
     $this->data->get();

     foreach($this->data->check("use", "form") as $name)
      if($this->data->check("properties", "hidden", $name))
       $r.=$this->field($name);

     $r.="<div class=\"field\">\n";
     $r.="<div class=\"title\">\n";
     $r.=$this->l->t("Query");
     $r.="</div><div class=\"cms\">\n";
     $r.=$this->data->e->show();
     $r.="</div>\n";
     $r.="</div>\n";

     break;
    }

    case "order":
    {
     $this->data->get();

     foreach($this->data->check("use", "form") as $name)
      if($this->data->check("properties", "hidden", $name))
       $r.=$this->field($name);

     break;
    }

    default: break;
   }

   foreach($this->vars as $k => $v)
    $r.=$this->form->input("hidden", $k, $v)."\n";

   return $r;
  }

  function formShowSubmit()
  {
   $r="";

   if($this->data->requestVar)
    $fieldname=$this->data->requestVar."[s]";
   else
    $fieldname="s";

   if($this->config["useStyling"])
   {
    $r.="<div class=\"field\">\n";
    $r.="<div class=\"title\">\n";
    $r.=$this->l->t("Submit");
    $r.="</div>\n";

    $r.="<div class=\"input\">\n";
   }

   switch($this->data->action)
   {
    case "add":
    case "copy":
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Add"), "submit")."\n";
     $r.=$this->form->input("hidden", "cmsContinue", 0)."\n";

     if($this->data->action=="copy")
      $r.=$this->form->input("hidden", "dontgetdata", 1)."\n";

     if($this->config["showSubmitAdditional"])
     {
      $r.=$this->form->input("submit", $fieldname, $this->l->t("Add and update"), "submit", false, "onclick=\"this.form.cmsContinue.value='edit'; \"")."\n";
      $r.=$this->form->input("submit", $fieldname, $this->l->t("Add another"), "submit", false, "onclick=\"this.form.cmsContinue.value='add'; \"")."\n";
     }
    break;

    case "edit";
    case "rights";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Update"), "submit")."\n";
     $r.=$this->form->input("hidden", "cmsContinue", 0)."\n";
     $r.=$this->form->input("hidden", "dontgetdata", 1)."\n";

     if($this->config["showSubmitAdditional"])
      $r.=$this->form->input("submit", $fieldname, $this->l->t("Update and continue"), "submit", false, "onclick=\"this.form.cmsContinue.value=1; \"")."\n";
    break;

    case "move";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Move"), "submit")."\n";
    break;

    case "delete";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Delete"), "submit")."\n";
    break;

    case "undelete";
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Undelete"), "submit")."\n";
    break;

    case "user":
    {
     switch($this->data->extra)
     {
      case "login";
       $r.=$this->form->input("submit", $fieldname, $this->l->t("Login"), "submit")."\n";
      break;

      case "logout";
       $r.=$this->form->input("submit", $fieldname, $this->l->t("Logout"), "submit")."\n";
      break;

      case "password";
       $r.=$this->form->input("submit", $fieldname, $this->l->t("Update"), "submit")."\n";
      break;
     }
     break;
    }

    case "view";
     $this->data->next(); //set rights for checking

     $fieldname=$this->data->requestVar."['a']";
     $fieldname2=$this->data->requestVar."['e']";

     if($this->data->r("copy"))
      $r.=$this->form->input("submit", "", $this->l->t("Copy"), "submit", false, "onclick=\"this.form".$fieldname.".value='copy';\"")."\n";
     if($this->data->r("edit"))
      $r.=$this->form->input("submit", "", $this->l->t("Edit"), "submit", false, "onclick=\"this.form".$fieldname.".value='edit';\"")."\n";
     if($this->data->r("move"))
      $r.=$this->form->input("submit", "", $this->l->t("Move"), "submit", false, "onclick=\"this.form".$fieldname.".value='move';\"")."\n";
     if($this->data->r("delete"))
      $r.=$this->form->input("submit", "", $this->l->t("Delete"), "submit", false, "onclick=\"this.form".$fieldname.".value='delete';\"")."\n";
     if($this->data->module=="categories"&&$this->data->r("rights"))
      $r.=$this->form->input("submit", "", $this->l->t("Rights"), "submit", false, "onclick=\"this.form".$fieldname.".value='rights';\"")."\n";
    break;


    default:
     $r.=$this->form->input("submit", $fieldname, $this->l->t("Submit"), "submit")."\n";
    break;
   }

   if($this->config["useStyling"])
   {
    $r.="</div>\n";
    $r.="</div>\n";
   }

   return $r;
  }


  function formShowCancel()
  {
   $r="";

   if($this->config["useStyling"])
   {
    $r.="<div class=\"field\">\n";
    $r.="<div class=\"title\">\n";
    $r.=$this->l->t("Cancel");
    $r.="</div>\n<div class=\"input\">\n";
   }

   $hs=new History();

   if($hs->getTotal()&&($page=$hs->previous())) //history in use
    $r.=$this->form->input("button", "", $this->l->t("Back"), "submit", false, "onclick=\"document.location='".$page."'\"")."\n";
   else //no history, use browser history
    $r.=$this->form->input("button", "", $this->l->t("Back"), "submit", false, "onclick=\"history.back();\"")."\n";

   if($this->config["useStyling"])
   {
    $r.="</div>\n";
    $r.="</div>\n";
   }

   return $r;
  }


  function formEnd()
  {
   return $this->form->end()."\n\n";
  }


  function field($names, $type=false, $nr=false)
  {
   global $_path;

   $r="";
   $d=&$this->data;

   if(!is_array($names))
    $names=Array($names);

   $v=&$d->field[$names[0]];

   //Show non hidden fields, type show overwrites anyway
   if($this->config["useStyling"]&&($type=="show"||(!$d->check("properties", "hidden", $names[0])&&$type!="hidden")))
   {
    switch($v["type"])
    {
     case "boolean":
      $r.="<div class=\"field\">";

      if($type=="fake")
       $r.="<div class=\"title\">".$v["title"]."</div>\n";
      else
       $r.="<div class=\"title\">&nbsp;</div>\n";

      if($this->config["showTips"]&&$v["text"]!="")
       $r.="<div class=\"text\">".$v["text"]."</div>";

      $r.="<div class=\"input\">\n";

      foreach($names as $name)
       $r.=$this->fieldInput($name, $type, $nr);

      $r.="</div>\n";
      $r.="</div>\n";
     break;

     default:
      $r.="<div class=\"field\">";
      $r.="<div class=\"title\">";

      if($v["min"]>0)
      {
       if($this->config["requiredIndicatorFile"])
       {
        if(file_exists($_path["project"]["images"].$this->config["requiredIndicatorFile"]))
         $r.="<img src=\"".$_path["HTML"]["prefix"].$_path["project"]["images"].$this->config["requiredIndicatorFile"]."\" alt=\"".$this->l->t("Required")."\" /> ";
        else
         $r.=$this->config["requiredIndicatorFile"];
       }
       else
        $r.=" * ";
      }

      if($v["title"])
       $r.=$v["title"];
      else
       $r.="&nbsp; ";

      $r.="</div>\n";

      if($this->config["showTips"]&&$v["text"]!="")
       $r.="<div class=\"text\">".$v["text"]."</div>";

      if(($type=="fake"&&isset($d->reference[$names[0]]))||(isset($this->showReference)&&isset($this->showReference[$names[0]])&&!$this->showReference[$names[0]]))
       $r.="<div class=\"input reference\">\n";
      else
       $r.="<div class=\"input\">\n";

      $cmsOptions=false;
      foreach($names as $name)
       $r.=$this->fieldInput($name, $type, $nr, $cmsOptions);

      $r.="</div>\n";

      if($cmsOptions!==false) //strict check, "" adds empty options container
       $r.="<div class=\"options\">".$cmsOptions."</div>";

      $r.="</div>\n";
     break;
    }
   }
   else //no fancy showing
   {
    foreach($names as $name)
     $r.=$this->fieldInput($name, $type, $nr);
   }

   return $r;
  }


  function fieldInput($name, $type=false, $nr=false, &$cmsOptions=false)
  {
   global $_config, $_path;

   $r="";
   $d=&$this->data;
   $f=&$this->form;

   $v=&$d->field[$name];

   if($d->requestVar)
   {
    if($nr===false||$name=="m"||$name=="a"||$name=="e"){$fieldname=$d->requestVar."[".$name."]"; $nr=0;}
    else{$fieldname=$d->requestVar."[".$name."]"."[".$nr."]";}
   }
   else
   {
    if($nr===false||$name=="m"||$name=="a"||$name=="e"){$fieldname=$name; $nr=0;}
    else{$fieldname=$name."[".$nr."]";}
   }

   if($type=="show"||(!$d->check("properties", "hidden", $name)&&$type!="hidden"))
   {
    if($this->config["anchoring"])
     $r.="<a id=\"".$fieldname."\"></a>";

    if($type!="fake")
    {
     $extra="";
     if($d->check("properties", "reload", $name))
     {
      if($this->config["anchoring"])
       $extra.="onchange=\"this.form.action=this.form.action.replace(/\#.*$/, ''); this.form.action+='#".$fieldname."'; this.form.reload_trigger.value='".$name."'; this.form.submit();\" ";
      else
       $extra.="onchange=\"this.form.reload_trigger.value='".$name."'; this.form.submit();\" ";
     }
     if($d->check("properties", "disabled", $name))
      $extra.="disabled ";

     //show references
     if(isset($d->reference[$name]))
     {
      $class="form ".$d->reference[$name]["module"]." ".$name;

      if($d->reference[$name]["type"]=="1:1")
      {
       $subd=Data::cache($d->reference[$name]["module"], $this->data->action, $this->data->extra, Array("request" => md5($name)));

       if(($subd->action=="copy"||$subd->action=="edit")&&!request("s")&&!request("dontgetdata"))//get data the first time
       {
        $subd->setValue($d->reference[$name]["to"], $d->getValue($d->reference[$name]["from"], $nr), $nr);
        $subd->get();
       }

       $subcms=new CMS();
       $subcms->setDataForm($subd, $this->form);

       foreach($subd->check("use", "form") as $field)
        if($subd->check("use", "sql", $field)&&!$subd->check("properties", "owner", $field)&&!$subd->check("properties", "rightsReference", $field)&&!$subd->check("properties", "identifier", $field))
         $r.=$subcms->field($field, $type);

       unset($subcms, $subd);
      }
      else //*:0, *:1, *:*, 0:* and 1:*
      {
       if($this->data->requestVar)
        $ref_name=request($this->data->requestVar, "_refmanip");
       else
        $ref_name=request("_refmanip");

       if(is_array($ref_name))
       {
        $ref_action=strtolower(reset($ref_name));
        $ref_name=array_keys($ref_name)[0];
       }
       else
       {
        $ref_name=false;
        $ref_action=false;
       }

       if($this->referenceManipulation&&$ref_action&&$ref_name==$name) //reference manipulation
       {
        if(isset($this->showReference)&&isset($this->showReference[$name])&&!$this->showReference[$name])
        {
         $subd=Data::cache($d->reference[$name]["module"], ($ref_action=="edit"?"edit":"add"), "item", Array("request" => $this->data->requestVar."SubCMS"));

         if($subd->action=="edit")
         {
          if($d->reference[$name]["type"]=="0:*"||$d->reference[$name]["type"]=="1:*"||$d->reference[$name]["type"]=="*:*")
           $subd->setValue($subd->getID(), end($d->getValue($ref_name)));
          else
           $subd->setValue($subd->getID(), $d->getValue($ref_name));

          if(!request($subd->requestVar, "dontgetdata"))//get data the first time
           if(!$subd->get()||$subd->e->any("warning"))
            $r.=$subd->e->show();
         }

         $subcms=new CMS();
         $subcms->referenceManipulation=false;
         $subcms->setDataForm($subd, $this->form);

         $r.=$subcms->formShowFields();

         if($this->data->requestVar)
          $this->addVar($this->data->requestVar."[_refmanip][".$name."]", $subd->action);
         else
          $this->addVar("_refmanip[".$name."]", $subd->action);

         if($d->reference[$name]["type"]=="0:*"||$d->reference[$name]["type"]=="1:*"||$d->reference[$name]["type"]=="*:*")
         {
          foreach(array_keys($v["value"][$nr]) as $k)
           $r.=$f->input("hidden", $fieldname."[".$nr."][".$k."]", $v["value"][$nr][$k]);
         }
         else
          $r.=$f->input("hidden", $fieldname, $v["value"][$nr]);

         $r.=$f->input("hidden", $subd->requestVar."[dontgetdata]", 1)."\n";

         $r.="<div class=\"field\">\n";
         $r.="<div class=\"title\">\n";
         $r.="</div>\n<div class=\"input\">\n";

         if($subd->action=="add")
          $r.=$f->input("submit", $subd->requestVar."[s]", $subcms->l->t("Add"), "submit")."\n";
         else
          $r.=$f->input("submit", $subd->requestVar."[s]", $subcms->l->t("Update"), "submit")."\n";

         $r.=$f->input("button", "", $subcms->l->t("Back"), "submit", false, "onclick=\"history.back();\"")."\n";

         $r.="</div>\n";
         $r.="</div>\n";
        }

        unset($subcms, $subd);
       }

       if(!isset($this->showReference)||!isset($this->showReference[$name])||$this->showReference[$name])
       {
        if($this->referenceManipulation)
        {
         if($this->data->requestVar)
          $_refmanip=$this->data->requestVar."[_refmanip][".$name."]";
         else
          $_refmanip="_refmanip[".$name."]";
        }

        if($d->reference[$name]["type"]=="0:*"||$d->reference[$name]["type"]=="1:*"||$d->reference[$name]["type"]=="*:*")
        {
         if(isset($d->listOptions[$name]))
         {
          if(isset($this->config["tomanyReferenceStyle"])&&$this->config["tomanyReferenceStyle"]=="list")
           $r.=$f->listview("multiple", $fieldname."[".$nr."][]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
          else
           $r.=$f->select("multiple", $fieldname."[".$nr."][]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
         }
         else
         {
          if(isset($this->config["tomanyReferenceStyle"])&&$this->config["tomanyReferenceStyle"]=="list")
           $r.=$f->dataListview($d->reference[$name]["module"], $fieldname."[".$nr."][]", $v["value"][$nr], $class, false, $extra, true)."\n";
          else
           $r.=$f->dataSelect($d->reference[$name]["module"], $fieldname."[".$nr."][]", $v["value"][$nr], $class, false, $extra, true)."\n";
         }

         $cmsOptions=""; //enable cmsOptions
         if($this->referenceManipulation&&isset($this->config["manytoReferenceManip"])&&$this->config["manytoReferenceManip"])
         {
          $cmsOptions.=$f->input("submit", $_refmanip, "Add");
          $cmsOptions.=$f->input("submit", $_refmanip, "Edit");
         }
        }
        else //*:0 and *:1
        {
         if($d->reference[$name]["from"]!=$name)
         {
          $extra="";
          if($d->check("properties", "reload", $d->reference[$name]["from"]))
           $extra.="onchange=\"this.form.submit();\" ";
          if($d->check("properties", "disabled", $d->reference[$name]["from"]))
           $extra.="disabled ";
         }

         if(isset($d->listOptions[$name]))
         {
          if(isset($this->config["manytoReferenceStyle"])&&$this->config["manytoReferenceStyle"]=="list")
           $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
          else
           $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra)."\n";
         }
         else
         {
          if(isset($this->config["manytoReferenceStyle"])&&$this->config["manytoReferenceStyle"]=="list")
           $r.=$f->dataListview($d->reference[$name]["module"], $fieldname, $v["value"][$nr], $class, false, $extra, false, true)."\n";
          else if($d->reference[$name]["module"]=="categories")
           $r.=$f->dataSelect($d->reference[$name]["module"], $fieldname, $v["value"][$nr], $class, false, $extra, true, false)."\n";
          else
           $r.=$f->dataSelect($d->reference[$name]["module"], $fieldname, $v["value"][$nr], $class, false, $extra, false, true)."\n";

         }

         $cmsOptions=""; //enable cmsOptions
         if($this->referenceManipulation&&isset($this->config["manytoReferenceManip"])&&$this->config["manytoReferenceManip"])
         {
          $cmsOptions.=$f->input("submit", $_refmanip, "Add");
          $cmsOptions.=$f->input("submit", $_refmanip, "Edit");
         }
        }
       }
      }
     }
     else
     {
      $class="form ".$v["type"]." ".$name;

      if($d->check("properties", "options", $name))
       $cmsOptions=""; //enable cmsOptions

     //show fields
      switch($v["type"])
      {
       case "list":
       {
        if(isset($this->config["listStyle"])&&$this->config["listStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }
       case "multilist":
       {
        $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

        if(isset($this->config["multilistStyle"])&&$this->config["multilistStyle"]=="list")
         $r.=$f->listview("multiple", $fieldname."[".$nr."][]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select("multiple", $fieldname."[".$nr."][]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "module":
       {
        //Set module to upper lying category module by default
        if(!$v["value"][$nr]&&!request("s")&&$d->module=="categories")
        {
         $d2=Data::cache("categories", "view", "item");
         $d2->setValue("nr", $d->getValue("catnr"));
         $d2->get();
         $v["value"][$nr]=$d2->getValue("module", $nr);
        }

        if(isset($this->config["moduleStyle"])&&$this->config["moduleStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }
       case "modules":
       {
        //Set module to upper lying category module by default
        if(!$v["value"][$nr]&&!request("s")&&$d->module=="categories")
        {
         $d2=Data::cache("categories", "view", "item");
         $d2->setValue("nr", $d->getValue("catnr"));
         $d2->get();
         $v["value"][$nr]=$d2->getValue("module", $nr);
        }

        $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

        if(isset($this->config["modulesStyle"])&&$this->config["modulesStyle"]=="list")
         $r.=$f->listview("multiple", $fieldname."[".$nr."][]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select("multiple", $fieldname."[".$nr."][]", $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "localfile":
       {
        $options=$d->listOptions[$name];
        array_unshift($options, "");

        if(isset($this->config["localfileStyle"])&&$this->config["localfileStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "localfiles":
       {
        $options=$d->listOptions[$name];

        $r.=$f->input("hidden", $fieldname."[".$nr."]", "");

        if(isset($this->config["localfilesStyle"])&&$this->config["localfilesStyle"]=="list")
         $r.=$f->listview("multiple", $fieldname."[".$nr."][]", $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select("multiple", $fieldname."[".$nr."][]", $options, $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "localdirectory":
       {
        $options=$d->listOptions[$name];
        array_unshift($options, "");

        if(isset($this->config["localfileStyle"])&&$this->config["localfileStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "plugin":
       {
        if(isset($this->config["pluginStyle"])&&$this->config["pluginStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "project":
       {
        if(isset($this->config["projectStyle"])&&$this->config["projectStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $d->listOptions[$name], $v["value"][$nr], $class, false, $extra);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "action":
       {
        $options=$d->actionList;
        array_unshift($options, "");

        if(isset($this->config["actionStyle"])&&$this->config["actionStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "extra":
       {
        $options=Array("");
        foreach($d->extraList as $actions)
         foreach($actions as $extra)
          if(!in_array($extra, $options))
           $options[]=$extra;

        if(isset($this->config["extraStyle"])&&$this->config["extraStyle"]=="list")
         $r.=$f->listview(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);
        else
         $r.=$f->select(false, $fieldname, $options, $v["value"][$nr], $class, false, $extra);

        unset($options);

        $cmsOptions=""; //enable cmsOptions
        break;
       }

       case "rights":
       {
        if($d->module=="rights")
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        else
         $r.=$this->rights($fieldname, $v["value"][$nr], $class)."\n";

        break;
       }

       case "html":
       case "text":
       {
        if($v["max"]>=256)
        {
         if($v["type"]=="html"&&$d->check("properties", "wysiwyg", $name))
          $r.=$f->wysiwyg($fieldname, $v["value"][$nr], $class, false, $extra)."\n";
         else
         {
          $r.=$f->input("textarea", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

          if(isset($d->listOptions[$name]))
           $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");
         }

         break;
        }

        //don't break; use logic below
       }

       case "url":
       case "title":
       case "hostname":
       case "domain":
       case "email":
       case "phone":
       case "regexp":
       case "filename":
       case "alphabetic":
       {
        if($v["max"]>0)
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra."maxlength=\"".$v["max"]."\"")."\n";
        else
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

        if(isset($d->listOptions[$name]))
         $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");

        break;
       }

       case "nr":
       {
        if($d->check("properties", "dropdown", $name)&&$v["max"]>0)
        {
         $r.=$f->selectStart(false, $fieldname, $class, false, $extra)."\n";

         for($c=$v["min"];$c<=$v["max"];$c++)
         {
          if($c==$v["value"][$nr])
           $r.=$f->selectOption($c, false, "selected")."\n";
          else
           $r.=$f->selectOption($c)."\n";
         }
         $r.=$f->selectEnd()."\n";
        }
        else
        {
         $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

         if(isset($d->listOptions[$name]))
          $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");
        }

        break;
       }

       case "float":
       case "price":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

        if(isset($d->listOptions[$name]))
         $r.=$f->select(false, "", $d->listOptions[$name], $v["value"][$nr], "form quicklist", false, "onchange=\"this.form['".$fieldname."'].value=this.options[this.selectedIndex].value; this.form['".$fieldname."'].onchange();\"");

        break;
       }


       case "md5":
       case "password":
       {
        if(
         !preg_match("/^verify.+$/", $name)&& //non verify field
         ( //add, edit, change password
          $d->action=="add"||
          $d->action=="edit"||
          ($d->action=="user"&&$d->extra=="passwordlower")
         )
        )
        {
         $cmsOptions=""; //enable cmsOptions

         //generate password requirements
         $placeholder=Array();
         if($v["min"])
          $placeholder[]=$v["min"]." long";
         if($_config["passwordupper"])
          $placeholder[]=$_config["passwordupper"]." upper";
         if($_config["passwordnumbers"])
          $placeholder[]=$_config["passwordnumbers"]." numbers";
         if($_config["passwordspecials"])
          $placeholder[]=$_config["passwordspecials"]." special";

         if($placeholder)
          $placeholder=" placeholder=\"".implode(", ", $placeholder)."\"";
        }
        else
         $placeholder="";

        $r.=$f->input("password", $fieldname, (isset($v["value"][$nr])?$v["value"][$nr]:""), $class, false, $extra.$placeholder)."\n";
        break;
       }

       case "captcha":
       {
        $captcha=new Captcha();

        if(request("captchaRefresh"))
         $captcha->random();

        $r.=$this->l->t("Please enter the following ".$captcha->length." characters in the field below")."<br/>";
        $r.="<img src=\"".$_path["root"]."captcha.php?".rand(0,32768)."\" alt=\"Captcha\" />\n"; //random hack for IE (6+8) caching problem...
        $r.=$f->input("hidden", "captchaRefresh", 0)."<a href=\"#\" onclick=\"this.previousSibling.value=1; this.previousSibling.form.submit(); return false;\" title=\"".$this->l->t("New image")."\">".$this->l->t("New image")."</a><br/>\n";
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";

        unset($captcha);
        break;
       }


       case "boolean":
       {
        //IE onchange triggered onblur after onclick: reload fix: onclick triggers onchange
        $r.=$f->input("hidden", $fieldname, 0, false, false, $extra);
        $r.=$f->checkbox($v["value"][$nr], $v["title"], $fieldname, 1, $class, false, $extra.($d->check("properties", "reload", $name)?"onclick=\"this.onchange();\"":""))."\n";
        break;
       }

       case "enum":
       {
        foreach($d->listOptions[$name] as $k)
        {
         if($v["value"][$nr]===false){$v["value"][$nr]=$k;}

         if($v["value"][$nr]==$k)
          $r.=$f->radio(true, $k, $fieldname, $k, $class, false, $extra)."\n";
         else
          $r.=$f->radio(false, $k, $fieldname, $k, $class, false, $extra)."\n";
        }

        break;
       }


       case "file":
       case "image":
       {
        $r.=$f->input("file", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }

       case "date":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra." placeholder=\"".$v["format"][$d->dataformat]["dateformat"]."\"")." ";

        if($this->config["showDateformat"])
         $r.=$this->l->t("Format").": <a href=\"http://php.net/date\" rel=\"external\" title=\"Format description\">".$v["format"][$d->dataformat]["dateformat"]."</a>\n";

        break;
       }

       case "time":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)." ";

        if($this->config["showDateformat"])
         $r.=$this->l->t("Format").": <a href=\"http://php.net/date\" rel=\"external\" title=\"Format description\">".$v["format"][$d->dataformat]["timeformat"]."</a>\n";

        break;
       }

       case "datetime":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra." placeholder=\"".$v["format"][$d->dataformat]["datetimeformat"]."\"")." ";

        if($this->config["showTimezone"])
         $r.=$this->l->t("Timezone").": ".$v["format"][$d->dataformat]["timezone"].($this->config["showDateformat"]?", ":"");
        if($this->config["showDateformat"])
         $r.=$this->l->t("Format").": <a href=\"http://php.net/date\" rel=\"external\" title=\"Format description\">".$v["format"][$d->dataformat]["datetimeformat"]."</a>\n";

        break;
       }


       case "version":
       {
        $r.=$f->input("text", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       case "label":
       {
        if(isset($v["value"][$nr]))
         $r.=htmlentities($v["value"][$nr])."\n";
        break;
       }
       case "submit":
       {
        $r.=$f->input("submit", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       default:
       {
        $r.=$f->input("fake", $fieldname, $d->getValue($name, $nr), $class)."\n";
        break;
       }
      }
     }
    }
    else //fake fields
    {
     if(isset($d->reference[$name]))
     {
      $dr=Data::cache($d->reference[$name]["module"], "view", "item");
      $dr->setValue("nr", $v["value"][$nr]);
      $dr->get();

      $cms=new CMS();
      $cms->setDataForm($dr, $this->form);

      foreach($dr->check("use", "form") as $name2)
       if($dr->check("use", "show", $name2))
        if(!isset($dr->reference[$name2]))
         if(!$dr->check("properties", "default", $name2))
          for($nr2=0;$nr2<$dr->vmax;$nr2++)
           $r.=$cms->field($name2, "fake", $nr2);

      unset($cms, $dr);
     }
     else
     {
      switch($v["type"])
      {
       case "boolean":
        $r.=$f->input("fake", $fieldname, ($v["value"][$nr]?$this->l->t("Yes"):$this->l->t("No")))."\n";
       break;


       case "nr":
       {
        $r.=$f->input("fake", $fieldname, $d->getValue($name, $nr))."\n";
        break;
       }


       case "image":
       {
        $r.="<img src=\"".$_path["HTML"]["root"]."image.php?m=".$d->module."&amp;field=".$name."&amp;nr=".$d->getValue("nr", $nr)."\" style=\"width: 100px;\" alt=\"".$v["title"]."\">";
        break;
       }

       case "file":
       {
        $r.="<a href=\"".$_path["HTML"]["root"]."file.php?m=".$d->module."&amp;field=".$name."&amp;nr=".$d->getValue("nr", $nr)."\" title=\"Download ".$d->getValue($name, $nr)."\">".$d->getValue($name, $nr)."</a>";
        break;

       }

       case "multilist":
       {
        if($d->getValue($name, $nr))
         $r.=$f->input("fake", $fieldname, implode(", ", $d->getValue($name, $nr)))."\n";
        else
         $r.=$f->input("fake", $fieldname, "")."\n";

        break;
       }

       case "modules":
       {
        if($d->getValue($name, $nr))
         $r.=$f->input("fake", $fieldname, implode(", ", $d->getValue($name, $nr)))."\n";
        else
         $r.=$f->input("fake", $fieldname, "")."\n";

        break;
       }

       case "localfiles":
       {
        if($d->getValue($name, $nr))
         $r.=$f->input("fake", $fieldname, implode(", ", $d->getValue($name, $nr)))."\n";
        else
         $r.=$f->input("fake", $fieldname, "")."\n";

        break;
       }


       case "label":
       {
        $r.=htmlentities($v["value"][$nr])."\n";
        break;
       }
       case "submit":
       {
        $r.=$f->input("submit", $fieldname, $v["value"][$nr], $class, false, $extra)."\n";
        break;
       }


       default:
       {
        $r.=$f->input("fake", $fieldname, $d->getValue($name, $nr))."\n";
        break;
       }
      }
     }
    }
   }
   else
   {
    if($name=="reload_trigger")
     $r.=$f->input("hidden", $fieldname, "")."\n";
    else
    {
     $value=$d->getValue($name, $nr);

     if(!is_array($value))
      $r.=$f->input("hidden", $fieldname, $value)."\n";
     else
      foreach($value as $k => $v)
       $r.=$f->input("hidden", $fieldname."[0][".$k."]", $v)."\n";
    }
   }

   return $r;
  }

  function rights($name, $value, $class)
  {
   $r="";
   $f=&$this->form;
   $d=&$this->data;

   if(!$value)
    $value=$d->rights2bin($d->field[$name]["defaultvalue"]);

   if(!$this->javascriptRights)
    $r.=$this->javascriptRights();

   $r.=$f->input("hidden", $name, $value, "form", "rights_".$name);

   $keys=Array();
   if($d->module=="categories"||$d->module=="groupRights"||$d->module=="userRights")
    $types=Array("Category", "Content");
   else
    $types=Array("Content");

   foreach($types as $type)
   {
    $r.=$this->l->t($type).":<br/>\n";
    foreach($d->rights[strtolower($type)] as $k => $v)
    {
     if(isset($value[$v])&&$value[$v]==1)
      $checked=true;
     else
      $checked=false;

     if(!in_array($v, $keys))
     {
      $r.=$f->checkbox($checked, $this->l->t($k), "", $v, $class, false, "onchange=\"setrights('rights_".$name."', this);\"");
      $keys[]=$v;
     }
    }
    $r.="<br/>\n";
   }

   return $r;
  }


  function javascriptRights()
  {
   $this->javascriptRights=true;

   ob_start();
   ?>
   <script type="text/javascript"><!--
    function setrights(id, rightObj)
    {
     var obj=document.getElementById(id);
     var newString="";

     for(c=0;c<obj.value.length;c++)
     {
      if(c==rightObj.value)
      {
       if(rightObj.checked)
        newString=newString+"1";
       else
        newString=newString+"0";
      }
      else
       newString=newString+obj.value.charAt(c);
     }

     obj.value=newString;
    }
   --></script>
   <?
   $r=ob_get_contents();
   ob_end_clean();
   return $r;
  }

 }
?>
