<?

 /*****************************************************************************
 * Filename: include/config.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 if(!isset($_jses["config"]))
  $_jses["config"]=Array();

 $_config=&$_jses["config"];

 class Config
 {
  function Config()
  {
   $this->e=new Error();
   $this->config=Array();
  }


  function reload()
  {
   global $_config;

   $_config["reload"]=true;

   return Config::load();
  }

  function load()
  {
   global $_jses, $_config, $_path;

   //load config if not yet done, on reload or if previous load failed
   if(!$_config||(isset($_config["reload"])&&$_config["reload"])||(isset($_config["failed"])&&$_config["failed"]))
   {
    if(isset($_config["usernr"])) //keep user logged in on reload and failures
    {
     if(isset($_config["userbackend"]))
      $_config=Array("usernr" => $_config["usernr"], "userbackend" => $_config["userbackend"]);
     else
      $_config=Array("usernr" => $_config["usernr"]);
    }
    else //normal load, get everything
     $_config=Array();

    $_config["language"]="en";//set default language for language.php

    require_once($_path["include"]."data.php");

    $customConfigModules=Array();


    //general config
    $dc=new Data("config", "view", "all");
    $customConfigModules[]=$dc->module;

    if(!$dc->get()||!$dc->next()) //problem in loading config, make fake config
    {
     foreach($dc->check("use", "show") as $k)
      if($k!="r")
       if(!isset($_config[$k])||$k=="language") //only set fields (except language) that aren't set yet, ie: usernr will not be overwritten on reload and failures
        $_config[$k]=$dc->field[$k]["defaultvalue"];

     $_config["failed"]=true; //keep reloading config till successful
     $_config["useModRewrite"]=false; //make sure modRewrite is handled properly
     $_config["language"]=strtolower($_config["language"]);
     $_config["title"]="Project configuration is invalid, please fix ASAP!";

     return false;
    }
    else
    {
     foreach($dc->check("use", "show") as $k)
      if($k!="r")
       if(!isset($_config[$k])||$k=="language") //only set fields (except language) that aren't set yet, ie: usernr will not be overwritten on reload and failures
        $_config[$k]=$dc->v[$k];

     $_config["language"]=strtolower($_config["language"]);

     //reload group rights
     $users=Data::cache($_config["usermodule"], "view", "item");
     $_config["groupnrs"]=$users->moduleObject->getGroups($_config["usernr"]);
     unset($users);
    }

    //API config: only done here (instead of below) because the APIs doesn't match config key name
    $dc=new Data("configAPIs", "view", "all");
    $customConfigModules[]=$dc->module;

    if($dc->get()&&$dc->next())
    {
     $_config["api"]=Array();
     foreach($dc->check("use", "show") as $k)
      if($k!="r")
       $_config["api"][$k]=$dc->v[$k];
    }


    //Modules configs, multiple rows
    $_config["modules"]=Array();
    $dc=new Data("configModules", "view", "all");
    $customConfigModules[]=$dc->module;

    if($dc->get())
    {
     while($dc->next())
     {
      $_config["modules"][$dc->v["module"]]=Array();
      foreach($dc->check("use", "show") as $k)
      {
       if($k!="r")
       {
        if($dc->v[$k]===""||(($k=="decimals"||$k=="formdecimals")&&$dc->v[$k]==-1)) //empty, use config
         $_config["modules"][$dc->v["module"]][$k]=&$_config[$k];
        else
         $_config["modules"][$dc->v["module"]][$k]=$dc->v[$k];
       }
      }
     }
    }

    //Module fields configs, multiple rows and are placed under applicable module
    $dc=new Data("configFields", "view", "all");
    $customConfigModules[]=$dc->module;

    if($dc->get())
    {
     while($dc->next())
     {
      if(!isset($_config["modules"][$dc->v["module"]]))
       $_config["modules"][$dc->v["module"]]=Array();

      $_config["modules"][$dc->v["module"]]["field"]=Array();

      foreach($dc->check("use", "show") as $k)
      {
       if($k!="r")
       {
        if($dc->v[$k]===""||(($k=="decimals"||$k=="formdecimals")&&$dc->v[$k]==-1)) //empty, use config/moduleConfig
        {
         if(isset($_config["modules"][$dc->v["module"]][$k]))
          $_config["modules"][$dc->v["module"]]["field"][$dc->v["field"]][$k]=&$_config["modules"][$dc->v["module"]][$k];
         else
          $_config["modules"][$dc->v["module"]]["field"][$dc->v["field"]][$k]=&$_config[$k];
        }
        else
         $_config["modules"][$dc->v["module"]]["field"][$dc->v["field"]][$k]=$dc->v[$k];
       }
      }
     }
    }


    //Repositories config, multiple rows
    $_config["repositories"]=Array();
    $dc=new Data("configRepositories", "view", "all");
    $customConfigModules[]=$dc->module;

    if($dc->get())
    {
     while($dc->next())
     {
      $_config["repositories"][$dc->v["name"]]=Array();
      foreach($dc->check("use", "show") as $k)
      {
       if($k!="r"&&$k!="name")
        $_config["repositories"][$dc->v["name"]][$k]=$dc->v[$k];
      }
     }
    }


    //Sites config, multiple rows
    $_config["sites"]=Array();
    $dc=new Data("configSites", "view", "all");
    $customConfigModules[]=$dc->module;

    if($dc->get())
    {
     while($dc->next())
     {
      $_config["sites"][$dc->v["domain"]]=Array();
      foreach($dc->check("use", "show") as $k)
      {
       if($k!="r"&&$k!="domain")
        $_config["sites"][$dc->v["domain"]][$k]=$dc->v[$k];
      }
     }
    }


    //Other "generic" configs
    foreach(modules() as $module)
    {
     if(strpos($module, "config")===0&&!in_array($module, $customConfigModules)) //should start with config prefix and shouldn't be already gathered above
     {
      $dc=new Data($module, "view", "all");
      $key=strtolower(str_replace("config", "", $dc->module));

      if(isset($_config[$key]))
      {
       $dc->e->add("Config key: \"".$key."\" for module: \"".$dc->module."\" already defined by another config module", "module", "Please rename: \"".$dc->module."\"");
       continue;
      }

      if($dc->get()&&$dc->next())
      {
       $_config[$key]=Array();
       foreach($dc->check("use", "show") as $k)
        if($k!="r")
         $_config[$key][$k]=$dc->v[$k];
      }
     }
    }

    //detect if mod rewrite is active
    if(!isset($_jses["modRewriteActive"]))
    {
     if(isset($_SERVER["REDIRECT_URL"]))
      $_jses["modRewriteActive"]=true;
     else if(function_exists("apache_get_modules")&&in_array("mod_rewrite", apache_get_modules())) //mod_rewrite loaded
     {
      require_once($_path["include"]."fs.php");
      $_jses["modRewriteActive"]=FS::grep("/RewriteEngine\s+on/i", $_path["root"].".htaccess"); //can still be deactivated or .htaccess is ignored in global apache config
     }
     else
      $_jses["modRewriteActive"]=false;
    }

    //set whether mod rewrite should actually be used
    switch($_config["modRewrite"])
    {
     case "Yes":
     {
      $_config["useModRewrite"]=true; //this will get you into trouble if mod rewrite is not active
      break;
     }
     case "No":
     {
      $_config["useModRewrite"]=false;
      break;
     }
     default:
     case "Auto":
     {
      $_config["useModRewrite"]=$_jses["modRewriteActive"];
      break;
     }
    }
   }

   return true;
  }


  function merge($key, $config=false)
  {
   global $_config;

   if(!$config) //no config parameter
    $this->config=Array(); //use empty config
   else if(!is_array($config)) //illegal config parameter
   {
    $this->e->add($key." config merge failed: config parameter should be an array", "system"); //don't return false, just use default/configured values
    $this->config=Array(); //use empty config
//    return false;
   }
   else //use config parameter
    $this->config=$config;

   $var=strtolower($key);

   //fill in the blanks in the config array
   if(isset($_config[$var])&&is_array($_config[$var])) //_config is set: use configured values
   {
    if(!$this->config)
     $this->config=$_config[$var];
    else
    {
     foreach($_config[$var] as $field => $value)
      if(!isset($this->config[$field]))
       $this->config[$field]=$value;
    }
   }
   else //no $key _config: use default values
   {
    if($key=="API")
     $module="configAPIs";
    else
     $module="config".$key;

    if(!in_array($module, modules()))
    {
     $this->e->add($key." config merge failed: unknown config type", "system");
     return false;
    }

    $d=Data::cache($module, "view", "item", Array("skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true)); //don't fully init object, only default values needed

    if($d->e->any())
     $this->e->add($d->e);

    foreach($d->check("use", "show") as $field)
     if(!isset($this->config[$field]))
      $this->config[$field]=$d->getDefaultValue($field);
   }

   return true;
  }

 }
?>
