<?

 /*****************************************************************************
 * Filename: include/csv.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."email.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."fs.php");

 class CSV 
 {
  function CSV()
  {
   $this->fs=new FS();

   $this->e=new Error();
   $this->e->l->read("csv");

   $this->filename="export.csv";

   $this->separators(";", "\"", "\n\r");
   $this->header(false);

   $this->clear();
  }

  function clear()
  {
   $this->vc=0;
   $this->vmax=0;
   $this->v=Array();
   $this->grid=Array();
  }

  function separators($split, $quote, $linebreak, $subSplit=false)
  {
   if($split==$quote)
   {
    $this->e->add("Split and quote character can't be the same", "error");
    return false;
   }
   if($split==$linebreak)
   {
    $this->e->add("Split and linebreak character can't be the same", "error");
    return false;
   }
   if($quote==$linebreak)
   {
    $this->e->add("Quote and linebreak character can't be the same", "error");
    return false;
   }

   if($subSplit!==false)
   {
    if($quote) //quote
    {
     if($subSplit==$quote)
     {
      $this->e->add("Sub split and quote character can't be the same", "error");
      return false;
     }
    }
    else //non quoted
    {
     if($subSplit==$split)
     {
      $this->e->add("Sub split and split character can't be the same", "error");
      return false;
     }
     if($subSplit==$linebreak)
     {
      $this->e->add("Sub split and linebreak character can't be the same", "error");
      return false;
     }
    }
   }
   else //auto subSplit
   {
    if($quote) //quoted, use linebreak
     $subSplit=$linebreak;
    else if($split==",") //non quoted, use non split
     $subSplit="; ";
    else
     $subSplit=", ";
   }

   $this->split=$split;
   $this->quote=$quote; 
   $this->linebreak=$linebreak; 
   $this->subSplit=$subSplit;

   return true;
  }

  function header($headerLine)
  {
   $this->headerLine=($headerLine?true:false); //use first line as array keys when reading and manipulating and output/write array keys as first line
   return true;
  }

  function detectLineBreaks($input)
  {
   if($linebreak=Email::detectLineBreaks($input))
   {
    $this->linebreak=$linebreak;
    return true;
   }

   return false;
  }

  function data(&$d, $fields=false, $useTitlesHeader=false)
  {
   $this->clear();

   $this->filename=$d->module.".".$d->extra.".csv";

   if($fields)
   {
    foreach($fields as $field)
    {
     if(!isset($d->field[$field]))
     {
      $this->e->add("Unknown field: \"".$field."\"", "error");
      return false;
     }
    }
   }
   else
    $fields=$d->check("use", "show");

   if($useTitlesHeader)
    $this->header(true);

   $l=sizeof($fields);
   while($d->next())
   {
    $x=0;
    $tmp=Array();
    for($c=0;$c<$l;$c++)
    {
     if(!isset($d->v[$fields[$c]])) 
      continue;

     if($useTitlesHeader) //title header
      $key=$d->field[$fields[$c]]["title"];
     else if($this->headerLine) //field header
      $key=$fields[$c];
     else //no header
      $key=$x++;

     if(is_array($d->v[$fields[$c]]))
     {
      if(isset($d->reference[$fields[$c]]))
      {
       foreach($d->reference[$fields[$c]]["data"]->check("use", "show") as $k2)
       {
        if(!isset($d->v[$fields[$c]][$k2]))
         continue;

        if(is_array($d->v[$fields[$c]][$k2]))
         $tmp[$key]=implode($this->subSplit, $d->v[$fields[$c]][$k2]);
        else
         $tmp[$key]=$d->v[$fields[$c]][$k2];
       }
      }
      else
       $tmp[$key]=implode($this->subSplit, $d->v[$fields[$c]]);
     }
     else
      $tmp[$key]=$d->v[$fields[$c]];
    }

    if(!$this->addLine($tmp))
    {
     $this->clear();
     return false;
    }
   }

   return true;
  }

  function checkLine($values)
  {
   foreach($values as $value)
   {
    if(is_array($value))
    {
     $this->e->add("Multi dimensional array lines not allowed", "error");
     return false;
    }
   }

   if(!$this->headerLine) //indexed
   {
    if(array_type($values)!="indexed")
    {
     $this->e->add("Line should be indexed array for non-header mode", "error");
     return false;
    }

    if($this->grid) //check size with first line
    {
     if(sizeof($this->grid[0])!=sizeof($values))
     {
      $this->e->add("Number of entries in line don't match previous line(s)", "warning");
     }
    }
   }
   else //associative
   {
    if(array_type($values)!="associative")
    {
     $this->e->add("Line should be associative array for header mode", "error");
     return false;
    }

    if($this->grid) //check keys with first line
    {
     if(array_keys($this->grid[0])!=array_keys($values))
     {
      $this->e->add("Line keys don't match previous line(s)", "error");
      return false;
     }
    }
   }

   return true;
  }

  function addLine()
  {
   $values=func_get_args();
   if(sizeof($values)==1&&is_array($values[0]))
    $values=$values[0];

   if($this->quote) //quote escape 
   {
    $values=preg_replace("/([^\\\])".preg_quote($this->quote)."/", "\\1\\".$this->quote, $values);
    $values=preg_replace("/([^\\\])".preg_quote($this->quote)."/", "\\1\\".$this->quote, $values); //run twice to make sure "" gets properly quoted
   }

   if(!$this->checkLine($values))
    return false;

   $this->grid[$this->vmax++]=$values;
   return true;
  }

  function editLine($line, $array)
  {
   if(!isset($this->grid[$line]))
   {
    $this->e->add("Can't edit line ".$line." in CSV: \"".$this->filename."\"");
    return false;
   }

   $values=func_get_args();
   array_shift($values); //remove line param
   if(sizeof($values)==1&&is_array($values[0]))
    $values=$values[0];

   if($this->quote) //quote escape 
   {
    $values=preg_replace("/([^\\\])".preg_quote($this->quote)."/", "\\1\\".$this->quote, $values);
    $values=preg_replace("/([^\\\])".preg_quote($this->quote)."/", "\\1\\".$this->quote, $values); //run twice to make sure "" gets properly quoted
   }

   if(!$this->checkLine($values))
    return false;

   $this->grid[$line]=$array;
   return true;
  }

  function deleteLine($line)
  {
   if(!isset($this->grid[$line]))
   {
    $this->e->add("Can't delete line ".$line." in CSV: \"".$this->filename."\"");
    return false;
   }

   $l=sizeof($this->grid);
   for($c=$line+1;$c<$l;$c++)
    $this->grid[$c-1]=$this->grid[$c];

   unset($this->grid[$l-1]);
   $this->vmax--;

   return true;
  }

  function next()
  {
   if($this->vc>=$this->vmax)
   {
    $this->vc=0;
    return false;
   }

   $this->v=preg_replace("/\\\\".preg_quote($this->quote)."/", $this->quote, $this->grid[$this->vc]); //quote unescape

   $this->vc++;
   return true;
  }

  function read($file, $headerLine=false, $detectLineBreaks=true)
  {
   $this->clear();

   $this->filename=$file;
   
   if(($input=$this->fs->read($this->filename))===false)
   {
    $this->e->add($this->fs->e);
    return false;
   }

   return $this->parse($input, $headerLine, $detectLineBreaks);
  }

  function parse($input, $headerLine=false, $detectLineBreaks=true)
  {
   $this->header($headerLine);

   if($detectLineBreaks) //detect used line breaks in input
    if(!$this->detectLineBreaks($input))
     return false;

   if(preg_match_all("/((".preg_quote($this->quote).".*?(?<!\\\)".preg_quote($this->quote).preg_quote($this->split)."?)+)".preg_quote($this->linebreak)."/s", $input, $lines))
    $lines=$lines[1];
   else
   {
    $lines=explode($this->quote.$this->linebreak, $input); 

    if(trim($lines[$l-1])) //last line is bogus
     $l--;
   }

   $l=sizeof($lines);

   $headers=false;
   for($c=0;$c<$l;$c++) //loop through all lines
   {
    $line=explode($this->quote.$this->split.$this->quote, $lines[$c]); //split all values

    $l2=sizeof($line); //get number of values on line
    $line[0]=ltrim($line[0], $this->quote); //remove start quote
    $line[$l2-1]=rtrim($line[$l2-1], $this->split); //remove end split
    $line[$l2-1]=rtrim($line[$l2-1], $this->quote); //remove end quote

    if($l2==1&&!$line[0]) //empty line, only quotes
     continue; //skip

    $line=str_replace("\\".$this->quote, $this->quote, $line); //remove escapes from quotes

    if($this->headerLine) //associative
    {
     if(!$headers) //get header line
      $headers=$line; 
     else //normal lines
     {
      $x=0;
      $line2=Array();
      foreach($headers as $header) //create associative line
       if(isset($line[$x])) //addLine will fail on missing entry, 
        $line2[$header]=$line[$x++];

      if(!$this->addLine($line2))
      {
       $this->clear();
       return false;
      }
     }
    }
    else
    {
     if(!$this->addLine($line))
     {
      $this->clear();
      return false;
     }
    }
   }

   return true;
  }

  function write($file=false)
  {
   if($this->e->any("fatal"))
    return false;

   if($file)
    $this->filename=$file;

   if($this->fs->write($this->filename, $this->output(false), "overwrite")===false)
   {
    $this->e->add($this->fs->e);
    return false;
   }

   return true;
  }

  function output($headers=true)
  {
   if($this->e->any("fatal"))
    return $this->e->show("fatal");

   if($this->vmax&&$this->headerLine)
    $output=$this->quote.implode($this->quote.$this->split.$this->quote, array_keys($this->grid[0])).$this->quote.$this->linebreak;
   else
    $output="";

   for($c=0;$c<$this->vmax;$c++)
    $output.=$this->quote.implode($this->quote.$this->split.$this->quote, $this->grid[$c]).$this->quote.$this->linebreak;

   if($headers)
   {
    header('Content-type: text/comma-separated-values');
    header('Content-length: '.strlen($output));
 
    if($this->filename)
     header("Content-Disposition: attachment; filename=\"".$this->filename."\"");
   }

   return $output;
  }

 }
?>
