<?

 /*****************************************************************************
 * Filename: include/dt.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");

 class DT
 {
  function DT()
  {
   $this->e=new Error();
   $this->clear();
  }

  function clear()
  {
  }


  function isInitialized()
  {
   if(!isset($this))
    return false;

   if(get_class($this)=="DT") //object initialized
    return true;

   $tmp=$this;
   while($tmp=get_parent_class($tmp)) //lookup if parent class is DT
    if($tmp=="DT") //object initialized
     return true;

   return false;
  }

  function dateExtract($date, $format)
  {
   //array values
   if(is_array($date))
   {
    if(DT::isInitialized()) //object initialized
    {
     foreach($date as $k => $v)
      $date[$k]=$this->dateExtract($v, $format);
    }
    else
    {
     foreach($date as $k => $v)
      $date[$k]=DT::dateExtract($v, $format);
    }

    return $date;
   }

   //current format support:
   $formatConvert=Array(
    "/d/" => "(\d{2})", //days
    "/j/" => "(\d{1,2})",
    "/m/" => "(\d{2})", //months
    "/n/" => "(\d{1,2})",
    "/Y/" => "(\d{4})", //years
    "/y/" => "(\d{2})",
    "/H/" => "(\d{2})", //hours
    "/G/" => "(\d{1,2})",
    "/i/" => "(\d{2})", //minutes
    "/s/" => "(\d{2})", //seconds
    "/W/" => "(\d{1,2})", //weeks
    "/z/" => "(\d{1,3})", //days (per year)
    "/U/" => "(\d+)", //unix timestamp
   );

   $regexp=preg_replace(array_keys($formatConvert), $formatConvert, preg_quote($format));

   if(!preg_match_all("/[a-z]/i", preg_quote($format), $keys))
    return false;

   if(!preg_match("/".$regexp."/", $date, $values))
    return false;

   //ugly workaround for pregmatch
   $keys=$keys[0];
   for($c=1;$c<sizeof($values);$c++)
    $tmp[]=$values[$c];
   $values=$tmp;

   if(($key=array_search("U", $keys))!==false) //unix timestamp (seconds sinds 1970, in UTC)
    list($year, $month, $day, $hour, $minute, $second)=explode(" ", date("Y m d H i s", $values[$key])); //overwrite all values
   else //no unix timestamp, parse values
   {
    if(($key=array_search("d", $keys))!==false) //numeric day of the month
     $day=$values[$key];
    else if(($key=array_search("j", $keys))!==false) //numeric day of the month without leading zero
     $day=$values[$key];
    else
     $day=1;

    if(($key=array_search("m", $keys))!==false) //numeric month
     $month=$values[$key];
    else if(($key=array_search("n", $keys))!==false) //numeric month without leading zero
     $month=$values[$key];
    else
     $month=1;

    if(($key=array_search("Y", $keys))!==false) //4 digit year
     $year=$values[$key];
    else if(($key=array_search("y", $keys))!==false) //2 digit year
    {
     $year=$values[$key];
     if($year>=50) //we use it from 1950 till 2049
      $year+=1900;
     else
      $year+=2000;
    }
    else
     $year=1970;

    if(($key=array_search("H", $keys))!==false) //24 hours
     $hour=$values[$key];
    else if(($key=array_search("G", $keys))!==false) //24 hours without leading zero
     $hour=$values[$key];
  //implement 12 hour clock (h+g) with am/pm (a/A)
    else
     $hour=0;

    if(($key=array_search("i", $keys))!==false) //2 digit minute
     $minute=$values[$key];
    else
     $minute=0;

    if(($key=array_search("s", $keys))!==false) //2 digit second
     $second=$values[$key];
    else
     $second=0;

    if(($key=array_search("W", $keys))!==false) //numeric week of the year
    {
     $date=(($values[$key]-1)*7); //determine day of the year
     $date=mktime(0, 0, 0, 1, $date, $year); //determine date based on set year

     if(date("W", mktime(0,0,0,12,31,$year))==53) //53 weeks year
     {
      if(($weekday=date("N", $date))>1) //not monday
       $date-=86400*($weekday-1); //round date to previous monday
     }
     else //52 weeks year
     {
      if(($weekday=date("N", $date))>1) //not monday
       $date+=86400*(8-$weekday); //round date to next monday
     }

     if(array_search("d", $keys)===false&&array_search("j", $keys)===false) //day not set
      $day=date("d", $date); //determine day of the month

     if(array_search("m", $keys)===false&&array_search("n", $keys)===false) //month not set
      $month=date("m", $date); //determine month

     $year=date("Y", $date); //overwrite year in case start of the week false in another year
    }

    if(($key=array_search("z", $keys))!==false) //numeric day of the year
    {
     $date=$values[$key];
     $date=mktime(0, 0, 0, 1, $date, $year); //determine date based on set year

     if(array_search("d", $keys)===false&&array_search("j", $keys)===false) //day not set
      $day=date("d", $date); //determine day of the month

     if(array_search("m", $keys)===false&&array_search("n", $keys)===false) //month not set
      $month=date("m", $date); //determine month

     $year=date("Y", $date); //overwrite year in case start of the week false in another year
    }
   }

   return Array(
    "year" => (int)$year,
    "month" => (int)$month,
    "day" => (int)$day,
    "hour" => (int)$hour,
    "minute" => (int)$minute,
    "second" => (int)$second);
  }

  function dateConvert($date, $inputformat, $outputformat=false)
  {
   //array values
   if(is_array($date))
   {
    if(DT::isInitialized()) //object initialized
    {
     foreach($date as $k => $v)
      $date[$k]=$this->dateConvert($v, $inputformat, $outputformat);
    }
    else
    {
     foreach($date as $k => $v)
      $date[$k]=DT::dateConvert($v, $inputformat, $outputformat);
    }

    return $date;
   }

   if(DT::isInitialized()) //object initialized
    $date=$this->dateExtract($date, $inputformat);
   else
    $date=DT::dateExtract($date, $inputformat);

   if(!$date)
    return false;

   $output=mktime($date["hour"], $date["minute"], $date["second"], $date["month"], $date["day"], $date["year"]);

   if($outputformat)
    $output=date($outputformat, $output);

   return $output;
  }

  function dateDelta($date, $delta, $interval, $format)
  {
   //array values
   if(is_array($date))
   {
    if(DT::isInitialized()) //object initialized
    {
     foreach($date as $k => $v)
      $date[$k]=$this->dateDelta($v, $delta, $interval, $format);
    }
    else
    {
     foreach($date as $k => $v)
      $date[$k]=DT::dateDelta($v, $delta, $interval, $format);
    }

    return $date;
   }

   if(DT::isInitialized()) //object initialized
    $date=$this->dateExtract($date, $format);
   else
    $date=DT::dateExtract($date, $format);

   if(!$date)
    return false;

   switch(strtolower($interval))
   {
    case "second":
    case "seconds":
    {
     $date["second"]+=$delta;
     break;
    }
    case "minute":
    case "minutes":
    {
     $date["minute"]+=$delta;
     break;
    }
    case "hour":
    case "hours":
    {
     $date["hour"]+=$delta;
     break;
    }
    case "day":
    case "days":
    {
     $date["day"]+=$delta;
     break;
    }
    case "week":
    case "weeks":
    {
     $date["day"]+=$delta*7;
     break;
    }
    case "month":
    case "months":
    {
     $date["month"]+=$delta;
     break;
    }
    case "year":
    case "years":
    {
     $date["year"]+=$delta;
     break;
    }
    default:
    {
     if(DT::isInitialized()) //object initialized
      $this->e->add("Unknown date delta interval: ".$interval, "error");

     return $date;
    }
   }


   $date=mktime($date["hour"], $date["minute"], $date["second"], $date["month"], $date["day"], $date["year"]);

   return date($format, $date);
  }


  function timezoneConvert($date, $inputtimezone, $outputtimezone, $dateformat)
  {
   if($inputtimezone==$outputtimezone)
    return $date;

   //array values
   if(is_array($date))
   {
    if(DT::isInitialized()) //object initialized
    {
     foreach($date as $k => $v)
      $date[$k]=$this->timezoneConvert($v, $inputtimezone, $outputtimezone, $dateformat);
    }
    else
    {
     foreach($date as $k => $v)
      $date[$k]=DT::timezoneConvert($v, $inputtimezone, $outputtimezone, $dateformat);
    }

    return $date;
   }

   $timezones=timezone_identifiers_list();
   $current=date_default_timezone_get();

   if(!in_array($inputtimezone, $timezones))
   {
    if(DT::isInitialized()) //object initialized
     $this->e->add("Unknown timezone: ".$inputtimezone, "error");

    return $date;
   }

   if(!in_array($outputtimezone, $timezones))
   {
    if(DT::isInitialized()) //object initialized
     $this->e->add("Unknown timezone: ".$outputtimezone, "error");

    return $date;
   }

   if($current!=$inputtimezone)
    date_default_timezone_set($inputtimezone);

   if(FS::isInitialized()) //object initialized
    $date=$this->dateConvert($date, $dateformat);
   else
    $date=DT::dateConvert($date, $dateformat);

   date_default_timezone_set($outputtimezone);
   $date=date($dateformat, $date);

   if($current!=date_default_timezone_get())
    date_default_timezone_set($current);

   return $date;
  }

 }

?>
