<?

 /*****************************************************************************
 * Filename: include/email.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."config.php");
 require_once($_path["include"]."socket.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."mime.php");
 define("REGEXP_EMAIL_ADDRESS", "/^([a-z0-9_\-\.]+)\@([a-z0-9_\-\.]+)\.([a-z0-9]+)$/i");

 class Email extends Config
 {
  function Email($config=false)
  {
   Config::Config();
   $this->merge("Email", $config);

   $this->mimeVersion=$this->config["mimeVersion"];
   $this->charSet=$this->config["charSet"];
   $this->userAgent=$this->config["userAgent"];
   $this->defaultDomain=$this->config["defaultDomain"];
   $this->wordwrap=$this->config["wordwrap"];

   $this->server=$this->config["server"];
   $this->port=$this->config["port"];
   $this->username=$this->config["username"];
   $this->password=$this->config["password"];

   $this->mime=new Mime();
   $this->boundaryPrefix="--------";
   $this->headerLinebreak="\r\n";
   $this->bodyLinebreak="\r\n";

   $this->clear();
  }

  function server($server, $username=false, $password=false)
  {
   $server=explode(":", $server);
   $this->server=$server[0];
   $this->port=(isset($server[1])?$server[1]:25);

   $this->username=$username;
   $this->password=$password;

   return true;
  }

  function to($address, $name=false)
  {
   if(!preg_match(REGEXP_EMAIL_ADDRESS, $address))
   {
    $this->e->add("Illegal E-mail address: \"".$address."\"", "error");
    return false;
   }

   if($name)
    $this->to[preg_replace("/\"/", "", $name)]=$address;
   else
    $this->to[]=$address;

   return true;
  }

  function cc($address, $name=false)
  {
   if(!preg_match(REGEXP_EMAIL_ADDRESS, $address))
   {
    $this->e->add("Illegal E-mail address: \"".$address."\"", "error");
    return false;
   }

   if($name)
    $this->cc[preg_replace("/\"/", "", $name)]=$address;
   else
    $this->cc[]=$address;

   return true;
  }

  function bcc($address, $name=false)
  {
   if(!preg_match(REGEXP_EMAIL_ADDRESS, $address))
   {
    $this->e->add("Illegal E-mail address: \"".$address."\"", "error");
    return false;
   }

   if($name)
    $this->bcc[preg_replace("/\"/", "", $name)]=$address;
   else
    $this->bcc[]=$address;

   return true;
  }

  function from($address, $name=false)
  {
   if(!preg_match(REGEXP_EMAIL_ADDRESS, $address))
   {
    $this->e->add("Illegal E-mail address: \"".$address."\"", "error");
    return false;
   }

   $this->from=Array();//only one from address

   if($name)
    $this->from[preg_replace("/\"/", "", $name)]=$address;
   else
    $this->from[]=$address;

   return true;
  }

  function subject($subject)
  {
   $this->subject=preg_replace("/\n/", "", $subject);
   return true;
  }

  function message($message, $addAlternative=false, $embeddedHTML=true)
  {
   if(file_exists($message))
   {
    $filename=$message;
    $message=false;
    $mime=$this->mime->filename($filename);
   }
   else
   {
    $filename=false;
    $mime=$this->mime->content($message);
   }

   if($mime=="text/html") //HTML message
   {
    if(!$addAlternative)
    {
     if($embeddedHTML)
     {
      if($this->attachEmbeddedHTML($filename, $message, false)===false)
       return false;
     }
     else
     {
      if($this->attach($filename, $message, false, "text/html", "7bit")===false)
       return false;
     }
    } 
    else
    {
     if($filename)
      $text=strip_tags(preg_replace("!<br/?>!", $this->bodyLinebreak, @file_get_contents($filename)));
     else
      $text=strip_tags(preg_replace("!<br/?>!", $this->bodyLinebreak, $message));

     if($this->wordwrap)
      $text=wordwrap($text, $this->wordwrap, $this->bodyLinebreak);

     if(($entry=$this->attach(false, false, false, "multipart/alternative", false))===false)
      return false;
     if($this->attach(false, $text, $entry, "text/plain", "7bit")===false)
      return false;

     if($embeddedHTML)
     {
      if($this->attachEmbeddedHTML($filename, $message, $entry)===false)
       return false;
     }
     else
     {
      if($this->attach($filename, $message, $entry, "text/html", "7bit")===false)
       return false;
     }
    }
   }
   else //text message
   {
    if(!$addAlternative)
    {
     if($this->wordwrap)
      $message=wordwrap($message, $this->wordwrap, $this->bodyLinebreak);

     if($this->attach($filename, $message, false, "text/plain", "7bit")===false)
      return false;
    }
    else
    {
     $html="<html>".$this->bodyLinebreak;
     $html.=" <head>".$this->bodyLinebreak;

     if($this->subject)
      $html.="  <title>".$this->subject."</title>".$this->bodyLinebreak;

     $html.=" </head>".$this->bodyLinebreak;
     $html.=" <body>".$this->bodyLinebreak;

     if($filename)
      $html.=preg_replace("/".$this->bodyLinebreak."/", "<br/>".$this->bodyLinebreak, @file_get_contents($filename));
     else
      $html.=preg_replace("/".$this->bodyLinebreak."/", "<br/>".$this->bodyLinebreak, $message);

     $html.=" <body>".$this->bodyLinebreak;
     $html.="<html>".$this->bodyLinebreak;

     if($this->wordwrap)
      $message=wordwrap($message, $this->wordwrap, $this->bodyLinebreak);

     if(($entry=$this->attach(false, false, false, "multipart/alternative", false))===false)
      return false;
     if($this->attach($filename, $message, $entry, "text/plain", "7bit")===false)
      return false;

     if($embeddedHTML)
     {
      if($this->attachEmbeddedHTML(false, $html, $entry)===false)
       return false;
     }
     else
     {
      if($this->attach(false, $html, $entry, "text/html", "7bit")===false)
       return false;
     }
    }
   }

   return true;
  }

  function attachEmbeddedHTML($filename, $html=false, $parent=false)
  {
   global $_path;

   if($filename&&!$html)
   {
    if(!file_exists($filename))
    {
     $this->e->add("File: \"".$filename."\" doesn't exist", "error");
     return false;
    }
    if(!($html=@file_get_contents($filename)))
    {
     $this->e->add("Unable to read contents for filename: \"".$filename."\"", "error");
     return false;
    }
   }

   if($this->mime->content($html)!="text/html") 
   {
    $this->e->add("Embedded HTML mime type not text/html", "error");
    return false;
   }

   $files=Array();
   $regexps=Array(
    "/\<img .*src=\"([^\"]+)\"/im",
    "/\<img .*src=\'([^\"]+)\'/im",
    "/\<script .*src=\"([^\"]+)\"/im",
    "/\<script .*src=\'([^\"]+)\'/im",
    "/\<link .*href=\"([^\"]+)\"/im",
    "/\<link .*href=\'([^\"]+)\'/im",
   );

   foreach($regexps as $regexp)
   {
    if(preg_match_all($regexp, $html, $m))
    {
     foreach($m[1] as $file)
      if(!preg_match("![a-z]+://!", $file)) //remote file
       if(!in_array($file, $files))
        $files[]=$file;
    }
   }

   if(!$files) //just a HTML message
   {
    if($this->attach($filename, $html, $parent, "text/html", "7bit")===false)
     return false;

    return true;
   }

   //detect path
   if(!$filename||strpos($filename, "/")===false) //no file or same dir: use current
    $path="";
   else
   {
    $path=preg_replace("!/[^/]+$!", "/", $filename); //get path

    if($path[0]!="/") //don't add current for root path
     $path=$_path["root"].$path;
   }

   if(($entry=$this->attach(false, false, $parent, "multipart/related"))===false)
    return false;

   if(($htmlEntry=$this->attach($filename, $html, $entry, "text/html", "7bit"))===false)
    return false;

   foreach($files as $file) //get css linked images
   {
    if(preg_match("/\.css$/i", $file))
    {
     $css=file_get_contents($path.$file);
     if(preg_match_all("/url\(([^\)]+)\)/mi", $css, $m))
     {
      foreach($m[1] as $cssFile)
       if(!in_array($cssFile, $files))
        $files[]=$cssFile;
     }
    }
   }

   foreach($files as $file)
   {
    if(($fileEntry=$this->attach($path.$file, false, $entry))===false)
     return false;

    $this->attachment[$htmlEntry]["content"]=str_replace($file, "cid:".$this->attachment[$fileEntry]["boundary"], $this->attachment[$htmlEntry]["content"]);
   }

   $l=sizeof($this->attachment);
   for($c=0;$c<$l;$c++)
   {
    if(preg_match("/\.css$/i", $this->attachment[$c]["name"]))
    {
     for($x=0;$x<$l;$x++)
      if($this->attachment[$x]["name"])
       $this->attachment[$c]["content"]=str_replace(str_replace($path, "", $this->attachment[$x]["name"]), "cid:".$this->attachment[$x]["boundary"], $this->attachment[$c]["content"]);
    }
   }

   return true;
  }

  function attach($filename, $content=false, $parent=false, $contentType=false, $contentTransferEncodig=false)
  {
   $entry=sizeof($this->attachment);

   $this->attachment[$entry]=Array(
    "parent" => $parent,
    "name" => $filename,
    "boundary" => md5(uniqid(mt_rand(), true)),
   );

   if($filename&&!$content)
   {
    if(!file_exists($filename))
    {
     $this->e->add("File: \"".$filename."\" doesn't exist", "error");
     return false;
    }
    if(!($content=@file_get_contents($filename)))
    {
     $this->e->add("Unable to read contents for filename: \"".$filename."\"", "error");
     return false;
    }
   }

   if($contentType)
    $this->attachment[$entry]["contentType"]=$contentType;
   else if($filename)
    $this->attachment[$entry]["contentType"]=$this->mime->filename($filename);
   else
    $this->attachment[$entry]["contentType"]=$this->mime->content($content);

   if($contentTransferEncodig)
    $this->attachment[$entry]["contentTransferEncodig"]=$contentTransferEncodig;
   else
   {
    if(strpos($this->attachment[$entry]["contentType"], "text")===0) //text
     $this->attachment[$entry]["contentTransferEncodig"]="7bit";
    else if(strpos($this->attachment[$entry]["contentType"], "multipart")===0) //multipart 
     $this->attachment[$entry]["contentTransferEncodig"]=false;
    else
     $this->attachment[$entry]["contentTransferEncodig"]="base64";
   }

   switch($this->attachment[$entry]["contentTransferEncodig"])
   {
    case "7bit":
    {
     $this->attachment[$entry]["content"]=$this->fixLineBreaks($content);
     break;
    }
    case "base64":
    {
     $this->attachment[$entry]["content"]=base64_encode($content);
     break;
    }
    default:
    {
     $this->attachment[$entry]["content"]=$content;
     break;
    }
   }

   return $entry;
  }

  function output()
  {
   if(($header=$this->header())===false)
    return false;

   if(($body=$this->body())===false)
    return false;

   return $header.$this->headerLinebreak.$body.$this->bodyLinebreak;
  }

  function send()
  {
   if($this->server)
    return $this->_sendSMTP();
   else
    return $this->_sendPHP();
  }

  function _sendSMTP()
  {
   if(($header=$this->header())===false)
    return false;

   if(($body=$this->body())===false)
    return false;

   $rcpts=Array();
   foreach($this->to as $rcpt)
    $rcpts[]=$rcpt;

   foreach($this->cc as $rcpt)
    $rcpts[]=$rcpt;

   foreach($this->bcc as $rcpt)
    $rcpts[]=$rcpt;

   if(!$rcpts)
   {
    $this->e->add("Not sending message: no recipients", "error");
    return false;
   }

   $s=new Socket("tcp://".$this->server.":".$this->port);

   if(!($recv=$s->read()))
   {
    $this->e->add($s->e);
    return false;
   }

   if(strpos($recv, "220")!==0)
   {
    $this->e->add("SMTP connection failed: expected 220 from: \"".$this->server.":".$this->port."\"", "error", $recv);
    return false;
   }

   $send="EHLO ".$this->defaultDomain.$this->headerLinebreak;

   if(!$s->write($send))
   {
    $this->e->add($s->e);
    return false;
   }

   if(!($recv=$s->read()))
   {
    $this->e->add($s->e);
    return false;
   }

   if(strpos($recv, "250")!==0)
   {
    $this->e->add("SMTP connection failed: expected 250 from: \"".$this->server.":".$this->port."\"", "error", $recv);
    return false;
   }

/*
   if(strpos($recv, "250-AUTH LOGIN PLAIN")!==false) //authentication support
   {
    if($this->username||$this->password)
    {
     $send="AUTH PLAIN ";
     $send.=base64_encode("\0".$this->username."\0".$this->password);
     $send.=$this->headerLinebreak;

     if(!(@socket_write($sock, $send)))
     {
      $this->e->add("SMTP connection failed: unable to send data to: \"".$this->server.":".$this->port."\"", "error", socket_strerror(socket_last_error($sock)));
      $s->close();
      return false;
     }

     if(!($recv=@socket_read($sock, 255)))
     {
      $this->e->add("SMTP connection failed: no data comming from: \"".$this->server.":".$this->port."\"", "error", socket_strerror(socket_last_error($sock)));
      $s->close();
      return false;
     }

     if(strpos($recv, "250")!==0)
     {
      $this->e->add("SMTP connection failed: expected 250 from: \"".$this->server.":".$this->port."\"", "error", $recv);
      $s->close();
      return false;
     }
    }
   }
*/ 

   $send="MAIL FROM:";
   if(sizeof($this->from)==1) 
   {
    foreach($this->from as $v)
     $send.="<".$v.">";
   }
   else
    $send.="<>";

   $send.=$this->headerLinebreak;

   if(!$s->write($send))
   {
    $this->e->add($s->e);
    return false;
   }

   if(!($recv=$s->read()))
   {
    $this->e->add($s->e);
    return false;
   }

   if(strpos($recv, "250")!==0)
   {
    $this->e->add("SMTP connection failed: expected 250 from: \"".$this->server.":".$this->port."\"", "error", $recv);
    return false;
   }

   foreach($rcpts as $rcpt)
   {
    $send="RCPT TO:";
    $send.="<".$rcpt.">";
    $send.=$this->headerLinebreak;

    if(!$s->write($send))
    {
     $this->e->add($s->e);
     return false;
    }

    if(!($recv=$s->read()))
    {
     $this->e->add($s->e);
     return false;
    }

    if(strpos($recv, "250")!==0)
    {
     $this->e->add("SMTP connection failed: expected 250 from: \"".$this->server.":".$this->port."\"", "error", $recv);
     return false;
    }
   }

   $send="DATA".$this->headerLinebreak;

   if(!$s->write($send))
   {
    $this->e->add($s->e);
    return false;
   }

   if(!($recv=$s->read()))
   {
    $this->e->add($s->e);
    return false;
   }

   if(strpos($recv, "354")!==0)
   {
    $this->e->add("SMTP connection failed: expected 354 from: \"".$this->server.":".$this->port."\"", "error", $recv);
    return false;
   }

   $send=$header.$this->headerLinebreak.$body.$this->bodyLinebreak.".".$this->bodyLinebreak;

   if(!$s->write($send))
   {
    $this->e->add($s->e);
    return false;
   }

   if(!($recv=$s->read()))
   {
    $this->e->add($s->e);
    return false;
   }

   if(strpos($recv, "250")!==0)
   {
    $this->e->add("SMTP connection failed: expected 250 from: \"".$this->server.":".$this->port."\"", "error", $recv);
    return false;
   }

   $s->close();
   return true;
  }

  function _sendPHP()
  {
   if(($header=$this->header(true))===false)
    return false;

   if(($body=$this->body())===false)
    return false;

   $to="";
   if($this->to)
   {
    foreach($this->to as $k => $v)
    {
     if(is_numeric($k)) //no name
      $to.=$v.", ";
     else
      $to.=$k." <".$v.">, ";
    }
    $to=substr($to, 0, -2);
   }
   else
    $to.="undisclosed-recipients:;";

   return mail($to, $this->subject, $body, $header);
  }

  function header($noToAndSubject=false)
  {
   if($this->e->any("fatal"))
    return false;

   if(!$this->_fixAttachments())
    return false;

   $header="";

   if(sizeof($this->from)==1) //only 1 from...  ;)
   {
    $header.="From: ";
    foreach($this->from as $k => $v)
    {
     if(is_numeric($k)) //no name
      $header.=$v;
     else
      $header.=$k." <".$v.">";
    }
    $header.=$this->headerLinebreak;
   }

   if(!$noToAndSubject)
   {
    if($this->to)
    {
     $header.="To: ";
     foreach($this->to as $k => $v)
     {
      if(is_numeric($k)) //no name
       $header.=$v.", ";
      else
       $header.=$k." <".$v.">, ";
     }
     $header=substr($header, 0, -2).$this->headerLinebreak;
    }
    else
     $header.="To: undisclosed-recipients:;".$this->headerLinebreak;
   }

   if($this->cc)
   {
    $header.="Cc: ";
    foreach($this->cc as $k => $v)
    {
     if(is_numeric($k)) //no name
      $header.=$v.", ";
     else
      $header.=$k." <".$v.">, ";
    }
    $header=substr($header, 0, -2).$this->headerLinebreak;
   }

   if($this->bcc)
   {
    $header.="Bcc: ";
    foreach($this->bcc as $k => $v)
    {
     if(is_numeric($k)) //no name
      $header.=$v.", ";
     else
      $header.=$k." <".$v.">, ";
    }
    $header=substr($header, 0, -2).$this->headerLinebreak;
   }

   if($this->replyTo)
    $header.="Reply-To: ".$this->replyTo.$this->headerLinebreak;

   if($this->inReplyTo)
    $header.="In-Reply-To: ".$this->inReplyTo.$this->headerLinebreak;

   if($this->messageID)
    $header.="Message-ID: ".$this->messageID.$this->headerLinebreak;
   else
   {
    if($this->from)
    {
     foreach($this->from as $v)
      $domain=preg_replace("/^[^@]+@/", "", $v);
    }
    else
     $domain=$this->defaultDomain;

    $header.="Message-ID: <".md5(uniqid(mt_rand(), true))."@".$domain.">".$this->headerLinebreak;
   }

   if(!$noToAndSubject)
    $header.="Subject: ".$this->subject.$this->headerLinebreak;

   $header.="Date: ".date("D, j M Y H:i:s O").$this->headerLinebreak;

   if($this->priority)
    $header.="X-Priority: ".$this->priority.$this->headerLinebreak;

   if($this->mimeVersion)
    $header.="MIME-Version: ".$this->mimeVersion.$this->headerLinebreak;

   if($this->userAgent)
    $header.="User-Agent: ".$this->userAgent.$this->headerLinebreak;

   $header.="Content-Type: ".$this->attachment[0]["contentType"];

   if($this->attachment[0]["contentType"]=="text/plain"||$this->attachment[0]["contentType"]=="text/html") 
    $header.="; charset=".$this->charSet;
   else
    $header.="; boundary=".$this->boundaryPrefix.$this->attachment[0]["boundary"];

   $header.=$this->headerLinebreak;

   if($this->attachment[0]["contentTransferEncodig"])
    $header.="Content-Transfer-Encoding: ".$this->attachment[0]["contentTransferEncodig"].$this->headerLinebreak;

   return $header;
  }

  function body()
  {
   if($this->e->any("fatal"))
    return false;

   if(!$this->_fixAttachments())
    return false;

   if(sizeof($this->attachment)==1)
    $body=$this->attachment[0]["content"].$this->bodyLinebreak;
   else
   {
    $body="This is a multi-part message in MIME format.".$this->bodyLinebreak;

    $body.=$this->_bodyAttachments();
   }

   return $body;
  }

  function _bodyAttachments($parent=0)
  {
   if(!isset($this->attachment[$parent]))
    return "";

   $body="";
   $l=sizeof($this->attachment);
   for($c=0;$c<$l;$c++)
   {
    if($this->attachment[$c]["parent"]!==$parent)
     continue;

    $body.="--".$this->boundaryPrefix.$this->attachment[$parent]["boundary"].$this->bodyLinebreak;

    $body.="Content-Type: ".$this->attachment[$c]["contentType"];

    if($this->attachment[$c]["contentType"]=="multipart/alternative"||$this->attachment[$c]["contentType"]=="multipart/related") 
     $body.="; boundary=".$this->boundaryPrefix.$this->attachment[$c]["boundary"];

    if($this->attachment[$c]["contentType"]=="text/plain"||$this->attachment[$c]["contentType"]=="text/html") 
     $body.="; charset=".$this->charSet;

    if($this->attachment[$c]["name"])
     $body.="; name=\"".preg_replace("!^.*/([^/]+)$!", "\\1", $this->attachment[$c]["name"])."\"";

    $body.=$this->bodyLinebreak;

    $body.="Content-Transfer-Encoding: ".$this->attachment[$c]["contentTransferEncodig"].$this->bodyLinebreak;

    if($this->attachment[$c]["contentType"]=="text/plain") 
     $body.="Content-Disposition: inline".$this->bodyLinebreak;
    else if($this->attachment[$c]["name"])
    {
     $body.="Content-Disposition: attachment; filename=\"".preg_replace("!^.*/([^/]+)$!", "\\1", $this->attachment[$c]["name"])."\"".$this->bodyLinebreak;
     $body.="Content-Location: ".$this->attachment[$c]["name"].$this->bodyLinebreak;
     $body.="Content-ID: <".$this->attachment[$c]["boundary"].">".$this->bodyLinebreak;
    }

    $body.=$this->bodyLinebreak.$this->attachment[$c]["content"].$this->bodyLinebreak;
    $body.=$this->_bodyAttachments($c);
   }

   if($body)
    $body.="--".$this->boundaryPrefix.$this->attachment[$parent]["boundary"]."--".$this->bodyLinebreak;

   return $body;
  }

  function fixLineBreaks($content)
  {
   if(($linebreaks=$this->detectLineBreaks($content))&&$linebreaks!=$this->bodyLinebreak)
   {
    switch($linebreaks)
    {
     case "\r\n":
      $content=preg_replace("/\r\n/", $this->bodyLinebreak, $content);
      break;

     case "\n\r":
      $content=preg_replace("/\n\r/", $this->bodyLinebreak, $content);
      break;

     case "\n":
      $content=preg_replace("/\n([^\n\r]?)/", $this->bodyLinebreak."\\1", $content);
     break;

     case "\r":
      $content=preg_replace("/\r([^\n\r]?)/", $this->bodyLinebreak."\\1", $content);
     break;

     default: break;
    }
   }

   return $content;
  }

  function detectLineBreaks($string)
  {
   if(preg_match("/\n\r./", $string))
    return "\n\r";
   else if(preg_match("/\r\n./", $string))
    return "\r\n";
   else if(preg_match("/\n([^\n\r]?)/", $string))
    return "\n";
   else if(preg_match("/\r([^\n\r]?)/", $string))
    return "\r";

   return "";
  }

  function _fixAttachments()
  {
   $new=false;
   $l=sizeof($this->attachment);
   $contentType=$this->attachment[0]["contentType"];

   if($contentType=="multipart/mixed")
    return true;

   for($c=1;$c<$l;$c++)
   {
    if($this->attachment[$c]["parent"]!==false)
     continue;

    if($contentType!==$this->attachment[$c]["contentType"])
    {
     $new=Array(
      "parent" => false,
      "name" => false,
      "boundary" => md5(uniqid(mt_rand(), true)),
      "charSet" => false,
      "contentTransferEncodig" => false,
      "content" => false,
      "contentType" => "multipart/mixed",
     );

     break;
    }
   }

   if($new)
   {
    for($c=0;$c<$l;$c++)
    {
     if($this->attachment[$c]["parent"]===false)
      $this->attachment[$c]["parent"]=0;
     else
      $this->attachment[$c]["parent"]++;
    }

    array_unshift($this->attachment, $new);
   }

   return true;
  }

  function clear($part="all")
  {
   switch($part)
   {
    case "all":
    {
     $this->e->clear();

     $this->to=Array();
     $this->cc=Array();
     $this->bcc=Array();
     $this->from=Array();
     $this->subject="";

     $this->attachment=Array();

     $this->replyTo=false;
     $this->inReplyTo=false;
     $this->messageID=false;
     $this->priority=false; //1->5 (high->low)

     break;
    }
    case "to":
    {
     $this->to=Array();
     break;
    }
    case "cc":
    {
     $this->cc=Array();
     break;
    }
    case "bcc":
    {
     $this->bcc=Array();
     break;
    }
    case "from":
    {
     $this->from=Array();
     break;
    }
    case "subject":
    {
     $this->subject="";
     break;
    }
    case "attachment":
    {
     $this->attachment=Array();
     break;
    }
    case "replyTo":
    {
     $this->replyTo=false;
     break;
    }
    case "inReplyTo":
    {
     $this->inReplyTo=false;
     break;
    }
    case "messageID":
    {
     $this->messageID=false;
     break;
    }
    case "priority":
    {
     $this->priority=false;
     break;
    }
    default: 
    {
     $this->e->add("Unknown message part to clear", "error");
     return false;
    }
   }

   return true;  
  }


 }

?>
