<?

 /*****************************************************************************
 * Filename: include/error.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."language.php");

 if(isset($_config["log"])&&$_config["log"])
  require_once($_path["include"]."data.php");



/******************************************************************
 Error class
 
 functions:
 - Error:      Constructor
 + add:        Add's an error 
 + any:        Any notes, warings or errors ?
 + isABC:      Checks of txt is alfabetic
 

******************************************************************/

 class Error 
 {


/******************************************************************
 Error 
 Constructor, initialises arrays 

******************************************************************/
  function Error()
  {
   $this->clear(); //init main vars
   $this->errorType=array("debug", "note", "warning", "error", "hack", "sql", "system", "module", "plugin");
   $this->logErrorTypes=array("debug", "error", "hack", "system", "module", "plugin");
   
   global $_config;

   if(isset($_config["logging"])&&$_config["logging"])
    $this->logging="data";
   else
    $this->logging="none";

   $this->l=new Language("error");
  }

  function clear()
  {
   $this->errors=0; //error counter
   $this->error=Array(); //errors array
   $this->extra=Array(); //extra info array
   $this->errorlevel=Array(); //errors level array
  }

  function errors(){return $this->errors;}
  function getError($entry){return $this->error[$entry];} 
  function getExtra($entry){return $this->extra[$entry];} 
  function getLevel($entry){return $this->level($this->errorlevel[$entry]);} 
   

/******************************************************************
 level 
 Converts level string to nr or nr to string
 
 + level: level string or nr 
 returns: error level, default: error 
    0 = note 
    1 = warning
    2 = error 
    3 = hack attempt
 
******************************************************************/
  function level($level)
  {
   if(is_numeric($level))
    return $this->errorType[$level];

   else
   {
    $l=sizeof($this->errorType);

    for($c=0;$c<$l;$c++)
     if($this->errorType[$c]==$level){return $c;}
   }
   return 2; 
  }

/******************************************************************
 add 
 Add's a new array to the array
 
 + text: error text 
 - level: error level string

******************************************************************/
  function add($error, $level="error", $extra=false, $log=true)
  {
   if(is_object($error)&&strtolower(get_class($error))=="error")
   {
    for($c=0;$c<$error->errors;$c++)
     $this->add($error->error[$c], $this->level($error->errorlevel[$c]), $error->extra[$c], false);

    return true;
   }
   else
   {
    $this->error[$this->errors]=$error;
    $this->extra[$this->errors]=$extra;
    $this->errorlevel[$this->errors]=$this->level($level);
    $this->errors++;

    if($log&&in_array($level, $this->logErrorTypes)) 
    {
     if($this->logging=="stderr"||$this->logging=="stdout")
     {
      file_put_contents("php://".$this->logging, strtoupper($level).": ".$this->l->t($error)."\n");

      if(isset($_config["debug"])&&$_config["debug"]&&$extra)
       file_put_contents("php://".$this->logging, $extra."\n");
     }
     else if($this->logging=="data")
     {
      $d=Data::cache("log", "add", "item", false, "_errorLogging");
      $d->clearValues();

      if($d->e->logErrorTypes)
       $d->e->logErrorTypes=array();

      $d->setValue("level", $level);
      $d->setValue("title", $error);
      $d->setValue("text", $extra);

      $req="";
  
      foreach($_REQUEST as $k => $v)
      {
       if(is_array($v))
       {
        foreach($v as $k2 => $v2)
        {
         if(!is_array($v2)&&strlen($v2)<512)
          $req.=" ".$k2." => ".addslashes($v2)."\n";
        }
       }
       else if(strlen($v)<512)
        $req.=$k." => ".addslashes($v)."\n";
      }
      $d->setValue("request", $req);

      if(!$d->submit()) //error on submitting
       return false;
     }
     else if($this->logging=="screen")
     {
      print strtoupper($level).": ".$this->l->t($error)."\n";

      if(isset($_config["debug"])&&$_config["debug"]&&$extra)
       print $extra."\n";
     }
//     else if($this->logging=="none")
//      return true;
    }

    return true;
   }

  }

 
/******************************************************************
 any 
 return if there are notes, warnings or errors 
 
 + level: error level text, either:
   - note 
   - warnings
   - error
   - hack
 
******************************************************************/
  function any($level=false) 
  {
   if(!$level) //any errors at all?
   {
    if(isset($_config["debug"])&&$_config["debug"]) //really check all error types
    {
     if($this->errors)
      return true;
     else
      return false;
    }
    else //all except debug
    {
     $errorTypes=Array("note", "warning", "error", "hack", "sql", "system", "module", "plugin");

     foreach($errorTypes as $errorType)
      if($this->any($errorType))
       return true;

     return false;
    }
   }
   else if($level=="fatal") //only fatal errors
   {
    $errorTypes=Array("error", "hack", "sql", "system", "module", "plugin");

    foreach($errorTypes as $errorType)
     if($this->any($errorType))
      return true;

    return false;
   }
   else if($level=="show") //only showable errors
   {
    $errorTypes=Array("note", "warning", "error");

    foreach($errorTypes as $errorType)
     if($this->any($errorType))
      return true;

    return false;
   }

   $level=$this->level($level); //convert error level

   for($c=0;$c<$this->errors;$c++)
    if($this->errorlevel[$c]==$level){return true;}

   return false;
  }


/******************************************************************
 show
 returns a string with all error of a certain level

 + level: error level text, either:
   - note 
   - warnings
   - error
   - hack
 
******************************************************************/
  function show($level=false)
  {
   global $_config;

   $show="";

   if(!$level)
   {
    if(isset($_config["debug"])&&$_config["debug"])
     $errorTypes=Array("debug", "note", "warning", "error", "hack", "sql", "system", "module", "plugin");
    else
     $errorTypes=Array("note", "warning", "error", "hack", "sql", "system", "module", "plugin");

    foreach(array_reverse($errorTypes) as $errorType)
     if($this->any($errorType))
      $show.=$this->_showErrors($errorType)."\n";

    if($show)
     $show="<ul class=\"errors\">\n".$show."</ul>\n";

    return $show;
   }
   else if($level=="fatal") //errors or hacks
   {
    $errorTypes=Array("error", "hack", "sql", "system", "module", "plugin");

    foreach(array_reverse($errorTypes) as $errorType)
     if($this->any($errorType))
      $show.=$this->_showErrors($errorType)."\n";

    if($show)
     $show="<ul class=\"errors ".$level."\">\n".$show."</ul>\n";

    return $show;
   }
   else if($level=="show") //useful for users
   {
    $errorTypes=Array("note", "warning", "error");

    foreach(array_reverse($errorTypes) as $errorType)
     if($this->any($errorType))
      $show.=$this->_showErrors($errorType)."\n";

    if($show)
     $show="<ul class=\"errors ".$level."\">\n".$show."</ul>\n";

    return $show;
   }

   if($this->any($level))
    $show.=$this->_showErrors($level)."\n";

   if($show)
    $show="<ul class=\"errors ".$level."\">\n".$show."</ul>\n";

   return $show;
  }

  function _showErrors($level=false)
  {
   global $_config;

   $show="";

   $level=$this->level($level); //convert error level

   for($c=0;$c<$this->errors;$c++)
   {
    if($this->errorlevel[$c]==$level)
    {
     $show.="<li class=\"".$this->getLevel($c)."\">".$this->l->t(htmlentities($this->error[$c]));

     if(isset($_config["debug"])&&$_config["debug"]&&$this->extra[$c]!="")
      $show.=" ".$this->l->t(htmlentities($this->extra[$c]))."<br/>";

     $show.="</li>\n";
    }
   }

   return $show;
  }
 }
?>
