<?

 /*****************************************************************************
 * Filename: include/exif.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."dt.php");

 class EXIF
 {
  function EXIF($file=false)
  {
   global $_path;

   $this->e=new Error();
   $this->e->l->read("exif");

   $this->exif=false;

   if($file)
    $this->read($file);
  }

  function read($file)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!($this->exif=@exif_read_data($file)))
   {
    $this->e->add("File: \"".$file."\" doesn't contain EXIF information", "error");
    return false;
   }

   return true;
  }

  function get($keys=false)
  {
   if($this->exif===false)
   {
    $this->e->add("No EXIF information read", "error");
    return false;
   }

   if(!$keys)
    return $this->exif;

   $keys=explode("->", $keys);

   $tmp=$this->exif;
   $l=sizeof($keys);
   for($c=0;$c<$l;$c++)
   {
    if(!isset($tmp[$keys[$c]]))
    {
     $tmp2=Array();
     for($x=0;$x<=$c;$x++)
      $tmp2[]=$keys[$x];

     $this->e->add("Key: \"".implode("->", $tmp2)."\" not found in EXIF information", "error");
     return false;
    }

    $tmp=$tmp[$keys[$c]];
   }

   return $tmp;
  }


  function info()
  {
   if($this->exif===false)
   {
    $this->e->add("No EXIF information read", "error");
    return false;
   }

   $info=Array(
    "time" => $this->time(),
    "width" => $this->width(),
    "height" => $this->height(),
    "orientation" => $this->orientation(),
    "exposure" => $this->exposure(),
    "aperture" => $this->aperture(),
    "iso" => $this->iso(),
    "camera" => $this->camera(),
    "model" => $this->model(),
   );

   return $info;
  }
  function time()
  {
   if(($info=$this->get("DateTime"))===false)
    return false;

   return DT::dateConvert($info, "Y:m:d H:i:s");
  }
  function orientation()
  {
   if(($info=$this->get("Orientation"))===false)
    return false;

   switch($info)
   {
    case 1:
     $orientation="0";
     break;
    case 2:
     $orientation="0m";
     break;
    case 3:
     $orientation="180";
     break;
    case 4:
     $orientation="180m";
     break;
    case 5:
     $orientation="270m";
     break;
    case 6:
     $orientation="270";
     break;
    case 7:
     $orientation="90m";
     break;
    case 8:
     $orientation="90";
     break;
    default:
     return false;
   }

   return $orientation;
  }
  function size()
  {
   if(($info=$this->get("COMPUTED"))===false)
    return false;

   return Array("width" => $info["Width"], "height" => $info["Height"]);
  }
  function width()
  {
   if(($info=$this->get("COMPUTED"))===false)
    return false;

   return $info["Width"];
  }
  function height()
  {
   if(($info=$this->get("COMPUTED"))===false)
    return false;

   return $info["Height"];
  }
  function exposure()
  {
   if(($info=$this->get("ExposureTime"))===false)
    return false;

   $info=explode("/", $info);
   $info=$info[0]/$info[1];

   return $info;
  }
  function aperture()
  {
   if(($info=$this->get("FNumber"))===false)
    return false;

   $info=explode("/", $info);
   $info=number_format($info[0]/$info[1], 1);

   return $info;
  }
  function iso()
  {
   if(($info=$this->get("ISOSpeedRatings"))===false)
    return false;

   return $info;
  }
  function camera()
  {
   if(($info=$this->get("Make"))===false)
    return false;

   return rtrim($info);
  }
  function model()
  {
   if(($info=$this->get("Model"))===false)
    return false;

   return rtrim($info);
  }

  
  function set($key, $value)
  {
   $this->e->add("Set function not implemented", "system");
   return false;
  }
  function write($file)
  {
   $this->e->add("Write function not implemented", "system");
   return false;
  }

 }
?>
