<?

 /*****************************************************************************
 * Filename: include/form.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."language.php");

/******************************************************************
 FORM class

 functions:
 + start:      Returns html form opening tage
 + input:      Returns input field of given type
 - end:        Returns html form closing tage


******************************************************************/

 class Form
 {

  function Form()
  {
   $this->l=new Language("form");

   $this->form=0;
   $this->name="";

   $this->listValuesOrdering=true; //order list based on ordering used in values (listview/select)
  }


/******************************************************************
 Start
 Returns the start tag of a form

 + action:  what file open
 - name:    form name, default: frm
 - id:      id of form
 - extra:   extra options like javascript

******************************************************************/
  function start($action=false, $name=false, $method=false, $id=false, $extra=false)
  {
   global $_jses, $_path;

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" $extra";}
   if(!$name){$name="frm".$this->form;}
   if(!$action){$action=$_path["HTML"]["url"];}
   if(!$method){$method="post";}
   $this->name=$name;

   //XXX no form name... xhtml says so
   //$r="<form action=\"".$action."\" name=\"".$name."\" method=\"".$method."\" enctype=\"multipart/form-data\"".$id.$extra.">";
   $r="<form action=\"".$action."\" method=\"".$method."\" enctype=\"multipart/form-data\"".$id.$extra.">";

   $r.="<div class=\"invisible\">";

   return $r;
  }


/******************************************************************
 Input
 Returns the input of a specified type

 + type:    type of input field: text, password, hidden
 - name:    input field name
 - value:   value of input field
 - class:   css class reference, default: form
 - id:      id of form
 - extra:   extra options like javascript
 - text:    extra text

******************************************************************/
  function input($type="hidden", $name="", $value=false, $class=false, $id=false, $extra=false)
  {
   $value=htmlentities($value, ENT_NOQUOTES|ENT_XHTML, "ISO-8859-15");

   if(!$class)
    $class="form";

   $class.=" ".$type;

   $class=" class=\"$class\"";

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" $extra";}
   $extra=preg_replace(Array("/  /", "/ checked/i", "/ disabled/i"), Array(" ", " checked=\"checked\"", " disabled=\"disabled\""), $extra);

   $r="";
   switch($type)
   {
    case "textarea":
     $r.=" <textarea name=\"".$name."\"".$class.$id.$extra." cols=\"\" rows=\"\">";
     $r.=$value;
     $r.="</textarea>";
    break;

    case "fake":
     if($value==""){$value="&nbsp; ";}
     $r.="<p".$class.$id.$extra.">".$value."</p>";
    break;

    case "radio":
    case "checkbox":
     $extra=" style=\"width: 15px; height: 15px;\"".$extra;

    default:
     $r.=" <input type=\"".$type."\" name=\"".$name."\" value=\"".$value."\"".$class.$id.$extra."/>";
    break;
   }

   return $r;
  }

  function checkbox($checked=false, $text="", $name="", $value=false, $class=false, $id=false, $extra=false)
  {
   if($checked)
    $extra.=" checked";

   if(!$id)
    $id="checkbox".rand(0,32768);

   $r="";
   if($text!="")
    $r.="<label for=\"".$id."\">";

   $r.=$this->input("checkbox", $name, $value, $class, $id, $extra);

   if($text!="")
    $r.=" ".$text."</label>";

   return $r;
  }

  function radio($checked=false, $text="", $name="", $value=false, $class=false, $id=false, $extra=false)
  {
   if($checked)
    $extra.=" checked";

   if(!$id)
    $id="radio".rand(0,32768);

   $r="";
   if($text!="")
    $r.="<label for=\"".$id."\">";

   $r.=$this->input("radio", $name, $value, $class, $id, $extra);

   if($text!="")
    $r.=" ".$text."</label>";

   return $r;
  }

  function selectStart($type=false, $name="", $class=false, $id=false, $extra=false)
  {
   if(!$class)
    $class="form";

   if($type=="multiple")
   {
    $type=" multiple=\"multiple\"";
    $class.=" multi";
   }
   else
    $class.=" select";

   $class=" class=\"$class\"";

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" ".$extra;}

   return "<select name=\"".$name."\"".$class.$id.$extra.$type.">";
  }
  function selectOption($value, $text=false, $selected=false, $extra=false)
  {
   if(!$text){$text=$value;}
   if($extra){$extra=" ".$extra;}
   if($selected){$selected=" selected=\"selected\"";}

   $value=" value=\"".$value."\"";

   return "<option".$value.$selected.$extra.">".$text."</option>";
  }
  function selectEnd()
  {
   return "</select>";
  }

  function select($type=false, $name="", $list, $values=false, $class=false, $id=false, $extra=false, $addEmptyOption=false)
  {
   if(!is_array($values))
    $values=Array($values);

   $array_type=array_type($list);

   //create normal list from data object
   if($array_type=="data")
   {
    $tmp=Array();
    if($list->get())
    {
     while($list->next())
     {
      if(isset($list->v["treedepth"])&&$list->v["treedepth"])
       $depth=str_repeat("&nbsp; ", ($list->v["treedepth"]-1)*3)."|_ ";
      else
       $depth="";

      $tmp[$list->v("id")]=$depth.$list->v("title");
     }
    }

    $array_type="associative";

    $list=$tmp;
    unset($tmp);
   }

   if($this->listValuesOrdering)
    $list=$this->orderListValues($list, $values);

   $r=$this->selectStart($type, $name, $class, $id, $extra);

   if($addEmptyOption)
    $r.=$this->selectOption("", "");

   switch($array_type)
   {
    case "indexed":
    {
     foreach($list as $v)
      $r.=$this->selectOption($v, $v, (in_array($v, $values)?"selected":false));

     break;
    }
    case "associative":
    {
     foreach($list as $k => $v)
      $r.=$this->selectOption($k, $v, (in_array($k, $values)?"selected":false));

     break;
    }
    default: break;
   }

   $r.=$this->selectEnd();
   return $r;
  }

  function listview($type=false, $name="", $list, $values=false, $class=false, $id=false, $extra=false, $addEmptyOption=false)
  {
   if(!is_array($values))
    $values=Array($values);

   $array_type=array_type($list);

   //create normal list from data object
   if($array_type=="data")
   {
    $tmp=Array();
    if($list->get())
    {
     while($list->next())
     {
      if(isset($list->v["treedepth"])&&$list->v["treedepth"])
       $depth=str_repeat("&nbsp; ", ($list->v["treedepth"]-1)*3)."|_ ";
      else
       $depth="";

      $tmp[$list->v("id")]=$depth.$list->v("title");
     }
    }

    $array_type="associative";

    $list=$tmp;
    unset($tmp);
   }

   if($this->listValuesOrdering)
    $list=$this->orderListValues($list, $values);

   if(!$class)
    $class="form";

   if($id){$id=" id=\"$id\"";}

   if($type=="multiple")
   {
    $type="checkbox";
    $class.=" multilistview";
   }
   else
   {
    $type="radio";
    $class.=" listview";
   }

   $r="<div class=\"".str_replace("form ", "", $class)." ".$name."\"".$id.">";

   if($addEmptyOption)
    $r.=$this->$type((in_array("", $values)), "", $name, "", $class, $id, $extra)."<br/>";

   switch(array_type($list))
   {
    case "indexed":
    {
     foreach($list as $v)
      $r.=$this->$type((in_array($v, $values)), $v, $name, $v, $class, $id, $extra)."<br/>";

     break;
    }
    case "associative":
    {
     foreach($list as $k => $v)
      $r.=$this->$type((in_array($k, $values)), $v, $name, $k, $class, $id, $extra)."<br/>";

     break;
    }
    default: break;
   }

   $r.="</div>";
   return $r;
  }

  function dataSelect($module="categories", $name="catnr", $values=false, $class=false, $id=false, $extra=false, $multi=false, $addEmptyOption=false)
  {
   $d=Data::cache($module, "view", "all");
   $d->clearValues();

   if(($treeReference=$d->getSelfReference())&&!$d->extends)
   {
    $d->setAction("tree");
    $d->setExtra("down");

    $d->setValue($treeReference, 0);
    $d->setShowFields(array_merge($d->check("properties", "title"), Array("treedepth", $treeReference)));
   }
   else
    $d->setShowFields($d->check("properties", "title"));

   return $this->select(($multi?"multiple":false), $name, $d, $values, $class, $id, $extra, $addEmptyOption);
  }

  function dataListview($module="categories", $name="catnr", $values=false, $class=false, $id=false, $extra=false, $multi=false, $addEmptyOption=false)
  {
   $d=Data::cache($module, "view", "all");
   $d->clearValues();

   if(($treeReference=$d->getSelfReference())&&!$d->extends)
   {
    $d->setAction("tree");
    $d->setExtra("down");

    $d->setValue($treeReference, 0);
    $d->setShowFields(array_merge($d->check("properties", "title"), Array("treedepth", $treeReference)));
   }
   else
    $d->setShowFields($d->check("properties", "title"));

   return $this->listview(($multi?"multiple":false), $name, $d, $values, $class, $id, $extra, $addEmptyOption);
  }

  function orderListValues($list, $values)
  {
   if(sizeof($values)<2) //no ordered values
    return $list; //don't bother

   $newlist=Array();

   switch(array_type($list))
   {
    case "indexed":
    {
     foreach($list as $item)
     {
      if(in_array($item, $newlist)) //item already in list
       continue; //skip

      if(!in_array($item, $values)) //not in values
       $newlist[]=$item; //add in current order
      else //in values
      {
       //ensure all previous values are in list
       foreach($values as $value)
       {
        if($value==$item) //current value
        {
         $newlist[]=$item; //add to list
         break; //done
        }
        else if(in_array($value, $list)&&!in_array($value, $newlist)) //earlier valid value, not in list yet
         $newlist[]=$value; //add to list
       }
      }
     }

     break;
    }
    case "associative":
    {
     foreach($list as $key => $item)
     {
      if(isset($newlist[$key])) //item already in list
       continue; //skip

      if(!in_array($key, $values)) //not in values
       $newlist[$key]=$item; //add in current order
      else //in values
      {
       //ensure all previous values are in list
       foreach($values as $value)
       {
        if($value==$key) //current value
        {
         $newlist[$key]=$item; //add to list
         break; //done
        }
        else if(isset($list[$value])&&!isset($newlist[$value])) //earlier valid value, not in list yet
         $newlist[$value]=$list[$value]; //add to list
       }
      }
     }

     break;
    }
    default: break;
   }

   return $newlist;
  }

  function wysiwyg($name, $value, $class=false, $id=false, $extra=false)
  {
   global $_path, $_jses;

   if(!$class)
    $class="form";

   $class.=" wysiwyg";

   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" ".$extra;}

   $r="";
   $r.=$this->input("textarea", $name, $value, $class, $id, $extra)."\n";

   $r.="<script type=\"text/javascript\">

    function checkWYSIWYG()
    {
     tmp=document.getElementsByTagName('textarea');
     for(c=0;c<tmp.length;c++)
     {
      if(tmp[c].className.match('wysiwyg'))
      {
       if(typeof(initWYSIWYG)=='undefined')
        tmp[c].parentNode.innerHTML+='WYSIWYG script not found on page, make sure you\'ve included wysiwyg.js<br/>';
       if(typeof(onload)=='undefined')
        tmp[c].parentNode.innerHTML+='Onload script not found on page, make sure you\'ve included onload.js<br/>';
       if(typeof(setGetElementFunctions)=='undefined')
        tmp[c].parentNode.innerHTML+='GetElements script not found on page, make sure you\'ve included getelements.js<br/>';
       if(typeof(browser)=='undefined'||typeof(rootpath)=='undefined')
        tmp[c].parentNode.innerHTML+='JAS Scripting not loaded, needed for browser and path detection<br/>';
      }
     }
    }

    checkWYSIWYG();

   </script>\n";

   return $r;
  }


/******************************************************************
 End
 Returns the end tag of a form

******************************************************************/
  function end()
  {
   $this->form++;
   return "</div>\n</form>\n";
  }

 }
?>
