<?

 /*****************************************************************************
 * Filename: include/fs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."cli.php");
 require_once($_path["include"]."diff.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."email.php");

 class FS
 {
  function FS()
  {
   $this->e=new Error();
   $this->e->l->read("fs");

   $this->cli=new CLI();
   $this->_getACL="getfacl";
   $this->_setACL="setfacl";

   if(preg_match("/^(\d+\.\d+)/", phpversion(), $m))
    $this->phpversion=(float)$m[1];
   else
    $this->phpversion=0; //?
  }

  function isInitialized()
  {
   if(!isset($this))
    return false;

   if(get_class($this)=="FS") //object initialized
    return true;

   $tmp=$this;
   while($tmp=get_parent_class($tmp)) //lookup if parent class is FS
    if($tmp=="FS") //object initialized
     return true;

   return false;
  }

  function which($file)
  {
   $PATH=explode(":", $_SERVER["PATH"]);

   if(FS::isInitialized()) //object initialized
   {
    foreach($PATH as $dir)
     if($this->exists($dir."/".$file))
      if(($mode=$this->getMode($dir."/".$file, "array"))&&$mode["user"]["execute"])
       return $dir."/".$file;
   }
   else
   {
    foreach($PATH as $dir)
     if(FS::exists($dir."/".$file))
      if(($mode=FS::getMode($dir."/".$file, "array"))&&$mode["user"]["execute"])
       return $dir."/".$file;
   }

   return false;
  }

  function inDir($dir, $file)
  {
   if(FS::isInitialized()) //object initialized
   {
    $dir=explode("/", $this->realPath($dir));
    $file=explode("/", $this->realPath($file));
   }
   else
   {
    $dir=explode("/", FS::realPath($dir));
    $file=explode("/", FS::realPath($file));
   }

   $l=sizeof($dir);
   if(!$dir[$l-1]) //empty entry should not be checked
    $l--;

   for($c=0;$c<$l;$c++) //see if all subdirs are the same
    if(!isset($file[$c])||$dir[$c]!=$file[$c])
     return false;
   
   return true;
  }

  function realPath($file)
  {
   $file=preg_replace("!/{2,}!", "/", $file); //remove bogus slashed

   if($file[0]!="/") //relative path, add current dir
    $file=@getcwd()."/".$file;

   $file=explode("/", $file);
   $l=sizeof($file);

   $new=Array();
   for($c=0;$c<$l;$c++)
   {
    if($file[$c]==".") //same dir, skip
     continue;
    else if($file[$c]=="..") //dir up, remove previous
    {
     $l2=sizeof($new);
     if($l2>1||($l2==1&&$new[0]!="")) //not on first level, leave first slash in tact
      array_pop($new);
    }
    else //dir down, add
     $new[]=$file[$c];
   }

   return implode("/", $new);
  }

  function recurseDir($dir, $stripRoot=true, $getDotFiles=false)
  {
   if(FS::isInitialized()) //object initialized
    return $this->files($dir, true, $stripRoot, $getDotFiles);
   else
    return FS::files($dir, true, $stripRoot, $getDotFiles);
  }

  function files($dir, $recursive=false, $stripRoot=false, $getDotFiles=false)
  {
   if(!file_exists($dir))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Directory: \"".$dir."\" doesn't exist", "error");

    return false;
   }

   if(!is_dir($dir))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Directory: \"".$dir."\" isn't a directory", "error");

    return false;
   }

   $files=Array();

   if(($dh=opendir($dir)))
   { 
    while($file=readdir($dh))
    {
     if(preg_match("/^\./", $file))
     {
      if($file=="."||$file=="..")
       continue;
      else if(!$getDotFiles)
       continue;
     }

     if(preg_match("!/$!", $dir))
      $file=$dir.$file;
     else
      $file=$dir."/".$file;

     $files[]=$file;

     if($recursive&&is_dir($file))
     {
      if(FS::isInitialized()) //object initialized
      {
       if(($subfiles=$this->files($file, true, false, $getDotFiles))===false)
        return false;
      }
      else
      {
       if(($subfiles=FS::files($file, true, false, $getDotFiles))===false)
        return false;
      }

      $files=array_merge($files, $subfiles);
     }
    }
   }

   if($stripRoot)
    $files=preg_replace("!^".preg_quote($dir, "!")."!", "", $files);

   return $files;
  }


  function size($file)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(is_dir($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't get file size for directory: \"".$file."\"", "error");

    return false;
   }

   return @filesize($file);
  }

  function exists($file, $type=false)
  {
   if(!file_exists($file))
    return false;

   if($type)
   {
    switch($type)
    {
     case "file":
     {
      if(!is_file($file))
       return false;

      break;
     }
     case "directory":
     {
      if(!is_dir($file))
       return false;

      break;
     }
     default:
     {
      if(FS::isInitialized()) //object initialized
       $this->e->add("Unknown file type: \"".$type."\"", "error");

      return false;
     }
    }
   }

   return true;
  }

  function mkdir($dir)
  {
   if(file_exists($dir))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Directory: \"".$dir."\" already exist", "error");

    return false;
   }

   if(!@mkdir($dir))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't create directory: \"".$dir."\"", "error");

    return false;
   }

   return true;
  }

  function delete($file, $recursive=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(is_file($file))
   {
    if(!@unlink($file))
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("Can't delete file: \"".$file."\"", "error");

     return false;
    }
   }
   else //directory
   {
    if(FS::isInitialized()) //object initialized
    {
     if(($files=$this->files($file, false, false, false))===false)
      return false;
    }
    else
    {
     if(($files=FS::files($file, false, false, false))===false)
      return false;
    }

    if(!$recursive&&$files)
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("Refusing to delete non-empty directory: \"".$file."\"", "error");

     return false;
    }

    $l=sizeof($files);
    for($c=0;$c<$l;$c++)
    {
     if(is_dir($files[$c]))
     {
      if(FS::isInitialized()) //object initialized
      {
       if(!$this->delete($files[$c], $recursive))
        return false;
      }
      else
      {
       if(!FS::delete($files[$c], $recursive))
        return false;
      }
     }
     else //file
     {
      if(!@unlink($files[$c]))
      {
       if(FS::isInitialized()) //object initialized
        $this->e->add("Can't delete file: \"".$files[$c]."\"", "error");

       return false;
      }
     }
    }
    
    if(!@rmdir($file))
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("Can't delete directory: \"".$file."\"", "error");

     return false;
    }
   }

   return true;
  }

  function read($file)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(!is_readable($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" is not readable", "error");

    return false;
   }

   if(($data=@file_get_contents($file))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't read file: \"".$file."\"", "error");

    return false;
   }

   return $data;
  }

  function readLines($file, $skipEmptyLines=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(!is_readable($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" is not readable", "error");

    return false;
   }

   $flags=FILE_IGNORE_NEW_LINES;

   if($skipEmptyLines)
    $flags|=FILE_SKIP_EMPTY_LINES;

   if(($data=@file($file, $flags))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't read file: \"".$file."\"", "error");

    return false;
   }

   return $data;
  }

  function grep($pattern, $file)
  {
   if(FS::isInitialized()) //object initialized
   {
    if(!($data=$this->read($file)))
     return false;
   }
   else
   {
    if(!($data=FS::read($file)))
     return false;
   }

   if(preg_match($pattern, $data))
    return true;
     
   return false;
  }

  function write($file, $data, $option=false, $lock=false)
  {
   if(file_exists($file))
   {
    if($option!="overwrite"&&$option!="append")
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("File: \"".$file."\" already exist", "error");

     return false;
    }

    if(!is_writable($file))
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("File: \"".$file."\" is not writable", "error");

     return false;
    }
   }

   $flags=0;

   if($lock)
    $flags|=LOCK_EX;

   if($option=="append")
    $flags|=FILE_APPEND;

   if(($bytes=@file_put_contents($file, $data, $flags))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't write file: \"".$file."\"", "error");

    return false;
   }

   if($bytes!=strlen($data))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" wasn't written correctly", "error");

    return false;
   }

   return true;
  }

  function copy($file, $destination)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(!is_readable($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" is not readable", "error");

    return false;
   }

   if(is_dir($destination))
   {
    if(substr($destination, -1)!="/")
     $destination.="/";

    $destination.=preg_replace("!^.*/([^/]+)$!", "\\1", $file);
   }

   if(!@copy($file, $destination))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't copy file: \"".$file."\" to: \"".$destination."\"", "error");

    return false;
   }

   return true;
  }

  function rename($file, $destination)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(!is_readable($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" is not readable", "error");

    return false;
   }

   if(is_dir($destination))
   {
    if(substr($destination, -1)!="/")
     $destination.="/";

    $destination.=preg_replace("!^.*/([^/]+)$!", "\\1", $file);
   }

   if(!@rename($file, $destination))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't rename file: \"".$file."\" to: \"".$destination."\"", "error");

    return false;
   }

   return true;
  }

  function fileModeConvert($mode, $type) //type: array, int, string
  {
   if(!in_array($type, Array("array", "string", "int")))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unknown conversion type: \"".$type."\"", "error");

    return false;
   }

   if(is_array($mode)) //array mode
   {
    if($type=="array")
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("No conversion need", "warning");

     return $mode;
    }

    if(isset($mode["user"])&&isset($mode["group"])&&isset($mode["other"])) //user, group and other
    {
     if(FS::isInitialized()) //object initialized
     {
      if(($user=$this->fileModeConvert($mode["user"], "int"))===false)
       return false;
      if(($group=$this->fileModeConvert($mode["group"], "int"))===false)
       return false;
      if(($other=$this->fileModeConvert($mode["other"], "int"))===false)
       return false;

      $nr=$user.$group.$other;
     }
     else
     {
      if(($user=FS::fileModeConvert($mode["user"], "int"))===false)
       return false;
      if(($group=FS::fileModeConvert($mode["group"], "int"))===false)
       return false;
      if(($other=FS::fileModeConvert($mode["other"], "int"))===false)
       return false;

      $nr=$user.$group.$other;
     }
    }
    else if(isset($mode["read"])&&isset($mode["write"])&&isset($mode["execute"])) //single mode
    {
     $nr=0;
     if($mode["read"])
      $nr+=4;
     if($mode["write"])
      $nr+=2;
     if($mode["execute"])
      $nr+=1;
     $nr=(string)$nr;
    }
    else
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("Can't convert unknown array based file mode", "error");

     return false;
    }
   }
   else if(preg_match("/^[rwx\-]+$/", $mode)) //string mode
   {
/* 
 * allow string "rectification" conversion (rx => r-x)
    if($type=="string")
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("No conversion need", "warning");

     return $mode;
    }
 */

    if(strlen($mode)==9) //user, group and other
    {
     $user=substr($mode, 0, 3);
     $group=substr($mode, 3, 3);
     $other=substr($mode, 6, 3);

     if(FS::isInitialized()) //object initialized
     {
      if(($user=$this->fileModeConvert($user, "int"))===false)
       return false;
      if(($group=$this->fileModeConvert($group, "int"))===false)
       return false;
      if(($other=$this->fileModeConvert($other, "int"))===false)
       return false;

      $nr=$user.$group.$other;
     }
     else
     {
      if(($user=FS::fileModeConvert($user, "int"))===false)
       return false;
      if(($group=FS::fileModeConvert($group, "int"))===false)
       return false;
      if(($other=FS::fileModeConvert($other, "int"))===false)
       return false;

      $nr=$user.$group.$other;
     }
    }
    else //single mode
    {
     $nr=0;
     if(strchr($mode, "r")) //read
      $nr+=4;
     if(strchr($mode, "w")) //write
      $nr+=2;
     if(strchr($mode, "x")) //execute
      $nr+=1;

     $nr=(string)$nr;
    }
   }
   else if(preg_match("/^[0-7]+$/", $mode)) //int mode
   {
    if($type=="int")
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("No conversion need", "warning");

     return $mode;
    }

    $nr=(string)$mode;
    //no convertion needed here
   }
   else if(!$mode)
    $nr="0";
   else
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't convert unknown file mode: \"".$mode."\"", "error");

    return false;
   }

   if($type=="int")
   {
    $output=$nr;
   }
   else if($type=="string")
   {
    $output="";
    $l=strlen($nr);
    for($c=0;$c<$l;$c++)
    {
     $output.=($nr[$c]&4?"r":"-");
     $output.=($nr[$c]&2?"w":"-");
     $output.=($nr[$c]&1?"x":"-");
    }
   }
   else if($type=="array")
   {
    $output=Array();
    $l=strlen($nr);
    if($l==3) //user, group and other
    {
     $tmp=Array("user", "group", "other");
     for($c=0;$c<$l;$c++)
     {
      $output[$tmp[$c]]["read"]=($nr[$c]&4?true:false);
      $output[$tmp[$c]]["write"]=($nr[$c]&2?true:false);
      $output[$tmp[$c]]["execute"]=($nr[$c]&1?true:false);
     }
    }
    else //single mode
    {
     $c=0;
     $output["read"]=($nr[$c]&4?true:false);
     $output["write"]=($nr[$c]&2?true:false);
     $output["execute"]=($nr[$c]&1?true:false);
    }
   }
   else
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Can't convert to unknown file mode type: \"".$type."\"", "error");

    return false;
   }

   return $output;
  }

  function setUser($file, $user)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(!@chown($file, $user))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unable to change user for file: \"".$file."\"", "error");

    return false;
   }

   return true;
  }

  function getUser($file, $returnID=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(($id=@fileowner($file))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unable to get user for file: \"".$file."\"", "error");

    return false;
   }

   if($returnID)
    return $id;

   $user=posix_getpwuid($id);
   return $user["name"];
  }

  function setGroup($file, $group)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(!@chgrp($file, $group))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unable to change group for file: \"".$file."\"", "error");

    return false;
   }

   return true;
  }

  function getGroup($file, $returnID=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(($id=@filegroup($file))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unable to get group for file: \"".$file."\"", "error");

    return false;
   }

   if($returnID)
    return $id;

   $group=posix_getgrgid($id);
   return $group["name"];
  }

  function setUserMode($file, $mode)
  {
   if(FS::isInitialized()) //object initialized
    return $this->setMode($file, $mode, false, false);
   else
    return FS::setMode($file, $mode, false, false);
  }
  function setGroupMode($file, $mode)
  {
   if(FS::isInitialized()) //object initialized
    return $this->setMode($file, false, $mode, false);
   else
    return FS::setMode($file, false, $mode, false);
  }
  function setOtherMode($file, $mode)
  {
   if(FS::isInitialized()) //object initialized
    return $this->setMode($file, false, false, $mode);
   else
    return FS::setMode($file, false, false, $mode);
  }
  function setMode($file, $userMode=false, $groupMode=false, $otherMode=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(FS::isInitialized()) //object initialized
   {
    $mode=$this->getMode($file);
    if($userMode!==false)
     $mode[0]=$this->fileModeConvert($userMode, "int");
    if($groupMode!==false)
     $mode[1]=$this->fileModeConvert($groupMode, "int");
    if($otherMode!==false)
     $mode[2]=$this->fileModeConvert($otherMode, "int");
   }
   else
   {
    $mode=FS::getMode($file);
    if($userMode!==false)
     $mode[0]=FS::fileModeConvert($userMode, "int");
    if($groupMode!==false)
     $mode[1]=FS::fileModeConvert($groupMode, "int");
    if($otherMode!==false)
     $mode[2]=FS::fileModeConvert($otherMode, "int");
   }

   $mode=octdec("100".$mode);

   if((@chmod($file, $mode))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unable to set mode: \"".$mode."\" for file: \"".$file."\"", "error");

    return false;
   }

   //update file status cache
   if(FS::isInitialized()) //object initialized
   {
    if($this->phpversion>=5.3)
     clearstatcache(true, $file); 
    else
     clearstatcache();
   }
   else
   {
    if(preg_match("/^(\d+\.\d+)/", phpversion(), $m)&&(float)$m[1]>=5.3)
     clearstatcache(true, $file); 
    else
     clearstatcache();
   }

   return true;
  }

  function getUserMode($file, $type="int")
  {
   if(FS::isInitialized()) //object initialized
    return $this->getMode($file, $type, "user");
   else
    return FS::getMode($file, $type, "user");
  }
  function getGroupMode($file, $type="int")
  {
   if(FS::isInitialized()) //object initialized
    return $this->getMode($file, $type, "group");
   else
    return FS::getMode($file, $type, "group");
  }
  function getOtherMode($file, $type="int")
  {
   if(FS::isInitialized()) //object initialized
    return $this->getMode($file, $type, "other");
   else
    return FS::getMode($file, $type, "other");
  }
  function getMode($file, $type="int", $userGroupOther=true)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if(($mode=@fileperms($file))===false)
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("Unable to get mode for file: \"".$file."\"", "error");

    return false;
   }

   $mode=substr(decoct($mode), -3); 

   if($userGroupOther==="user")
    $mode=$mode[0];
   else if($userGroupOther==="group")
    $mode=$mode[1];
   else if($userGroupOther==="other")
    $mode=$mode[2];

   if($type=="int")
    return $mode;

   if(FS::isInitialized()) //object initialized
    return $this->fileModeConvert($mode, $type);
   else
    return FS::fileModeConvert($mode, $type);
  }

  function getTime($file, $type=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if($type)
   {
    if($type=="created")
     return @filectime($file);
    else if($type=="modified")
     return @filemtime($file);
    else if($type=="accessed")
     return @fileatime($file);
    else
    {
     if(FS::isInitialized()) //object initialized
      $this->e->add("Unknown file time type: \"".$type."\"", "error");

     return false;
    }
   }

   $times=Array(
    "created" => @filectime($file),
    "modified" => @filemtime($file),
    "accessed" => @fileatime($file),
   );

   return $times;
  }

  function setTime($file, $time, $type=false)
  {
   if(!file_exists($file))
   {
    if(FS::isInitialized()) //object initialized
     $this->e->add("File: \"".$file."\" doesn't exist", "error");

    return false;
   }

   if($type)
   {
    if(FS::isInitialized()) //object initialized
    {
     if($type=="modified")
      @touch($file, $time, $this->getTime($file, "accessed"));
     else if($type=="accessed")
      @touch($file, $this->getTime($file, "modified"), $time);
     else
     {
      $this->e->add("Can't set file time type: \"".$type."\"", "error");
      return false;
     }
    }
    else
    {
     if($type=="modified")
      @touch($file, $time, FS::getTime($file, "accessed"));
     else if($type=="accessed")
      @touch($file, FS::getTime($file, "modified"), $time);
     else
      return false;
    }
   }
   else
    @touch($file, $time, $time);

   //update file status cache
   if(FS::isInitialized()) //object initialized
   {
    if($this->phpversion>=5.3)
     clearstatcache(true, $file); 
    else
     clearstatcache();
   }
   else
   {
    if(preg_match("/^(\d+\.\d+)/", phpversion(), $m)&&(float)$m[1]>=5.3)
     clearstatcache(true, $file); 
    else
     clearstatcache();
   }


   return true;
  }


  function diff($file1, $file2) 
  {
   if(FS::isInitialized()) //object initialized
   {
    if(($data1=$this->read($file1))===false)
     return false;
    if(($data2=$this->read($file2))===false)
     return false;
   }
   else
   {
    if(($data1=FS::read($file1))===false)
     return false;
    if(($data2=FS::read($file2))===false)
     return false;
   }

   $data1=rtrim($data1);
   $data2=rtrim($data2);

   $linebreak1=Email::detectLineBreaks($data1);
   $linebreak2=Email::detectLineBreaks($data2);

   $array1=explode($linebreak1, $data1);
   $array2=explode($linebreak2, $data2);

   if(!Diff::arrays($array1, $array2))
    return false;

   $diff=Array();
   $l=sizeof($array2);
   for($c=0;$c<$l;$c++)
    if($array2[$c][0]!="=")
     $diff[$c]=$array2[$c];

   return $diff;
  }


  function getACLUser($files, $user, $type=false)
  {
   if(!FS::isInitialized()) //object should be initialized
    return FS::getACL($files, "int"); //always fail

   if(!($acl=$this->getACL($files, "int")))
    return false;

   if(!$type)
    $type="string";

   if(!is_array($files)) //single file
   {
    if(isset($acl["users"][$user]))
     return $this->fileModeConvert($acl["users"][$user], $type);
   }
   else
   {
    $r=Array();
    foreach(array_keys($acl) as $file)
    {
     if(isset($acl[$file]["users"][$user]))
      $r[$file]=$this->fileModeConvert($acl[$file]["users"][$user], $type);
     else
      $r[$file]=$this->fileModeConvert(0, $type);
    }
    return $r;
   }

   return $this->fileModeConvert(0, $type);
  }

  function getACLGroup($files, $group, $type=false)
  {
   if(!FS::isInitialized()) //object should be initialized
    return FS::getACL($files, "int"); //always fail

   if(!($acl=$this->getACL($files, "int")))
    return false;

   if(!$type)
    $type="string";

   if(!is_array($files)) //single file
   {
    if(isset($acl["groups"][$group]))
     return $this->fileModeConvert($acl["groups"][$group], $type);
   }
   else
   {
    $r=Array();
    foreach(array_keys($acl) as $file)
    {
     if(isset($acl[$file]["groups"][$group]))
      $r[$file]=$this->fileModeConvert($acl[$file]["groups"][$group], $type);
     else
      $r[$file]=$this->fileModeConvert(0, $type);
    }
    return $r;
   }

   return $this->fileModeConvert(0, $type);
  }

  function getACL($files, $type=false)
  {
   if(!FS::isInitialized()) //object should be initialized
   {
    global $_config;
    if(isset($_config["debug"])&&$_config["debug"]) 
     echo "FS::getACL doesn't support SRO call!";

    return false;
   }

   if(!is_array($files))
    $files=Array($files);

   $l=sizeof($files);
   for($c=0;$c<$l;$c++)
   {
    if(!file_exists($files[$c]))
    {
     $this->e->add("File: \"".$files[$c]."\" doesn't exist", "error");
     return false;
    }
   }

   if(!$type)
    $type="string";

   if(!in_array($type, Array("array", "string", "int")))
   {
    $this->e->add("Unknown conversion type: \"".$type."\"", "error");
    return false;
   }

   if(!$this->cli->exec($this->_getACL." \"".implode("\" \"", $files)."\""))
   {
    $this->e->add($this->cli->e);
    return false;
   }

   $lines=&$this->cli->output;

   $file=false;
   $acl=Array();
   $l=sizeof($lines);
   for($c=0;$c<$l;$c++)
   {
    if(preg_match("/^# file: (.+)$/i", $lines[$c], $m))
    {
     $file=$m[1];
     $acl[$file]=Array(
      //normal rights
      "userName" => false,
      "groupName" => false,
      "user" => false,
      "group" => false,
      "other" => false,
      "mask" => false,
      //acl rights
      "users" => Array(),
      "groups" => Array(),
     );
    }
    else if($file)
    {
     if(preg_match("/^default/i", $lines[$c])) //skip default mask
      continue;
     else if(preg_match("/^# (owner|group): ([^\s]+)$/i", $lines[$c], $m))
     {
      if($m[1]=="owner")
       $acl[$file]["userName"]=$m[2];
      else
       $acl[$file]["groupName"]=$m[2];
     }
     else if(preg_match("/^(user|group|other|mask):([^:]*):([^\s]+)$/i", $lines[$c], $m))
     {
      if($m[2]) //acl
      { 
       if($m[1]=="other") // huh?
        continue;

       $acl[$file][$m[1]."s"][$m[2]]=($type=="string"?$m[3]:$this->fileModeConvert($m[3], $type));
      }
      else //normal rights
       $acl[$file][$m[1]]=($type=="string"?$m[3]:$this->fileModeConvert($m[3], $type));
     }
    }
   }

   if(sizeof($files)==1)
   {
    $file=($files[0][0]=="/"?substr($files[0], 1):$files[0]); //remove leading slash
    return $acl[$file];
   }

   return $acl;
  }


  function setACLUser($file, $user, $mode, $recursive=false)
  {
   if(!FS::isInitialized()) //object should be initialized
    return FS::setACL($file, false); //always fail

   $acl=Array("users" => Array($user => $mode));
   return $this->setACL($file, $acl, $recursive);
  }

  function setACLGroup($file, $group, $mode, $recursive=false)
  {
   if(!FS::isInitialized()) //object should be initialized
    return FS::setACL($file, false); //always fail

   $acl=Array("groups" => Array($group => $mode));
   return $this->setACL($file, $acl, $recursive);
  }

  function setACL($file, $acl, $recursive=false)
  {
   if(!FS::isInitialized()) //object should be initialized
   {
    global $_config;
    if(isset($_config["debug"])&&$_config["debug"]) 
     echo "FS::setACL doesn't support SRO call!";

    return false;
   }

   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   $args="";

   if($acl===false)
    $args.=" --remove-all"; 
   else //if(is_array($acl))
   {
    if(!($cur=$this->getACL($file, "string")))
     return false;

    if(isset($acl["user"])) 
    {
     if(($mode=$this->fileModeConvert($acl["user"], "string"))===false)
      return false;

     if($mode!=$cur["user"])
      $args.=" --modify user::".$mode; //change user rights
    }

    if(isset($acl["group"])) 
    {
     if(($mode=$this->fileModeConvert($acl["group"], "string"))===false)
      return false;

     if($mode!=$cur["group"])
      $args.=" --modify group::".$mode; //change group rights
    }

    if(isset($acl["other"])) 
    {
     if(($mode=$this->fileModeConvert($acl["other"], "string"))===false)
      return false;

     if($mode!=$cur["other"])
      $args.=" --modify other::".$mode; //change other rights
    }

    if(isset($acl["users"]))
    {
     foreach($acl["users"] as $user => $mode)
     {
      if(!preg_match("/^[a-z0-9]+$/", $user))
      {
       $this->e->add("Illegal username: \"".$user."\"", "error");
       return false;
      }

      if(($mode=$this->fileModeConvert($mode, "string"))===false)
       return false;

      if(!isset($cur["users"][$user])||$mode!=$cur["users"][$user])
      {
       if($mode=="---")
        $args.=" --remove user:".$user;
       else
        $args.=" --modify user:".$user.":".$mode;
      }
     }
    }

    if(isset($acl["groups"]))
    {
     foreach($acl["groups"] as $group => $mode)
     {
      if(!preg_match("/^[a-z0-9]+$/", $group))
      {
       $this->e->add("Illegal groupname: \"".$group."\"", "error");
       return false;
      }

      if(($mode=$this->fileModeConvert($mode, "string"))===false)
       return false;

      if(!isset($cur["groups"][$group])||$mode!=$cur["groups"][$group])
      {
       if($mode=="---")
        $args.=" --remove group:".$group;
       else
        $args.=" --modify group:".$group.":".$mode;
      }
     }
    }
   }

   if(!$args)
   {
    $this->e->add("No ACL changes for: \"".$file."\"", "note");
    return true;
   }

   if($recursive)
    $args.=" --recursive --physical"; //ignore symlinks

   $args.=" \"".$file."\"";

   if(!$this->cli->exec($this->_setACL.$args))
   {
    if($this->cli->output)
    {
     if(stristr($this->cli->output[0], "not supported"))
      $this->e->add("Set ACL not support", "error");
     else if(stristr($this->cli->output[0], "not permitted"))
      $this->e->add("Set ACL not permitted", "error");
     else
      $this->e->add($this->cli->e);
    }
    else
     $this->e->add($this->cli->e);

    return false;
   }

   return true;
  }


 }
?>
