<?

 /*****************************************************************************
 * Filename: include/functions.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."plugin.php");

 function d()
 {
  echo "<pre>";
  foreach(func_get_args() as $arg)
  {
   if(array_type($arg)=="data")
   {
    print $arg->title." (".$arg->module."): ";
    print $arg->action." -&gt; ".$arg->extra."\n\n";
    foreach(array_keys($arg->field) as $name)
    {
     if($arg->check("use", "show", $name)||$arg->check("properties", "default", $name))
     {
      print $arg->title($name)." (".$name."):\n";
      foreach($arg->field[$name] as $k => $v)
      {
       if(is_array($v))
        print " - ".$k." => \"".join("\", \"", $v)."\"\n";
       else
        print " - ".$k." => \"".$v."\"\n";
      }
      print "\n\n";
     }
    }
   }
   else if(is_string($arg)&&preg_match("/^SELECT/i", $arg))
    print preg_replace("/(SELECT |FROM |LEFT JOIN |WHERE | AND |GROUP BY |HAVING |ORDER BY |IF)/mi", "\n\\1", $arg)."\n\n";
   else
    var_dump($arg);
  }
  echo "</pre>";
 }

 function modules($moduleType="all", $plugins="all")
 {
  global $_path, $_modules, $_jses;

  if($plugins=="all")
  {
   if(!isset($_modules[$_jses["project"]]))
    $_modules[$_jses["project"]]=Array();

   if(!isset($_modules[$_jses["project"]]["_modules"])) //abuse _modules register in _modules cache 
    $_modules[$_jses["project"]]["_modules"]=Array();

   $cache=&$_modules[$_jses["project"]]["_modules"];

   if(isset($cache[$moduleType])&&$cache[$moduleType])
    return $cache[$moduleType];
  }
   
  $modules=Array();

  $dirs=Array();
  if(isset($_path["project"]["modules"]))
   $dirs["project"]=$_path["project"]["modules"];

  if(isset($_path["modules"]))
   $dirs["system"]=$_path["modules"];

  foreach($dirs as $type => $dir)
  {
   if($moduleType=="all"||$moduleType==$type)
   {
    if(is_dir($dir))
    {
     $readdir=opendir($dir);
     while($file=readdir($readdir))
     {
      if(preg_match("/^([a-z]{1,25})\.php$/i", $file))
      {
       $file=substr($file, 0, -4);
       if(!in_array($file, $modules))
        array_push($modules, $file);
      } 
     }
    }
   }
  }

  if($moduleType=="all"||$moduleType=="plugins")
  {
   if($plugins=="all")
    $plugins=plugins();
   else if(in_array($plugins, plugins()))
    $plugins=Array($plugins);
   else
    $plugins=false;

   if($plugins) 
   {
    foreach($plugins as $p)
    {
     $plugin=new Plugin($p);
     $dir=$plugin->path["modules"];

     if($plugin->isEnabled()&&is_dir($dir))
     {
      $readdir=opendir($dir);
      while($file=readdir($readdir))
      {
       if(preg_match("/^([a-z]{1,25})\.php$/i", $file))
       {
        $module=substr($file, 0, -4);
        $module=$plugin->name."_".$module;

        if(!in_array($module, $modules))
         array_push($modules, $module);
       } 
      }
     }
    }
   }
  }

  sort($modules);

  if(isset($cache))
   $cache[$moduleType]=$modules;

  return $modules;
 }

 function plugins($pluginType="all")
 {
  global $_path;

  $plugins=Array();

  $dirs=Array();
  if(isset($_path["project"]["plugins"]))
   $dirs["project"]=$_path["project"]["plugins"];

  if(isset($_path["plugins"]))
   $dirs["system"]=$_path["plugins"];

  foreach($dirs as $type => $dir)
  {
   if($pluginType=="all"||$pluginType==$type)
   {
    if(is_dir($dir))
    {
     if($readdir=@opendir($dir))
     {
      while($file=readdir($readdir))
      {
       if(is_dir($dir.$file)&&!preg_match("/^\./", $file))
       {
        if(!in_array($file, $plugins))
         array_push($plugins, $file);
       } 
      }
     }
    }
   }
  }

  sort($plugins);
  return $plugins;
 }

 function projects()
 {
  global $_path;

  $projects=Array();
  $dh=opendir($_path["projects"]);
  while($project=readdir($dh))
  {
   if(preg_match("/^([a-z0-9]{1,25})$/i", $project))
    $projects[]=$project;
  }

  sort($projects);
  return $projects;
 }

 function request()
 {
  $keys=func_get_args();

  //cast to a string, inserting 0 in a switch results in it matching the first case
  switch((string)$keys[(sizeof($keys)-1)]) //check if last argument is method selection
  {
   case "POST":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_POST;
    break;
   }
   case "GET":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_GET;
    break;
   }
   case "COOKIE":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_COOKIE;
    break;
   }
   case "SESSION":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_SESSION;
    break;
   }
   case "SERVER":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_SERVER;
    break;
   }
   case "FILES":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_FILES;
    break;
   }
   case "REQUEST":
   {
    array_pop($keys); //remove last "method" argument
    $r=&$_REQUEST;
    break;
   }
   default: //autodetect
   {
    if(isset($_POST[$keys[0]]))
     $r=&$_POST;
    else if(isset($_GET[$keys[0]]))
     $r=&$_GET;
    else
     return false;

    break;
   }
  }

  foreach($keys as $key)
  {
   if(is_array($r)&&isset($r[$key]))
   {
    $tmp=$r[$key];
    unset($r); //remove reference to old predefined variable
    $r=&$tmp;
   }
   else
    return false;
  }
  return $r;
 }

 function array_type(&$array)
 {
  if(is_array($array))
  {
   $type="indexed";

   for($c=0;$c<sizeof($array);$c++)
   {
    if(!isset($array[$c]))
    {
     $type="associative";
     break;
    }
   }
  }
  else if(is_object($array)&&preg_match("/data/i", get_class($array)))
   $type="data";
  else
   $type="unknown";

  return $type;
 }
?>
