<?

 /*****************************************************************************
 * Filename: include/headers.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."mime.php");

 class Headers
 {
  function Headers()
  {
   $this->e=new Error();
   $this->e->l->read("headers");

   $this->mime=false;
  }

  function isInitialized()
  {
   if(!isset($this))
    return false;

   if(get_class($this)=="Headers") //object initialized
    return true;

   $tmp=$this;
   while($tmp=get_parent_class($tmp)) //lookup if parent class is Headers
    if($tmp=="Headers") //object initialized
     return true;

   return false;
  }


  function header($key, $value)
  {
   if(headers_sent())
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Can't add HTTP header: \"".$key."\", cache already sent", "error", $value);

    return false;
   }

   header($key.": ".$value, true);
   return true;
  }


  function get($key=false)
  {
   //get all headers
   $headers=Array();
   foreach(headers_list() as $header)
   {
    list($k, $v)=explode(":", $header, 2);
    $headers[$k]=ltrim($v, " ");
   }

   if(!$key) //no key search
    return $headers; //return associative header list

   if(isset($headers[$key]))
    return $headers[$key];

   if(Headers::isInitialized()) //object initialized
    $this->e->add("Can't get HTTP header: \"".$key."\", no such header", "error");

   return false;
  }


  function delete($key=false)
  {
   if(!function_exists("header_remove"))
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Can't delete HTTP header: \"".$key."\", PHP 5.3 or higher is required", "error");

    return false;
   }

   //remove all headers
   if($key===false)
   {
    header_remove();
    return true;
   }

   //remove header by key
   foreach(headers_list() as $header) //get all headers
   {
    if(strpos($header, $key.":")!==false) //key matches
    {
     header_remove($key);
     return true;
    }
   }

   if(Headers::isInitialized()) //object initialized
    $this->e->add("Can't delete HTTP header: \"".$key."\", no such header", "error");

   return false;
  }


  function basic()
  {
   //send basic headers
   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->cache()) //caching not allowed
     return false;
   }
   else
   {
    if(!Headers::cache()) //caching not allowed
     return false;
   }

   return true;
  }


  function cache($method=false, $store=true, $expires=false, $delta=false)
  {
   //set default method
   if(!$method)
   {
    if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
    {
     $method="private"; //default is private to ensure files can be saved to disk
     $store=false; //don't store so caching is still effectively disabled
    }
    else //non HTTPS or non IE
     $method="no-cache"; //no caching
   }

   if(!in_array($method, Array("private", "public", "no-cache")))
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Unknown header method: \"".$method."\"", "error");

    return false;
   }

   //cache method
   $control=$method;

   if($method=="no-cache")
    $control.=", must-revalidate";

   //store cache
   if(!$store)
    $control.=", no-store";

   //expiry time
   if($expires===false) //no expiry time given
   {
    $expires=0; //beginning of time
    $delta=false; //no delta
   }

   if($delta) //expiry is a delta compared to current time
    $expires+=time(); //make expiry delta an exact date time

   //add max-age (delta) control directive
   $control.=", max-age=".($expires-time());

   //proper expiry time format
   $expires=gmdate("D, d, M Y H:i:s T", $expires);

   //send headers
   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->header("Cache-Control", $control)) //http 1.1
     return false;

    if(!$this->header("Pragma", $method)) //http 1.0
     return false;

    if(!$this->header("Expires", $expires)) //proxies
     return false;
   }
   else
   {
    if(!Headers::header("Cache-Control", $control)) //http 1.1
     return false;

    if(!Headers::header("Pragma", $method)) //http 1.0
     return false;

    if(!Headers::header("Expires", $expires)) //proxies
     return false;
   }

   return true;
  }


  function filename($filename, $attachment=true)
  {
   if(!$filename)
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Won't send filename HTTP header", "error", "No filename given");

    return false;
   }

   $disposition="";
   if($attachment)
    $disposition.="attachment";

   $disposition.=($disposition?"; ":"")."filename=\"".$filename."\"";

   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->header("Content-Disposition", $disposition)) //filename
     return false;
   }
   else
   {
    if(!Headers::header("Content-Disposition", $disposition)) //filename
     return false;
   }

   return true;
  }


  function type($type)
  {
   if(!preg_match("!^[A-Z0-9]+/.+$!i", $type))
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Won't send type HTTP header", "error", "Invalid content type: \"".$type."\"");

    return false;
   }

   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->header("Content-Type", $type)) //mime type
     return false;
   }
   else
   {
    if(!Headers::header("Content-Type", $type)) //mime type
     return false;
   }

   return true;
  }


  function size($size)
  {
   if(!preg_match("/^\d+$/", $size))
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Won't send size HTTP header", "error", "Size: \"".$size."\" isn't a positive number");

    return false;
   }

   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->header("Content-Length", $size)) //file size
     return false;
   }
   else
   {
    if(!Headers::header("Content-Length", $size)) //file size
     return false;
   }

   return true;
  }


  function file($filename, $type=false, $size=false)
  {
   if(!$filename)
   {
    if(Headers::isInitialized()) //object initialized
     $this->e->add("Won't send file HTTP header", "error", "No filename given");

    return false;
   }

   //lookup file type
   if($type===true)
   {
    if(Headers::isInitialized()) //object initialized
    {
     if(!$this->mime) //cache Mime database
      $this->mime=new Mime();

     $mime=&$this->mime;
    }
    else
     $mime=new Mime(); //no Mime database caching

    //lookup mime type for filename
    $type=$mime->filename($filename);
   }

   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->filename($filename, true)) //filename (attachement)
     return false;

    if($type)
     if(!$this->type($type)) //mime type
      return false;

    if($size!==false)
     if(!$this->size($size)) //file size
      return false;
   }
   else
   {
    if(!Headers::filename($filename, true)) //filename (attachement)
     return false;

    if($type)
     if(!Headers::type($type)) //mime type
      return false;

    if($size!==false)
     if(!Headers::size($size)) //file size
      return false;
   }

   return true;
  }


  function server($jas=false, $apache=false, $php=false, $ssl=false, $custom=false)
  {
   //lookup jas version 
   if($jas===true)
   {
    $jas="JAS/1.0"; //XXX: static version
   }

   //lookup apache version
   if($apache===true)
   {
    if(preg_match("!(Apache/[0-9\.]+)(\s\([^\)]+\))!", apache_get_version(), $m))
     $apache=$m[0];
    else
    {
     if(Headers::isInitialized()) //object initialized
      $this->e->add("Server HTTP header failed", "warning", "Unable to get Apache version from: \"".apache_get_version()."\"");

     $apache=false;
    }
   }

   //lookup php version
   if($php===true)
   {
    if(preg_match("!(PHP/[0-9\.]+)!", apache_get_version(), $m))
     $php=$m[0];
    else
    {
     if(Headers::isInitialized()) //object initialized
      $this->e->add("Server HTTP header failed", "warning", "Unable to get PHP version from: \"".apache_get_version()."\"");

     $php=false;
    }
   }

   //lookup ssl version
   if($ssl===true)
   {
    if(preg_match("!(OpenSSL/[0-9a-z\.]+)!", apache_get_version(), $m))
     $ssl=$m[0];
    else
    {
     if(Headers::isInitialized()) //object initialized
      $this->e->add("Server HTTP header failed", "warning", "Unable to get OpenSSL version from: \"".apache_get_version()."\"");

     $ssl=false;
    }
   }

   $server="";

   if($jas)
    $server.=($server?" ":"").$jas;
   if($apache)
    $server.=($server?" ":"").$apache;
   if($ssl)
    $server.=($server?" ":"").$ssl;
   if($php)
    $server.=($server?" ":"").$php;
   if($custom)
    $server.=($server?" ":"").$custom;

   if(Headers::isInitialized()) //object initialized
   {
    if(!$this->header("Server", $server))
     return false;
   }
   else
   {
    if(!Headers::header("Server", $server))
     return false;
   }

   return true;
  }


 }
?>
