<?

 /*****************************************************************************
 * Filename: include/http.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."socket.php");
 require_once($_path["include"]."jas.php");

 class HTTP extends Socket
 {
  function HTTP($userAgent=false)
  {
   parent::Socket();
   $this->e->l->read("http");

   if($userAgent!==false)
    $this->userAgent=$userAgent;
   else
    $this->userAgent="JAS/".JAS::version();

   $this->lb="\r\n";

   $this->methods=Array("GET", "POST");

   $this->clear();

   $this->blocking(true);

   $this->requestTime=&$this->connectionTime;
  }

  function clear()
  {
   parent::clear();

   $this->vars=Array();

   $this->header=Array();
   $this->body=false;

   return true;
  }

  function addVar($key, $value)
  {
   $c=sizeof($this->vars);
   $this->vars[$c]["key"]=$key;
   $this->vars[$c]["value"]=$value;

   return true;
  }

  function addCookie($key, $value)
  {
   foreach($this->header as $k => $v)
   {
    if(preg_match("/Cookie:\s(.*)$/i", $v))
    {
     $this->header[$k]=$v."; $key=".$value;
     return true;
    }
   }

   $this->addHeader("Cookie: ".$key."=".$value);

   return true;
  }

  function addHeader($value, $replace=false, $start=false)
  {
   if($replace)
   {
    if(preg_match("/^([^:]+):/i", $value, $tmp)) //find key
    {
     if(($entry=$this->searchHeader($tmp[1], true))!==false) //entry found
     {
      if($start) //insert/replace at start
      {
       if($entry>0) //currently not at start, delete
       {
        $l=sizeof($this->header);
        for($c=$entry+1;$c<$l;$c++) //remove entry
         $this->header[$c-1]=$this->header[$c];

        unset($this->header[$l-1]);

        array_unshift($this->header, $value); //add entry to start
        return true;
       }
       else //just replace first entry
       {
        $this->header[$entry]=$value; //overwrite 
        return true;
       }
      }
      else
      {
       $this->header[$entry]=$value; //overwrite entry
       return true;
      }
     }
     // else // not found, add below
    }
    else
    {
     $this->e->add("Unable to detect header name from header: \"".$value."\"", "error");
     return false;
    }
   }

   //just add, either at start or end
   if($start)
    array_unshift($this->header, $value);
   else    
    $this->header[]=$value;

   return true;
  }

  function searchHeader($search, $returnEntry=false)
  {
   foreach($this->header as $k => $v)
   {
    if(preg_match("/".$search."[^:]*:\s(.*)$/i", $v, $tmp))
    {
     if($returnEntry)
      return $k;
     else
      return $tmp[1];
    } 
   }

   return false;
  }
  
  function request($url, $method="GET")
  {
   $method=strtoupper($method);
   if(!in_array($method, $this->methods))
   {
    $this->e->add("Unknown method: \"".$protocol."\"");
    return false;
   }
   
   if(!preg_match("/^(\w+)\:\/\/(.*)$/", $url, $tmp))
   {
    $this->e->add("Can't get the protocol from URL");
    return false;
   }
   $protocol=$tmp[1];  

   if(!preg_match("/^([a-z0-9\.\-\_]+)(\:.*|\/.*|.*)$/i", $tmp[2], $tmp))
   {
    $this->e->add("Can't get the server name from URL");
    return false;
   }
   $server=$tmp[1];

   if(preg_match("/^:(\d+)(.*)$/i", $tmp[2], $tmp2))
   {
    $port=$tmp2[1];
    $tmp=$tmp2;
    unset($tmp2);
   }
   else 
    $port=80;

   if(!preg_match("/^(\/?[^\?]*)(.*)$/i", $tmp[2], $tmp))
   {
    $this->e->add("Can't get the location from URL");
    return false;
   }
   $location=$tmp[1];

   if(preg_match("/^\?(.*)$/i", $tmp[2], $tmp))
   {
    foreach(preg_split("/\&|$/", $tmp[1]) as $v)
    {
     if($v&&list($key, $value)=explode("=", $v))
      $this->addVar($key, $value);
    }
   }

   $data="";
   foreach($this->vars as $var)
    $data.=$var["key"]."=".urlencode($var["value"])."&";

   $buffer="";
   if($method=="GET"&&sizeof($data)>0)
    $buffer.=$method." ".$location."?".$data." HTTP/1.1".$this->lb;
   else
    $buffer.=$method." ".$location." HTTP/1.1".$this->lb;

   if($this->userAgent)
    $buffer.="User-Agent: ".$this->userAgent.$this->lb;

   $buffer.="Host: ".$server.$this->lb;
   $buffer.="Content-type: application/x-www-form-urlencoded".$this->lb;

   foreach($this->header as $header)
    $buffer.=$header.$this->lb;

   if($method=="POST")
    $buffer.="Content-length: ".strlen($data).$this->lb;

   $buffer.="Connection: close".$this->lb.$this->lb;

   if($method=="POST")
    $buffer.=$data;


   if(!$this->connect($server, $port, "tcp"))
   { 
    $this->e->add("HTTP request to: \"".$url."\" failed", "error");
    return false;
   }

   if(!$this->write($buffer))
   {
    $this->close();
    return false;
   }

   $response=$this->read();
   $this->close();

   if(!$response)
   {
    $this->e->add("No response data");
    return false;
   }


   $response=explode($this->lb.$this->lb, $response, 2);
   $this->header=explode($this->lb, $response[0]);
   $this->body=$response[1];

   if(!preg_match("/^HTTP\/1.\d\s([0-9]+)\s(.*)$/i", $this->header[0], $tmp))
   {
    $this->e->add("Illegal HTTP header in response");
    return false;
   }
    
   switch($tmp[1])
   {
    case "200": //OK
    {
     break;
    }
    case "301": //page moved
    case "302":
    {
     $this->e->add("Server response: ".$tmp[2]. " (".$tmp[1].")", "warning");
     $this->e->add("New location: ".$this->searchHeader("location"), "note");
     return false;
    }
    default: //other errors
    {
     $this->e->add("Server response: ".$tmp[2]. " (".$tmp[1].")");
     return false;
    }
   }

   return true; 
  }

 } 

 class DataHTTP extends HTTP
 {
  function DataHTTP($url, $project, $username, $password)
  {
   parent::HTTP();
   $this->e->l->read("http");

   if(preg_match("/\.php$/i", $url))
    $this->url=$url;
   else
    $this->url=$url."data.php"; //default communications file

   $this->addVar("project", $project);
   $this->addVar("username", $username);
   $this->addVar("password", $password);

   $this->data=Array();

   $this->compression();
  }

  function compression()
  {
   $compression=Array();
   if(function_exists("bzcompress"))
    $compression[]="BZ2";
   if(function_exists("gzcompress"))
    $compression[]="GZ";

   if(isset($this))
    $this->compression=$compression;

   return $compression;
  }

  function addData(&$data)
  {
   $this->data[]=&$data;
  }

  function &getData($entry)
  {
   return $this->data[$entry];
  }

  function sizeofData()
  {
   return sizeof($this->data);
  } 

  function send($compression=false) 
  {
   if(strtolower($compression)=="none")
    $compression=false;

   if($compression)
   {
    if(!in_array($compression, $this->compression))
    {
     $this->e->add("Compression method: \"".$compression."\" not supported");
     return false;
    }

    $this->addVar("dataCompression", $compression);
   }

   for($c=0;$c<sizeof($this->data);$c++)
   {
    switch($compression)
    {
     case "BZ2":
     {
      $this->addVar("data[]", bzcompress(serialize($this->data[$c])));
      break;
     }
     case "GZ":
     {
      $this->addVar("data[]", gzcompress(serialize($this->data[$c])));
      break;
     }
     default:
     {
      $this->addVar("data[]", serialize($this->data[$c]));
      break; 
     }
    }
   
    $this->data[$c]->__wakeup(); //make original object usable again
   }

   if(!$this->request($this->url, "POST"))
    return false;

   if(preg_match("/dataCompression=([^&".$this->lb."]*)&?/", $this->body, $tmp))
    $compression=$tmp[1];
   else
    $compression=false;

   if(preg_match("/error=([^&".$this->lb."]*)&?/", $this->body, $tmp))
   {
    switch($compression)
    {
     case "BZ2":
     {
      $tmp=unserialize(bzdecompress(urldecode($tmp[1])));
      break;
     }
     case "GZ":
     {
      $tmp=unserialize(gzuncompress(urldecode($tmp[1])));
      break;
     }
     default:
     {
      $tmp=unserialize(urldecode($tmp[1]));
      break; 
     }
    }

    $this->e->add($tmp);
    unset($tmp);
   }

   if($this->e->any("fatal"))
    return false;
   
   $this->data=Array(); //reset array for response

   if($matches=preg_match_all("/data\[\]=([^&".$this->lb."]*)&?/", $this->body, $tmp))
   {
    for($c=0;$c<$matches;$c++)
    {
     switch($compression)
     {
      case "BZ2":
      {
       $this->data[$c]=unserialize(bzdecompress(urldecode($tmp[1][$c])));
       break;
      }
      case "GZ":
      {
       $this->data[$c]=unserialize(gzuncompress(urldecode($tmp[1][$c])));
       break;
      }
      default:
      {
       $this->data[$c]=unserialize(urldecode($tmp[1][$c]));
       break; 
      }
    }

    }
   }
   else
   {
    $this->e->add("No data objects in response");    
    return false;
   }

   return true;
  }

 }
?>
