<?

 /*****************************************************************************
 * Filename: include/httpd.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."http.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."mime.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."mp.php");

 class HTTPD extends HTTP
 {
  function HTTPD()
  {
   parent::HTTP();

   $this->e->logging="stderr"; //display errors via stderr
   $this->e->logErrorTypes=$this->e->errorType; //send all errors to stderr

   $this->e->l->read("httpd");

   $this->serverString="JAS HTTP Server PHP/".phpversion();
   $this->serverRoot=getcwd();

   $this->remoteHost=false;
   $this->remotePort=false;

   $this->readBreak=true; //stop reading if less then readBytes are received 

   $this->mime=new Mime();

   if((!($this->tmpDir=ini_get("upload_tmp_dir")))||!is_dir($this->tmpDir))
   {
    if(!is_dir("/tmp/"))
    {
     $this->tmpDir="~/";
     $this->e->add("Can't determine tmp directory, using home", "system");
    }
    else
     $this->tmpDir="/tmp";
   }

   if(!($this->sessionName=ini_get("session.name")))
    $this->sessionName=PHPSESSID;

   if($this->tmpDir[strlen($this->tmpDir)-1]!="/")
    $this->tmpDir.="/";

   $this->upload_max_filesize=ini_get("upload_max_filesize");
   if(preg_match("/^(\d+)(G|M|K|B)/i", $this->upload_max_filesize, $tmp))
   {
    switch(strtoupper($tmp[2]))
    {
     case "G":
      $this->upload_max_filesize=$tmp[1]*pow(10, 9);
     break;
     case "M":
      $this->upload_max_filesize=$tmp[1]*pow(10, 6);
     break;
     case "K":
      $this->upload_max_filesize=$tmp[1]*pow(10, 3);
     break;
     default:
      $this->upload_max_filesize=$tmp[1];
     break;
    }
   }
   else
    $this->upload_max_filesize=(int)$this->upload_max_filesize;

   $this->clear();
   $this->readConfig();

   $this->_SERVER=Array(
    "HTTP_HOST" => (isset($_SERVER["HOSTNAME"])?$_SERVER["HOSTNAME"]:"unknown"), 
    "SERVER_SIGNATURE" => "", 
    "SERVER_SOFTWARE" => &$this->serverString, 
    "SERVER_NAME" => "localhost", 
    "SERVER_ADDR" => "127.0.0.1", 
    "SERVER_PORT" => &$this->config["port"], 
    "DOCUMENT_ROOT" => &$this->serverRoot, 
    "PATH" => $_SERVER["PATH"], 
   );
  }

  function readConfig()
  {
   global $_path;

   $this->config=Array();
   $d=new Data("httpdConfig", "view", "all", Array("db" => "xml://".$_path["include"]."httpd.xml"));

   foreach($d->check("use", "show") as $field)
    $this->config[$field]=$d->getDefaultValue($field);

   $d->setValue("show_total", 1);

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   foreach($d->check("use", "show") as $field)
    if($field!="r")
     $this->config[$field]=$d->v[$field];

   return true;
  }

  function start()
  {
   if($this->socket!==false)
   {
    $this->e->add("HTTP Server already started", "system");
    return true;
   }

   $this->e->add("HTTP Server starting", "system");

   if(!$this->setReadType("binary"))
    return false;

   if(!$this->listen("0.0.0.0", $this->config["port"], "tcp"))
    return false;

   if(!$this->setOption(SO_LINGER, Array('l_onoff' => 1, 'l_linger' => 0)))
   {
    $this->stop();
    return false;
   }

   if($this->config["gid"]&&posix_getegid()!=$this->config["gid"])
   {
    if(!posix_setegid($this->config["gid"]))
    {
     $this->e->add("HTTP can't become group: \"".$this->config["gid"]."\"", "error");
     $this->stop();
     return false;
    }
   }

   if($this->config["uid"]&&posix_geteuid()!=$this->config["uid"])
   {
    if(!posix_seteuid($this->config["uid"]))
    {
     $this->e->add("HTTP can't become user: \"".$this->config["uid"]."\"", "error");
     $this->stop();
     return false;
    }
   }

   pcntl_alarm($this->config["alarm"]);

   $this->e->add("HTTP Server started", "system");
   return true;
  }

  function stop()
  {
   if($this->socket===false)
   {
    $this->e->add("HTTP Server already stopped", "system");
    return false;
   }

   $this->e->add("HTTP Server stopping", "system");

   $this->close();

   $this->e->add("HTTP Server stopped", "system");
   return true;
  }

  function accept()
  {
   $this->e->clear();

   if($this->socket===false)
   {
    $this->e->add("HTTP Server not running", "error");
    return false;
   }

   while(!($socket=&parent::accept()))
   {
    if($this->errno>0) //real error or just an alarm?
     return false;

    pcntl_alarm($this->config["alarm"]); //reset alarm
   }

   pcntl_alarm(0); //disable alarm during connection handling

   if(!@socket_getpeername($socket->socket, $this->remoteHost, $this->remotePort))
   {
    $this->remoteHost=false;
    $this->remotePort=false;
   }

   //might need while to ensure full body is gathered
   $request=$socket->read();

   $request=explode($this->lb.$this->lb, $request);
   $this->header=explode($this->lb, $request[0]);
   $this->body="";

   if(preg_match("/^POST/", $this->header[0]))
   {
    if($contentLength=$this->searchHeader("Content-Length"))
    {
     if(isset($request[1])&&$request[1])
      $this->body=$request[1];
     else
      $this->body=$socket->read();

     if($contentLength>strlen($this->body)) //too much received
      $this->body=substr($this->body, 0, $contentLength);
     else if($contentLength<strlen($this->body)) //too little received
     {
      $this->e->add("Body length is: ".strlen($this->body).", it should be: ".$contentLength."", "error");
      return false;
     }
    }
   }

   if(($FORKSCRIPT=$this->handleRequest())&&$FORKSCRIPT!==true) //return FORKSCRIPT to global variable scope
    return $FORKSCRIPT;

   $headers=implode($this->lb, $this->header);

   $socket->write($headers.$this->lb.$this->lb.$this->body);
   $socket->close();

   pcntl_alarm($this->config["alarm"]); //reset alarm

   $this->bytesSend=$socket->bytesSend;
   $this->bytesReceived=$socket->bytesReceived;
   $this->requestTime=$socket->connectionTime;

   return true;
  }

  function handleRequest()
  {
   $this->request=Array();
   $this->request["header"]=$this->header;
   $this->request["body"]=$this->body;
   
   $this->header=Array();
   $this->body="";

   $this->addHeader("Date: ".date("r"));
   $this->addHeader("Server: ".$this->serverString);

   if(!$this->_setRequestMethod()) 
   {
    $this->e->add("Unknown request", "error"); 

    $this->addHeader("HTTP/1.1 501 Not Implemented", false, true);
    $this->addHeader("Content-Type: text/html; charset=iso-8859-1");
    $this->addHeader("Content-Length: 0");

    unset($this->request);
    $this->body="";

    return false;
   }

   if(!$this->_setRequestFileAndArgs()) 
   {
    $this->e->add("File doesn't exist: ".$this->request["filename"], "error"); 

    $this->addHeader("HTTP/".$this->request["version"]." 404 Not Found", false, true);
    $this->addHeader("Content-Type: text/html; charset=iso-8859-1");
    $this->addHeader("Content-Length: 0");

    unset($this->request);
    $this->body="";

    return false;
   }

   if(preg_match("/\.php$/", $this->request["file"])) //PHP 
   {
    $this->_setRequestVars(); //_GET, _POST, _FILES, _COOKIE
    $this->_generateServerVar(); //_SERVER

    if(!isset($this->request["_COOKIE"][$this->sessionName]))
    {
     $this->request["_COOKIE"][$this->sessionName]=md5($this->remoteHost.$this->remotePort.time());
     $this->addHeader("Set-Cookie: ".$this->sessionName."=".$this->request["_COOKIE"][$this->sessionName]);
    }

    if($this->config["mode"]=="fork")
    {
     if(!($FORKSCRIPT=$this->_handlePHPFork()))
     {
      $this->e->add("PHP request failed: ".$this->request["filename"], "error"); 

      $this->addHeader("HTTP/".$this->request["version"]." 500 Internal Server Error", false, true);
      $this->addHeader("Content-Type: text/html; charset=iso-8859-1");
      $this->addHeader("Content-Length: 0");

      unset($this->request);
      $this->body="";

      return false;
     }

     if($FORKSCRIPT!==true) //return FORKSCRIPT to accept loop
      return $FORKSCRIPT;
    }
    else //exec
    {
     if(!$this->_handlePHPExec())
     {
      $this->e->add("PHP request failed: ".$this->request["filename"], "error"); 

      $this->addHeader("HTTP/".$this->request["version"]." 500 Internal Server Error", false, true);
      $this->addHeader("Content-Type: text/html; charset=iso-8859-1");
      $this->addHeader("Content-Length: 0");

      unset($this->request);
      $this->body="";

      return false;
     }
    }

    $this->e->add("PHP request: ".$this->request["filename"], "note"); 

    $this->addHeader("HTTP/".$this->request["version"]." 200 OK", false, true);
   }
   else //normal files
   {
    if(!$this->_handleFileRequest())
    {
     $this->e->add("Request failed: ".$this->request["filename"], "error"); 

     $this->addHeader("HTTP/".$this->request["version"]." 403 Forbidden", false, true);
     $this->addHeader("Content-Type: text/html; charset=iso-8859-1");
     $this->addHeader("Content-Length: 0");

     unset($this->request);
     $this->body="";

     return false;
    }

    $this->e->add("Request: ".$this->request["filename"], "note"); 

    $this->addHeader("HTTP/".$this->request["version"]." 200 OK", false, true);
    $this->addHeader("Last-Modified: ".date("r", filemtime($this->request["file"])));

    if(isset($this->request["contentType"]))
     $this->addHeader("Content-Type: ".$this->request["contentType"]);
   }

   $this->addHeader("Content-Length: ".strlen($this->body));
   unset($this->request);
   return true;
  }

  function _setRequestMethod()
  {
   if(preg_match("/^(GET|POST)\s([^\s]+)\sHTTP\/(\d\.\d)/", $this->request["header"][0], $m))
   {
    $this->request["method"]=$m[1];
    $this->request["string"]=$m[2];
    $this->request["version"]=$m[3];

    return true;
   }

   return false;
  }

  function _setRequestFileAndArgs()
  {
   $tmp=explode("?", $this->request["string"]);

   $this->request["filename"]=$tmp[0];
   $this->request["file"]=FS::realPath($this->serverRoot.$this->request["filename"]);

   if(isset($tmp[1]))
    $this->request["args"]=$tmp[1];
   else
    $this->request["args"]="";

   if(is_dir($this->request["file"]))
    $this->request["file"].="index.php";

   if(!FS::inDir($this->serverRoot, $this->request["file"])) //tries to break out of web root?
    return false; 

   if(!file_exists($this->request["file"]))
    return false;

   return true;
  }

  function _setRequestVars()
  {
   $this->request["_GET"]=Array();
   $this->request["_POST"]=Array();
   $this->request["_FILES"]=Array();
   $this->request["_COOKIE"]=Array();
   $this->request["_REQUEST"]=Array();

   if(preg_match_all("/([^?&=]+)=([^?&=]+)/", $this->request["string"], $m))
   {
    for($c=0;$c<sizeof($m[1]);$c++)
     $this->request["_GET"][$m[1][$c]]=$m[2][$c];
   }

   if($this->request["method"]=="POST"&&preg_match("/boundary=(.+)$/", $this->searchRequestHeader("Content-Type"), $m))
   {
    $boundary="--".$m[1];

    foreach(explode($boundary, $this->request["body"]) as $var)
    {
     if(preg_match("/Content-Disposition: form-data; name=\"([^\\\"]+)\"(; filename=\"([^\\\"]+)\"[\n\r]+Content-Type: ([^\n\r]+))?/s", $var, $m, PREG_OFFSET_CAPTURE))
     {
      $key=$m[1][0];

      $linebreaksize=$m[0][1];
      $value=substr($var, (3*$linebreaksize)+strlen($m[0][0]), -($linebreaksize));

      $filename=(isset($m[3])?$m[3][0]:false);
      $type=(isset($m[4])?$m[4][0]:false);

      if(!$filename) // _POST
       $this->request["_POST"][$key]=$value;
      else // _FILES
      {
//XXX: strip ../ from filename, ensure 
       d($filename);
       $size=strlen($value);
       $tmpfile=$this->tmpDir.md5($filename.$host);

       if($size>$this->upload_max_filesize)
       {
        $error=UPLOAD_ERR_INI_SIZE;
        $tmpfile="";
       }
       else if(isset($post["MAX_FILE_SIZE"])&&$size>$post["MAX_FILE_SIZE"])
       {
        $error=UPLOAD_ERR_FORM_SIZE;
        $tmpfile="";
       }
       else if(!@file_put_contents($tmpfile, $value))
       {
        $error=UPLOAD_ERR_CANT_WRITE;
        $tmpfile="";
       }
       else
        $error=UPLOAD_ERR_OK;

       $this->request["_FILES"][$key]=Array(
        "name" => $filename,
        "type" => $type,
        "tmp_name" => $tmpfile,
        "size" => $size,
        "error" => $error,
       );

      } 
     }
    }
   }

   if(preg_match_all("/\s?([^=;]+)=([^;]*);?/", $this->searchRequestHeader("Cookie"), $m))
   {
    for($c=0;$c<sizeof($m[0]);$c++)
     $this->request["_COOKIE"][$m[1][$c]]=$m[2][$c];
   }

   $this->request["_REQUEST"]=array_merge($this->request["_COOKIE"], $this->request["_GET"], $this->request["_POST"]);
   return true;
  }

  function _generateServerVar()
  {
   $this->request["_SERVER"]=$this->_SERVER;

   $this->request["_SERVER"]["HTTP_HOST"]=$this->searchRequestHeader("Host");
   $this->request["_SERVER"]["HTTP_USER_AGENT"]=$this->searchRequestHeader("User-Agent");
   $this->request["_SERVER"]["HTTP_ACCEPT"]=$this->searchRequestHeader("Accept");
   $this->request["_SERVER"]["HTTP_ACCEPT_LANGUAGE"]=$this->searchRequestHeader("Accept-Language");
   $this->request["_SERVER"]["HTTP_ACCEPT_CHARSET"]=$this->searchRequestHeader("Accept-Charset");
   $this->request["_SERVER"]["HTTP_ACCEPT_ENCODING"]=$this->searchRequestHeader("Accept-Encoding");
   $this->request["_SERVER"]["HTTP_CACHE_CONTROL"]=$this->searchRequestHeader("Cache-Control");
   $this->request["_SERVER"]["HTTP_CONNECTION"]=$this->searchRequestHeader("Connection");
   $this->request["_SERVER"]["HTTP_TE"]=$this->searchRequestHeader("TE");
   $this->request["_SERVER"]["HTTP_REFERER"]=$this->searchRequestHeader("Referer");
   $this->request["_SERVER"]["HTTP_COOKIE"]=$this->searchRequestHeader("Cookie");
   $this->request["_SERVER"]["HTTP_COOKIE2"]=$this->searchRequestHeader("Cookie2");

   $this->request["_SERVER"]["CONTENT_LENGTH"]=$this->searchRequestHeader("Content-Length");
   $this->request["_SERVER"]["CONTENT_TYPE"]=$this->searchRequestHeader("Content-Type");

   if($this->remoteHost)
    $this->request["_SERVER"]["REMOTE_ADDR"]=$this->remoteHost;
   if($this->remotePort)
    $this->request["_SERVER"]["REMOTE_PORT"]=$this->remotePort;

   $this->request["_SERVER"]["SERVER_PROTOCOL"]="HTTP/".$this->request["version"];
   $this->request["_SERVER"]["REQUEST_METHOD"]=$this->request["method"];
   $this->request["_SERVER"]["QUERY_STRING"]=$this->request["args"];
   $this->request["_SERVER"]["REQUEST_URI"]=$this->request["string"];
   $this->request["_SERVER"]["SCRIPT_FILENAME"]=$this->request["file"];
   $this->request["_SERVER"]["SCRIPT_NAME"]=$this->request["filename"];
   $this->request["_SERVER"]["PHP_SELF"]=$this->request["filename"];
   $this->request["_SERVER"]["REQUEST_TIME"]=time();

   return true;
  }

  function _handleFileRequest()
  {
   if(($this->body=@file_get_contents($this->request["file"]))===false)
    return false;

   $contentType=$this->mime->filename($this->request["filename"]);

   if($contentType!="application/octet-stream")
    $this->request["contentType"]=$contentType;

   return true;
  }

  function _handlePHPFork()
  {
   $___mp=new MP();

   if(!$___mp->fork(true))
    return false;

   if($___mp->process=="parent")
   {
    if(!($response=$___mp->recv()))
     return false;

    //filter headers;
    if(preg_match("/^.*".$this->lb.$this->lb."/ms", $response, $m))
    {
     foreach(explode($this->lb, $m[0]) as $header)
      if($header)
       $this->addHeader($header, true); //replace existing headers

     $this->body=str_replace($m, "", $response); //remove headers
    }
    else //no headers
     $this->body=$response;
    //$___mp->end(); //close message bus and wait for child to die
    //we'll skip the dying part, the message bus automatically closed at end of this function
   }
   else //child
   {
    global $_GET, $_POST, $_FILES, $_COOKIE, $_REQUEST, $_SERVER;

    $_GET=$this->request["_GET"];
    $_POST=$this->request["_POST"];
    $_FILES=$this->request["_FILES"];
    $_COOKIE=$this->request["_COOKIE"];
    $_REQUEST=$this->request["_REQUEST"];
    $_SERVER=$this->request["_SERVER"];

    set_error_handler(Array($this, "_childErrorHandler"), E_ALL);
    register_shutdown_function(Array($this, "_childDone"), $___mp);

    //clean up this and mp objects
    unset($this, $___mp);

    //cleanup httpd globals
    global $httpd;
    unset($httpd);

    //clean up JAS globals
    global $_path, $_modules, $_config, $_language, $_jses, $_xmlStorage, $_jses, $_jseskey, $_project, $_benchmark;
    unset($_path, $_modules, $_config, $_language, $_jses, $_xmlStorage, $_jses, $_jseskey, $_project, $_benchmark);

    if(function_exists("override_function")) //override header function, it doesn't work in cli mode, requires APD extension
    {
     global $___headers; 
     $___headers=Array();
     override_function("header", '$value, $replace=true, $http_response_code=false', 'global $___headers; $___headers[]=Array("value" => $value, "replace" => $replace, "http_response_code" => $http_response_code);');
    }

    ob_start();

    //Globals not handled properly, hacked return file to main loop in global variable scope
    return $_SERVER["SCRIPT_FILENAME"];
   }

   return true;
  }

  function _childErrorHandler($errno, $errstr, $errfile, $errline)
  {
   switch($errno)
   {
    case E_ERROR: 
    case E_CORE_ERROR: 
    case E_USER_ERROR: 
    case E_COMPILE_ERROR: 
    {
     $errlevel="FATAL";
     break;
    }
    case E_WARNING: 
    case E_CORE_WARNING: 
    case E_USER_WARNING: 
    case E_COMPILE_WARNING: 
    {
     $errlevel="Warning";
     break;
    }
    case E_NOTICE: 
    case E_USER_NOTICE: 
    {
     $errlevel="Notice";
     break;
    }
    case E_PARSE: 
    {
     $errlevel="Parse";
     break;
    }
    default: 
    {
     $errlevel="Unknown";
     break;
    }
   }

   print "<br/><b>".$errlevel."</b>: ".$errstr." in <b>".$errfile."</b> on line <b>".$errline."</b><br/>\n";
  }

  function _childDone(&$___mp)
  {
   if($error=error_get_last())
   {
    switch($error["type"])
    {
     case E_ERROR: 
     case E_PARSE: 
     case E_CORE_ERROR: 
     case E_USER_ERROR: 
     case E_COMPILE_ERROR: 
     {
      $this->_childErrorHandler($error["type"], $error["message"], $error["file"], $error["line"]);
      break;
     }
     default: break;
    }
   }

   $output=ob_get_contents();
   ob_end_clean();

   global $___headers;
   if($___headers)
   {
    $headers="";
    foreach($___headers as $header)
     $headers.=$header["value"].$this->lb;

    $output=$headers.$this->lb.$output;
   }

   $___mp->send($output);
   $___mp->end(); //close message bus and exit
  }

  function _handlePHPExec()
  {
   $code="<?\n";
   $code.="\$_GET=".preg_replace("/\n/", "", var_export($this->request["_GET"], true))."; \n";
   $code.="\$_POST=".preg_replace("/\n/", "", var_export($this->request["_POST"], true))."; \n";
   $code.="\$_FILES=".preg_replace("/\n/", "", var_export($this->request["_FILES"], true))."; \n";
   $code.="\$_COOKIE=".preg_replace("/\n/", "", var_export($this->request["_COOKIE"], true))."; \n";
   $code.="\$_REQUEST=".preg_replace("/\n/", "", var_export($this->request["_REQUEST"], true))."; \n";
   $code.="\$_SERVER=".preg_replace("/\n/", "", var_export($this->request["_SERVER"], true))."; \n";
   $code.="chdir('".getcwd()."'); \n";
   $code.="include(\"".$this->request["file"]."\"); \n";
   $code.="\n?>";

   $response="";
   $tmpfile=$this->tmpDir.md5(date("c")).".php";

   file_put_contents($tmpfile, $code);
   $exec="php-cgi ".$tmpfile." 2>&1 ";

   exec($exec, $response, $ret);
   unlink($tmpfile);

   while($header=array_shift($response))
    $this->addHeader($header);

   $this->body=implode("\n", $response);
   return true;
  }
  
  function searchRequestHeader($search, $returnEntry=false)
  {
   foreach($this->request["header"] as $k => $v)
   {
    if(preg_match("/".$search."[^:]*:\s(.*)$/i", $v, $tmp))
    {
     if($returnEntry)
      return $k;
     else
      return $tmp[1];
    } 
   }
   return false;
  }

 }
?>
