<?

 /*****************************************************************************
 * Filename: include/jas.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."dt.php");

 class JAS
 {
  function JAS()
  {
   $this->e=new Error();
   $this->e->l->read("JAS");
  }

  function isInitialized()
  {
   if(!isset($this))
    return false;

   if(get_class($this)=="JAS") //object initialized
    return true;

   $tmp=$this;
   while($tmp=get_parent_class($tmp)) //lookup if parent class is JAS
    if($tmp=="JAS") //object initialized
     return true;

   return false;
  }


  function URL()
  {
   return "https://jas.junerules.com/";
  }


  function changelogVersion()
  {
   global $_path, $_jses;

   $changelogFile=$_path["include"]."changelog.txt";

   if(!FS::exists($changelogFile, "file"))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't detect JAS version", "error", "Changelog file: \"".$changelogFile."\" not found");

    return false;
   }

   if(!($changelog=FS::read($changelogFile)))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't detect JAS version", "error", "Can't read changelog file: \"".$changelogFile."\"");

    return false;
   }

   if(!preg_match("/\nVersion ([0-9\.]+)[ \-]*([A-Z0-9]+)?/i", $changelog, $version))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't detect JAS version info", "error", "Can't obtain version from changelog file: \"".$changelogFile."\"");

    return false;
   }

   $development=(preg_match("/\nDevelopment version/", $changelog));

   if(!preg_match("/\n\t(\d{4}-\d{2}-\d{2})/", $changelog, $updated))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't detect JAS version info", "error", "Can't get last updated from changelog file: \"".$changelogFile."\"");

    return false;
   }

   $modified=FS::getTime($changelogFile, "modified");


   $_jses["jas"]["version"]=$version[1];
   $_jses["jas"]["status"]=(isset($version[2])?$version[2]:"");
   $_jses["jas"]["development"]=$development;

   $_jses["jas"]["updated"]=DT::dateConvert($updated[1], "Y-m-d");
   $_jses["jas"]["modified"]=$modified;

   return true;
  }

  function changelogUpdated()
  {
   global $_path, $_jses;

   if(!isset($_jses["jas"]["modified"])) //version information not cached
    return true;

   $changelogFile=$_path["include"]."changelog.txt";

   if(FS::exists($changelogFile, "file"))
    if(FS::getTime($changelogFile, "modified")!=$_jses["jas"]["modified"])
     return true;

   return false;
  }

  function changelog($html=false)
  {
   global $_path, $_jses;

   $changelogFile=$_path["include"]."changelog.txt";

   if(!FS::exists($changelogFile, "file"))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't open JAS changelog", "error", "Changelog file: \"".$changelogFile."\" not found");

    return false;
   }

   if(!($changelog=FS::read($changelogFile)))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't read JAS changelog", "error", "Changelog file: \"".$changelogFile."\"");

    return false;
   }

   if($html) //html mode (otherwise raw text will be used
   {
    $changelog=preg_replace("!\n(Development version)\n!", "<h3>\\1</h3>", $changelog);
    $changelog=preg_replace("!\n(Version [\d\.+[A-Za-z\-]*)\n!", "<h3>\\1</h3>", $changelog);
    $changelog=preg_replace("!(\d+\-\d+\-\d+)\n!", "<h4>\\1</h4>", $changelog);
    $changelog=str_replace("\n", "<br/>\n", $changelog);
    $changelog=str_replace("\t*", "<div class=\"hilite\">*</div>", $changelog);
   }

   return $changelog;
  }


  function version($type=false)
  {
   global $_jses;

   if(!$type)
    $type="version";

   if(JAS::isInitialized()) //object initialized
   {
    if($this->changelogUpdated()) //version information not cached
     if(!$this->changelogVersion())
      return false;
   }
   else
   {
    if(JAS::changelogUpdated()) //version information not cached
     if(!JAS::changelogVersion())
      return false;
   }

   if(!isset($_jses["jas"][$type]))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("JAS version type: \"".$type."\" not supported", "error");

    return false;
   }

   return $_jses["jas"][$type];
  }


  function license($html=false, $type=false)
  {
   global $_path, $_jses;

   if(strtoupper($type)=="GPL")
    $licenseFile=$_path["root"]."GPL.txt";
   else if(strtoupper($type)=="LGPL")
    $licenseFile=$_path["root"]."LGPL.txt";
   else
    $licenseFile=$_path["root"]."LICENSE.txt";

   if(!FS::exists($licenseFile, "file"))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't open JAS license", "error", "License file: \"".$licenseFile."\" not found");

    return false;
   }

   if(!($license=FS::read($licenseFile)))
   {
    if(JAS::isInitialized()) //object initialized
     $this->e->add("Can't read JAS license", "error", "License file: \"".$licenseFile."\"");

    return false;
   }

   if($html) //html mode (otherwise raw text will be used
   {
    $license=preg_replace("!([a-z]+\:\/\/[^\s<>]+)!", "<a href=\"\\1\" title=\"\\1\" rel=\"external\">\\1</a>", $license);
    $license=preg_replace("!\n\s*(\d+\.\s.+)\n!i", "\n\n<h3>\\1</h3>", $license);
    $license=str_replace("\n", "<br/>\n", $license);
    $license=str_replace("\t*", "<div class=\"hilite\">*</div>", $license);
   }

   return $license;
  }


 }
?>
