<?

 /*****************************************************************************
 * Filename: include/language.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."xml.php");

 // lang2xml :%s/^\([^\<\=]\+\)=\(.\+\)$/  <translation original="\1" translated="\2" \/>/                                    

 class Language
 {
  function Language($set=false)
  {
   global $_language;

   $this->default="en";

   if($_language===NULL) //global _language not filled yet, see if it can be restored from session
    $this->restore();

   $this->language=&$_language["translation"];
   $this->loadedFiles=&$_language["loadedFiles"];
   
   $this->storeNotTranslated=false;

   if($set)
    $this->read($set);
  }

  function clear()
  {
   global $_language, $_jses;

   unset($_jses["language"]);
   $_language["translation"]=Array("normal" => Array(), "lowercase" => Array(), "regexp" => Array(), "regexpTypes" => Array(), "notranslation" => Array());
   $_language["loadedFiles"]=Array();
  }
  function store()
  {
   global $_jses, $_language;

   $_jses["language"]=Array();
   $_jses["language"]["translation"]=$_language["translation"];
   $_jses["language"]["loadedFiles"]=$_language["loadedFiles"];
  }
  function restore()
  {
   global $_jses, $_language;

   if(!request("reloadTranslations")&&isset($_jses["language"]))
   {
    $_language["translation"]=$_jses["language"]["translation"];
    $_language["loadedFiles"]=$_jses["language"]["loadedFiles"];
   }
   else
    $this->clear();
  }

  function getFilename($set, $language=false)
  {
   global $_path, $_config;

   if(!$language)
   {
    if(isset($_config["language"]))
     $language=$_config["language"];
    else
     $language=$this->default;
   }

   $files=Array();

   if(preg_match("/^plugin_(.+)$/", $set, $tmp))
   {
    $plugin=$tmp[1];

    if(isset($_path["project"])&&file_exists($_path["project"]["plugins"].$plugin."/language/".$language."/".$plugin.".xml"))
     $file=$_path["project"]["plugins"].$plugin."/language/".$language."/".$plugin.".xml";
    else if(file_exists($_path["plugins"].$plugin."/language/".$language."/".$plugin.".xml"))
     $file=$_path["plugins"].$plugin."/language/".$language."/".$plugin.".xml";
    else
     return false;
   }
   else if(preg_match("/^module_(.+)_(.+)$/", $set, $tmp))
   {
    $plugin=$tmp[1];
    $module=$tmp[2];

    if(isset($_path["project"])&&file_exists($_path["project"]["plugins"].$plugin."/language/".$language."/modules/".$module.".xml"))
     $file=$_path["project"]["plugins"].$plugin."/language/".$language."/modules/".$module.".xml";
    else if(file_exists($_path["plugins"].$plugin."/language/".$language."/modules/".$module.".xml"))
     $file=$_path["plugins"].$plugin."/language/".$language."/modules/".$module.".xml";
    else
     return false;
   }
   else if(preg_match("/^module_(.+)$/", $set, $tmp))
   {
    $module=$tmp[1];

    if(isset($_path["project"])&&file_exists($_path["project"]["include"]."language/".$language."/modules/".$module.".xml"))
     $file=$_path["project"]["include"]."language/".$language."/modules/".$module.".xml";
    else if(file_exists($_path["include"]."language/".$language."/modules/".$module.".xml"))
     $file=$_path["include"]."language/".$language."/modules/".$module.".xml";
    else
     return false;
   }
   else
   {
    if(isset($_path["project"])&&file_exists($_path["project"]["include"]."language/".$language."/".$set.".xml"))
     $file=$_path["project"]["include"]."language/".$language."/".$set.".xml";
    else if(file_exists($_path["include"]."language/".$language."/".$set.".xml"))
     $file=$_path["include"]."language/".$language."/".$set.".xml";
    else
     return false;
   }

   return $file;
  }

  function read($set, $language=false)
  {
   global $_config;

   if(!$language)
   {
    if(isset($_config["language"]))
     $language=$_config["language"];
    else
     $language=$this->default;
   }

   if(!($file=$this->getFilename($set, $language)))
    return false;

   if(isset($this->loadedFiles[$file])&&filemtime($file)<=$this->loadedFiles[$file])
    return true;

   $this->loadedFiles[$file]=filemtime($file);

   $xml=new XML();
   if(!$xml->parseFile($file))
    return false;

   while(($key=$xml->search("language"))!==false)
   {
    if(($originalLanguage=$xml->getAttribute($key, "original"))==$language)
     return true;

    if(($translatedLanguage=$xml->getAttribute($key, "translated"))==$language)
    {
     $childs=$xml->getChildren($key);
     $l=sizeof($childs);

     for($c=0;$c<$l;$c++)
     {
      $original=$xml->getAttribute($childs[$c], "original");
      $translated=$xml->getAttribute($childs[$c], "translated");

      if(preg_match("/\|(translate|notranslate)\|/", $original))
      {
       $regexp="/^".preg_quote($original, "/")."$/sm";
       $regexp=str_replace("\|translate\|", "(.*)", $regexp);
       $regexp=str_replace("\|notranslate\|", "(.*)", $regexp);
       $this->language["regexp"][$originalLanguage][$translatedLanguage][$regexp]=$translated;

       preg_match_all("/\|(translate|notranslate)\|/", $original, $tmp);
       $this->language["regexpTypes"][$originalLanguage][$translatedLanguage][$regexp]=$tmp[1];
      }
      else
      {
       $this->language["normal"][$originalLanguage][$translatedLanguage][$original]=$translated;
       $this->language["lowercase"][$originalLanguage][$translatedLanguage][strtolower($original)]=strtolower($translated);
      }
     }
    }
   }

   return true;
  }


  function t($text, $from=false, $to=false)
  {
   if(!$text)
    return false;

   global $_config;

   if(!$from)
    $from=$this->default;

   if(!$to)
   {
    if(isset($_config["language"]))
     $to=$_config["language"];
    else
     $to=$this->default;
   }

   if($from==$to)
    return $text;

   if(isset($this->language["normal"][$from][$to][$text]))
    return $this->language["normal"][$from][$to][$text];

   $lowercase=strtolower($text);
   if(isset($this->language["lowercase"][$from][$to][$lowercase]))
    return $this->language["lowercase"][$from][$to][$lowercase];

   if(isset($this->language["regexp"][$from][$to]))
   {
    foreach(array_keys($this->language["regexp"][$from][$to]) as $regexp)
    {
     if(preg_match($regexp, $text, $tmp))
     {
      $newText=$this->language["regexp"][$from][$to][$regexp];
      for($c=1;$c<sizeof($tmp);$c++)
      {
       if(is_numeric($tmp[$c])||$this->language["regexpTypes"][$from][$to][$regexp][$c-1]=="notranslate"||in_array($tmp[$c], $this->language["normal"][$from][$to])||in_array($tmp[$c], $this->language["lowercase"][$from][$to]))
        $newText=str_replace("|".$c."|", $tmp[$c], $newText);
       else
        $newText=str_replace("|".$c."|", $this->t($tmp[$c], $from, $to), $newText);
      }

      return $newText;
     }
    }
   }

   if($this->storeNotTranslated)
    $this->language["notranslation"][$from][$to][preg_replace("/\"([^\"\|]*)\"/", "\"|translate|\"", $text)]=preg_replace("/\"([^\"\|]*)\"/", "\"|1|\"", $text);

   if(isset($_config["translationerrors"])&&$_config["translationerrors"])
    return "-!-".$text."-!-";
   else 
    return $text;
  }

  function getNotTranslatedXML()
  {
   global $_path;

   $xml=new XML();
   $xml->parseFile($_path["include"]."xmli/1.0/jas.xml");
   $xml->setAttribute("?xml", "version", "1.0");
   $xml->setAttribute("?xml", "encoding", "ISO-8859-1");
   $xml->setAttribute("jas", "version", "1.0");

   $xmll=new XML();
   $xmll->parseFile($_path["include"]."xmli/1.0/language.xml");

   foreach(array_keys($this->language["notranslation"]) as $from)
   {
    foreach(array_keys($this->language["notranslation"][$from]) as $to)
    {
     $tmp=$xmll->getOuterXML("language");
     $tmp->setAttribute("language", "original", $from);
     $tmp->setAttribute("language", "translated", $to);
     $tmp->set("language->translation", "", true);
     $tmp2=$xmll->getOuterXML("language->translation");

     foreach($this->language["notranslation"][$from][$to] as $regexp => $text)
     {
      $tmp2->setAttribute("translation", "original", $regexp);
      $tmp2->setAttribute("translation", "translated", $text);
      $tmp->addInsideAtEnd("language", $tmp2->output()."\n");
     }
     $xml->addInsideAtEnd("jas", $tmp->output()."\n");
    }
   }

   return $xml->output();
  }
 }
?>
