<?

 /*****************************************************************************
 * Filename: include/layout.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 if(isset($_config["testing"])&&$_config["testing"])
  require_once($_path["include"]."data.php");

/******************************************************************
 Layout class
 
 functions:
 + layout:         Constuctor
 + doctype:        HTML document type 
 + header:         HTML header
 + body:           HTML body start
 - HTMLvalidator:  Displays HTML validator link
 - CSSvalidator:   Displays validator link
 - img:            IMG tag
 - link:           A HREF tag
 - cms:            A HREF tag for CMS
 + footer:         HTML footer (end of body and html)
******************************************************************/
 
 class Layout
 {


/******************************************************************
 Layout 
 Constructor

 + path: current path 
 - frame: frame (default: main) 

******************************************************************/
  function Layout($frame="main")
  {
   $this->time=explode(" ", microtime());
   $this->frame=$frame;
   $this->doctype="xhtml";
  }


/******************************************************************
 Doctype
 Return HTML document type

 - type: type of document: 401strict (default), frameset 

******************************************************************/
  function doctype($version=false, $mode=false)
  {
   $this->doctype=Array();
   $this->doctype["version"]=strtoupper($version);
   $this->doctype["mode"]=strtolower($mode);
   
   switch($this->doctype["version"])
   {
    case "HTML 4.01":
    {
     switch($this->doctype["mode"])
     {
      case "transitional":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n";
      case "quirks":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
      case "frameset":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\" \"http://www.w3.org/TR/html4/frameset.dtd\">\n";
      default:
       $this->doctype["mode"]="strict";
      case "strict":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n";
     }
    }

    case "XHTML 1.0": 
    {
     switch($this->doctype["mode"])
     {
      case "transitional":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
      case "frameset":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Frameset//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd\">\n";
      default:
       $this->doctype["mode"]="strict";
      case "strict":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
     }
    }

    default: 
     $this->doctype["version"]="XHTML 1.1";
    case "XHTML 1.1": 
    {
     switch($this->doctype["mode"])
     {
      default:
       $this->doctype["mode"]="strict";
      case "strict":
       return "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n";
     }
    }
   }
  }


/******************************************************************
 Header
 Returns HTML header 

 + title: Page title
 - style: extra style sheet (directory) (besides default.css),  autopath=style/ 
   (maybe ; separated for multiple files)
 - java: script functions or .js files
   (maybe ; separated for multiple files)
 - icon: icon file 
 - extra: extra header information

******************************************************************/
  function header($title=false, $style=false, $java=false, $icon=false, $extra=false)
  {
   global $_jses, $_path, $_config;

   if($this->doctype["version"]=="XHTML 1.0"||$this->doctype["version"]=="XHTML 1.1")
    $r="<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n";
   else
    $r="<html>\n";

   $r.=" <head>\n";

   if($title)
    $r.="  <title>".$title."</title>\n\n";
   else if(isset($_config["title"]))
    $r.="  <title>".$_config["title"]."</title>\n\n";

   if(isset($_config["layout"]["description"]))
    $r.="  <meta name=\"description\" content=\"".$_config["layout"]["description"]."\"/>\n";
   if(isset($_config["layout"]["keywords"]))
    $r.="  <meta name=\"keywords\" content=\"".$_config["layout"]["keywords"]."\"/>\n\n";

//META !!!
   $r.="  <meta http-equiv=\"Pragma\" content=\"no-cache\"/>\n";
   $r.="  <meta http-equiv=\"Expires\" content=\"-1\"/>\n";

   $r.="  <meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-1\"/>\n\n";

//style !!!

//default 
   if(file_exists($_path["project"]["style"]."default.css"))
    $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"]."default.css\"/>\n";

   if(file_exists($_path["project"]["style"]."print.css"))
    $r.="  <link rel=\"stylesheet\" type=\"text/css\" media=\"print\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"]."print.css\"/>\n";

//non default style overwrites all previous 
   if($_config["style"]&&$_config["style"]!="default")
    if(file_exists($_path["project"]["style"].$_config["style"].".css"))
     $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$_config["style"].".css\"/>\n";

//browser specific stylesheet overwrite all previous
   if(file_exists($_path["project"]["style"].$_jses["stylesheet"].".css"))
    $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$_jses["stylesheet"].".css\"/>\n";

//frame specific stylesheets (same priority as above
   if(file_exists($_path["project"]["style"].$this->frame."default.css"))
    $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$this->frame."default.css\"/>\n";

   if($_config["style"]&&$_config["style"]!="default")
    if(file_exists($_path["project"]["style"].$this->frame."/".$_config["style"].".css"))
     $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$this->frame."/".$_config["style"].".css\"/>\n";
   
   if(file_exists($_path["project"]["style"].$this->frame."/".$_jses["stylesheet"].".css"))
    $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$this->frame."/".$_jses["stylesheet"].".css\"/>\n";


   if($style)
   {
    preg_match_all("/([a-z0-9\/\-\_]+\.css|[a-z0-9\/\-\_]+)/i", $style, $tmp);

    $tmp=$tmp[1];
    $nr=sizeof($tmp);

    for($c=0;$c<$nr;$c++)
    {
     //specified style sheet
     if(preg_match("/\.css$/i", $tmp[$c])&&file_exists($_path["project"]["style"].$tmp[$c])) 
      $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$tmp[$c]."\"/>\n";
     else //Style sheet directory
     {
      if(file_exists($_path["project"]["style"].$tmp[$c]."/default.css")) //default
       $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$tmp[$c]."/default.css\"/>\n";
      if(file_exists($_path["project"]["style"].$tmp[$c]."/".$_config["style"].".css")&&$_config["style"]!="default")// style overwrite
       $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$tmp[$c]."/".$_config["style"].".css\"/>\n";
      if(file_exists($_path["project"]["style"].$tmp[$c]."/".$_jses["stylesheet"].".css")) // browser overwrite
       $r.="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["style"].$tmp[$c]."/".$_jses["stylesheet"].".css\"/>\n";
     }
    }
   }

   $r.="\n";
 
//javascript !!!
   if(file_exists($_path["project"]["script"]."pngfix.js"))
    $r.="  <!--[if lt IE 7.0]><script type=\"text/javascript\" src=\"".$_path["HTML"]["prefix"].$_path["project"]["script"]."pngfix.js\"></script><![endif]-->\n";

   $r.=" <script type=\"text/javascript\"><!--\n var browser=\"".$_jses["browser"]."\";\n var path=\"".$_path["HTML"]["prefix"].$_path["project"]["root"]."\";\n var rootpath=\"".$_path["HTML"]["prefix"].$_path["root"]."\";\n --></script>\n";

   if(file_exists($_path["project"]["script"]."/default.js"))
    $r.="  <script type=\"text/javascript\" src=\"".$_path["HTML"]["prefix"].$_path["project"]["script"]."default.js\"></script>\n";

   if($java)
   {
    if(preg_match("/\.js/i", $java)) // include ; separated .js files list
    {
     $tmp=explode(";", $java);

     $nr=sizeof($tmp);
     for($c=0;$c<$nr;$c++)
     {
      if(file_exists($_path["project"]["script"]."/".$tmp[$c]))
       $r.="  <script type=\"text/javascript\" src=\"".$_path["HTML"]["prefix"].$_path["project"]["script"].$tmp[$c]."\"></script>\n";
      else if(file_exists($_path["script"]."/".$tmp[$c]))
       $r.="  <script type=\"text/javascript\" src=\"".$_path["HTML"]["prefix"].$_path["script"].$tmp[$c]."\"></script>\n";
     }
    }
    else //or set functions between script tags
    {
     $r.="  <script type=\"text/javascript\"><!--\n\n";
     $r.=$java;
     $r.="  --></script>\n";
    }
   }
   $r.="\n";
  
//ICON !!!
   if(!$icon){$icon="favicon.ico";}
   if(file_exists($_path["project"]["images"].$icon)) 
    $r.="  <link rel=\"shortcut icon\" href=\"".$_path["HTML"]["prefix"].$_path["project"]["images"].$icon."\" />\n";

   if($extra){$r.=$extra;}

   $r.=" </head>\n\n";

   return $r;
  } 


/******************************************************************
 Body
 Returns HTML body start

 - extra: extra options like javascript

******************************************************************/
  function body($extra=false)
  {
   if($extra){$extra=" ".$extra;}

   return " <body".$extra.">\n\n";
  }


/******************************************************************
 HTML validator
 Return image+link 

******************************************************************/
  function HTMLvalidator()
  {
   return $this->link("http://validator.w3.org/check/referer", "<img src=\"".$_path["HTML"]["prefix"].$_path["project"]["images"]."/vhtml.gif\" alt=\"HTML Validator\" class=\"navCertificat\"/>", "W3C HTML Validator", "_blank"); 
  }


/******************************************************************
 CSS validator
 Returns image+link 

******************************************************************/
 function CSSvalidator()
  {
   return $this->link("http://jigsaw.w3.org/css-validator/check/referer", "<img src=\"".$_path["HTML"]["prefix"].$_path["project"]["images"]."/vcss.gif\" alt=\"CSS Validator\" class=\"navCertificat\"/>", "Cascading Style Sheets validator", "_blank"); 
  }

/******************************************************************
 Image
 Returns HTML image tag

 + src:   location of image, autopath = images/
 - alt:   title of image
 - class: css class
 - id:    element id
 - extra: extra options like javascript

******************************************************************/
  function img($src, $alt="", $class=false, $id=false, $extra=false)
  {
   global $_path;

   if($class){$class=" class=\"$class\"";}
   if($id){$id=" id=\"$id\"";}
   if($extra){$extra=" $extra";}

   return "<img src=\"".$_path["HTML"]["prefix"].$_path["project"]["images"]."/".$src."\" alt=\"".$alt."\"".$class.$id.$extra."/>";
  }


/******************************************************************
 Link
 Returns HTML link tag 

 + src: link
 + text: linked text
 - title: title text (if none given $text is used)
 - target: target frame 
 - extra: extra options like javascript
  
******************************************************************/
  function link($src, $text, $title=false, $target=false, $extra=false)
  {
   global $_jses;

   if($title){$title=addslashes($title);}
   else{$title=addslashes($text);}

   if($target) //nasty java script fix for frames, this fuckes up when using ONMOUSEUP in $extra !
   {
 //   $target=" onclick=\"parent.".$target.".location='".$src."';\"";
 //   $src="#";
    $target=" target=\"".$target."\"";
   }
   if($extra){$extra=" ".$extra;}

   return "<a href=\"".$src."\" title=\"".$title."\"".$target.$extra.">".$text."</a>";   
  }

 
/******************************************************************
 Footer
 Return 

 - body: close body (true (default) / false)
 - comments: add copyrights+time comments (true (default) / false)

******************************************************************/
  function footer($body=true, $comments=true)
  { 
   global $_config, $_path;

   $r="\n\n";
   clearstatcache();

   $tmp=explode(" ", microtime());
   $time=floatval($tmp[1]-$this->time[1])+floatval($tmp[0]-$this->time[0]);

   if(isset($_config["layout"]["footer"])&&$_config["layout"]["footer"]&&$this->frame=="main")
   {
    $r.="<div class=\"footer\">\n";

    $r.="<table class=\"cat\">\n";
    $r.="<tr><td class=\"title\">Logic updated:</td><td class=\"text\">".date($_config["datetimeformat"], filemtime($_path["file"]))."</td>\n";
    $r.="<td class=\"title\">Execution time:</td><td class=\"text\">".$time." seconds</td></tr>\n";
    $r.="</table>\n\n";

    $r.="</div>\n\n";
   }

   if($body)
    $r.=" </body>\n\n";

   if($comments)
   {
    $r.=" <!-- Copyright by Verty / JuNe -->\n";
    $r.=" <!-- Generated: ".date("d-m-Y H:m:s")."-->\n\n";
   }

   $r.="</html>\n";

   return $r;
  }

 } 
?>
