<?

 /*****************************************************************************
 * Filename: include/mime.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");

 class Mime
 {
  function Mime()
  {
   global $_path;

   $this->d=Data::cache("mimetypes", "view", "all", Array("db" => "mime"));
   $this->e=&$this->d->e;

   $this->readDB();
  }

  function readDB()
  {
   $this->d->clearValues();

   if(!$this->d->get())
   {
    $this->e->add($this->d->e);
    return false;
   }

   return true;
  }

  function file($filename, $content=false)
  {
   if(!($fileType=$this->filename($filename)))
    return false;

   if($content===false) //read file
   {
    if(!file_exists($filename))
    {
     $this->e->add("File: \"".$filename."\" doesn't exist", "error");
     return false;
    }
    if(!($content=@file_get_contents($filename)))
    {
     $this->e->add("Unable to read contents for filename: \"".$filename."\"", "error");
     return false;
    }
   }

   if(!($contentType=$this->content($content)))
    return false;

   if($fileType!=$contentType)
   {
    $this->e->add("Filename mime type: \"".$fileType."\" doesn't match content mime type: \"".$contentType."\" for filename \"".$filename."\"", "warning");
    return false;
   }

   return $fileType;
  }

  function filename($filename)
  {
   if(!preg_match("/\.([a-z0-9]+)$/i", $filename, $tmp))
   {
    $this->e->add("Unable to get extension from filename: \"".$filename."\"", "error");
    return false;
   }

   $extension=strtolower($tmp[1]);

   $this->d->reset();
   while($this->d->next())
   {
    if(!$this->d->v["extension"])
     continue;

    if($extension==$this->d->v["extension"])
     return $this->d->v["mime"];
   }

   $this->e->add("Unable to detect mime type for filename: \"".$filename."\"", "warning");
   return "application/octet-stream";
  }

  function extension($extension)
  {
   if(!$extension)
   {
    $this->e->add("No extension given", "error");
    return false;
   }

   $extension=strtolower($extension);

   $this->d->reset();
   while($this->d->next())
   {
    if(!$this->d->v["extension"])
     continue;

    if($extension==$this->d->v["extension"])
     return $this->d->v["mime"];
   }

   $this->e->add("Unable to detect mime type for extension: \"".$extension."\"", "warning");
   return "application/octet-stream";
  }

  function content(&$content)
  {
   $this->d->reset();
   while($this->d->next())
   {
    if(!$this->d->v["content"])
     continue;

    if(strpos($content, $this->d->v["content"])===0)
     return $this->d->v["mime"];

    if(preg_match("/^".$this->d->v["content"]."/", $content))
     return $this->d->v["mime"];
   }

   if(preg_match('/[\x00-\x08\x0b-\x0c\x0e\x1f]/', $content))
    return "application/octet-stream";
   else
    return "text/plain";
  }

  function add($mime, $extension, $content)
  {
   $d=Data::cache("mimetypes", "view", "all", Array("db" => "mime"));
   $d->setValue("mime", $mime);
   $d->setValue("extension", strtolower($extension));
   $d->setValue("content", $content);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return false;
   }

   return $this->readDB();
  }
 }


