<?

 /*****************************************************************************
 * Filename: include/modules/categories.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Module_categories extends Module
 {
  function init(&$data)
  {
   Module::init($data);

   $this->data->title=$this->data->l->t("Category");
   $this->data->addField("ordernr", "nr", "all", "order", false, false, 0, "Order number", "Leave: 0 for automatic number");
   $this->data->addField("name", "text", "all", "title", 1, 25, false, "Name", "Name of the category");
   $this->data->addField("text", "text", "all", false, 0, 255, false, "Explanation", "Explanation about it's content");
   $this->data->addField("moduleLock", "boolean", "all", false, false, false, true, "Module lock", "Only allow selected modules in this category");
   $this->data->addField("module", "modules", "all", false, 0, 65536, false, "Module", "Allowed modules in this category");

   $this->data->addField("recurseCategories", "boolean", "", false, false, false, false, "Recurse categories", "Delete all underlying categories");
   $this->data->addField("recurseContent", "boolean", "", false, false, false, false, "Recurse content", "Delete all underlying content");

   $this->data->defaultFields();

   $this->data->addUnique("uniqueInCategory", "catnr", "name");

   $this->data->sortOrder("ordernr", "name");

   //rights manipulation
   $this->data->addActions("rights");
  }

  function postInit()
  {
   if($this->data->action=="rights")
    return $this->initRightsFields();

   return true;
  }

  function addPostSubmit()
  {
   //copy rights from parent category
   $du=Data::cache("userRights", "view", "all");
   if(!($dus=$du->storageObject()))
    return false;

   $dg=Data::cache("groupRights", "view", "all");
   if(!($dgs=$dg->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!$this->data->getValue("catnr", $c)) //skip top level category
     continue;

    //copy user rights
    $du->setAction("view");
    $du->setExtra("all");

    $du->clearValues();
    $du->setValue("catnr", $this->data->getValue("catnr", $c));

    if(!$du->get())
    {
     $this->data->e->add($du->e);
     return false;
    }

    if($du->vmax)
    {
     for($du->vc=0;$du->vc<$du->vmax;$du->vc++)
      $du->setValue("catnr", $this->data->getValue("nr", $c), $du->vc);

     $du->setAction("add");
     $du->setExtra("item");

     //no checking, just add: should be ok
     for($du->vc=0;$du->vc<$du->vmax;$du->vc++)
     {
      if(!$dus->add())
      {
       $this->data->e->add($dus->e);
       return false;
      }
     }
    }

    //copy group rights
    $dg->setAction("view");
    $dg->setExtra("all");

    $dg->clearValues();
    $dg->setValue("catnr", $this->data->getValue("catnr", $c));

    if(!$dg->get())
    {
     $this->data->e->add($dg->e);
     return false;
    }

    if($dg->vmax)
    {
     for($dg->vc=0;$dg->vc<$dg->vmax;$dg->vc++)
      $dg->setValue("catnr", $this->data->getValue("nr", $c), $dg->vc);

     $dg->setAction("add");
     $dg->setExtra("item");

     //no checking, just add: should be ok
     for($dg->vc=0;$dg->vc<$dg->vmax;$dg->vc++)
     {
      if(!$dgs->add())
      {
       $this->data->e->add($dgs->e);
       return false;
      }
     }
    }
   }

   return true;
  }

  function movePostSubmitErrorCheck()
  {
   //additional move checks
   $d=Data::cache("categories", "tree", "down");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!($newcatnr=$this->data->getValue("newcatnr", $c))) //allow per entry and just 1 newcatnr
     $newcatnr=$this->data->getValue("newcatnr");

    if($this->data->getValue("nr", $c)==$newcatnr)
    {
     $this->data->e->add("Unable to move the category to itself", "error", "Entry: ".($c+1));
     return false;
    }

    //check if new category is a sub category of it's own
    $d->clearValues();
    $d->setShowFields(); //don't get any data, only id's
    $d->setValue("catnr", $this->data->getValue("nr", $c));

    if(!$d->get())
    {
     $this->data->e->add("Can't check tree for moving the category", "error", "Entry: ".($c+1));
     $this->data->e->add($d->e);
     return false;
    }

    while($d->next())
    {
     if($d->v["nr"]==$newcatnr)
     {
      $this->data->e->add("Unable to move the category to a subcategory of itself", "error", "Entry: ".($c+1));
      return false;
     }
    }
   }

   return true;
  }

  function deletePostSubmitErrorCheck()
  {
   //check if category has underlying data in case of no recursive delete
   $return=true;
   $d=Data::cache("categories", "view", "list");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //subcategories ?
    if(!$this->data->getValue("recurseCategories")||!$this->data->check("valuetype", "set", "recurseCategories"))
    {
     $d->setExtra("list");

     $d->clearValues();
     $d->setValue("catnr", $this->data->getValue("nr", $c));

     if(!$d->get(false, true))
     {
      $this->data->e->add($d->e);
      return false;
     }

     if($d->getValue("show_count")>0)
     {
      $this->data->e->add("Unable to delete category because it contains subcategories", "error", "Entry: ".($c+1));
      $return=false;
     }
    }

    //content for category referenced modules
    if(!$this->data->getValue("recurseContent")||!$this->data->check("valuetype", "set", "recurseContent"))
    {
     $d->setExtra("item");

     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("nr", $c));
     $d->setShowFields("module");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     foreach($d->getValue("module") as $m)
     {
      $d2=Data::cache($m, "view", "list");
      $d2->setValue("catnr", $this->data->getValue("nr", $c));

      if(!$d2->get(false, true))
      {
       $this->data->e->add($d2->e);
       return false;
      }

      if($d2->getValue("show_count")>0)
      {
       $this->data->e->add("Unable to delete category because it contains ".$d2->title." data", "error", "Entry: ".($c+1));
       $return=false;
      }
     }
    }
   }

   return $return;
  }

  function deletePreSubmit()
  {
   //delete underlying data in case of recursive delete
   $d=Data::cache("categories", "view", "list");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //subcategories ?
    if($this->data->getValue("recurseCategories")&&$this->data->check("valuetype", "set", "recurseCategories"))
    {
     $d->setAction("delete");
     $d->setExtra("list");

     $d->clearValues();
     $d->setValue("catnr", $this->data->getValue("nr", $c));
     $d->setShowFields("nr");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     //delete if any subcategories
     if($d->vmax)
     {
      //set recursive categories/content flag to underlying categories
      $d->setValue("recurseCategories", $this->data->getValue("recurseCategories"));
      $d->setValue("recurseContent", $this->data->getValue("recurseContent"));

      if(!$d->submit())
      {
       $this->data->e->add($d->e);
       return false;
      }
     }
    }

    //content for category referenced modules
    if($this->data->getValue("recurseContent")&&$this->data->check("valuetype", "set", "recurseContent"))
    {
     $d->setAction("view");
     $d->setExtra("item");

     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("nr", $c));
     $d->setShowFields("module");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     foreach($d->getValue("module") as $m)
     {
      $d2=Data::cache($m, "delete", "list");
      $d2->setValue("catnr", $this->data->getValue("nr", $c));
      $d2->setShowFields("nr");

      if(!$d2->get())
      {
       $this->data->e->add($d2->e);
       return false;
      }

      //delete if any content
      if($d2->vmax&&!$d2->submit())
      {
       $this->data->e->add($d2->e);
       return false;
      }
     }
    }
   }

   return true;
  }

  function deletePostSubmit()
  {
   //remove rights for category
   $du=Data::cache("userRights", "delete", "all");
   if(!($dus=$du->storageObject()))
    return false;

   $dg=Data::cache("groupRights", "delete", "all");
   if(!($dgs=$dg->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //delete user rights
    $du->clearValues();
    $du->setValue("catnr", $this->data->getValue("nr", $c));

    $du->vc=0;
    if(!$dus->delete(true))
    {
     $this->data->e->add($dus->e);
     return false;
    }

    //delete group rights
    $dg->clearValues();
    $dg->setValue("catnr", $this->data->getValue("nr", $c));

    $dg->vc=0;
    if(!$dgs->delete(true))
    {
     $this->data->e->add($dgs->e);
     return false;
    }

    //delete rights from group
    $dg->clearValues();
    $dg->setValue("nr", $this->data->getValue("nr", $c));

    $dg->vc=0;
    if(!$dgs->delete(true))
    {
     $this->data->e->add($dgs->e);
     return false;
    }
   }


   //users module usersAdditionalGroups
   $users=Data::cache("users", "view", "item");
   if($users->check("reference", "categories", "usersAdditionalGroups"))
   {
    $da=$users->getReferenceData("usersAdditionalGroups");
    if(!($das=$da->storageObject()))
     return false;

    for($c=0;$c<$this->data->vmax;$c++)
    {
     //delete additional user rights
     $da->clearValues();
     $da->setValue("categories", $this->data->getValue("nr", $c));

     $du->vc=0;
     if(!$das->delete(true))
     {
      $this->data->e->add($das->e);
      return false;
     }
    }
   }

   return true;
  }

  function initRightsFields()
  {
   global $_config;

   $this->data->addField("usernr", "nr", "form", "reload", 0, false, false, "Users list");
   $this->data->addReference("usernr", $_config["usermodule"], "nr", "*:0");
   $this->data->addField("userrights", "rights", "form", "", 0, false, "", "User rights", "Grant the selected rights to selected user");

   $this->data->addField("groupnr", "nr", "form", "reload", 0, false, false, "User groups list");
   $this->data->addReference("groupnr", "categories", "nr", "*:0");
   $this->data->addField("grouprights", "rights", "form", "", 0, false, "", "Group rights", "Grant the selected rights to selected group");

   $this->data->setRequestValue("usernr");
   $this->data->setRequestValue("userrights");
   $this->data->setRequestValue("groupnr");
   $this->data->setRequestValue("grouprights");

   $du=Data::cache("userRights", "view", "all");
   $dg=Data::cache("groupRights", "view", "all");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($this->data->getValue("reload_trigger")=="usernr") //user changed
    {
     if($this->data->getValue("usernr", $c))
     {
      $du->clearValues();
      $du->setValue("catnr", $this->data->getValue("nr", $c));
      $du->setValue("nr", $this->data->getValue("usernr", $c));
      $du->get();

      if($du->next())
       $this->data->setValue("userrights", $du->v["rights"], $c);
      else
       $this->data->setValue("userrights", $this->data->rights2bin(0), $c);
     }
     else
      $this->data->setValue("userrights", $this->data->rights2bin(0), $c);
    }
    else if($this->data->getValue("reload_trigger")=="groupnr") //group changed
    {
     if($this->data->getValue("groupnr", $c))
     {
      $dg->clearValues();
      $dg->setValue("catnr", $this->data->getValue("nr", $c));
      $dg->setValue("nr", $this->data->getValue("groupnr", $c));
      $dg->get();

      if($dg->next())
       $this->data->setValue("grouprights", $dg->v["rights"], $c);
      else
       $this->data->setValue("grouprights", $this->data->rights2bin(0), $c);
     }
     else
      $this->data->setValue("grouprights", $this->data->rights2bin(0), $c);
    }
   }

   return true;
  }

  function rightsPostSubmitErrorCheck()
  {
   global $_config;

   $this->data->checkField("usernr");
   $this->data->checkField("userrights");
   $this->data->checkField("groupnr");
   $this->data->checkField("grouprights");

   $d=$this->data->copy();
   $d->setShowFields("module");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($this->data->getValue("usernr", $c)==$_config["rootnr"])
     $this->data->e->add("Can't change root rights", "error", "Entry: ".($c+1));

    if($this->data->getValue("groupnr", $c)&&$this->data->sqlValue("grouprights", $c))
    {
     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("groupnr", $c));
     $d->get();

     $usercat=false;
     if($d->next())
     {
      if(is_array($d->v["module"])&&in_array($_config["usermodule"], $d->v["module"]))
       $usercat=true;
      else if(!is_array($d->v["module"])&&$_config["usermodule"]==$d->v["module"])
       $usercat=true;
     }

//     if(!$usercat)
//      $this->data->e->add("Can't change rights for non user category", "error", "Entry: ".($c+1));
    }
   }

   if($this->data->e->any("fatal"))
    return false;

   return true;
  }

  function rightsPostSubmit()
  {
   $du=Data::cache("userRights", "view", "all");
   if(!($dus=$du->storageObject()))
    return false;

   $dg=Data::cache("groupRights", "view", "all");
   if(!($dgs=$dg->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($this->data->getValue("usernr")) //user changed
    {
     $du->setAction("view");
     $du->setExtra("all");

     $du->clearValues();
     $du->setValue("catnr", $this->data->getValue("nr", $c));
     $du->setValue("nr", $this->data->getValue("usernr", $c));

     $du->get();

     if($du->next()) //already in database: update/delete/nothing
     {
      if(!$this->data->sqlValue("userrights", $c)) //delete
      {
       $du->setAction("delete");
       $du->setExtra("item");

       $du->clearValues();
       $du->setValue("catnr", $this->data->getValue("nr", $c));
       $du->setValue("nr", $this->data->getValue("usernr", $c));

       $du->vc=0;
       if(!$dus->delete(true))
       {
        $this->data->e->add($dus->e);
        return false;
       }

       $this->data->e->add("User rights revoked", "note");
      }
      else if($this->data->getValue("userrights", $c)!=$du->v["rights"]) //update
      {
       $du->setAction("edit");
       $du->setExtra("item");

       $du->clearValues();
       $du->setValue("catnr", $this->data->getValue("nr", $c));
       $du->setValue("nr", $this->data->getValue("usernr", $c));
       $du->setValue("rights", $this->data->getValue("userrights", $c));
       $du->field["rights"]["valuetype"]="request";

       $du->vc=0;
       if(!$dus->edit(true))
       {
        $this->data->e->add($dus->e);
        return false;
       }

       $this->data->e->add("User rights granted", "note");
      }
     }
     else //not in database: add/nothing
     {
      if($this->data->sqlValue("userrights", $c)) //add
      {
       $du->setAction("add");
       $du->setExtra("item");

       $du->clearValues();
       $du->setValue("catnr", $this->data->getValue("nr", $c));
       $du->setValue("nr", $this->data->getValue("usernr", $c));
       $du->setValue("rights", $this->data->getValue("userrights", $c));

       $du->vc=0;
       if(!$dus->add())
       {
        $this->data->e->add($dus->e);
        return false;
       }

       $this->data->e->add("User rights granted", "note");
      }
     }
    }

    if($this->data->getValue("groupnr")) //group changed
    {
     $dg->setAction("view");
     $dg->setExtra("all");

     $dg->clearValues();
     $dg->setValue("catnr", $this->data->getValue("nr", $c));
     $dg->setValue("nr", $this->data->getValue("groupnr", $c));
     $dg->get();

     if($dg->next()) //already in database: update/delete/nothing
     {
      if(!$this->data->sqlValue("grouprights", $c)) //delete
      {
       $dg->setAction("delete");
       $dg->setExtra("item");

       $dg->clearValues();
       $dg->setValue("catnr", $this->data->getValue("nr", $c));
       $dg->setValue("nr", $this->data->getValue("groupnr", $c));

       $dg->vc=0;
       if(!$dgs->delete(true))
       {
        $this->data->e->add($dgs->e);
        return false;
       }

       $this->data->e->add("Group rights revoked", "note");
      }
      else if($this->data->getValue("grouprights", $c)!=$dg->v["rights"]) //update
      {
       $dg->setAction("edit");
       $dg->setExtra("item");

       $dg->clearValues();
       $dg->setValue("catnr", $this->data->getValue("nr", $c));
       $dg->setValue("nr", $this->data->getValue("groupnr", $c));
       $dg->setValue("rights", $this->data->getValue("grouprights", $c));
       $dg->field["rights"]["valuetype"]="request";

       $dg->vc=0;
       if(!$dgs->edit(true))
       {
        $this->data->e->add($dgs->e);
        return false;
       }

       $this->data->e->add("Group rights granted", "note");
      }
     }
     else //not in database: add/nothing
     {
      if($this->data->sqlValue("grouprights", $c)) //add
      {
       $dg->setAction("add");
       $dg->setExtra("item");

       $dg->clearValues();
       $dg->setValue("catnr", $this->data->getValue("nr", $c));
       $dg->setValue("nr", $this->data->getValue("groupnr", $c));
       $dg->setValue("rights", $this->data->getValue("grouprights", $c));

       $dg->vc=0;
       if(!$dgs->add())
       {
        $this->data->e->add($dgs->e);
        return false;
       }

       $this->data->e->add("Group rights granted", "note");
      }
     }
    }
   }

   return true;
  }

 }
?>
