<?

 /*****************************************************************************
 * Filename: include/modules/configAPIs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Module_configAPIs extends Module
 {
  function init(&$data)
  {
   global $_path;

   Module::init($data);

   $this->data->title=$this->data->l->t("API's configuration");

   $this->data->addField("nr", "nr", "sql;form", "identifier;hidden");
   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all
   $this->data->addField("faketitle", "text", "show", "title;hidden", 0, false, $this->data->title, "Title");

   $this->types=Array(
    "ViewItem" => "View item", 
    "ViewList" => "View list", 
    "ViewAll" => "View all", 
    "TreeUp" => "Tree up", 
    "TreeDown" => "Tree down", 
    "Search" => "Search", 
    "Add" => "Add",
    "Edit" => "Edit",
    "Move" => "Move",
    "Delete" => "Delete",
    "DatabaseInfo" => "Database info", 
   );

   $this->apis=Array(
    "xml" => "XML",
    "data" => "Data",
    "export" => "Export",
   );

   foreach($this->apis as $api => $title)
   {
    $group=Array();
    $this->data->addField($api, "boolean", "all", "reload", false, false, false, $title." API", "Enable the ".$title." API");

    if($api=="export")
    {
     $this->data->addField("exportMethods", "multilist", "all", false, 0, 255, Array("CSV"), "Export methods", "Which methods are allowed to be exported");
     $this->data->listOptions("exportMethods", "CSV");

     $group[]="exportMethods";
    }

    foreach($this->types as $type => $text)
    {
     if($api=="data") //data api
     {
      if($type=="Search"||$type=="DatabaseInfo") //skip search and database info
       continue;
     }
     else if($api=="export") //view only api
     {
      if($type=="Search") //stop at search
       break;
     }

     $this->data->addField($api.$type, "boolean", "all", "reload", false, false, false, $text." support", "Allow ".strtolower($text)." support for ".$title." API");
     $group[]=$api.$type;
    }

    if($api=="xml")
    {
     $this->data->addField("xmlUserLoginLogout", "boolean", "all", "", false, false, false, "Login/Logout support", "Allow user login and logout support for ".$title." API");
     $this->data->addField("xmlUserPassword", "boolean", "all", "", false, false, false, "Password support", "Allow user password change support for ".$title." API");
     $this->data->addField("xmlModules", "boolean", "all", "", false, false, false, "Modules support", "Allow modules listing for ".$title." API");
     $this->data->addField("xmlProjects", "boolean", "all", "", false, false, false, "Project support", "Allow projects listing for ".$title." API");
     $this->data->addField("xmlLanguage", "boolean", "all", "", false, false, false, "Language support", "Allow language support for ".$title." API");

     $group[]="xmlUserLoginLogout";
     $group[]="xmlUserPassword";
     $group[]="xmlModules";
     $group[]="xmlProjects";
     $group[]="xmlLanguage";
    }

    $this->data->group($title." API", $group);
   }


   $group=Array();
   foreach($this->types as $type => $text)
   {
    $this->data->addField($type."Modules", "modules", "all", "", 0, 65536, false, $text." modules", "Select the modules for which you would like ".strtolower($text)." support");
    $this->data->addField($type."AllModules", "boolean", "all", "reload", false, false, false, $text." for all modules", "Allow ".strtolower($text)." support for all modules");

    $group[]=$type."Modules";
    $group[]=$type."AllModules";
   }
   $this->data->group("Modules", $group);
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   foreach($this->apis as $api => $title)
   {
    if($this->data->getValue($api))
    {
     foreach(array_keys($this->types) as $type)
     {
      if($api=="data") //data api
      {
       if($type=="Search"||$type=="DatabaseInfo") //skip search and database info
        continue;
      }
      else if($api=="export") //view only api
      {
       if($type=="Search") //stop at search
        break;
      }

      $this->data->resetField($api.$type);
     }

     if($api=="xml")
     {
      $this->data->resetField("xmlSearch");
      $this->data->resetField("xmlUserLoginLogout");
      $this->data->resetField("xmlUserPassword");
      $this->data->resetField("xmlModules");
      $this->data->resetField("xmlProjects");
      $this->data->resetField("xmlLanguage");
     }
     else if($api=="export")
      $this->data->resetField("exportMethods");
    }
    else //XML disabled
    {
     foreach(array_keys($this->types) as $type)
     {
      if($api=="data") //data api
      {
       if($type=="Search"||$type=="DatabaseInfo") //skip search and database info
        continue;
      }
      else if($api=="export") //view only api
      {
       if($type=="Search") //stop at search
        break;
      }

      $this->data->field[$api.$type]["properties"]=";hidden";
     }

     if($api=="xml")
     {
      $this->data->field["xmlSearch"]["properties"]=";hidden";
      $this->data->field["xmlUserLoginLogout"]["properties"]=";hidden";
      $this->data->field["xmlUserPassword"]["properties"]=";hidden";
      $this->data->field["xmlModules"]["properties"]=";hidden";
      $this->data->field["xmlProjects"]["properties"]=";hidden";
      $this->data->field["xmlLanguage"]["properties"]=";hidden";
     }
     else if($api=="export")
      $this->data->field["exportMethods"]["properties"]=";hidden";
    }
   }

   //Only show module selection for types that are actually active 
   foreach(array_keys($this->types) as $type)
   {
    $found=false;
    foreach($this->apis as $api => $title)
    {
     if($this->data->getValue($api))
     {
      if($api=="data") //data api
      {
       if($type=="Search"||$type=="DatabaseInfo") //skip search and database info
        continue;
      }
      else if($api=="export") //view only api
      {
       if($type=="Search") //stop at search
        break;
      }
 
      if($this->data->getValue($api.$type))
      {
       $found=true;
       break;
      }
     }
    }

    if($found)
    {
     $this->data->resetField($type."AllModules");

     if($this->data->getValue($type."AllModules"))
      $this->data->field[$type."Modules"]["properties"]=";hidden";
     else
      $this->data->resetField($type."Modules");
    }
    else
    {
     $this->data->field[$type."Modules"]["properties"]=";hidden";
     $this->data->field[$type."AllModules"]["properties"]=";hidden";
    }
   }

   return true;
  }

  function viewPostGet()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    $this->data->setValue("faketitle", $this->data->title, $c);

   return true;
  }

 }

?>
