<?

 /*****************************************************************************
 * Filename: include/modules/configCache.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Module_configCache extends Module
 {
  function init(&$data)
  {
   global $_path;

   Module::init($data);

   $this->memoryMethods=Array();
   if(class_exists("Memcache"))
    $this->memoryMethods[]="Memcache";
   if(class_exists("Memcached"))
    $this->memoryMethods[]="Memcached";


   $this->data->title=$this->data->l->t("Cache configuration");

   $this->data->addField("nr", "nr", "sql;form", "identifier;hidden");
   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all
   $this->data->addField("faketitle", "text", "show", "title;hidden", 0, false, $this->data->title, "Title");

   $this->data->addField("module", "boolean", "all", "reload", false, false, true, "Allow database caching", "Uses cache module to store cache");
   $this->data->addField("moduleCleanup", "nr", "all", "", 0, false, 86400, "Clean up module cache", "Amount of seconds the cache shouldn't be accessed before it's removed");
   $this->data->listOptions("moduleCleanup", Array(0 => "Never", 3600 => "Hourly", 86400 => "Daily", 86400*7 => "Weekly"));

   $this->data->addField("file", "boolean", "all", "reload", false, false, true, "Allow file caching", "Uses file system to store cache");
   $this->data->addField("filePublic", "boolean", "all", false, false, false, false, "Public cache directory", "Allows redirecting to cache entries");
   $this->data->addField("fileCleanup", "nr", "all", "", 0, false, 86400, "Clean up file cache", "Amount of seconds the cache shouldn't be accessed before it's removed");
   $this->data->listOptions("fileCleanup", Array(0 => "Never", 3600 => "Hourly", 86400 => "Daily", 86400*7 => "Weekly"));

   if($this->memoryMethods)
   {
    $this->data->addField("memory", "boolean", "all", "reload", false, false, true, "Allow memory caching", "Uses persistent memory system to store cache");
    $this->data->addField("memoryCacheData", "boolean", "all", "", false, false, true, "Data objects memory caching", "Uses persistent memory system to store data objects");

    $this->data->addField("memoryMethod", "enum", "all", "reload", false, false, $this->memoryMethods[0], "Memory caching method", "Which memory caching method to use");
    $this->data->enumOptions("memoryMethod", $this->memoryMethods);

    $this->data->addField("memoryConnection", "text", "all", false, 3, 255, "/var/run/memcached/memcached.sock", "Memcache(d) connection string", "Either socket file location (/var/run/memcached/memcached.sock) or a connection string (localhost:11211)");
    $this->data->addField("memoryCleanup", "nr", "all", "", 0, 2592000, 86400, "Clean up memory cache", "Amount of seconds the cache shouldn't be accessed before it's removed");
    $this->data->listOptions("memoryCleanup", Array(0 => "Never", 3600 => "Hourly", 86400 => "Daily", 86400*7 => "Weekly"));
   }

   $this->data->group("Module cache", "module", "moduleCleanup");
   $this->data->group("File cache", "file", "filePublic", "fileCleanup");

   if($this->memoryMethods)
    $this->data->group("Memory cache", "memory", "memoryCacheData", "memoryMethod", "memoryConnection", "memoryCleanup");
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->getValue("module"))
    $this->data->resetField("moduleCleanup");
   else
    $this->data->field["moduleCleanup"]["properties"]=";hidden";

   if($this->data->getValue("file"))
   {
    $this->data->resetField("filePublic");
    $this->data->resetField("fileCleanup");
   }
   else
   {
    $this->data->field["filePublic"]["properties"]=";hidden";
    $this->data->field["fileCleanup"]["properties"]=";hidden";
   }

   if($this->memoryMethods)
   {
    if($this->data->getValue("memory"))
    {
     $this->data->resetField("memoryCacheData");
     $this->data->resetField("memoryMethod");
     $this->data->resetField("memoryConnection");
     $this->data->resetField("memoryCleanup");
    }
    else
    {
     $this->data->field["memoryCacheData"]["properties"]=";hidden";
     $this->data->field["memoryMethod"]["properties"]=";hidden";
     $this->data->field["memoryConnection"]["properties"]=";hidden";
     $this->data->field["memoryCleanup"]["properties"]=";hidden";
    }
   }

   return true;
  }

  function viewPostGet()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    $this->data->setValue("faketitle", $this->data->title, $c);

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->checkFields();}
  function copyPreSubmitErrorCheck(){return $this->checkFields();}
  function editPreSubmitErrorCheck(){return $this->checkFields();}

  function checkFields()
  {
   if($this->data->getValue("module"))
   {
    $d=Data::cache("cache", "view", "item", Array("format" => "sql")); //use SQL data format: no convertion

    if($d->e->any("fatal"))
    {
     $this->data->e->add($d->e);
     return false;
    }
   }

   if($this->data->getValue("file"))
   {
    //cache dir created in Cache constructor
   }

   if($this->memoryMethods)
   {
    if($this->data->getValue("memory"))
    {
     if(preg_match("/^([a-z0-9_\-\.]+):(\d+)$/i", $this->data->getValue("memoryConnection"), $tmp)) //domain:port
     {
      $server=$tmp[1];
      $port=$tmp[2];
     }
     else //socket filename
     {
      $server=$this->data->getValue("memoryConnection");
      $port=0;

      if(!file_exists($server))
      {
       $this->data->e->add("Local Memcached socket file: \"".$server."\" doesn't exist", "error");
       return false;
      }
     }

     if($this->data->getValue("memoryMethod")=="Memcache")
     {
      $memcache=new Memcache();
      if(!$port) //local server
      {
       if(!@$memcache->connect("unix://".$server, $port))
       {
        $this->data->e->add("Unable to connect to local Memcached server via: \"".$server."\"", "error");
        return false;
       }
      }
      else
      {
       if(!@$memcache->connect($server, $port))
       {
        $this->data->e->add("Unable to connect to Memcached server: \"".$server."\" on port: ".$port, "error");
        return false;
       }
      }

      $memcache->close();
     }
     else if($this->data->getValue("memoryMethod")=="Memcached")
     {
      $memcache=new Memcached(); //not persistent
      if(!$memcache->addServer($server, $port)||!$memcache->set("configurCacheTest", 1))
      {
       if(!$port) //local server
        $this->data->e->add("Unable to connect to local Memcached server via: \"".$server."\"", "error");
       else
        $this->data->e->add("Unable to connect to Memcached server: \"".$server."\" on port: ".$port, "error");

       return false;
      }

      $memcache->delete("configurCacheTest", 1);
      $memcache->resetServerList();
     }
    }
   }

   return true;
  }
 }

?>
