<?

 /*****************************************************************************
 * Filename: include/modules/configCaptcha.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Module_configCaptcha extends Module
 {
  function init(&$data)
  {
   global $_path;

   Module::init($data);

   $this->data->title=$this->data->l->t("Captcha configuration");

   $this->data->addField("nr", "nr", "sql;form", "identifier;hidden");
   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all
   $this->data->addField("faketitle", "text", "show", "title;hidden", 0, false, $this->data->title, "Title");

   $this->data->addField("imageFormat", "enum", "all", "reload", 3, 3, "PNG", "Image format");
   $this->data->enumOptions("imageFormat", "PNG", "JPG", "GIF");

   $this->data->addField("length", "nr", "all", "", 1, 20, 5, "Captcha length");
   $this->data->addField("fontSize", "nr", "all", "", 1, 100, 14, "Font size");

   $this->data->addField("imageWidth", "nr", "all", "", 20, 2000, 100, "Image width");
   $this->data->addField("imageHeight", "nr", "all", "", 20, 2000, 25, "Image width");

   $this->data->addField("lines", "nr", "all", "", 0, 200, 5, "Number of lines");
 
   $colors=Array(
    "255,255,255" => "White", 
    "64,64,64" => "Dark gray",
    "128,128,128" => "Gray",
    "192,192,192" => "Light gray",
    "0,0,0" => "Black",
    "" => "",
    "255,0,0" => "Red",
    "255,255,0" => "Yellow",
    "0,255,0" => "Green",
    "0,255,255" => "Cyan",
    "0,0,255" => "Blue",
    "255,0,255" => "Purple",
   );

   $this->data->addField("fontColor", "regexp", "all", "", 5, 11, "0,0,0", "Font color", "Red,Green,Blue");
   $this->data->listOptions("fontColor", $colors);
   $this->data->regexps("fontColor", "!^(\d{1,3}),(\d{1,3}),(\d{1,3})$!");

   $this->data->addField("linesColor", "regexp", "all", "", 5, 11, "192,192,192", "Lines color", "Red,Green,Blue");
   $this->data->listOptions("linesColor", $colors);
   $this->data->regexps("linesColor", "!^(\d{1,3}),(\d{1,3}),(\d{1,3})$!");

   $this->data->addField("backgroundTransparent", "boolean", "all", "reload", false, false, true, "Transparent background");
   $this->data->addField("backgroundColor", "regexp", "all", "", 5, 11, "255,255,255", "Background color", "Red,Green,Blue");
   $this->data->listOptions("backgroundColor", $colors);
   $this->data->regexps("backgroundColor", "!^(\d{1,3}),(\d{1,3}),(\d{1,3})$!");

   $this->data->group("Colors", "fontColor", "linesColor", "backgroundTransparent", "backgroundColor");
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->getValue("imageFormat")=="PNG"||$this->data->getValue("imageFormat")=="GIF")
   {
    $this->data->resetField("backgroundTransparent");

    if($this->data->getValue("backgroundTransparent"))
     $this->data->field["backgroundColor"]["properties"]=";hidden";
    else
     $this->data->resetField("backgroundColor");
   }
   else //JPG
   {
    for($c=0;$c<$this->data->vmax;$c++)
     $this->data->setValue("backgroundTransparent", false, $c);

    $this->data->field["backgroundTransparent"]["properties"]=";hidden";
    $this->data->resetField("backgroundColor");
   }

   return true;
  }

  function copyPreSubmit(){return $this->checkColors();}
  function editPreSubmit(){return $this->checkColors();}
  function movePreSubmit(){return $this->checkColors();}
  function deletePreSubmit(){return $this->checkColors();}

  function checkColors()
  {
   $failure=false;
   foreach(Array("fontColor", "linesColor", "backgroundColor") as $field)
   {
    for($c=0;$c<$this->data->vmax;$c++)
    {
     foreach(explode(",", $this->data->getValue($field, $c)) as $color)
     {
      if($color>255)
      {
       $this->data->e->add($this->data->field[$field]["title"]." entry can't be larger then 255", "error", "Entry: ".($this->data->vc+1)); 
       $failure=true;
      }
     }
    }
   }

   if($failure)
    return false;

   return true;
  }


  function viewPostGet()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    $this->data->setValue("faketitle", $this->data->title, $c);

   return true;
  }

 }

?>
