<?

 /*****************************************************************************
 * Filename: include/modules/configFields.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Module_configFields extends Module
 {
  function init(&$data)
  {
   Module::init($data);

   global $_config;

   $this->data->l->read("module_config");
   $this->data->l->read("module_configModules");

   $this->data->title=$this->data->l->t("Module fields configuration");

   $this->data->addField("nr", "nr", "sql;form", "identifier;hidden");
   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all

   $this->data->addField("module", "module", "all", "title;reload", 1, 25, "", "Module", "Which module this configuration applies to");
   $this->data->addField("field", "list", "all", "title;reload", 1, 25, "", "Field", "Which field this configuration applies to");

   if(function_exists("timezone_identifiers_list")) //no timezone support for PHP < 5.1.0 
   {
    $this->data->addField("timezone", "list", "all", false, 0, 25, "", "Display timezone", "Leave empty for global/module config value");
    $this->data->listOptions("timezone", array_merge(Array(""), timezone_identifiers_list()));
    $this->data->addField("formtimezone", "list", "all", false, 0, 25, "", "Input timezone", "Leave empty for global/module config value");
    $this->data->listOptions("formtimezone", array_merge(Array(""), timezone_identifiers_list()));
   }

   $this->data->addField("dateformat", "text", "all", false, 0, 10, "", "Display date format", "Leave empty for global/module config value");
   $this->data->addField("formdateformat", "text", "all", false, 0, 10, "", "Input date format", "Leave empty for global/module config value");
   $this->data->addField("timeformat", "text", "all", false, 0, 10, "", "Display time format", "Leave empty for global/module config value");
   $this->data->addField("formtimeformat", "text", "all", false, 0, 10, "", "Input time format", "Leave empty for global/module config value");
   $this->data->addField("datetimeformat", "text", "all", false, 0, 25, "", "Display date and time format", "Leave empty for global/module config value");
   $this->data->addField("formdatetimeformat", "text", "all", false, 0, 25, "", "Input date and time format", "Leave empty for global/module config value");

   $this->data->addField("decimals", "nr", "all", false, -1, 100, -1, "Display number of decimals", "Set to -1 for global/module config value");
   $this->data->addField("formdecimals", "nr", "all", false, -1, 100, -1, "Input number of decimals", "Set to -1 for global/module config value");
   $this->data->addField("decimalseparator", "text", "all", false, 0, 1, "", "Display decimal separation character", "Leave empty for global/module config value");
   $this->data->addField("formdecimalseparator", "text", "all", false, 0, 1, "", "Input decimal separation character", "Leave empty for global/module config value");
   $this->data->addField("thousandsseparator", "text", "all", false, 0, 1, "", "Display thousands separation character", "Leave empty for global/module config value");
   $this->data->addField("formthousandsseparator", "text", "all", false, 0, 1, "", "Input thousands separation character", "Leave empty for global/module config value");

   $this->data->group("Display formatting", "timezone", "dateformat", "timeformat", "datetimeformat", "decimals", "decimalseparator", "thousandsseparator");
   $this->data->group("Input formatting", "formtimezone", "formdateformat", "formtimeformat", "formdatetimeformat", "formdecimals", "formdecimalseparator", "formthousandsseparator");
  }

  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   $showableFields=Array("timezone", "dateformat", "timeformat", "datetimeformat", "decimals", "decimalseparator", "thousandsseparator");

   if(($module=$this->data->getValue("module"))&&$this->data->checkField("module"))
   {
    $this->data->resetField("field"); 

    //get fields
    $fields=Array("");
    $d=new Data($module);
    foreach(array_keys($d->field) as $field)
    {
     if(
      !$d->check("properties", "control", $field)&&
      ($d->check("use", "show", $field)||$d->check("use", "form", $field))&&
      (
       $d->check("type", "float", $field)||
       $d->check("type", "price", $field)||
       $d->check("type", "date", $field)||
       $d->check("type", "time", $field)||
       $d->check("type", "datetime", $field)
      )
     )
     {
      $fields[]=$field;
     }
    }
    $this->data->listOptions("field", $fields);

    //field set
    if(($field=$this->data->getValue("field"))&&isset($d->field[$field]))
    {
     $type=$d->field[$field]["type"];

     if($type=="date"||$type=="time"||$type=="datetime")
      $formats=Array("timezone", "dateformat", "timeformat", "datetimeformat");
     else if($type=="float"||$type=="price")
      $formats=Array("decimals", "decimalseparator", "thousandsseparator");

     foreach($showableFields as $showableField)
     {
      if(!in_array($showableField, $formats))
      {
       $this->data->field[$showableField]["properties"].=";hidden";
       $this->data->field["form".$showableField]["properties"].=";hidden";
      }
      else
      {
       $this->data->resetField($showableField);
       $this->data->resetField("form".$showableField);
      }
     }
    }
    else //field not set yet
    {
     foreach($showableFields as $showableField)
     {
      $this->data->field[$showableField]["properties"].=";hidden";
      $this->data->field["form".$showableField]["properties"].=";hidden";
     }
    }
   }
   else
   {
    $this->data->field["field"]["properties"].=";hidden";

    foreach($showableFields as $showableField)
    {
     $this->data->field[$showableField]["properties"].=";hidden";
     $this->data->field["form".$showableField]["properties"].=";hidden";
    }
   }

   return true;
  }

 }

?>
