<?

 /*****************************************************************************
 * Filename: include/modules/onlinePlugins.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."module.php");
 require_once($_path["include"]."fs.php");

 class Module_onlinePlugins extends Module
 {
  function init(&$data)
  {
   Module::init($data);

   $this->data->title=$this->data->l->t("Online plugins");

   $this->data->addField("plugin", "text", "sql;show", "", 1, 25, false, "Plugin name");
   $this->data->addField("title", "text", "sql;show", "title", 1, 25, false, "Title");
   $this->data->addField("description", "text", "sql;show", "", 0, 65535, false, "Description");

   $this->data->addField("version", "version", "sql;show", "title", 0, false, false, "Version");
   $this->data->addField("status", "enum", "sql;show", false, false, false, "", "Status");
   $this->data->enumOptions("status", "", "alpha", "beta", "stable", "deprecated");
   $this->data->addField("status2", "text", "show", false, false, false, "", "Status");

   $this->data->addField("author", "text", "sql;show", false, 0, 25, false, "Author");
   $this->data->addField("email", "email", "sql;show", false, 0, 25, false, "Email address");

   $this->data->addField("changelog", "text", "sql;show", "", 0, 65535, false, "Changelog");
   $this->data->addField("dependencies", "text", "sql;show", "", 0, 65535, false, "Dependencies");

   $this->data->addField("file", "file", "all", "", 100, 1024*1024*10, false, "File");

   $this->data->defaultFields();

   $this->data->addUnique("pluginVersion", "plugin", "version");
  }


  function viewPreGet()
  {
   if(!$this->data->check("use", "show", "status")&&$this->data->check("use", "show", "status2")) //get status if status2 is show field
    $this->data->field["status"]["use"].=";show";

   return true;
  }

  function viewPostGet()
  {
   if($this->data->check("use", "show", "status2"))
   {
    $statusConvert=Array("alpha" => "&alpha;", "beta" => "&beta;", "stable" => "&gamma;", "deprecated" => "&delta;");

    for($c=0;$c<$this->data->vmax;$c++)
     if(isset($statusConvert[$this->data->getValue("status", $c)]))
      $this->data->setValue("status2", $statusConvert[$this->data->getValue("status", $c)], $c);
   }

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->readFile();}
  function editPreSubmitErrorCheck(){return $this->readFile();}

  function readFile()
  {
   global $_path;

   if(!$this->data->vmax)
    return true;

   $fs=new FS();
   $zip=new Archive();
   $tmppath=$_path["project"]["plugins"].".tmp/";

   //create tmp folder
   if(!$fs->mkdir($tmppath))
   {
    $this->data->e->add($fs->e);
    return false;
   }

   //get plugin information for all uploaded plugin files
   for($c=0;$c<$this->data->vmax;$c++)
   {
    //check plugin filename
    if(!preg_match("/^([a-z0-9\_]{1,25})-(\d+\.\d+(\.\d+)?)/i", $this->data->getValue("file", $c), $m))
    {
     $fs->delete($tmppath, true);
     $this->data->e->add("Unable to detect a valid plugin name from file: \"".$this->data->getValue("file", $c)."\"", "error");
     return false;
    }

    $plugin=$m[1];
    $version=$m[2];
    unset($m);

    //write plugin archive to tmp folder
    if(!$fs->write($tmppath.$this->data->getValue("file", $c), $this->data->getValue("fileData", $c)))
    {
     $fs->delete($tmppath, true);
     $this->data->e->add($fs->e);
     return false;
    }

    //open plugin archive
    if(!$zip->open($tmppath.$this->data->getValue("file", $c)))
    {
     $fs->delete($tmppath, true);
     $this->data->e->add($zip->e);
     return false;
    }

    //get pluginInfo file from plugin archive
    $pluginInfo=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!pluginInfo\.xml$!", $file)&&!preg_match("!language/[a-z]+/modules/pluginInfo.xml$!i", $file))
     {
      $pluginInfo=$file;
      break;
     }
    }

    if(!$pluginInfo)
    {
     $fs->delete($tmppath, true);
     $this->data->e->add("Plugin info XML not found", "error");
     return false;
    }

    //reroot archive so pluginInfo is in the root
    $pluginRoot=preg_replace("!pluginInfo\.xml$!", "", $pluginInfo);

    if(!$zip->rerootFiles($pluginRoot))
    {
     $fs->delete($tmppath, true);
     $this->data->e->add($zip->e);
     return false;
    }

    //save pluginInfo file to tmp
    if(!$zip->saveFile("pluginInfo.xml", $tmppath, true))
    {
     $fs->delete($tmppath, true);
     $this->data->e->add($zip->e);
     return false;
    }

    //get pluginDependencies file from plugin archive
    $pluginDependencies=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!pluginDependencies.xml$!", $file))
     {
      $pluginDependencies=$file;
      break;
     }
    }

    if($pluginDependencies)
    {
     //save pluginDependencies file to tmp
     if(!$zip->saveFile("pluginDependencies.xml", $tmppath, true))
     {
      $fs->delete($tmppath, true);
      $this->data->e->add($zip->e);
      return false;
     }
    }

    //get changelog file from plugin archive
    $changelog=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!changelog\.txt$!", $file))
     {
      $changelog=$file;
      break;
     }
    }

    if($changelog)
    {
     //reroot archive so changelog is in the root
     if($pluginRoot=preg_replace("!changelog\.txt$!", "", $changelog))
     {
      if(!$zip->rerootFiles($pluginRoot))
      {
       $fs->delete($tmppath, true);
       $this->data->e->add($zip->e);
       return false;
      }
     }

     //save pluginInfo file to tmp
     if(!$zip->saveFile("changelog.txt", $tmppath, true))
     {
      $fs->delete($tmppath, true);
      $this->data->e->add($zip->e);
      return false;
     }
    }

    //get plugin information
    $di=Data::cache("pluginInfo", "view", "all", Array("db" => "xml://".$tmppath."pluginInfo.xml"));

    if(!$di->get())
    {
     $fs->delete($tmppath, true);
     $this->data->e->add($di->e);
     return false;
    }

    $di->next();

    //check plugin version of pluginInfo and archive
    if($di->v["version"]!=$version)
    { 
     $fs->delete($tmppath, true);
     $this->data->e->add("Plugin \"".$plugin."\" info version: \"".$di->v["version"]."\" doesn't match archive version: \"".$version."\"");
     return false;
    }

    $dependencies="";
    if($pluginDependencies)
    {
     //get plugin dependencies
     $dd=Data::cache("pluginDependencies", "view", "all", Array("db" => "xml://".$tmppath."pluginDependencies.xml"));

     if(!$dd->get())
     {
      $fs->delete($tmppath, true);
      $this->data->e->add($dd->e);
      return false;
     }

     while($dd->next())
      $dependencies.=$dd->v["plugin"]."-".$dd->v["version"].($dd->v["required"]?" - required":"")."\n";
    }


    $this->data->setValue("plugin", $plugin, $c);
    $this->data->setValue("title", $di->v["title"], $c);
    $this->data->setValue("description", $di->v["description"], $c);

    $this->data->setValue("version", $di->v["version"], $c);
    $this->data->setValue("status", $di->v["status"], $c);

    $this->data->setValue("author", $di->v["author"], $c);
    $this->data->setValue("email", $di->v["email"], $c);

    if($changelog)
     $this->data->setValue("changelog", $fs->read($tmppath."changelog.txt"), $c);
    else
     $this->data->setValue("changelog", "", $c);

    $this->data->setValue("dependencies", $dependencies, $c);
   }

   if(!$fs->delete($tmppath, true))
   {
    $this->data->e->add($fs->e);
    return false;
   }

   return true;
  }

 }
?>
