<?

 /*****************************************************************************
 * Filename: include/modules/users.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Module_users extends Module
 {
  function init(&$data)
  {
   Module::init($data);

   global $_config;

   $this->data->title=$this->data->l->t("Users");
   $this->data->addField("nr", "nr", "sql;form", "owner;identifier;hidden;global;default", 0, false, "usernr");

   $this->data->addField("username", "title", "all;login", "username;title", 4, 15, false, "Username");
   $this->data->addField("password", "password", "sql;form;login", "password", 7, false, false, "Password");

   $this->data->addField("backend", "boolean", "all", "", false, false, false, "Backend login", "If this user is allowed to log in to the JAS management tool");

   $this->data->addField("catnr", "nr", "sql;form", "rightsReference;global;default", 1, false, 1);
   $this->data->addReference("catnr", "categories", "nr", "*:1");

   $this->data->addReference("usersAdditionalGroups", "categories", "nr", "*:*", "nr");

   if(isset($this->data->reference["usersAdditionalGroups"]))  //reference initialized?
   {
    $this->data->field["usersAdditionalGroups"]["title"]=$this->data->l->t("Additional user groups");
    $this->data->field["usersAdditionalGroups"]["text"]=$this->data->l->t("Select additional rights groups for this user");
   }

   $this->data->addActions("user");
   $this->data->sortOrder("username");

   $this->data->addUnique("username", "username");
  }


  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   global $_config;

   //hide backend field for none root users
   if($_config["usernr"]==$_config["rootnr"])
    $this->data->resetField("backend");
   else
    $this->data->field["backend"]["properties"]=";hidden";

   return true;
  }


  function addPreSubmit(){return $this->editPreSubmit();}
  function editPreSubmit()
  {
   global $_config;

   //make sure backend value is only set by root user
   if($_config["usernr"]!=$_config["rootnr"])
    $this->data->unsetValue("backend"); 

   return true;
  }


  function editPreSubmitErrorCheck()
  {
   if(!isset($this->data->reference["usersAdditionalGroups"])) //skip check
    return true;

   $d=$this->data->copy();
   $d->setAction("view");
   $d->setExtra("item");
   $d->get();
   $current=$d->getValue("usersAdditionalGroups");

   $success=true;

   if(!$storage=$d->storageObject())
    return false;

   for($this->data->vc=0;$this->data->vc<$this->data->vmax;$this->data->vc++)
   {
    if($this->data->check("valuetype", "request", "usersAdditionalGroups")||$this->data->check("valuetype", "set", "usersAdditionalGroups"))
    {
     if(!($new=$this->data->getValue("usersAdditionalGroups", $this->data->vc)))
      $new=Array();
    }
    else
     $new=Array();

    if($current!=$new)
    {
     $d->setAction("add");
     $d->clearValues();

     foreach($new as $v)
     {
      if(!in_array($v, $current)) //new group
      {
       $d->setValue("catnr", $v);
       if(!$storage->checkRights("add"))
        $success=false; 
      }
     }

     $d->setAction("delete");
     $d->clearValues();

     foreach($current as $v)
     {
      if(!in_array($v, $new)) //removed group
      {
       $d->setValue("nr", $this->data->getValue("nr", $this->data->vc));
       if(!$storage->checkRights("delete"))
        $success=false; 
      }
     }
    }
   }

   if(!$success)
    $this->data->e->add($storage->e);

   return $success;
  }


  function deletePreSubmitErrorCheck()
  {
   global $_config;

   foreach($this->data->field["nr"]["value"] as $v)
   {
    if($v==$_config["rootnr"])
     $this->data->e->add("Can't delete root user", "hack");
    else if($v==$_config["usernr"])
     $this->data->e->add("Can't delete own user", "error");
   }

   return true;
  }

  function deletePostSubmit()
  {
   //remove rights for user 
   $du=Data::cache("userRights", "delete", "all");
   if(!($dus=$du->storageObject()))
    return false;

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //delete user rights
    $du->clearValues();
    $du->setValue("nr", $this->data->getValue("nr", $c));

    for($du->vc=0;$du->vc<$du->vmax;$du->vc++)
    {
     if(!$dus->delete(true))
     {
      $this->data->e->add($dus->e);
      return false;
     }
    }
   }

   return true;
  }


  function userPostSubmitErrorCheck()
  {
   if(!$this->data->check("use", "show", "backend"))
    $this->data->setShowFields(array_merge($this->check("use", "show"), Array("backend")));

   if($this->data->e->any()&&$this->data->extra=="login")
   {
    $this->data->e->clear();
    $this->data->e->add("Login failed");
   }

   return true;
  }

  function userPostSubmit()
  {
   global $_config;
   
   switch($this->data->extra)
   {
    case "login":
    {
     $this->data->e->clear();
     if($this->data->next())
     {
      $_config["usernr"]=$this->data->v("id");
      $_config["groupnrs"]=$this->getGroups($_config["usernr"]);
      $_config["userbackend"]=$this->data->v["backend"];

      $this->data->e->add("Login successful", "note");
     }
     else
     {  
      $this->data->e->add("Login failed", "error");
      return false;
     }

     break;
    }
    case "logout":
    {
     Config::reload();
     $_config["usernr"]=$_config["anonnr"];
     $_config["groupnrs"]=$this->getGroups($_config["usernr"]);
     unset($_config["userbackend"]);

     $this->data->e->add("Logout successful", "note");

     break;
    }
    default: break; //password, do nothing
   }

   return true;
  }


  function getGroups($usernr)
  {
   global $_config;
   $groups=Array();

   $d=Data::cache($this->data->module, "view", "item");
   $d->setValue("nr", $usernr);

   if(isset($d->reference["usersAdditionalGroups"]))
    $d->setShowFields("catnr", "usersAdditionalGroups");
   else
    $d->setShowFields("catnr");

   //sudo get groups 
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["rootnr"];
   if(!$d->get()) 
   {
    $_config["usernr"]=$tmp;
    return $groups;
   }
   $_config["usernr"]=$tmp;
   unset($tmp);

   if($d->next())
   {
    if(isset($d->v["usersAdditionalGroups"]))
     $groups=array_merge(Array($d->v["catnr"]), $d->v["usersAdditionalGroups"]);
    else
     $groups=Array($d->v["catnr"]);
   }

   return $groups;
  }

 }
?>
