<?

 /*****************************************************************************
 * Filename: include/mp.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."socket.php");

 class MP
 {
  function MP()
  {
   $this->e=new Error();
   $this->e->l->read("fs");

   $this->messageBusDir="/tmp/";
   $this->messageBusPrefix="message_bus-"; //just add PID

   $this->clear();
  }

  function clear()
  {
   $this->pid=false;
   $this->process=false;
   $this->socket=false;
  }

  function fork($messageBus=false)
  {
   if($messageBus)
   {
    $socket=new Socket();
    $messageBus=$this->messageBusDir.$this->messageBusPrefix;
   }

   $pid=pcntl_fork();
   if($pid==-1) //fail
   {
    $this->e->add("Unable to fork", "error");
    return false;
   }

   if($pid) //parent
   {
    $this->pid=$pid;
    $this->process="parent";

    if($messageBus)
    {
     $messageBus.=$this->pid;

     if(!$socket->listen($messageBus, 0, "file"))
     {
      $this->e->add("Unable to set up message bus: \"".$messageBus."\"", "error");
      $this->e->add($socket->e);
      return false;
     }

     if(!($socket2=$socket->accept()))
     {
      $this->e->add("Unable to set up message bus: \"".$messageBus."\"", "error");
      $this->e->add($socket->e);
      return false;
     }

     $socket->close();

     $this->socket=$socket2;
    }
   }
   else //child
   {
    $this->pid=getmypid();
    $this->process="child";

    if($messageBus)
    {
     $messageBus.=$this->pid;

     for($c=0;$c<10;$c++) //10 retries to connect to parent 
     {
      if(!file_exists($messageBus)||!$socket->connect($messageBus, 0, "file"))
       usleep(10); //wait 10 microseconds for parent process to set up socket
      else
       break;
     }

     if(!$socket->socket)
     {
      $this->e->add("Unable to connect to message bus: \"".$messageBus."\"", "error");
      return false;
     }

     $this->socket=$socket;
    }
   }

   return $this->process;
  }

  function recv()
  {
   if($this->socket==false)
   {
    $this->e->add("Message bus not set up", "error");
    return false;
   }

   return $this->socket->read();
  }

  function send($message)
  {
   if($this->socket==false)
   {
    $this->e->add("Message bus not set up", "error");
    return false;
   }

   return $this->socket->write($message);
  }

  function wait()
  {
   if($this->process=="child")
   {
    $this->e->add("Child process can't wait for itself", "error");
    return false;
   }

   $status=false;
   pcntl_waitpid($this->pid, $status);

   $this->e->add("[".$this->pid."] Child exit status: ".$status, "debug");
   return true;
  }

  function end() // end child
  {
   if($this->socket&&$this->socket->socket) //close down message bus
   {
    $this->socket->close();
    $this->socket=false;
   }

   if($this->process=="parent")
    return $this->wait();
   else //child
    exit(0);

   return false; //?? :)
  }

  function __destruct()
  {
   if($this->socket&&$this->socket->socket) //close down message bus
   {
    $this->socket->close();
    $this->socket=false;
   }
  }

 }

?>
