<?

 /*****************************************************************************
 * Filename: include/mutilate.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."language.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."dt.php");

 class Mutilate extends DT
 {
  function Mutilate()
  {
   parent::DT();
  }


//NUMBERS
 
  function humanReadableNumber($number, $decimals=false, $decimalseparator=false)
  {
   //array values
   if(is_array($number))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($number as $k => $v)
      $number[$k]=$this->humanReadableNumber($v, $decimals, $decimalseparator);
    }
    else
    {
     foreach($number as $k => $v)
      $number[$k]=Mutilate::humanReadableNumber($v, $decimals, $decimalseparator);
    }

    return $number;
   }

   if($decimals===false)
    $decimals=0;

   if(-1000000000000000>$number||$number>1000000000000000)
   {
    $number=round($number/1000000000000000, $decimals);
    $suffix="P";
   }
   else if(-1000000000000>$number||$number>1000000000000)
   {
    $number=round($number/1000000000000, $decimals);
    $suffix="T";
   }
   else if(-1000000000>$number||$number>1000000000)
   {
    $number=round($number/1000000000, $decimals);
    $suffix="G";
   }
   else if(-1000000>$number||$number>1000000)
   {
    $number=round($number/1000000, $decimals);
    $suffix="M";
   }
   else if(-1000>$number||$number>1000)
   {
    $number=round($number/1000, $decimals);
    $suffix="K";
   }
   else
   {
    $number=round($number, $decimals);
    $suffix="";
   }

   if($decimalseparator)
    $number=number_format($number, $decimals, $decimalseparator, " ");

   return $number.$suffix;
  }

  function humanReadableFileSize($size, $decimals=false, $decimalseparator=false)
  {
   //array values
   if(is_array($size))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($size as $k => $v)
     {
      if(is_array($v))
       $size[$k]=$this->humanReadableFileSize($v, $decimals, $decimalseparator);
      else
       $size[$k]=$this->humanReadableNumber($v, $decimals, $decimalseparator)."B";
     }
    }
    else
    {
     foreach($size as $k => $v)
     {
      if(is_array($v))
       $size[$k]=Mutilate::humanReadableFileSize($v, $decimals, $decimalseparator);
      else
       $size[$k]=Mutilate::humanReadableNumber($v, $decimals, $decimalseparator)."B";
     }
    }

    return $size;
   }

   if(Mutilate::isInitialized()) //object initialized
    return $this->humanReadableNumber($size, $decimals, $decimalseparator)."B";
   else
    return Mutilate::humanReadableNumber($size, $decimals, $decimalseparator)."B";
  }

  function leadingZeros($number, $length)
  {
   //array values
   if(is_array($number))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($number as $k => $v)
      $number[$k]=$this->leadingZeros($v, $length);
    }
    else
    {
     foreach($number as $k => $v)
      $number[$k]=Mutilate::leadingZeros($v, $length);
    }

    return $number;
   }

   $zeros=$length-strlen($number);

   if($zeros<=0)
    return $number;

   return str_repeat("0", $zeros).$number;
  }


//STRINGS
 
  function strip($text, $length=false, $suffix=false)
  {
   if($length===false)
    $length=32;

   //array values
   if(is_array($text))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($text as $k => $v)
      $text[$k]=$this->strip($v, $length, $suffix);
    }
    else
    {
     foreach($text as $k => $v)
      $text[$k]=Mutilate::strip($v, $length, $suffix);
    }

    return $text;
   }

   if(strlen($text)>$length)
   {
    if($suffix===false)
     $suffix="...";
 
    $length-=strlen($suffix);
   }
   else
    $suffix="";

   return substr($text, 0, $length).$suffix;
  }


  function replace($text, $pattern, $replacement=false)
  {
   if(!$replacement)
    $replacement="";

   //array values
   if(is_array($text))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($text as $k => $v)
      $text[$k]=$this->replace($v, $pattern, $replacement);
    }
    else
    {
     foreach($text as $k => $v)
      $text[$k]=Mutilate::replace($v, $pattern, $replacement);
    }

    return $text;
   }

   return preg_replace($pattern, $replacement, $text);
  }


  function changeCase($text, $type)
  {
   //array values
   if(is_array($text))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($text as $k => $v)
      $text[$k]=$this->changeCase($v, $type);
    }
    else
    {
     foreach($text as $k => $v)
      $text[$k]=Mutilate::changeCase($v, $type);
    }

    return $text;
   }

   switch(strtolower($type))
   {
    case "lower":
     return strtolower($text);

    case "upper":
     return strtoupper($text);

    case "first":
     return ucfirst(strtolower($text)); //only first character!

    case "words":
     return ucwords(strtolower($text));

    default: break;
   }

   return $text;
  }


  function trim($text, $chars=false, $type=false)
  {
   if(!$chars)
    $chars=" \t\n\r\0\x0B";

   if(!$type)
    $type="both";

   switch($type)
   {
    case "left":
    case "begin":
    case "ltrim":
    {
     $text=ltrim($text, $chars);
     break;
    }
    case "right":
    case "end":
    case "rtrim":
    {
     $text=rtrim($text, $chars);
     break;
    }
    default:
    {
     $text=trim($text, $chars);
     break;
    }
   }

   return $text;
  }


//HTML

  function stripTags($text)
  {
   //array values
   if(is_array($text))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($text as $k => $v)
      $text[$k]=$this->stripTags($v);
    }
    else
    {
     foreach($text as $k => $v)
      $text[$k]=Mutilate::stripTags($v);
    }

    return $text;
   }

   return strip_tags($text);
  }

  function htmlEncode($text)
  {
   //array values
   if(is_array($text))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($text as $k => $v)
      $text[$k]=$this->htmlEncode($v);
    }
    else
    {
     foreach($text as $k => $v)
      $text[$k]=Mutilate::htmlEncode($v);
    }

    return $text;
   }

   return htmlentities($text);
  }

  function htmlDecode($text)
  {
   //array values
   if(is_array($text))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($text as $k => $v)
      $text[$k]=$this->htmlDecode($v);
    }
    else
    {
     foreach($text as $k => $v)
      $text[$k]=Mutilate::htmlDecode($v);
    }

    return $text;
   }

   return html_entity_decode($text);
  }


//URL

  function urlEncode($url)
  {
   //array values
   if(is_array($url))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($url as $k => $v)
      $url[$k]=$this->urlEncode($v);
    }
    else
    {
     foreach($url as $k => $v)
      $url[$k]=Mutilate::urlEncode($v);
    }

    return $url;
   }

   if(preg_match("!(^[a-z0-9]+://[^\?]+)(.*)$!i", $url, $m))  //domain and args (args can contain ?)
   {
    $prefix=$m[1];

    if($m[2]) //args, ? included
    {
     $prefix.="?";
     $args=substr($m[2], 1);
    }
    else //no args
     $args="";
   }
   else if($url[0]=="?") //arg ? included
   {
    $prefix="?";
    $args=substr($url, 1);
   }
   else
   {
    $prefix="";
    $args=$url;
   }

   if($args)
   {
    $encargs=Array();
    foreach(explode("&", $args) as $arg)
    {
     $arg=explode("=", $arg);

     if(sizeof($arg)==1) //only key name
      $encargs[]=urlencode($arg[0]);
     else //key and value
      $encargs[]=urlencode($arg[0])."=".urlencode($arg[1]);
    }

    $args=implode("&", $encargs);
   }

   return $prefix.$args;
  }

  function urlDecode($url)
  {
   //array values
   if(is_array($url))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($url as $k => $v)
      $url[$k]=$this->urlDecode($v);
    }
    else
    {
     foreach($url as $k => $v)
      $url[$k]=Mutilate::urlDecode($v);
    }

    return $url;
   }

   return urldecode($url);
  }


//EMAIL

  function scrambleEmail($email)
  {
   //array values
   if(is_array($email))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($email as $k => $v)
      $email[$k]=$this->scrambleEmail($v);
    }
    else
    {
     foreach($email as $k => $v)
      $email[$k]=Mutilate::scrambleEmail($v);
    }

    return $email;
   }

   return str_replace("@", "(at)", str_replace(".", "[dot]", $email));
  }
  function unscrambleEmail($email)
  {
   //array values
   if(is_array($email))
   {
    if(Mutilate::isInitialized()) //object initialized
    {
     foreach($email as $k => $v)
      $email[$k]=$this->unscrambleEmail($v);
    }
    else
    {
     foreach($email as $k => $v)
      $email[$k]=Mutilate::unscrambleEmail($v);
    }

    return $email;
   }

   return str_replace("(at)", "@", str_replace("[dot]", ".", $email));
  }


 }

?>
