<?

 /*****************************************************************************
 * Filename: include/plugin.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."fs.php");

 class Plugin
 {
  function Plugin($name)
  {
   global $_path;

   $this->fs=new FS();
   $this->e=new Error();
   $this->l=new Language("plugin");

   $this->info=false;
   $this->deps=false;
   $this->class=false;
   $this->frontend=false;
   $this->backend=false;
   $this->backendClass=false;
   $this->setup=false;
   $this->setupClass=false;
   $this->config=Array();
   $this->baseURL="";
   $this->vars=Array();
   $this->enabled=false;

   $this->name=$name;

   $this->path=Array();
   $this->path["status"]=$_path["project"]["plugins"]."status.txt";

   if(!$this->fs->exists($_path["project"]["plugins"], "directory"))
    if(!$this->fs->mkdir($_path["project"]["plugins"])||!$this->fs->setMode($_path["project"]["plugins"], 7, 5, 5))
     $this->e->add("Unable to create plugins directory: \"".$_path["project"]["plugins"]."\", please make sure the webservice user has the appropriate permissions", "system");

   if($this->fs->exists($_path["project"]["plugins"].$this->name, "directory"))
   {
    $this->path["root"]=$_path["project"]["plugins"].$this->name."/";
    $this->type="project";
   }
   else if($this->fs->exists($_path["plugins"].$this->name, "directory"))
   {
    $this->path["root"]=$_path["plugins"].$this->name."/";
    $this->type="system";
   }
   else
   {
    $this->e->add("Plugin: \"".$this->name."\" doesn't exist", "hack");
    return;
   }

   $this->path["modules"]=$this->path["root"]."modules/";
   $this->path["templates"]=$this->path["root"]."templates/";
   $this->path["pluginInfo"]=$this->path["root"]."pluginInfo.xml";
   $this->path["pluginDependencies"]=$this->path["root"]."pluginDependencies.xml";

   if(!preg_match(REGEXP_DATA_PLUGIN, $this->name))
   {
    $this->e->add("Illegal plugin name: \"".$this->name."\"", "hack");
    return;
   }

   $this->l->read("plugin_".$this->name);

   $this->getStatus();
  }

  function copy()
  {
   if(substr(phpversion(), 0, 1)>=5)
   {
    eval('$p=clone($this);');
    foreach($this as $k => $v)
    {
     if(is_object($v))
      eval('$p->{$k}=clone($v);');
     else if(is_array($v))
     {
      $p->{$k}=Array();
      foreach($v as $k2 => $v2)
       $p->{$k}[$k2]=$v2;
     }
    }
   }
   else
    $p=$this;

   return $p;
  }

  function isEnabled()
  {
   return $this->enabled;
  }

  function hasFrontend()
  {
   return $this->frontend;
  }

  function hasBackend()
  {
   return $this->backend;
  }

  function hasSetup()
  {
   return $this->setup;
  }

  function export($method)
  {
   $archive=new Archive();
   $archive->addDir($this->path["root"], preg_replace("!/[^/]+/$!", "/", $this->path["root"]));

   if(!$archive->send($this->name."-".$this->version.".".$method))
   {
    $this->e->add($archive->e);
    return false;
   }

   return true;
  }

  function import($file)
  {
   global $_config, $_path;

   if($_config["rootnr"]!=$_config["usernr"])
   {
    $this->e->add("Plugins can only be imported by the root user", "error");
    return false;
   }

   $tmppath=$_path["project"]["plugins"].".tmp/";

   if($this->fs->exists($tmppath))
    if(!$this->_removeTmp())
     return false;

   if(!$this->fs->mkdir($tmppath))
   {
    $this->e->add("Unable to create plugin temporary import directory: \"".$tmppath."\"", "error");
    return false;
   }

   if(!preg_match("!/?([a-z0-9\_]{1,25})-(\d+\.\d+(\.\d+)?)[^/]+$!i", $file, $name))
   {
    $this->e->add("No plugin name detected from file name: \"".$file."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $this->name=$name[1];
   $this->version=$name[2];

   $archive=new Archive();

   if(!$archive->open($file))
   {
    $this->e->add($archive->e);
    $this->_removeTmp();
    return false;
   }

   $info=false;
   foreach($archive->getFiles() as $entry)
   {
    if(preg_match("!pluginInfo\.xml$!", $entry)&&!preg_match("!language/[a-z]+/modules/pluginInfo.xml$!i", $entry))
    {
     $info=$entry;
     break;
    }
   }

   if(!$info)
   {
    $this->e->add("Unabled to find \"pluginInfo.xml\" in import file: \"".$file."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $pluginroot=preg_replace("!pluginInfo\.xml$!", "", $info);

   if(!$archive->rerootFiles($pluginroot))
   {
    $this->e->add($archive->e);
    $this->_removeTmp();
    return false;
   }

   unset($pluginroot, $info);

   if(!$archive->saveFiles($tmppath))
   {
    $this->e->add($archive->e);
    $this->_removeTmp();
    return false;
   }

   unset($archive);

   $plugin=new Plugin(".tmp");
   $plugin->readInfo($this->name);

   $plugin->initClass();

   if($plugin->e->any())
   {
    $this->e->add($plugin->e);
    $this->_removeTmp();
    return false;
   }

   $plugin->callExists($plugin->main_function);
   $plugin->callExists($plugin->main_output);

   if($plugin->e->any())
   {
    $this->e->add($plugin->e);
    $this->_removeTmp();
    return false;
   }

   if($plugin->version!=$this->version)
   {
    $this->e->add("Plugin \"".$this->name."\" info version: \"".$plugin->version."\" doesn't match archive version: \"".$this->version."\"");
    $this->_removeTmp();
    return false;
   }

   if($plugin->initBackendClass())
   {
    $plugin->callExists($plugin->backend_function, "backend");
    $plugin->callExists($plugin->backend_output, "backend");
   }

   if($plugin->initSetupClass())
   {
    if($plugin->setup_init)
     $plugin->callExists($plugin->setup_init, "setup");
    if($plugin->setup_preSetup)
     $plugin->callExists($plugin->setup_preSetup, "setup");
    if($plugin->setup_postSetup)
     $plugin->callExists($plugin->setup_postSetup, "setup");
    if($plugin->setup_preDeSetup)
     $plugin->callExists($plugin->setup_preDeSetup, "setup");
    if($plugin->setup_postDeSetup)
     $plugin->callExists($plugin->setup_postDeSetup, "setup");
   }

   if($plugin->e->any())
   {
    $this->e->add($plugin->e);
    $this->_removeTmp();
    return false;
   }

   if(in_array($this->name, plugins("project")))
   {
    $plugin2=new Plugin($this->name);
    $plugin2->readInfo();

    if($plugin->version==$plugin2->version)
    {
     $this->e->add("Plugin: \"".$this->name." ".$plugin2->version."\" is already installed, do you want to reinstall it?", "warning");
     return false;
    }
    else
    {
     $this->e->add("Plugin: \"".$this->name." ".$plugin2->version."\" is already installed, do you want to install version ".$plugin->version."?", "warning");
     return false;
    }
   }

   $pluginpath=$_path["project"]["plugins"].$this->name;
   if(!$this->fs->rename($tmppath, $pluginpath))
   {
    $this->e->add("Unable to rename \"".$tmppath."\" to \"".$pluginpath."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $this->e->add("Plugin: \"".$this->name." ".$plugin->version."\" installed", "note");

   return true;
  }

  function importOverwrite($name)
  {
   global $_config, $_path;

   if($_config["rootnr"]!=$_config["usernr"])
   {
    $this->e->add("Plugins can only be imported by the root user", "error");
    return false;
   }

   $tmppath=$_path["project"]["plugins"].".tmp/";

   if(!$this->fs->exists($tmppath))
   {
    $this->e->add("Uploaded plugin not found in: \"".$tmppath."\"", "error");
    return false;
   }

   if(!preg_match("/^([a-z0-9\_]{1,25})$/i", $name))
   {
    $this->e->add("Illegal plugin name: \"".$name."\"", "error");
    $this->_removeTmp();
    return false;
   }

   $this->name=$name;

   $plugin=new Plugin(".tmp");
   $plugin->readInfo($name);

   if($plugin->e->any())
   {
    $this->e->add($plugin->e);
    $this->_removeTmp();
    return false;
   }

   $pluginpath=$_path["project"]["plugins"].$this->name;

   $enabled=false;
   if($this->fs->exists($pluginpath))
   {
    $plugin2=new Plugin($name);

    if($plugin2->isEnabled())
    {
     $plugin2->setStatus(false);
     $enabled=true;
    }

    if(!$plugin2->remove())
    {
     $this->e->add($plugin2->e);
     $this->_removeTmp();
     return false;
    }
   }

   if(!$this->fs->rename($tmppath, $pluginpath))
   {
    $this->e->add("Unable to rename \"".$tmppath."\" to \"".$pluginpath."\"", "error");
    $this->_removeTmp();
    return false;
   }

   if($enabled)
    $plugin2->setStatus(true);

   $this->e->add("Plugin: \"".$this->name." ".$plugin->version."\" installed", "note");

   return true;
  }

  function _removeTmp()
  {
   global $_path;

   $tmppath=$_path["project"]["plugins"].".tmp/";

   if(!$this->fs->delete($tmppath, true))
   {
    $this->e->add("Unable to remove plugin temporary import directory: \"".$tmppath."\"", "error");
    return false;
   }

   return true;
  }

  function remove()
  {
   global $_path, $_config;

   if($_config["rootnr"]!=$_config["usernr"])
   {
    $this->e->add("Plugins can only be removed by the root user", "error");
    return false;
   }

   if($this->type=="system")
   {
    $this->e->add("Plugin: \"".$this->name."\" is a system plugin, refusing to remove", "error");
    return false;
   }

   if($this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is enabled, refusing to remove", "error");
    return false;
   }

   if(!$this->fs->delete($this->path["root"], true))
   {
    $this->e->add("Unable to remove plugin directory: \"".$this->path["root"]."\"", "error");
    return false;
   }

   $this->e->add("Plugin: \"".$this->name."\" removed", "note");

   return true;
  }

  function getStatus()
  {
   if(!$this->fs->exists($this->path["status"]))
   {
    if(!$this->fs->write($this->path["status"], ""))
    {
     $this->e->add("Unable to create plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
     return false;
    }
   }

   if(($stats=$this->fs->readLines($this->path["status"], true))===false)
   {
    $this->e->add("Unable to open plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
    return false;
   }

   $l=sizeof($stats);
   for($c=0;$c<$l;$c++)
   {
    $tmp=explode("=", $stats[$c]);
    if($tmp[0]==$this->name)
    {
     if($tmp[1])
     {
      $this->enabled=true;
      return true;
     }

     break;
    }
   }

   $this->enabled=false;
   return false;
  }

  function setStatus($enabled)
  {
   global $_config, $_modules, $_jses;

   if($_config["rootnr"]!=$_config["usernr"])
   {
    $this->e->add("Plugins can only be enabled/disabled by the root user", "error");
    return false;
   }

   if($enabled)
    $enabled=1;
   else
    $enabled=0;

   $found=false;
   if(($stats=$this->fs->readLines($this->path["status"], true))===false)
   {
    $this->e->add("Unable to open plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
    return false;
   }

   $l=sizeof($stats);

   for($c=0;$c<$l;$c++)
   {
    $tmp=explode("=", $stats[$c]);
    if($tmp[0]==$this->name)
    {
     if($tmp[1]==$enabled) //already on correct state
      return true;

     $found=true;
     $stats[$c]=$this->name."=".$enabled;
     break;
    }
   }

   if(!$found)
    $stats[]=$this->name."=".$enabled;

   if(!$this->fs->write($this->path["status"], implode("\n", $stats), "overwrite"))
   {
    $this->e->add("Unable to write plugin status file: \"".$this->path["status"]."\", please make sure the webservice user has the appropriate permissions", "system", false, false);
    return false;
   }

   if($enabled)
    $this->enabled=true;
   else
    $this->enabled=false;

   //clear module cache (used by modules() call)
   unset($_modules[$_jses["project"]]["_modules"]);

   //clear plugin modules cache (used by modules() call)
   foreach(array_keys($_modules[$_jses["project"]]) as $module)
    if(strpos($module, $this->name."_")===0)
     unset($_modules[$_jses["project"]][$module]);

   //ensure plugin modules are recached
   if(isset($_config["cache"])&&isset($_config["cache"]["memory"])&&$_config["cache"]["memory"]&&$_config["cache"]["memoryCacheData"])
   {
    $cache=new Cache("memory");
    foreach($cache->search("DATA::CACHE(".$_jses["project"].",".$this->name."_") as $key)
     $cache->remove($key);
   }

   return true;
  }

  function readInfo($overwriteName=false)
  {
   if(!$overwriteName)
    $overwriteName=$this->name;

   $succes=true;
   if($this->info===false)
   {
    if(in_array($this->name."_pluginInfo", modules("plugins", $this->name)))
    {
     $this->info=Data::cache($this->name."_pluginInfo", "view", "all", Array("db" => "xml://".$this->path["pluginInfo"]));
     $this->info->clearValues();
    }
    else
    {
     $this->info=Data::cache("pluginInfo", "view", "all", Array("db" => "xml://".$this->path["pluginInfo"]));
     $this->info->clearValues();
     $this->info->plugin=$this;
    }

    $this->l->read("plugin_".$this->name);

    $this->info->field["plugin"]["defaultvalue"]=$overwriteName;
    $this->info->field["title"]["defaultvalue"]=$overwriteName;
    $this->info->field["main_file"]["defaultvalue"]=$overwriteName.".php";
    $this->info->field["main_class"]["defaultvalue"]=$overwriteName;
    $this->info->field["main_function"]["defaultvalue"]="main";
    $this->info->field["main_output"]["defaultvalue"]="output";

    $this->info->field["backend_file"]["defaultvalue"]="backend.php";
    $this->info->field["backend_class"]["defaultvalue"]=$overwriteName."_backend";
    $this->info->field["backend_function"]["defaultvalue"]="main";
    $this->info->field["backend_output"]["defaultvalue"]="output";

    $this->info->field["setup_file"]["defaultvalue"]="setup.php";
    $this->info->field["setup_class"]["defaultvalue"]=$overwriteName."_setup";
    $this->info->field["setup_init"]["defaultvalue"]="init";
    $this->info->field["setup_preSetup"]["defaultvalue"]="preSetup";
    $this->info->field["setup_postSetup"]["defaultvalue"]="postSetup";
    $this->info->field["setup_preDeSetup"]["defaultvalue"]="preDeSetup";
    $this->info->field["setup_postDeSetup"]["defaultvalue"]="postDeSetup";

    $this->info->get();
   }

   $this->info->setShowFields(array_merge($this->info->check("use", "show"), Array("version_major", "version_minor", "version_maintenance")));
   foreach($this->info->check("use", "show") as $field)
   {
    if($this->name!=".tmp"&&$field!="plugin")
     $this->info->checkField($field);

    if(isset($this->$field))
    {
     $this->e->add("Plugin info field: \"".$field."\" already in use in class!", "system");
     $succes=false;
    }
    else if(($value=$this->info->getValue($field))!==false)
    {
     if($field=="title")
      $this->$field=$this->l->t($value);
     else
      $this->$field=$value;
    }
    else
     $this->$field=$this->info->getDefaultValue($field);
   }

   if($this->main_file&&$this->fs->exists($this->path["root"].$this->main_file))
    $this->frontend=true;

   if($this->backend_file&&$this->fs->exists($this->path["root"].$this->backend_file))
    $this->backend=true;

   if($this->setup_file&&$this->fs->exists($this->path["root"].$this->setup_file))
    $this->setup=true;

   if(!$succes)
   {
    $this->info=false;
    return false;
   }

   if($this->info->e->any())
    $this->e->add($this->info->e);

   return true;
  }

  function readDeps()
  {
   if(!$this->fs->exists($this->path["pluginDependencies"])) //no dependencies
    return true;

   $this->deps=Data::cache("pluginDependencies", "view", "all", Array("db" => "xml://".$this->path["pluginDependencies"]));
   $this->deps->clearValues();
   $this->deps->plugin=$this;

   $succes=true;
   if(!$this->deps->e->any())
   {
    $this->deps->get();
    $this->deps->setShowFields(array_merge($this->deps->check("use", "show"), Array("version_major", "version_minor", "version_maintenance")));

    while($this->deps->next())
    {
     if($this->deps->v["required"])
     {
      $error="error";
      $required="required";
     }
     else
     {
      $error="warning";
      $required="preferred";
     }

     if(!$this->deps->v["plugin"]) //JAS dependency
     {
      if(!($version=explode(".", JAS::version())))
      {
       $this->e->add("Dependency ".$error.", JAS version can't be deteremined", $error);
       $succes=false;
      }
      else
      {
       if(!isset($version[1]))
        $version[1]=0;
       if(!isset($version[2]))
        $version[2]=0;

       if(
        $version[0]<$this->deps->v["version_major"]||
        ($version[0]==$this->deps->v["version_major"]&&$version[1]<$this->deps->v["version_minor"])||
        ($version[0]==$this->deps->v["version_major"]&&$version[1]==$this->deps->v["version_minor"]&&$version[2]<$this->deps->v["version_maintenance"])
       )
       {
        $this->e->add("Dependency ".$error.", JAS version \"".$this->deps->v["version"]."\" is ".$required.", version \"".JAS::version()."\" found", $error);
        $succes=false;
       }
      }
     }
     else
     {
      if(!in_array($this->deps->v["plugin"], plugins()))
      {
       $this->e->add("Dependency ".$error.", plugin: \"".$this->deps->v["plugin"]."\" not found", $error);
       $succes=false;
      }
      else
      {
       $tmp=new Plugin($this->deps->v["plugin"]);
       $tmp->readInfo();

       if(
        $tmp->version_major<$this->deps->v["version_major"]||
        ($tmp->version_major==$this->deps->v["version_major"]&&$tmp->version_minor<$this->deps->v["version_minor"])||
        ($tmp->version_major==$this->deps->v["version_major"]&&$tmp->version_minor==$this->deps->v["version_minor"]&&$tmp->version_maintenance<$this->deps->v["version_maintenance"])
       )
       {
        $this->e->add("Dependency ".$error.", \"".$this->deps->v["plugin"]."\" plugin version \"".$this->deps->v["version"]."\" is ".$required.", version \"".$tmp->version."\" found", $error);
        $succes=false;
       }
       else if(!$tmp->isEnabled())
       {
        $this->e->add("Dependency ".$error.", plugin: \"".$this->deps->v["plugin"]."\" is disabled", $error);
        $succes=false;
       }
      }
     }
    }
   }

   if(!$succes)
    return false;

   return true;
  }

  function config($key, $id=false)
  {
   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to read config", "error");
    return false;
   }

   if(in_array($this->name."_pluginConfigs", modules("plugins", $this->name)))
    $config=Data::cache($this->name."_pluginConfigs", "view", "all");
   else
    $config=Data::cache("pluginConfigs", "view", "all");

   $config->clearValues();
   $config->setValue("plugin", $this->name);

   if(!$id)
    $config->setValue("key", $key);
   else
    $config->setValue("nr", $key);

   if(!$config->get())
   {
    $this->e->add($config->e);
    return false;
   }

   if(!$config->next())
   {
    $this->e->add("Plugin configuration: \"".$key."\" not found", "error");
    return false;
   }

   $this->config=$config->v;
   return true;
  }

  function initClass()
  {
   global $_path;

   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->frontend)
    return false;

   if(!$this->fs->exists($this->path["root"].$this->main_file))
   {
    $this->e->add("Plugin main file: \"".$this->main_file."\" not found for: \"".$this->name."\"", "plugin");
    return false;
   }

   require_once($this->path["root"].$this->main_file);

   $class=$this->main_class;

   if(!class_exists($class))
   {
    $this->e->add("Plugin main class: \"".$class."\" not found for: \"".$this->name."\"", "plugin");
    return false;
   }

   $this->class=new $class($this, $this->config);

   return true;
  }

  function initBackendClass()
  {
   global $_path, $_jasPath;

   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->backend)
    return false;

   if(!$this->fs->exists($this->path["root"].$this->backend_file))
   {
    $this->e->add("Plugin backend file: \"".$this->backend_file."\" not found for: \"".$this->name."\"", "plugin");
    return false;
   }

   require_once($this->path["root"].$this->backend_file);

   $class=$this->backend_class;

   if(!$class)
   {
    $this->e->add("Plugin backend class not specified for: \"".$this->name."\"", "plugin");
    return false;
   }

   if(!class_exists($class))
   {
    $this->e->add("Plugin backend class: \"".$class."\" not found for: \"".$this->name."\"", "plugin");
    return false;
   }

   $this->backendClass=new $class($this);

   return true;
  }

  function initSetupClass()
  {
   global $_path, $_jasPath;

   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if($this->setup)
   {
    if(!$this->fs->exists($this->path["root"].$this->setup_file))
    {
     $this->e->add("Plugin setup file: \"".$this->setup_file."\" not found for: \"".$this->name."\"", "plugin");
     return false;
    }

    require_once($this->path["root"].$this->setup_file);

    $class=$this->setup_class;
   }
   else
    $class="BasePluginSetup";

   if(!$class)
   {
    $this->e->add("Plugin setup class not specified for: \"".$this->name."\"", "plugin");
    return false;
   }

   if(!class_exists($class))
   {
    $this->e->add("Plugin setup class: \"".$class."\" not found for: \"".$this->name."\"", "plugin");
    return false;
   }

   $this->setupClass=new $class($this);

   if(!in_array($this->setup_init, get_class_methods($class))&&$this->setup_init=="init")
    $this->setup_init=false;
   if(!in_array($this->setup_preSetup, get_class_methods($class))&&$this->setup_preSetup=="preSetup")
    $this->setup_preSetup=false;
   if(!in_array($this->setup_postSetup, get_class_methods($class))&&$this->setup_postSetup=="postSetup")
    $this->setup_postSetup=false;
   if(!in_array($this->setup_preDeSetup, get_class_methods($class))&&$this->setup_preDeSetup=="preDeSetup")
    $this->setup_preDeSetup=false;
   if(!in_array($this->setup_postDeSetup, get_class_methods($class))&&$this->setup_postDeSetup=="postDeSetup")
    $this->setup_postDeSetup=false;

   return true;
  }

  function callExists($method, $class=false)
  {
   if($class=="backend")
   {
    if(!$this->backend)
     return false;

    if(!$this->backendClass)
     if(!$this->initBackendClass())
      return false;

    $className=$this->backend_class;
   }
   else if($class=="setup")
   {
    if(!$this->setup)
     return false;

    if(!$this->setupClass)
     if(!$this->initSetupClass())
      return false;

    $className=$this->setup_class;
   }
   else
   {
    if(!$this->class)
     if(!$this->initClass())
      return false;

    $className=$this->main_class;
   }

   if(!method_exists($className, $method))
   {
    $this->e->add("Plugin function: \"".$className."->".$method."\" not found for: \"".$this->name."\"", "plugin");
    return false;
   }

   return true;
  }

  function call($method, $class=false)
  {
   if(!$this->callExists($method, $class))
    return false;

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to call \"".($class?$class."::":"").$method."\"", "error");
    return false;
   }

   $args=func_get_args();
   array_shift($args);
   array_shift($args);

   if($class=="backend")
    return $this->backendClass->$method($args);
   else if($class=="setup")
    return $this->setupClass->$method($args);
   else
    return $this->class->$method($args);
  }

  function main()
  {
   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to run main", "error");
    return false;
   }

   $method=$this->main_function;

   if(!$this->callExists($method))
    return false;

   $args=func_get_args();

   return $this->class->$method($args);
  }

  function output()
  {
   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to display output", "error");
    return $this->e->show();
   }

   $method=$this->main_output;

   if(!$this->callExists($method))
    return false;

   $args=func_get_args();

   return $this->class->$method($args);
  }

  function backend()
  {
   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to run backend", "error");
    return false;
   }

   $method=$this->backend_function;

   if(!$this->callExists($method, "backend"))
    return false;

   $args=func_get_args();

   $this->backendClass->$method($args);
   return true;
  }

  function backendOutput()
  {
   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to display backend output", "error");
    return $this->e->show();
   }

   $method=$this->backend_output;

   if(!$this->callExists($method, "backend"))
    return false;

   $args=func_get_args();

   return $this->backendClass->$method($args);
  }

  function initSetup($force, $deSetup=false)
  {
   global $_config;

   if($_config["rootnr"]!=$_config["usernr"])
   {
    $this->e->add("Plugins can only be setup by the root user", "error");
    return false;
   }

   if(!$this->isEnabled())
   {
    $this->e->add("Plugin: \"".$this->name."\" is disabled, refusing to setup", "error");
    return false;
   }

   if($this->info===false)
    if(!$this->readInfo())
     return false;

   if(!$this->setupClass)
    if(!$this->initSetupClass())
     return false;

   $this->setupClass->force=$force;
   $this->setupClass->deSetup=$deSetup;

   $method=$this->setup_init;

   if($method&&$this->callExists($method, "setup"))
   {
    $return=$this->setupClass->$method();

    if($return===NULL)
     $this->e->add("Init call: \"".$method."\" doesn't have a return value", "debug");
    else if($return)
     $this->e->add("Init call: \"".$method."\" successful", "note");
    else if(!$return)
    {
     $this->e->add("Init call: \"".$method."\" failed", "error");
     return false;
    }
   }
   else
   {
    $method="modules";
    $return=$this->setupClass->$method();

    if($return)
     $this->e->add("Init call: \"".$method."\" successful", "note");
    else if(!$return)
    {
     $this->e->add("Init call: \"".$method."\" failed", "error");
     return false;
    }
   }

   return true;
  }

  function setup($force=false)
  {
   if($this->initSetup($force)===false)
    return false;

   $method=$this->setup_preSetup;
   if($method&&$this->callExists($method, "setup"))
   {
    $return=$this->setupClass->$method();

    if($return===NULL)
     $this->e->add("Pre-Setup call: \"".$method."\" doesn't have a return value", "debug");
    else if($return)
     $this->e->add("Pre-Setup call: \"".$method."\" successful", "note");
    else if(!$return)
    {
     $this->e->add("Pre-Setup call: \"".$method."\" failed", "error");
     return false;
    }
   }

   if(!$this->setupClass->setup())
    return false;

   $method=$this->setup_postSetup;
   if($method&&$this->callExists($method, "setup"))
   {
    $return=$this->setupClass->$method();

    if($return===NULL)
     $this->e->add("Post-Setup call: \"".$method."\" doesn't have a return value", "debug");
    else if($return)
     $this->e->add("Post-Setup call: \"".$method."\" successful", "note");
    else if(!$return)
    {
     $this->e->add("Post-Setup call: \"".$method."\" failed", "error");
     return false;
    }
   }

   return true;
  }

  function deSetup()
  {
   if($this->initSetup(false, true)===false)
    return false;

   $method=$this->setup_preDeSetup;
   if($method&&$this->callExists($method, "setup"))
   {
    $return=$this->setupClass->$method();

    if($return===NULL)
     $this->e->add("Pre-deSetup call: \"".$method."\" doesn't have a return value", "debug");
    else if($return)
     $this->e->add("Pre-deSetup call: \"".$method."\" successful", "note");
    else if(!$return)
    {
     $this->e->add("Pre-deSetup call: \"".$method."\" failed", "error");
     return false;
    }
   }

   if(!$this->setupClass->deSetup())
    return false;

   $method=$this->setup_postDeSetup;
   if($method&&$this->callExists($method, "setup"))
   {
    $return=$this->setupClass->$method();

    if($return===NULL)
     $this->e->add("Post-deSetup call: \"".$method."\" doesn't have a return value", "debug");
    else if($return)
     $this->e->add("Post-deSetup call: \"".$method."\" successful", "note");
    else if(!$return)
    {
     $this->e->add("Post-deSetup call: \"".$method."\" failed", "error");
     return false;
    }
   }

   return true;
  }

  function baseURL($url)
  {
   $this->baseURL=$url;
   return true;
  }

  function addVar($name, $value)
  {
   if(isset($this->vars[$name]))
    $this->e->add("Overwriting variable: \"".$name."\"", "warning");

   $this->vars[$name]=$value;
   return true;
  }

  function url()
  {
   $url=$this->baseURL;

   $url.="?";
   foreach($this->vars as $k => $v)
    $url.=$k."=".$v."&amp;";

   return $url;
  }
 }


 /*
  *
  * BasePlugin
  */

 class BasePlugin
 {
  function BasePlugin(&$plugin, &$config)
  {
   $this->plugin=&$plugin;
   $this->config=&$config;
   $this->e=&$plugin->e;
  }

  function main()
  {
   return false;
  }

  function output()
  {
   if($this->e->any())
    return false;

   return "";
  }
 }


 /*
  *
  * BasePluginBackend
  */

 class BasePluginBackend
 {
  function BasePluginBackend(&$plugin)
  {
   $this->plugin=&$plugin;
   $this->e=&$plugin->e;
  }

  function main()
  {
   return false;
  }

  function output()
  {
   if($this->e->any())
    return false;

   return "";
  }
 }


 /*
  *
  * BasePluginSetup
  */

 class BasePluginSetup
 {
  function BasePluginSetup(&$plugin)
  {
   $this->plugin=&$plugin;
   $this->e=&$plugin->e;
   $this->fs=new FS();

   $this->directories=Array();
   $this->files=Array();
   $this->modules=Array();

   $this->done=true;
   $this->force=false;
   $this->deSetup=false;
  }

  function mkdir($directory)
  {
   if(!$this->deSetup)
   {
    if($this->fs->exists($directory))
    {
     if(!$this->fs->exists($directory, "directory"))
     {
      $status="Directory already exists as file";
      $this->done=false;
     }
     else
      $status="Setup correctly";
    }
    else
    {
     $status="Setup";
     $this->done=false;
    }
   }
   else //deSetup
   {
    if($this->fs->exists($directory))
    {
     if(!$this->fs->exists($directory, "directory"))
      $status="Directory is a file";
     else
      $status="DeSetup";
    }
    else
     $status="Not setup";

    $this->done=false;
   }

   $this->directories[]=Array(
    "directory" => $directory,
    "status" => $status,
   );

   return true;
  }

  function file($source, $destination)
  {
   if(substr($destination, -1)=="/")
    $destination.=preg_replace("!^.*/([^/]+)$!", "\\1", $source);

   if(!$this->fs->exists($this->plugin->path["root"].$source))
   {
    $status="File not found";
    $this->done=false;
   }
   else if(!$this->deSetup)
   {
    if($this->fs->exists($destination))
    {
     if($this->force)
      $status="Forced setup";
     else
     {
      if($this->fs->size($this->plugin->path["root"].$source)==$this->fs->size($destination))
       $status="Setup correctly";
      else
      {
       $status="File already exists";
       $this->done=false;
      }
     }
    }
    else
    {
     $status="Setup";
     $this->done=false;
    }
   }
   else //deSetup
   {
    if($this->fs->exists($destination))
    {
     if($this->fs->size($this->plugin->path["root"].$source)==$this->fs->size($destination))
      $status="DeSetup";
     else
      $status="File changed";
    }
    else
     $status="Not setup";

    $this->done=false;
   }

   $this->files[]=Array(
    "source" => $source,
    "destination" => $destination,
    "status" => $status,
   );

   return true;
  }

  function modules()
  {
   $modules=func_get_args();
   $pluginModules=modules("plugins", $this->plugin->name);

   if(!$modules)
   {
    if($pluginModules)
     return $this->modules($pluginModules);
    else
     return true;
   }
   else
   {
    if(is_array($modules[0]))
     $modules=$modules[0];

    foreach($modules as $module)
    {
     if(!preg_match(REGEXP_DATA_PLUGIN_MODULE, $module))
      $module=$this->plugin->name."_".$module;

     if($this->deSetup)
     {
      $this->modules[]=Array("module" => $module, "status" => "DeSetup not available");
      $this->done=false;
      continue;
     }

     if($module==$this->plugin->name."_pluginInfo"||$module==$this->plugin->name."_pluginDependencies")
      $this->modules[]=Array("module" => $module, "status" => "Skipped");
     else if(!in_array($module, $pluginModules))
     {
      $this->modules[]=Array("module" => $module, "status" => "Non plugin module");
      $this->done=false;
     }
     else
     {
      $d=Data::cache($module, "database", "create", Array("skipCheckTables" => true));
      if($d->tableExists())
      {
       if($this->force)
        $this->modules[]=Array("module" => $module, "status" => "Forced setup");
       else
        $this->modules[]=Array("module" => $module, "status" => "Setup correctly");
      }
      else
      {
       $this->modules[]=Array("module" => $module, "status" => "Setup");
       $this->done=false;
      }

      foreach(array_keys($d->reference) as $ref)
      {
       if($d->reference[$ref]["type"]=="*:*")
       {
        $d2=&$d->reference[$ref]["data"];
        if(!$d2->tableExists())
        {
         $this->modules[sizeof($this->modules)-1]["status"]="Setup";
         $this->done=false;
        }
        unset($d2);
       }
      }

      foreach(array_keys($d->extends) as $parentModule)
      {
       $d2=Data::cache($parentModule, "database", false, Array("skipCheckTables" => true));
       if(!$d2->tableExists())
       {
        $this->modules[]=Array("module" => $d2->module, "status" => "Setup");
        $this->done=false;
       }
       unset($d2);
      }

      unset($d);
     }
    }
   }

   return true;
  }

  function setup()
  {
   foreach($this->directories as $dir)
   {
    if($dir["status"]=="Setup"||$dir["status"]=="Forced setup")
    {
     if(!$this->fs->mkdir($dir["directory"]))
      $this->e->add("Unable to create directory: \"".$dir["directory"]."\"", "system");
    }
   }

   foreach($this->files as $file)
   {
    if($file["status"]=="Setup"||$file["status"]=="Forced setup")
    {
     if(!$this->fs->copy($this->plugin->path["root"].$file["source"], $file["destination"]))
      $this->e->add("Unable to copy file: \"".$file["source"]."\" to: \"".$file["destination"]."\"", "system");
    }
   }

   foreach($this->modules as $module)
   {
    if($module["status"]=="Setup"||$module["status"]=="Forced setup")
    {
     $d=Data::cache($module["module"], "database", "create", Array("skipCheckTables" => true));

     if($d->tableExists())
      $d->extra="alter";

     if(!$d->submit())
      $this->e->add($d->e);

     foreach(array_keys($d->reference) as $ref)
     {
      if($d->reference[$ref]["type"]=="*:*")
      {
       $d2=&$d->reference[$ref]["data"];

       if($d2->tableExists())
        $d2->extra="alter";

       if(!$d2->submit())
        $this->e->add($d2->e);
      }
     }
    }
   }

   if($this->e->any("fatal"))
    return false;

   if($this->force)
    $this->e->add("Setup forced successfully", "note");
   else
    $this->e->add("Setup successful", "note");

   return true;
  }

  function deSetup()
  {
   foreach($this->files as $file)
   {
    if($file["status"]=="DeSetup")
    {
     if(!$this->fs->delete($file["destination"]))
      $this->e->add("Unable to remove file: \"".$file["destination"]."\"", "system");
    }
   }

   foreach($this->directories as $dir)
   {
    if($dir["status"]=="DeSetup")
    {
     if($this->fs->recurseDir($dir["directory"]))
      $this->e->add("Not removing directory: \"".$dir["directory"]."\", not empty", "warning");
     else if(!$this->fs->delete($dir["directory"]))
      $this->e->add("Unable to remove directory: \"".$dir["directory"]."\"", "system");
    }
   }

   if($this->e->any("fatal"))
    return false;

   $this->e->add("DeSetup successful", "note");

   return true;
  }
 }

?>
