<?

 /*****************************************************************************
 * Filename: include/plugins/pager/backend.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."form.php");
 require_once($_path["include"]."cache.php");
 require_once($_jasPath["project"]["include"]."view.php");

 class pager_backend extends BasePluginBackend
 {
  function main()
  {
   if(!$this->plugin->config("default"))
   {
    $d=Data::cache("pager_pluginConfigs", "add", "item");
    $d->setValue("key", "default");

    $this->e->clear();
    $this->cmsOutput=$this->cms($d);
    return true;
   }

   global $_pager_simple;
   $_pager_simple=$this->plugin->config["simple"];

   if(request("cms")) //execute and render CMS
   {
    if(!($catnr=request("catnr"))||!is_numeric($catnr))
     $catnr=false;

    $d=Data::cache();
    $this->cmsOutput=$this->cms($d, Array("catnr" => $catnr));
   }
   else
    $this->cmsOutput=false;

   if(request("clearcache")) //clear cache
    if(!$this->clearCache(request("clearcache")))
     return false;

   return true;
  }

  function output($args)
  {
   //display control
   if($args[0]=="control")
   {
    if($this->plugin->config("default"))
     return $this->tree();

    return false;
   }

   //display cms output (rendered in main)
   if($this->cmsOutput)
    return $this->cmsOutput;
   else if(!$this->plugin->config("default")) //no default config, done
    return false;

   //display main
   if(!($catnr=request("catnr"))||!is_numeric($catnr))
    $catnr=false;

   if($catnr||(isset($this->plugin->config["rootpagenr"])&&($catnr=$this->plugin->config["rootpagenr"])))
   {
    $this->plugin->addVar("catnr", $catnr);

    $d=Data::cache("categories", "tree", "up");
    $d->moduleObject->extendBy("pager_categories");
    $d->setShowFields("name", "templatenr", "disabled", "symlink");

    $d->clearValues();
    $d->setValue("catnr", $catnr);

    $output="";

    if($d->get()&&$d->search("nr", $catnr))
    {
     //title
     $output.="<h1 class=\"page\">";

     //determine page type
     if($d->v["nr"]==$this->plugin->config["rootpagenr"])
     {
      if($d->v["disabled"])
       $type="rootpage-disabled";
      else if($d->v["symlink"])
       $type="rootpage-symlink";
      else
       $type="rootpage";
     }
     else if($d->v["disabled"])
      $type="normalpage-disabled";
     else if($d->v["symlink"])
      $type="normalpage-symlink";
     else if($d->v["templatenr"])
      $type="normalpage";
     else
      $type="containerpage";


     //get crumb trail
     $d->reset();
     if($d->vmax>1) //skip first (root category)
      $d->next();

     $path=Array();
     while($d->next())
      $path[]=$d->v["name"];

     $output.=icon($type)." ".implode("/", $path)."</h1>\n";

     if($this->e->any())
      $output.=$this->e->show();

     //content
     if(isset($this->plugin->config["rootpagenr"])&&$d->v["nr"]==$this->plugin->config["rootpagenr"]) //root page
      $output.=$this->template($catnr); //show template
     else if($d->v["symlink"]) //symlink, show info
      $output.=$this->info($catnr);
     else if($d->v["templatenr"]) //normal page
      $output.=$this->template($catnr); //show template
     else //normal category
      $output.=$this->content($catnr); //show content list
    }

    return $output;
   }

   return false;
  }


  function tree()
  {
   global $_config;

   $d=Data::cache("categories", "view", "all");
   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);
   $d->sortOrder("updated-");
   $d->setShowFields("updated");

   if(!($catnr=request("catnr"))||!is_numeric($catnr))
    $catnr=$this->plugin->config["rootpagenr"];

   //tree menu
   $menu=Array();

   $menu[]=Array(
    "text" => "Add page",
    "desc" => "Add ".strtolower($d->title),
    "icon" => Array("add", $d->module),
    "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=pager_categories&amp;catnr=".$catnr."&amp;moduleLock=0",
    "class" => "cms");

   if($this->plugin->config["simple"]) //simple mode
    $menu[]=Array(
     "text" => "Advanced mode",
     "desc" => "Switch to advanced mode",
     "icon" => "mode-advanced",
     "link" => $this->plugin->url()."cms=1&amp;catnr=1&amp;a=edit&amp;m=pager_pluginConfigs&amp;simple=0&amp;s=1&amp;catnr=".$catnr."&amp;nr=".$this->plugin->config["nr"],
     "class" => "mode");
   else
    $menu[]=Array(
     "text" => "Simple mode",
     "desc" => "Switch to simple mode",
     "icon" => "mode-advanced",
     "link" => $this->plugin->url()."cms=1&amp;catnr=1&amp;a=edit&amp;m=pager_pluginConfigs&amp;simple=1&amp;s=1&amp;catnr=".$catnr."&amp;nr=".$this->plugin->config["nr"],
     "class" => "mode");

   $menu[]=Array(
    "text" => "Pager config",
    "desc" => "Edit pager plugin configuration",
    "icon" => Array("edit", "pager_pluginConfigs"),
    "link" => $this->plugin->url()."cms=1&amp;catnr=1&amp;a=edit&amp;m=pager_pluginConfigs&amp;nr=".$this->plugin->config["nr"],
    "class" => "cms");

   //cache submenu
   $menu2=Array();

   $menu2[]=Array(
    "text" => "Clear page cache",
    "icon" => "cache-clear-page",
    "link" => $this->plugin->url()."clearcache=page",
    "class" => "cms");
   $menu2[]=Array(
    "text" => "Clear image cache",
    "icon" => "cache-clear-image",
    "link" => $this->plugin->url()."clearcache=image",
    "class" => "cms");
   $menu2[]=Array(
    "text" => "Clear all cache",
    "icon" => "cache-clear",
    "link" => $this->plugin->url()."clearcache=all",
    "class" => "cms");


   $menu[]=Array(
    "menu" => $this->menu("cache", $menu2, "Clear cache"),
    "class" => "cache");

   unset($menu2);

   $menu=$this->menu("menu4", $menu, false, "right");

   unset($catnr);

   if(!$d->get()||!$d->next())
    $cache=false;
   else
   {
    $cache=new Cache("file");
    if($cache->e->any("fatal")) //caching prolly disabled
     $cache=false;
    else
    {
     $cacheKey=$this->plugin->name."_uid".$_config["usernr"]."_backend_tree.html";

     //use last updated category
     $cacheTime=DT::dateConvert($d->v["updated"], $d->field["updated"]["format"][$d->dataformat]["datetimeformat"]);

     //see if config is updated later then cacheTime
     $configCacheTime=DT::dateConvert($this->plugin->config["updated"], $d->field["updated"]["format"][$d->dataformat]["datetimeformat"]);
     if($configCacheTime>$cacheTime)
      $cacheTime=$configCacheTime;
     unset($configCacheTime);

     if(($time=$cache->time($cacheKey))!==false) //exists
     {
      if($cacheTime==$time) //cache up-to-date
      {
       if(($tree=$cache->read($cacheKey))!==false) //no error, use the cache
        return $menu.$tree;
      }
      else //cache out of date
       $cache->remove($cacheKey); //remove the cache and re-cache after template is succesfully parsed
     }
    }
   }

   $r="<div class=\"pages\">\n";

   $r.="<ul class=\"tree\">\n";

   $d->setAction("tree");
   $d->setExtra("down");
   $d->clearValues();

   $d->moduleObject->extendBy("pager_categories");
   $d->setShowFields("name", "templatenr", "disabled", "catnr", "symlink", "treedepth");
   $d->sortOrder("ordernr", "name");
   $d->setValue("catnr", 0);
   $d->get();

   $last=false;
   while($d->next())
   {
    if($last>$d->v["treedepth"]) //going back up
    {
     //close all open list of deeper levels
     for($c=$d->v["treedepth"];$c<$last;$c++)
      $r.="</li>\n</ul>\n";

     $r.="</li>\n"; //close previous list item on current level
    }
    else if($last<$d->v["treedepth"]) //going down
     $r.="<ul class=\"".$d->v["treedepth"]."\">\n"; //open new list
    else if($last!==false) //same level
     $r.="</li>\n"; //close previous list item

    //determine page type
    if($d->v["nr"]==$this->plugin->config["rootpagenr"])
    {
     if($d->v["disabled"])
      $type="rootpage-disabled";
     else if($d->v["symlink"])
      $type="rootpage-symlink";
     else
      $type="rootpage";
    }
    else if($d->v["disabled"])
     $type="normalpage-disabled";
    else if($d->v["symlink"])
     $type="normalpage-symlink";
    else if($d->v["templatenr"])
     $type="normalpage";
    else
     $type="containerpage";

    $r.="<li class=\"$type\">";

    if($d->v["treedepth"]==0) //root
     $r.=$d->v["name"]; //just show category name
    else if(!$this->plugin->config["simple"]||$type!="containerpage") //show links
     $r.="<a href=\"".$this->plugin->url()."catnr=".$d->v["nr"]."\" title=\"Show ".$d->v["name"]."\">".icon($type)." ".$d->v["name"]."</a>";
    else //container in simple mode
     $r.=icon($type)." ".$d->v["name"]; //don't show link

    $last=$d->v["treedepth"];
   }

   //close all open list of deeper levels
   for($c=0;$c<$last;$c++)
    $r.="</li>\n</ul>\n"; //close previous list item

   $r.="</li>\n</ul>\n"; //close tree

   $r.="</div>\n";

   if($cache&&$cacheKey&&$cacheTime)
    if(!$cache->save($cacheKey, $r, $cacheTime))
     $this->e->add($cache->e);

   return $menu.$r;
  }


  function info($catnr)
  {
   global $_jasPath, $_config, $_jses;

   $r="";

   $d=Data::cache("categories", "view", "item");
   $d->moduleObject->extendBy("pager_categories");

   $d->setShowFields("name", "urlname", "templatenr", "doctype", "javascriptglobal", "javascript", "stylesheet", "disabled", "symlink", "symlinkpage", "disabled");
   $d->getReferenceData("symlinkpage")->setShowFields("name");
   $d->setValue("nr", $catnr);
   $d->get(1);

   if($d->next())
   {
    $template=$d->getValue("templatenr");

    if($d->v["nr"]==$this->plugin->config["rootpagenr"]||(!$template&&!$d->v["symlink"]))
     $module="categories";
    else
     $module="pager_categories";

    //main menu
    $menu=Array();

    if($module=="categories"||!$d->v["disabled"])
    {
     if($template||$d->v["symlink"])
     {
      if(isset($_SESSION["jses"]["default"])&&isset($_SESSION["jses"]["default"]["project"])&&$_SESSION["jses"]["default"]["project"]==$_jses["project"]) //default project
       $menu[]=Array(
        "text" => "Show page",
        "desc" => "Show ".$d->v["name"],
        "icon" => Array("view", $d->module),
        "link" => $_jasPath["HTML"]["root"].($_config["useModRewrite"]?($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])."/":"?p=".($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])),
        "rel"  => "external",
        "class" => "show");
      else //jas
       $menu[]=Array(
        "text" => "Show page",
        "desc" => "Show ".$d->v["name"],
        "icon" => Array("view", $d->module),
        "link" => $_jasPath["HTML"]["current"].$_jasPath["project"]["root"]."project/".($_config["useModRewrite"]?($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])."/":"?p=".($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])),
        "rel"  => "external",
        "class" => "show");
     }
     else //root page
     {
      if(isset($_SESSION["jses"]["default"])&&isset($_SESSION["jses"]["default"]["project"])&&$_SESSION["jses"]["default"]["project"]==$_jses["project"]) //default project
       $menu[]=Array(
        "text" => "Show homepage",
        "desc" => "Show ".$d->v["name"],
        "icon" => Array("view", $d->module),
        "link" => $_jasPath["HTML"]["root"],
        "rel"  => "external",
        "class" => "show");
      else //jas
       $menu[]=Array(
        "text" => "Show homepage",
        "desc" => "Show ".$d->v["name"],
        "icon" => Array("view", $d->module),
        "link" => $_jasPath["HTML"]["current"].$_jasPath["project"]["root"]."project/",
        "rel"  => "external",
        "class" => "show");
     }
    }

    if($d->r("add")||$d->r("edit")||$d->r("move")||$d->r("delete"))
     $menu[]="";

    if($d->r("add"))
     $menu[]=Array(
      "text" => "Add page",
      "desc" => "Add ".strtolower($d->title),
      "icon" => Array("add", $d->module),
      "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=pager_categories&amp;catnr=".$d->v["nr"]."&amp;templatenr=".$template."&amp;moduleLock=0",
      "class" => "cms");

    if($d->r("edit"))
     $menu[]=Array(
      "text" => "Edit page",
      "desc" => "Edit ".$d->v["name"],
      "icon" => Array("edit", $d->module),
      "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=".$module."&amp;nr=".$d->v["nr"],
      "class" => "cms");

    if($d->r("move")&&!$this->plugin->config["simple"])
     $menu[]=Array(
      "text" => "Move page",
      "desc" => "Move ".$d->v["name"],
      "icon" => Array("move", $d->module),
      "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=".$module."&amp;nr=".$d->v["nr"],
      "class" => "cms");

    if($d->r("delete"))
     $menu[]=Array(
      "text" => "Delete page",
      "desc" => "Delete ".$d->v["name"],
      "icon" => Array("delete", $d->module),
      "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=".$module."&amp;nr=".$d->v["nr"],
      "class" => "cms");


    if($d->r("edit"))
    {
     $menu[]="";

     $menu[]=Array(
      "text" => "Move page up",
      "desc" => "Move ".$d->v["name"]." up",
      "icon" => Array("orderup", $module),
      "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=up&amp;s=1&amp;m=".$module."&amp;nr=".$d->v["nr"],
      "class" => "cms");

     $menu[]=Array(
      "text" => "Move page down",
      "desc" => "Move ".$d->v["name"]." down",
      "icon" => Array("orderdown", $module),
      "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=down&amp;s=1&amp;m=".$module."&amp;nr=".$d->v["nr"],
      "class" => "cms");

     if($module=="pager_categories")
     {
      if($d->v["disabled"])
       $menu[]=Array(
        "text" => "Enable",
        "desc" => "Enable ".$d->v["name"],
        "icon" => Array("enable", $module),
        "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;disabled=0&amp;s=1&amp;m=".$module."&amp;nr=".$d->v["nr"],
        "class" => "cms");
      else
       $menu[]=Array(
        "text" => "Disable ",
        "desc" => "Disable ".$d->v["name"],
        "icon" => Array("disable", $module),
        "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;disabled=1&amp;s=1&amp;m=".$module."&amp;nr=".$d->v["nr"],
        "class" => "cms");
     }
    }

    $r.=$this->menu("menu1", $menu, false, "left");


    //show info
    foreach($d->check("use", "show") as $k)
    {
     if(!$d->check("properties", "default", $k)&&!$d->check("properties", "hidden", $k)&&isset($d->v[$k])&&$d->v[$k])
     {
      $r.="<div class=\"field\">\n";
      $r.=" <div class=\"title\">".$d->t[$k]."</div>\n";
      $r.=" <div class=\"input\">\n";

      if(is_array($d->v[$k]))
      {
       if(isset($d->reference[$k]))
       {
        if($d->v["symlink"])
         $r.="<a href=\"".$this->plugin->baseURL."?catnr=".$d->reference[$k]["data"]->v("id")."\" title=\"Show ".$d->reference[$k]["data"]->v("title")."\">".$d->reference[$k]["data"]->v("title")."</a>\n";
        else
         $r.=$d->reference[$k]["data"]->v("title");
       }
       else
        $r.=join(", ", $d->v[$k]);
      }
      else if($d->check("type", "boolean", $k))
       $r.=($d->v[$k]?"Yes":"No");
      else
       $r.=$d->v[$k];

      $r.=" </div>\n";
      $r.="</div>\n";
     }
    }
   }

   return $r;
  }


  function template($catnr)
  {
   global $_jasPath, $_config, $_jses;

   $r="";

   $d=Data::cache("categories", "view", "item");
   $d->moduleObject->extendBy("pager_categories");

   $d->setShowFields("name", "urlname", "templatenr", "disabled");
   $d->setValue("nr", $catnr);
   $d->get(1);

   if($d->next())
   {
    if($d->v["nr"]==$this->plugin->config["rootpagenr"])
    {
     $isRoot=true;
     $template=false;
    }
    else if($d->getValue("templatenr"))
    {
     $isRoot=false;
     $template=$d->getValue("templatenr");
    }
    else
     return $r; //container, don't bother to show template

    //header
    if($isRoot) //root templates category
    {
     //get first template nr if no template is selected
     if(!($template=request("templatenr"))||!is_numeric($template))
     {
      $tmp=Data::cache("pager_templates", "view", "all");
      $tmp->setShowFields("nr");
      $tmp->setValue("show_total", 1);

      if($tmp->get())
       $template=$tmp->getValue("nr");

      unset($tmp);
     }

     //main menu
     $menu=Array();

     if(isset($_SESSION["jses"]["default"])&&isset($_SESSION["jses"]["default"]["project"])&&$_SESSION["jses"]["default"]["project"]==$_jses["project"]) //default project
      $menu[]=Array(
       "text" => "Show homepage",
       "desc" => "Show ".$d->v["name"],
       "icon" => Array("view", $d->module),
       "link" => $_jasPath["HTML"]["root"],
       "rel"  => "external",
       "class" => "show");
     else //jas
      $menu[]=Array(
       "text" => "Show homepage",
       "desc" => "Show ".$d->v["name"],
       "icon" => Array("view", $d->module),
       "link" => $_jasPath["HTML"]["current"].$_jasPath["project"]["root"]."project/",
       "rel"  => "external",
       "class" => "show");

     if($d->r("add")||$d->r("edit")||$d->r("move")||$d->r("delete"))
      $menu[]="";

     if($d->r("add"))
      $menu[]=Array(
       "text" => "Add template",
       "icon" => Array("add", "templates"),
       "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=pager_templates&amp;catnr=".$d->v["nr"],
       "class" => "cms");

     if($template)
     {
      if($d->r("edit"))
       $menu[]=Array(
        "text" => "Edit template",
        "icon" => Array("edit", "templates"),
        "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=pager_templates&amp;nr=".$template,
        "class" => "cms");

      if($d->r("move")&&!$this->plugin->config["simple"])
       $menu[]=Array(
        "text" => "Move template",
        "icon" => Array("move", "templates"),
        "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=pager_templates&amp;nr=".$template,
        "class" => "cms");

      if($d->r("delete"))
       $menu[]=Array(
        "text" => "Delete template",
        "icon" => Array("delete", "templates"),
        "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=pager_templates&amp;nr=".$template,
        "class" => "cms");
     }

     if($d->r("add")||$d->r("edit")||$d->r("move")||$d->r("delete"))
      $menu[]="";


     if($d->r("add"))
      $menu[]=Array(
       "text" => "Add page",
       "desc" => "Add ".strtolower($d->title),
       "icon" => Array("add", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=pager_categories&amp;catnr=".$d->v["nr"]."&amp;templatenr=".$template."&amp;moduleLock=0",
       "class" => "cms");

     if($d->r("edit"))
      $menu[]=Array(
       "text" => "Edit root page",
       "desc" => "Edit ".$d->v["name"],
       "icon" => Array("edit", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("move")&&!$this->plugin->config["simple"])
      $menu[]=Array(
       "text" => "Move root page",
       "desc" => "Move ".$d->v["name"],
       "icon" => Array("move", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("delete"))
      $menu[]=Array(
       "text" => "Delete root page",
       "desc" => "Delete ".$d->v["name"],
       "icon" => Array("delete", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=categories&amp;nr=".$d->v["nr"],
       "class" => "cms");


     if($d->r("edit"))
     {
      $menu[]="";

      $menu[]=Array(
       "text" => "Move page up",
       "desc" => "Move ".$d->v["name"]." up",
       "icon" => Array("orderup", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=up&amp;s=1&amp;m=categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

      $menu[]=Array(
       "text" => "Move page down",
       "desc" => "Move ".$d->v["name"]." down",
       "icon" => Array("orderdown", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=down&amp;s=1&amp;m=categories&amp;nr=".$d->v["nr"],
       "class" => "cms");
     }

     $r.=$this->menu("menu1", $menu, $d->title("templatenr"), "left");


     //show template selector
     $r.="<div class=\"field\">";
     $r.="<div class=\"title\"></div>";
     $r.="<div class=\"input\">";

     $f=new Form();
     $r.=$f->start();
     foreach($this->plugin->vars as $k => $v)
      $r.=$f->input("hidden", $k, $v)."\n";

     $r.=$f->dataSelect("pager_templates", "templatenr", $template, "form", false, "onchange=\"this.form.submit()\"", false, true);
     $r.=$f->end();
     unset($f);

     $r.="</div>";
     $r.="</div>";
    }
    else //normal page
    {
     //main menu
     $menu=Array();

     if(isset($_SESSION["jses"]["default"])&&isset($_SESSION["jses"]["default"]["project"])&&$_SESSION["jses"]["default"]["project"]==$_jses["project"]) //default project
      $menu[]=Array(
       "text" => "Show page",
       "desc" => "Show ".$d->v["name"],
       "icon" => Array("view", $d->module),
       "link" => $_jasPath["HTML"]["root"].($_config["useModRewrite"]?($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])."/":"?p=".($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])),
       "rel"  => "external",
       "class" => "show");
     else //jas
      $menu[]=Array(
       "text" => "Show page",
       "desc" => "Show ".$d->v["name"],
       "icon" => Array("view", $d->module),
       "link" => $_jasPath["HTML"]["current"].$_jasPath["project"]["root"]."project/".($_config["useModRewrite"]?($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])."/":"?p=".($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])),
       "rel"  => "external",
       "class" => "show");

     if($d->r("add")||$d->r("edit")||$d->r("move")||$d->r("delete"))
      $menu[]="";

     if($d->r("add"))
      $menu[]=Array(
       "text" => "Add page",
       "desc" => "Add ".strtolower($d->title),
       "icon" => Array("add", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=pager_categories&amp;catnr=".$d->v["nr"]."&amp;templatenr=".$template."&amp;moduleLock=0",
       "class" => "cms");

     if($d->r("edit"))
      $menu[]=Array(
       "text" => "Edit page",
       "desc" => "Edit ".$d->v["name"],
       "icon" => Array("edit", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("move")&&!$this->plugin->config["simple"])
      $menu[]=Array(
       "text" => "Move page",
       "desc" => "Move ".$d->v["name"],
       "icon" => Array("move", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("delete"))
      $menu[]=Array(
       "text" => "Delete page",
       "desc" => "Delete ".$d->v["name"],
       "icon" => Array("delete", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("edit"))
     {
      $menu[]="";

      $menu[]=Array(
       "text" => "Move page up",
       "desc" => "Move ".$d->v["name"]." up",
       "icon" => Array("orderup", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=up&amp;s=1&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

      $menu[]=Array(
       "text" => "Move page down",
       "desc" => "Move ".$d->v["name"]." down",
       "icon" => Array("orderdown", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=down&amp;s=1&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");
     }

     if($d->v["disabled"])
      $menu[]=Array(
       "text" => "Enable",
       "desc" => "Enable ".$d->v["name"],
       "icon" => Array("enable", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;disabled=0&amp;s=1&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");
     else
      $menu[]=Array(
       "text" => "Disable ",
       "desc" => "Disable ".$d->v["name"],
       "icon" => Array("disable", "pager_categories"),
       "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;disabled=1&amp;s=1&amp;m=pager_categories&amp;nr=".$d->v["nr"],
       "class" => "cms");

     $r.=$this->menu("menu1", $menu, $d->title("templatenr"), "left");


     $r.="<div class=\"field\">";
     $r.="<div class=\"title\"></div>";
     $r.="<div class=\"input\">".(isset($d->v["templatenr"]["name"])?$d->v["templatenr"]["name"]:"Unknown");

     $r.="</div>";
     $r.="</div>";
    }

    //display template
    if($template)
    {
     //get markers
     $dm=Data::cache("pager_markers", "view", "all");
     $dm->setValue("templatenr", $template);
     $dm->get();

     //get content
     $dc=Data::cache("pager_content", "view", "all");
     $dc->setShowFields("nr", "catnr", "module", "contentnr", "markernr", "disabled", "displayfor", "cache", "symlink", "disabled");

     $dc->clearValues();
     $dc->setValue("catnr", $this->plugin->config["rootpagenr"]);
     $dc->setOperant("catnr", "OR");
     $dc->setValue("catnr", $catnr, 1);
     $dc->get();

     if($this->plugin->config["simple"]) //simple mode
     {
      $places=Array();

      //find active places
      foreach($dm->listOptions["placing"] as $place)
      {
       $dm->reset();
       while($dm->next())
       {
        if($dm->v["placing"]==$place)
        {
         $places[]=$place;
         break;
        }
       }
      }

      if(!$places) //no active places
      {
       //shown only 1 place
       if(sizeof($dm->listOptions["placing"])>1) //multiple places
        $places=Array($dm->listOptions["placing"][1]); //show 2nd
       else //1 place
        $places=Array($dm->listOptions["placing"][0]);
      }
     }
     else //show all places
      $places=$dm->listOptions["placing"];

     $r.="<div class=\"places\">";
     foreach($places as $place)
     {
      $r.="<table class=\"place ".$place.($isRoot?" rootPlace":"")."\"><tr>\n";
      $r.="<td class=\"name\">\n";

      //show place title
      if($this->plugin->config["simple"]) //simple mode
      {
       //add content link
       $menu=Array();

       if($dc->r("add"))
        $menu[]=Array(
         "text" => "Add content",
         "desc" => "Add ".$dc->title,
         "icon" => Array("add", "pager_content"),
         "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=pager_content&amp;placing=".$place."&amp;templatenr=".$template,
         "class" => "cms");

       $r.=$this->menu("menu2", $menu, ucfirst($place));
      }
      else if($isRoot)
      {
       //add marker link on root page
       $menu=Array();

       if($dc->r("add"))
        $menu[]=Array(
         "text" => "Add marker",
         "desc" => "Add ".$dm->title,
         "icon" => Array("add", $dm->module),
         "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=".$dm->module."&amp;placing=".$place."&amp;templatenr=".$template."&amp;catnr=".$this->plugin->config["rootpagenr"],
         "class" => "cms");

       $r.=$this->menu("menu2", $menu, ucfirst($place));
      }
      else
       $r.=ucfirst($place);

      $r.="</td>\n";

      $columns=0;
      $dm->reset();
      while($dm->next())
       if($dm->v["placing"]==$place)
        $columns++;

      if($columns)
       $columnWidth=(1/$columns)*100; //width in %
      else
       $columnWidth=false;

      //show markers
      $dm->reset();
      while($dm->next())
      {
       if($dm->v["placing"]==$place)
       {
        //marker title and cms links
        $r.="<td class=\"marker\" style=\"".($columnWidth&&$dm->v["small"]?"width: ".($columnWidth/2)."%;":"")."\">";

        //marker menu
        $menu=Array();

        if($this->plugin->config["simple"]) //simple mode
        {
         if($dc->r("add"))
          $menu[]=Array(
           "text" => "Add content",
           "desc" => "Add ".$dc->title,
           "icon" => Array("add", "pager_content"),
           "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=".$dc->module."&amp;placing=".$place."&amp;marker=".urlencode(html_entity_decode($dm->v["marker"]))."&amp;templatenr=".$template,
           "class" => "cms");
        }
        else
        {
         if($dc->r("add"))
          $menu[]=Array(
           "text" => "Add content",
           "desc" => "Add ".$dc->title,
           "icon" => Array("add", "pager_content"),
           "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=".$dc->module."&amp;catnr=".$catnr."&amp;markernr=".$dm->v["nr"],
           "class" => "cms");
        }

        if($isRoot)
        {
         if($dm->r("edit"))
          $menu[]=Array(
           "text" => "Edit marker",
           "desc" => "Edit ".$dm->v["title"],
           "icon" => Array("edit", $dm->module),
           "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=".$dm->module."&amp;nr=".$dm->v["nr"],
           "class" => "cms");

         if($dm->r("move")&&!$this->plugin->config["simple"])
          $menu[]=Array(
           "text" => "Move marker",
           "desc" => "Move ".$dm->v["title"],
           "icon" => Array("move", $dm->module),
           "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=".$dm->module."&amp;nr=".$dm->v["nr"],
           "class" => "cms");

         if($dm->r("delete"))
          $menu[]=Array(
           "text" => "Delete marker",
           "desc" => "Delete ".$dm->v["title"],
           "icon" => Array("delete", $dm->module),
           "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=".$dm->module."&amp;nr=".$dm->v["nr"],
           "class" => "cms");

         if($dc->r("edit")&&$columns>1)
         {
          $menu[]="";

          $menu[]=Array(
           "text" => "Move marker left",
           "desc" => "Move ".$dm->v["title"]." left",
           "icon" => Array("orderleft", $dm->module),
           "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=up&amp;s=1&amp;m=".$dm->module."&amp;nr=".$dm->v["nr"],
           "class" => "cms");

          $menu[]=Array(
           "text" => "Move marker right",
           "desc" => "Move ".$dm->v["title"]." right",
           "icon" => Array("orderright", $dm->module),
           "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=down&amp;s=1&amp;m=".$dm->module."&amp;nr=".$dm->v["nr"],
           "class" => "cms");
         }
        }

        $r.=$this->menu("menu3", $menu, $dm->v["title"]);


        //show content for marker
        $c=0;
        while($dc->next())
        {
         if($dc->v["markernr"]==$dm->v["nr"])
         {
          //try to get content
          $dtmp=Data::cache($dc->v["module"], "view", "item");
          $dtmp->field["plugin"]["properties"]=str_replace("title", "", $dtmp->field["plugin"]["properties"]);

          $dtmp->setValue("nr", $dc->v["contentnr"]);
          $dtmp->get();
          $dtmp->next();

          //content menu
          $menu=Array();

          if(($isRoot&&$dc->v["catnr"]==$this->plugin->config["rootpagenr"])||(!$isRoot&&$dc->v["catnr"]!=$this->plugin->config["rootpagenr"]))
          {
           if($dm->r("edit"))
            $menu[]=Array(
             "text" => "Edit content",
             "desc" => "Edit ".$dtmp->v("title"),
             "icon" => Array("edit", $dc->module),
             "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
             "class" => "cms");

           if($dm->r("move")&&!$this->plugin->config["simple"])
            $menu[]=Array(
             "text" => "Move content",
             "desc" => "Move ".$dtmp->v("title"),
             "icon" => Array("move", $dc->module),
             "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
             "class" => "cms");

           if($dm->r("delete"))
            $menu[]=Array(
             "text" => "Delete content",
             "desc" => "Delete ".$dtmp->v("title"),
             "icon" => Array("delete", $dc->module),
             "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
             "class" => "cms");


           if($dc->r("edit"))
           {
            $menu[]="";

            $menu[]=Array(
             "text" => "Move content up",
             "desc" => "Move ".$dtmp->v("title")." up",
             "icon" => Array("orderup", $dc->module),
             "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=up&amp;s=1&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
             "class" => "cms");

            $menu[]=Array(
             "text" => "Move content down",
             "desc" => "Move ".$dtmp->v("title")." down",
             "icon" => Array("orderdown", $dc->module),
             "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=down&amp;s=1&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
             "class" => "cms");


            if($dc->v["disabled"])
            {
             $menu[]=Array(
              "text" => "Enable",
              "desc" => "Enable ".$dtmp->v("title"),
              "icon" => Array("enable", $dc->module),
              "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;disabled=0&amp;s=1&amp;dontgetdata=1&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
              "class" => "cms");
            }
            else
            {
             $menu[]=Array(
              "text" => "Disable ",
              "desc" => "Disable ".$dtmp->v("title"),
              "icon" => Array("disable", $dc->module),
              "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;disabled=1&amp;s=1&amp;dontgetdata=1&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
              "class" => "cms");
            }

            //cache submenu
            $menu2=Array();

            foreach(Array("Disabled", "Generic", "Anonymous", "User", "Group") as $cache)
            {
             if($dc->v["cache"]!=$cache)
              $menu2[]=Array(
               "text" => $cache." cache",
               "desc" => $cache." cache for ".$dtmp->v("title"),
               "icon" => ($cache=="Generic"?"cache":($cache=="Anonymous"?"cache-anon":("cache-".strtolower($cache)))),
               "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;e=item&amp;cache=".$cache."&amp;s=1&amp;dontgetdata=1&amp;m=".$dc->module."&amp;nr=".$dc->v["nr"],
               "class" => "cache");
             else
              $menu2[]=Array(
               "text" => ($cache=="Disabled"?"Cache disabled":$cache." cache"),
               "icon" => ($cache=="Generic"?"cache":($cache=="Anonymous"?"cache-anon":("cache-".strtolower($cache)))),
               "class" => "cache");
            }

            if($dc->v["cache"]=="Disabled")
             $cache="cache-disabled";
            else if($dc->v["cache"]=="Anonymous")
             $cache="cache-anon";
            else if($dc->v["cache"]=="User")
             $cache="cache-user";
            else if($dc->v["cache"]=="Group")
             $cache="cache-group";
            else
             $cache="cache";

            $menu[]=Array(
             "menu" => $this->menu($cache, $menu2, "Cache"),
             "class" => "cache");
           }
          }


          //additional content information
          $info="<div class=\"up info\">";
          $info.="<b>".$dc->title."</b><br/>\n";

          foreach($dc->check("use", "show") as $field)
           if($field!="data"&&$field!="contentnr")
            $info.=$this->field($dc, $field);

          $info.="<br/><b>".$dtmp->title."</b><br/>\n";

          foreach($dtmp->check("use", "sql") as $field)
           if($field!="r"&&!$dtmp->check("properties", "default", $field))
            $info.=$this->field($dtmp, $field);

          foreach($dtmp->check("properties", "default") as $field)
           $info.=$this->field($dtmp, $field);

          $menu[]=Array(
           "text" => "Info".$info,
           "desc" => "Move ".$dm->v["title"]." right",
           "icon" => Array("info"),
           "class" => "pop");

          unset($info);


          //determine content type
          if($dc->v["catnr"]==$this->plugin->config["rootpagenr"])
          {
           if($dc->v["disabled"])
            $type="rootpage-disabled";
           else if($dc->v["symlink"])
            $type="rootpage-symlink";
           else
            $type="rootpage";
          }
          else if($dc->v["disabled"])
           $type="normalpage-disabled";
          else if($dc->v["symlink"])
           $type="normalpage-symlink";
          else
           $type="normalpage";

          $r.=$this->menu($type, $menu, $dtmp->v("title")." (".ucfirst(str_replace("_", " ", $dtmp->plugin->name)).")", "content");
         }
        }

        $r.="</td>\n";
       }
      }

      $r.="</tr></table>\n";
     }
     $r.="</div>\n";
    }
   }

   return $r;
  }


  function content($catnr)
  {
   $r="";

   $d=Data::cache("categories", "view", "item");
   $d->moduleObject->extendBy("pager_categories");

   $d->setShowFields("module");
   $d->setValue("nr", $catnr);
   $d->get();

   while($d->next())
   {
    foreach($d->v["module"] as $module)
     $r.=$this->contentList($module, "view", "list", $catnr);

    $r.=$this->contentList("categories", "view", "list", $catnr);
   }

   $r.="</div>";

   return $r;
  }


  function contentList($module, $action, $extra, $catnr)
  {
   global $_jasPath;

   $d=Data::cache($module, $action, $extra);
   $d->setShowFields(array_merge(Array("nr", "catnr"), $d->check("properties", "title")));
   $d->setValue("catnr", $catnr);

   if(!$d->get())
    $r=$d->e->show();
   else
   {
    $r="<div class=\"contentList ".$d->module."\">";

    //category menu
    $menu=Array();

    if($d->r("add"))
     $menu[]=Array(
      "text" => "Add ".strtolower($d->title),
      "icon" => Array("add", $d->module),
      "link" => $this->plugin->url()."cms=1&amp;a=add&amp;m=".$d->module."&amp;catnr=".$d->getValue("catnr"),
      "class" => "cms");

    if($d->r("edit"))
     $menu[]=Array(
      "text" => "Edit category",
      "icon" => Array("edit", "categories"),
      "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=categories&amp;nr=".$d->getValue("catnr"),
      "class" => "cms");

    if($d->r("move")&&!$this->plugin->config["simple"])
     $menu[]=Array(
      "text" => "Move category",
      "icon" => Array("move", "categories"),
      "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=categories&amp;nr=".$d->getValue("catnr"),
      "class" => "cms");

    if($d->r("delete"))
     $menu[]=Array(
      "text" => "Delete category",
      "icon" => Array("delete", $d->module),
      "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=categories&amp;nr=".$d->getValue("catnr"),
      "class" => "cms");


    if($d->r("edit"))
    {
     $menu[]="";

     $menu[]=Array(
      "text" => "Move category up",
      "icon" => Array("orderup", "categories"),
      "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=up&amp;s=1&amp;m=categories&amp;nr=".$d->getValue("catnr"),
      "class" => "cms");

     $menu[]=Array(
      "text" => "Move category down",
      "icon" => Array("orderdown", "categories"),
      "link" => $this->plugin->url()."cms=1&amp;a=order&amp;e=down&amp;s=1&amp;m=categories&amp;nr=".$d->getValue("catnr"),
      "class" => "cms");
    }

    $r.=$this->menu("menu1", $menu, false, "left");


    //content
    $r.="<h2>".$d->title." (".$d->vmax.")</h2>\n";

    while($d->next())
    {
     $r.="<div class=\"contentItem ".($d->vc%2?"odd":"even")."\">";
     $r.="<a href=\"".$_jasPath["HTML"]["current"]."data/browse/?m=".$d->module."&amp;nr=".$d->v["nr"]."&amp;catnr=".($d->module=="categories"?$d->v["nr"]:(isset($d->v["catnr"])?$d->v["catnr"]:""))."\" title=\"".$d->v("title")."\">".$d->v("title")."</a>\n";

     //content menu
     $menu=Array();

     if($d->r("edit"))
      $menu[]=Array(
       "text" => "Edit ".strtolower($d->title),
       "desc" => "Edit ".$d->v("title"),
       "icon" => Array("edit", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=edit&amp;m=".$d->module."&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("move")&&!$this->plugin->config["simple"])
      $menu[]=Array(
       "text" => "Move ".strtolower($d->title),
       "desc" => "Move ".$d->v("title"),
       "icon" => Array("edit", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=move&amp;m=".$d->module."&amp;nr=".$d->v["nr"],
       "class" => "cms");

     if($d->r("delete"))
      $menu[]=Array(
       "text" => "Delete ".strtolower($d->title),
       "desc" => "Delete ".$d->v("title"),
       "icon" => Array("edit", $d->module),
       "link" => $this->plugin->url()."cms=1&amp;a=delete&amp;m=".$d->module."&amp;nr=".$d->v["nr"],
       "class" => "cms");

     $r.=$this->menu("menu2", $menu, "", "left");

     $r.="</div>\n";
    }

    $r.="<br class=\"clear\"/>\n";
    $r.="</div>\n";
   }

   return $r;
  }


  function cms($d, $vars=false)
  {
   $r="";

   $cms=new CMS();
   foreach($this->plugin->vars as $k => $v)
    $cms->addVar($k, $v);

   if($vars&&is_array($vars))
    foreach($vars as $k => $v)
     $cms->addVar($k, $v);

   $cms->addVar("cms", "true");

   $d->dataformat="form";

   if($d->module=="pager_content"&&$d->action!="order")
   {
    if($d->action=="copy"||$d->action=="edit"||$d->action=="move"||$d->action=="delete") //get module
    {
     $d->get();

     if(request("dontgetdata")) //use submitted value
     {
      $d->setRequestValue("cache");
      $d->setRequestValue("markernr");
      $d->setRequestValue("disabled");
      $d->setRequestValue("displayfor");
      $d->setRequestValue("fixscript");
      $d->setRequestValue("fixstyle");
      $d->setRequestValue("fiximages");

      if($this->plugin->config["simple"]) //simple mode
      {
       $d->setRequestValue("templatenr");
       $d->setRequestValue("marker");
       $d->setRequestValue("placing");
      }

      if($d->getValue("symlink")) //use potential new contentnr for symlinks
       $d->setRequestValue("contentnr");
     }

     if($d->action=="move")
      $d->setRequestValue("newcatnr");
    }

    if(!$d->getValue("symlink")) //don't show content pick list for non symlinks
    {
     $d->field["contentnr"]["properties"].=";hidden";

     if($d->getValue("module"))
     {
      $dc=Data::cache($d->getValue("module"), false, false, Array("request" => "content", "format" => "form"));
      $dc->field["plugin"]["properties"].=";hidden";

      if($d->action=="add")
      {
       $d->field["catnr"]["properties"].=";hidden";

       if($dc->check("valuetype", "default", "catnr")) //use pager_content
        $dc->setValue("catnr", $d->getValue("catnr"));
       else //use module
        $d->setValue("catnr", $dc->getValue("catnr"));
      }
      else if($d->action=="move")
      {
       $d->field["newcatnr"]["properties"].=";control";
       $d->setValue("newcatnr", $dc->getValue("newcatnr"));
      }
     }
    }

    if($d->action=="edit"||$d->action=="move"||$d->action=="delete")
    {
     $d->field["module"]["properties"].=";disabled";
     $d->field["symlink"]["properties"].=";disabled";

     $d->unsetValue("ordernr");

     if(isset($dc))
      $dc->setValue("nr", $d->getValue("contentnr"));
    }
   }

   if(request("s"))
   {
    if(isset($dc)) //module+non symlink
    {
     $d->field["contentnr"]["min"]=0; //ensure d errorCheck doesn't fail on contentnr (set after dc is submitted)

     if(!$d->errorCheck(true)) //check d
     {
      $cms->e->add($d->e);
      $d->e->clear();

      if(!$dc->errorCheck(true)) //also get potential dc errors
      {
       $cms->e->add($dc->e);
       $dc->e->clear();
      }
     }
     else if(!$dc->submit())
     {
      $cms->e->add($dc->e);
      $dc->e->clear();
     }
     else
     {
      if($dc->action=="add")
      {
       $d->field["contentnr"]["min"]=1;
       $d->setValue("contentnr", $dc->getValue("nr"));
      }

      if(!$d->submit())
      {
       $cms->e->add($d->e);
       $d->e->clear();

       if($dc->action=="add") //clean up previously inserted
       {
        $dc->setAction("delete");
        $dc->submit();
        $dc->setAction("add");
       }
      }
      else
      {
       //$this->e->add($d->e);
       $this->e->add($dc->e);
       $d->e->clear();
       $dc->e->clear();

       if(!request("cmsContinue"))//oh you want more ?
        return false; //done
      }
     }
    }
    else //normal module / pager_content with symlink
    {
     if(!$d->submit())
     {
      $cms->e->add($d->e);
      $d->e->clear();
     }
     else
     {
      $this->e->add($d->e);
      $d->e->clear();

      if(!request("cmsContinue"))//oh you want more ?
       return false; //done
     }
    }
   }

   //handle reference manipulation
   if(isset($dc))
   {
    if($cms->referenceManipulation)
    {
     $cms->setData($d);
     $r.=$cms->formReferenceManipulation();

     $cms->setData($dc);
     $r.=$cms->formReferenceManipulation();
    }
   }
   else //normal module
   {
    if($cms->referenceManipulation)
    {
     $cms->setData($d);
     $r.=$cms->formReferenceManipulation();
    }
   }

   $r.="<h2>".$d->title."</h2>\n";
   $r.=$cms->formShowErrors();

   $r.=$cms->formStart(".");

   if(isset($dc))
   {
    $cms->setData($d);
    $r.=$cms->formShowFields();

    $cms->vars=Array(); //clear vars, already printed by previous formShowFields

    $cms->setData($dc);
    $r.=$cms->formShowFields();

    $cms->setData($d);
   }
   else //normal module / pager_content with symlink
   {
    $cms->setData($d);
    $r.=$cms->formShowFields();
   }

   $r.=$cms->formShowSubmit();
   $r.=$cms->formShowCancel();
   $r.=$cms->formEnd();

   return $r;
  }

  function field(&$d, $field)
  {
   if(!isset($d->v[$field]))
    return false;

   $title=$d->title($field);
   $value=$d->v[$field];
   $type=$d->field[$field]["type"];

   if(isset($d->reference[$field]))
   {
    $dr=Data::cache($d->reference[$field]["module"], "view", "item");
    $tfield=$dr->check("properties", "title");
    $tfield=$tfield[0];

    $dr->setValue("nr", $value);
    $dr->setShowFields($tfield);
    $dr->get();

    while($dr->next())
    {
     $value=$dr->v[$tfield];
     $type=$dr->field[$tfield]["type"];
    }
   }

   switch($type)
   {
    case "boolean":
     $value=($value?"Yes":"No");
    break;

    default:
    break;
   }

   if(is_array($value))
    $value=implode(", ", $value);

   return "<div class=\"field\"><div class=\"title\">".$title."</div><div class=\"input\">".$value."</div></div>\n";
  }

  function menu($icon, $entries, $title=false, $class=false)
  {
   if(!is_array($icon))
    $icon=Array($icon);

   $r="";

   $r.="<div class=\"pop menu".($class?" ".$class:"")."\">".icon($icon[0], (isset($icon[1])?$icon[1]:false)).($title?" ".$title:"")."\n";
   $r.=" <ul class=\"up\">\n";
   foreach($entries as $entry)
   {
    if(!is_array($entry))
    {
     if($entry)
      $r.="  <li>".$entry;
     else
      $r.="  <li class=\"empty\">&nbsp; ";
    }
    else
    {
     $r.="  <li".(isset($entry["class"])?" class=\"".$entry["class"]."\"":"").">";

     if(isset($entry["menu"])) //sub menu entry
      $r.=$entry["menu"];
     else //normal menu entry
     {
      if(isset($entry["text"]))
       $text=$entry["text"];
      else
       $text="&nbsp; ";

      if(isset($entry["icon"]))
      {
       if(!is_array($entry["icon"]))
        $icon=icon($entry["icon"]);
       else
        $icon=icon($entry["icon"][0], (isset($entry["icon"][1])?$entry["icon"][1]:false));
      }
      else
       $icon=false;

      if(isset($entry["link"]))
      {
       $r.="<a href=\"".$entry["link"]."\"".(isset($entry["rel"])?" rel=\"".$entry["rel"]."\"":"")." title=\"".(isset($entry["desc"])?$entry["desc"]:$entry["text"])."\">";
       $r.=($icon?$icon." ":"").$text;
       $r.="</a>";
      }
      else
       $r.=($icon?$icon." ":"").$text;
     }
    }

    $r.="</li>\n";
   }

   $r.=" </ul>\n";
   $r.="</div>\n";

   return $r;
  }

  function clearCache($type)
  {
   global $_path;

   $fs=new FS();

   switch($type)
   {
    case "page":
    {
     foreach($fs->files($_path["project"]["cache"]) as $file)
      if(preg_match("/\.html$/i", $file))
       $fs->delete($file);

     break;
    }
    case "image":
    {
     foreach($fs->files($_path["project"]["cache"]) as $file)
      if(preg_match("/\.(png|gif|jpg|jpeg)$/i", $file))
       $fs->delete($file);

     break;
    }
    case "all":
    {
     foreach($fs->files($_path["project"]["cache"]) as $file)
      $fs->delete($file);

     break;
    }
    default:
    {
     $this->e->add("Clear \"".$type."\" cache not supported", "hack");
     return false;
    }
   }

   //errors during removal?
   if($fs->e->any("fatal"))
   {
    $this->e->add($fs->e);
    return false;
   }

   $this->e->add(ucfirst($type)." cache cleared", "note");
   return true;
  }

 }
?>
