<?

 /*****************************************************************************
 * Filename: include/plugins/pager/pager.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."layout.php");
 require_once($_path["include"]."template.php");

 class pager extends BasePlugin
 {
  function main()
  {
   global $_path, $_config, $_benchmark;

   if($_benchmark) $_benchmark->start("Page");

   if(!isset($this->config)||!$this->config)
    $page="-1";
   else if($page=request("p")) //can be array or page nr/name
   {
    if(is_array($page)) //rewrite, use first argument
    {
     if(isset($page[0]))
      $page=$page[0];
     else
      $page=false;
    }
   }

   if(!$page)
   {
    if($this->config["startpagenr"])
     $page=$this->config["startpagenr"];
    else
     $page=$this->config["rootpagenr"];
   }


   if($_benchmark) $_benchmark->start("Get page info");

   //get page
   $d=Data::cache("pager_categories", "view", "all");
   $d->setShowFields("name", "urlname", "templatenr", "doctype", "javascriptglobal", "javascript", "stylesheet", "disabled", "updated", "symlink", "pagenr");
   $d->setValue("show_total", 1);
   $d->clearValues();

   if(preg_match("/^[0-9]+$/", $page)) //page number
    $d->setValue("nr", $page);
   else //page name
    $d->setValue("urlname", $page);

   if(!$d->get(1))
    $this->e->add("Unable to get the page", "error", "Sorry this page could not be loaded");
   else if(!$d->next())
    $this->e->add("Page not found", "error", "Sorry this page could not be found");
   else if($d->v["disabled"])
    $this->e->add("Page not available", "error", "Sorry this page is not available at the moment");
   else
   {
    if($d->v["symlink"])
    {
     $d->setValue("show_total", 1);
     $d->clearValues();
     $d->setValue("nr", $d->v["pagenr"]);

     if(!$d->get(1))
      $this->e->add("Unable to get the page", "error", "Sorry this page could not be loaded");
     else if(!$d->next())
      $this->e->add("Page not found", "error", "Sorry this page could not be found");
     else if($d->v["disabled"])
      $this->e->add("Page not available", "error", "Sorry this page is not available at the moment");
    }

    if(!$d->v["templatenr"])
     $this->e->add("No template specified", "error", "Page should have a template specified to be displayed");
   }

   if($this->e->any("error"))
    return false;

   $this->lastUpdate=DT::dateConvert($d->v["updated"], $d->field["updated"]["format"][$d->dataformat]["datetimeformat"]); //page last updated
   $this->pageID=$d->v["nr"]; //get page number;
   $this->pageName=$d->v["urlname"]; //get page name (will be id if not set)

   //get page markers
   $dm=Data::cache("pager_markers", "view", "all");
   $dm->setValue("templatenr", $d->v["templatenr"]["nr"]);
   $dm->setShowFields("nr", "marker", "clean");

   if(!$dm->get(1))
   {
    $this->e->add("Unable to get the page markers", "error", "Sorry this page could not be loaded");
    return false;
   }

   $markers=Array();
   $cleanMarkers=Array();
   while($dm->next())
   {
    $markers[$dm->v["nr"]]=$dm->v["marker"];

    if($dm->v["clean"])
     $cleanMarkers[]=$dm->v["marker"];
   }

   unset($dm);

   if(!$markers)
   {
    $this->e->add("No page markers found", "error", "Sorry this page could not be loaded");
    return false;
   }

   if($_benchmark) $_benchmark->stop("Get page info");



   if($_benchmark) $_benchmark->start("Setup template");

   $this->template=Array();
   $this->template["xml"]=false;
   $this->template["title"]=false;
   $this->template["jasscript"]=Array();
   $this->template["javascript"]=Array();
   $this->template["stylesheet"]=Array();
   $this->template["doctype"]=Array();

   $cache=new Cache("file");
   if($cache->e->any("fatal")) //caching prolly disabled
    $cache=false;
   else
   {
    $cacheKey=$this->config["plugin"]."_templage_".$d->v["templatenr"]["nr"].".html";

    //template record updated
    $cacheTime=DT::dateConvert($d->v["templatenr"]["updated"], $d->reference["templatenr"]["data"]->field["updated"]["format"][$d->reference["templatenr"]["data"]->dataformat]["datetimeformat"]);

    if($this->lastUpdate<$cacheTime) //template updated after page entry?
     $this->lastUpdate=$cacheTime;

    //template updated
    $time=FS::getTime($_path["project"]["templates"].$d->v["templatenr"]["template"], "modified");

    if($time>$cacheTime)
     $cacheTime=$time;

    if(($time=$cache->time($cacheKey))!==false) //exists
    {
    if($this->lastUpdate<$time) //template file updated after page entry?
     $this->lastUpdate=$time;

     if($cacheTime==$time) //cache up-to-date
     {
      if(($this->template["xml"]=$cache->read($cacheKey))!==false) //no error, use the cache
      {
       $this->template["xml"]=new Template($this->template["xml"]);
       $this->template["xml"]->filename=$cacheKey;
      }
      //else //should prolly remove it... then again... this is impossible
     }
     else //cache out of date
      $cache->remove($cacheKey); //remove the cache and re-cache after template is succesfully parsed
    }
    //else //doesn't exist yet, cache it at after template is succesfully parsed
   }

   if(!$this->template["xml"])
   {
    $this->template["xml"]=new Template($_path["project"]["templates"].$d->v["templatenr"]["template"]);

    if($this->template["xml"]->e->any())
    {
     $this->e->add($this->template["xml"]->e);
     return false;
    }

    $reparse=false;
    if($d->v["templatenr"]["fixscript"])
    {
     $this->template["xml"]->xml=preg_replace("!(<script.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $this->template["xml"]->xml);
     $reparse=true;
    }

    if($d->v["templatenr"]["fixstyle"])
    {
     $this->template["xml"]->xml=preg_replace("!(<link.*css.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $this->template["xml"]->xml);
     $reparse=true;
    }

    if($d->v["templatenr"]["fiximages"])
    {
     $this->template["xml"]->xml=preg_replace("!(<link.*ico.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $this->template["xml"]->xml);
     $this->template["xml"]->xml=preg_replace("!(<img.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $this->template["xml"]->xml);
     $reparse=true;
    }

    if($reparse)
     $this->template["xml"]->reparse();

    unset($reparse);

    //add/update doctype
    list($version, $mode)=explode(" - ", $d->v["templatenr"]["doctype"]);

    if($this->template["xml"]->getKey("DOCTYPE")===false)
     $this->template["xml"]->addBefore("html", Layout::doctype($version, $mode));
    else
     $this->template["xml"]->set("DOCTYPE", substr(Layout::doctype($version, $mode), 0, -1), true, true); //stripping additional linebreak

    //add/update title
    if($this->template["xml"]->getKey("html->head->title")===false)
    {
     $this->template["xml"]->addInsideAtBegin("html->head", "<title>".$d->v["templatenr"]["title"]."</title>\n");
     $this->template["xml"]->reparse();
    }
    else
     $this->template["xml"]->set("html->head->title", $d->v["templatenr"]["title"], true, false);

    //add template javascript
    foreach($d->v["templatenr"]["javascriptglobal"] as $file)
    {
     $tmp="  <script type=\"text/javascript\" src=\"".$_path["script"].$file."\"></script>\n";
     $this->template["xml"]->addInsideAtEnd("html->head", $tmp);
    }
    foreach($d->v["templatenr"]["javascript"] as $file)
    {
     $tmp="  <script type=\"text/javascript\" src=\"".$_path["project"]["script"].$file."\"></script>\n";
     $this->template["xml"]->addInsideAtEnd("html->head", $tmp);
    }

    //add template style sheets
    foreach($d->v["templatenr"]["stylesheet"] as $file)
    {
     $tmp="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["project"]["style"].$file."\"/>\n";
     $this->template["xml"]->addInsideAtEnd("html->head", $tmp);
    }

    //clean markers
    foreach($cleanMarkers as $marker)
     $this->template["xml"]->set(html_entity_decode($marker), "", true, false); //empty tag

    //cache template
    if(!$this->template["xml"]->e->any()&&$cache&&$cacheKey&&$cacheTime)
    {
     if(!$cache->save($cacheKey, $this->template["xml"]->output(), $cacheTime))
     {
      $this->e->add($cache->e);
      return false;
     }
    }
   }


   //page specific doctype
   if($d->v["doctype"]&&$d->v["doctype"]!="Use template") //custom doctype
   {
    list($version, $mode)=explode(" - ", $d->v["doctype"]);
    $this->setDoctype($version, $mode);
   }

   //page specific title
   if($d->v["templatenr"]["title"])
    $this->setTitle($d->v["templatenr"]["title"]." - ".$d->v["name"]);
   else
    $this->setTitle($d->v["name"]);

   if($d->v["templatenr"]["jasscript"])
    $this->jasScript(true);
   else
    $this->jasScript(true);

   //page specific javascript
   foreach($d->v["javascriptglobal"] as $file)
    $this->addJavaScript($_path["script"].$file);

   foreach($d->v["javascript"] as $file)
    $this->addJavaScript($_path["project"]["script"].$file);

   //page specific stylesheets
   foreach($d->v["stylesheet"] as $file)
    $this->addStyleSheet($_path["project"]["style"].$file);

   if($_benchmark) $_benchmark->stop("Setup template");



   if($_benchmark) $_benchmark->start("Main plugin logic");

   //page content, used later for rendering
   $contentCounter=0;
   $this->content=Array();

   $dc=Data::cache("pager_content", "view", "all");
   $dc->setShowFields("markernr", "module", "contentnr", "cache", "displayfor", "fixscript", "fixstyle", "fiximages");
   $dc->sortOrder("markernr", "ordernr");

   $dc->setValue("catnr", $this->config["rootpagenr"], 0);
   $dc->setValue("disabled", false, 0);
   $dc->setOperant("catnr", "OR");
   $dc->setValue("catnr", $this->pageID, 1);
   $dc->setValue("disabled", false, 1);

   if(!$dc->get())
   {
    $this->e->add($dc->e);
    return false;
   }

   while($dc->next())
   {
    if(!isset($markers[$dc->v["markernr"]])) //not a marker for this template (must be gathered from rootpagenr)
     continue;

    if($_benchmark) $_benchmark->start($dc->v["module"]);

    $this->content[$contentCounter]["error"]=new Error();

    $this->content[$contentCounter]["nr"]=$dc->v["nr"];
    $this->content[$contentCounter]["displayfor"]=$dc->v["displayfor"];
    $this->content[$contentCounter]["cache"]=Array("method" => $dc->v["cache"], "key" => false, "time" => false);

    $this->content[$contentCounter]["module"]=$dc->v["module"];
    $this->content[$contentCounter]["marker"]=html_entity_decode($markers[$dc->v["markernr"]]);

    $this->content[$contentCounter]["fixscript"]=$dc->v["fixscript"];
    $this->content[$contentCounter]["fixstyle"]=$dc->v["fixstyle"];
    $this->content[$contentCounter]["fiximages"]=$dc->v["fiximages"];


    $plugin=preg_replace("/_pluginConfigs$/", "", $dc->v["module"]);
    $this->content[$contentCounter]["plugin"]=new Plugin($plugin);

    $this->content[$contentCounter]["plugin"]->config($dc->v["contentnr"], true);
    $this->content[$contentCounter]["plugin"]->addVar("p", $this->pageName);
    $this->content[$contentCounter]["plugin"]->pager=$this; //set link to pager

    if(!$this->content[$contentCounter]["plugin"]->main()) //execute main logic
     $this->content[$contentCounter]["error"]->add($this->content[$contentCounter]["plugin"]->e);

    if($_benchmark) $_benchmark->stop($this->content[$contentCounter]["module"]);

    $contentCounter++;
   }

   if($_benchmark) $_benchmark->stop("Main plugin logic");

   return true;
  }


  function output()
  {
   global $_path, $_config, $_benchmark, $_jses;

   if($this->e->any())
    return $this->error();

   $cache=new Cache("file");
   if($cache->e->any("fatal")) //caching prolly disabled
    $cache=false;

   if($_benchmark) $_benchmark->start("Render template");

   //doctype
   if($this->template["doctype"]) //custom doctype
    $this->template["xml"]->set("DOCTYPE", substr(Layout::doctype($this->template["doctype"]["version"], $this->template["doctype"]["mode"]), 0, -1), true, true); //stripping additional linebreak

   //title
   if($this->template["title"]) //custom title
    $this->template["xml"]->set("html->head->title", $this->template["title"], true, false);

    //add JAS script
   if($this->template["jasscript"])
    $this->template["xml"]->addInsideAtBegin("html->head", "\n <script type=\"text/javascript\">\n  var browser=\"".$_jses["browser"]."\";\n  var path=\"".$_path["HTML"]["prefix"].$_path["project"]["root"]."\";\n  var rootpath=\"".$_path["HTML"]["prefix"].$_path["root"]."\";\n </script>\n");

   //additional javascript
   foreach($this->template["javascript"] as $file)
   {
    $tmp="  <script type=\"text/javascript\" src=\"".$file."\"></script>\n";
    $this->template["xml"]->addInsideAtEnd("html->head", $tmp);
   }

   //additional stylesheet
   foreach($this->template["stylesheet"] as $file)
   {
    $tmp="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$file."\"/>\n";
    $this->template["xml"]->addInsideAtEnd("html->head", $tmp);
   }

   //set page last updated & generated
   $this->template["xml"]->exactPath=false;
   if($this->template["xml"]->getKey("LASTUPDATE")!==false)
    $this->template["xml"]->set("LASTUPDATE", date($_config["datetimeformat"], $this->lastUpdate), true, true);
   if($this->template["xml"]->getKey("GENERATED")!==false)
    $this->template["xml"]->set("GENERATED", date($_config["datetimeformat"]), true, true);
   $this->template["xml"]->exactPath=true;


   //fix links, favicons, style sheets and javascript tags for rewrite prefix
   if($_config["useModRewrite"]&&$_path["HTML"]["prefix"])
   {
    //don't fix paths that start with a / or contain a : (http: / mailto:)
    $this->template["xml"]->xml=preg_replace("!(<[^>]+href=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $this->template["xml"]->xml);
    $this->template["xml"]->xml=preg_replace("!(<[^>]+src=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $this->template["xml"]->xml);
    $this->template["xml"]->reparse();
   }

   //add _benchmark hook
   if($_benchmark&&$this->plugin->config["benchmark"])
    if($this->template["xml"]->getKey("html->body")!==false)
     $this->template["xml"]->addInsideAtEnd("html->body", "#_benchmark#");

   //add _contentmarker hooks
   $l=sizeof($this->content);
   for($contentCounter=0;$contentCounter<$l;$contentCounter++)
    $this->template["xml"]->addInsideAtEnd($this->content[$contentCounter]["marker"], "#_contentmarker".$contentCounter."#");

   if($_benchmark) $_benchmark->stop("Render template");



   if($_benchmark) $_benchmark->start("Render content");

   #render all contents
   for($contentCounter=0;$contentCounter<$l;$contentCounter++)
   {
    //display for anonymous, user or root?
    if($this->content[$contentCounter]["displayfor"]!="All")
    {
     //not logged in?
     if($this->content[$contentCounter]["displayfor"]=="Anonymous"&&$_config["usernr"]!=$_config["anonnr"])
      continue;

     //logged in?
     if($this->content[$contentCounter]["displayfor"]=="Logged in"&&$_config["usernr"]==$_config["anonnr"])
      continue;

     //are we root?
     if($this->content[$contentCounter]["displayfor"]=="Root"&&$_config["usernr"]!=$_config["rootnr"])
      continue;
    }


    if($_benchmark) $_benchmark->start($this->content[$contentCounter]["module"]);

    $content=false;
    $cacheKey=false;
    $cacheTime=false;
    if($cache&&$this->content[$contentCounter]["cache"]["method"]!="Disabled")
    {
     switch($this->content[$contentCounter]["cache"]["method"])
     {
      case "Generic":
      {
       $cacheKey=""; //use generic cache
       break;
      }
      case "Anonymous":
      {
       if($_config["usernr"]==$_config["anonnr"])
        $cacheKey="anon_"; //use anonymous cache
       else
        $cacheKey=""; //use generic cache

       break;
      }
      case "Group":
      {
       $gids=$_config["groupnrs"];
       sort($gids); //sort groups so users with same groups use same cache
       $cacheKey="gids_".implode("-", $gids)."_"; //use groups cache
       break;
      }
      case "User":
      {
       $cacheKey="uid_".$_config["usernr"]."_"; //use user specific cache
       break;
      }
      default:
      {
       if($this->content[$contentCounter]["cache"]["method"]=="true"||$this->content[$contentCounter]["cache"]["method"]=="false") //old style caching
        $this->e->add("Unknown cache method: \"".$this->content[$contentCounter]["cache"]["method"]."\"", "error", "Please upgrade your database by running the following queries:
         ALTER TABLE `pager_content` CHANGE `cache` `cache` ENUM('true', 'false', 'Disabled', 'Generic', 'Anonymous', 'Group', 'User');
         UPDATE `pager_content` SET `cache`='Disabled' WHERE `cache`='false';
         UPDATE `pager_content` SET `cache`='Generic' WHERE `cache`='true';
         ALTER TABLE `pager_content` CHANGE `cache` `cache` ENUM('Disabled', 'Generic', 'Anonymous', 'Group', 'User');
        ");
       else
        $this->e->add("Unknown cache method: \"".$this->content[$contentCounter]["cache"]["method"]."\"", "error", "Please make sure all cache methods are properly implemented in the pager plugin");

       $cacheKey=""; //use generic cache
       break;
      }
     }

     $configUpdated=DT::dateConvert($this->content[$contentCounter]["plugin"]->config["updated"], $_config["datetimeformat"]);

     if($this->content[$contentCounter]["plugin"]->callExists("cache")) //custom cache implementation
     {
      $cacheKey=$this->content[$contentCounter]["plugin"]->name."_".$cacheKey; //added plugin name prefix

      $contentUpdated=false;
      if($this->content[$contentCounter]["plugin"]->class->cache($contentUpdated, $cacheKey))
      {
       if($contentUpdated>$configUpdated)
        $cacheTime=$contentUpdated;
       else
        $cacheTime=$configUpdated;
      }
      else //cache call failed, don't cache
       $cacheKey=false;
     }
     else //standard pager based cache implementation
     {
      $cacheTime=$configUpdated;
      $cacheKey=$this->config["plugin"]."_".$cacheKey."content_".$this->content[$contentCounter]["nr"].".html"; //generic content cache key name
     }

     if($this->lastUpdate<$cacheTime) //config/content updated after page entry?
      $this->lastUpdate=$cacheTime;

     if($cacheKey)
     {
      $this->content[$contentCounter]["cache"]["key"]=$cacheKey;
      $this->content[$contentCounter]["cache"]["time"]=$cacheTime;

      if(($time=$cache->time($cacheKey))!==false) //exists
      {
       if($this->lastUpdate<$time) //content template file updated after page entry?
        $this->lastUpdate=$time;

       if($cacheTime==$time) //cache up-to-date
        $content=$cache->read($cacheKey); //no error, use the cache
       else //cache out of date
        $cache->remove($cacheKey); //remove the cache and re-cache after template is succesfully parsed
      }
      //else //doesn't exist yet, cache it at after template is succesfully parsed
     }
    }

    if(!$content) //no cache/error getting cache
    {
     if($this->content[$contentCounter]["plugin"]->config)
      $content="\n<a id=\"".str_replace(" ", "_", $this->content[$contentCounter]["plugin"]->config["key"])."\"></a>\n";

     $content.="<div class=\"".$this->content[$contentCounter]["plugin"]->name."\">\n";

     //generate content
     if(($output=$this->content[$contentCounter]["plugin"]->output())===false) //failure generating content, won't be cached
     {
      $content.=$this->content[$contentCounter]["plugin"]->e->show();
      $content.="\n</div>";
     }
     else //successfully generate content
     {
      //change path
      if($this->content[$contentCounter]["fixscript"])
       $output=preg_replace("!(<script.* src=)(['\"])([^'\":]+)!i", "\\1\\2".$_path["project"]["templates"]."\\3", $output);

      if($this->content[$contentCounter]["fixstyle"])
       $output=preg_replace("!(<link.*css.* href=)(['\"])([^'\":]+)!i", "\\1\\2".$_path["project"]["templates"]."\\3", $output);

      if($this->content[$contentCounter]["fiximages"])
       $output=preg_replace("!(<img.* src=)(['\"])([^'\":]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $output);

      $content.=$output;
      $content.="\n</div>";

      //save cached content
      if($cache&&$this->content[$contentCounter]["cache"]["key"])
       if(!$cache->save($this->content[$contentCounter]["cache"]["key"], $content, $this->content[$contentCounter]["cache"]["time"]))
        $content=$this->content[$contentCounter]["error"]->show().$content;
     }
    }

    if($_config["useModRewrite"]&&$_path["HTML"]["prefix"]) //fix links, favicons, style sheets and javascript tags for rewrite prefix
    {
     //don't fix paths that start with a / or contain a : (http: / mailto:)
     $content=preg_replace("!(<[^>]+href=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $content);
     $content=preg_replace("!(<[^>]+src=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $content);
    }

    //replace _contentmarker hook with content
    $this->template["xml"]->xml=str_replace("#_contentmarker".$contentCounter."#", $content, $this->template["xml"]->xml);

    if($_benchmark) $_benchmark->stop($this->content[$contentCounter]["module"]);
   }

   //clean up unused _contentmarkers
   for($contentCounter=0;$contentCounter<$l;$contentCounter++)
    $this->template["xml"]->xml=str_replace("#_contentmarker".$contentCounter."#", "", $this->template["xml"]->xml);

   if($_benchmark) $_benchmark->stop("Render content");

   if($_benchmark) $_benchmark->stop("Page");

   if($_benchmark&&$this->plugin->config["benchmark"])
    $this->template["xml"]->xml=str_replace("#_benchmark#", $_benchmark->show(), $this->template["xml"]->xml);

   //we might need to reparse to catch errors... then agian, we probably already caught them all
   //$this->template["xml"]->reparse();

   if($this->template["xml"]->e->any())
   {
    $this->e->add($this->template["xml"]->e);
    return $this->error();
   }

   return $this->template["xml"]->output();
  }



  function setDoctype($version, $mode)
  {
   $this->template["doctype"]["version"]=$version;
   $this->template["doctype"]["mode"]=$mode;
   return true;
  }

  function setTitle($title)
  {
   $this->template["title"]=$title;
   return true;
  }

  function jasScript($show)
  {
   $this->template["jasscript"]=($show?true:false);
   return true;
  }

  function addJavaScript($file)
  {
   $this->template["javascript"][]=$file;
   return true;
  }

  function addStyleSheet($file)
  {
   $this->template["stylesheet"][]=$file;
   return true;
  }



  function error()
  {
   global $_path, $_benchmark;

   if($_benchmark) $_benchmark->start("Setup error template");

   if(isset($this->config)&&$this->config&&$this->config["errorTemplate"])
   {
    if(!file_exists($_path["project"]["templates"].$this->config["errorTemplate"]))
    {
     $template=new Template($this->plugin->path["templates"].$this->plugin->errorTemplate, true);
     $template->e->add("Error template: \"".$_path["project"]["templates"].$this->config["errorTemplate"]."\" not found", "system");
    }
    else
    {
     $template=new Template($_path["project"]["templates"].$this->config["errorTemplate"], true);

     $template->xml=preg_replace("!(<script.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->xml);
     $template->xml=preg_replace("!(<link.*css.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->xml);
     $template->xml=preg_replace("!(<link.*ico.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->xml);
     $template->xml=preg_replace("!(<img.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->xml);
     $template->reparse();
    }
   }
   else if(isset($this->plugin->errorTemplate))
    $template=new Template($this->plugin->path["templates"].$this->plugin->errorTemplate, true);
   else
    $template=new Template("<html><head><title>#error#</title></head><body><div>#error#</div></body></html>", true);

   if($template->getKey("error")===false)
    $template->e->add("Marker: \"error\" not found in error template: \"".$template->filename."\"", "system");

   if($template->e->any("fatal"))
    return $template->e->show();

   while($template->getKey("error")!==false)
    $template->set("error", $this->e->l->t($this->e->getError(0)), true, true);

   if($template->getKey("extra")!==false)
    $template->set("extra", $this->e->l->t($this->e->getExtra(0)), true, true);

   $output=$template->output();

   if($_benchmark) $_benchmark->stop("Setup error template");
   return $output;
  }

 }

?>
