<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/unit_testing.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["include"]."xml.php");
 require_once($_path["include"]."fs.php");

 class unit_testing extends BasePlugin
 {
  function main()
  {
   return true; //fake for unit test
  }

  function output()
  {
   global $_path;

   $args=func_get_args();
   $args=$args[0];

   if(isset($args[0]))
    $display=$args[0];
   else if(isset($this->config)&&$this->config)
    $display=$this->config["display"];
   else
    return false;

   if(isset($args[1]))
    $unit=$args[1];
   else
    $unit=request("unit");

   if(isset($args[2]))
    $this->runtests=$args[2];
   else if(request("test"))
    $this->runtests=true;
   else
    $this->runtests=false;

   $this->units=false;
   $this->plugin->path["images"]=$_path["HTML"]["current"].$this->plugin->path["root"]."images/";
   $this->plugin->path["ut"]="ut/";

   switch($display)
   {
    case "list_units":
     return $this->list_units();
    case "list_tests":
     return $this->list_tests($unit);
    default:
    {
     $this->e->add("Unknown display option: \"".$display."\"", "plugin");
     return false;
    } 
   }
  }

  function list_tests($unit)
  {
   global $_benchmark;

   if($_benchmark)
    $_benchmark->start("Unit testing ".($this->config?$this->config["key"]:""));

   global $_path;

   $this->statistics=Array(
    "tests" => 0,
    "tested" => 0,
    "failed" => 0,
    "ok" => 0,
   );

   $template=new XML(true);

   if($this->config&&$this->config["template"])
    $template->parseFile($_path["project"]["templates"].$this->config["template"]);
   else
    $template->parseFile($this->plugin->path["templates"].$this->plugin->list_tests_template);

   if($this->units===false)
    $this->initUnits();  

   if($unit&&in_array($unit, array_keys($this->units)))
    $parent=$this->units[$unit]->_getParentUnitName();
   else
   {
    $unit="UT";
    $parent="UT";
   }

   $tmp=$this->list_tests_tree($parent, $template->getInner("units"), $unit);

   $stats=Array();

   $stats["okImage"]=$this->plugin->path["images"]."ok.png";
   $stats["okReal"]=$this->statistics["ok"];

   if($this->statistics["tests"])
    $stats["okPercentage"]=round(($this->statistics["ok"]/$this->statistics["tests"])*100, 1);
   else
    $stats["okPercentage"]="0";

   $stats["skippedImage"]=$this->plugin->path["images"]."nottested.png";
   $stats["skippedReal"]=($this->statistics["tests"]-$this->statistics["tested"]);

   if($this->statistics["tests"])
    $stats["skippedPercentage"]=round((($this->statistics["tests"]-$this->statistics["tested"])/$this->statistics["tests"])*100, 1);
   else
    $stats["skippedPercentage"]="0";

   $stats["failedImage"]=$this->plugin->path["images"]."failed.png";
   $stats["failedReal"]=$this->statistics["failed"];
   
   if($this->statistics["tests"])
    $stats["failedPercentage"]=round(($this->statistics["failed"]/$this->statistics["tests"])*100, 1);
   else
    $stats["failedPercentage"]="0";


   if($this->statistics["ok"]==$this->statistics["tested"])
    $stats["totalImage"]=$this->plugin->path["images"]."ok.png";
   else if($this->statistics["failed"]==$this->statistics["tested"])
    $stats["totalImage"]=$this->plugin->path["images"]."failed.png";
   else if($this->statistics["tested"]==0)
    $stats["totalImage"]=$this->plugin->path["images"]."skipped.png";
   else
    $stats["totalImage"]=$this->plugin->path["images"]."partiallyfailed.png";

   $stats["totalReal"]=$this->statistics["tests"];
   $stats["totalPercentage"]="Total";
 
   $template->set("stats", $stats, true, true);

   $template->addAfter("units", $stats);
   $template->set("units", $tmp, true, true);
   
   $template->set("url", $this->plugin->url(), true, true);
   $template->set("checkunit", $unit, true, true);

   $output=$template->output();

   if($_benchmark)
    $_benchmark->stop();

   return $output;
  }

  function list_tests_tree($parent, $templateContent, $unitfilter=false)
  {
   $template=new XML(true);
   $template->parse($templateContent);

   if($unitfilter=="UT")
    $unitfilter=false;

   $c=0;
   $tmp=Array();
   foreach(array_keys($this->units) as $unit) 
   {
    if(
     $this->units[$unit]->_getParentUnitName()==$parent
     &&($unitfilter===false||$unit==$unitfilter)
    )
    {
     if($this->runtests)
      $this->units[$unit]->_run();

     $tmp=Array(
      "name" => $unit, 
      "title" => preg_replace("/_/", " ", preg_replace("/UT_/", "", $unit)),
      "status" => "Not tested",
      "statusImage" => $this->plugin->path["images"]."nottested.png",
      "subunits" => $this->list_tests_tree($unit, $templateContent),
     );

     if($this->runtests)
     {
      switch($this->units[$unit]->results["status"])
      {
       case UT_OK:
       {
        $tmp["status"]="OK";
        $tmp["statusImage"]=$this->plugin->path["images"]."ok.png";

        $parent=$this->units[$unit]->_getParentUnitName();

        if($parent!="UT") 
        {
         if($this->units[$parent]->results["status"]==UT_NOTESTS)
          $this->units[$parent]->results["status"]=UT_OK;
         else if($this->units[$parent]->results["status"]==UT_FAILED)
          $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
        }
        break;
       }
       case UT_FAILED:
       {
        $tmp["status"]="Failed";
        $tmp["statusImage"]=$this->plugin->path["images"]."failed.png";

        $parent=$this->units[$unit]->_getParentUnitName();

        if($parent!="UT") 
        {
         if($this->units[$parent]->results["status"]==UT_NOTESTS)
          $this->units[$parent]->results["status"]=UT_FAILED;
         else if($this->units[$parent]->results["status"]==UT_OK)
          $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
        }
        break;
       }
       case UT_PARTIALLYFAILED:
       {
        $tmp["status"]="Partially failed";
        $tmp["statusImage"]=$this->plugin->path["images"]."partiallyfailed.png";

        $parent=$this->units[$unit]->_getParentUnitName();

        if($parent!="UT") 
        {
         if($this->units[$parent]->results["status"]==UT_NOTESTS)
          $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
         else if($this->units[$parent]->results["status"]==UT_OK)
          $this->units[$parent]->results["status"]=UT_PARTIALLYFAILED;
        }
        break;
       }
      }
     }

     $c=0;
     $tmp["tests"]=Array();
     $tmp["tests"]["test"]=Array();
     foreach($this->units[$unit]->_getTests() as $test)
     {
      $this->statistics["tests"]++;

      $tmp["tests"]["test"][$c]=Array(
       "title" => $test,
       "status" => "Not tested",
       "error" => "Not tested",
       "statusImage" => $this->plugin->path["images"]."nottested.png",
      );

      if($this->runtests)
      {
       switch($this->units[$unit]->results["tests"][$test]["status"])
       {
        case UT_OK:
        {
         $this->statistics["ok"]++;
         $this->statistics["tested"]++;

         $tmp["tests"]["test"][$c]["status"]="OK";
         $tmp["tests"]["test"][$c]["error"]=$this->units[$unit]->results["tests"][$test]["e"]->show();
         $tmp["tests"]["test"][$c]["statusImage"]=$this->plugin->path["images"]."ok.png";
         break;
        }
        case UT_FAILED:
        {
         $this->statistics["failed"]++;
         $this->statistics["tested"]++;

         $tmp["tests"]["test"][$c]["status"]="Failed";
         $tmp["tests"]["test"][$c]["error"]=$this->units[$unit]->results["tests"][$test]["e"]->show();
         $tmp["tests"]["test"][$c]["statusImage"]=$this->plugin->path["images"]."failed.png";
         break;
        }
        case UT_SKIPPED:
        {
         $tmp["tests"]["test"][$c]["status"]="Skipped";
         $tmp["tests"]["test"][$c]["error"]=$this->units[$unit]->results["tests"][$test]["e"]->show();
         break;
        }
       }
      }
      $c++;
     }

     $template->set("unit", $tmp, false, true);
     $template->set("tests", $template->getInner("tests"), true, true);
     $template->set("test", $template->getInner("test"), true, true);
    }
   }

   $template->delete("unit");

   if(sizeof($tmp)==0)
    return "";

   return $template->output();
  }


  function list_units()
  {
   global $_path;

   global $_benchmark;

   if($_benchmark)
    $_benchmark->start("Unit testing ".($this->config?$this->config["key"]:""));

   $template=new XML(true);

   if($this->config&&$this->config["template"])
    $template->parseFile($_path["project"]["templates"].$this->config["template"]);
   else
    $template->parseFile($this->plugin->path["templates"].$this->plugin->list_units_template);

   if($this->units===false)
    $this->initUnits();  

   $tmp=$this->list_units_recurse("UT", $template->getInner("units"));
   $template->set("units", $tmp, true, true);

   $output=$template->output();

   if($_benchmark)
    $_benchmark->stop();

   return $output;
  }

  function list_units_recurse($parent, $templateContent)
  {
   $template=new XML(true);
   $template->parse($templateContent);

   $c=0;
   $tmp=Array();
   foreach(array_keys($this->units) as $unit) 
   {
    if($this->units[$unit]->_getParentUnitName()==$parent)
    {
     $tmp=Array(
      "url" => $this->plugin->url(), 
      "name" => $unit, 
      "title" => preg_replace("/_/", " ", preg_replace("/^UT_/", "", $unit)),
      "subunits" => $this->list_units_recurse($unit, $templateContent),
     );

     $template->set("unit", $tmp, false, true);
    }
   }

   $template->delete("unit");

   if(sizeof($tmp)==0)
    return "";

   return $template->output();
  }

  function initUnits()
  {
   global $_path;
   require_once($this->plugin->path["root"]."ut.php");

   //get paths
   $paths=Array();

   //system plugins (includes unit_testing plugin)
   foreach(plugins("system") as $plugin)
    if(FS::exists($_path["plugins"].$plugin."/".$this->plugin->path["ut"], "directory"))
     $paths[]=$_path["plugins"].$plugin."/".$this->plugin->path["ut"];

   //project plugins
   foreach(plugins("project") as $plugin)
    if(FS::exists($_path["project"]["plugins"].$plugin."/".$this->plugin->path["ut"], "directory"))
     $paths[]=$_path["project"]["plugins"].$plugin."/".$this->plugin->path["ut"];

   foreach($paths as $path)
    foreach(FS::recurseDir($path, false) as $testFile)
     if(preg_match("/\.php$/", $testFile))
      require_once($testFile);

   foreach(get_declared_classes() as $class)
    if(is_subclass_of($class, "UT"))
     $this->units[$class]=new $class();

   ksort($this->units);
  }

 }

?>
