<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."functions.php");

 define("UT_OK", 1, true);
 define("UT_FAILED", 0, true);
 define("UT_PARTIALLYFAILED", -1, true);
 define("UT_NOTESTS", -2, true);
 define("UT_SKIPPED", -3, true);

 class UT
 {
  function UT()
  {
   $this->e=new Error();

   $this->tests=false;
   $this->supportFunctions=false;

   $this->baseFunctions=get_class_methods("UT");
   $this->parentFunctions=get_class_methods(get_parent_class($this));

   $this->results=false;
  }

  function _initTests()
  {
   $this->tests=Array();
   $this->supportFunctions=Array();
   foreach(get_class_methods($this) as $function)
   {
    if(in_array($function, $this->baseFunctions)) //ignore base functions
     continue;

    $refMet=new ReflectionMethod($this, $function); //check who declares the method
    if($refMet->getDeclaringClass()->getName()!=get_class($this)) //if not current it's not a test
     continue;

    if(preg_match("/^_/", $function))
     $this->supportFunctions[]=$function;
    else
     $this->tests[]=$function;
   }
  }

  function _run()
  {
   global $_config;

   if($this->tests===false)
    $this->_initTests();

   if(!($tests=func_get_args()))
    $tests=$this->_getTests();

   $this->results=Array(
    "status" => UT_NOTESTS,
    "tests" => Array(),
   );

   if(isset($_config["logging"]))
   {
    $logging=$_config["logging"];
    unset($_config["logging"]);
   }
   else
    $logging=false;

   foreach($tests as $test)
   {
    if(in_array($test, $this->tests))
    {
     $this->e=new Error();

     $tmp=Array();
     $tmp["status"]=$this->$test();

     switch($tmp["status"])
     {
      case UT_OK:
      {
       $this->e->add("Test: \"".$this->_getUnitPath()."::".$test."\" OK", "note");

       if($this->results["status"]===UT_NOTESTS)
        $this->results["status"]=UT_OK;
       else if($this->results["status"]===UT_FAILED)
        $this->results["status"]=UT_PARTIALLYFAILED;

       break;
      }
      case UT_FAILED:
      {
       $this->e->add("Test: \"".$this->_getUnitPath()."::".$test."\" failed", "error"); 

       if($this->results["status"]===UT_NOTESTS)
        $this->results["status"]=UT_FAILED;
       else if($this->results["status"]===UT_OK)
        $this->results["status"]=UT_PARTIALLYFAILED;

       break;
      }
      case UT_SKIPPED:
      {
       $this->e->add("Test: \"".$this->_getUnitPath()."::".$test."\" skipped", "warning"); 

       break;
      }
      default:
      {
       $this->e->add("Test: \"".$this->_getUnitPath()."::".$test."\" has unknown result", "error");

       $this->results["status"]=UT_FAILED;
       break;
      }
     }

     $tmp["e"]=$this->e;

     $this->results["tests"][$test]=$tmp;
    }
   }

   if($logging)
    $_config["logging"]=$logging;

   return $this->results;
  }

  function _getUnitPath()
  {
   $tmp=get_class($this);
   $path=$tmp;;

   while($tmp=get_parent_class($tmp))
   {
    if($tmp=="UT")
     break;

    $path=$tmp."->".$path;
   }

   return $path;
  }

  function _getUnitName()
  {
   return get_class($this);
  }

  function _getParentUnitName()
  {
   return get_parent_class($this);
  }

  function _getTests()
  {
   if($this->tests===false)
    $this->_initTests();

   return $this->tests;
  }

  function _getSupportFunctions()
  {
   if($this->supportFunctions===false)
    $this->_initTests();

   return $this->supportFunctions;
  }
 }

?>
