<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/cache.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."fs.php");

 class UT_Cache extends UT_Classes
 {
  function Constructor()
  {
   global $_path;

   $createDirTest=false;

   if(!FS::exists($_path["project"]["cache"]))
    $createDirTest=true;
   else if(!FS::recurseDir($_path["project"]["cache"])) //empty
   {
    foreach(FS::recurseDir($_path["project"]["cache"], false, true) as $dotfile)
     FS::delete($dotfile);

    if(FS::delete($_path["project"]["cache"]))
     $createDirTest=true;
   }

   //file based cache
   $object=new Cache("file");

   if(!is_object($object)||get_class($object)!="Cache")
   {
    $this->e->add("Constructor doesn't return a Cache object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->e->any())
   {
    $this->e->add("File caching initialization failed", "error");
    $this->e->add($object->e);
    return UT_FAILED;
   }

   if($object->type!="file")
   {
    $this->e->add("Cache type isn't \"file\"", "error");
    return UT_FAILED;
   }

   if($createDirTest)
   {
    if(!FS::exists($_path["project"]["cache"], "directory"))
    {
     $this->e->add("Cache directory not created", "error");
     $this->e->add($object->e);
     return UT_FAILED;
    }

    if(!$object->filePublic&&!in_array(".htaccess", FS::recurseDir($_path["project"]["cache"], true, true)))
    {
     $this->e->add("Cache directory doesn't contain .htaccess", "error");
     $this->e->add($object->e);
     return UT_FAILED;
    }
   }

   //module based cache
   $object=new Cache("module");

   if(!is_object($object)||get_class($object)!="Cache")
   {
    $this->e->add("Constructor doesn't return a Cache object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->e->any())
   {
    $this->e->add("Module caching initialization failed", "error");
    $this->e->add($object->e);
    return UT_FAILED;
   }

   if($object->type!="module")
   {
    $this->e->add("Cache type isn't \"module\"", "error");
    return UT_FAILED;
   }

   $object=new Cache("whatever");

   if(!is_object($object)||get_class($object)!="Cache")
   {
    $this->e->add("Constructor doesn't return a Cache object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if(!$object->e->any())
   {
    $this->e->add("Whatever caching initialization should fail", "error");
    return UT_FAILED;
   }

   if($object->type)
   {
    $this->e->add("Cache type shouldn't be set", "error");
    return UT_FAILED;
   }

   //config merge
   $object=new Cache("file", Array("module" => true, "filePublic" => true));

   if(!$object->moduleCaching)
   {
    $this->e->add("Cache module caching should be enabled");
    return UT_FAILED;
   }

   if(!$object->filePublic)
   {
    $this->e->add("Cache public file caching should be enabled");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function _init()
  {
   $this->cacheOffset=86400;
   $this->cacheKey="ut_cache_test_key.txt";
   $this->cacheValue="Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vestibulum non nisi dui, blandit adipiscing dolor. Vestibulum eu velit a turpis accumsan ultricies nec vitae neque. Nullam luctus nisi at enim eleifend tincidunt. Cras facilisis posuere laoreet. Maecenas eu lacus ut nisi luctus scelerisque. Pellentesque nec lorem nulla, ac lobortis risus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Quisque neque quam, egestas et luctus vel, dignissim nec odio. Sed ullamcorper tempus risus, tempor ultrices risus pellentesque vel. Aenean et mauris est, in pulvinar erat. Integer vehicula turpis eu nulla pulvinar volutpat elementum nulla porta. Sed faucibus, lorem a viverra ultrices, diam elit viverra ante, non iaculis lorem felis ut diam. Etiam metus tellus, adipiscing ac adipiscing ac, semper et justo. Nunc sit amet libero id sem auctor tempus quis at sem. Suspendisse a diam augue, ac volutpat ipsum.

    Donec non posuere risus. Ut justo sapien, suscipit sed elementum ut, rhoncus sed elit. Morbi auctor purus ac sem varius vitae dictum ligula mollis. Nulla eu dui massa. Maecenas at eros sed tellus imperdiet fermentum lacinia nec augue. Etiam bibendum erat at est fringilla aliquet. Quisque id arcu nisi. Morbi pellentesque, risus non luctus sollicitudin, nisi magna fringilla magna, ut bibendum felis dui nec metus. Nunc eget nibh non quam tincidunt vestibulum a eget felis. Vivamus sodales porta sem, non cursus mi convallis et. Maecenas quis mi vitae est ultrices blandit at sed leo. Nam quis urna ac felis gravida auctor quis sed nisi.

    Sed ac libero eget nunc suscipit bibendum. Sed ac lorem felis, quis bibendum nulla. Vivamus eu pharetra massa. Aenean orci odio, accumsan id lacinia ut, imperdiet in tellus. Aliquam feugiat rutrum fermentum. Suspendisse nulla magna, luctus quis luctus ut, vulputate at erat. Praesent tincidunt ligula non mauris hendrerit vel tincidunt quam euismod. Etiam hendrerit, lectus sit amet fermentum euismod, massa tellus auctor odio, non lobortis nibh felis ac orci. Sed ut ante nibh. Nam sed lorem velit. Pellentesque tristique libero et orci aliquet laoreet ultrices purus semper. Ut ut est risus, lobortis faucibus risus. Pellentesque rhoncus suscipit neque, sit amet gravida sapien posuere in.";
  }
 }


 class UT_Cache_File_Based extends UT_Cache
 {
  function Save()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   $time=time();
   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache save failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!FS::exists($_path["project"]["cache"].$this->cacheKey))
   {
    $this->e->add("Cache save failed, file doesn't exist", "error");
    return UT_FAILED;
   }

   if(FS::read($_path["project"]["cache"].$this->cacheKey)!=$this->cacheValue)
   {
    $this->e->add("Cache save failed, file content doesn't match cache value", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(FS::getTime($_path["project"]["cache"].$this->cacheKey, "modified")!=$time)
   {
    $this->e->add("Cache save failed, file time doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(FS::size($_path["project"]["cache"].$this->cacheKey)!=strlen($this->cacheValue))
   {
    $this->e->add("Cache save failed, file size doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   $time=time()-$this->cacheOffset;
   if(!$cache->save($this->cacheKey, $this->cacheValue, $time)) //save with modification time
   {
    $this->e->add("Cache save failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(FS::getTime($_path["project"]["cache"].$this->cacheKey, "modified")!=$time)
   {
    $this->e->add("Cache save failed, file time doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->save($this->cacheKey, $cache->config)) //array
   {
    $this->e->add("Cache save should fail on array", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($cache->save($this->cacheKey, $cache)) //object
   {
    $this->e->add("Cache save should fail on object", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Exist()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache exists failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->exists($this->cacheKey))
   {
    $this->e->add("Cache exists failed, file doesn't exist", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->exists($this->cacheKey))
   {
    $this->e->add("Cache exists should fail", "error");
    return UT_FAILED;
   }

   if($cache->exists(""))
   {
    $this->e->add("Cache exists should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->exists("."))
   {
    $this->e->add("Cache exists should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->exists(".htaccess"))
   {
    $this->e->add("Cache exists should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Keys()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache keys failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->save($this->cacheKey."_2", $this->cacheValue))
   {
    $this->e->add("Cache keys failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    FS::delete($_path["project"]["cache"].$this->cacheKey."_2");
    return UT_FAILED;
   }
   
   if(!($keys=$cache->keys()))
   {
    $this->e->add("Cache keys failed, no keys found", "error");
    $this->e->add($cache->e);

    return UT_FAILED;
   }

   if(!in_array($this->cacheKey, $keys)||!in_array($this->cacheKey."_2", $keys))
   {
    $this->e->add("Cache keys failed, keys don't match", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    FS::delete($_path["project"]["cache"].$this->cacheKey."_2");
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);
   FS::delete($_path["project"]["cache"].$this->cacheKey."_2");

   $keys=$cache->keys();
   if(in_array($this->cacheKey, $keys)||in_array($this->cacheKey."_2", $keys))
   {
    $this->e->add("Cache keys failed, no keys should be found", "error");
    $this->e->add($cache->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache read failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   $accessed=(time()-$this->cacheOffset);
   if(!FS::setTime($_path["project"]["cache"].$this->cacheKey, $accessed, "accessed"))
   {
    $this->e->add("Cache cleanup failed, can't changed accessed time", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!($content=$cache->read($this->cacheKey)))
   {
    $this->e->add("Cache read failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(FS::getTime($_path["project"]["cache"].$this->cacheKey, "accessed")==$accessed)
   {
    $this->e->add("Cache read failed, cache accessed time not updated", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($content!=$this->cacheValue)
   {
    $this->e->add("Cache read failed, cache value doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->read($this->cacheKey))
   {
    $this->e->add("Cache read should fail", "error");
    return UT_FAILED;
   }

   if($cache->read(""))
   {
    $this->e->add("Cache read should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->read("."))
   {
    $this->e->add("Cache read should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->read(".htaccess"))
   {
    $this->e->add("Cache read should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Time()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   $time=time();
   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache time failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!($modified=$cache->time($this->cacheKey)))
   {
    $this->e->add("Cache time failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($time!=$modified)
   {
    $this->e->add("Cache time failed, time doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->time($this->cacheKey))
   {
    $this->e->add("Cache time should fail", "error");
    return UT_FAILED;
   }

   if($cache->time(""))
   {
    $this->e->add("Cache time should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->time("."))
   {
    $this->e->add("Cache time should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->time(".htaccess"))
   {
    $this->e->add("Cache time should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Size()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache size failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!($size=$cache->size($this->cacheKey)))
   {
    $this->e->add("Cache size failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($size!=FS::size($_path["project"]["cache"].$this->cacheKey))
   {
    $this->e->add("Cache size failed, size doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->size($this->cacheKey))
   {
    $this->e->add("Cache size should fail", "error");
    return UT_FAILED;
   }

   if($cache->size(""))
   {
    $this->e->add("Cache size should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->size("."))
   {
    $this->e->add("Cache size should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->size(".htaccess"))
   {
    $this->e->add("Cache size should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Remove()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache remove failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->remove($this->cacheKey))
   {
    $this->e->add("Cache remove failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(FS::exists($_path["project"]["cache"].$this->cacheKey))
   {
    $this->e->add("Cache remove failed, file shouldn't exist", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($cache->remove($this->cacheKey))
   {
    $this->e->add("Cache remove should fail", "error");
    return UT_FAILED;
   }

   if($cache->remove(""))
   {
    $this->e->add("Cache remove should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->remove("."))
   {
    $this->e->add("Cache remove should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->remove(".htaccess"))
   {
    $this->e->add("Cache remove should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Cleanup()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!FS::setTime($_path["project"]["cache"].$this->cacheKey, (time()-$this->cacheOffset), "accessed"))
   {
    $this->e->add("Cache cleanup failed, can't changed accessed time", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->cleanup($this->cacheOffset+10)) //clean up all older then offset+10 seconds (should exclude cacheKey)
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!FS::exists($_path["project"]["cache"].$this->cacheKey))
   {
    $this->e->add("Cache cleanup failed, file should exist", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->cleanup($this->cacheOffset-10)) //clean up all older then offset-10 seconds (should include cacheKey)
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(FS::exists($_path["project"]["cache"].$this->cacheKey))
   {
    $this->e->add("Cache cleanup failed, file shouldn't exist", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Search()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!($results=$cache->search($this->cacheKey)))
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($results!=Array($this->cacheKey))
   {
    $this->e->add("Cache search failed, should return an array with 1 entry", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!($results=$cache->search(substr($this->cacheKey, 4, 10)))) //sub string
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($results!=Array($this->cacheKey))
   {
    $this->e->add("Cache search failed, should return an array with 1 entry", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->search($this->cacheKey))
   {
    $this->e->add("Cache search should fail", "error");
    return UT_FAILED;
   }

   if($cache->search(""))
   {
    $this->e->add("Cache search should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->search("."))
   {
    $this->e->add("Cache search should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->search(".htaccess"))
   {
    $this->e->add("Cache search should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Output() //similar to Read
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache output failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if(!($output=$cache->output($this->cacheKey, false))) //only try without headers
   {
    $this->e->add("Cache output failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($output!=$this->cacheValue)
   {
    $this->e->add("Cache output failed, cache value doesn't match", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->output($this->cacheKey, false))
   {
    $this->e->add("Cache output should fail", "error");
    return UT_FAILED;
   }

   if($cache->output("", false))
   {
    $this->e->add("Cache output should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->output(".", false))
   {
    $this->e->add("Cache output should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->output(".htaccess", false))
   {
    $this->e->add("Cache output should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Redirect()
  {
   global $_path;
   $this->_init();

   $cache=new Cache("file");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   $cache->filePublic=false; //disable public folder and thus redirect

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache redirect failed", "error");
    $this->e->add($cache->e);

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   if($cache->redirect($this->cacheKey))
   {
    $this->e->add("Cache redirect should fail", "error");

    FS::delete($_path["project"]["cache"].$this->cacheKey);
    return UT_FAILED;
   }

   FS::delete($_path["project"]["cache"].$this->cacheKey);

   if($cache->redirect("", false))
   {
    $this->e->add("Cache redirect should fail on empty file", "error");
    return UT_FAILED;
   }

   if($cache->redirect(".", false))
   {
    $this->e->add("Cache redirect should fail on . file", "error");
    return UT_FAILED;
   }

   if($cache->redirect(".htaccess", false))
   {
    $this->e->add("Cache redirect should fail on .htaccess file", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }


 class UT_Cache_Module_Based extends UT_Cache
 {
  function Save()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   $time=time();
   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache save failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!$d->get())
   {
    $this->e->add("Cache save failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($d->vmax==0)
   {
    $this->e->add("Cache save failed, entry doesn't exist", "error");
    return UT_FAILED;
   }

   if($d->getValue("value")!=$this->cacheValue)
   {
    $this->e->add("Cache save failed, entry content doesn't match cache value", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $modified=DT::dateConvert($d->getValue("modified"), $d->field["modified"]["format"][$d->dataformat]["datetimeformat"]); //convert to unix timestamp
   if($modified!=$time&&$modified!=$time+1&&$modified!=$time-1) //add safety margin
   {
    $this->e->add("Cache save failed, entry time doesn't match", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }
   
   //don't bother checking size: value already checked
   $d->setAction("delete");
   $d->submit();

   $time=time()-$this->cacheOffset;
   if(!$cache->save($this->cacheKey, $this->cacheValue, $time)) //save with modification time
   {
    $this->e->add("Cache save failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setShowFields("key", "modified", "accessed");
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   if(!$d->get())
   {
    $this->e->add("Cache save failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $modified=DT::dateConvert($d->getValue("modified"), $d->field["modified"]["format"][$d->dataformat]["datetimeformat"]); //convert to unix timestamp
   if($modified!=$time) //should be exact
   {
    $this->e->add("Cache save failed, entry modified doesn't match", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   if($cache->save($this->cacheKey, $cache->config)) //array
   {
    $this->e->add("Cache save should fail on array", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($cache->save($this->cacheKey, $cache)) //object
   {
    $this->e->add("Cache save should fail on object", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Exist()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache exists failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!$cache->exists($this->cacheKey))
   {
    $this->e->add("Cache exists failed, entry doesn't exist", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   if($cache->exists($this->cacheKey))
   {
    $this->e->add("Cache exists should fail", "error");
    return UT_FAILED;
   }

   if($cache->exists(""))
   {
    $this->e->add("Cache exists should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->exists("."))
   {
    $this->e->add("Cache exists should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->exists(".htaccess"))
   {
    $this->e->add("Cache exists should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Keys()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);
   $d->setOperant("key", "OR");
   $d->setValue("key", $this->cacheKey."_2", 1);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache keys failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!$cache->save($this->cacheKey."_2", $this->cacheValue))
   {
    $this->e->add("Cache keys failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }
   
   if(!($keys=$cache->keys()))
   {
    $this->e->add("Cache keys failed, no keys found", "error");
    $this->e->add($cache->e);

    return UT_FAILED;
   }

   if(!in_array($this->cacheKey, $keys)||!in_array($this->cacheKey."_2", $keys))
   {
    $this->e->add("Cache keys failed, keys don't match", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }
   
   $d->setAction("delete");
   $d->submit();

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   $keys=$cache->keys();
   if(in_array($this->cacheKey, $keys)||in_array($this->cacheKey."_2", $keys))
   {
    $this->e->add("Cache keys failed, no keys should be found", "error");
    $this->e->add($cache->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache read failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $accessed=time()-$this->cacheOffset;
   $d->setAction("edit");
   $d->setValue("accessed", date($d->field["modified"]["format"][$d->dataformat]["datetimeformat"], $accessed));

   if(!$d->submit())
   {
    $this->e->add("Cache cleanup failed, can't changed accessed time", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   if(!($content=$cache->read($this->cacheKey)))
   {
    $this->e->add("Cache read failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   if(!$d->get())
   {
    $this->e->add("Cache read failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($accessed==DT::dateConvert($d->getValue("accessed"), $d->field["modified"]["format"][$d->dataformat]["datetimeformat"]))
   {
    $this->e->add("Cache read failed, cache accessed time not updated", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($content!=$this->cacheValue)
   {
    $this->e->add("Cache read failed, cache value doesn't match", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }
   
   $d->setAction("delete");
   $d->submit();

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   if($cache->read($this->cacheKey))
   {
    $this->e->add("Cache read should fail", "error");
    return UT_FAILED;
   }

   if($cache->read(""))
   {
    $this->e->add("Cache read should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->read("."))
   {
    $this->e->add("Cache read should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->read(".htaccess"))
   {
    $this->e->add("Cache read should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Time()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache time failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!$d->get())
   {
    $this->e->add("Cache save failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $time=DT::dateConvert($d->getValue("modified"), $d->field["modified"]["format"][$d->dataformat]["datetimeformat"]);

   if(!($modified=$cache->time($this->cacheKey)))
   {
    $this->e->add("Cache time failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($modified!=$time&&$modified!=$time+1&&$modified!=$time-1) //add safety margin
   {
    $this->e->add("Cache time failed, time doesn't match", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   if($cache->time($this->cacheKey))
   {
    $this->e->add("Cache time should fail", "error");
    return UT_FAILED;
   }

   if($cache->time(""))
   {
    $this->e->add("Cache time should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->time("."))
   {
    $this->e->add("Cache time should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->time(".htaccess"))
   {
    $this->e->add("Cache time should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Size()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache size failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!($size=$cache->size($this->cacheKey)))
   {
    $this->e->add("Cache size failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($size!=strlen($this->cacheValue))
   {
    $this->e->add("Cache size failed, size doesn't match", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   if($cache->size($this->cacheKey))
   {
    $this->e->add("Cache size should fail", "error");
    return UT_FAILED;
   }

   if($cache->size(""))
   {
    $this->e->add("Cache size should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->size("."))
   {
    $this->e->add("Cache size should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->size(".htaccess"))
   {
    $this->e->add("Cache size should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Remove()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache remove failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!$cache->remove($this->cacheKey))
   {
    $this->e->add("Cache remove failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!$d->get())
   {
    $this->e->add("Cache save failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($d->vmax!=0)
   {
    $this->e->add("Cache remove failed, entry shouldn't exist", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   if($cache->remove($this->cacheKey))
   {
    $this->e->add("Cache remove should fail", "error");
    return UT_FAILED;
   }

   if($cache->remove(""))
   {
    $this->e->add("Cache remove should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->remove("."))
   {
    $this->e->add("Cache remove should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->remove(".htaccess"))
   {
    $this->e->add("Cache remove should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Cleanup()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("edit");
   $d->setValue("accessed", date($d->field["modified"]["format"][$d->dataformat]["datetimeformat"], time()-$this->cacheOffset));

   if(!$d->submit())
   {
    $this->e->add("Cache cleanup failed, can't changed accessed time", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   if(!$cache->cleanup($this->cacheOffset+10)) //clean up all older then offset+10 seconds (should exclude cacheKey)
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("view");
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   if(!$d->get())
   {
    $this->e->add("Cache save failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($d->vmax==0)
   {
    $this->e->add("Cache cleanup failed, entry doesn't exist", "error");
    return UT_FAILED;
   }

   if(!$cache->cleanup($this->cacheOffset-10)) //clean up all older then offset-10 seconds (should include cacheKey)
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   if(!$d->get())
   {
    $this->e->add("Cache save failed, unable to get cache entry", "error");
    $this->e->add($d->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($d->vmax!=0)
   {
    $this->e->add("Cache cleanup failed, entry shouldn't exist", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Search()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!($results=$cache->search($this->cacheKey)))
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($results!=Array($this->cacheKey))
   {
    $this->e->add("Cache search failed, should return an array with 1 entry", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!($results=$cache->search(substr($this->cacheKey, 4, 10)))) //sub string
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($results!=Array($this->cacheKey))
   {
    $this->e->add("Cache search failed, should return an array with 1 entry", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   if($cache->search($this->cacheKey))
   {
    $this->e->add("Cache search should fail", "error");
    return UT_FAILED;
   }

   if($cache->search(""))
   {
    $this->e->add("Cache search should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->search("."))
   {
    $this->e->add("Cache search should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->search(".htaccess"))
   {
    $this->e->add("Cache search should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Output() //similar to Read
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache output failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if(!($output=$cache->output($this->cacheKey, false))) //only try without headers
   {
    $this->e->add("Cache output failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($output!=$this->cacheValue)
   {
    $this->e->add("Cache output failed, cache value doesn't match", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   $cache->d->clearValues(); //module based cache is stateful and thus doesn't know what we just did... clear it for the next bit

   if($cache->output($this->cacheKey, false))
   {
    $this->e->add("Cache output should fail", "error");
    return UT_FAILED;
   }

   if($cache->output("", false))
   {
    $this->e->add("Cache output should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->output(".", false))
   {
    $this->e->add("Cache output should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->output(".htaccess", false))
   {
    $this->e->add("Cache output should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Redirect()
  {
   $this->_init();

   $d=Data::cache("cache", "view", "item", Array("format" => "sql"));
   $d->clearValues();
   $d->setValue("key", $this->cacheKey);

   $cache=new Cache("module");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache redirect failed", "error");
    $this->e->add($cache->e);

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   if($cache->redirect($this->cacheKey))
   {
    $this->e->add("Cache redirect should fail", "error");

    $d->setAction("delete");
    $d->submit();
    return UT_FAILED;
   }

   $d->setAction("delete");
   $d->submit();

   if($cache->redirect($this->cacheKey))
   {
    $this->e->add("Cache redirect should fail", "error");
    return UT_FAILED;
   }

   if($cache->redirect(""))
   {
    $this->e->add("Cache redirect should fail on empty entry", "error");
    return UT_FAILED;
   }

   if($cache->redirect("."))
   {
    $this->e->add("Cache redirect should fail on . entry", "error");
    return UT_FAILED;
   }

   if($cache->redirect(".htaccess"))
   {
    $this->e->add("Cache redirect should fail on .htaccess entry", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }


 class UT_Cache_Memory_Based extends UT_Cache
 {
  function Save()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   $time=time();
   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache save failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->exists($this->cacheKey))
   {
    $this->e->add("Cache save failed, memory doesn't exist", "error");
    return UT_FAILED;
   }

   if($cache->read($this->cacheKey)!=$this->cacheValue)
   {
    $this->e->add("Cache save failed, memory content doesn't match cache value", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->time($this->cacheKey)!=$time)
   {
    $this->e->add("Cache save failed, memory time doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->size($this->cacheKey)!=strlen($this->cacheValue))
   {
    $this->e->add("Cache save failed, memory size doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   $time=time()-$this->cacheOffset;
   if(!$cache->save($this->cacheKey, $this->cacheValue, $time)) //save with modification time
   {
    $this->e->add("Cache save failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->time($this->cacheKey)!=$time)
   {
    $this->e->add("Cache save failed, memory time doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if(!$cache->save($this->cacheKey, $cache->config)) //array
   {
    $this->e->add("Cache save failed on array", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->read($this->cacheKey)!=$cache->config) //array
   {
    $this->e->add("Cache save failed, memory doesn't match cache array", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if(!$cache->save($this->cacheKey, $cache)) //object
   {
    $this->e->add("Cache save failed on object", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(get_class($cache->read($this->cacheKey))!=get_class($cache)) //object
   {
    $this->e->add("Cache save failed, memory doesn't match cache object", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   return UT_OK;
  }


  function Exist()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache exists failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->exists($this->cacheKey))
   {
    $this->e->add("Cache exists failed, memory doesn't exist", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->exists($this->cacheKey))
   {
    $this->e->add("Cache exists should fail", "error");
    return UT_FAILED;
   }

   if($cache->exists(""))
   {
    $this->e->add("Cache exists should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->exists("."))
   {
    $this->e->add("Cache exists should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->exists(".htaccess"))
   {
    $this->e->add("Cache exists should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Keys()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache keys failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->save($this->cacheKey."_2", $this->cacheValue))
   {
    $this->e->add("Cache keys failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($keys=$cache->keys()))
   {
    $this->e->add("Cache keys failed, no keys found", "error");
    $this->e->add($cache->e);

    return UT_FAILED;
   }

   if(!in_array($this->cacheKey, $keys)||!in_array($this->cacheKey."_2", $keys))
   {
    $this->e->add("Cache keys failed, keys don't match", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    $cache->remove($this->cacheKey."_2");
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);
   $cache->remove($this->cacheKey."_2");

   $keys=$cache->keys();
   if(in_array($this->cacheKey, $keys)||in_array($this->cacheKey."_2", $keys))
   {
    $this->e->add("Cache keys failed, no keys should be found", "error");
    $this->e->add($cache->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Read()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache read failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($content=$cache->read($this->cacheKey)))
   {
    $this->e->add("Cache read failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($content!=$this->cacheValue)
   {
    $this->e->add("Cache read failed, cache value doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->read($this->cacheKey))
   {
    $this->e->add("Cache read should fail", "error");
    return UT_FAILED;
   }

   if($cache->read(""))
   {
    $this->e->add("Cache read should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->read("."))
   {
    $this->e->add("Cache read should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->read(".htaccess"))
   {
    $this->e->add("Cache read should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Time()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   $time=time();
   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache time failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($modified=$cache->time($this->cacheKey)))
   {
    $this->e->add("Cache time failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($time!=$modified)
   {
    $this->e->add("Cache time failed, time doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->time($this->cacheKey))
   {
    $this->e->add("Cache time should fail", "error");
    return UT_FAILED;
   }

   if($cache->time(""))
   {
    $this->e->add("Cache time should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->time("."))
   {
    $this->e->add("Cache time should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->time(".htaccess"))
   {
    $this->e->add("Cache time should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Size()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache size failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($size=$cache->size($this->cacheKey)))
   {
    $this->e->add("Cache size failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($size!=$cache->size($this->cacheKey))
   {
    $this->e->add("Cache size failed, size doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->size($this->cacheKey))
   {
    $this->e->add("Cache size should fail", "error");
    return UT_FAILED;
   }

   if($cache->size(""))
   {
    $this->e->add("Cache size should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->size("."))
   {
    $this->e->add("Cache size should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->size(".htaccess"))
   {
    $this->e->add("Cache size should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Remove()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache remove failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->remove($this->cacheKey))
   {
    $this->e->add("Cache remove failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->exists($this->cacheKey))
   {
    $this->e->add("Cache remove failed, memory shouldn't exist", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->remove($this->cacheKey))
   {
    $this->e->add("Cache remove should fail", "error");
    return UT_FAILED;
   }

   if($cache->remove(""))
   {
    $this->e->add("Cache remove should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->remove("."))
   {
    $this->e->add("Cache remove should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->remove(".htaccess"))
   {
    $this->e->add("Cache remove should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Cleanup()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue, time()-$this->cacheOffset))
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->memoryObject->set($this->cacheKey."_accessed", (time()-$this->cacheOffset)))
   {
    $this->e->add("Cache cleanup failed, can't changed accessed time", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->cleanup($this->cacheOffset+10)) //clean up all older then offset+10 seconds (should exclude cacheKey)
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->exists($this->cacheKey))
   {
    $this->e->add("Cache cleanup failed, memory should exist", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!$cache->cleanup($this->cacheOffset-10)) //clean up all older then offset-10 seconds (should include cacheKey)
   {
    $this->e->add("Cache cleanup failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->exists($this->cacheKey))
   {
    $this->e->add("Cache cleanup failed, memory shouldn't exist", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Search()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($results=$cache->search($this->cacheKey)))
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($results!=Array($this->cacheKey))
   {
    $this->e->add("Cache search failed, should return an array with 1 entry", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($results=$cache->search(substr($this->cacheKey, 4, 10)))) //sub string
   {
    $this->e->add("Cache search failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($results!=Array($this->cacheKey))
   {
    $this->e->add("Cache search failed, should return an array with 1 entry", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->search($this->cacheKey))
   {
    $this->e->add("Cache search should fail", "error");
    return UT_FAILED;
   }

   if($cache->search(""))
   {
    $this->e->add("Cache search should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->search("."))
   {
    $this->e->add("Cache search should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->search(".htaccess"))
   {
    $this->e->add("Cache search should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Output() //similar to Read
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache output failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if(!($output=$cache->output($this->cacheKey, false))) //only try without headers
   {
    $this->e->add("Cache output failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($output!=$this->cacheValue)
   {
    $this->e->add("Cache output failed, cache value doesn't match", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->output($this->cacheKey, false))
   {
    $this->e->add("Cache output should fail", "error");
    return UT_FAILED;
   }

   if($cache->output("", false))
   {
    $this->e->add("Cache output should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->output(".", false))
   {
    $this->e->add("Cache output should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->output(".htaccess", false))
   {
    $this->e->add("Cache output should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Redirect()
  {
   $this->_init();

   $cache=new Cache("memory");
   if($cache->e->any())
   {
    $this->e->add($cache->e);
    return UT_SKIPPED;
   }

   if(!$cache->save($this->cacheKey, $this->cacheValue))
   {
    $this->e->add("Cache redirect failed", "error");
    $this->e->add($cache->e);

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   if($cache->redirect($this->cacheKey))
   {
    $this->e->add("Cache redirect should fail", "error");

    $cache->remove($this->cacheKey);
    return UT_FAILED;
   }

   $cache->remove($this->cacheKey);

   if($cache->redirect("", false))
   {
    $this->e->add("Cache redirect should fail on empty memory", "error");
    return UT_FAILED;
   }

   if($cache->redirect(".", false))
   {
    $this->e->add("Cache redirect should fail on . memory", "error");
    return UT_FAILED;
   }

   if($cache->redirect(".htaccess", false))
   {
    $this->e->add("Cache redirect should fail on .htaccess memory", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }


?>
