<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/csv.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."csv.php");

 class UT_CSV extends UT_Classes
 {
  var $_module="unit_testing_testmodule";

  function Constructor()
  {
   $object=new CSV();

   if(!is_object($object)||get_class($object)!="CSV")
   {
    $this->e->add("Constructor doesn't return a CSV object", "error");
    return UT_FAILED;
   }

   if($object->split!=";")
   {
    $this->e->add("Split character isn't: \";\"", "error");
    return UT_FAILED;
   }
   if($object->quote!="\"")
   {
    $this->e->add("Quote character isn't: \"\"\"", "error");
    return UT_FAILED;
   }
   if($object->linebreak!="\n\r")
   {
    $this->e->add("Line break character isn't: \"\\n\\r\"", "error");
    return UT_FAILED;
   }
   if($object->subSplit!="\n\r")
   {
    $this->e->add("Sub split character isn't: \"\\n\\r\"", "error");
    return UT_FAILED;
   }
   if($object->headerLine!==false)
   {
    $this->e->add("Header line isn't: \"false\"", "error");
    return UT_FAILED;
   }

   if(!is_object($object->fs)||get_class($object->fs)!="FS")
   {
    $this->e->add("Object: \"e\" isn't a FS object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->filename!="export.csv")
   {
    $this->e->add("Default filename isn't: \"export.csv\"", "error");
    return UT_FAILED;
   }

   if($object->vc!==0)
   {
    $this->e->add("Value counter should be at 0", "error");
    return UT_FAILED;
   }
   if($object->vmax!==0)
   {
    $this->e->add("Value max counter should be at 0", "error");
    return UT_FAILED;
   }
   if(!is_array($object->v))
   {
    $this->e->add("Values array isn't an array", "error");
    return UT_FAILED;
   }
   if($object->v)
   {
    $this->e->add("Values array isn't empty", "error");
    return UT_FAILED;
   }

   if(!is_array($object->grid))
   {
    $this->e->add("Grid isn't an array", "error");
    return UT_FAILED;
   }
   if($object->grid)
   {
    $this->e->add("Grid isn't empty", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function _init()
  {
   global $_path;

   $this->testfileI=$_path["plugins"]."unit_testing/misc/indexed.csv";
   $this->testfileA=$_path["plugins"]."unit_testing/misc/associative.csv";
   $this->outputfile=$_path["plugins"]."unit_testing/misc/output.csv";

   $this->valuesI=Array(
    Array("1", "test1", "Lorem Ipsum?"),
    Array("2", "test2", "Ipsum?"),
    Array("3", "test3", "Lorem?"),
   );

   $this->valuesA=Array(
    Array("id" => 1, "title" => "test1", "text" => "Lorem Ipsum?"),
    Array("id" => 2, "title" => "test2", "text" => "Ipsum?"),
    Array("id" => 3, "title" => "test3", "text" => "Lorem?"),
   );
  }

  function _data()
  {
   $d=new Data("empty", "view", "all", Array("db" => "stub"));
   $d->addField("id", "nr", "show");
   $d->addField("title", "text", "show", false);
   $d->addField("text", "text", "show", false);

   //manual title overwrite to avoid non translations
   $d->field["id"]["title"]="ID";
   $d->field["title"]["title"]="Title";
   $d->field["text"]["title"]="Text";

   $c=0;
   foreach($this->valuesA as $line)
   {
    foreach($line as $k => $v)
     $d->setValue($k, $v, $c);

    $c++;
   }

   return $d;
  }

  function Seperators()
  {
   $csv=new CSV();

   if(!$csv->separators(",", "", "\n"))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->split!=",")
   {
    $this->e->add("Split character isn't: \",\"", "error");
    return UT_FAILED;
   }
   if($csv->quote!="")
   {
    $this->e->add("Quote character isn't: \"\"", "error");
    return UT_FAILED;
   }
   if($csv->linebreak!="\n")
   {
    $this->e->add("Line break character isn't: \"\\n\"", "error");
    return UT_FAILED;
   }
   if($csv->subSplit!="; ")
   {
    $this->e->add("Sub split character isn't: \"; \"", "error");
    return UT_FAILED;
   }

   if(!$csv->separators(";", "\"", "\n"))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->split!=";")
   {
    $this->e->add("Split character isn't: \";\"", "error");
    return UT_FAILED;
   }
   if($csv->quote!="\"")
   {
    $this->e->add("Quote character isn't: \"\"\"", "error");
    return UT_FAILED;
   }
   if($csv->linebreak!="\n")
   {
    $this->e->add("Line break character isn't: \"\\n\"", "error");
    return UT_FAILED;
   }
   if($csv->subSplit!="\n")
   {
    $this->e->add("Sub split character isn't: \"\\n\"", "error");
    return UT_FAILED;
   }

   if(!$csv->separators(";", "\"", "\n", " - "))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->subSplit!=" - ")
   {
    $this->e->add("Sub split character isn't: \" - \"", "error");
    return UT_FAILED;
   }

   if($csv->separators(",", ",", ","))
   {
    $this->e->add("Split, quote and linebreak character shouldn't be allowed to be the same", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Header()
  {
   $csv=new CSV();

   if(!$csv->header(true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->headerLine!==true)
   {
    $this->e->add("Header line isn't: \"true\"", "error");
    return UT_FAILED;
   }

   if(!$csv->header(false))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->headerLine!==false)
   {
    $this->e->add("Header line isn't: \"false\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CheckLine()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(false); //Indexed check

   if(!$csv->checkLine($this->valuesI[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //force first line check
   $csv->grid[0]=$this->valuesI[1];
   $csv->vmax++;

   if(!$csv->checkLine($this->valuesI[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }


   $csv->e->clear();

   $tmp=$this->valuesI[1];
   array_pop($tmp); //remove line value

   if(!$csv->checkLine($tmp)) //too little (indexed) values only creates a (non-fatal) warning
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->e->errors()!=1||$csv->e->getLevel(0)!="warning")
   {
    $this->e->add("Line should have too little values warning", "error");
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CheckLineHeader()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(true); //Associative check

   if(!$csv->checkLine($this->valuesA[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //force first line check
   $csv->grid[0]=$this->valuesA[1];
   $csv->vmax++;

   if(!$csv->checkLine($this->valuesA[0]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->checkLine($this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   $tmp=$this->valuesA[1];
   array_pop($tmp); //remove line value

   if($csv->checkLine($tmp))
   {
    $this->e->add("Line should have too little values", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   $tmp["Lorem"]="Impsum"; //same size

   if($csv->checkLine($tmp))
   {
    $this->e->add("Line should have incorrect keys", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function AddLine()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if($csv->grid!=$this->valuesI)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Add line should increased vmax", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine 
   if($csv->addLine($this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function AddLineHeader()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if($csv->grid!=$this->valuesA)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Add line should increased vmax", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine 
   if($csv->addLine($this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function EditLine()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->editLine(1, $this->valuesI[2]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->grid[1]!=$this->valuesI[2])
   {
    $this->e->add("Edit line should have updated line: \"1\"", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Edit line shouldn't have changed vmax", "error");
    return UT_FAILED;
   }

   if($csv->editLine(1241, $this->valuesI[2]))
   {
    $this->e->add("Edit line should fail on non existing line: \"1241\"", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //reset line 1
   if(!$csv->editLine(1, $this->valuesI[1]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->grid!=$this->valuesI)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine 
   if($csv->editLine(0, $this->valuesA[0]))
   {
    $this->e->add("Associative line should fail in non-header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function EditLineHeader()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->editLine(1, $this->valuesA[2]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->grid[1]!=$this->valuesA[2])
   {
    $this->e->add("Edit line should have updated line: \"1\"", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=3)
   {
    $this->e->add("Edit line shouldn't have changed vmax", "error");
    return UT_FAILED;
   }

   if($csv->editLine(1241, $this->valuesA[2]))
   {
    $this->e->add("Edit line should fail on non existing line: \"1241\"", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   //reset line 1
   if(!$csv->editLine(1, $this->valuesA[1]))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->grid!=$this->valuesA)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   //check if addLine uses checkLine 
   if($csv->editLine(0, $this->valuesI[0]))
   {
    $this->e->add("Indexed line should fail in header mode", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function DeleteLine()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   if(!$csv->deleteLine(1))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(sizeof($csv->grid)!=2)
   {
    $this->e->add("Delete line should decreased the number of lines in the grid", "error");
    return UT_FAILED;
   }

   if($csv->vmax!=2)
   {
    $this->e->add("Delete line should decreased vmax", "error");
    return UT_FAILED;
   }

   if($csv->grid[0]!=$this->valuesI[0])
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->grid[1]!=$this->valuesI[2])
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->deleteLine(1241, $this->valuesI[2]))
   {
    $this->e->add("Delete line should fail on non existing line: \"1241\"", "error");
    return UT_FAILED;
   }
   $csv->e->clear();

   return UT_OK;
  }

  function Next()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(false); //Indexed check

   foreach($this->valuesI as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   $c=0;
   while($csv->next())
   {
    if($csv->v!=$this->valuesI[$c])
    {
     $this->e->add("Next value doesn't match line", "error");
     return UT_FAILED;
    }
    
    if($csv->vc!=++$c)
    {
     $this->e->add("Next value should increased vc", "error");
     return UT_FAILED;
    }
   }
   
   return UT_OK;
  }

  function NextHeader()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(true); //Associative check

   foreach($this->valuesA as $v)
   {
    if(!$csv->addLine($v))
    {
     $this->e->add($csv->e);
     return UT_FAILED;
    }
   }

   $c=0;
   while($csv->next())
   {
    if($csv->v!=$this->valuesA[$c])
    {
     $this->e->add("Next value doesn't match line", "error");
     return UT_FAILED;
    }
    
    if($csv->vc!=++$c)
    {
     $this->e->add("Next value should increased vc", "error");
     return UT_FAILED;
    }
   }
   
   return UT_OK;
  }

  function Read()
  {
   $this->_init();

   $csv=new CSV();

   if(!$csv->read($this->testfileI, false))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->grid!=$this->valuesI)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   if($csv->read("nonexistingfile", false))
   {
    $this->e->add("Read shouldn't be able to read \"nonexistingfile\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ReadHeader()
  {
   $this->_init();

   $csv=new CSV();

   if(!$csv->read($this->testfileA, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if($csv->grid!=$this->valuesA)
   {
    $this->e->add("Grid doesn't match lines", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $this->_init();

   $csv=new CSV();

   if(!$csv->read($this->testfileI, false, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output="";
   foreach($csv->grid as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if($csv->output(false)!=$output)
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function OutputHeader()
  {
   $this->_init();

   $csv=new CSV();

   if(!$csv->read($this->testfileA, true, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, array_keys($csv->grid[0])).$csv->quote.$csv->linebreak;
   foreach($csv->grid as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if($csv->output(false)!=$output)
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Write()
  {
   $this->_init();

   $csv=new CSV();

   if(!$csv->read($this->testfileI, false, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$csv->write($this->outputfile))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output="";
   foreach($csv->grid as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if(file_get_contents($this->outputfile)!=$output)
   {
    $this->e->add("Output file doesn't match expected output", "error");
    @unlink($this->outputfile);
    return UT_FAILED;
   }

   @unlink($this->outputfile);
   return UT_OK;
  }

  function WriteHeader()
  {
   $this->_init();

   $csv=new CSV();

   if(!$csv->read($this->testfileA, true, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   if(!$csv->write($this->outputfile))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $output=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, array_keys($csv->grid[0])).$csv->quote.$csv->linebreak;
   foreach($csv->grid as $line)
    $output.=$csv->quote.implode($csv->quote.$csv->split.$csv->quote, $line).$csv->quote.$csv->linebreak;

   if(file_get_contents($this->outputfile)!=$output)
   {
    $this->e->add("Output file doesn't match expected output", "error");
    @unlink($this->outputfile);
    return UT_FAILED;
   }

   @unlink($this->outputfile);
   return UT_OK;
  }

  function Data()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(false); //Indexed check

   $d=&$this->_data();

   if(!$csv->data($d))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $values=$this->valuesI;
   if($csv->grid!=$values)
   {
    $this->e->add("Grid doesn't match data", "error");
    return UT_FAILED;
   }

   if(!$csv->data($d, Array("title", "text")))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $values=Array();
   foreach($this->valuesA as $k => $v)
    foreach($v as $k2 => $v2)
     if($k2!="id")
      $values[$k][]=$v2;

   if($csv->grid!=$values)
   {
    $this->e->add("Grid doesn't match data", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DataHeader()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(true); //Associative check

   $d=&$this->_data();

   if(!$csv->data($d))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $values=$this->valuesA;
   if($csv->grid!=$values)
   {
    $this->e->add("Grid doesn't match data", "error");
    return UT_FAILED;
   }

   if(!$csv->data($d, Array("title", "text")))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $values=Array();
   foreach($this->valuesA as $k => $v)
    foreach($v as $k2 => $v2)
     if($k2!="id")
      $values[$k][$k2]=$v2;

   if($csv->grid!=$values)
   {
    $this->e->add("Grid doesn't match data", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DataTitles()
  {
   $this->_init();

   $csv=new CSV();
   $csv->header(true); //Associative check

   $d=&$this->_data();

   if(!$csv->data($d, false, true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $values=Array();
   foreach($this->valuesA as $k => $v)
    foreach($v as $k2 => $v2)
      $values[$k][$d->field[$k2]["title"]]=$v2;

   if($csv->grid!=$values)
   {
    $this->e->add("Grid doesn't match data", "error");
    return UT_FAILED;
   }

   if(!$csv->data($d, Array("title", "text"), true))
   {
    $this->e->add($csv->e);
    return UT_FAILED;
   }

   $values=Array();
   foreach($this->valuesA as $k => $v)
    foreach($v as $k2 => $v2)
     if($k2!="id")
      $values[$k][$d->field[$k2]["title"]]=$v2;

   if($csv->grid!=$values)
   {
    $this->e->add("Grid doesn't match data", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


 }

?>
