<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/dt.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."dt.php");

 class DT_EXTENDS_TEST extends DT
 {
 }

 class UT_DT extends UT_Classes
 {
  function Constructor()
  {
   $object=new DT();

   if(!is_object($object)||get_class($object)!="DT")
   {
    $this->e->add("Constructor doesn't return a DT object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function IsInitialized()
  {
   $dt=new DT();

   if(!$dt->isInitialized())
   {
    $this->e->add("\$dt should be initialized", "error");
    return UT_FAILED;
   }

   if(DT::isInitialized())
   {
    $this->e->add("DT:: shouldn't be initialized", "error");
    return UT_FAILED;
   }

   $dt=New DT_EXTENDS_TEST();
   if(!$dt->isInitialized())
   {
    $this->e->add("\$dt should be initialized", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtract()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 9,
    "day" => 4,
    "hour" => 23,
    "minute" => 15,
    "second" => 10,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractWeeks()
  {
   $dt=new DT();

   $format="Y W";

//normal year
   $time="2012 01";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 1,
    "day" => 2,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 26";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 6,
    "day" => 25,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 52";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 24,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//first week starting in previous year
   $time="2013 1";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 31,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//week number over 52 weeks (next year)
   $time="2012 60";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2013,
    "month" => 2,
    "day" => 18,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//year starting on the first of jan
   $time="2006 52";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2006,
    "month" => 12,
    "day" => 25,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2007 1";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2007,
    "month" => 1,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//53 weeks year
   $time="2009 1";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2008,
    "month" => 12,
    "day" => 29,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2009 26";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2009,
    "month" => 6,
    "day" => 22,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2009 53";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2009,
    "month" => 12,
    "day" => 28,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

//leap day
   $time="2012 9";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 2,
    "day" => 27,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2011 9";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2011,
    "month" => 2,
    "day" => 28,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractDays()
  {
   $dt=new DT();

   $format="Y z";

//normal year
   $time="2012 01";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 1,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 182";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 6,
    "day" => 30,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 365";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 30,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//leap day
   $time="2012 366";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 31,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 60";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 2,
    "day" => 29,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractUnixTimestamp()
  {
   $dt=new DT();

   //set timezone to UTC
   $timezone=date_default_timezone_get();
   date_default_timezone_set("UTC");

   $format="U";

//Unix epoch
   $time=0;

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 1970,
    "month" => 1,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//"now"
   $time=1351271722;

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 10,
    "day" => 26,
    "hour" => 17,
    "minute" => 15,
    "second" => 22,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//multi value date
   $format="d-m-Y U H:i:s";
   $time="26-10-2012 1351271722 17:15:22";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 10,
    "day" => 26,
    "hour" => 17,
    "minute" => 15,
    "second" => 22,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   date_default_timezone_set($timezone); //revert timezone
   return UT_OK;
  }


  function DateExtractArray()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if(!($date=$dt->dateExtract(Array($time1, $time2), $format)))
   {
    $this->e->add("Date extract array shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    Array(
     "year" => 2012,
     "month" => 9,
     "day" => 4,
     "hour" => 23,
     "minute" => 15,
     "second" => 10,
    ),
    Array(
     "year" => 1920,
     "month" => 10,
     "day" => 23,
     "hour" => 1,
     "minute" => 2,
     "second" => 3,
    ),
   ))
   {
    $this->e->add("Date extract array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvert()
  {
   $dt=new DT();

//date
   $tstamp=time();
   $inf="Y-m-d";
   $outf="d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

//date time
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

//to unix timestamp
   $inf="d m Y H i s";
   $outf=false;

   $in=date($inf, $tstamp);
   $out=$tstamp;

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

//one big input string
   $inf="YmdHis";
   $outf="j n y G i s";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvertArray()
  {
   $dt=new DT();

   $tstamp=time();
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in1=date($inf, $tstamp);
   $out1=date($outf, $tstamp);

   $in2=date($inf, $tstamp+73945);
   $out2=date($outf, $tstamp+73945);

   if(!($values=$dt->dateConvert(Array($in1, $in2), $inf, $outf)))
   {
    $this->e->add("Date time array convert failed", "error");
    return UT_FAILED;
   }
   if($values[0]!=$out1)
   {
    $this->e->add("Date time array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$out2)
   {
    $this->e->add("Date time array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDelta()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if($dt->dateDelta($time, 5, "second", $format)!="2012-09-04 23:15:15")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "seconds", $format)!="2012-09-04 23:14:55")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 55, "minute", $format)!="2012-09-05 00:10:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -16, "minutes", $format)!="2012-09-04 22:59:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 2, "hour", $format)!="2012-09-05 01:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -28, "hours", $format)!="2012-09-03 19:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 13, "day", $format)!="2012-09-17 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "days", $format)!="2012-08-20 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 5, "week", $format)!="2012-10-09 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -2, "weeks", $format)!="2012-08-21 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 5, "month", $format)!="2013-02-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "months", $format)!="2011-06-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 24, "year", $format)!="2036-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "years", $format)!="1997-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDeltaArray()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if($dt->dateDelta(Array($time1, $time2), -70, "second", $format)!=Array("2012-09-04 23:14:00", "1920-10-23 01:00:53"))
   {
    $this->e->add("Date delta array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimezoneConvert()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";

   $intime="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if($dt->timezoneConvert($intime, $intz, $outtz, $format)!==$outtime)
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimezoneConvertArray()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";

   $intime1="2012-01-15 23:15:10";
   $intime2="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime1="2012-01-16 07:15:10";
   $outtime2="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if(!($values=$dt->timezoneConvert(Array($intime1, $intime2), $intz, $outtz, $format)))
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }
   if($values[0]!=$outtime1)
   {
    $this->e->add("Timezone array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$outtime2)
   {
    $this->e->add("Timezone array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
