<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/dt.sro.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/dt.php");
 require_once($_path["include"]."dt.php");

 class UT_DT_Scope_Resolution_Operator extends UT_DT
 {
  function IsInitialized()
  {
   if(DT::isInitialized())
   {
    $this->e->add("DT:: shouldn't be initialized", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
  
  
  function DateExtract()
  {
   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(!($date=DT::dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 9,
    "day" => 4,
    "hour" => 23,
    "minute" => 15,
    "second" => 10,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractArray()
  {
   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if(!($date=DT::dateExtract(Array($time1, $time2), $format)))
   {
    $this->e->add("Date extract array shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    Array(
     "year" => 2012,
     "month" => 9,
     "day" => 4,
     "hour" => 23,
     "minute" => 15,
     "second" => 10,
    ),
    Array(
     "year" => 1920,
     "month" => 10,
     "day" => 23,
     "hour" => 1,
     "minute" => 2,
     "second" => 3,
    ),
   ))
   {
    $this->e->add("Date extract array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvert()
  {
//date
   $tstamp=time();
   $inf="Y-m-d";
   $outf="d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if(DT::dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

//date time
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if(DT::dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

//to unix timestamp
   $inf="d m Y H i s";
   $outf=false;

   $in=date($inf, $tstamp);
   $out=$tstamp;

   if(DT::dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

//one big input string
   $inf="YmdHis";
   $outf="j n y G i s";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if(DT::dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DateConvertArray()
  {
   $tstamp=time();
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in1=date($inf, $tstamp);
   $out1=date($outf, $tstamp);

   $in2=date($inf, $tstamp+73945);
   $out2=date($outf, $tstamp+73945);

   if(!($values=DT::dateConvert(Array($in1, $in2), $inf, $outf)))
   {
    $this->e->add("Date time array convert failed", "error");
    return UT_FAILED;
   }
   if($values[0]!=$out1)
   {
    $this->e->add("Date time array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$out2)
   {
    $this->e->add("Date time array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDelta()
  {
   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(DT::dateDelta($time, 5, "second", $format)!="2012-09-04 23:15:15")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -15, "seconds", $format)!="2012-09-04 23:14:55")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, 55, "minute", $format)!="2012-09-05 00:10:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -16, "minutes", $format)!="2012-09-04 22:59:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, 2, "hour", $format)!="2012-09-05 01:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -28, "hours", $format)!="2012-09-03 19:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, 13, "day", $format)!="2012-09-17 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -15, "days", $format)!="2012-08-20 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, 5, "week", $format)!="2012-10-09 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -2, "weeks", $format)!="2012-08-21 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, 5, "month", $format)!="2013-02-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -15, "months", $format)!="2011-06-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, 24, "year", $format)!="2036-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if(DT::dateDelta($time, -15, "years", $format)!="1997-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDeltaArray()
  {
   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if(DT::dateDelta(Array($time1, $time2), -70, "second", $format)!=Array("2012-09-04 23:14:00", "1920-10-23 01:00:53"))
   {
    $this->e->add("Date delta array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimezoneConvert()
  {
   $format="Y-m-d H:i:s";

   $intime="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if(DT::timezoneConvert($intime, $intz, $outtz, $format)!==$outtime)
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function TimezoneConvertArray()
  {
   $format="Y-m-d H:i:s";

   $intime1="2012-01-15 23:15:10";
   $intime2="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime1="2012-01-16 07:15:10";
   $outtime2="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if(!($values=DT::timezoneConvert(Array($intime1, $intime2), $intz, $outtz, $format)))
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    return UT_FAILED;
   }
   if($values[0]!=$outtime1)
   {
    $this->e->add("Timezone array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$outtime2)
   {
    $this->e->add("Timezone array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }
?>
