<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/email.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."email.php");

 class UT_Email extends UT_Classes
 {
  function Constructor()
  {
   $object=new Email();

   if(!is_object($object)||get_class($object)!="Email")
   {
    $this->e->add("Constructor doesn't return a Email object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   //config merge
   $object=new Email(Array("mimeVersion" => "0.1", "server" => "test.com"));

   if($object->mimeVersion!="0.1")
   {
    $this->e->add("Email mime version should be \"0.1\"");
    return UT_FAILED;
   }

   if($object->server!="test.com")
   {
    $this->e->add("Email server should be \"test.com\"");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Clear()
  {
   $email=new Email();

   if(!$email->to("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->clear("to"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->to)
   {
    $this->e->add("Clear to address failed", "error");
    return UT_FAILED;
   }

   if(!$email->cc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->clear("cc"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->cc)
   {
    $this->e->add("Clear cc address failed", "error");
    return UT_FAILED;
   }

   if(!$email->bcc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->clear("bcc"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->bcc)
   {
    $this->e->add("Clear bcc address failed", "error");
    return UT_FAILED;
   }

   if(!$email->from("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->clear("from"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->from)
   {
    $this->e->add("Clear from address failed", "error");
    return UT_FAILED;
   }

   if(!$email->subject("Test subject"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->clear("subject"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->subject)
   {
    $this->e->add("Clear subject failed", "error");
    return UT_FAILED;
   }

   if(!$email->message("Test message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->clear("attachment"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->attachment)
   {
    $this->e->add("Clear attachments failed", "error");
    return UT_FAILED;
   }

   $email=new Email();
   $email->to("test@unit.jas");
   $email->cc("test@unit.jas");
   $email->bcc("test@unit.jas");
   $email->from("test@unit.jas");
   $email->subject("Test subject");
   $email->message("Test message");

   if(!$email->clear("all"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->to)
   {
    $this->e->add("Clear all didn't clear to", "error");
    return UT_FAILED;
   }
   if($email->cc)
   {
    $this->e->add("Clear all didn't clear cc", "error");
    return UT_FAILED;
   }
   if($email->bcc)
   {
    $this->e->add("Clear all didn't clear bcc", "error");
    return UT_FAILED;
   }
   if($email->from)
   {
    $this->e->add("Clear all didn't clear from", "error");
    return UT_FAILED;
   }
   if($email->subject)
   {
    $this->e->add("Clear all didn't clear subject", "error");
    return UT_FAILED;
   }
   if($email->attachment)
   {
    $this->e->add("Clear all didn't clear attachments", "error");
    return UT_FAILED;
   }

   if($email->clear("ASDASDASDASDASDASDASD"))
   {
    $this->e->add("Clear shouldn't be able to clear: \"ASDASDASDASDASDASDASD\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function To()
  {
   $email=new Email();

   if(!$email->to("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->to[0]!="test@unit.jas")
   {
    $this->e->add("Adding to address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->to("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->to["Test2"]!="test2@unit.jas")
   {
    $this->e->add("Adding to address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->to("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->to["Test3"]!="test3@unit.jas")
   {
    $this->e->add("Adding to address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->to("test4.unit.jas"))
   {
    $this->e->add("Adding illegal to address didn't failed", "error");
    return UT_FAILED;
   }

   if(sizeof($email->to)!=3)
   {
    $this->e->add("Incorrect number of to addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CC()
  {
   $email=new Email();

   if(!$email->cc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->cc[0]!="test@unit.jas")
   {
    $this->e->add("Adding cc address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->cc("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->cc["Test2"]!="test2@unit.jas")
   {
    $this->e->add("Adding cc address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->cc("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->cc["Test3"]!="test3@unit.jas")
   {
    $this->e->add("Adding cc address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->cc("test4.unit.jas"))
   {
    $this->e->add("Adding illegal cc address didn't failed", "error");
    return UT_FAILED;
   }

   if(sizeof($email->cc)!=3)
   {
    $this->e->add("Incorrect number of cc addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function BCC()
  {
   $email=new Email();

   if(!$email->bcc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->bcc[0]!="test@unit.jas")
   {
    $this->e->add("Adding bcc address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->bcc("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->bcc["Test2"]!="test2@unit.jas")
   {
    $this->e->add("Adding bcc address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->bcc("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->bcc["Test3"]!="test3@unit.jas")
   {
    $this->e->add("Adding bcc address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->bcc("test4.unit.jas"))
   {
    $this->e->add("Adding illegal bcc address didn't failed", "error");
    return UT_FAILED;
   }

   if(sizeof($email->bcc)!=3)
   {
    $this->e->add("Incorrect number of bcc addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function From()
  {
   $email=new Email();

   if(!$email->from("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->from[0]!="test@unit.jas")
   {
    $this->e->add("Adding from address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->from("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->from["Test2"]!="test2@unit.jas")
   {
    $this->e->add("Adding from address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->from("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->from["Test3"]!="test3@unit.jas")
   {
    $this->e->add("Adding from address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->from("test4.unit.jas"))
   {
    $this->e->add("Adding illegal from address didn't failed", "error");
    return UT_FAILED;
   }

   if(sizeof($email->from)!=1)
   {
    $this->e->add("Incorrect number of from addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Subject()
  {
   $email=new Email();

   if(!$email->subject("Test subject"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->subject!="Test subject")
   {
    $this->e->add("Adding subject failed", "error");
    return UT_FAILED;
   }

   if(!$email->subject("Test subject\n\n"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->subject!="Test subject")
   {
    $this->e->add("Adding subject failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MessageText()
  {
   $email=new Email();

   if(!$email->message("Test message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->attachment[0]["content"]!="Test message")
   {
    $this->e->add("Adding text message failed, text message incorrect", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["contentType"]!="text/plain")
   {
    $this->e->add("Adding text message failed, text message has incorrect content type", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MessageHTML()
  {
   $email=new Email();

   if(!$email->message("<html><body>Test message</body></html>"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!strstr($email->attachment[0]["content"], "Test message"))
   {
    $this->e->add("Adding HTML message failed, HTML message incorrect", "error");
    return UT_FAILED;
   }
   if(!strstr($email->attachment[0]["content"], "<html>"))
   {
    $this->e->add("Adding HTML message failed, HTML message doesn't contain HTML", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["contentType"]!="text/html")
   {
    $this->e->add("Adding HTML message failed, HTML message has incorrect content type", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MessageAlternative()
  {
   $email=new Email();

   if(!$email->message("Test message", true))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->attachment[0]["content"])
   {
    $this->e->add("Adding text+HTML message failed, message container should have no content", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["contentType"]!="multipart/alternative")
   {
    $this->e->add("Adding text+HTML message failed, message container has incorrect content type", "error");
    return UT_FAILED;
   }

   if($email->attachment[1]["content"]!="Test message")
   {
    $this->e->add("Adding text+HTML message failed, text message incorrect", "error");
    return UT_FAILED;
   }
   if($email->attachment[1]["contentType"]!="text/plain")
   {
    $this->e->add("Adding text+HTML message failed, text message has incorrect content type", "error");
    return UT_FAILED;
   }

   if(!strstr($email->attachment[2]["content"], "Test message"))
   {
    $this->e->add("Adding text+HTML message failed, HTML message incorrect", "error");
    return UT_FAILED;
   }
   if(!strstr($email->attachment[2]["content"], "<html>"))
   {
    $this->e->add("Adding text+HTML message failed, HTML message doesn't contain HTML", "error");
    return UT_FAILED;
   }
   if($email->attachment[2]["contentType"]!="text/html")
   {
    $this->e->add("Adding text+HTML message failed, HTML message has incorrect content type", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MessageFromTextFile()
  {
   global $_path;
   $email=new Email();

   $file=$_path["project"]["plugins"]."status.txt";

   if($email->message($file)===false)
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->attachment[0]["name"]!=$file)
   {
    $this->e->add("Message filename doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["contentType"]!="text/plain")
   {
    $this->e->add("Message content type is incorrect, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["content"]!=$email->fixLineBreaks(@file_get_contents($file)))
   {
    $this->e->add("Message contents doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MessageFromHTMLFile()
  {
   global $_path;
   $email=new Email();

   $file=$_path["plugins"]."pager/templates/error.html";

   if($email->message($file)===false)
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->attachment[0]["name"]!=$file)
   {
    $this->e->add("Message filename doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["contentType"]!="text/html")
   {
    $this->e->add("Message content type is incorrect, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["content"]!=$email->fixLineBreaks(@file_get_contents($file)))
   {
    $this->e->add("Message contents doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function MessageFromFileAlternative()
  {
   global $_path;
   $email=new Email();

   $file=$_path["project"]["plugins"]."status.txt";

   if($email->message($file, true)===false)
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->attachment[1]["name"]!=$file)
   {
    $this->e->add("Message filename doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["content"])
   {
    $this->e->add("Adding text+HTML message failed, message container should have no content", "error");
    return UT_FAILED;
   }
   if($email->attachment[0]["contentType"]!="multipart/alternative")
   {
    $this->e->add("Adding text+HTML message failed, message container has incorrect content type", "error");
    return UT_FAILED;
   }

   if($email->attachment[1]["content"]!=$email->fixLineBreaks(@file_get_contents($file)))
   {
    $this->e->add("Adding text+HTML message failed, text message incorrect", "error");
    return UT_FAILED;
   }
   if($email->attachment[1]["contentType"]!="text/plain")
   {
    $this->e->add("Adding text+HTML message failed, text message has incorrect content type", "error");
    return UT_FAILED;
   }

   if(!strstr($email->attachment[2]["content"], @file_get_contents($file)))
   {
    $this->e->add("Adding text+HTML message failed, HTML message incorrect", "error");
    return UT_FAILED;
   }
   if(!strstr($email->attachment[2]["content"], "<html>"))
   {
    $this->e->add("Adding text+HTML message failed, HTML message doesn't contain HTML", "error");
    return UT_FAILED;
   }
   if($email->attachment[2]["contentType"]!="text/html")
   {
    $this->e->add("Adding text+HTML message failed, HTML message has incorrect content type", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
  function AttachementTextFile()
  {
   global $_path;
   $email=new Email();

   $file=$_path["project"]["plugins"]."status.txt";

   if(($entry=$email->attach($file))===false)
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->attachment[$entry]["name"]!=$file)
   {
    $this->e->add("Filename doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[$entry]["contentType"]!="text/plain")
   {
    $this->e->add("Attachment's content type is incorrect, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if(!$email->attachment[$entry]["boundary"])
   {
    $this->e->add("Attachment doesn't have boundary, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[$entry]["contentTransferEncodig"]!="7bit")
   {
    $this->e->add("Attachment's content transfer encoding is incorrect, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[$entry]["content"]!=$email->fixLineBreaks(@file_get_contents($file)))
   {
    $this->e->add("File contents doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AttachementPNGFile()
  {
   global $_path;
   $email=new Email();

   $file=$_path["images"]."imagenotfound.png";

   if(($entry=$email->attach($file))===false)
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->attachment[$entry]["name"]!=$file)
   {
    $this->e->add("Filename doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[$entry]["contentType"]!="image/png")
   {
    $this->e->add("Attachment's content type is incorrect, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if(!$email->attachment[$entry]["boundary"])
   {
    $this->e->add("Attachment doesn't have boundary, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[$entry]["contentTransferEncodig"]!="base64")
   {
    $this->e->add("Attachment's content transfer encoding is incorrect, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }
   if($email->attachment[$entry]["content"]!=base64_encode(@file_get_contents($file)))
   {
    $this->e->add("File contents doesn't match input file, for file: \"".$file."\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Header()
  {
   $email=new Email();

   $email=new Email();
   $email->to("test@unit.jas");
   $email->cc("test@unit.jas");
   $email->bcc("test@unit.jas");
   $email->from("test@unit.jas");
   $email->subject("Test subject");
   $email->message("Test message");

   if(!$header=$email->header())
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/From: test@unit.jas/", $header))
   {
    $this->e->add("From address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/To: test@unit.jas/", $header))
   {
    $this->e->add("To address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Cc: test@unit.jas/", $header))
   {
    $this->e->add("CC address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Bcc: test@unit.jas/", $header))
   {
    $this->e->add("BCC address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Subject: Test subject/", $header))
   {
    $this->e->add("Subject not found in header", "error");
    return UT_FAILED;
   }

   if(!preg_match("/Message-ID: <[^>]+>/", $header))
   {
    $this->e->add("Message-ID not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/MIME-Version: [0-9\.]+/", $header))
   {
    $this->e->add("MIME-Version not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Date: /", $header))
   {
    $this->e->add("Date not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Date: [A-Za-z]{3}, [0-9]{1,2} [A-Za-z]{3} [0-9]{4} [0-9]{2}:[0-9]{2}:[0-9]{2} \+[0-9]{4}/", $header))
   {
    $this->e->add("Date format incorrect in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/User-Agent: ".preg_quote($email->userAgent)."/", $header))
   {
    $this->e->add("User-Agent not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Type: text\/plain; charset=".preg_quote($email->charSet)."/", $header))
   {
    $this->e->add("Content-Type not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Transfer-Encoding: 7bit/", $header))
   {
    $this->e->add("Content-Transfer-Encoding not found in header", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function HeaderPHPMailer()
  {
   $email=new Email();

   $email=new Email();
   $email->to("test@unit.jas");
   $email->cc("test@unit.jas");
   $email->bcc("test@unit.jas");
   $email->from("test@unit.jas");
   $email->subject("Test subject");
   $email->message("Test message");

   if(!$header=$email->header(true))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/From: test@unit.jas/", $header))
   {
    $this->e->add("From address not found in header", "error");
    return UT_FAILED;
   }
   if(preg_match("/To: test@unit.jas/", $header))
   {
    $this->e->add("To address found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Cc: test@unit.jas/", $header))
   {
    $this->e->add("CC address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Bcc: test@unit.jas/", $header))
   {
    $this->e->add("BCC address not found in header", "error");
    return UT_FAILED;
   }
   if(preg_match("/Subject: Test subject/", $header))
   {
    $this->e->add("Subject found in header", "error");
    return UT_FAILED;
   }

   if(!preg_match("/Message-ID: <[^>]+>/", $header))
   {
    $this->e->add("Message-ID not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/MIME-Version: [0-9\.]+/", $header))
   {
    $this->e->add("MIME-Version not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Date: /", $header))
   {
    $this->e->add("Date not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Date: [A-Za-z]{3}, [0-9]{1,2} [A-Za-z]{3} [0-9]{4} [0-9]{2}:[0-9]{2}:[0-9]{2} \+[0-9]{4}/", $header))
   {
    $this->e->add("Date format incorrect in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/User-Agent: ".preg_quote($email->userAgent)."/", $header))
   {
    $this->e->add("User-Agent not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Type: text\/plain; charset=".preg_quote($email->charSet)."/", $header))
   {
    $this->e->add("Content-Type not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Transfer-Encoding: 7bit/", $header))
   {
    $this->e->add("Content-Transfer-Encoding not found in header", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Body()
  {
   $email=new Email();

   $email=new Email();
   $email->to("test@unit.jas");
   $email->message("Test message");

   if(!$body=$email->body())
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/Test message/", $body))
   {
    $this->e->add("Body doesn't contain message", "error");
    return UT_FAILED;
   }

   $email->clear("attachment");
   $email->message("Test message", true);

   if(!$body=$email->body())
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/Test message/", $body))
   {
    $this->e->add("Body doesn't contain message", "error");
    return UT_FAILED;
   }
   if(!preg_match("/This is a multi-part message in MIME format./", $body))
   {
    $this->e->add("Multi-part message not found in body", "error");
    return UT_FAILED;
   }
   if(!preg_match("/--".$email->boundaryPrefix.$email->attachment[0]["boundary"]."/", $body))
   {
    $this->e->add("Attachments not found in body", "error");
    return UT_FAILED;
   }
   if(!preg_match("/--".$email->boundaryPrefix.$email->attachment[0]["boundary"]."--/", $body))
   {
    $this->e->add("Attachments close not found in body", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Type: text\/plain; charset=".preg_quote($email->charSet)."/", $body))
   {
    $this->e->add("Text message Content-Type not found in body", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Transfer-Encoding: 7bit/", $body))
   {
    $this->e->add("Content-Transfer-Encoding not found in body", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Disposition: inline/", $body))
   {
    $this->e->add("Content-Body not found in body", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
