<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/headers.sro.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."headers.php");

 class UT_Headers_Scope_Resolution_Operator extends UT_Headers
 {
  function _storeHeaders()
  {
   //store original headers
   $this->headers=headers_list();
  }

  function _restoreHeaders()
  {
   //clear current headers (if remove function is available)
   if(function_exists("header_remove"))
    header_remove();

   //restore original headers
   foreach($this->headers as $header)
    header($header, true);
  }


  function IsInitialized()
  {
   if(Headers::isInitialized())
   {
    $this->e->add("Headers:: shouldn't be initialized", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Header()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::header("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!in_array("key: value", headers_list()))
   {
    $this->e->add("Adding new header failed", "error", "Header not found");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Get()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::header("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("key")===false)
   {
    $this->e->add("Get new header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("key")!="value")
   {
    $this->e->add("Get new header failed", "error", "Value doesn't match");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //array
   if(($headers=Headers::get())===false)
   {
    $this->e->add("Get headers failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   foreach(headers_list() as $header)
   {
    list($key, $value)=explode(": ", $header, 2);

    if(!isset($headers[$key]))
    {
     $this->e->add("Get headers failed", "error", "Key not found");

     $this->_restoreHeaders();
     return UT_FAILED;
    }

    if($headers[$key]!=$value)
    {
     $this->e->add("Get headers failed", "error", "Value doesn't match");

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Delete()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   if(!function_exists("header_remove"))
   {
    $this->e->add("Header_remove function doesn't exist", "warning");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   if(!Headers::header("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("key")===false)
   {
    $this->e->add("Adding new header failed", "error", "Key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!Headers::delete("key"))
   {
    $this->e->add("Delete header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("key")!==false)
   {
    $this->e->add("Delete header failed", "error", "Key still exists");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //all headers
   if(!Headers::header("key", "value"))
   {
    $this->e->add("Adding new header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("key")===false)
   {
    $this->e->add("Adding new header failed", "error", "Key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!Headers::delete())
   {
    $this->e->add("Delete headers failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get())
   {
    $this->e->add("Delete headers failed", "error", "Keys still exist");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Cache()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //no-cache
   if(!Headers::cache())
   {
    $this->e->add("Adding cache header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(!preg_match("/^private, no-store, max-age=[0-9\-]+$/", Headers::get("Cache-Control")))
    {
     $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(!preg_match("/^no-cache, must-revalidate, max-age=[0-9\-]+$/", Headers::get("Cache-Control")))
    {
     $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::get("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(Headers::get("Pragma")!="private")
    {
     $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(Headers::get("Pragma")!="no-cache")
    {
     $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::get("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Expires")!="Thu, 01, Jan 1970 00:00:00 GMT")
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //private+store+now
   if(!Headers::cache("private", true, time(), false))
   {
    $this->e->add("Adding cache header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Cache-Control")!="private, max-age=0")
   {
    $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Pragma")!="private")
   {
    $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Expires")!=gmdate("D, d, M Y H:i:s T"))
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //public+no-store+now
   if(!Headers::cache("public", false, 10, true))
   {
    $this->e->add("Adding cache header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Cache-Control")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Cache key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Cache-Control")!="public, no-store, max-age=10")
   {
    $this->e->add("Adding cache header failed", "error", "Cache value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Pragma")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Pragma key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Pragma")!="public")
   {
    $this->e->add("Adding cache header failed", "error", "Pragma value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Expires")===false)
   {
    $this->e->add("Adding cache header failed", "error", "Expires key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Expires")!=gmdate("D, d, M Y H:i:s T", time()+10))
   {
    $this->e->add("Adding cache header failed", "error", "Expires value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Basic()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //no-cache
   if(!Headers::basic())
   {
    $this->e->add("Adding basic headers failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Cache-Control")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Cache key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(!preg_match("/^private, no-store, max-age=[0-9\-]+$/", Headers::get("Cache-Control")))
    {
     $this->e->add("Adding basic headers failed", "error", "Cache value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(!preg_match("/^no-cache, must-revalidate, max-age=[0-9\-]+$/", Headers::get("Cache-Control")))
    {
     $this->e->add("Adding basic headers failed", "error", "Cache value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::get("Pragma")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Pragma key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(isset($_SERVER["HTTPS"])&&isset($_SERVER["HTTP_USER_AGENT"])&&preg_match("/MSIE/i", $_SERVER["HTTP_USER_AGENT"])) //HTTPS + IE
   {
    if(Headers::get("Pragma")!="private")
    {
     $this->e->add("Adding basic headers failed", "error", "Pragma value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }
   else //no HTTPS or non IE
   {
    if(Headers::get("Pragma")!="no-cache")
    {
     $this->e->add("Adding basic headers failed", "error", "Pragma value doesn't match");
     $this->e->add(Headers::e);

     $this->_restoreHeaders();
     return UT_FAILED;
    }
   }

   if(Headers::get("Expires")===false)
   {
    $this->e->add("Adding basic headers failed", "error", "Expires key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Expires")!="Thu, 01, Jan 1970 00:00:00 GMT")
   {
    $this->e->add("Adding basic headers failed", "error", "Expires value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   $this->_restoreHeaders();
   return UT_OK;
  }


  function Filename()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //filename
   if(!Headers::filename("file.tst"))
   {
    $this->e->add("Adding filename header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")===false)
   {
    $this->e->add("Adding filename header failed", "error", "File key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding filename header failed", "error", "File value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //filename+no attachment
   if(!Headers::filename("file.tst", false))
   {
    $this->e->add("Adding filename header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")===false)
   {
    $this->e->add("Adding filename header failed", "error", "File key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")!="filename=\"file.tst\"")
   {
    $this->e->add("Adding filename header failed", "error", "File value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //no filename
   if(Headers::filename(""))
   {
    $this->e->add("Adding filename header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Type()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //type
   if(!Headers::type("text/html"))
   {
    $this->e->add("Adding type header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")===false)
   {
    $this->e->add("Adding type header failed", "error", "Size key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")!="text/html")
   {
    $this->e->add("Adding type header failed", "error", "Type value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //invalid type
   if(Headers::type("text"))
   {
    $this->e->add("Adding type header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //no type
   if(Headers::type(""))
   {
    $this->e->add("Adding type header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Size()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //type
   if(!Headers::size(2346))
   {
    $this->e->add("Adding size header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Length")===false)
   {
    $this->e->add("Adding size header failed", "error", "Size key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Length")!=2346)
   {
    $this->e->add("Adding size header failed", "error", "Size value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //invalid size
   if(Headers::type(-2346))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::type("text"))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //no siz3
   if(Headers::type(""))
   {
    $this->e->add("Adding size header should fail", "error");

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function File()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //filename
   if(!Headers::file("file.tst"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "File key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "File value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+type
   $this->_restoreHeaders();

   if(!Headers::file("file.tst", "text/html"))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")!="text/html")
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+auto type
   $this->_restoreHeaders();

   if(!Headers::file("file.css", true))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")!="attachment; filename=\"file.css\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")!="text/css")
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //file+type+size
   $this->_restoreHeaders();

   if(!Headers::file("file.tst", "text/html", 245345))
   {
    $this->e->add("Adding file header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")===false)
   {
    $this->e->add("Adding file header failed", "error", "Filename key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Disposition")!="attachment; filename=\"file.tst\"")
   {
    $this->e->add("Adding file header failed", "error", "Filename value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")===false)
   {
    $this->e->add("Adding file header failed", "error", "Type key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Type")!="text/html")
   {
    $this->e->add("Adding file header failed", "error", "Type value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Length")===false)
   {
    $this->e->add("Adding file header failed", "error", "Size key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Content-Length")!=245345)
   {
    $this->e->add("Adding file header failed", "error", "Size value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }


  function Server()
  {
   if(headers_sent())
   {
    $this->e->add("Please use a project that uses \"output buffering\"", "warning", "The W3C plugin will suffice");
    return UT_SKIPPED;
   }

   $this->_storeHeaders();

   //empty server
   if(!Headers::server())
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Server")!="")
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //jas+apache+php+ssl
   if(!Headers::server(true, true, true, true))
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^JAS/[0-9\.]+ Apache/[0-9\.]+ \([^\)]+\) OpenSSL/[0-9a-z\.]+ PHP/[0-9\.]+$!", Headers::get("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   //custom
   if(!Headers::server(false, false, false, false, "Test/1.0"))
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^Test/[0-9\.]+$!", Headers::get("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   //custom jas+php+custom
   if(!Headers::server("JAS/1.0", false, true, false, "Test/1.0"))
   {
    $this->e->add("Adding server header failed", "error");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(Headers::get("Server")===false)
   {
    $this->e->add("Adding server header failed", "error", "Server key not found");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }

   if(!preg_match("!^JAS/[0-9\.]+ PHP/[0-9\.]+ Test/[0-9\.]+$!", Headers::get("Server")))
   {
    $this->e->add("Adding server header failed", "error", "Server value doesn't match");
    $this->e->add(Headers::e);

    $this->_restoreHeaders();
    return UT_FAILED;
   }


   $this->_restoreHeaders();
   return UT_OK;
  }

 }

?>
