<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/http.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["plugins"]."unit_testing/ut/socket.php");
 require_once($_path["include"]."http.php");

 class UT_HTTP extends UT_Socket
 {
  function Constructor()
  {
   $object=new HTTP();

   if(!is_object($object)||get_class($object)!="HTTP")
   {
    $this->e->add("Constructor doesn't return a HTTP object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Clear()
  {
   $http=new HTTP();

   if(!$http->addVar("key", "value"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   $http->body="bodybodybodybody";

   if(!$http->clear())
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if($http->vars)
   {
    $this->e->add("Clear vars failed", "error");
    return UT_FAILED;
   }

   if($http->header)
   {
    $this->e->add("Clear header failed", "error");
    return UT_FAILED;
   }

   if($http->body)
   {
    $this->e->add("Clear body failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddVar()
  {
   $http=new HTTP();

   if(!$http->addVar("key", "value"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->vars)!=1)
   {
    $this->e->add("Vars should contain 1 value", "error");
    return UT_FAILED;
   }

   if($http->vars[0]["key"]!="key")
   {
    $this->e->add("First vars key should be: \"key\"", "error");
    return UT_FAILED;
   }

   if($http->vars[0]["value"]!="value")
   {
    $this->e->add("First vars value should be: \"value\"", "error");
    return UT_FAILED;
   }

   if(!$http->addVar("key2", "value2"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->vars)!=2)
   {
    $this->e->add("Vars should contain 2 values", "error");
    return UT_FAILED;
   }

   if($http->vars[0]["key"]!="key")
   {
    $this->e->add("First vars key should be: \"key\"", "error");
    return UT_FAILED;
   }

   if($http->vars[0]["value"]!="value")
   {
    $this->e->add("First vars value should be: \"value\"", "error");
    return UT_FAILED;
   }

   if($http->vars[1]["key"]!="key2")
   {
    $this->e->add("Second vars key should be: \"key2\"", "error");
    return UT_FAILED;
   }

   if($http->vars[1]["value"]!="value2")
   {
    $this->e->add("Second vars value should be: \"value2\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddHeader()
  {
   $http=new HTTP();

   if(!$http->addHeader("header: value", false, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=1)
   {
    $this->e->add("Header should contain 1 value", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header: value")
   {
    $this->e->add("First header should be: \"header: value\"", "error");
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value2", false, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=2)
   {
    $this->e->add("Header should contain 2 values", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header: value")
   {
    $this->e->add("First header should be: \"header: value\"", "error");
    return UT_FAILED;
   }

   if($http->header[1]!="header2: value2")
   {
    $this->e->add("Second header value should be: \"header2: value2\"", "error");
    return UT_FAILED;
   }

   if(!$http->addHeader("header: value3", false, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=3)
   {
    $this->e->add("Header should contain 3 values", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header: value")
   {
    $this->e->add("First header should be: \"header: value\"", "error");
    return UT_FAILED;
   }

   if($http->header[1]!="header2: value2")
   {
    $this->e->add("Second header value should be: \"header2: value2\"", "error");
    return UT_FAILED;
   }

   if($http->header[2]!="header: value3")
   {
    $this->e->add("Third header value should be: \"header: value3\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddHeaderReplace()
  {
   $http=new HTTP();

   if(!$http->addHeader("header: value", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header: value2", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=1)
   {
    $this->e->add("Header should contain 1 value", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header: value2")
   {
    $this->e->add("First header should be: \"header: value2\"", "error");
    return UT_FAILED;
   }


   if(!$http->clear())
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }


   if(!$http->addHeader("header: value", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value2", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header3: value3", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value3", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=3)
   {
    $this->e->add("Header should contain 3 values", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header: value")
   {
    $this->e->add("First header should be: \"header: value\"", "error");
    return UT_FAILED;
   }

   if($http->header[1]!="header2: value3")
   {
    $this->e->add("Second header should be: \"header2: value3\"", "error");
    return UT_FAILED;
   }

   if($http->header[2]!="header3: value3")
   {
    $this->e->add("Third header should be: \"header3: value3\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddHeaderAtStart()
  {
   $http=new HTTP();

   if(!$http->addHeader("header: value", false, true))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value2", false, true))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=2)
   {
    $this->e->add("Header should contain 2 values", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header2: value2")
   {
    $this->e->add("First header should be: \"header2: value2\"", "error");
    return UT_FAILED;
   }

   if($http->header[1]!="header: value")
   {
    $this->e->add("Second header should be: \"header: value\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddHeaderReplaceAtStart()
  {
   $http=new HTTP();

   if(!$http->addHeader("header: value", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value2", true, false))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header: value2", true, true))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(sizeof($http->header)!=2)
   {
    $this->e->add("Header should contain 2 values", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header: value2")
   {
    $this->e->add("First header should be: \"header: value2\"", "error");
    return UT_FAILED;
   }

   if($http->header[1]!="header2: value2")
   {
    $this->e->add("Second header should be: \"header2: value2\"", "error");
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value3", true, true))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!preg_match("/^header2/", $http->header[0]))
   {
    $this->e->add("First header should be: \"header2\"", "error");
    return UT_FAILED;
   }

   if($http->header[0]!="header2: value3")
   {
    $this->e->add("First header should be: \"header2: value3\"", "error");
    return UT_FAILED;
   }

   if($http->header[1]!="header: value2")
   {
    $this->e->add("Second header should be: \"header: value2\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SearchHeader()
  {
   $http=new HTTP();

   if(!$http->addHeader("header: value"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if(!$http->addHeader("header2: value2"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if($http->searchHeader("header")!="value")
   {
    $this->e->add("Header search should return: \"value\"", "error");
    return UT_FAILED;
   }

   if($http->searchHeader("header2")!="value2")
   {
    $this->e->add("Header search should return: \"value2\"", "error");
    return UT_FAILED;
   }

   if($http->searchHeader("header", true)!==0)
   {
    $this->e->add("Header entry search should return: \"0\"", "error");
    return UT_FAILED;
   }

   if($http->searchHeader("header2", true)!==1)
   {
    $this->e->add("Header entry search should return: \"1\"", "error");
    return UT_FAILED;
   }

   if(!$http->addHeader("header3"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if($http->searchHeader("header3")!==false)
   {
    $this->e->add("Header search should fail", "error");
    return UT_FAILED;
   }

   if($http->searchHeader("header3", true)!==false)
   {
    $this->e->add("Header entry search should fail", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddCookie()
  {
   $http=new HTTP();

   if(!$http->addCookie("key", "value"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if($http->searchHeader("Cookie")!="key=value")
   {
    $this->e->add("Cookie header should be: \"key=value\"", "error");
    return UT_FAILED;
   }

   if(!$http->addCookie("key2", "value2"))
   {
    $this->e->add($http->e);
    return UT_FAILED;
   }

   if($http->searchHeader("Cookie")!="key=value; key2=value2")
   {
    $this->e->add("Cookie header should be: \"key=value; key2=value2\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
