<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/plugin.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."plugin.php");

 class UT_Plugin extends UT_Classes
 {
  function Constructor()
  {
   $object=new Plugin("unit_testing");

   if(!is_object($object)||get_class($object)!="Plugin")
   {
    $this->e->add("Constructor doesn't return a Plugin object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function IsEnabled()
  {
   global $_config;

   $plugin=new Plugin("unit_testing");

   if($plugin->isEnabled()!==true)
   {
    $this->e->add("Plugin should be enabled", "error");
    return UT_FAILED;
   }

   $usernr=$_config["usernr"];
   $_config["usernr"]=$_config["rootnr"];

   $plugin->setStatus(false);
   $isEnabled=$plugin->isEnabled();
   $plugin->setStatus(true);

   $_config["usernr"]=$usernr;

   if($isEnabled!==false)
   {
    $this->e->add("Plugin should be disabled", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function HasBackend()
  {
   $plugin=new Plugin("unit_testing");
   $plugin->readInfo();

   if($plugin->hasBackend()===true)
   {
    $this->e->add("Plugin shouldn't have a backend", "error");
    return UT_FAILED;
   }

   $plugin=new Plugin("pager");
   $plugin->readInfo();

   if($plugin->hasBackend()===false)
   {
    $this->e->add("Plugin should have a backend", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Export()
  {
   $plugin=new Plugin("unit_testing");
   $plugin->readInfo();

   ob_start();
   if($plugin->export("asdasdasdasdadsasdasdasdasdasdadasdasdasdasdasdasdasd")) //non supported compression
   {
    ob_end_clean();
    $this->e->add("Plugins shouldn't be able to be exported using a non supported compression method", "error");
    return UT_FAILED;
   }
   ob_end_clean();

   return UT_OK;
  }

  function Import()
  {
   global $_config, $_path;

   $usernr=$_config["usernr"];

   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }

   $plugin=new Plugin(false);

   $_config["usernr"]=$_config["rootnr"];
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $_config["usernr"]=$usernr;


   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }

   if(!$imported)
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ImportOverwrite()
  {
   global $_config, $_path;

   $usernr=$_config["usernr"];

   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }

   $plugin=new Plugin(false);

   $_config["usernr"]=$_config["rootnr"];
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $_config["usernr"]=$usernr;

   $_config["usernr"]=$_config["rootnr"];
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $_config["usernr"]=$usernr;

   $_config["usernr"]=$_config["rootnr"];
   $imported=$plugin->importOverwrite("unit_testing_test");
   $_config["usernr"]=$usernr;

   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }

   if(!$imported)
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Remove()
  {
   global $_config, $_path;

   $usernr=$_config["usernr"];

   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }

   $plugin=new Plugin(false);

   $_config["usernr"]=$_config["rootnr"];
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $_config["usernr"]=$usernr;

   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $removed=$remove->remove();
    $_config["usernr"]=$usernr;
   }
   else
    $removed=false;

   if(!$removed)
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetStatus()
  {
   global $_config, $_path;

   $plugin=new Plugin("unit_testing");

   if($plugin->enabled!==true)
   {
    $this->e->add("Plugin should be enabled", "error");
    return UT_FAILED;
   }

   $usernr=$_config["usernr"];

   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }


   $plugin=new Plugin(false);

   $_config["usernr"]=$_config["rootnr"];
   $plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_testtest-0.1.tgz");
   $_config["usernr"]=$usernr;


   if(in_array("unit_testing_test", plugins()))
   {
    $remove=new Plugin("unit_testing_test");
    $enabled=$remove->enabled;

    $_config["usernr"]=$_config["rootnr"];
    $remove->remove();
    $_config["usernr"]=$usernr;
   }
   else
    $enabled=false;

   if($enabled!==false) //newly import plugin always disabled by defalut
   {
    $this->e->add("Plugin should be disabled", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetStatus()
  {
   global $_config;

   $plugin=new Plugin("unit_testing");

   $usernr=$_config["usernr"];
   $_config["usernr"]=$_config["rootnr"];

   $plugin->setStatus(false);
   $enabled1=$plugin->enabled;
   $plugin->setStatus(true);
   $enabled2=$plugin->enabled;

   $_config["usernr"]=$usernr;

   if($enabled1!==false)
   {
    $this->e->add("Plugin should be disabled", "error");
    return UT_FAILED;
   }
   if($enabled2!==true)
   {
    $this->e->add("Plugin should be enabled", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ReadInfo()
  {
   global $_path;

   $plugin=new Plugin("unit_testing");

   if(!$plugin->readInfo())
   {
    $this->e->add($plugin->e, "error");
    return UT_FAILED;
   }

   if($plugin->title=="")
   {
    $this->e->add("Plugin title not set", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Config()
  {
   $plugin=new Plugin("unit_testing");

   if(!$plugin->config("Units")) //jas uses this one in code
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   if($plugin->config("asdasdasdasdadsasdasdasdasdasdadasdasdasdasdasdasdasd")) //bogus... we hope
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function InitMainClass()
  {
   $plugin=new Plugin("unit_testing");

   if(!$plugin->initClass())
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function InitBackendClass()
  {
   $plugin=new Plugin("unit_testing");

   if($plugin->initBackendClass())
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   $plugin=new Plugin("pager");

   if(!$plugin->initBackendClass())
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CallExists()
  {
   $plugin=new Plugin("unit_testing");

   if(!$plugin->callExists("main"))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   if($plugin->callExists("asdasdasdasdadsasdasdasdasdasdadasdasdasdasdasdasdasd"))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

    return UT_OK;
  }

  function Call()
  {
   $plugin=new Plugin("unit_testing");

   if(!$plugin->call("output", false, "list_units", false, false))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   if($plugin->call("asdasdasdasdadsasdasdasdasdasdadasdasdasdasdasdasdasd"))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   if($plugin->call("main", "backend"))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Main()
  {
   $plugin=new Plugin("unit_testing");

   if(!$plugin->main())
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $plugin=new Plugin("unit_testing");

   if(!$plugin->output("list_units", false, false))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   if($plugin->output("asd", false, false))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Backend()
  {
   $plugin=new Plugin("unit_testing");

   if($plugin->backend())
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddVar()
  {
   $plugin=new Plugin("unit_testing");

   if($plugin->vars)
   {
    $this->e->add("Plugins vars not empty", "error");
    return UT_FAILED;
   }

   $plugin->addVar("asd", "123");

   if(!isset($plugin->vars["asd"])||$plugin->vars["asd"]!=123)
   {
    $this->e->add("Plugins vars test value not found", "error");
    return UT_FAILED;
   }

   $plugin->addVar("asd", "123");

   if(!$plugin->e->any("warning"))
   {
    $this->e->add("Plugins vars overwrite should generate a warning", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function URL()
  {
   $plugin=new Plugin("unit_testing");

   if($plugin->url()!="?")
   {
    $this->e->add("Plugins URL should be \"?\"", "error");
    return UT_FAILED;
   }

   $plugin->addVar("asd", "123");

   if($plugin->url()!="?asd=123&amp;")
   {
    $this->e->add("Plugins URL should be \"?ad=123&amp;\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_BasePlugin extends UT_Plugin
 {
  function Constructor()
  {
   $plugin=new Plugin("unit_testing");
   $plugin->config("main");

   $object=new BasePlugin($plugin, $plugin->config);

   if(!is_object($object)||get_class($object)!="BasePlugin")
   {
    $this->e->add("Constructor doesn't return a BasePlugin object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Main()
  {
   $plugin=new Plugin("unit_testing");
   $plugin->config("main");

   $bp=new BasePlugin($plugin, $plugin->config);

   if($bp->main()!==false)
   {
    $this->e->add("BasePlugin main function should return false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $plugin=new Plugin("unit_testing");
   $plugin->config("Units");

   $bp=new BasePlugin($plugin, $plugin->config);

   if($bp->output()!="")
   {
    $this->e->add("BasePlugin output function should return nothing", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }


 class UT_BasePluginBackend extends UT_Plugin
 {
  function Constructor()
  {
   $plugin=new Plugin("unit_testing");

   $object=new BasePluginBackend($plugin);

   if(!is_object($object)||get_class($object)!="BasePluginBackend")
   {
    $this->e->add("Constructor doesn't return a BasePluginBackend object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $plugin=new Plugin("unit_testing");

   $bp=new BasePluginBackend($plugin);

   if($bp->output()!="")
   {
    $this->e->add("BasePluginBackend output function should return nothing", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_BasePluginSetup extends UT_Plugin
 {
  function Constructor()
  {
   $plugin=new Plugin("unit_testing");

   $object=new BasePluginSetup($plugin);

   if(!is_object($object)||get_class($object)!="BasePluginSetup")
   {
    $this->e->add("Constructor doesn't return a BasePluginSetup object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
