<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/storage.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."data.php");

 class UT_Storage extends UT_Classes
 {
  var $_module="unit_testing_testmodule";
  var $_db="default";

  function Constructor()
  {
   $d=new Data($this->_module, false, false, Array("db" => $this->_db));
   $object=new Storage($d, false);

   if(!is_object($object)||get_class($object)!="Storage")
   {
    $this->e->add("Constructor doesn't return a Storage object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetConfig()
  {
   //default config
   $d=new Data($this->_module, false, false, Array("db" => "default"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get default config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if(!$db["type"])
   {
    $this->e->add("Config type empty", "error");
    return UT_FAILED;
   }


   //JAS session config
   $d=new Data($this->_module, false, false, Array("db" => "session"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get session config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="session")
   {
    $this->e->add("Config type should be \"session\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if(!$db["host"]) //session variable
   {
    $this->e->add("Config host empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }


   //JAS Mime config
   $d=new Data($this->_module, false, false, Array("db" => "mime"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get mime config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="xml")
   {
    $this->e->add("Config type should be \"xml\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if(!$db["host"]) //first part of path variable
   {
    $this->e->add("Config host empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if(!$db["path"]) //second part of path variable
   {
    $this->e->add("Config path empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }


   //JAS XML config
   $d=new Data($this->_module, false, false, Array("db" => "xml"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get xml config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="xml")
   {
    $this->e->add("Config type should be \"xml\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if(!$db["host"]) //first part of path variable
   {
    $this->e->add("Config host empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if(!$db["path"]) //second part of path variable
   {
    $this->e->add("Config path empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }


   //custom config: full
   $d=new Data($this->_module, false, false, Array("db" => "mysql://user:pass@localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]!="user") 
   {
    $this->e->add("Config user should be \"user\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]!="pass") 
   {
    $this->e->add("Config pass should be \"pass\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no pass
   $d=new Data($this->_module, false, false, Array("db" => "mysql://user@localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]!="user") 
   {
    $this->e->add("Config user should be \"user\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no pass
   $d=new Data($this->_module, false, false, Array("db" => "mysql://user:@localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]!="user") 
   {
    $this->e->add("Config user should be \"user\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user 
   $d=new Data($this->_module, false, false, Array("db" => "mysql://:pass@localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]!="pass") 
   {
    $this->e->add("Config pass should be \"pass\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user+pass
   $d=new Data($this->_module, false, false, Array("db" => "mysql://:@localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user+pass
   $d=new Data($this->_module, false, false, Array("db" => "mysql://@localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user+pass
   $d=new Data($this->_module, false, false, Array("db" => "mysql://localhost/database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]!="localhost") 
   {
    $this->e->add("Config host should be \"localhost\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user+pass+host
   $d=new Data($this->_module, false, false, Array("db" => "mysql:///database#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]) 
   {
    $this->e->add("Config host is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]!="/database") 
   {
    $this->e->add("Config path should be \"/database\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user+pass+host+path
   $d=new Data($this->_module, false, false, Array("db" => "mysql://#channel"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="mysql")
   {
    $this->e->add("Config type should be \"mysql\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]) 
   {
    $this->e->add("Config host is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]) 
   {
    $this->e->add("Config path is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]!="channel") 
   {
    $this->e->add("Config channel should be \"channel\"", "error");
    return UT_FAILED;
   }


   //custom config: no user+pass+host+path+channel, odd storage type
   $d=new Data($this->_module, false, false, Array("db" => "ASDASDA1234124://"));
   $s=new Storage($d, false);

   if(!($db=$s->getConfig()))
   {
    $this->e->add("Unable to get custom config", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!is_array($db))
   {
    $this->e->add("Config is not an array", "error");
    return UT_FAILED;
   }

   if(!isset($db["type"]))
   {
    $this->e->add("Config type not set", "error");
    return UT_FAILED;
   }

   if($db["type"]!="asdasda1234124")
   {
    $this->e->add("Config type should be \"asdasda1234124\"", "error");
    return UT_FAILED;
   }

   if(!isset($db["host"]))
   {
    $this->e->add("Config host not set", "error");
    return UT_FAILED;
   }

   if($db["host"]) 
   {
    $this->e->add("Config host is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["path"]))
   {
    $this->e->add("Config path not set", "error");
    return UT_FAILED;
   }

   if($db["path"]) 
   {
    $this->e->add("Config path is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["user"]))
   {
    $this->e->add("Config user not set", "error");
    return UT_FAILED;
   }

   if($db["user"]) 
   {
    $this->e->add("Config user is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["pass"]))
   {
    $this->e->add("Config pass not set", "error");
    return UT_FAILED;
   }

   if($db["pass"]) 
   {
    $this->e->add("Config pass is not empty", "error");
    return UT_FAILED;
   }

   if(!isset($db["channel"]))
   {
    $this->e->add("Config channel not set", "error");
    return UT_FAILED;
   }

   if($db["channel"]) 
   {
    $this->e->add("Config channel is not empty", "error");
    return UT_FAILED;
   }


   //custom config: incorrect type 
   $d=new Data($this->_module, false, false, Array("db" => "://"));
   $s=new Storage($d, false);

   if(($db=$s->getConfig()))
   {
    $this->e->add("Should not be able to get incorrect config", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Clear()
  {
   return UT_SKIPPED;
  }

  function Next()
  {
   return UT_SKIPPED;
  }

  function TreeUp()
  {
   return UT_SKIPPED;
  }

  function TreeDown()
  {
   return UT_SKIPPED;
  }

  function Unique()
  {
   return UT_SKIPPED;
  }

  function Lost()
  {
   return UT_SKIPPED;
  }

  function FilterDataSet()
  {
   return UT_SKIPPED;
  }

  function GatherFilterRights()
  {
   return UT_SKIPPED;
  }

  function Sort()
  {
   return UT_SKIPPED;
  }

  function SortField()
  {
   return UT_SKIPPED;
  }

  function FilterShowTotal()
  {
   return UT_SKIPPED;
  }

  function FilterCount()
  {
   return UT_SKIPPED;
  }

  function GetValues()
  {
   return UT_SKIPPED;
  }

  function CheckRights()
  {
   return UT_SKIPPED;
  }

  function InsertID()
  {
   return UT_SKIPPED;
  }

  function Table()
  {
   return UT_SKIPPED;
  }

  function Get()
  {
   return UT_SKIPPED;
  }

  function Add()
  {
   return UT_SKIPPED;
  }

  function Edit()
  {
   return UT_SKIPPED;
  }

  function Delete()
  {
   return UT_SKIPPED;
  }

  function Create()
  {
   return UT_SKIPPED;
  }

  function Alter()
  {
   return UT_SKIPPED;
  }

 }

?>
