<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/template.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/xml.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."template.php");

 class UT_Template extends UT_XML
 {
  var $testHTML="<html>
 <head>
  <title>#title#</title>

 </head>

 <body>
  #body#
 </body>

</html>";

  function Constructor()
  {
   $object=new Template($this->testHTML);

   if(!is_object($object)||get_class($object)!="Template")
   {
    $this->e->add("Constructor doesn't return a Template object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $template=new Template($this->testHTML);

   if($template->output()!=$this->testHTML)
   {
    $this->e->add("Template output doesn't match input", "error");
    return UT_FAILED;
   }

   $template->addInsideAtEnd("html->body", "asdasdasdasdasd");

   if(!preg_match("/asdasdasdasdasd/", $template->output()))
   {
    $this->e->add("Template output should contain new text", "error");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $template->addInsideAtEnd("html->asdasd", "asdasdasdasdasd");

   if(!preg_match("/warning/", $template->output()))
   {
    $this->e->add("Template output should contain a warning", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Data()
  {
   $template=new Template($this->testHTML, true);

   $d=Data::cache("empty");
   $d->addField("title", "text", "show");
   $d->addField("body", "text", "show");

   $d->setValue("title", "Unit testing");
   $d->setValue("body", "is teh BomB");
   $d->vmax=1;

   $template->data($d);

   if($template->output()!="<html>
 <head>
  <title>Unit testing</title>

 </head>

 <body>
  is teh BomB
 </body>

</html>")
   {
    $this->e->add("Template output is incorrect", "error");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  /*
   *
   * Analyze
   */

  function Analyze()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//show fields
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
#text#
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "show fields");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   if($d->check("use", "show")!=Array("r", "text"))
   {
    $this->e->add("Template analyzation should set show fields", "error", "show fields");
    return UT_FAILED;
   }


//incorrect general marker
   $template=new Template("
<h1><!_fail></h1>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "Incorrect general marker: _fail");
    return UT_FAILED;
   }


//incorrect field marker
   $template=new Template("
<h1><!fail></h1>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "Incorrect field marker: fail");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TemplateMarker()
  {
   $d=Data::cache("categories");

//clean
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "clean");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->getName(0)!="_template")
   {
    $this->e->add("No _template marker set as first tag", "error", "clean");
    return UT_FAILED;
   }


//short comment
   $template=new Template("<!_template>
<h1><!_module_title>Module<!/_module_title></h1>
<!/_template>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "short comment");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->getName(0)!="_template")
   {
    $this->e->add("No _template marker set as first tag", "error", "short comment");
    return UT_FAILED;
   }

   if($template->getName(1)=="_template")
   {
    $this->e->add("_Template marker set also set as second", "error", "short comment");
    return UT_FAILED;
   }


   return UT_OK;
  }


  /*
   *
   * Template information
   */

  function ShowCount()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<h1><!_total_rows>Total<!/_total_rows></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->showCount())
   {
    $this->e->add("Template should show count", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker#
   $template=new Template("
<h1>#_total_rows#</h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->showCount())
   {
    $this->e->add("Template should show count", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//no show count
   $template=new Template("
<h1>#_current_rows#</h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "no show count");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->showCount())
   {
    $this->e->add("Template should not show count", "error", "no show count");
    return UT_FAILED;
   }


//no analyze
   $template=new Template("
<h1>#_current_rows#</h1>
");

   if($template->showCount()!==false)
   {
    $this->e->add("Template show count should fail", "error", "no analyze");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function GetLevel()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<h1><!name>Total<!/name></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->getLevel()!=0)
   {
    $this->e->add("Template level should be \"0\"", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker#
   $template=new Template("
<h1>#name#</h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->getLevel()!=0)
   {
    $this->e->add("Template level should be \"0\"", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//!block level 1
   $template=new Template("
<h1><!catnr.name>Total<!/catnr.name></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->getLevel()!=1)
   {
    $this->e->add("Template level should be \"1\"", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker#
   $template=new Template("
<h1>#catnr.catnr.name#</h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->getLevel()!=2)
   {
    $this->e->add("Template level should be \"2\"", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//no analyze
   $template=new Template("
<h1>#name#</h1>
");

   if($template->getLevel()!==false)
   {
    $this->e->add("Template get level should fail", "error", "no analyze");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function MarkerExists()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<h1><!_module_title>Total<!/_module_title></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->markerExists("_module_title"))
   {
    $this->e->add("Template marker: \"_module_title\" should exist", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->markerExists("_module_title", "general"))
   {
    $this->e->add("Template general marker: \"_module_title\" should exist", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->markerExists("_module_title", "field"))
   {
    $this->e->add("Template field marker: \"_module_title\" should not exist", "error", "!block");
    return UT_FAILED;
   }


//#marker#
   $template=new Template("
<h1>#name#</h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->markerExists("name"))
   {
    $this->e->add("Template marker: \"name\" should exist", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->markerExists("name", "field"))
   {
    $this->e->add("Template field marker: \"name\" should exist", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->markerExists("name", "general"))
   {
    $this->e->add("Template general marker: \"name\" should not exist", "error", "#marker#");
    return UT_FAILED;
   }


//no analyze
   $template=new Template("
<h1>#name#</h1>
");

   if($template->markerExists("name")!==false)
   {
    $this->e->add("Template marker exist should fail", "error", "no analyze");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function ShowFields()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<!name>Name<!/name>
<!text>Text<!/text>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($d->check("use", "show")!=Array("r", "name", "text"))
   {
    $this->e->add("Template data show field incorrect", "error", "!block");
    return UT_FAILED;
   }


//#marker#
   $template=new Template("
#name#
#nr#
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($d->check("use", "show")!=Array("r", "name", "nr"))
   {
    $this->e->add("Template data show field incorrect", "error", "#marker#");
    return UT_FAILED;
   }


//reference
   $template=new Template("
#nr#
#catnr.name#
#catnr.text#
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "reference");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($d->check("use", "show")!=Array("r", "nr", "catnr"))
   {
    $this->e->add("Template data show field incorrect", "error", "reference");
    return UT_FAILED;
   }

   if($d->reference["catnr"]["data"]->check("use", "show")!=Array("r", "name", "text"))
   {
    $this->e->add("Template data reference show field incorrect", "error", "reference");
    return UT_FAILED;
   }


//reference
   $template=new Template("
#nr#
#catnr.name#
#catnr.catnr.name#
#catnr.catnr.catnr.name#
#catnr.catnr.catnr.text#
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "reference");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($d->check("use", "show")!=Array("r", "nr", "catnr"))
   {
    $this->e->add("Template data show field incorrect", "error", "reference");
    return UT_FAILED;
   }

   if($d->reference["catnr"]["data"]->check("use", "show")!=Array("r", "name", "catnr"))
   {
    $this->e->add("Template data reference show field incorrect", "error", "reference");
    return UT_FAILED;
   }

   if($d->reference["catnr"]["data"]->reference["catnr"]["data"]->check("use", "show")!=Array("r", "name", "catnr"))
   {
    $this->e->add("Template data reference show field incorrect", "error", "reference");
    return UT_FAILED;
   }

   if($d->reference["catnr"]["data"]->reference["catnr"]["data"]->reference["catnr"]["data"]->check("use", "show")!=Array("r", "name", "text"))
   {
    $this->e->add("Template data reference show field incorrect", "error", "reference");
    return UT_FAILED;
   }


   return UT_OK;
  }


  /*
   *
   * Populate
   */

  function PopulateModule()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Category</h1>
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<h1 class=\"#_module_name#\"><!_module_title></h1>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1 class=\"categories\">Category</h1>
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateTitleFields()
  {
   $d=Data::cache("categories");

   $d->setValue("name", "Name!");

//!block
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
<h2><!_title_field_title>Title<!/_title_field_title></h2>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Category</h1>
<h2>Name</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<h1 class=\"#_module_name#\"><!_module_title></h1>
<h2 class=\"#_title_field_name# #_title_field_type#\"><!_title_field_title>: <!_title_field_value></h2>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1 class=\"categories\">Category</h1>
<h2 class=\"name text\">Name: Name!</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  
  function PopulateFields()
  {
   $d=Data::cache("categories");
   $d->setShowFields("nr", "text", "module");

   $d->setValue("name", "Name!");
   $d->setValue("text", "TEXT");
   $d->setValue("module", "ModuleS");

//!block
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
<h2><!_title_field_title>Title<!/_title_field_title></h2>
<!_fields><h3><!_field_title>Field title<!/_field_title></h3>
<!/_fields>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Category</h1>
<h2>Name</h2>
<h3>Explanation</h3>
<h3>Module</h3>
<h3>nr</h3>
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<h1 class=\"#_module_name#\"><!_module_title></h1>
<h2 class=\"#_title_field_name# #_title_field_type#\"><!_title_field_title></h2>
<!_fields><h3 class=\"#_field_name# #_field_type#\"><!_field_title>: <!_field_value></h3>
<!/_fields>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1 class=\"categories\">Category</h1>
<h2 class=\"name text\">Name</h2>
<h3 class=\"text text\">Explanation: TEXT</h3>
<h3 class=\"module modules\">Module: ModuleS</h3>
<h3 class=\"nr nr\">nr: 0</h3>
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//show fields based on template
   $template=new Template("
<h1><!_module_title>Module<!/_module_title></h1>
<h2><!_title_field_title>Title<!/_title_field_title></h2>
#text#
<!_fields><h3><!_field_title>Field title<!/_field_title></h3>
<!/_fields>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "show fields");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "show fields");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Category</h1>
<h2>Name</h2>
TEXT
<h3>Explanation</h3>
")
   {
    $this->e->add("Template output is incorrect", "error", "show fields");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   return UT_OK;
  }


  function PopulateRows()
  {
   $d=Data::cache("categories");
   $d->setValue("show_count", 25);
   $d->vmax=10;

//!block
   $template=new Template("
Rows: <!_current_rows>Rows<!/_current_rows>
Total rows: <!_total_rows>Total<!/_total_rows>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Rows: 10
Total rows: 25
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
Rows: #_current_rows#
Total rows: <!_total_rows>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Rows: 10
Total rows: 25
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateFieldTypeTitle()
  {
   $d=Data::cache("categories");

//!block
   $template=new Template("
<h1><!name_title>Field<!/name_title></h1>
<h2><!name_type>Type<!/name_type></h2>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name</h1>
<h2>text</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<h1>#name_title#</h1>
<h2><!name_type></h2>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name</h1>
<h2>text</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateValues()
  {
   $d=Data::cache("categories");
   $d->setValue("name", "Name!");
   $d->setValue("text", "TEXT");

//!block
   $template=new Template("
<h1><!name>Name<!/name></h1>
<h2><!text>Text<!/text></h2>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name!</h1>
<h2>TEXT</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<h1><!name></h1>
<h2>#text#</h2>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name!</h1>
<h2>TEXT</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateNoValuesRow()
  {
   $d=Data::cache("categories");

   //create fake subcats reference
   $d->addReference("subcats", "categories", "catnr", "1:*", "nr");

   //create fake subcats subcats reference
   $dtmp=$d->getReferenceData("subcats");
   $dtmp->addReference("subcats", "categories", "catnr", "1:*", "nr");

   //no values
   $d->clearValues();
   $dtmp=$d->getReferenceData("subcats");
   $dtmp->clearValues();
   $dtmp=$dtmp->getReferenceData("subcats");
   $dtmp->clearValues();


//!block
   $template=new Template("
<!catnr>CatNR<!/catnr>
<!_row><h1><!name>Name<!/name></h1>
<h2><!text>Text<!/text></h2>
<!/_row>
<!catnr.updated>Updated<!/catnr.updated>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
".$d->getDefaultValue("catnr")."

".$d->reference["catnr"]["data"]->getDefaultValue("updated")."
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
#subcats.catnr#
<!_row><h1><!name></h1>
<h2>#text#</h2>
<!/_row>
<!catnr.catnr.updated>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
".$d->reference["subcats"]["data"]->getDefaultValue("catnr")."

".$d->reference["catnr"]["data"]->reference["catnr"]["data"]->getDefaultValue("updated")."
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }
  
  
  function PopulateValuesRow()
  {
   $d=Data::cache("categories");

   $d->setValue("name", "Name 1", 0);
   $d->setValue("text", "Text 1", 0);

   $d->setValue("name", "Name 2", 1);
   $d->setValue("text", "Text 2", 1);

//!block
   $template=new Template("
<!_row><h1><!name>Name<!/name></h1>
<h2><!text>Text<!/text></h2>

<!/_row>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name 1</h1>
<h2>Text 1</h2>

<h1>Name 2</h1>
<h2>Text 2</h2>

")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<!_row><h1><!name></h1>
<h2>#text#</h2>

<!/_row>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name 1</h1>
<h2>Text 1</h2>

<h1>Name 2</h1>
<h2>Text 2</h2>

")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateValuesMultipleRow()
  {
   $d=Data::cache("categories");

   $d->setValue("name", "Name 1", 0);
   $d->setValue("text", "Text 1", 0);

   $d->setValue("name", "Name 2", 1);
   $d->setValue("text", "Text 2", 1);

//!block
   $template=new Template("
<!_row><h1><!name>Name<!/name></h1>
<!/_row>

<!_row><h2><!text>Text<!/text></h2>
<!/_row>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name 1</h1>
<h1>Name 2</h1>


<h2>Text 1</h2>
<h2>Text 2</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<!_row><h1><!name></h1>
<!/_row>

<!_row><h2>#text#</h2>
<!/_row>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name 1</h1>
<h1>Name 2</h1>


<h2>Text 1</h2>
<h2>Text 2</h2>
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateValuesRowFieldEntry()
  {
   $d=Data::cache("categories", "view", "item");

   //create fake subcats reference
   $d->addReference("subcats", "categories", "catnr", "1:*", "nr");

   //create fake subcats subcats reference
   $dtmp=$d->getReferenceData("subcats");
   $dtmp->addReference("subcats", "categories", "catnr", "1:*", "nr");


//!block
   $template=new Template("
Name: <!name>Name<!/name>
Sub: <!subcats[1].name>Sub<!/subcats[1].name>
Module: <!module[0]>Module<!/module[0]>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes")); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."
Sub: ".$d->v["subcats"][1]["name"]."
Module: ".$d->v["module"][0]."
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
Name: #name#
Sub: <!subcats[1].name> #subcats[0].subcats[1].name#
Module: #module[1]# <!subcats[1].module[0]>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   $d->clearValues();
   $d->setValue("nr", 1);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes")); 
   $dtmp=$d->getReferenceData("subcats");
   $dtmp->setValue("module", Array("categories", "notes"), 1); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."
Sub: ".$d->v["subcats"][1]["name"]." ".$d->v["subcats"][0]["subcats"][1]["name"]."
Module: ".$d->v["module"][1]." ".$d->v["subcats"][1]["module"][0]."
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//illegal field/reference
   $template=new Template("
Name: #asd[0]#
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "illegal field/reference");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

//illegal field type
   $template=new Template("
Name: #name[0]#
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "illegal field type");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

//illegal reference type
   $template=new Template("
Name: #catnr[0]#
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "illegal reference tpye");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

   return UT_OK;
  }

  
  function PopulateValuesRowFieldEntries()
  {
   global $_config;

   $d=Data::cache("categories", "view", "item");

   //create fake subcats reference
   $d->addReference("subcats", "categories", "catnr", "1:*", "nr");

   //create fake subcats subcats reference
   $dtmp=$d->getReferenceData("subcats");
   $dtmp->addReference("subcats", "categories", "catnr", "1:*", "nr");

//!block
   $template=new Template("
Name: <!name>Name<!/name>
<!_row subcats>
Sub: <!subcats.name>Sub<!/subcats.name> #subcats.nr#
Module: <!module[0]>Module<!/module[0]>
<!/_row>
NR: <!nr>NR<!/nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes")); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."

Sub: ".$d->v["subcats"][0]["name"]." ".$d->v["subcats"][0]["nr"]."
Module: ".$d->v["module"][0]."

Sub: ".$d->v["subcats"][1]["name"]." ".$d->v["subcats"][1]["nr"]."
Module: ".$d->v["module"][0]."

Sub: ".$d->v["subcats"][2]["name"]." ".$d->v["subcats"][2]["nr"]."
Module: ".$d->v["module"][0]."

Sub: ".$d->v["subcats"][3]["name"]." ".$d->v["subcats"][3]["nr"]."
Module: ".$d->v["module"][0]."

Sub: ".$d->v["subcats"][4]["name"]." ".$d->v["subcats"][4]["nr"]."
Module: ".$d->v["module"][0]."

Sub: ".$d->v["subcats"][5]["name"]." ".$d->v["subcats"][5]["nr"]."
Module: ".$d->v["module"][0]."

Sub: ".$d->v["subcats"][6]["name"]." ".$d->v["subcats"][6]["nr"]."
Module: ".$d->v["module"][0]."

NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
Name: <!name>
<!_row module>
Module: #module#
<!/_row>
NR: #nr#
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   $d->clearValues();
   $d->setValue("nr", 1);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "#marker# and !short");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes")); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."

Module: ".$d->v["module"][0]."

Module: ".$d->v["module"][1]."

NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//multple row
   $template=new Template("
Name: <!name>
<!_row subcats>
 Sub: #subcats.name# <!subcats.nr>
<!_row subcats.module> Module: #subcats.module#
<!/_row><!/_row><!_row module>
 Module: #module#
<!/_row>
NR: <!nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "multple row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "multple row");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "config"), 2); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "multple row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."

 Sub: ".$d->v["subcats"][0]["name"]." ".$d->v["subcats"][0]["nr"]."
 Module: ".$d->v["subcats"][0]["module"][0]."
 Module: ".$d->v["subcats"][0]["module"][1]."

 Sub: ".$d->v["subcats"][1]["name"]." ".$d->v["subcats"][1]["nr"]."
 Module: ".$d->v["subcats"][1]["module"][0]."

 Sub: ".$d->v["subcats"][2]["name"]." ".$d->v["subcats"][2]["nr"]."
 Module: ".$d->v["subcats"][2]["module"][0]."
 Module: ".$d->v["subcats"][2]["module"][1]."

 Sub: ".$d->v["subcats"][3]["name"]." ".$d->v["subcats"][3]["nr"]."

 Sub: ".$d->v["subcats"][4]["name"]." ".$d->v["subcats"][4]["nr"]."

 Sub: ".$d->v["subcats"][5]["name"]." ".$d->v["subcats"][5]["nr"]."

 Sub: ".$d->v["subcats"][6]["name"]." ".$d->v["subcats"][6]["nr"]."

 Module: ".$d->v["module"][0]."

 Module: ".$d->v["module"][1]."

NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "multple row");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//multi level
   $template=new Template("
Name: <!name>
<!_row subcats>
 Sub: #subcats.name# <!subcats.nr>
 <!_row subcats.module>Module: #subcats.module#
 <!/_row><!/_row>
NR: <!nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "multi level");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "multi level");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "config"), 2); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "multi level");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."

 Sub: ".$d->v["subcats"][0]["name"]." ".$d->v["subcats"][0]["nr"]."
 Module: ".$d->v["subcats"][0]["module"][0]."
 Module: ".$d->v["subcats"][0]["module"][1]."
 
 Sub: ".$d->v["subcats"][1]["name"]." ".$d->v["subcats"][1]["nr"]."
 Module: ".$d->v["subcats"][1]["module"][0]."
 
 Sub: ".$d->v["subcats"][2]["name"]." ".$d->v["subcats"][2]["nr"]."
 Module: ".$d->v["subcats"][2]["module"][0]."
 Module: ".$d->v["subcats"][2]["module"][1]."
 
 Sub: ".$d->v["subcats"][3]["name"]." ".$d->v["subcats"][3]["nr"]."
 
 Sub: ".$d->v["subcats"][4]["name"]." ".$d->v["subcats"][4]["nr"]."
 
 Sub: ".$d->v["subcats"][5]["name"]." ".$d->v["subcats"][5]["nr"]."
 
 Sub: ".$d->v["subcats"][6]["name"]." ".$d->v["subcats"][6]["nr"]."
 
NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "multi level");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//if statement multi row
   $template=new Template("
Name: <!name>
<!_row subcats><!_if subcats.module>Sub: #subcats.name# #subcats.nr#
<!/_if><!/_row>
<!_row module><!_if module=config>- Module: #module# *
<!_else>- Module: #module#
<!/_if><!/_row>
NR: <!nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "if statement multi row");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "if statement multi row");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes", "config"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array(), 1); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "config", "notes"), 2); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "if statement multi row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."
Sub: ".$d->v["subcats"][0]["name"]." ".$d->v["subcats"][0]["nr"]."
Sub: ".$d->v["subcats"][2]["name"]." ".$d->v["subcats"][2]["nr"]."

- Module: ".$d->v["module"][0]."
- Module: ".$d->v["module"][1]."
- Module: ".$d->v["module"][2]." *

NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "if statement multi row");
    $this->e->add($template->e);
    return UT_FAILED;
   }



//if statement multi level
   $template=new Template("
Name: <!name>
<!_row subcats><!_if subcats.module>Sub: #subcats.name# #subcats.nr#
<!_row subcats.module><!_if subcats.module=config>- Module: #subcats.module# *
<!_else>- Module: #subcats.module#
<!/_if><!/_row>
<!/_if><!/_row>
<!_row module><!_if module=config>- Module: #module# *
<!_else>- Module: #module#
<!/_if><!/_row>
NR: <!nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "if statement multi level");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "if statement multi level");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes", "config"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array(), 1); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "config", "notes"), 2); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "if statement multi level");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."
Sub: ".$d->v["subcats"][0]["name"]." ".$d->v["subcats"][0]["nr"]."
- Module: ".$d->v["subcats"][0]["module"][0]."
- Module: ".$d->v["subcats"][0]["module"][1]."

Sub: ".$d->v["subcats"][2]["name"]." ".$d->v["subcats"][2]["nr"]."
- Module: ".$d->v["subcats"][2]["module"][0]."
- Module: ".$d->v["subcats"][2]["module"][1]." *
- Module: ".$d->v["subcats"][2]["module"][2]."


- Module: ".$d->v["module"][0]."
- Module: ".$d->v["module"][1]."
- Module: ".$d->v["module"][2]." *

NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "if statement multi level");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//if rights statement
   $template=new Template("
Name: <!name>
<!_row subcats><!_if_rights add>Sub: #subcats.name#
<!_row subcats.module><!_if_rights edit>- Module: #subcats.module# *
<!_else>- Module: #subcats.module#
<!/_if_rights><!/_row>
<!/_if_rights><!/_row>
NR: <!nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "if rights statement");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "if rights statement");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes", "config"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("notes"), 1); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "config", "notes"), 2); 

   //fake rights
   $d->setValue("r", "1100000011000000", 0);

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "if rights statment");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="
Name: ".$d->v["name"]."
Sub: ".$d->v["subcats"][0]["name"]."
- Module: ".$d->v["subcats"][0]["module"][0]."
- Module: ".$d->v["subcats"][0]["module"][1]."

Sub: ".$d->v["subcats"][1]["name"]."
- Module: ".$d->v["subcats"][1]["module"][0]."

Sub: ".$d->v["subcats"][2]["name"]."
- Module: ".$d->v["subcats"][2]["module"][0]."
- Module: ".$d->v["subcats"][2]["module"][1]."
- Module: ".$d->v["subcats"][2]["module"][2]."

Sub: ".$d->v["subcats"][3]["name"]."

Sub: ".$d->v["subcats"][4]["name"]."

Sub: ".$d->v["subcats"][5]["name"]."

Sub: ".$d->v["subcats"][6]["name"]."


NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "if rights statement");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//mutilations
   $template=new Template("
Name: <!name>
<!_row subcats>Sub: #subcats.nr leadingZeros(3)#
<!/_row>
<!_row subcats><!_if subcats.module>Sub: #subcats.name strip(5,=)#
<!_row subcats.module>- Module: #subcats.module# #name replace(y,O)#
<!/_row>
<!/_if><!/_row>
NR: <!nr>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "mutilations");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "mutilations");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //fake modules
   $d->setValue("module", Array("categories", "notes", "config"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "notes"), 0); 
   $d->reference["subcats"]["data"]->setValue("module", Array(), 1); 
   $d->reference["subcats"]["data"]->setValue("module", Array("categories", "config", "notes"), 2); 

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "mutilations");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->v["name"]."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][0]["nr"], 3)."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][1]["nr"], 3)."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][2]["nr"], 3)."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][3]["nr"], 3)."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][4]["nr"], 3)."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][5]["nr"], 3)."
Sub: ".Mutilate::leadingZeros($d->v["subcats"][6]["nr"], 3)."

Sub: ".Mutilate::strip($d->v["subcats"][0]["name"], 5, "=")."
- Module: ".$d->v["subcats"][0]["module"][0]." ".str_replace("y", "O", $d->v["name"])."
- Module: ".$d->v["subcats"][0]["module"][1]." ".str_replace("y", "O", $d->v["name"])."

Sub: ".Mutilate::strip($d->v["subcats"][2]["name"], 5, "=")."
- Module: ".$d->v["subcats"][2]["module"][0]." ".str_replace("y", "O", $d->v["name"])."
- Module: ".$d->v["subcats"][2]["module"][1]." ".str_replace("y", "O", $d->v["name"])."
- Module: ".$d->v["subcats"][2]["module"][2]." ".str_replace("y", "O", $d->v["name"])."


NR: ".$d->v["nr"]."
")
   {
    $this->e->add("Template output is incorrect", "error", "mutilations");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//illegal field/reference
   $template=new Template("
<!_row asd>
Name: #asd#
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "illegal field/reference");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

//illegal field type
   $template=new Template("
<!_row name>
Name: #name#
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "illegal field type");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

//illegal reference type
   $template=new Template("
<!_row subcats.catnr>
Name: #subcats.catnr.name#
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "illegal reference tpye");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

//incorrect foreach field
   $template=new Template("
<!_row asd>
Name: #module#
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "incorrect foreach field");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   

   return UT_OK;
  }


  function PopulateValuesRowOddEven()
  {
   $d=Data::cache("categories");

   $d->setValue("name", "Name 1", 0);
   $d->setValue("text", "Text 1", 0);

   $d->setValue("name", "Name 2", 1);
   $d->setValue("text", "Text 2", 1);

//!block
   $template=new Template("
<!_row><h1><!name>Name<!/name></h1>
<h2><!text>Text<!/text></h2>
<!_row_oddeven>
<!/_row>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name 1</h1>
<h2>Text 1</h2>
odd
<h1>Name 2</h1>
<h2>Text 2</h2>
even
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
<!_row><h1><!name></h1>
<h2>#text#</h2>
#_row_oddeven#
<!/_row>");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
<h1>Name 1</h1>
<h2>Text 1</h2>
odd
<h1>Name 2</h1>
<h2>Text 2</h2>
even
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function PopulateValuesRowRecursive()
  {
   $c=0;
   $d=Data::cache("categories", "tree", "down");

   $d->setValue("name", "Name 1", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("name", "Name 2", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("name", "Name 2.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("name", "Name 2.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;

   $d->setValue("name", "Name 2.2", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("name", "Name 3", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("name", "Name 3.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("name", "Name 3.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;

   $d->setValue("name", "Name 3.1.1.1", $c);
   $d->setValue("treedepth", 3, $c);
   $c++;

   $d->setValue("name", "Name 4", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("name", "Name 4.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("name", "Name 4.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;


//Sub row
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <div class=\"sub\"><!_row>Sub row
 <!/_row></div>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 <div class=\"sub\"></div>
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 <div class=\"sub\"></div>
</div>
Name: Name 2.2
 <div class=\"sub\"></div>
</div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 <div class=\"sub\"></div>
</div>
</div>
</div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 <div class=\"sub\"></div>
</div>
</div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with #_row#
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\">#_row#  </div><!/_row>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with <!_row> 
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\"><!_row>  </div><!/_row>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with <!_row> 
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\"><!_row>Sub<!/_row>  </div><!/_row>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   return UT_OK;
  }

  
  function PopulateValuesMultipleRowRecursive()
  {
   $c=0;
   $d=Data::cache("categories", "tree", "down");

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 1", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2.2", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3.1.1.1", $c);
   $d->setValue("treedepth", 3, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 4", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 4.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 4.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;


//Sub row
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <div class=\"sub\"><!_row>Sub row
 <!/_row></div>
<!/_row>

<!_row>#: <!nr>NR<!/nr>
 <div class=\"sub\"><!_row>Sub row
 <!/_row></div>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 <div class=\"sub\"></div>
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 <div class=\"sub\"></div>
</div>
Name: Name 2.2
 <div class=\"sub\"></div>
</div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 <div class=\"sub\"></div>
</div>
</div>
</div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 <div class=\"sub\"></div>
</div>
</div>


#: 1
 <div class=\"sub\"></div>
#: 2
 <div class=\"sub\">#: 3
 <div class=\"sub\">#: 4
 <div class=\"sub\"></div>
</div>
#: 5
 <div class=\"sub\"></div>
</div>
#: 6
 <div class=\"sub\">#: 7
 <div class=\"sub\">#: 8
 <div class=\"sub\">#: 9
 <div class=\"sub\"></div>
</div>
</div>
</div>
#: 10
 <div class=\"sub\">#: 11
 <div class=\"sub\">#: 12
 <div class=\"sub\"></div>
</div>
</div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with #_row#
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\">#_row#  </div><!/_row>
<!/_row>

<!_row>#: <!nr>NR<!/nr>
 <!_row><div class=\"sub\">#_row#  </div><!/_row>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>


#: 1
 
#: 2
 <div class=\"sub\">#: 3
 <div class=\"sub\">#: 4
 
  </div>
#: 5
 
  </div>
#: 6
 <div class=\"sub\">#: 7
 <div class=\"sub\">#: 8
 <div class=\"sub\">#: 9
 
  </div>
  </div>
  </div>
#: 10
 <div class=\"sub\">#: 11
 <div class=\"sub\">#: 12
 
  </div>
  </div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with <!_row> 
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\"><!_row>  </div><!/_row>
<!/_row>

<!_row>#: <!nr>NR<!/nr>
 <!_row><div class=\"sub\"><!_row>  </div><!/_row>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>


#: 1
 
#: 2
 <div class=\"sub\">#: 3
 <div class=\"sub\">#: 4
 
  </div>
#: 5
 
  </div>
#: 6
 <div class=\"sub\">#: 7
 <div class=\"sub\">#: 8
 <div class=\"sub\">#: 9
 
  </div>
  </div>
  </div>
#: 10
 <div class=\"sub\">#: 11
 <div class=\"sub\">#: 12
 
  </div>
  </div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with <!_row> 
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\"><!_row>Sub<!/_row>  </div><!/_row>
<!/_row>

<!_row>#: <!nr>NR<!/nr>
 <!_row><div class=\"sub\"><!_row>Sub<!/_row>  </div><!/_row>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>


#: 1
 
#: 2
 <div class=\"sub\">#: 3
 <div class=\"sub\">#: 4
 
  </div>
#: 5
 
  </div>
#: 6
 <div class=\"sub\">#: 7
 <div class=\"sub\">#: 8
 <div class=\"sub\">#: 9
 
  </div>
  </div>
  </div>
#: 10
 <div class=\"sub\">#: 11
 <div class=\"sub\">#: 12
 
  </div>
  </div>

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   return UT_OK;
  }


  function PopulateValuesMultipleRowRecursiveMixed()
  {
   $c=0;
   $d=Data::cache("categories", "tree", "down");

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 1", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 2.2", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 3.1.1.1", $c);
   $d->setValue("treedepth", 3, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 4", $c);
   $d->setValue("treedepth", 0, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 4.1", $c);
   $d->setValue("treedepth", 1, $c);
   $c++;

   $d->setValue("nr", $c+1, $c);
   $d->setValue("name", "Name 4.1.1", $c);
   $d->setValue("treedepth", 2, $c);
   $c++;


//Sub row
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <div class=\"sub\"><!_row>Sub row
 <!/_row></div>
<!/_row>

<!_row>#: <!nr>NR<!/nr> (#treedepth#)
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 <div class=\"sub\"></div>
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 <div class=\"sub\"></div>
</div>
Name: Name 2.2
 <div class=\"sub\"></div>
</div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 <div class=\"sub\"></div>
</div>
</div>
</div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 <div class=\"sub\"></div>
</div>
</div>


#: 1 (0)
#: 2 (0)
#: 3 (1)
#: 4 (2)
#: 5 (1)
#: 6 (0)
#: 7 (1)
#: 8 (2)
#: 9 (3)
#: 10 (0)
#: 11 (1)
#: 12 (2)

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with #_row#
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\">#_row#  </div><!/_row>
<!/_row>

<!_row>#: <!nr>NR<!/nr> (<!treedepth>Depth<!/treedepth>)
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>


#: 1 (0)
#: 2 (0)
#: 3 (1)
#: 4 (2)
#: 5 (1)
#: 6 (0)
#: 7 (1)
#: 8 (2)
#: 9 (3)
#: 10 (0)
#: 11 (1)
#: 12 (2)

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with #_row#");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with <!_row> 
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\"><!_row>  </div><!/_row>
<!/_row>

<!_row>#: <!nr>NR<!/nr> (<!treedepth>Depth<!/treedepth>)
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>


#: 1 (0)
#: 2 (0)
#: 3 (1)
#: 4 (2)
#: 5 (1)
#: 6 (0)
#: 7 (1)
#: 8 (2)
#: 9 (3)
#: 10 (0)
#: 11 (1)
#: 12 (2)

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with <!_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//Sub _row with <!_row> 
   $template=new Template("
<!_row>Name: <!name>Name<!/name>
 <!_row><div class=\"sub\"><!_row>Sub<!/_row>  </div><!/_row>
<!/_row>

<!_row>#: <!nr>NR<!/nr> (<!treedepth>Depth<!/treedepth>)
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   if($template->output()!="
Name: Name 1
 
Name: Name 2
 <div class=\"sub\">Name: Name 2.1
 <div class=\"sub\">Name: Name 2.1.1
 
  </div>
Name: Name 2.2
 
  </div>
Name: Name 3
 <div class=\"sub\">Name: Name 3.1
 <div class=\"sub\">Name: Name 3.1.1
 <div class=\"sub\">Name: Name 3.1.1.1
 
  </div>
  </div>
  </div>
Name: Name 4
 <div class=\"sub\">Name: Name 4.1
 <div class=\"sub\">Name: Name 4.1.1
 
  </div>
  </div>


#: 1 (0)
#: 2 (0)
#: 3 (1)
#: 4 (2)
#: 5 (1)
#: 6 (0)
#: 7 (1)
#: 8 (2)
#: 9 (3)
#: 10 (0)
#: 11 (1)
#: 12 (2)

")
   {
    $this->e->add("Template output is incorrect", "error", "Sub row with <!_row><!/_row>");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   return UT_OK;
  }

  
  function PopulateValuesReference()
  {
   $d=Data::cache("categories", "view", "item");

//!block
   $template=new Template("
Name: <!name>Name<!/name>
Parent: <!catnr.name>Parent<!/catnr.name>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->getValue("name")."
Parent: ".$d->reference["catnr"]["data"]->getValue("name")."
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $template=new Template("
Name: #name#
Parent: <!catnr.name>
Parent's parent: #catnr.catnr.name# (#catnr.catnr.text#)
Parent's parent owner: #catnr.catnr.owner.username#
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   $d->clearValues();
   $d->setValue("nr", 3);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".$d->getValue("name")."
Parent: ".$d->reference["catnr"]["data"]->getValue("name")."
Parent's parent: ".$d->reference["catnr"]["data"]->reference["catnr"]["data"]->getValue("name")." (".$d->reference["catnr"]["data"]->reference["catnr"]["data"]->getValue("text").")
Parent's parent owner: ".$d->reference["catnr"]["data"]->reference["catnr"]["data"]->reference["owner"]["data"]->getValue("username")."
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  
  function PopulateValuesMutilations()
  {
   $d=Data::cache("categories", "view", "item");
   $d->field["text"]["type"]="html";

//!block
   $template=new Template("
Name: <!name case(upper)>Name<!/name> (<!name strip(5,[more])>Name<!/name>)
Parent: <!catnr.name replace(A,AA)>Parent<!/catnr.name>
Text: <!text stripTags>text<!/text> <!text case(words)>text<!/text>
Name: #name decode(url)# <!name decode(html)>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //forge values
   $d->setValue("text", "<bold>some bold text</bold>");
   $d->setValue("name", "2g@G4y h3J%1rfq1&quot;v02?sd%20");

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
Name: ".Mutilate::changeCase($d->getValue("name"), "upper")." (".Mutilate::Strip($d->getValue("name"), 5, "[more]").")
Parent: ".Mutilate::replace($d->reference["catnr"]["data"]->getValue("name"), "/A/", "AA")."
Text: ".Mutilate::stripTags($d->getValue("text"))." ".Mutilate::changeCase($d->getValue("text"), "words")."
Name: ".Mutilate::urlDecode($d->getValue("name"))." ".Mutilate::htmlDecode($d->getValue("name"))."
")
   {
    $this->e->add("Template output is incorrect", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//#marker# and !short
   $d->field["text"]["type"]="email";

   $template=new Template("
NR: <!nr> #nr humanReadableNumber(3,\,)# #nr humanReadableFileSize(2, )# <!nr leadingZeros(10)>
Text: #text scrambleEmail#
Parent: <!catnr.name strip(4) /> <!catnr.name /> #catnr.text case(first)#
Time: <!updated dateConvert(Y-m-d)> #catnr.updated timezoneConvert(Europe/Amsterdam)# #updated dateConvert(Y-m-d H:i:s) timezoneConvert(Europe/Amsterdam)#
Name: #name encode(url)# <!name encode(html)>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   
   $d->clearValues();
   $d->setValue("nr", 3);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "!block");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   //forge values
   $d->setValue("nr", 12345);
   $d->setValue("name", "2g@G4y h3J%1rfq1<v02?sd%20");
   $d->setValue("text", "email@address.com");
   $d->reference["catnr"]["data"]->setValue("text", "SOME CAPS");

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "!block");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="
NR: ".$d->getValue("nr")." ".Mutilate::humanReadableNumber($d->getValue("nr"), 3, ",")." ".Mutilate::humanReadableFileSize($d->getValue("nr"), 2, " ")." ".Mutilate::leadingZeros($d->getValue("nr"), 10)."
Text: ".Mutilate::scrambleEmail($d->getValue("text"))."
Parent: ".Mutilate::strip($d->reference["catnr"]["data"]->getValue("name"), 4)." ".$d->reference["catnr"]["data"]->getValue("name")." ".Mutilate::changeCase($d->reference["catnr"]["data"]->getValue("text"), "first")."
Time: ".Mutilate::dateConvert($d->getValue("updated"), "H:i:s d-m-Y", "Y-m-d")." ".Mutilate::timezoneConvert($d->reference["catnr"]["data"]->getValue("updated"), "UTC", "Europe/Amsterdam", "H:i:s d-m-Y")." ".Mutilate::timezoneConvert(Mutilate::dateConvert($d->getValue("updated"), "H:i:s d-m-Y", "Y-m-d H:i:s"), "UTC", "Europe/Amsterdam", "Y-m-d H:i:s")."
Name: ".Mutilate::urlEncode($d->getValue("name"))." ".Mutilate::htmlEncode($d->getValue("name"))."
")
   {
    $this->e->add("Template output is incorrect", "error", "#marker# and !short");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  
  function PopulateValuesIfStatements()
  {
   $d=Data::cache("categories", "view", "all");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Name 1", 0);
   $d->setValue("text", "Text 1", 0);
   $d->setValue("module", "categories", 0);
   $d->setValue("moduleLock", true, 0);
   $d->setValue("updated", "12:01:01 01-01-2012", 0);

   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Name 2", 1);
   $d->setValue("text", "Text 2", 1);
   $d->setValue("module", "", 1);
   $d->setValue("moduleLock", false, 1);
   $d->setValue("updated", "12:02:02 02-02-2012", 1);


//int comparison
   $template=new Template("
<!_row>
<!_if nr>Name: <!name>Name<!/name><!/_if>
<!_if nr!=1>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "int comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "int comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1


Name: Name 2
Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "int comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//int larger comparison
   $template=new Template("
<!_row>
<!_if nr&gt;1>Name: <!name>Name<!/name><!/_if>
<!_if nr&gt;=1>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "int larger comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "int larger comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="


Text: Text 1

Name: Name 2
Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "int larger comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//int smaller comparison
   $template=new Template("
<!_row>
<!_if nr&lt;2>Name: <!name>Name<!/name><!/_if>
<!_if nr&lt;=2>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "int smaller comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "int smaller comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1
Text: Text 1


Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "int smaller comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//time comparison
   $template=new Template("
<!_row>
<!_if updated=12:01:01 01-01-2012>Name: <!name>Name<!/name><!/_if>
<!_if updated!=12:01:01 01-01-2012>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "time comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "time comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1



Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "time comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//boolean comparison
   $template=new Template("
<!_row>
<!_if moduleLock>Name: <!name>Name<!/name><!/_if>
<!_if !moduleLock>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "boolean comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "boolean comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1



Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "boolean comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//boolean comparison
   $template=new Template("
<!_row>
<!_if moduleLock=true>Name: <!name>Name<!/name><!/_if>
<!_if moduleLock!=1>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "boolean comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "boolean comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1



Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "boolean comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//boolean not comparison
   $template=new Template("
<!_row>
<!_if not moduleLock=true>Name: <!name>Name<!/name><!/_if>
<!_if ! moduleLock!=1>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "boolean not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "boolean not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="


Text: Text 1

Name: Name 2


")
   {
    $this->e->add("Template output is incorrect", "error", "boolean not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//string comparison
   $template=new Template("
<!_row>
<!_if name=Name 1>Name: <!name>Name<!/name><!/_if>
<!_if name!=Name 1>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "string comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "string comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1



Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "string comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//string any comparison
   $template=new Template("
<!_row>
<!_if name>Name: <!name>Name<!/name><!/_if>
<!_if module>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "string any comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "string any comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1
Text: Text 1

Name: Name 2


")
   {
    $this->e->add("Template output is incorrect", "error", "string any comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//string not comparison
   $template=new Template("
<!_row>
<!_if not name=Name 1>Name: <!name>Name<!/name><!/_if>
<!_if ! name!=Name 1>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "string not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "string not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="


Text: Text 1

Name: Name 2


")
   {
    $this->e->add("Template output is incorrect", "error", "string not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//string else comparison
   $template=new Template("
<!_row>
<!_if name=Name 1>Name: <!name>Name<!/name>
<!_else>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "string else comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "string else comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: Name 1


Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "string else comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//string else multi comparison
   $template=new Template("
<!_row>
<!_if name=Name 1>
 <!_if text=Text 1>First name: <!name>Name<!/name>
 <!_else>First text: <!text>Text<!/text><!/_if>
<!_else>
 <!_if text=Text 2>Second name: <!name>Name<!/name>
 <!_else>Second text: <!text>Text<!/text><!/_if>
<!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "string else multi comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "string else multi comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="


 First name: Name 1
 



 Second name: Name 2
 


")
   {
    $this->e->add("Template output is incorrect", "error", "string else multi comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//reference
   $template=new Template("
<!_row>
<!_if catnr.nr=1>Name: <!name>Name<!/name><!/_if>
<!_if catnr.name!=JAS>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "reference comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "reference comparison");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "reference comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: ".$d->getValue("name")."


")
   {
    $this->e->add("Template output is incorrect", "error", "reference comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//reference not
   $template=new Template("
<!_row>
<!_if not catnr.nr&gt;1>Name: <!name>Name<!/name><!/_if>
<!_if not catnr.text=asd>Text: <!text>Text<!/text><!/_if>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "reference not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   $d->clearValues();
   $d->setValue("nr", 2);
   if(!$d->get($template->getLevel(), $template->showCount()))
   {
    $this->e->add("Get data failed", "error", "reference not comparison");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$template->populate())
   {
    $this->e->add("Template population failed", "error", "reference not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   if($template->output()!="

Name: ".$d->getValue("name")."
Text: ".$d->getValue("text")."

")
   {
    $this->e->add("Template output is incorrect", "error", "reference not comparison");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail no block
   $template=new Template("
<!_row>
#_if not catnr.nr&gt;1#Name: <!name>Name<!/name>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail no block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail no block
   $template=new Template("
<!_row>
<!_if not catnr.nr&gt;1>Name: <!name>Name<!/name>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail no block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail no attributes
   $template=new Template("
<!_row>
<!_if>Name: <!name>Name<!/name><!/_if>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail no attributes");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail incorrect attributes
   $template=new Template("
<!_row>
<!_if !!!!>Name: <!name>Name<!/name><!/_if>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail incorrect attributes");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   
//fail incorrect operator
   $template=new Template("
<!_row>
<!_if name=!Name 1>Name: <!name>Name<!/name><!/_if>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail incorrect operator");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   return UT_OK;
  }

  
  function PopulateValuesIfRightsStatements()
  {
   global $_config;

   $d=Data::cache("categories", "view", "all");
   $d->clearValues();

   $d->setValue("nr", 1, 0);
   $d->setValue("name", "Name 1", 0);
   $d->setValue("text", "Text 1", 0);
   $d->setValue("r", "1111111111111111", 0);

   $d->setValue("nr", 2, 1);
   $d->setValue("name", "Name 2", 1);
   $d->setValue("text", "Text 2", 1);
   $d->setValue("r", "0000000000000000", 1);


//add category
   $template=new Template("
<!_row>
<!_if_rights add>Name: <!name>Name<!/name>
<!_else>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "add category");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "add category");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1


Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "add category");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//copy / edit category
   $template=new Template("
<!_row>
<!_if_rights copy>Name: <!name>Name<!/name><!/_if_rights>
<!_if_rights edit>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "copy / edit category");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "copy / edit category");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1
Text: Text 1




")
   {
    $this->e->add("Template output is incorrect", "error", "copy / edit category");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//move / delete category
   $template=new Template("
<!_row>
<!_if_rights move>Name: <!name>Name<!/name><!/_if_rights>
<!_if_rights delete>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "move / delete category");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "move / delete category");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1
Text: Text 1




")
   {
    $this->e->add("Template output is incorrect", "error", "move / delete category");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//owner category
   $template=new Template("
<!_row>
<!_if_rights owner>Name: <!name>Name<!/name><!/_if_rights>
<!_if_rights not owner>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "owner category");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "owner category");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1



Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "owner category");
    $this->e->add($template->e);
    return UT_FAILED;
   }



   $d->module="notes";

//add note
   $template=new Template("
<!_row>
<!_if_rights add>Name: <!name>Name<!/name>
<!_else>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "add note");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "add note");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1


Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "add note");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//copy / edit note
   $template=new Template("
<!_row>
<!_if_rights copy>Name: <!name>Name<!/name><!/_if_rights>
<!_if_rights edit>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "copy / edit note");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "copy / edit note");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1
Text: Text 1




")
   {
    $this->e->add("Template output is incorrect", "error", "copy / edit note");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//move / delete note
   $template=new Template("
<!_row>
<!_if_rights move>Name: <!name>Name<!/name><!/_if_rights>
<!_if_rights delete>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "move / delete note");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "move / delete note");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1
Text: Text 1




")
   {
    $this->e->add("Template output is incorrect", "error", "move / delete note");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//owner note
   $template=new Template("
<!_row>
<!_if_rights owner>Name: <!name>Name<!/name><!/_if_rights>
<!_if_rights not owner>Text: <!text>Text<!/text><!/_if_rights>
<!/_row>
");

   if(!$template->analyze($d))
   {
    $this->e->add("Template analyzation failed", "error", "owner note");
    $this->e->add($template->e);
    return UT_FAILED;
   }

   //sudo (anon)
   $tmp=$_config["usernr"];
   $_config["usernr"]=$_config["anonnr"];
   if(!$template->populate())
   {
    $_config["usernr"]=$tmp;
    $this->e->add("Template population failed", "error", "owner note");
    $this->e->add($template->e);
    return UT_FAILED;
   }
   $_config["usernr"]=$tmp;

   if($template->output()!="

Name: Name 1



Text: Text 2

")
   {
    $this->e->add("Template output is incorrect", "error", "owner note");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail no block
   $template=new Template("
<!_row>
#_if_rights add#Name: <!name>Name<!/name>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail no block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail no block
   $template=new Template("
<!_row>
<!_if_rights add>Name: <!name>Name<!/name>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail no block");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail no attributes
   $template=new Template("
<!_row>
<!_if_rights>Name: <!name>Name<!/name><!/_if_rights>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail no attributes");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail incorrect attributes
   $template=new Template("
<!_row>
<!_if_rights !!!!>Name: <!name>Name<!/name><!/_if_rights>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail incorrect attributes");
    $this->e->add($template->e);
    return UT_FAILED;
   }


//fail incorrect rights
   $template=new Template("
<!_row>
<!_if_rights asd>Name: <!name>Name<!/name><!/_if_rights>
<!/_row>
");

   if($template->analyze($d))
   {
    $this->e->add("Template analyzation should fail", "error", "fail incorrect rights");
    $this->e->add($template->e);
    return UT_FAILED;
   }


   return UT_OK;
  }

 }

?>
