<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."xml.php");

 class UT_XML extends UT_Classes
 {
  var $testXML="<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>
  <test ut=\"xml\">
   <!-- comment -->
   <unit test=\"asd\">
   #marker#
   <!shortcomment>
 </test>";

  var $testTag=Array( //tag validation array, based on testXML
   Array(
    "name" => "?xml",
    "type" => "short",
    "outer" => Array(
     "start" => 0,
     "end" => 43,
     "length" => 43,
    ),
    "attributes" => Array(
     "version" => "1.0",
     "encoding" => "ISO-8859-1",
    ),
    "parent" => false,
   ),

   Array(
    "name" => "test",
    "type" => "block",
    "inner" => Array(
     "start" => 61,
     "end" => 135,
     "length" => 74,
    ),
    "outer" => Array(
     "start" => 46,
     "end" => 142,
     "length" => 96,
    ),
    "attributes" => Array(
     "ut" => "xml",
    ),
    "parent" => false,
    "close" => 6,
   ),

   Array(
    "name" => "comment",
    "type" => "!--short",
    "outer" => Array(
     "start" => 65,
     "end" => 81,
     "length" => 16,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "unit",
    "type" => "short",
    "outer" => Array(
     "start" => 85,
     "end" => 102,
     "length" => 17,
    ),
    "attributes" => Array(
     "test" => "asd",
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "marker",
    "type" => "marker",
    "outer" => Array(
     "start" => 106,
     "end" => 114,
     "length" => 8,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "shortcomment",
    "type" => "!short",
    "outer" => Array(
     "start" => 118,
     "end" => 133,
     "length" => 15,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   ),

   Array(
    "name" => "test",
    "type" => "close",
   ),
  );


  function Constructor()
  {
   $object=new XML();

   if(!is_object($object)||get_class($object)!="XML")
   {
    $this->e->add("Constructor doesn't return a XML object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->onlyCommentsAndMarkers)
   {
    $this->e->add("\"OnlyCommentsAndMarkers\" isn't false", "error");
    return UT_FAILED;
   }
   if(!$object->exactPath)
   {
    $this->e->add("\"ExactPath\" isn't true", "error");
    return UT_FAILED;
   }


   $object=new XML(true, false);

   if(!$object->onlyCommentsAndMarkers)
   {
    $this->e->add("\"OnlyCommentsAndMarkers\" isn't true", "error");
    return UT_FAILED;
   }
   if($object->exactPath)
   {
    $this->e->add("\"ExactPath\" isn't false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function _validateTags($tag, $expected=false)
  {
   $validTypes=Array(
    "block", "!block", "!--block", 
    "close", "!close", "!--close", 
    "short", "!short", "!--short", 
    "marker", 
   );

   $l=sizeof($tag);
   for($c=0;$c<$l;$c++)
   {
    //name validation
    if(!isset($tag[$c]["name"]))
    {
     $this->e->add("Tag validation failed: no name for tag: \"".$c."\"", "error");
     return UT_FAILED;
    }

    //type validation
    if(!isset($tag[$c]["type"]))
    {
     $this->e->add("Tag validation failed: no type for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!in_array($tag[$c]["type"], $validTypes))
    {
     $this->e->add("Tag validation failed: invalid type \"".$tag[$c]["type"]."\" for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //no more validation for close tag
    if(strstr($tag[$c]["type"], "close"))
     continue;

    //outer validation
    if(!isset($tag[$c]["outer"]))
    {
     $this->e->add("Tag validation failed: no outer array for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!isset($tag[$c]["outer"]["start"]))
    {
     $this->e->add("Tag validation failed: no outer start for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!isset($tag[$c]["outer"]["end"]))
    {
     $this->e->add("Tag validation failed: no outer end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!isset($tag[$c]["outer"]["length"]))
    {
     $this->e->add("Tag validation failed: no outer end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["outer"]["start"]>$tag[$c]["outer"]["end"])
    {
     $this->e->add("Tag validation failed: outer start position after end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(($tag[$c]["outer"]["end"]-$tag[$c]["outer"]["start"])!=$tag[$c]["outer"]["length"])
    {
     $this->e->add("Tag validation failed: outer start position after end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //parent validations
    if(!isset($tag[$c]["parent"]))
    {
     $this->e->add("Tag validation failed: no parent for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]<0)
    {
     $this->e->add("Tag validation failed: invalid parent \"".$tag[$c]["parent"]."\" for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]>=$l)
    {
     $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" doesn't exist for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]>$c)
    {
     $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" can't be after tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if($tag[$c]["parent"]===$c)
    {
     $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" can't be the same tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //attributes validations
    if(!isset($tag[$c]["attributes"]))
    {
     $this->e->add("Tag validation failed: no attributes for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }
    if(!is_array($tag[$c]["attributes"]))
    {
     $this->e->add("Tag validation failed: attributes should be an array for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
     return UT_FAILED;
    }

    //block tag specific validations
    if(strstr($tag[$c]["type"], "block"))
    {
     //inner validation
     if(!isset($tag[$c]["inner"]))
     {
      $this->e->add("Tag validation failed: no inner array for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if(!isset($tag[$c]["inner"]["start"]))
     {
      $this->e->add("Tag validation failed: no inner start for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if(!isset($tag[$c]["inner"]["end"]))
     {
      $this->e->add("Tag validation failed: no inner end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if(!isset($tag[$c]["inner"]["length"]))
     {
      $this->e->add("Tag validation failed: no inner end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["inner"]["start"]>$tag[$c]["inner"]["end"])
     {
      $this->e->add("Tag validation failed: inner start position after end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if(($tag[$c]["inner"]["end"]-$tag[$c]["inner"]["start"])!=$tag[$c]["inner"]["length"])
     {
      $this->e->add("Tag validation failed: inner start position after end for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //close reference validations
     if(!isset($tag[$c]["close"]))
     {
      $this->e->add("Tag validation failed: no close reference for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]<0)
     {
      $this->e->add("Tag validation failed: invalid close reference \"".$tag[$c]["close"]."\" for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]>=$l)
     {
      $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" doesn't exist for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]<$c)
     {
      $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" can't be before tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["close"]==$c)
     {
      $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" can't be the same tag for tag: \"".$tag[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
    }
   }

   if($expected&&is_array($expected))
   {
    for($c=0;$c<$l;$c++)
    {
     //more tags then expected
     if(!isset($expected[$c]))
     {
      $this->e->add("Tag validation failed: tag \"".$tag[$c]["name"]."\" (".$c.") not expected", "error");
      return UT_FAILED;
     }

     //name validation
     if($tag[$c]["name"]!=$expected[$c]["name"])
     {
      $this->e->add("Tag validation failed: name \"".$tag[$c]["name"]."\" doesn't match for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //type validation
     if($tag[$c]["type"]!=$expected[$c]["type"])
     {
      $this->e->add("Tag validation failed: type \"".$tag[$c]["type"]."\" doesn't match \"".$expected[$c]["type"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //no more validation for close tag
     if(strstr($tag[$c]["type"], "close"))
      continue;

     //outer validation
     if($tag[$c]["outer"]["start"]!=$expected[$c]["outer"]["start"])
     {
      $this->e->add("Tag validation failed: outer start \"".$tag[$c]["outer"]["start"]."\" doesn't match \"".$expected[$c]["outer"]["start"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["outer"]["end"]!=$expected[$c]["outer"]["end"])
     {
      $this->e->add("Tag validation failed: outer end \"".$tag[$c]["outer"]["end"]."\" doesn't match \"".$expected[$c]["outer"]["end"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }
     if($tag[$c]["outer"]["length"]!=$expected[$c]["outer"]["length"])
     {
      $this->e->add("Tag validation failed: outer length \"".$tag[$c]["outer"]["length"]."\" doesn't match \"".$expected[$c]["outer"]["length"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //parent validations
     if($tag[$c]["parent"]!==$expected[$c]["parent"])
     {
      $this->e->add("Tag validation failed: parent \"".$tag[$c]["parent"]."\" doesn't match \"".$expected[$c]["parent"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      return UT_FAILED;
     }

     //attributes validations
     if($tag[$c]["attributes"]!=$expected[$c]["attributes"])
     {
      foreach(array_keys($expected[$c]["attributes"]) as $k)
      {
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$expected[$c]["attributes"][$k]."\" expected for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       if(!isset($tag[$c]["attributes"][$k]))
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$expected[$c]["attributes"][$k]."\" expected for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       else if($tag[$c]["attributes"][$k]!=$expected[$c]["attributes"][$k])
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$tag[$c]["attributes"][$k]."\" doesn't match \"".$expected[$c]["attributes"][$k]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      }

      foreach(array_keys($tag[$c]["attributes"]) as $k)
      {
       if(!isset($expected[$c]["attributes"][$k]))
        $this->e->add("Tag validation failed: attribute \"".$k."\" value \"".$tag[$c]["attributes"][$k]."\" not expected for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
      }

      return UT_FAILED;
     }

     //block tag specific validations
     if(strstr($tag[$c]["type"], "block"))
     {
      //inner validations
      if($tag[$c]["inner"]["start"]!=$expected[$c]["inner"]["start"])
      {
       $this->e->add("Tag validation failed: inner start \"".$tag[$c]["inner"]["start"]."\" doesn't match \"".$expected[$c]["inner"]["start"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }
      if($tag[$c]["inner"]["end"]!=$expected[$c]["inner"]["end"])
      {
       $this->e->add("Tag validation failed: inner end \"".$tag[$c]["inner"]["end"]."\" doesn't match \"".$expected[$c]["inner"]["end"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }
      if($tag[$c]["inner"]["length"]!=$expected[$c]["inner"]["length"])
      {
       $this->e->add("Tag validation failed: inner length \"".$tag[$c]["inner"]["length"]."\" doesn't match \"".$expected[$c]["inner"]["length"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }

      //close validations
      if($tag[$c]["close"]!=$expected[$c]["close"])
      {
       $this->e->add("Tag validation failed: close reference \"".$tag[$c]["close"]."\" doesn't match \"".$expected[$c]["close"]."\" for tag: \"".$expected[$c]["name"]."\" (".$c.")", "error");
       return UT_FAILED;
      }
     }
    }


    $l2=sizeof($expected);
    for($c=$l;$c<$l2;$c++) //check rest of expected array, i.e. more tags expected
    {
     $this->e->add("Tag validation failed: tag \"".$expected[$c]["name"]."\" (".$c.") expected", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }



  function Parse()
  {
   $xml=new XML();
   if(!$xml->parse($this->testXML))
   {
    $this->e->add("Parsing test XML failed");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->original!=$this->testXML)
   {
    $this->e->add("Original XML not set correctly");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML not set correctly");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->clear();
   $xml->e->clear();
   if($xml->parse("var test=1;

   function lala()
   {
    test++;
   }
   "))
   {
    $this->e->add("Parser should fail on java script");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->original!==false)
   {
    $this->e->add("Original XML should not be set");
    return UT_FAILED;
   }

   if($xml->xml!==false)
   {
    $this->e->add("XML should not be set");
    return UT_FAILED;
   }

   if($xml->tag)
   {
    $this->e->add("XML tag variable should be empty");
    return UT_FAILED;
   }

   if($xml->tags!==0)
   {
    $this->e->add("XML tags counter should be 0");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ParseFile()
  {
   global $_path;

   $xml=new XML();
   if(!$xml->parseFile($_path["include"]."xmli/1.0/jas.xml"))
   {
    $this->e->add("Parsing file failed");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->filename!=$_path["include"]."xmli/1.0/jas.xml")
   {
    $this->e->add("Filename parameter not set after loading");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->clear();
   $xml->e->clear();
   if($xml->parseFile($_path["root"]."jakdsajsdkhahjsdhajksdasd.lkjasd"))
   {
    $this->e->add("Parser should not find file: \"".$_path["root"]."jakdsajsdkhahjsdhajksdasd.lkjasd\"");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   $xml->clear();

   if($xml->filename!==false)
   {
    $this->e->add("Filename not cleared");
    return UT_FAILED;
   }

   if($xml->original!==false)
   {
    $this->e->add("Original not cleared");
    return UT_FAILED;
   }

   if($xml->xml!==false)
   {
    $this->e->add("XML not cleared");
    return UT_FAILED;
   }

   if($xml->tags!=0)
   {
    $this->e->add("Number of tags not cleared");
    return UT_FAILED;
   }

   if($xml->tag!=Array())
   {
    $this->e->add("Tags array not cleared");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ReInit()
  {
   $xml=new XML();

   if($xml->reinit())
   {
    $this->e->add("XML should not be initialized");
    return UT_FAILED;
   }

   $xml->parse($this->testXML);
   $xml->delete("test");

   if(!$xml->reinit())
   {
    $this->e->add("XML should reinitialize");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=$xml->original)
   {
    $this->e->add("XML should be the same as original");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;

   return UT_OK;
  }


  function ReParse()
  {
   $xml=new XML();

   if($xml->reparse())
   {
    $this->e->add("XML should not be initialized");
    return UT_FAILED;
   }

   $xml->parse($this->testXML);
   $xml->xml=preg_replace("!</test>!", "", $xml->xml);

   if(!$xml->reparse())
   {
    $this->e->add("XML should reparse");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->tag[1]["type"]!="short")
   {
    $this->e->add("XML test tag should now be a short tag");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function ParseInternalChecks()
  {
   $xml=new XML();

   //correct type for short sub tag with same name
   $xml->parse("<tag block><tag short/></tag>");

   if($xml->tag[0]["type"]!="block")
   {
    $this->e->add("XML parse failed", "error", "first tag should be a block tag");
    return UT_FAILED;
   }

   if($xml->tag[1]["type"]!="short")
   {
    $this->e->add("XML parse failed", "error", "second tag should be a short tag");
    return UT_FAILED;
   }


   //correct type for short sub tag with same name (comments)
   $xml->parse("<!tag block><!tag block><!tag short><!/tag><!/tag>");

   if($xml->tag[0]["type"]!="!block")
   {
    $this->e->add("XML parse failed", "error", "first tag should be a block tag");
    return UT_FAILED;
   }

   if($xml->tag[1]["type"]!="!block")
   {
    $this->e->add("XML parse failed", "error", "second tag should be a block tag");
    return UT_FAILED;
   }
   
   if($xml->tag[2]["type"]!="!short")
   {
    $this->e->add("XML parse failed", "error", "third tag should be a short tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  
  function InternalSetFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add text before begin
   if($xml->_set(-1, 0, "asd")) 
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add text after end
   if($xml->_set(strlen($this->testXML)+1, 0, "asd")) 
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add text ending after end
   if($xml->_set(strlen($this->testXML)-1, 10, "asd")) 
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add array
   if($xml->_set(0, 0, Array("asd"))) 
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//add object
   if($xml->_set(0, 0, $this)) 
   {
    $this->e->add("XML _set should fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//break tags test, i.e. replace < or >... 
   return UT_OK;
  }


  function InternalSetText()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add text at begin
   if(!$xml->_set(0, 0, "asd")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!="asd".$this->testXML)
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=3; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner position off all tags
   $l=sizeof($testTag);
   for($c=0;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit text at begin
   if(!$xml->_set(0, 3, "trala")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!="trala".$this->testXML)
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=2; //length difference (trala-asd)

   //adjust outer+inner position off all tags
   $l=sizeof($testTag);
   for($c=0;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete text at begin
   if(!$xml->_set(0, 5, "")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text at end 
   if(!$xml->_set(strlen($xml->xml), 0, "asd")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML."asd")
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit text at end 
   if(!$xml->_set(strlen($xml->xml)-3, 3, "trala")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML."trala")
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete text at end 
   if(!$xml->_set(strlen($xml->xml)-5, 5, "")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text after <test>
   if(!$xml->_set(61, 0, "asd")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, 61)."asd".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=3; //length difference
   $testTag=$this->testTag;

   //adjust outer+inner end position+length of test tag
   $testTag[1]["outer"]["end"]+=$diff;
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["end"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;

   //adjust outer+inner position off all tags after test tag
   $l=sizeof($testTag);
   for($c=2;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit text after <test>
   if(!$xml->_set(61, 3, "trala")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, 61)."trala".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=2; //length difference

   //adjust outer+inner end position+length of test tag
   $testTag[1]["outer"]["end"]+=$diff;
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["end"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;

   //adjust outer+inner position off all tags after test tag
   $l=sizeof($testTag);
   for($c=2;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete text after <test>
   if(!$xml->_set(61, 5, "")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function InternalSetTags()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add tag at begin
   if(!$xml->_set(0, 0, "<asd>")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!="<asd>".$this->testXML)
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //add asd tag at begin of array
   array_unshift($testTag, Array(
    "name" => "asd",
    "type" => "short",
    "outer" => Array(
     "start" => 0,
     "end" => $diff,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => false,
   ));

   //adjust outer+inner position off all (except new) tags
   $l=sizeof($testTag);
   for($c=1;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["parent"])&&$testTag[$c]["parent"]!==false)
     $testTag[$c]["parent"]++; //1 tag added

    if(isset($testTag[$c]["close"]))
     $testTag[$c]["close"]++; //1 tag added
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//overwrite tag at begin
   if(!$xml->_set(0, 5, "<trala>")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!="<trala>".$this->testXML)
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=2; //length difference (<trala>-<asd>)

   //update tag
   $testTag[0]["name"]="trala";
   $testTag[0]["outer"]["end"]+=$diff;
   $testTag[0]["outer"]["length"]+=$diff;

   //adjust outer+inner position off all (except overwritten) tags
   $l=sizeof($testTag);
   for($c=1;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete tag at begin
   if(!$xml->_set(0, 7, "")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add tag at end 
   if(!$xml->_set(strlen($xml->xml), 0, "<asd>")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML."<asd>")
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //add asd tag at end of array
   array_push($testTag, Array(
    "name" => "asd",
    "type" => "short",
    "outer" => Array(
     "start" => strlen($this->testXML),
     "end" => strlen($this->testXML)+$diff,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => false,
   ));


   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//overwrite tag at end 
   if(!$xml->_set(strlen($xml->xml)-5, 5, "<trala>")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML."<trala>")
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=2; //length difference (<trala>-<asd>)

   //update tag
   $testTag[sizeof($testTag)-1]["name"]="trala";
   $testTag[sizeof($testTag)-1]["outer"]["end"]+=$diff;
   $testTag[sizeof($testTag)-1]["outer"]["length"]+=$diff;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete tag at end 
   if(!$xml->_set(strlen($xml->xml)-7, 7, "")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add tag after <test>
   if(!$xml->_set(61, 0, "<asd>")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, 61)."<asd>".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the new value");
    return UT_FAILED;
   }

   $diff=5; //length difference
   $testTag=$this->testTag;

   //add asd tag after <test>
   array_splice($testTag, 2, 0, Array(Array(
    "name" => "asd",
    "type" => "short",
    "outer" => Array(
     "start" => 61,
     "end" => 61+$diff,
     "length" => $diff,
    ),
    "attributes" => Array(
    ),
    "parent" => 1,
   )));


   //adjust outer+inner end position+length and close reference of test tag
   $testTag[1]["outer"]["end"]+=$diff;
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["end"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;
   $testTag[1]["close"]++; //1 tag added

   //adjust outer+inner position off all tags after new tag
   $l=sizeof($testTag);
   for($c=3;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//edit tag after <test>
   if(!$xml->_set(61, 5, "<trala>")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, 61)."<trala>".substr($this->testXML, 61))
   {
    $this->e->add("XML should include the updated value");
    return UT_FAILED;
   }

   $diff=2; //length difference

   //update tag
   $testTag[2]["name"]="trala";
   $testTag[2]["outer"]["end"]+=$diff;
   $testTag[2]["outer"]["length"]+=$diff;

   //adjust outer+inner end position+length of test tag
   $testTag[1]["outer"]["end"]+=$diff;
   $testTag[1]["outer"]["length"]+=$diff;
   $testTag[1]["inner"]["end"]+=$diff;
   $testTag[1]["inner"]["length"]+=$diff;

   //adjust outer+inner position off all tags after test tag
   $l=sizeof($testTag);
   for($c=3;$c<$l;$c++)
   {
    if(isset($testTag[$c]["outer"]))
    {
     $testTag[$c]["outer"]["start"]+=$diff;
     $testTag[$c]["outer"]["end"]+=$diff;
    }

    if(isset($testTag[$c]["inner"]))
    {
     $testTag[$c]["inner"]["start"]+=$diff;
     $testTag[$c]["inner"]["end"]+=$diff;
    }
   }

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;


   //no reinit

//delete tag after <test>
   if(!$xml->_set(61, 7, "")) 
   {
    $this->e->add("XML _set should not fail");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   //no changes to existing tags
   $testTag=$this->testTag;

   if($this->_validateTags($xml->tag, $testTag)==UT_FAILED)
    return UT_FAILED;



   return UT_OK;
  }


  function SetFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//replace non existing tag
   if($xml->set("asd", "asd", true, true))
   {
    $this->e->add("XML should not set value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//replace non existing tag
   if($xml->set(-1, "asd", true, true))
   {
    $this->e->add("XML should not set value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//replace non existing tag
   if($xml->set($xml->tags, "asd", true, true))
   {
    $this->e->add("XML should not set value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetBlock()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//replace inside of tag with text 
   if(!$xml->set("test", "asd", true, false))
   {
    $this->e->add("XML should replace the inside of the existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["end"]))
   {
    $this->e->add("XML should include the new value inside of the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->set("test", "asd", false, false))
   {
    $this->e->add("XML should copy the existing block tag and replace inside of the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["end"]).substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the block tag and the new value inside of the original block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text 
   if(!$xml->set("test", "asd", true, true))
   {
    $this->e->add("XML should replace the existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."asd")
   {
    $this->e->add("XML should include the new value instead of the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test", "asd", false, true))
   {
    $this->e->add("XML should copy the existing block tag and replace the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the block tag and the new value instead of the original block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with tag 
   if(!$xml->set("test", "<asd test=\"asd\">", true, false))
   {
    $this->e->add("XML should replace inside of the existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["inner"]["end"]))
   {
    $this->e->add("XML should include the new value inside of the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with tag leaving a copy of the original
   if(!$xml->set("test", "<asd test=\"asd\">", false, false))
   {
    $this->e->add("XML should copy the existing block tag and replace inside of the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["inner"]["end"]).substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the block tag and the new value inside of the original block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test", "<asd test=\"asd\">", true, true))
   {
    $this->e->add("XML should replace the existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."<asd test=\"asd\">")
   {
    $this->e->add("XML should include the new value instead of the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test", "<asd test=\"asd\">", false, true))
   {
    $this->e->add("XML should copy the existing block tag and replace the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the block tag and the new value instead of the original block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetShort()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//replace tag with text 
   if(!$xml->set("test->unit", "asd", true, false))
   {
    $this->e->add("XML should replace the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->set("test->unit", "asd", false, false))
   {
    $this->e->add("XML should copy the existing short tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the short tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text 
   if(!$xml->set("test->unit", "asd", true, true))
   {
    $this->e->add("XML should replace the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test->unit", "asd", false, true))
   {
    $this->e->add("XML should copy the existing short tag and replace the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short tag and the new value instead of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->unit", "<asd test=\"asd\">", true, false))
   {
    $this->e->add("XML should replace the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->unit", "<asd test=\"asd\">", false, false))
   {
    $this->e->add("XML should copy an existing short tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the short tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->unit", "<asd test=\"asd\">", true, true))
   {
    $this->e->add("XML should replace the existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->unit", "<asd test=\"asd\">", false, true))
   {
    $this->e->add("XML should copy the existing short tag and replace the original short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short tag and the new value instead of the original short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetComment()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//replace tag with text 
   if(!$xml->set("test->comment", "asd", true, false))
   {
    $this->e->add("XML should replace the existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->set("test->comment", "asd", false, false))
   {
    $this->e->add("XML should copy the existing comment tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the comment tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text 
   if(!$xml->set("test->comment", "asd", true, true))
   {
    $this->e->add("XML should replace the existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test->comment", "asd", false, true))
   {
    $this->e->add("XML should copy the existing comment tag and replace the original comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the comment tag and the new value instead of the original comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->comment", "<asd test=\"asd\">", true, false))
   {
    $this->e->add("XML should replace the existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->comment", "<asd test=\"asd\">", false, false))
   {
    $this->e->add("XML should copy an existing comment tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the comment tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->comment", "<asd test=\"asd\">", true, true))
   {
    $this->e->add("XML should replace the existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->comment", "<asd test=\"asd\">", false, true))
   {
    $this->e->add("XML should copy the existing comment tag and replace the original comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the comment tag and the new value instead of the original comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetShortComment()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//replace tag with text 
   if(!$xml->set("test->shortcomment", "asd", true, false))
   {
    $this->e->add("XML should replace the existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->set("test->shortcomment", "asd", false, false))
   {
    $this->e->add("XML should copy the existing short comment tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the short comment tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text 
   if(!$xml->set("test->shortcomment", "asd", true, true))
   {
    $this->e->add("XML should replace the existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test->shortcomment", "asd", false, true))
   {
    $this->e->add("XML should copy the existing short comment tag and replace the original short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short comment tag and the new value instead of the original short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->shortcomment", "<asd test=\"asd\">", true, false))
   {
    $this->e->add("XML should replace the existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->shortcomment", "<asd test=\"asd\">", false, false))
   {
    $this->e->add("XML should copy an existing short comment tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the short comment tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->shortcomment", "<asd test=\"asd\">", true, true))
   {
    $this->e->add("XML should replace the existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->shortcomment", "<asd test=\"asd\">", false, true))
   {
    $this->e->add("XML should copy the existing short comment tag and replace the original short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the short comment tag and the new value instead of the original short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetMarker()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//replace tag with text 
   if(!$xml->set("test->marker", "asd", true, false))
   {
    $this->e->add("XML should replace the existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with text leaving a copy of the original
   if(!$xml->set("test->marker", "asd", false, false))
   {
    $this->e->add("XML should copy the existing marker tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the marker tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text 
   if(!$xml->set("test->marker", "asd", true, true))
   {
    $this->e->add("XML should replace the existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test->marker", "asd", false, true))
   {
    $this->e->add("XML should copy the existing marker tag and replace the original marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the marker tag and the new value instead of the original marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->marker", "<asd test=\"asd\">", true, false))
   {
    $this->e->add("XML should replace the existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->marker", "<asd test=\"asd\">", false, false))
   {
    $this->e->add("XML should copy an existing marker tag and add the new value");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include a copy of the marker tag and the new value");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag 
   if(!$xml->set("test->marker", "<asd test=\"asd\">", true, true))
   {
    $this->e->add("XML should replace the existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value instead of the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with tag leaving a copy of the original
   if(!$xml->set("test->marker", "<asd test=\"asd\">", false, true))
   {
    $this->e->add("XML should copy the existing marker tag and replace the original marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }
  
   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."<asd test=\"asd\">".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include the a copy of the marker tag and the new value instead of the original marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetMultiple()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if(!$xml->set("test->comment", "asd", false, false))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->set("test->shortcomment", "asd", false, false))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->set("test->unit", "asd", false, false))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!$xml->set("test", "asd", false, false))
   {
    $this->e->add("XML should add the new value to the existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetArray()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   $values=Array(
    "unit" => "Unit replacement",
    "comment" => "Comment replacement",
    "shortcomment" => "Short comment replacement",
    "marker" => "Marker replacement",
   );


//replace inside of tag with array
   if(!$xml->set("test", $values, true, false))
   {
    $this->e->add("XML should replace inside of an existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit!s", $xml->xml))
   {
    $this->e->add("XML Unit replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment!s", $xml->xml))
   {
    $this->e->add("XML Comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment!s", $xml->xml))
   {
    $this->e->add("XML Short comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker!s", $xml->xml))
   {
    $this->e->add("XML Marker replacement should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with array leaving a copy of the original
   if(!$xml->set("test", $values, false, false))
   {
    $this->e->add("XML should copy an existing block tag and replace inside of the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit!s", $xml->xml))
   {
    $this->e->add("XML Unit replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment!s", $xml->xml))
   {
    $this->e->add("XML Comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment!s", $xml->xml))
   {
    $this->e->add("XML Short comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker!s", $xml->xml))
   {
    $this->e->add("XML Marker replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->xml))
   {
    $this->e->add("XML unit tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->xml))
   {
    $this->e->add("XML comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->xml))
   {
    $this->e->add("XML short comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->xml))
   {
    $this->e->add("XML marker tag should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with array
   if(!$xml->set("test", $values, true, true))
   {
    $this->e->add("XML should replace an existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit replacement!", $xml->xml))
   {
    $this->e->add("XML Unit replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment replacement!", $xml->xml))
   {
    $this->e->add("XML Comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment replacement!", $xml->xml))
   {
    $this->e->add("XML Short comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker replacement!", $xml->xml))
   {
    $this->e->add("XML Marker replacement should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test", $values, false, true))
   {
    $this->e->add("XML should copy an existing block tag and replace the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit replacement!", $xml->xml))
   {
    $this->e->add("XML Unit replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment replacement!", $xml->xml))
   {
    $this->e->add("XML Comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment replacement!", $xml->xml))
   {
    $this->e->add("XML Short comment replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker replacement!", $xml->xml))
   {
    $this->e->add("XML Marker replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->xml))
   {
    $this->e->add("XML unit tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->xml))
   {
    $this->e->add("XML comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->xml))
   {
    $this->e->add("XML short comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->xml))
   {
    $this->e->add("XML marker tag should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetMultiArray()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   $values=Array(
    Array(
     "unit" => "Unit1 replacement",
     "comment" => "Comment1 replacement",
     "shortcomment" => "Short comment1 replacement",
     "marker" => "Marker1 replacement",
    ),
    Array(
     "unit" => "Unit2 replacement",
     "comment" => "Comment2 replacement",
     "shortcomment" => "Short comment2 replacement",
     "marker" => "Marker2 replacement",
    ),
   );


//replace inside of tag with array
   if(!$xml->set("test", $values, true, false))
   {
    $this->e->add("XML should replace inside of an existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit1!s", $xml->xml))
   {
    $this->e->add("XML Unit1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment1!s", $xml->xml))
   {
    $this->e->add("XML Comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment1!s", $xml->xml))
   {
    $this->e->add("XML Short comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker1!s", $xml->xml))
   {
    $this->e->add("XML Marker1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit2!s", $xml->xml))
   {
    $this->e->add("XML Unit2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment2!s", $xml->xml))
   {
    $this->e->add("XML Comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment2!s", $xml->xml))
   {
    $this->e->add("XML Short comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker2!s", $xml->xml))
   {
    $this->e->add("XML Marker2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace inside of tag with array leaving a copy of the original
   if(!$xml->set("test", $values, false, false))
   {
    $this->e->add("XML should copy an existing block tag and replace inside of the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit1!s", $xml->xml))
   {
    $this->e->add("XML Unit1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment1!s", $xml->xml))
   {
    $this->e->add("XML Comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment1!s", $xml->xml))
   {
    $this->e->add("XML Short comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker1!s", $xml->xml))
   {
    $this->e->add("XML Marker1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Unit2!s", $xml->xml))
   {
    $this->e->add("XML Unit2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Comment2!s", $xml->xml))
   {
    $this->e->add("XML Comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Short comment2!s", $xml->xml))
   {
    $this->e->add("XML Short comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+Marker2!s", $xml->xml))
   {
    $this->e->add("XML Marker2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->xml))
   {
    $this->e->add("XML unit tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->xml))
   {
    $this->e->add("XML comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->xml))
   {
    $this->e->add("XML short comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->xml))
   {
    $this->e->add("XML marker tag should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with array
   if(!$xml->set("test", $values, true, true))
   {
    $this->e->add("XML should replace an existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit1 replacement!", $xml->xml))
   {
    $this->e->add("XML Unit1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment1 replacement!", $xml->xml))
   {
    $this->e->add("XML Comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment1 replacement!", $xml->xml))
   {
    $this->e->add("XML Short comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker1 replacement!", $xml->xml))
   {
    $this->e->add("XML Marker1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Unit2 replacement!", $xml->xml))
   {
    $this->e->add("XML Unit2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment2 replacement!", $xml->xml))
   {
    $this->e->add("XML Comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment2 replacement!", $xml->xml))
   {
    $this->e->add("XML Short comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker2 replacement!", $xml->xml))
   {
    $this->e->add("XML Marker2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//replace tag with text leaving a copy of the original
   if(!$xml->set("test", $values, false, true))
   {
    $this->e->add("XML should copy an existing block tag and replace the original block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!Unit1 replacement!", $xml->xml))
   {
    $this->e->add("XML Unit1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment1 replacement!", $xml->xml))
   {
    $this->e->add("XML Comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment1 replacement!", $xml->xml))
   {
    $this->e->add("XML Short comment1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker1 replacement!", $xml->xml))
   {
    $this->e->add("XML Marker1 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Unit2 replacement!", $xml->xml))
   {
    $this->e->add("XML Unit2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Comment2 replacement!", $xml->xml))
   {
    $this->e->add("XML Comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Short comment2 replacement!", $xml->xml))
   {
    $this->e->add("XML Short comment2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!Marker2 replacement!", $xml->xml))
   {
    $this->e->add("XML Marker2 replacement should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<unit!s", $xml->xml))
   {
    $this->e->add("XML unit tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!-- comment!s", $xml->xml))
   {
    $this->e->add("XML comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+<\!shortcomment!s", $xml->xml))
   {
    $this->e->add("XML short comment tag should exist in the block tag");
    return UT_FAILED;
   }

   if(!preg_match("!<test .+#marker!s", $xml->xml))
   {
    $this->e->add("XML marker tag should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddBeforeFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//AddBefore non existing tag
   if($xml->addBefore("asd", "asd"))
   {
    $this->e->add("XML should not addBefore value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddBefore non existing tag
   if($xml->addBefore(-1, "asd"))
   {
    $this->e->add("XML should not addBefore value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddBefore non existing tag
   if($xml->addBefore($xml->tags, "asd"))
   {
    $this->e->add("XML should not addBefore value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddBefore()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add text before test tag
   if(!$xml->addBefore("test", "asd"))
   {
    $this->e->add("XML should addBefore value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before short tag
   if(!$xml->addBefore("test->unit", "asd"))
   {
    $this->e->add("XML should addBefore value for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before comment tag
   if(!$xml->addBefore("test->comment", "asd"))
   {
    $this->e->add("XML should addBefore value for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before shortcomment tag
   if(!$xml->addBefore("test->shortcomment", "asd"))
   {
    $this->e->add("XML should addBefore value for existing shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the shortcomment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before marker tag
   if(!$xml->addBefore("test->marker", "asd"))
   {
    $this->e->add("XML should addBefore value for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["start"]))
   {
    $this->e->add("XML should include the new value before the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddAfterFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//AddAfter non existing tag
   if($xml->addAfter("asd", "asd"))
   {
    $this->e->add("XML should not addAfter value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddAfter non existing tag
   if($xml->addAfter(-1, "asd"))
   {
    $this->e->add("XML should not addAfter value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddAfter non existing tag
   if($xml->addAfter($xml->tags, "asd"))
   {
    $this->e->add("XML should not addAfter value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddAfter()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add text before test tag
   if(!$xml->addAfter("test", "asd"))
   {
    $this->e->add("XML should addAfter value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["end"])."asd".substr($this->testXML, $this->testTag[1]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value before the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before short tag
   if(!$xml->addAfter("test->unit", "asd"))
   {
    $this->e->add("XML should addAfter value for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["end"])."asd".substr($this->testXML, $this->testTag[3]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value before the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before comment tag
   if(!$xml->addAfter("test->comment", "asd"))
   {
    $this->e->add("XML should addAfter value for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["end"])."asd".substr($this->testXML, $this->testTag[2]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value before the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before shortcomment tag
   if(!$xml->addAfter("test->shortcomment", "asd"))
   {
    $this->e->add("XML should addAfter value for existing shortcomment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["end"])."asd".substr($this->testXML, $this->testTag[5]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value before the shortcomment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add text before marker tag
   if(!$xml->addAfter("test->marker", "asd"))
   {
    $this->e->add("XML should addAfter value for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["end"])."asd".substr($this->testXML, $this->testTag[4]["outer"]["end"]))
   {
    $this->e->add("XML should include the new value before the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtBeginFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//AddInsideAtBegin non existing tag
   if($xml->addInsideAtBegin("asd", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin non existing tag
   if($xml->addInsideAtBegin(-1, "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin non existing tag
   if($xml->addInsideAtBegin($xml->tags, "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin short tag
   if($xml->addInsideAtBegin("test->unit", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for short tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin comment tag
   if($xml->addInsideAtBegin("test->comment", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for comment tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin shortcomment tag
   if($xml->addInsideAtBegin("test->shortcomment", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for shortcomment tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtBegin marker tag
   if($xml->addInsideAtBegin("test->marker", "asd"))
   {
    $this->e->add("XML should not addInsideAtBegin value for marker tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtBegin()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add text before test tag
   if(!$xml->addInsideAtBegin("test", "asd"))
   {
    $this->e->add("XML should addInsideAtBegin value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["inner"]["start"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["start"]))
   {
    $this->e->add("XML should include the new value before the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtEndFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//AddInsideAtEnd non existing tag
   if($xml->addInsideAtEnd("asd", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd non existing tag
   if($xml->addInsideAtEnd(-1, "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd non existing tag
   if($xml->addInsideAtEnd($xml->tags, "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd short tag
   if($xml->addInsideAtEnd("test->unit", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for short tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd comment tag
   if($xml->addInsideAtEnd("test->comment", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for comment tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd shortcomment tag
   if($xml->addInsideAtEnd("test->shortcomment", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for shortcomment tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//AddInsideAtEnd marker tag
   if($xml->addInsideAtEnd("test->marker", "asd"))
   {
    $this->e->add("XML should not addInsideAtEnd value for marker tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function AddInsideAtEnd()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add text before test tag
   if(!$xml->addInsideAtEnd("test", "asd"))
   {
    $this->e->add("XML should addInsideAtEnd value for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["inner"]["end"])."asd".substr($this->testXML, $this->testTag[1]["inner"]["end"]))
   {
    $this->e->add("XML should include the new value before the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function setAttributeFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//SetAttribute non existing tag
   if($xml->setAttribute("asd", "asd"))
   {
    $this->e->add("XML should not setAttribute value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->setAttribute(-1, "asd"))
   {
    $this->e->add("XML should not setAttribute value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->setAttribute($xml->tags, "asd"))
   {
    $this->e->add("XML should not setAttribute value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }

  function SetAttributeBlock()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add attribute for block tag
   if(!$xml->setAttribute("test", "lala", "asd")) 
   {
    $this->e->add("XML should set attribute for existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test ut=\"xml\" lala=\"asd\">!", $xml->xml))
   {
    $this->e->add("New XML attribute should exist in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//edit attribute for block tag
   if(!$xml->setAttribute("test", "lala", "asdasdads")) 
   {
    $this->e->add("XML should set attribute for existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test ut=\"xml\" lala=\"asdasdads\">!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be changed in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for block tag
   if(!$xml->setAttribute("test", "ut")) 
   {
    $this->e->add("XML should set attribute for existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test lala=\"asdasdads\">!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for block tag
   if(!$xml->setAttribute("test", "lala")) 
   {
    $this->e->add("XML should set attribute for existing block tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<test>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the block tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeShort()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add attribute for short tag
   if(!$xml->setAttribute("test->unit", "lala", "asd")) 
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\" lala=\"asd\">!", $xml->xml))
   {
    $this->e->add("New XML attribute should exist in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//edit attribute for short tag
   if(!$xml->setAttribute("test->unit", "lala", "asdasdads")) 
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\" lala=\"asdasdads\">!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be changed in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short tag
   if(!$xml->setAttribute("test->unit", "lala")) 
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\">!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short tag
   if(!$xml->setAttribute("test->unit", "test")) 
   {
    $this->e->add("XML should set attribute for existing short tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the short tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//add attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "lala", "asd")) 
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\" encoding=\"ISO-8859-1\" lala=\"asd\"\?>!", $xml->xml))
   {
    $this->e->add("New XML attribute should exist in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//edit attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "lala", "asdasdads")) 
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\" encoding=\"ISO-8859-1\" lala=\"asdasdads\"\?>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be changed in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "lala")) 
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\" encoding=\"ISO-8859-1\"\?>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "encoding")) 
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml version=\"1.0\"\?>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the short ?xml tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short ?xml tag
   if(!$xml->setAttribute("?xml", "version")) 
   {
    $this->e->add("XML should set attribute for existing short ?xml tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\?xml\?>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the short ?xml tag");
    return UT_FAILED;
   }

   /*
    * Skip: <?xml?> name is interpreted as ?xml?
    *
   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;
   */

   return UT_OK;
  }


  function SetAttributeComment()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add attribute for comment tag
   if(!$xml->setAttribute("test->comment", "lala", "asd")) 
   {
    $this->e->add("XML should set attribute for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!-- comment lala=\"asd\" -->!", $xml->xml))
   {
    $this->e->add("New XML attribute should exist in the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//edit attribute for comment tag
   if(!$xml->setAttribute("test->comment", "lala", "asdasdads")) 
   {
    $this->e->add("XML should set attribute for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!-- comment lala=\"asdasdads\" -->!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be changed in the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for comment tag
   if(!$xml->setAttribute("test->comment", "lala")) 
   {
    $this->e->add("XML should set attribute for existing comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!-- comment -->!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeShortComment()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add attribute for short comment tag
   if(!$xml->setAttribute("test->shortcomment", "lala", "asd")) 
   {
    $this->e->add("XML should set attribute for existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!shortcomment lala=\"asd\">!", $xml->xml))
   {
    $this->e->add("New XML attribute should exist in the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//edit attribute for short comment tag
   if(!$xml->setAttribute("test->shortcomment", "lala", "asdasdads")) 
   {
    $this->e->add("XML should set attribute for existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!shortcomment lala=\"asdasdads\">!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be changed in the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for short comment tag
   if(!$xml->setAttribute("test->shortcomment", "lala")) 
   {
    $this->e->add("XML should set attribute for existing short comment tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<\!shortcomment>!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the short comment tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function SetAttributeMarker()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//add attribute for marker tag
   if(!$xml->setAttribute("test->marker", "lala", "asd")) 
   {
    $this->e->add("XML should set attribute for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!#marker lala=\"asd\"#!", $xml->xml))
   {
    $this->e->add("New XML attribute should exist in the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//edit attribute for marker tag
   if(!$xml->setAttribute("test->marker", "lala", "asdasdads")) 
   {
    $this->e->add("XML should set attribute for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!#marker lala=\"asdasdads\"#!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be changed in the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete attribute for marker tag
   if(!$xml->setAttribute("test->marker", "lala")) 
   {
    $this->e->add("XML should set attribute for existing marker tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!#marker#!", $xml->xml))
   {
    $this->e->add("Existing XML attribute should be removed in the marker tag");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function RenameFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//SetAttribute non existing tag
   if($xml->rename("asd", "lala"))
   {
    $this->e->add("XML should not rename value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->rename(-1, "lala"))
   {
    $this->e->add("XML should not rename value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->rename($xml->tags, "lala"))
   {
    $this->e->add("XML should not rename value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function Rename()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//rename first
   if(!$xml->rename("?xml", "?lala"))
   {
    $this->e->add("XML should rename existing first tag");
    return UT_FAILED;
   }

   if($xml->xml!=str_replace("<?xml", "<?lala", $this->testXML))
   {
    $this->e->add("XML tag ?xml should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//rename block
   if(!$xml->rename("test", "lala"))
   {
    $this->e->add("XML should rename existing block tag");
    return UT_FAILED;
   }

   if($xml->xml!=str_replace("<test", "<lala", str_replace("</test", "</lala", $this->testXML)))
   {
    $this->e->add("XML tag test should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//rename short
   if(!$xml->rename("test->unit", "lala"))
   {
    $this->e->add("XML should rename existing block tag");
    return UT_FAILED;
   }

   if($xml->xml!=str_replace("<unit", "<lala", $this->testXML))
   {
    $this->e->add("XML tag unit should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//rename comment
   if(!$xml->rename("test->comment", "lala"))
   {
    $this->e->add("XML should rename existing comment tag");
    return UT_FAILED;
   }

   if($xml->xml!=str_replace("<!-- comment", "<!-- lala", $this->testXML))
   {
    $this->e->add("XML tag comment should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//rename short comment
   if(!$xml->rename("test->shortcomment", "lala"))
   {
    $this->e->add("XML should rename existing short comment tag");
    return UT_FAILED;
   }

   if($xml->xml!=str_replace("<!shortcomment", "<!lala", $this->testXML))
   {
    $this->e->add("XML tag should be renamed");
    return UT_FAILED;
   }


   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//rename marker
   if(!$xml->rename("test->marker", "lala"))
   {
    $this->e->add("XML should rename existing marker tag");
    return UT_FAILED;
   }

   if($xml->xml!=str_replace("#marker#", "#lala#", $this->testXML))
   {
    $this->e->add("XML tag should be renamed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function DeleteFailures()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//SetAttribute non existing tag
   if($xml->delete("asd", "asd"))
   {
    $this->e->add("XML should not delete value for non existing tag");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->delete(-1, "asd"))
   {
    $this->e->add("XML should not delete value for negative key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


//SetAttribute non existing tag
   if($xml->delete($xml->tags, "asd"))
   {
    $this->e->add("XML should not delete value for non existing key number");
    return UT_FAILED;
   }

   if($xml->xml!=$this->testXML)
   {
    $this->e->add("XML should be the old value");
    return UT_FAILED;
   }

   if($this->_validateTags($xml->tag, $this->testTag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function Delete()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//delete first
   if(!$xml->delete("?xml"))
   {
    $this->e->add("XML should delete existing first tag");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, $this->testTag[0]["outer"]["end"]))
   {
    $this->e->add("XML first tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//delete block
   if(!$xml->delete("test"))
   {
    $this->e->add("XML should delete existing block tag");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[1]["outer"]["start"]).substr($this->testXML, $this->testTag[1]["outer"]["end"]))
   {
    $this->e->add("XML block tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//delete short
   if(!$xml->delete("test->unit"))
   {
    $this->e->add("XML should delete existing short tag");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[3]["outer"]["start"]).substr($this->testXML, $this->testTag[3]["outer"]["end"]))
   {
    $this->e->add("XML short tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//delete comment
   if(!$xml->delete("test->comment"))
   {
    $this->e->add("XML should delete existing comment tag");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[2]["outer"]["start"]).substr($this->testXML, $this->testTag[2]["outer"]["end"]))
   {
    $this->e->add("XML comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//delete short comment
   if(!$xml->delete("test->shortcomment"))
   {
    $this->e->add("XML should delete existing short comment tag");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[5]["outer"]["start"]).substr($this->testXML, $this->testTag[5]["outer"]["end"]))
   {
    $this->e->add("XML short comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//delete marker
   if(!$xml->delete("test->marker"))
   {
    $this->e->add("XML should delete existing marker tag");
    return UT_FAILED;
   }

   if($xml->xml!=substr($this->testXML, 0, $this->testTag[4]["outer"]["start"]).substr($this->testXML, $this->testTag[4]["outer"]["end"]))
   {
    $this->e->add("XML marker tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   return UT_OK;
  }


  function DeleteMultiple()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//delete marker
   if(!$xml->delete("test->marker"))
   {
    $this->e->add("XML should delete existing marker tag");
    return UT_FAILED;
   }

   if(preg_match("!#marker#!", $xml->xml))
   {
    $this->e->add("XML marker tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete short comment
   if(!$xml->delete("test->shortcomment"))
   {
    $this->e->add("XML should delete existing short comment tag");
    return UT_FAILED;
   }

   if(preg_match("!<\!shortcomment>!", $xml->xml))
   {
    $this->e->add("XML short comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete comment
   if(!$xml->delete("test->comment"))
   {
    $this->e->add("XML should delete existing comment tag");
    return UT_FAILED;
   }

   if(preg_match("!<\!-- comment -->!", $xml->xml))
   {
    $this->e->add("XML comment tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   $xml->reinit();

//delete short
   if(!$xml->delete("test->unit"))
   {
    $this->e->add("XML should delete existing short tag");
    return UT_FAILED;
   }

   if(preg_match("!<unit test=\"asd\">!", $xml->xml))
   {
    $this->e->add("XML short tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete block
   if(!$xml->delete("test"))
   {
    $this->e->add("XML should delete existing block tag");
    return UT_FAILED;
   }

   if(preg_match("!<test ut=\"xml\">!", $xml->xml))
   {
    $this->e->add("XML block tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


//delete first
   if(!$xml->delete("?xml"))
   {
    $this->e->add("XML should delete existing first tag");
    return UT_FAILED;
   }

   if(preg_match("!\?xml!", $xml->xml))
   {
    $this->e->add("XML first tag should be removed");
    return UT_FAILED;
   }

   //validate tags against fresh parse of the XML
   $testXML=new XML();
   $testXML->parse($xml->xml);

   if($this->_validateTags($xml->tag, $testXML->tag)==UT_FAILED)
    return UT_FAILED;


   if(trim($xml->xml))
   {
    $this->e->add("XML should be empty");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function GetPathAndKey()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getPathAndKey("asd"))
   {
    $this->e->add("XML should not get path and key for non existing tag");
    return UT_FAILED;
   }

   if(!($key=$xml->getPathAndKey("test->unit")))
   {
    $this->e->add("XML should get path and key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key[0]!="test->unit")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   if($key[1]!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   if(!($key=$xml->getPathAndKey(3)))
   {
    $this->e->add("XML should get path and key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key[0]!="test[0]->unit[0]")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   if($key[1]!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   $xml->next(); //test
   $xml->next(); //test->comment
   $xml->next(); //test->unit

   if(!($key=$xml->getPathAndKey(false)))
   {
    $this->e->add("XML should get path and key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key[0]!="test[0]->unit[0]")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   if($key[1]!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetKey()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getKey("asd"))
   {
    $this->e->add("XML should not get key for non existing tag");
    return UT_FAILED;
   }

   if(($key=$xml->getKey("test->unit"))===false)
   {
    $this->e->add("XML should get key for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3)
   {
    $this->e->add("XML key incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetName()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getName("asd"))
   {
    $this->e->add("XML should not get name for non existing tag");
    return UT_FAILED;
   }

   if(!($path=$xml->getName(3)))
   {
    $this->e->add("XML should get name for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($path!="unit")
   {
    $this->e->add("XML name incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetPath()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getPath("asd"))
   {
    $this->e->add("XML should not get path for non existing tag");
    return UT_FAILED;
   }

   if(!($path=$xml->getPath(3)))
   {
    $this->e->add("XML should get path for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($path!="test[0]->unit[0]")
   {
    $this->e->add("XML path incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetInner()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getInner("asd"))
   {
    $this->e->add("XML should not get inner for non existing tag");
    return UT_FAILED;
   }

   if(($inner=$xml->getInner("test"))===false)
   {
    $this->e->add("XML should get inner for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!<unit test=\"asd\">!", $inner))
   {
    $this->e->add("XML inner should contain unit tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetOuter()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getOuter("asd"))
   {
    $this->e->add("XML should not get outer for non existing tag");
    return UT_FAILED;
   }

   if(($outer=$xml->getOuter("test->unit"))===false)
   {
    $this->e->add("XML should get outer for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!preg_match("!^<unit test=\"asd\">$!", $outer))
   {
    $this->e->add("XML outer should contain unit tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetAttributes()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getAttributes("asd"))
   {
    $this->e->add("XML should not get attributes for non existing tag");
    return UT_FAILED;
   }

   if(($attributes=$xml->getAttributes("test"))===false)
   {
    $this->e->add("XML should get attributes for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!isset($attributes["ut"])||$attributes["ut"]!="xml")
   {
    $this->e->add("XML attributes incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetAttribute()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getAttribute("asd", "asd"))
   {
    $this->e->add("XML should not get attribute value for non existing tag");
    return UT_FAILED;
   }

   if($xml->getAttribute("test", "asd"))
   {
    $this->e->add("XML should not get attribute value for non existing attribute");
    return UT_FAILED;
   }

   if(($attribute=$xml->getAttribute("test", "ut"))===false)
   {
    $this->e->add("XML should get value for existing attribute");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($attribute!="xml")
   {
    $this->e->add("XML attribute incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetChildren()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getChildren("asd"))
   {
    $this->e->add("XML should not get children for non existing tag");
    return UT_FAILED;
   }

   if(!($childs=$xml->getChildren("test")))
   {
    $this->e->add("XML should get children for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($childs[0]!=2)
   {
    $this->e->add("XML children incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetOuterXML()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getOuterXML("asd"))
   {
    $this->e->add("XML should not get outer XML object for non existing tag");
    return UT_FAILED;
   }

   if(!($xml2=$xml->getOuterXML("test")))
   {
    $this->e->add("XML should get outer XML object for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!is_object($xml2)||get_class($xml2)!="XML")
   {
    $this->e->add("XML sub XML object incorrect");
    return UT_FAILED;
   }

   if($xml2->getName(0)!="test")
   {
    $this->e->add("XML get outer XML doesn't have outer tag as first tag");
    $this->e->add($xml2->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetInnerXML()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getInnerXML("asd"))
   {
    $this->e->add("XML should not get inner XML object for non existing tag");
    return UT_FAILED;
   }

   if(!($xml2=$xml->getInnerXML("test")))
   {
    $this->e->add("XML should get inner XML object for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if(!is_object($xml2)||get_class($xml2)!="XML")
   {
    $this->e->add("XML sub XML object incorrect");
    return UT_FAILED;
   }

   if($xml2->getName(0)!="comment")
   {
    $this->e->add("XML get iner XML doesn't have inner tag as first tag");
    $this->e->add($xml2->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetType()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->getType("asd"))
   {
    $this->e->add("XML should not get type for non existing tag");
    return UT_FAILED;
   }

   if(($type=$xml->getType("test"))===false)
   {
    $this->e->add("XML should get type for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($type!="block")
   {
    $this->e->add("XML type incorrect");
    return UT_FAILED;
   }

   if(($type=$xml->getType("test->unit"))===false)
   {
    $this->e->add("XML should get type for existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($type!="short")
   {
    $this->e->add("XML type incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Current()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->current("asd"))
   {
    $this->e->add("XML should not current non existing tag");
    return UT_FAILED;
   }

   if($xml->current("test")===false)
   {
    $this->e->add("XML should current existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->current!=1)
   {
    $this->e->add("XML current incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Next()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//test   
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->current!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->comment
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=2||$xml->current!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="comment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->current!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->shortcomment
   if(($key=$xml->next())===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->current!=5)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="shortcomment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next()!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function NextFilters()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

//comments, markers, close tags
//
//test   
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->current!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->comment
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=2||$xml->current!=2)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="comment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->current!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->shortcomment
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->current!=5)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="shortcomment")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }


//test (close)
   if(($key=$xml->next(true, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->current!=6)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(true, true, true)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }


//markers, close tags
//
//test   
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->current!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->marker
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=4||$xml->current!=4)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="marker")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test (close)
   if(($key=$xml->next(false, true, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->current!=6)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(false, true, true)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

//close tags
//
//test   
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->current!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test (close)
   if(($key=$xml->next(false, false, true))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->current!=6)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(false, false, true)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

//normal tags
//
//test   
   if(($key=$xml->next(false, false, false))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->current!=1)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="test")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

//test->unit
   if(($key=$xml->next(false, false, false))===false)
   {
    $this->e->add("XML should goto next existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML goto next incorrect");
    return UT_FAILED;
   }

   if($xml->getName($key)!="unit")
   {
    $this->e->add("XML goto next incorrect key");
    return UT_FAILED;
   }

   if($xml->next(false, false, false)!==false)
   {
    $this->e->add("XML should not goto next non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Search()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if(($key=$xml->search("unit"))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }


   $xml->reset();

   if(($key=$xml->search("shortcomment"))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=5||$xml->current!=5)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }

   if($xml->search("shortcomment")!==false)
   {
    $this->e->add("XML search should not find unique tag twice");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("asd")!==false)
   {
    $this->e->add("XML should not search non existing tag");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SearchFilters()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if(($key=$xml->search("unit", false, false, false))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=3||$xml->current!=3)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }


   $xml->reset();

   if(($key=$xml->search("test", false, false, true))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=1||$xml->current!=1)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }

   if(($key=$xml->search("test", false, false, true))===false)
   {
    $this->e->add("XML search should find existing tag");
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($key!=6||$xml->current!=6)
   {
    $this->e->add("XML search incorrect");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("comment", false, false, false)!==false)
   {
    $this->e->add("XML search should not find comment tag");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("shortcomment", false, false, false)!==false)
   {
    $this->e->add("XML search should not find comment tag");
    return UT_FAILED;
   }


   $xml->reset();

   if($xml->search("marker", false, false, false)!==false)
   {
    $this->e->add("XML search should not find marker");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function Reset()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   $xml->next();
   $xml->reset();

   if($xml->current!==0)
   {
    $this->e->add("XML reset incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   if($xml->output()!=$this->testXML)
   {
    $this->e->add("XML output incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Display()
  {
   $xml=new XML();
   $xml->parse($this->testXML);

   $display=$xml->display();
   $display=preg_replace("!<[^>]+>!", "", $display);
   $display=html_entity_decode($display);
   $display=preg_replace("!>[^<]*<!", "><", $display);
   $display=trim($display);

   $testXML=preg_replace("!>[^<]*<!", "><", $this->testXML);

   if($display!=$testXML)
   {
    $this->e->add("XML display incorrect");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function BugShortTag()
  {
   $xml=new XML();

   if(!$xml->parse("<test1/><!test2/>"))
   {
    $this->e->add($xml->e);
    return UT_FAILED;
   }

   if($xml->getName(0)!="test1")
   {
    $this->e->add("First tag should be: test1");
    return UT_FAILED;
   }

   if($xml->getName(1)!="test2")
   {
    $this->e->add("Second tag should be: test2");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
