<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/xmli.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/xml.php");
 require_once($_path["include"]."xmli.php");

 class UT_XMLi extends UT_XML
 {
  function Constructor()
  {
   $object=new XMLi();

   if(!is_object($object)||get_class($object)!="XMLi")
   {
    $this->e->add("Constructor doesn't return a XMLi object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->xmlVersion!="1.0")
   {
    $this->e->add("XML version isn't 1.0");
    return UT_FAILED;
   }

   if($object->xmlEncoding!="ISO-8859-1")
   {
    $this->e->add("XML enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   if(!file_exists($object->templates))
   {
    $this->e->add("Templates directory doesn't exist");
    return UT_FAILED;
   }

   if(!is_dir($object->templates))
   {
    $this->e->add("Templates directory isn't a directory");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_XMLi_1_0 extends UT_XMLi
 {
  function ParseTemplate()
  {
   $xmli=new XMLi();

   if($xmli->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   if(!$xmli->parseTemplate("jas"))
   {
    $this->e->add("Can't parse template \"jas\"");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(!$xmli->parseTemplate("errors"))
   {
    $this->e->add("Can't parse template \"errors\"");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(!$xmli->parseTemplate("data"))
   {
    $this->e->add("Can't parse template \"data\"");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(!$xmli->parseTemplate("modules"))
   {
    $this->e->add("Can't parse template \"modules\"");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(!$xmli->parseTemplate("projects"))
   {
    $this->e->add("Can't parse template \"projects\"");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(!$xmli->parseTemplate("language"))
   {
    $this->e->add("Can't parse template \"language\"");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if($xmli->parseTemplate("foobar"))
   {
    $this->e->add("Parsing template \"foobar\" should fail");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }


 class UT_XMLiServer extends UT_XMLi
 {
  function Constructor()
  {
   $object=new XMLiServer(false);

   if(!is_object($object)||get_class($object)!="XMLiServer")
   {
    $this->e->add("Constructor doesn't return a XMLiServer object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->xmlVersion!="1.0")
   {
    $this->e->add("XML version isn't 1.0");
    return UT_FAILED;
   }

   if($object->xmlEncoding!="ISO-8859-1")
   {
    $this->e->add("XML enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   if(!file_exists($object->templates))
   {
    $this->e->add("Templates directory doesn't exist");
    return UT_FAILED;
   }

   if(!is_dir($object->templates))
   {
    $this->e->add("Templates directory isn't a directory");
    return UT_FAILED;
   }

   //config merge
   $object=new XMLiServer(false, Array("xmlViewItem" => true, "ViewItemModules" => true, "xmlUserLoginLogout" => true));

   if(!$object->actions["view"]["item"])
   {
    $this->e->add("XMLiServer view item should be enabled");
    return UT_FAILED;
   }

   if(!$object->actions["user"]["logout"])
   {
    $this->e->add("XMLiServer user logout should be enabled");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function Error()
  {
   $xmli=new XMLiServer(false);

   if(!$xmli->error())
   {
    $this->e->add("Call failed");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if($xmli->getChildren("jas->errors"))
   {
    $this->e->add("Response should contain any errors");
    return UT_FAILED;
   }

   $xmli->e->add("Test");
   if(!$xmli->error())
   {
    $this->e->add("Call failed");
    return UT_FAILED;
   }

   if(sizeof($xmli->getChildren("jas->errors"))!=1)
   {
    $this->e->add("Response should contain 1 error");
    return UT_FAILED;
   }

   if(!$xmli->error("Test"))
   {
    $this->e->add("Call with error failed");
    return UT_FAILED;
   }

   if(sizeof($xmli->getChildren("jas->errors"))!=2)
   {
    $this->e->add("Response should contain 2 errors");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Handle()
  {
   $xmli=new XMLiServer(false);

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <modules></modules>
</jas>';

   if(!$xmli->handle($request))
   {
    $this->e->add("Modules request failed");
    return UT_FAILED;
   }

   if($xmli->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   $xmli->reset();
   if($xmli->search("modules")===false)
   {
    $this->e->add("Modules request should respond with modules list");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <projects></projects>
</jas>';

   if(!$xmli->handle($request))
   {
    $this->e->add("Projects request failed");
    return UT_FAILED;
   }

   if($xmli->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   $xmli->reset();
   if($xmli->search("projects")===false)
   {
    $this->e->add("Projects request should respond with projects list");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   $request='<?xml version="1.0" encoding="ISO-8859-1"?>
<jas version="1.0">
 <foobar></foobar>
</jas>';

   if($xmli->handle($request))
   {
    $this->e->add("FooBar request should fail");
    return UT_FAILED;
   }

   if($xmli->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   $xmli->reset();
   if($xmli->search("errors")===false)
   {
    $this->e->add("FooBar request should respond with errors list");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HandleData()
  {
   return UT_SKIPPED;
  }


  function HandleModules()
  {
   $subXML=new XML();
   $subXML->parse("<modules></modules>");

   $xmli=new XMLiServer(false);
   if(!$xmli->handle_modules_1_0($subXML))
   {
    $this->e->add("Call failed");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(sizeof($xmli->getChildren("jas->modules"))<1)
   {
    $this->e->add("Response should contain at least 1 module");
    return UT_FAILED;
   }

   $subXML=new XML();
   $subXML->parse("<modules><module name=\"categories\"></modules>");

   $xmli=new XMLiServer(false);
   if(!$xmli->handle_modules_1_0($subXML))
   {
    $this->e->add("Call failed");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(sizeof($xmli->getChildren("jas->modules"))!=1)
   {
    $this->e->add("Response should contain 1 module");
    return UT_FAILED;
   }

   if(sizeof($xmli->getChildren("jas->modules->module"))<1)
   {
    $this->e->add("Response should contain at least 1 field or reference");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HandleProjects()
  {
   $subXML=new XML();
   $subXML->parse("<projects></projects>");

   $xmli=new XMLiServer(false);
   if(!$xmli->handle_projects_1_0($subXML))
   {
    $this->e->add("Call failed");
    $this->e->add($xmli->e);
    return UT_FAILED;
   }

   if(sizeof($xmli->getChildren("jas->projects"))<1)
   {
    $this->e->add("Response should contain at least 1 project");
    return UT_FAILED;
   }

   $found=false;
   $xmli->search("projects");

   while(($key=$xmli->next())!==false)
   {
    if($xmli->getName($key)!="project")
     break;

    if($xmli->getAttribute($key, "current")=="yes")
     $found=true;
   }

   if(!$found)
   {
    $this->e->add("No current project found");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function HandleLanguage()
  {
   return UT_SKIPPED;
  }


 }


 class UT_XMLiClient extends UT_XMLi
 {
  function Constructor()
  {
   $object=new XMLiClient("http://test.com/");

   if(!is_object($object)||get_class($object)!="XMLiClient")
   {
    $this->e->add("Constructor doesn't return a XMLiClient object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Error")
   {
    $this->e->add("Object: \"e\" isn't an Error object", "error");
    return UT_FAILED;
   }

   if($object->xmlVersion!="1.0")
   {
    $this->e->add("XML version isn't 1.0");
    return UT_FAILED;
   }

   if($object->xmlEncoding!="ISO-8859-1")
   {
    $this->e->add("XML enconding isn't ISO-8859-1");
    return UT_FAILED;
   }

   if($object->xmlJasVersion!="1.0")
   {
    $this->e->add("Jas XML version isn't 1.0");
    return UT_FAILED;
   }

   if(!file_exists($object->templates))
   {
    $this->e->add("Templates directory doesn't exist");
    return UT_FAILED;
   }

   if(!is_dir($object->templates))
   {
    $this->e->add("Templates directory isn't a directory");
    return UT_FAILED;
   }

   if($object->url!="http://test.com/xml.php")
   {
    $this->e->add("XMLi Request URL doesn't match");
    return UT_FAILED;
   }

   if($object->session)
   {
    $this->e->add("XMLi Request sesion shouldn't be set");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

?>
