<?

 /*****************************************************************************
 * Filename: include/socket.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");

 class Socket
 {
  function Socket($connection=false) // file:///dir/file.sock, udp://192.168.0.1:234 or www.test.com:80 (tcp is default) 
  {
   $this->e=new Error();
   $this->e->l->read("socket");

   $this->ipv4=true;
   $this->ipv6=true;

   $this->blocking=true;
   $this->timeout=NULL; //seconds to wait for select, default no timeout

   $this->readBytes=1024; //number of bytes to read per read call
   $this->readType=PHP_BINARY_READ; //read type: PHP_BINARY_READ or PHP_NORMAL_READ
   $this->readBreak=false; //set to true to stop reading if less then readBytes are received 

   $this->connectionTypes=Array("tcp", "udp", "file");

   $this->socket=false;
   $this->clear();

   if($connection)
    $this->connection($connection);
  }

  function clear()
  {
   if($this->socket)
    $this->close();

   $this->mode=false;

   $this->file=false;
   $this->host=false;
   $this->port=0;

   $this->readBuffer="";

   $this->bytesSend=0;
   $this->bytesReceived=0;

   $this->connectionTime=0;
   $this->startTime=0;

   $this->errno=false;
   $this->error=false;

   return true;
  }

  function error()
  {
   if($this->socket)
    $this->errno=@socket_last_error($this->socket);
   else
    $this->errno=@socket_last_error();

   $this->error=@socket_strerror($this->errno);

   return $this->errno;
  }

  function inet($ipv4, $ipv6)
  {
   if($this->socket)
   {
    $this->e->add("Socket already active, can't set inet mode", "error");
    return false;
   }

   if(!$ipv4&&!$ipv6)
   {
    $this->e->add("At least IPv4 or IPv6 should be allowed", "error");
    return false;
   }

   if($ipv4)
    $this->ipv4=true;
   else
    $this->ipv4=false;

   if($ipv6)
    $this->ipv6=true;
   else
    $this->ipv6=false;

   return true;
  }

  function blocking($blocking)
  {
   if($blocking)
    $this->blocking=true;
   else
    $this->blocking=false;

   if($this->socket)
   {
    if($this->blocking)
    {
     if(!@socket_set_block($this->socket))
     {
      $this->error();
      $this->e->add("Unable to set connection to blocking", "error", "[".$this->errno."] ".$this->error);
      return false;
     }
    }
    else //non blocking
    {
     if(!@socket_set_nonblock($this->socket))
     {
      $this->error();
      $this->e->add("Unable to set connection to non-blocking", "error", "[".$this->errno."] ".$this->error);
      return false;
     }
    }
   }

   return true;
  }

  function setTimeout($timeout)
  {
   if(!is_int($timeout)&&$timeout!==NULL)
   {
    $this->e->add("Timeout should be an integer or NULL", "error");
    return false;
   }

   $this->timeout=$timeout;
   return true;
  }

  function setReadBytes($bytes)
  {
   if(!is_int($bytes))
   {
    $this->e->add("Read bytes should be an integer", "error");
    return false;
   }

   $this->readBytes=$bytes;
   return true;
  }

  function setReadType($type)
  {
   if($type=="binary"||$type==PHP_BINARY_READ)
   {
    $this->readType=PHP_BINARY_READ;
    return true;
   }
   else if($type=="normal"||$type==PHP_NORMAL_READ) //stops at \n or \r
   {
    $this->readType=PHP_NORMAL_READ;
    return true;
   }

   $this->e->add("Unknown read type: \"".$type."\"", "error");
   return false;
  }

  function getOption($key)
  {
   if(!($value=@socket_get_option($this->socket, SOCK_STREAM, $key)))
   {
    $this->error();
    $this->e->add("Unable to get socket option", "error", "[".$this->errno."] ".$this->error);
    return false;
   }

   return $value;
  }

  function setOption($key, $value)
  {
   if(!@socket_set_option($this->socket, SOCK_STREAM, $key, $value))
   {
    $this->error();
    $this->e->add("Unable to set socket option", "error", "[".$this->errno."] ".$this->error);
    return false;
   }

   return true;
  }



  function connection($connection)
  {
   if(!preg_match("!^([a-z]+)://(.*)$!", $connection, $m)) //tcp
    $type="tcp";
   else
   {
    $type=$m[1];
    $connection=$m[2];

    if(!in_array($type, $this->connectionTypes))
    {
     $this->e->add("Unknown connection type: \"".$type."\"", "error");
     return false;
    }
   }

   if($type=="file")
   {
    $port=0;
    $host=$connection;
   }
   else
   {
    if(!preg_match("!^(.*):(\d+)$!", $connection, $m)) //host:port
    {
     $this->e->add("Connection failed: no port number in connection string: \"".$connection."\"", "error");
     return false;
    }

    $host=$m[1];
    $port=$m[2];
   }

   return $this->connect($host, $port, $type);
  }

  function connect($host, $port=false, $type="tcp")
  {
   if($this->socket)
   {
    $this->e->add("Socket already active, refusing to connect", "error");
    return false;
   }

   if(!in_array($type, $this->connectionTypes))
   {
    $this->e->add("Unknown connection type: \"".$type."\"", "error");
    return false;
   }

   $this->clear();
   $this->mode="connection";
   $this->startTime=microtime(true);

   if($type=="file")
   {
    if(!file_exists($host)) 
    {
     $this->e->add("Connection failed: socket: \"".$host."\" doesn't exists", "error");
     return false;
    }

    if(filetype($host)!="socket")
    {
     $this->e->add("Connection failed: file: \"".$host."\" isn't a socket", "error");
     return false;
    }

    $this->file=$host;
    $this->host=false;
    $this->port=0;

    if(($this->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0))===false) 
    {
     $this->error();
     $this->e->add("Connection failed: unable to create ".$type." socket", "error", "[".$this->errno."] ".$this->error);
     return false;
    }

    if(!@socket_connect($this->socket, $this->file))
    {
     $this->error();
     $this->e->add("Connection failed: unable to connect to: \"".$this->file."\"", "error", "[".$this->errno."] ".$this->error);
     $this->close();
     return false;
    }
   }
   else //tcp/udp
   {
    if(preg_match("/^\d+\.\d+\.\d+\.\d+$/", $host)) //ipv4
     $inet="ipv4";
    else if(preg_match("/^[0-9A-F\:]+$/i", $host)) //ipv6
     $inet="ipv6";
    else //host name
     $inet="both";

    if($inet=="ipv4"&&!$this->ipv4)
    {
     $this->e->add("Listen failed: IPv4 is disabled", "error");
     return false;
    }
    
    if($inet=="ipv6"&&!$this->ipv6)
    {
     $this->e->add("Listen failed: IPv6 is disabled", "error");
     return false;
    }
    
    if(!$port)
    {
     $this->e->add("Connection failed: port number not set", "error");
     return false;
    }

    $this->file=false;
    $this->host=$host;
    $this->port=$port;

    if($type=="udp")
     $protocol=SOL_UDP;
    else
     $protocol=SOL_TCP;

    if($this->ipv6&&($inet=="ipv6"||$inet=="both"))
    {
     if(($this->socket=@socket_create(AF_INET6, SOCK_STREAM, $protocol))===false) 
     {
      if(!$this->ipv4) //no retry
      {
       $this->error();
       $this->e->add("Connection failed: unable to create ".$type." socket", "error", "[".$this->errno."] ".$this->error);
       return false;
      }
     }
     else if(!@socket_connect($this->socket, $this->host, $this->port))
     {
      if(!$this->ipv4) //no retry
      {
       $this->error();
       $this->e->add("Connection failed: unable to connect to: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);
       $this->close();
       return false;
      }

      $this->close();
     }
    }

    if($this->ipv4&&($inet=="ipv4"||$inet=="both"))
    {
     if(!$this->socket) //IPv4 retry or first try if ipv6 is disabled
     {
      if(($this->socket=@socket_create(AF_INET, SOCK_STREAM, $protocol))===false) 
      {
       $this->error();
       $this->e->add("Connection failed: unable to create ".$type." socket", "error", "[".$this->errno."] ".$this->error);
       return false;
      }

      if(!@socket_connect($this->socket, $this->host, $this->port))
      {
       $this->error();
       $this->e->add("Connection failed: unable to connect to: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);
       $this->close();
       return false;
      }
     }
    }
   }

   if(!$this->blocking($this->blocking))
   {
    $this->close();
    return false;
   }

   return true;
  }



  function listen($host, $port=false, $type="tcp")
  {
   if($this->socket)
   {
    $this->e->add("Socket already active, refusing to listen", "error");
    return false;
   }

   if(!in_array($type, $this->connectionTypes))
   {
    $this->e->add("Unknown connection type: \"".$type."\"", "error");
    return false;
   }

   $this->clear();
   $this->mode="listen";

   if($type=="file")
   {
    if(file_exists($host)) //done before setting filename, otherwise close will unlink the file
    {
     $this->e->add("Listen failed: socket: \"".$host."\" already exists", "error");
     return false;
    }

    $this->file=$host;
    $this->host=false;
    $this->port=0;

    if(($this->socket=@socket_create(AF_UNIX, SOCK_STREAM, 0))===false) 
    {
     $this->error();
     $this->e->add("Listen failed: unable to create ".$type." socket", "error", "[".$this->errno."] ".$this->error);
     return false;
    }

    if(!@socket_bind($this->socket, $this->file))
    {
     $this->error();
     $this->e->add("Listen failed: unable to bind on: \"".$this->file."\"", "error", "[".$this->errno."] ".$this->error);
     $this->close();
     return false;
    }
   }
   else //tcp/udp
   {
    if(preg_match("/^\d+\.\d+\.\d+\.\d+$/", $host)) //ipv4
     $inet="ipv4";
    else if(preg_match("/^[0-9A-F\:]+$/i", $host)) //ipv6
     $inet="ipv6";
    else //host name
     $inet="both";

    if($inet=="ipv4"&&!$this->ipv4)
    {
     $this->e->add("Listen failed: IPv4 is disabled", "error");
     return false;
    }
    
    if($inet=="ipv6"&&!$this->ipv6)
    {
     $this->e->add("Listen failed: IPv6 is disabled", "error");
     return false;
    }
    
    if(!$port)
    {
     $this->e->add("Listen failed: port number not set", "error");
     return false;
    }

    $this->file=false;
    $this->host=$host;
    $this->port=$port;

    if($type=="udp")
     $protocol=SOL_UDP;
    else
     $protocol=SOL_TCP;

    if($this->ipv6&&($inet=="ipv6"||$inet=="both"))
    {
     if(($this->socket=@socket_create(AF_INET6, SOCK_STREAM, $protocol))===false) 
     {
      if(!$this->ipv4) //no retry
      {
       $this->error();
       $this->e->add("Listen failed: unable to create ".$type." socket", "error", "[".$this->errno."] ".$this->error);
       return false;
      }
     }
     else if(!@socket_bind($this->socket, $this->host, $this->port))
     {
      if(!$this->ipv4) //no retry
      {
       $this->error();
       $this->e->add("Listen failed: unable to bind on: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);
       $this->close();
       return false;
      }

      $this->close();
     }
    }

    if($this->ipv4&&($inet=="ipv4"||$inet=="both"))
    {
     if(!$this->socket) //IPv4 retry or first try if ipv6 is disabled
     {
      if(($this->socket=@socket_create(AF_INET, SOCK_STREAM, $protocol))===false) 
      {
       $this->error();
       $this->e->add("Listen failed: unable to create ".$type." socket", "error", "[".$this->errno."] ".$this->error);
       return false;
      }

      if(!@socket_bind($this->socket, $this->host, $this->port))
      {
       $this->error();
       $this->e->add("Listen failed: unable to bind on: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);
       $this->close();
       return false;
      }
     }
    }
   }

   if(!@socket_listen($this->socket))
   {
    $this->error();
    $this->e->add("Listen failed: unable to listen on: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);
    $this->close();
    return false;
   }

   if(!$this->blocking($this->blocking))
   {
    $this->close();
    return false;
   }

   return true;
  }

  function accept()
  {
   if(!$this->socket)
   {
    $this->e->add("Socket not active, refusing to accept connections", "error");
    return false;
   }

   if($this->mode!="listen")
   {
    $this->e->add("Socket not in listen mode, refusing to accept connections", "error");
    return false;
   }

   if(($socket=@socket_accept($this->socket))===false) 
   {
    if($this->error()) //signal interrupt can trigger a failure, errno is 0 returned 
     $this->e->add("Unable to accept", "error", "[".$this->errno."] ".$this->error);

    return false;
   }

   $s=new Socket();
   $s->socket=&$socket;
   $s->mode="connection";

   $s->startTime=microtime(true);
   $s->connectionTime=0;

   $s->ipv4=$this->ipv4;
   $s->ipv6=$this->ipv6;

   $s->host=$this->host;
   $s->port=$this->port;
   $s->file=$this->file;

   $s->blocking($this->blocking);
   $s->setTimeout($this->timeout);

   $s->readBytes=$this->readBytes;
   $s->readType=$this->readType;
   $s->readBreak=$this->readBreak; 

   $s->readBuffer="";

   $s->bytesReceived=0;
   $s->bytesSend=0;

   return $s;
  }

  function read()
  {
   if(!$this->socket)
   {
    $this->e->add("Socket not active, refusing to read", "error");
    return false;
   }

   $read=Array($this->socket);
   $write=NULL;
   $exception=NULL;

   if(($sockets=@socket_select($read, $write, $exception, $this->timeout))===false)
   {
    if($this->error()) //signal interrupt can trigger a failure, errno is 0 returned 
     $this->e->add("Connection read failed: unable to select on socket", "error", "[".$this->errno."] ".$this->error);

    return false;
   }

   if($sockets===0) //Timeout/non blocking: nothing to read
    return "";

   $received="";

   //start reading
   while($data=@socket_read($this->socket, $this->readBytes, $this->readType)) //false fail, "" done reading
   {
    $received.=$data;
    $bytes=strlen($data);
    $this->bytesReceived+=$bytes;

    if($this->readBreak&&$bytes<$this->readBytes) //less then read bytes, done reading
     break;
   }

   if($received)
    $this->readBuffer.=$received;

   if($data===false&&$this->error()) //signal interrupt can trigger a failure, errno is 0 returned, in this case the received buffer will still be returned 
   {
    $this->e->add("Connection read failed from: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);
    return false; 
   }

   return $received;
  }

  function write($data)
  {
   if(!$this->socket)
   {
    $this->e->add("Socket not active, refusing to write", "error");
    return false;
   }

   $length=strlen($data);
   if(($bytes=@socket_write($this->socket, $data, $length))===false)
   {
    if($this->error()) //signal interrupt can trigger a failure, errno is 0 returned 
     $this->e->add("Connection write failed to: \"".($this->host?$this->host.":".$this->port:$this->file)."\"", "error", "[".$this->errno."] ".$this->error);

    return false;
   }

   $this->bytesSend+=$bytes;
   return true;
  }



  function close($message=false)
  {
   if(!$this->socket)
   {
    $this->e->add("Socket not active, refusing to close", "error");
    return false;
   }

   if($message)
    $this->write($message);

   socket_close($this->socket);
   $this->socket=false;

   if($this->mode=="connection")
    $this->connectionTime=microtime(true)-$this->startTime;
   else if($this->mode=="listen")
   {
    if($this->file&&file_exists($this->file)) //remove listening file socket
    {
     if(!@unlink($this->file))
     {
      $this->e->add("Unable to remove socket file: \"".$this->file."\"", "error");
      return false;
     }
    }
   }

   return true;
  }



  function __destruct()
  {
   if($this->socket)
    $this->close();
  }

 }

?>
