<?

 /*****************************************************************************
 * Filename: include/storage/session.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."error.php");

 class sessionStorage extends Storage 
 {
  function sessionStorage(&$d)
  {
   global $_jses;

   parent::Storage($d, "session");
   $this->session="sessionStorage";

   if(!($db=$this->getConfig()))
    return false;

   if(isset($db["var"])) //array style
    $var=$db["var"];
   else if(isset($db["host"])) //uri style
    $var=$db["host"];
   else
   {
    $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: \"var\" parameter not set", "system");
    return false;
   }

   if(isset($db["path"])&&$db["path"]=="/global") //store in global _SESSION variable
   {
    if(!isset($_SESSION[$this->session]))
     $_SESSION[$this->session]=Array();

    if(!isset($_SESSION[$this->session][$var]))
     $_SESSION[$this->session][$var]=Array();

    if(!isset($_SESSION[$this->session][$var][$this->data->module]))
     $_SESSION[$this->session][$var][$this->data->module]=Array();

    $this->storage=&$_SESSION[$this->session][$var][$this->data->module];
   }
   else //store in _jses
   {
    if(!isset($_jses[$this->session]))
     $_jses[$this->session]=Array();

    if(!isset($_jses[$this->session][$var]))
     $_jses[$this->session][$var]=Array();

    if(!isset($_jses[$this->session][$var][$this->data->module]))
     $_jses[$this->session][$var][$this->data->module]=Array();

    $this->storage=&$_jses[$this->session][$var][$this->data->module];
   }

   $this->clear();
  }

  function table()
  {
   return true;
  }

  function checkRights($action)
  {
   return true;
  }

  function insertID()
  {
   $d=&$this->data; //D for short

   if(!($id=$d->getID()))
   {
    $this->e->add("Refusing to get insertID, no ID field", "sql");
    return false;
   }
  
   if(($d->dataformat=="sql"||$d->extends)) //allow force id insertion
   {
    if($insertID=$d->sqlValue($id))
    {
     if(!$this->unique($id))
     {
      $this->e->add("Refusing to get insertID, ID \"".$insertID."\" already exists", "sql");
      return false;
     }

     return $insertID;
    }
   }

   $insertID=0;

   for($c=0;$c<sizeof($this->storage);$c++)
    if(isset($this->storage[$c][$id])&&$this->storage[$c][$id]>$insertID)
     $insertID=$this->storage[$c][$id];

   $insertID++;

   return $insertID;
  } 
 
  function get($count=false)
  {
   $d=&$this->data; //D for short

   if($d->check("valuetype", "set", "aggregation_field"))
   {
    $this->e->add($this->type." storage doesn't support: \"aggregation\"", "system");
    return false;
   }

   if(!$this->_filterDataset($this->storage, $count))
    return false;

   if(!$count)
   {
    if(!$this->_sort())
     return false;

    if(!$this->_filterShowTotal())
     return false;
   }
   else if(!$this->_filterCount())
    return false;

   return true;
  }

  function add()
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("add"))) //nothing to add... ?
    return true;

   if(!($insertID=$this->insertID()))
    return false;

   $id=$d->getID();
   $d->setValue($id, $insertID, $d->vc);

   $c=sizeof($this->storage);
   $this->storage[$c]=Array();
   $this->storage[$c][$id]=$insertID;

   foreach($values as $k => $v)
    $this->storage[$c][$k]=$v; 

   return true;
  } 

  function edit()
  {
   $d=&$this->data; //D for short

   if(!($values=$this->_getValues("edit"))) //nothing to update
    return true;

   $id=$d->getID();
   $idValue=$d->sqlValue($id);

   $l=sizeof($this->storage);
   for($c=0;$c<$l;$c++)
   {
    if($this->storage[$c][$id]==$idValue)
    {
     foreach($values as $k => $v)
      $this->storage[$c][$k]=$v; 

     break;
    }
   }

   return true;
  } 

  function delete()
  {
   $d=&$this->data; //D for short

   $id=$d->getID();
   $idValue=$d->sqlValue($id);

   $l=sizeof($this->storage);
   for($c=0;$c<$l;$c++)
   {
    if($this->storage[$c][$id]==$idValue)
    {
     for($x=$c+1;$x<$l;$x++)
      $this->storage[$x-1]=$this->storage[$x];

     unset($this->storage[$l-1]);

     break;
    }
   }

   return true;
  } 

 }
?>
