<?

 /*****************************************************************************
 * Filename: include/storage/xmli.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."xmli.php");
 require_once($_path["include"]."fs.php");

 class xmliStorage extends Storage 
 {
  function xmliStorage(&$d)
  {
   global $_jses;

   parent::Storage($d, "xmli");
   $logging=$this->e->logging;
   $this->e->logging="none"; //no logging, done mostly server side

   $this->xmli=false;

   if(!isset($_jses["xmliStorage"]))
    $_jses["xmliStorage"]=Array();

   if(!isset($_jses["xmliStorage"][$_jses["project"]]))
   {
    $_jses["xmliStorage"][$_jses["project"]]=Array();
    $_jses["xmliStorage"][$_jses["project"]]["session"]=false;
    $_jses["xmliStorage"][$_jses["project"]]["project"]=false;
    $_jses["xmliStorage"][$_jses["project"]]["modules"]=Array();
   }

   if(!($db=$this->getConfig()))
    return false;

   if(!isset($db["host"]))
   {      
    $this->e->add("Database configuration: \"".$this->data->db."\" incorrect: \"host\" parameter not set", "system");
    return false;  
   }              

   $this->xmli=new XMLiClient(($db["channel"]=="secure"?"https":"http")."://".$db["host"].$db["path"], $_jses["xmliStorage"][$_jses["project"]]["session"]);
   $this->xmli->e->logging="none";

   //connection test
   if(!$_jses["xmliStorage"][$_jses["project"]]["session"]) //session not set: first run
   {
    if(!$this->xmli->request()) //do empty request 
    {
     if($this->xmli->e->errors()>1||$this->xmli->e->getError(0)!="No XML tags found") //should only have 1 error: No XML tags found
     {
      for($c=0;$c<$this->xmli->e->errors();$c++) //save errors as system errors
       $this->e->add($this->xmli->e->getError($c), "system", $this->xmli->e->getExtra($c));
 
      return false;
     }
    }
   }

   //store session on successful connection test
   $_jses["xmliStorage"][$_jses["project"]]["session"]=&$this->xmli->session; //referenced: ensures it's stored on change


   //switch project
   if(isset($db["project"])&&!$_jses["xmliStorage"][$_jses["project"]]["project"]) //project specified and project not yet set 
   {
    if(!$this->xmli->setProject($db["project"])) //switch project
    {
     for($c=0;$c<$this->xmli->e->errors();$c++) //save errors as system errors
      $this->e->add($this->xmli->e->getError($c), "system", $this->xmli->e->getExtra($c));

     return false;
    }

    $_jses["xmliStorage"][$_jses["project"]]["project"]=true;
   }

   $this->clear();
  }


  function insertID($forcedID=false)
  {
  }


  function checkRights($action)
  {
   global $_config;

   if($_config["usernr"]==$_config["rootnr"])
    return true;

   $this->e->add("No ".$action." rights"."!", "hack");

   return false; 
  }


  function table()
  {
   global $_jses;

   //module already check this session?
   if(isset($_jses["xmliStorage"][$_jses["project"]]["modules"][$this->data->module])&&$_jses["xmliStorage"][$_jses["project"]]["modules"][$this->data->module])
    return true;

   //get module
   if(!$this->xmli->getModule(($this->data->initializingModule?$this->data->initializingModule.":":"").$this->data->module)) //added initializing module as prefix: initModule:module
   {
    $this->e->add($this->xmli->e);
    return false;
   }

   $_jses["xmliStorage"][$_jses["project"]]["modules"][$this->data->module]=true; //store module ok
   return true;
  }


  function info()
  {
   $this->clear();
   $d=$this->data->copy(); //copy data object

   if(!$this->xmli->data($d)) //handle data request: result in $d
   {
    $this->e->add($this->xmli->e);
    return false;
   }

   $fields=Array();
   foreach($d->check("use", "info") as $field)
    $fields[]=$field;

   $l=sizeof($fields);

   for($c=0;$c<$d->vmax;$c++)
   {
    $this->results[$c]=Array();

    for($x=0;$x<$l;$x++)
     $this->results[$c][$fields[$x]]=$d->field[$fields[$x]]["value"][$c];

    if(isset($this->results[$c]["info_type"]))
     $this->results[$c]["info_type"]="JAS XMLI ".$this->xmli->xmlJasVersion;
   }

   return true;
  }


  function treeUp($count=false){return $this->get();}
  function treeDown($treeDepth=0, $count=false){return $this->get();}
  function get($count=false)
  {
   $this->clear();
   $d=$this->data->copy(); //copy data object

   if($count) //don't get results, only show_count
   {
    $d->setValue("show_total", 1); 
    $d->setShowFields("nr");
   }

   if($d->action=="tree") //tree up/down
    $d->field["treedepth"]["use"]=""; //don't get treedepth field, will be calculated locally if needed
   else if($d->action=="user"&&$d->extra=="login") //user login handled in login function
   {
    //get actual login information
    $d->setAction("view");
    $d->setExtra("all");
   }
   else if($d->action=="add"||$d->action=="copy"||$d->action=="edit"||$d->action=="delete") //manipulation actions
   {
    //get item
    $d->setAction("view");
    $d->setExtra("item");
   }

   if(!$this->xmli->data($d, $count)) //handle data request: result in $d
   {
    $this->e->add($this->xmli->e);
    return false;
   }

   if($count)
    $this->results[0]["count"]=$d->getValue("show_count");
   else //get data
   {
    $fields=Array();
    foreach($d->check("use", "sql") as $field)
     if($field!="r")
      if($d->check("use", "show", $field)||$d->check("properties", "identifier", $field))
       $fields[]=$field;

    $l=sizeof($fields);

    for($c=0;$c<$d->vmax;$c++)
    {
     $this->results[$c]=Array();

     for($x=0;$x<$l;$x++)
      $this->results[$c][$fields[$x]]=$d->field[$fields[$x]]["value"][$c];
    }
   }

   return true;
  }


  function add(){return $this->edit();}
  function delete(){return $this->edit();}
  function edit()
  {
   //HACK: don't add userRights+groupRights, done server side
   if( 
    $this->data->module=="userRights"||
    $this->data->module=="groupRights"
   )
    return true;

   $this->clear();
   $d=$this->data->copy(); //copy data object

   if($d->action=="copy")
    $d->setAction("add");

   if(!$this->xmli->data($d)) //handle data request: result in $d
   {
    $this->e->add($this->xmli->e);
    return false;
   }

   $this->e->add($this->xmli->e);
   return true;
  }


  function logout(){return $this->login();}
  function login()
  {
   $this->clear();
   $d=$this->data->copy(); //copy data object

   if(!$this->xmli->data($d)) //handle data request: result in $d
   {
    $this->e->add($this->xmli->e);
    return false;
   }

   return true;
  }


 }
?>
