<?

 /*****************************************************************************
 * Filename: include/template.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 require_once($_path["include"]."xml.php");
 require_once($_path["include"]."mutilate.php");

 class Template extends XML
 {
  function Template($input, $onlyCommentsAndMarkers=false, $exactPath=true)
  {
   XML::XML($onlyCommentsAndMarkers, $exactPath);

   //allowed structure, see include/plugins/lister/templates/markers.html for more information
   $this->structure=Array(
    "_template" => Array(
     "_module_name" => true,
     "_module_title" => true,

     "_title_field_name" => true,
     "_title_field_title" => true,
     "_title_field_value" => true,
     "_title_field_type" => true,

     "_fields" => Array(
      "_field_name" => true,
      "_field_title" => true,
      "_field_value" => true,
      "_field_type" => true,
     ),
 /*
  * field markers, allowed everywhere
     "$field" => true,
     "$field_title" => true,
     "$field_type" => true,
 */

     "_row" => Array(
      "_row_oddeven" => true,
     ),
     "_current_rows" => true,
     "_total_rows" => true,

     "_if" => Array(
      "_else" => true,
     ),
     "_if_rights" => Array(
      "_else" => true,
     ),
    )
   );

   if(file_exists($input))  // if input is a file: parse the file, otherwise parse input
    $this->parseFile($input);
   else
    $this->parse($input);
  }

  function clear()
  {
   parent::clear();

   $this->d=false;
   $this->level=0;
   $this->tree=false;

   $this->marker=Array();
   $this->markers=0;

   $this->ifStatements=Array();
  }

  function output()
  {
   $r="";

   if($this->e->any())
    $r.=$this->e->show();

   $r.=XML::output();
   $r=str_replace("##", "#", $r);

   return $r;
  }


  /*
   *
   * Analyze template and validate against data object
   */

  function analyze(&$d)
  {
   $this->marker=Array();
   $this->markers=0;

   $this->d=&$d;
   $failure=false;

   if($this->d->e->any("fatal"))
   {
    $this->e->add($this->d->e);
    return false;
   }

   //add global _template marker
   if($this->getName(0)!="_template")
   {
    $this->xml="<!_template>".$this->xml."<!/_template>";
    $this->reparse();
   }

   //detect tree mode, data tree action and sub _row in template
   $this->reset();
   if($this->d->action=="tree")
    if(($key=$this->search("_row"))!==false)
     if(strstr($this->getInner($key), "<!_row"))
      $this->tree=true;

   $this->reset();
   unset($key);

   //get general+if+field markers
   $ifCounter=0;
   $path=Array();
   $fields=Array();
   $foreachCounter=0;
   $foreach=Array();
   $markers=Array();
   for($c=0;$c<$this->tags;$c++)
   {
    if($this->tag[$c]["type"]!="marker"&&$this->tag[$c]["type"][0]!="!") //only markers or comments
     continue;

    //close marker
    if($this->tag[$c]["type"]=="!close")
    {
     if($this->tag[$c]["name"]==$path[sizeof($path)-1]) //same as deepest path
      array_pop($path); //remove path entry

     if($this->tag[$c]["name"]=="_row") //row entry
      array_pop($foreach); //remove foreach entry

     continue;
    }


    //create base marker
    $marker=Array(
     "index" => $c,
     "path" => $path,
     "name" => $this->tag[$c]["name"],
    );


    if($this->tag[$c]["name"][0]=="_") //general marker
    {



     if($this->tag[$c]["name"]=="_row") //row
     {
      if(!($attributes=$this->getAttributes($c)))
      {
       $marker["type"]="row";
       $foreach[]=false; //store fake foreach entry to ensure correct close tag is used (above)
      }
      else //field argument (foreach)
      {
       $marker["type"]="foreach";

       if($this->tag[$c]["type"]!="!block"&&$this->tag[$c]["type"]!="!--block")
        $marker["error"]="not block";
       else
       {
        $attributes=implode(" ", $attributes);

        //get the field names and entries (field[0])
        if(!preg_match_all("/([^\.\[]+)(\[(\d+)\])?/", $attributes, $m))
        {
         $marker["error"]="incorrect field";
         continue;
        }

        //store fields
        $marker["field"]=$m[1];

        //store foreach id (for renaming)
        $marker["id"]=$foreachCounter++;

        //store entries
        $marker["entry"]=$m[3];

        //replace last entry is foreach
        array_pop($marker["entry"]);
        $marker["entry"][]="foreach";

        //store field names, used to validate fields within foreach marker
        $foreach[]=Array(
         "id"=> $marker["id"],
         "fields"=> $marker["field"],
        );
       }
      }
     }



     else if($this->tag[$c]["name"]=="_if") //if statement
     {
      $marker["type"]="if";

      if($this->tag[$c]["type"]!="!block"&&$this->tag[$c]["type"]!="!--block")
       $marker["error"]="not block";
      else if(!($attributes=$this->getAttributes($c)))
       $marker["error"]="no attributes";
      else
      {
       $attributes=implode(" ", $attributes);
       if(!preg_match("/(!|not)?\s*([a-z0-9\_\.\[\]]+)\s*([\!\=\>\<]+)?\s*(.+)?/i", $attributes, $ifAttr)) //match all $field, $field=$value, $field>=$value, etc
        $marker["error"]="incorrect attributes";
       else if(!preg_match_all("/([^\.\[]+)(\[(\d+)\])?/", $ifAttr[2], $m)) //get the field names and entries (field[0])
        $marker["error"]="incorrect fields";
       else
       {
        //store fields
        $marker["field"]=$m[1];

        //store entries
        $marker["entry"]=$m[3];
        $marker["foreach"]=Array();

        //set foreach entries
        foreach($foreach as $fields)
        {
         if(!$fields)
          continue;

         $l=sizeof($fields["fields"]);
         for($x=0;$x<$l;$x++)
         {
          if(!isset($marker["field"][$x])||$marker["field"][$x]!=$fields["fields"][$x]) //field not relevant for foreach
           break;

          if($x==$l-1&&!$marker["entry"][$x]) //last foreach entry and no static entry set
          {
           $marker["entry"][$x]="foreach"; //set foreach entry
           $marker["foreach"][$x]=$fields["id"]; //set foreach id
          }
         }
        }

        //store operator, value and not
        $marker["not"]=($ifAttr[1]?true:false);

        if(isset($ifAttr[3])) //operator set
         $marker["operator"]=$ifAttr[3]; //set value
        else //empty string
         $marker["operator"]=""; //boolean operator

        if(isset($ifAttr[4])) //value set
         $marker["value"]=trim($ifAttr[4], "\""); //remove quotes
        else //empty string
         $marker["value"]=""; //no value
       }
      }
     }



     else if($this->tag[$c]["name"]=="_if_rights") //if rights statement
     {
      $marker["type"]="if_rights";

      if($this->tag[$c]["type"]!="!block"&&$this->tag[$c]["type"]!="!--block")
       $marker["error"]="not block";
      else if(!($attributes=$this->getAttributes($c)))
       $marker["error"]="no attributes";
      else
      {
       $attributes=implode(" ", $attributes);
       if(!preg_match("/(!|not)?\s*([a-z]+)/i", $attributes, $ifAttr)) //match rights type and ! type
        $marker["error"]="incorrect attributes";
       else //valid if rights statement
       {
        //store rights type and not
        $marker["rights"]=$ifAttr[2];
        $marker["not"]=($ifAttr[1]?true:false);
       }
      }
     }



     else //general marker
      $marker["type"]="general";
    }



    else //field
    {
     $marker["type"]="field";

     //get the field names and entries (field[0])
     if(!preg_match_all("/([^\.\[]+)(\[(\d+)\])?/", $this->tag[$c]["name"], $m))
      $marker["error"]="incorrect field";
     else
     {
      //store fields
      $marker["field"]=$m[1];

      //store entries
      $marker["entry"]=$m[3];
      $marker["foreach"]=Array();

      //set foreach entries
      foreach($foreach as $fields)
      {
       if(!$fields)
        continue;

       $l=sizeof($fields["fields"]);
       for($x=0;$x<$l;$x++)
       {
        if(!isset($marker["field"][$x])||$marker["field"][$x]!=$fields["fields"][$x]) //field not relevant for foreach
         break;

        if($x==$l-1&&!$marker["entry"][$x]) //last foreach entry and no static entry set
        {
         $marker["entry"][$x]="foreach"; //set foreach entry
         $marker["foreach"][$x]=$fields["id"]; //set foreach id
        }
       }
      }

      //get mutilations
      if(($attributes=$this->getAttributes($c))&&$attributes[0]!="/") //get attributes unless it's short tag /
      {
       $attributes=implode(" ", $attributes);

       //prepare mutilations
       if(!preg_match_all("/(([a-z0-9\_]+)(\((.*)\))? )/iU", $attributes." ", $m)) //match all $function or $function(arg1,arg2);, args may contain ()
       {
        $this->e->add("Template error: Mutilations mismatch:  \"".$attributes."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $marker["error"]="incorrect mutilations";
       }
       else
       {
        //rename name to unique name
        $marker["name"].="_".$c;
        $this->rename($c, $marker["name"]);

        //add mutilations
        $marker["mutilations"]=Array();
        $l3=sizeof($m[0]); //number of matches
        for($z=0;$z<$l3;$z++) //loop through all mutilations functions
        {
         $function=$m[2][$z];

         if($m[4][$z]!=="") //arguments
         {
          $arguments=str_replace("\\,", "\b", $m[4][$z]); //rename escaped commas (\,) to \b
          $arguments=explode(",", $arguments); //explode on ,

          //replace \b back to ,
          foreach($arguments as $key => $argument)
           $arguments[$key]=str_replace("\b", ",", $argument);
         }
         else
          $arguments=Array();

         //replace function: check if match has delimiters, allowed: / ! # ~ +
         if($function=="replace"&&!preg_match("/^[\/!#~\+].+[\/!#~\+][a-z]*$/", $arguments[0]))
          $arguments[0]="/".preg_quote($arguments[0], "/")."/"; //add / delimiters

         $marker["mutilations"][]=Array(
          "function" => $function,
          "arguments" => $arguments,
         );
        }
       }
      }
     }
    }

    $markers[]=$marker;

    if($this->tag[$c]["type"]=="!block")
     $path[]=$this->tag[$c]["name"];
   }


   //validate general+if+field markers
   $validMultiReferences=Array("0:*", "1:*", "*:*");
   $validMultiFields=Array("localfiles", "multilist", "modules", "fields");
   $allowedRights=Array("add", "copy", "edit", "move", "delete", "owner");
   $allowedOperators=Array("=", "!=", ">=", "<=", ">", "<");
   foreach($markers as $marker)
   {


    //global type checks
    if($marker["type"]=="general"||$marker["type"]=="row"||$marker["type"]=="foreach"||$marker["type"]=="if"||$marker["type"]=="if_rights")
    {
     //validate path
     $tmp=$this->structure;
     foreach($marker["path"] as $path)
     {
      if($path[0]!="_") //only general markers
       continue;

      if(!isset($tmp[$path]))
      {
       if(isset($this->structure["_template"][$path])) //general marker exists in upper level, allowed as well
        $tmp=$this->structure["_template"]; //restart from base level
       else
       {
        $this->e->add("Template error: Illegal general marker path: \"".$path."\" for marker \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $tmp=false;
        break;
       }
      }

      if(isset($tmp[$marker["name"]]))
       break;

      $tmp=$tmp[$path];
     }

     if(!$tmp)
      continue;

     //valid path structure, validate marker
     if(!isset($tmp[$marker["name"]]))
     {
      $this->e->add("Template error: Illegal general marker: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
      $failure=true;
      continue;
     }
    }



/*    //custom type checks
    if($marker["type"]=="general")
    {
     //no custom checks
    }



    else if($marker["type"]=="row")
    {
     //no custom checks
    }



    else*/ if($marker["type"]=="foreach")
    {
     //log errors found in initial parsing
     if(isset($marker["error"]))
     {
      switch($marker["error"])
      {
       case "not block":
       {
        $this->e->add("Template error: Only block _row tag are allowed for row with field argument: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
       case "incorrect field":
       {
        $this->e->add("Template error: Incorrect field name for _row tag: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
      }

      $failure=true;
      continue;
     }

     //validate fields
     $valid=true;
     $dtmp=$this->d;
     $l=sizeof($marker["field"]);
     for($x=0;$x<$l;$x++)
     {
      //validate entries, only allowed for validMultiReferences and validMultiFields
      if($marker["entry"][$x]!=="") //field[entry] set
      {
       if(isset($dtmp->reference[$marker["field"][$x]])) //reference
       {
        if(!in_array($dtmp->reference[$marker["field"][$x]]["type"], $validMultiReferences))
        {
         $this->e->add("Template error: Field entry not supported for reference type: \"".$dtmp->reference[$marker["field"][$x]]["type"]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
         $failure=true;
         $valid=false;
        }
       }
       else if(isset($dtmp->field[$marker["field"][$x]])) //field
       {
        if(!in_array($dtmp->field[$marker["field"][$x]]["type"], $validMultiFields))
        {
         $this->e->add("Template error: Field entry not supported for field type: \"".$dtmp->field[$marker["field"][$x]]["type"]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
         $failure=true;
         $valid=false;
        }
       }
       //else: unknown field, checked below
      }

      if($x==$l-1) //last field
      {
       if(isset($dtmp->field[$marker["field"][$x]])) //field exists
       {
        $field=$marker["field"][$x];
        $marker["valuetype"]=$dtmp->field[$field]["type"];
       }
       else //nothing valid
       {
        $this->e->add("Template error: Unknown field \"".$marker["field"][$x]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $valid=false;
       }
      }
      else
      {
       if(!isset($dtmp->reference[$marker["field"][$x]]))
       {
        $this->e->add("Template error: Unknown reference \"".$marker["field"][$x]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $valid=false;
        break;
       }

       if($dtmp->reference[$marker["field"][$x]]["type"]=="*:*") //skip *:* module
       {
        //get next reference level
        $dtmp=&$dtmp->getReferenceData($marker["field"][$x]);

        $tmp=$dtmp->check("use", "to"); //get to field
        $dtmp=&$dtmp->getReferenceData($tmp[0]); //get to module
        unset($tmp);
       }
       else //non *:* references
       {
        //get next reference level
        $dtmp=&$dtmp->getReferenceData($marker["field"][$x]);
       }
      }
     }

     //invalid module field
     if(!$valid)
      continue;

     //increase refer level if necessary
     if($l>$this->level+1)
      $this->level=$l-1;
    }



    else if($marker["type"]=="if")
    {
     //log errors found in initial parsing
     if(isset($marker["error"]))
     {
      switch($marker["error"])
      {
       case "not block":
       {
        $this->e->add("Template error: Only block _if tag are allowed for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
       case "no attributes":
       {
        $this->e->add("Template error: No attributes for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
       case "incorrect attributes":
       {
        $this->e->add("Template error: Incorrect attributes for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
      }

      $failure=true;
      continue;
     }

     //validate operator
     if($marker["operator"]&&!in_array($marker["operator"], $allowedOperators))
     {
      $this->e->add("Template error: Unknown _if operator: \"".$marker["operator"]."\" for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
      $failure=true;
      continue;
     }

     //validate fields
     $valid=true;
     $dtmp=$this->d;
     $l=sizeof($marker["field"]);
     for($x=0;$x<$l;$x++)
     {
      //validate entries, only allowed for validMultiReferences and validMultiFields
      if($marker["entry"][$x]!=="") //field[entry] set
      {
       if(isset($dtmp->reference[$marker["field"][$x]])) //reference
       {
        if(!in_array($dtmp->reference[$marker["field"][$x]]["type"], $validMultiReferences))
        {
         $this->e->add("Template error: Field entry not supported for reference type: \"".$dtmp->reference[$marker["field"][$x]]["type"]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
         $failure=true;
         $valid=false;
        }
       }
       else if(isset($dtmp->field[$marker["field"][$x]])) //field
       {
        if(!in_array($dtmp->field[$marker["field"][$x]]["type"], $validMultiFields))
        {
         $this->e->add("Template error: Field entry not supported for field type: \"".$dtmp->field[$marker["field"][$x]]["type"]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
         $failure=true;
         $valid=false;
        }
       }
       //else: unknown field, checked below
      }

      if($x==$l-1) //last field
      {
       if(isset($dtmp->field[$marker["field"][$x]])) //field exists
       {
        $field=$marker["field"][$x];
        $marker["valuetype"]=$dtmp->field[$field]["type"];

        //type cast value to field type, operator based uses strict checking
        switch($marker["valuetype"])
        {
         case "nr":
         {
          $marker["value"]=(int)$marker["value"];
          break;
         }
         case "float":
         case "price":
         {
          $marker["value"]=(float)$marker["value"];
          $marker["decimalseparator"]=$dtmp->field[$field]["format"][$dtmp->dataformat]["decimalseparator"];
          $marker["thousandsseparator"]=$dtmp->field[$field]["format"][$dtmp->dataformat]["thousandsseparator"];
          break;
         }
         case "boolean":
         {
          if($marker["value"]=="true")
           $marker["value"]=true;
          else if($marker["value"]=="false")
           $marker["value"]=false;
          else
           $marker["value"]=(boolean)$marker["value"];

          break;
         }
         default:
          break;
        }
       }
       else //nothing valid
       {
        $this->e->add("Template error: Unknown field \"".$marker["field"][$x]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $valid=false;
       }
      }
      else
      {
       if(!isset($dtmp->reference[$marker["field"][$x]]))
       {
        $this->e->add("Template error: Unknown reference \"".$marker["field"][$x]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $valid=false;
        break;
       }

       if($dtmp->reference[$marker["field"][$x]]["type"]=="*:*") //skip *:* module
       {
        //get next reference level
        $dtmp=&$dtmp->getReferenceData($marker["field"][$x]);

        $tmp=$dtmp->check("use", "to"); //get to field
        $dtmp=&$dtmp->getReferenceData($tmp[0]); //get to module
        unset($tmp);
       }
       else //non *:* references
       {
        //get next reference level
        $dtmp=&$dtmp->getReferenceData($marker["field"][$x]);
       }
      }
     }

     //invalid module field
     if(!$valid)
      continue;

     //increase refer level if necessary
     if($l>$this->level+1)
      $this->level=$l-1;
    }



    else if($marker["type"]=="if_rights")
    {
     //log errors found in initial parsing
     if(isset($marker["error"]))
     {
      switch($marker["error"])
      {
       case "not block":
       {
        $this->e->add("Template error: Only block _if_rights tag are allowed for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
       case "no attributes":
       {
        $this->e->add("Template error: No attributes for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
       case "incorrect attributes":
       {
        $this->e->add("Template error: Incorrect attributes for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
      }

      $failure=true;
      continue;
     }

     //validate rights
     if(!in_array($marker["rights"], $allowedRights))
     {
      $this->e->add("Template error: Unknown _if_rights type: \"".$marker["rights"]."\" for if statement: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
      $failure=true;
      continue;
     }
    }



    else if($marker["type"]=="field")
    {
     //log errors found in initial parsing
     if(isset($marker["error"]))
     {
      switch($marker["error"])
      {
       case "incorrect field":
       {
        $this->e->add("Template error: Incorrect field name for marker: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        break;
       }
      }

      $failure=true;
      continue;
     }


     $valid=true;
     $dtmp=$this->d;

     $l=sizeof($marker["field"]);
     for($x=0;$x<$l;$x++)
     {
      //validate entries, only allowed for validMultiReferences and validMultiFields
      if($marker["entry"][$x]!=="") //field[entry] set
      {
       if(isset($dtmp->reference[$marker["field"][$x]])) //reference
       {
        if(!in_array($dtmp->reference[$marker["field"][$x]]["type"], $validMultiReferences))
        {
         $this->e->add("Template error: Field entry not supported for reference type: \"".$dtmp->reference[$marker["field"][$x]]["type"]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
         $failure=true;
         $valid=false;
        }
       }
       else if(isset($dtmp->field[$marker["field"][$x]])) //field
       {
        if(!in_array($dtmp->field[$marker["field"][$x]]["type"], $validMultiFields))
        {
         $this->e->add("Template error: Field entry not supported for field type: \"".$dtmp->field[$marker["field"][$x]]["type"]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
         $failure=true;
         $valid=false;
        }
       }
       //else: unknown field, checked below
      }

      if($x==$l-1) //last field
      {
       if(isset($dtmp->field[$marker["field"][$x]])) //field exists
       {
        $field=$marker["field"][$x];

        //validate mutilations
        if(isset($marker["mutilations"]))
        {
         foreach($marker["mutilations"] as $key => $mutilation)
         {
          if(($mutilation["arguments"]&&$dtmp->addMutilation($field, $mutilation["function"], $mutilation["arguments"]))||(!$mutilation["arguments"]&&$dtmp->addMutilation($field, $mutilation["function"]))) //check if mutilation is allowed for this field
          {
           $dtmp->removeMutilation($field, $mutilation["function"]); //remove mutilation, only used for testing

           //add default mutilation values
           switch($mutilation["function"])
           {
            case "dateConvert":
            {
             $type=$dtmp->field[$field]["type"];

             //add current date/time format as first argument
             $marker["mutilations"][$key]["arguments"][1]=$marker["mutilations"][$key]["arguments"][0];
             $marker["mutilations"][$key]["arguments"][0]=$dtmp->field[$field]["format"][$dtmp->dataformat][$type."format"];
             break;
            }
            case "timezoneConvert":
            {
             $type=$dtmp->field[$field]["type"];

             //add current timzone format as first argument
             $marker["mutilations"][$key]["arguments"][1]=$marker["mutilations"][$key]["arguments"][0];
             $marker["mutilations"][$key]["arguments"][0]=(isset($dtmp->field[$field]["format"][$dtmp->dataformat]["timezone"])?$dtmp->field[$field]["format"][$dtmp->dataformat]["timezone"]:$dtmp->format[$dtmp->dataformat]["timezone"]);

             //add current date/time format as third argument
             $marker["mutilations"][$key]["arguments"][2]=(isset($dtmp->field[$field]["format"][$dtmp->dataformat][$type."format"])?$dtmp->field[$field]["format"][$dtmp->dataformat][$type."format"]:$dtmp->format[$dtmp->dataformat][$type."format"]);
             break;
            }
            case "humanReadableNumber":
            case "humanReadableFileSize":
            {
             //set field decimals as default
             if(!isset($marker["mutilations"][$key]["arguments"][0]))
              $marker["mutilations"][$key]["arguments"][0]=(isset($dtmp->field[$field]["format"][$dtmp->dataformat]["decimals"])?$dtmp->field[$field]["format"][$dtmp->dataformat]["decimals"]:$dtmp->format[$dtmp->dataformat]["decimals"]);

             //set field decimalseparator as default
             if(!isset($marker["mutilations"][$key]["arguments"][1]))
              $marker["mutilations"][$key]["arguments"][1]=(isset($dtmp->field[$field]["format"][$dtmp->dataformat]["decimalseparator"])?$dtmp->field[$field]["format"][$dtmp->dataformat]["decimalseparator"]:$dtmp->format[$dtmp->dataformat]["decimalseparator"]);

             break;
            }
            case "strip":
            case "replace":
            {
             //set false as default
             if(!isset($marker["mutilations"][$key]["arguments"][0]))
              $marker["mutilations"][$key]["arguments"][0]=false;

             if(!isset($marker["mutilations"][$key]["arguments"][1]))
              $marker["mutilations"][$key]["arguments"][1]=false;

             break;
            }
            case "case":
            {
             //set false as default
             if(!isset($marker["mutilations"][$key]["arguments"][0]))
              $marker["mutilations"][$key]["arguments"][0]=false;

             break;
            }
            case "encode":
            case "decode":
            {
             //set false as default
             if(!isset($marker["mutilations"][$key]["arguments"][0]))
             {
              if($dtmp->field[$field]["type"]=="url") //no encoding/decoding set in mutilation, use url for for url fields, html for others
               $marker["mutilations"][$key]["arguments"][0]="url";
              else
               $marker["mutilations"][$key]["arguments"][0]="html";
             }

             break;
            }
            case "stripTags":
            case "leadingZeros":
            case "scrambleEmail":
            {
             //no defaults needed
             break;
            }
            default: //unknown
            {
             $this->e->add("Template error: Mutilation: \"".$mutilation["function"]."\" not implemented in template engine, for marker \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "system", $this->filename);
             break;
            }
           }
          }
          else //failed to add mutilation
          {
           unset($marker["mutilations"][$key]); //remove mutilation

           $this->e->add($dtmp->e); //log error
           $dtmp->e->clear(); //cleanup data error object so multiple mutilation errors aren't displayed multiple times
          }
         }
        }
       }
       else if(preg_match("/^([a-zA-Z0-9_]+)_(type|title)/", $marker["field"][$x], $tmp2)&&isset($dtmp->field[$tmp2[1]])) //$field_type / $field_title maybe?
       {
        //force static type/title value
        if($tmp2[2]=="type")
         $marker["value"]=$dtmp->field[$tmp2[1]]["type"];
        else
         $marker["value"]=$dtmp->field[$tmp2[1]]["title"];
       }
       else //nothing valid
       {
        $this->e->add("Template error: Unknown field \"".$marker["field"][$x]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $valid=false;
       }
      }
      else
      {
       if(!isset($dtmp->reference[$marker["field"][$x]]))
       {
        $this->e->add("Template error: Unknown reference \"".$marker["field"][$x]."\" for field: \"".implode("->", $marker["path"])."->".$marker["name"]."\"", "error", $this->filename);
        $failure=true;
        $valid=false;
        break;
       }

       if($dtmp->reference[$marker["field"][$x]]["type"]=="*:*") //skip *:* module
       {
        //get next reference level
        $dtmp=&$dtmp->getReferenceData($marker["field"][$x]);

        $tmp=$dtmp->check("use", "to"); //get to field
        $dtmp=&$dtmp->getReferenceData($tmp[0]); //get to module
        unset($tmp);
       }
       else //non *:* references
       {
        //get next reference level
        $dtmp=&$dtmp->getReferenceData($marker["field"][$x]);
       }
      }
     }

     //invalid module field
     if(!$valid)
      continue;

     //increase refer level if necessary
     if($l>$this->level+1)
      $this->level=$l-1;
    }

    //store marker
    $this->marker[]=$marker;
   }


   //store markers size
   $this->markers=sizeof($this->marker);


   //rename _row field foreach
   for($c=0;$c<$this->markers;$c++)
    if($this->marker[$c]["type"]=="foreach")
     $this->rename($this->marker[$c]["index"], "_foreach_".$this->marker[$c]["id"]); //rename tag to _foreach_$id, to avoid sub row confusion below

   //rename _if + _if_rights markers
   $prefix=strlen($this->markers); //number of leading zero
   for($c=0;$c<$this->markers;$c++)
   {
    if($this->marker[$c]["type"]=="if"||$this->marker[$c]["type"]=="if_rights")
    {
     $this->marker[$c]["name"]="_if_".Mutilate::leadingZeros($c, $prefix); //add leading zeros to ensure values ksort sorts if statements correctly
     $this->rename($this->marker[$c]["index"], $this->marker[$c]["name"]); //rename tag to _if_rights_$nr, used to perform if statement later on
    }
   }


   //get show fields (including those used in _if statements)
   $showFields=Array();
   for($c=0;$c<$this->markers;$c++)
   {
    if($this->marker[$c]["type"]=="field"||$this->marker[$c]["type"]=="if")
    {
     $tmp=&$showFields;
     $l=sizeof($this->marker[$c]["field"]);
     for($x=0;$x<$l;$x++)
     {
      if(!isset($tmp[$this->marker[$c]["field"][$x]]))
       $tmp[$this->marker[$c]["field"][$x]]=Array();

      $tmp=&$tmp[$this->marker[$c]["field"][$x]];
     }
    }
   }
   unset($tmp);

   if($this->markerExists("_title_field_value")) //add title field
   {
    if($titleField=$this->d->check("properties", "title")) //title field exists, use first one
     if(!isset($showFields[$titleField[0]]))
      $showFields[$titleField[0]]=Array();
   }


   //added treedepth field for recursive rows
   if($this->tree)
    $showFields["treedepth"]=Array();

   //set show fields recursively
   if($showFields)
    $this->_setShowFields($showFields, $this->d);

   if($failure)
    return false;

   return true;
  }

  function _setShowFields($fields, &$d)
  {
   $d->setShowFields(array_keys($fields));

   foreach($fields as $field => $subfields)
   {
    if($subfields)
    {
     if($d->reference[$field]["type"]=="*:*") //*:* reference, use next level data reference
     {
      $tmp=$d->reference[$field]["data"]->check("use", "to"); //get to field

      if(!$this->_setShowFields($subfields, $d->reference[$field]["data"]->reference[$tmp[0]]["data"]))
       return false;

      unset($tmp);
     }
     else //non *:* reference, use data reference
     {
      if(!$this->_setShowFields($subfields, $d->reference[$field]["data"]))
       return false;
     }
    }
   }

   return true;
  }

  /*
   *
   * Get template information
   */

  function showCount()
  {
   if(!$this->d)
   {
    $this->e->add("Data object not set", "error", $this->filename);
    return false;
   }

   if(!$this->markerExists("_total_rows"))
    return false;

   return true;
  }

  function getLevel()
  {
   if(!$this->d)
   {
    $this->e->add("Data object not set", "error", $this->filename);
    return false;
   }

   return $this->level;
  }

  function markerExists($marker, $type=false)
  {
   //try to find marker (of correct type if type is given)
   for($c=0;$c<$this->markers;$c++)
    if($marker==$this->marker[$c]["name"]) //marker name found
     if(!$type||$type==$this->marker[$c]["type"]) //no type checking or type matches
      return true;

   return false;
  }


  /*
   *
   * Populate template with data
   */

  function populate()
  {
   if(!$this->d)
   {
    $this->e->add("Data object not set", "error", $this->filename);
    return false;
   }

   //set general (static) marker values
   $values=Array();

   //_module
   if($this->markerExists("_module_name"))
    $values["_module_name"]=$this->d->module;

   if($this->markerExists("_module_title"))
    $values["_module_title"]=$this->d->title;

   //get title field
   $titleField=$this->d->check("properties", "title");
   if($titleField) //title field exists, use first one
    $titleField=$titleField[0];

   //_title
   if($this->markerExists("_title_field_name"))
   {
    if($titleField)
     $values["_title_field_name"]=$titleField;
    else
     $values["_title_field_name"]="";
   }

   if($this->markerExists("_title_field_title"))
   {
    if($titleField)
     $values["_title_field_title"]=$this->d->title($titleField);
    else
     $values["_title_field_title"]="";
   }

   if($this->markerExists("_title_field_type"))
   {
    if($titleField)
     $values["_title_field_type"]=$this->d->field[$titleField]["type"];
    else
     $values["_title_field_type"]="";
   }

   //_fields
   if($this->markerExists("_fields"))
   {
    $_field=Array();
    $values["_fields"]=Array();
    foreach($this->d->check("use", "show") as $field)
    {
     if(
      $this->d->check("properties", "order", $field)||
      $this->d->check("properties", "control", $field)||
      $this->d->getReferences($field)
     )
      continue;

     if($field!=$titleField)
     {
      $_field[]=$field; //add to _field array;

      //set value
      $values["_fields"][]=Array(
       "_field_name" => $field,
       "_field_title" => $this->d->title($field),
       "_field_type" => $this->d->field[$field]["type"],
       "_field_value" => "#".$field."#", //replace with marker, will be replace with value later on
      );
     }
    }

    //cache _fields size
    $_fields=sizeof($_field);
   }

   //static marker values
   for($c=0;$c<$this->markers;$c++)
    if($this->marker[$c]["type"]!="if"&&$this->marker[$c]["type"]!="if_rights"&&isset($this->marker[$c]["value"]))
     $values[$this->marker[$c]["name"]]=$this->marker[$c]["value"];

   //number of row
   if($this->markerExists("_current_rows"))
    $values["_current_rows"]=$this->d->vmax;

   if($this->markerExists("_total_rows"))
    $values["_total_rows"]=$this->d->getValue("show_count");

   //set static values
   if($values)
   {
    $this->exactPath=false;
    $this->set("_template", $values, true, false);
    $this->exactPath=true;
   }


   //get _if statements inner, done after template changes are done
   for($c=0;$c<$this->markers;$c++)
   {
    if($this->marker[$c]["type"]!="if"&&$this->marker[$c]["type"]!="if_rights") //not an if statement
     continue;

    $this->reset();
    while(($key=$this->search($this->marker[$c]["name"]))!==false) //find if statement
    {
     for($x=$key;$x<$this->tags;$x++) //find else statement
     {
      if($this->tag[$x]["name"]=="_else"&&$this->tag[$x]["parent"]===$key)
      {
       $true=substr($this->xml, $this->tag[$key]["inner"]["start"], $this->tag[$x]["outer"]["start"]-$this->tag[$key]["inner"]["start"]); //true: get inside until _else tag
       $false=substr($this->xml, $this->tag[$x]["outer"]["end"], $this->tag[$key]["inner"]["end"]-$this->tag[$x]["outer"]["end"]); //false: get inside from _else tag

       //replace original if statement with new if / else blocks
       $value="<!".$this->marker[$c]["name"].">".$true."<!/".$this->marker[$c]["name"].">";
       $value.="<!".str_replace("_if", "_else", $this->marker[$c]["name"]).">".$false."<!/".str_replace("_if", "_else", $this->marker[$c]["name"]).">";
       $this->set($key, $value, true, true);

       $this->marker[$c]["else"]=true; //else section available for if statement
       break;
      }
     }
    }
   }


   //set dynamic field values
   $values=Array(); //clear static values

   //determine row based template
   $subs=0;
   $sub=Array();
   $rowBased=true;

   $this->reset();
   while(($key=$this->search("_row", true, false, false))!==false) //find all _row tags
   {
    $tmp=$this->getOuter($key); //use full _row section

    //add to sub cache
    $sub[$subs]=new XML(true);

    if(!$sub[$subs]->parse($tmp))
     return false;

    //replace _row with empty #_row_$nr# marker, note that sub _row (tree) are purposefully skipped
    $this->set($key, "#_row_".$subs."#", true, true);
    $this->reset(); //restart _row search from begin

    $subs++;
   }

   if(!$subs)//no row marker
   {
    $rowBased=false; //non row based
    $tmp="<!_row>".$this->getInner("_template")."<!/_row>"; //use whole template

    //add to sub cache
    $sub[$subs]=new XML(true);

    if(!$sub[$subs]->parse($tmp))
     return false;

    $subs++;
   }

   unset($tmp);


   $rows=0;
   $row=Array();
   $this->d->reset();
   while(true)
   {
    if($this->d->vmax&&!$this->d->next()) //values gathered and no rows left, (no values; run once to get defaults)
     break;

    $values=Array();
    //get module values
    for($c=0;$c<$this->markers;$c++)
    {
     if($this->marker[$c]["type"]=="field"||$this->marker[$c]["type"]=="if") //get values for field+if markers
     {
      $loop=1; //run loop at least once
      $foreach=Array();
      while($loop>0)
      {
       //find value in data::v array
       $l=sizeof($this->marker[$c]["field"]);
       if($this->d->vmax) //only if data is gathered
       {
        $v=$this->d->v;
        for($x=0;$x<$l;$x++) //find sub field
        {
         if(!is_array($v)||!isset($v[$this->marker[$c]["field"][$x]]))
         {
          $v=false;
          break;
         }

         $v=$v[$this->marker[$c]["field"][$x]];

         if($this->marker[$c]["entry"][$x]==="foreach") //foreach entry (comments[foreach].owner.name)
         {
          //create path string: comments.foreachID
          $tmp="";
          for($z=0;$z<=$x;$z++)
           $tmp.=$this->marker[$c]["field"][$z].".".$this->marker[$c]["foreach"][$z].".";

          //start foreach loop
          if(!isset($foreach[$tmp]))
           $foreach[$tmp]=0;

          //get all foreach entries
          if($foreach[$tmp]!==false)
          {
           if($foreach[$tmp]<sizeof($v)) //foreach entry exist
           {
            $v=$v[$foreach[$tmp]]; //set entry
            $loop++; //additional loop
           }
           else //last entry passed
            $foreach[$tmp]=false; //end of foreach loop
          }
         }
         else if($this->marker[$c]["entry"][$x]!=="") //entry
         {
          if(is_array($v)&&isset($v[$this->marker[$c]["entry"][$x]])) //entry found
           $v=$v[$this->marker[$c]["entry"][$x]];
          else //invalid entry
          {
           $v=false;
           break;
          }
         }
        }
       }
       else //no values gathered
        $v=false;

       if($v===false) //no value, try to get default
       {
        //lookup correct data object
        $dtmp=$this->d;
        for($x=0;$x<$l;$x++)
        {
         if($x==$l-1) //last field
         {
          if($dtmp->check("valuetype", "default", $this->marker[$c]["field"][$x])||$dtmp->check("valuetype", "none", $this->marker[$c]["field"][$x])) //only get default if no value is set
           $v=$dtmp->getDefaultValue($this->marker[$c]["field"][$x]);
         }
         else
         {
          if($dtmp->reference[$this->marker[$c]["field"][$x]]["type"]=="*:*") //skip *:* module
          {
           //get next reference level
           $dtmp=&$dtmp->getReferenceData($this->marker[$c]["field"][$x]);

           $tmp=$dtmp->check("use", "to"); //get to field
           $dtmp=&$dtmp->getReferenceData($tmp[0]); //get to module
           unset($tmp);
          }
          else //non *:* references
          {
           //get next reference level
           $dtmp=&$dtmp->getReferenceData($this->marker[$c]["field"][$x]);
          }
         }
        }
       }


       //add entry to values array
       $tmp=&$values;
       foreach($foreach as $id => $entry) //store in foreach sub entries
       {
        $id=preg_replace("/^.+\.(\d+)\.$/", "\\1", $id);

        if(!isset($tmp["_foreach_".$id]))
         $tmp["_foreach_".$id]=Array();

        if($entry!==false)
         $tmp=&$tmp["_foreach_".$id][$entry];
        else
        {
         unset($tmp);
         break;
        }
       }

       if(is_array($v)) //array value, set size as value
        $tmp[$this->marker[$c]["name"]]=sizeof($v);
       else //normal value
        $tmp[$this->marker[$c]["name"]]=$v;

       unset($tmp);


       //next entry for deepest active foreach
       $reset=false;
       foreach(array_reverse($foreach) as $k => $active) //reversed: find first active from end
       {
        if($active!==false) //last active in list
        {
         $foreach[$k]++; //next foreach entry for last active
         $reset=true;
         break;
        }
       }

       if($reset) //deepest active to next entry
       {
        //reset all underlying
        foreach(array_reverse($foreach) as $k => $active) //reversed: set all inactive from end
        {
         if($active===false) //inactive
          $foreach[$k]=0; //reset
         else //last active in list
          break; //done
        }
       }
       unset($reset);


       //stop looping (unless foreach are still active)
       $loop--;
      }
     }
    }


    //handle if statements
    for($c=0;$c<$this->markers;$c++)
    {
     if($this->marker[$c]["type"]!="if") //not an if statement
      continue;

     //foreach depth search
     $foreach=Array();
     while(true)
     {
      //set tmp values
      unset($tmpValues);
      $tmpValues=&$values;

      if(!$this->marker[$c]["foreach"]) //no foreach depth
      {
       if($foreach) //only do 1 run
        break; //done

       $foreach=true; //set for 2nd run exit
      }
      else //foreach depth
      {
       $done=false;
       $ids=array_values($this->marker[$c]["foreach"]);
       $l=sizeof($ids);

       for($x=0;$x<$l;$x++)
       {
        $tmpValues=&$tmpValues["_foreach_".$ids[$x]];

        if(!isset($foreach[$ids[$x]])) //new foreach loop
         $foreach[$ids[$x]]=0; //start checking
        else if($x+1>=$l)
         $foreach[$ids[$x]]++; //increase entry

        if(isset($tmpValues[$foreach[$ids[$x]]])) //continue array depth search
         $tmpValues=&$tmpValues[$foreach[$ids[$x]]];
        else //no values (left) in array
        {
         if($x==0)
         {
          $done=true;
          break;
         }
         else
         {
          //reset tmp values array
          unset($tmpValues);
          $tmpValues=&$values;

          $foreach[$ids[$x]]=-1;
          $foreach[$ids[$x-1]]++; //set current to false (used by lower level to detect if it can continue with next entry)
          $x=-1; //restart array lookup
         }
        }
       }

       if($done)
        break;
      }


      //handle if foreach depths
      $match=false;
      $value=$tmpValues[$this->marker[$c]["name"]];

      if(isset($this->marker[$c]["value"])&&is_bool($this->marker[$c]["value"]))
      {
       if($value=="true")
        $value=true;
       else
        $value=false;
      }

      //type cast value to field type, operator based uses strict checking
      switch($this->marker[$c]["valuetype"])
      {
       case "nr":
       {
        $value=(int)$value;
        break;
       }
       case "float":
       case "price":
       {
        $value=str_replace($this->marker[$c]["thousandsseparator"], "", $value); //remove thousands separator
        $value=str_replace($this->marker[$c]["decimalseparator"], ".", $value); //replace decimal separator with .
        $value=(float)$value;
        break;
       }
       case "boolean":
       {
        if($value=="true")
         $value=true;
        else if($value=="false")
         $value=false;
        else
         $value=(boolean)$value;

        break;
       }
       default:
        break;
      }

      if(!$this->marker[$c]["operator"]) //simple if statement (boolean operator)
      {
       if($value)
        $match=true;
      }
      else //operator based if statement
      {
       switch($this->marker[$c]["operator"])
       {
        case "=":
        {
         if($value===$this->marker[$c]["value"])
          $match=true;

         break;
        }
        case "!=":
        {
         if($value!==$this->marker[$c]["value"])
          $match=true;

         break;
        }
        case ">":
        {
         if($value>$this->marker[$c]["value"])
          $match=true;

         break;
        }
        case ">=":
        {
         if($value>=$this->marker[$c]["value"])
          $match=true;

         break;
        }
        case "<":
        {
         if($value<$this->marker[$c]["value"])
          $match=true;

         break;
        }
        case "<=":
        {
         if($value<=$this->marker[$c]["value"])
          $match=true;

         break;
        }
        default:
        {
         $this->e->add("Unknown _if statement operator: \"".$this->marker[$c]["operator"]."\"", "error", $this->filename);
         break;
        }
       }
      }

      if(($match&&!$this->marker[$c]["not"])||(!$match&&$this->marker[$c]["not"])) //if statement matches
      {
       unset($tmpValues[$this->marker[$c]["name"]]); //leave _if statement
       $tmpValues[str_replace("_if", "_else", $this->marker[$c]["name"])]=""; //remove _else statement
      }
      else
      {
       $tmpValues[$this->marker[$c]["name"]]=""; //remove _if statement
       unset($tmpValues[str_replace("_if", "_else", $this->marker[$c]["name"])]); //leave _else statement
      }
     }
    }

    unset($tmpValues, $foreach);


    //handle if rights statements
    for($c=0;$c<$this->markers;$c++)
    {
     if($this->marker[$c]["type"]!="if_rights") //not an if statement
      continue;

      if($this->d->r($this->marker[$c]["rights"]))
       $match=true;
      else
       $match=false;


     if(($match&&!$this->marker[$c]["not"])||(!$match&&$this->marker[$c]["not"])) //if statement matches
     {
      unset($values[$this->marker[$c]["name"]]); //leave _if statement
      $values[str_replace("_if", "_else", $this->marker[$c]["name"])]=""; //remove _else statement
     }
     else
     {
      $values[$this->marker[$c]["name"]]=""; //remove _if statement
      unset($values[str_replace("_if", "_else", $this->marker[$c]["name"])]); //leave _else statement
     }
    }


    //sort values on keys so _if(_rights) are applied to template first
    ksort($values);


    //mutilate values
    for($c=0;$c<$this->markers;$c++)
    {
     if($this->marker[$c]["type"]!="field"||!isset($this->marker[$c]["mutilations"]))
      continue;

     //foreach depth search
     $foreach=Array();
     while(true)
     {
      //set tmp values
      unset($tmpValues);
      $tmpValues=&$values;

      if(!$this->marker[$c]["foreach"]) //no foreach depth
      {
       if($foreach) //only do 1 run
        break; //done

       $foreach=true; //set for 2nd run exit
      }
      else //foreach depth
      {
       $done=false;
       $ids=array_values($this->marker[$c]["foreach"]);
       $l=sizeof($ids);

       for($x=0;$x<$l;$x++)
       {
        $tmpValues=&$tmpValues["_foreach_".$ids[$x]];

        if(!isset($foreach[$ids[$x]])) //new foreach loop
         $foreach[$ids[$x]]=0; //start checking
        else if($x+1>=$l)
         $foreach[$ids[$x]]++; //increase entry

        if(isset($tmpValues[$foreach[$ids[$x]]])) //continue array depth search
         $tmpValues=&$tmpValues[$foreach[$ids[$x]]];
        else //no values (left) in array
        {
         if($x==0)
         {
          $done=true;
          break;
         }
         else
         {
          //reset tmp values array
          unset($tmpValues);
          $tmpValues=&$values;

          $foreach[$ids[$x]]=-1;
          $foreach[$ids[$x-1]]++; //set current to false (used by lower level to detect if it can continue with next entry)
          $x=-1; //restart array lookup
         }
        }
       }

       if($done)
        break;
      }

      //mutilate foreach depths
      $dateformat="";
      foreach($this->marker[$c]["mutilations"] as $mutilation)
      {
       switch($mutilation["function"])
       {
        case "dateConvert":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::dateConvert($tmpValues[$this->marker[$c]["name"]],
          ($dateformat?$dateformat:$mutilation["arguments"][0]),
          $mutilation["arguments"][1]
         );

         //store date format for additional date/timezone mutilations
         $dateformat=$mutilation["arguments"][1];
         break;
        }
        case "timezoneConvert":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::timezoneConvert($tmpValues[$this->marker[$c]["name"]],
          $mutilation["arguments"][0],
          $mutilation["arguments"][1],
          ($dateformat?$dateformat:$mutilation["arguments"][2]) //overwrite default date format with previous conversions
         );

         break;
        }
        case "humanReadableNumber":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::humanReadableNumber($tmpValues[$this->marker[$c]["name"]],
          $mutilation["arguments"][0],
          $mutilation["arguments"][1]
         );

         break;
        }
        case "humanReadableFileSize":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::humanReadableFileSize($tmpValues[$this->marker[$c]["name"]],
          $mutilation["arguments"][0],
          $mutilation["arguments"][1]
         );

         break;
        }
        case "leadingZeros":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::leadingZeros($tmpValues[$this->marker[$c]["name"]],
          $mutilation["arguments"][0]
         );

         break;
        }
        case "scrambleEmail":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::scrambleEmail($tmpValues[$this->marker[$c]["name"]]);

         break;
        }
        case "strip":
        {
         $value=$tmpValues[$this->marker[$c]["name"]];

         $tmpValues[$this->marker[$c]["name"]]=Mutilate::strip($value,
          $mutilation["arguments"][0],
          $mutilation["arguments"][1]
         );

         break;
        }
        case "stripTags":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::stripTags($tmpValues[$this->marker[$c]["name"]]);
         break;
        }
        case "replace":
        {
         $value=$tmpValues[$this->marker[$c]["name"]];

         $tmpValues[$this->marker[$c]["name"]]=Mutilate::replace($value,
          $mutilation["arguments"][0],
          $mutilation["arguments"][1]
         );

         break;
        }
        case "case":
        {
         $tmpValues[$this->marker[$c]["name"]]=Mutilate::changeCase($tmpValues[$this->marker[$c]["name"]],
          $mutilation["arguments"][0]
         );

         break;
        }
        case "encode":
        {
         $value=$tmpValues[$this->marker[$c]["name"]];

         if(strstr($mutilation["arguments"][0], "url"))
          $tmpValues[$this->marker[$c]["name"]]=Mutilate::urlEncode($value);

         if(strstr($mutilation["arguments"][0], "html"))
          $tmpValues[$this->marker[$c]["name"]]=Mutilate::htmlEncode($value);

         break;
        }
        case "decode":
        {
         $value=$tmpValues[$this->marker[$c]["name"]];

         if(strstr($mutilation["arguments"][0], "url"))
          $tmpValues[$this->marker[$c]["name"]]=Mutilate::urlDecode($value);

         if(strstr($mutilation["arguments"][0], "html"))
          $tmpValues[$this->marker[$c]["name"]]=Mutilate::htmlDecode($value);

         break;
        }
        default: //unknown, just set normal value
        {
         $this->e->add("Mutilation: \"".$mutilation["function"]."\" not implemented in template engine, for marker: \"".implode("->", $this->marker[$c]["path"])."->".$this->marker[$c]["name"]."\"", "system", $this->filename);
         break;
        }
       }
      }
     }
    }

    unset($tmpValues, $foreach);

    //get row odd/even values
    if($this->markerExists("_row_oddeven"))
     $values["_row_oddeven"]=($this->d->vc%2?"odd":"even");

    //_title_field_value
    if($this->markerExists("_title_field_value"))
     $values["_title_field_value"]=$this->d->v[$titleField];

    //set _fields values
    if($this->markerExists("_fields"))
    {
     for($c=0;$c<$_fields;$c++)
      if(!isset($values["_fields"][$_field[$c]]))
       $values["_fields"][$_field[$c]]=$this->d->v[$_field[$c]];
    }



    //apply values to sub templates
    if($values)
    {
     if($this->d->vc==1||!$this->d->vmax) //first run or defaults run
      $this->set("_template", $values, true, false); //set values/if statements to global markers

     if($this->d->vmax) //store _row values if there are any values gathered
     {
      for($c=0;$c<$subs;$c++) //loop through all sub _row templates
      {
       $tmp=clone($sub[$c]);

       $row[$rows][$c]=Array(
        "depth" => (isset($this->d->v["treedepth"])?$this->d->v["treedepth"]:0),
        "value" => ($tmp->set("_row", $values, true, true)?$tmp->output():$tmp->getInner("_row")),
       );
      }

      $rows++;
     }
    }

    //non _row based templates or no data values gathered only do 1 line
    if(!$rowBased||!$this->d->vmax)
     break;
   }


   //merge rows into template
   if($rows)
   {
    for($c=0;$c<$subs;$c++)
    {
     $value="";
     $submarker=false;

     if(!$this->tree||!preg_match("/<!_row>.*(<!\/_row>|<!_row>|#_row#)/s", $row[0][$c]["value"])) //normal mode or no _row markers
     {
      for($x=0;$x<$rows;$x++)
       $value.=$row[$x][$c]["value"];
     }
     else //tree mode
     {
      $depth=$row[0][$c]["depth"];

      //detect sub sub _row
      if(preg_match("/<!_row.*(<!_row.*<!\/_row>|<!_row|#_row).*<!\/_row>/s", $row[0][$c]["value"], $m))
      {
       if(strstr($m[1], "<!/_row>"))
        $submarker="block";
       else if(strstr($m[1], "<!_row"))
        $submarker="short";
       else
        $submarker="marker";
      }

      for($x=0;$x<$rows;$x++)
      {
       if(!$value) //first entry
       {
        $value.=$row[$x][$c]["value"]; //add value
        $value.="#ROW".$row[$x][$c]["depth"]."#"; //add ROW marker at end
       }
       else if($depth<$row[$x][$c]["depth"]) //level deeper
       {
        if($submarker)
         $value=preg_replace("/<!_row.*?>(.*)(<!_row.*?<!\/_row>|<!_row.*?>|#_row.*?#)(.*)<!\/_row>/s", "\\1".$row[$x][$c]["value"]."#ROW".$row[$x][$c]["depth"]."#"."\\3", $value); //replace sub _row with value and current ROW depth
        else
         $value=preg_replace("/<!_row.*?<!\/_row>/s", $row[$x][$c]["value"]."#ROW".$row[$x][$c]["depth"]."#", $value); //replace sub _row with value and current ROW depth

        $depth=$row[$x][$c]["depth"]; //store depth
       }
       else //same level or level up
       {
        if($depth>$row[$x][$c]["depth"]) //level up
        {
         //remove all deeper ROW markers
         for($depth;$depth>$row[$x][$c]["depth"];$depth--)
          $value=str_replace("#ROW".$depth."#", "", $value);
        }

        //same level or up
        $split=strpos($value, "#ROW".$row[$x][$c]["depth"]."#"); //get ROW marker for this level
        $value=substr($value, 0, $split).$row[$x][$c]["value"].substr($value, $split); //insert new value before ROW marker

        //remove sub _row
        if($submarker=="block")
         $value=preg_replace("/<!_row.*?<!\/_row>.*?<!\/_row>/s", "", $value, 1);
        else
         $value=preg_replace("/<!_row.*?<!\/_row>/s", "", $value, 1);
       }
      }

      //remove all remaining ROW markers
      for($depth;$depth>=0;$depth--)
       $value=str_replace("#ROW".$depth."#", "", $value);
     }


     //remove remaining sub _row
     if($submarker=="block")
      $value=preg_replace("/<!_row.*?<!\/_row>.*?<!\/_row>/s", "", $value, 1);
     else
      $value=preg_replace("/<!_row.*?<!\/_row>/s", "", $value, 1);


     //set value to template
     if($rowBased) //row based
      $this->set("_template->_row_".$c, $value, true, true);
     else //non row based
      $this->set("_template", $value, true, false);
    }
   }


   //clean up general markers and tags (start with _)
   $replacements=0;
   $this->xml=preg_replace("!(<\!/?(_[^>]+)>|#(_[^#]+)#)!", "", $this->xml, -1, $replacements);

   //ensure tag cache is up-to-date
   if($replacements&&preg_match("!(#|<)!", $this->xml)) //reparse if there are any tags / markers
    $this->reparse();

   return true;
  }


  /*
   *
   * Data to xml
   */

  function data(&$d, $preTags=false)
  {
   if($preTags)
   {
    if($this->getKey($preTags."->".$d->module))
     $preTags.="->".$d->module;
   }
   else if($this->getKey($d->module)!==false)
    $preTags=$d->module;

   if($preTags)
   {
    while($d->next())
    {
     $this->set($preTags, $d->v);
    }

    $this->set($preTags, "", true);
   }
   else
   {
    $tmp="";

    $tags=Array();
    foreach(array_keys($d->field) as $field)
     if($this->getKey($field)!==false)
      $tags[]=$field;

    $l=sizeof($tags);
    while($d->next())
    {
     for($c=0;$c<$l;$c++)
      $this->set($tags[$c], $d->v[$tags[$c]], true, false);

     $tmp.=$this->output();
     $this->reinit();
    }

    $this->xml=$tmp;
    $this->reparse();
   }
  }


  /*
   *
   * Page browser
   */

  function pageBrowser(&$d, $page_url=false)
  {
   $this->d=&$d;

   $page=ceil($this->d->getValue("show_from")/$this->d->getValue("show_total"));
   $pages=ceil($this->d->getValue("show_count")/$this->d->getValue("show_total"));

   if($page>0&&$this->d->getValue("show_count"))
   {
    $previous=Array(
     "page_url" => $page_url,
     "previous_from" => ($page-1)*$this->d->getValue("show_total"),
     "previous_title" => "Previous",
    );
    $this->set("previous", $previous, true, true);
   }
   else
    $this->set("previous", "Previous", true, true);

   for($c=0;$c<$pages;$c++)
   {
    $current=Array(
     "page_url" => $page_url,
     "page_from" => ($c)*$this->d->getValue("show_total"),
     "page_title" => $c+1,
     "active" => ($c==$page?" active":"")
    );

    $this->set("page", $current, false, true);
   }

   $this->delete("page");

   if($page<$pages-1)
   {
    $next=Array(
     "page_url" => $page_url,
     "next_from" => ($page+1)*$this->d->getValue("show_total"),
     "next_title" => "Next",
    );
    $this->set("next", $next, true, true);
   }
   else
    $this->set("next", "Next", true, true);

   return true;
  }
 }

?>
