<?

 /*****************************************************************************
 * Filename: include/xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."error.php");
 
 class XML 
 {
  function XML($onlyCommentsAndMarkers=false, $exactPath=true)
  {
   global $_path;

   $this->e=new Error;

   $this->onlyCommentsAndMarkers=$onlyCommentsAndMarkers;
   $this->exactPath=$exactPath;

   $this->clear();
  }


  function clear()
  {
   $this->filename=false;
   $this->original=false;
   $this->xml=false;

   $this->tags=0;
   $this->tag=Array();
   $this->current=0;
  }


/*****************
 Parsing 
 *****************/

  function parseFile($filename)
  {
   if(!file_exists($filename)||!is_file($filename))
   {
    $this->e->add("Can't load XML file: \"".$filename."\"", "error");
    return false;
   }

   $this->filename=$filename;

   return $this->parse(file_get_contents($this->filename));
  }


  function reinit()
  {
   if(!$this->original)
   {
    $this->e->add("XML not initialized", "error");
    return false;
   }
   
   return $this->parse($this->original);
  }


  function reparse()
  {
   if(!$this->xml)
   {
    $this->e->add("XML not initialized", "error");
    return false;
   }
   
   return $this->parse($this->xml);
  }


  function parse($xml)
  {
   //get all tags in xml
   if(!preg_match_all("!<(\!?)(/?)([^>\s]*)([^>]*)>!si", $xml, $tags, PREG_OFFSET_CAPTURE)) //no tags?
   {
    if(!preg_match("!#([^#<>]+)#!si", $xml)) //and no markers?
    {
     if($this->filename)
      $this->e->add("No XML tags found: \"".$this->filename."\"", "error");
     else
      $this->e->add("No XML tags found", "error");
 
     return false;
    }
   }

   //reset previous XML
   $this->tags=0;
   $this->tag=Array();
   $this->current=0;

   if(!$this->original)
    $this->original=$xml;

   $this->xml=$xml;

   $parents=Array();

   //store all tags with meta data
   $l1=sizeof($tags[0]);
   for($c=0;$c<$l1;$c++)
   {
    if($this->onlyCommentsAndMarkers&&$tags[1][$c][0]!="!")
     continue;

    if($tags[2][$c][0]!="/") //open tag
    {
     if($tags[1][$c][0]=="!") //comment tag
     {
      if(substr($tags[3][$c][0],0,2)=="--") //!-- comment
      {
       $name=preg_replace("![^A-Z0-9]*([A-Z0-9_]+).*!i", "\\1", $tags[4][$c][0]); //get proper name
       $type="!--short";
      }
      else //!comment
      {
       $name=$tags[3][$c][0];
       $type="!short";
      }
     }
     else
     {
      $type="short";
      $name=$tags[3][$c][0];
     }

     //store tag meta data
     $this->tag[$this->tags]=Array();

     //remove end slash from name
     if(substr($name, -1)=="/")
      $name=substr($name, 0, -1);

     $this->tag[$this->tags]["name"]=$name;
     $this->tag[$this->tags]["type"]=$type;
     $this->tag[$this->tags]["attributes"]=Array();

     $this->tag[$this->tags]["outer"]=Array();
     $this->tag[$this->tags]["outer"]["start"]=$tags[0][$c][1];
     $this->tag[$this->tags]["outer"]["end"]=$tags[0][$c][1]+strlen($tags[0][$c][0]);
     $this->tag[$this->tags]["outer"]["length"]=$this->tag[$this->tags]["outer"]["end"]-$this->tag[$this->tags]["outer"]["start"];

     //store tag attributes
     if(preg_match_all("!([^\s]+)=\"([^\"]*)\"!", $tags[4][$c][0], $attributes)) //get all key=value pairs
     {
      $l2=sizeof($attributes[0]);
      for($x=0;$x<$l2;$x++)
       $this->tag[$this->tags]["attributes"][$attributes[1][$x]]=$attributes[2][$x];
     }
     else if($tags[4][$c][0]=trim($tags[4][$c][0])) //text/fake attributes 
     {
      $this->tag[$this->tags]["attributes"]=explode(" ", $tags[4][$c][0]); //space split values
      
      if($type=="!--block"||$type=="!--short")
      {
       array_shift($this->tag[$this->tags]["attributes"]); //remove name from attributes
       array_pop($this->tag[$this->tags]["attributes"]); //remove -- tag end from attributes
      }
     }

     $parents[]=$this->tags; //add to parents list, i.e. assume it's a block tag

     $this->tags++;
    }
    else //close tag
    {
     if($tags[1][$c][0]=="!") //comment tag
     {
      if(substr($tags[3][$c][0],0,2)=="--") //!-- comment
      {
       $name=preg_replace("![^A-Z0-9]*([A-Z0-9_]+).*!i", "\\1", $tags[4][$c][0]); //get proper name
       $type="!--short";
      }
      else //!comment
      {
       $name=$tags[3][$c][0];
       $type="!short";
      }
     }
     else //normal tag
     {
      $type="short";
      $name=$tags[3][$c][0];
     }

     //close tag
     $open=false;
     for($x=sizeof($parents)-1;$x>=0;$x--) //find matching open tag in parents list
     {
      if($this->tag[$parents[$x]]["name"]==$name&&$this->tag[$parents[$x]]["type"]==$type) //name and type should match
      {
       //detect number of remaining close tags
       $openTags=0;
       for($z=0;$z<$x;$z++)
        if($this->tag[$parents[$z]]["name"]==$name&&$this->tag[$parents[$z]]["type"]==$type) //name and type should match
         $openTags++;

       //detect number of remaining close tags
       $closeTags=0;
       for($z=$c+1;$z<$l1;$z++)
       {
        if($this->onlyCommentsAndMarkers&&$tags[1][$z][0]!="!")
         continue;

        if($tags[2][$z][0]=="/") //close tag
        {
         if($tags[1][$z][0]=="!") //comment tag
         {
          if(substr($tags[3][$z][0],0,2)=="--") //!-- comment
          {
           $name2=preg_replace("![^A-Z0-9]*([A-Z0-9_]+).*!i", "\\1", $tags[4][$z][0]); //get proper name
           $type2="!--short";
          }
          else //!comment
          {
           $name2=$tags[3][$z][0];
           $type2="!short";
          }
         }
         else
         {
          $type2="short";
          $name2=$tags[3][$z][0];
         }

         if($name==$name2&&$type==$type2) //name and type should match
          $closeTags++;
        }
       }

       //close tags only matches if there are enough remaining close tags for the amount of open parents (with same name and type)
       //this ensures highest tag is considered block instead of deepest: <tag><tag/></tag>
       if($openTags<=$closeTags)
       {
        $open=$parents[$x]; //get opening counter part
        $parents=array_slice($parents, 0, $x); //remove all deeper parents

        break; //search done
       }

       unset($openTags, $closeTags);
      }
     }

     if($open!==false) //opening counter part found, add close tag and update counter part meta data
     {
      //store tag meta data
      $this->tag[$this->tags]=Array();
      $this->tag[$this->tags]["name"]=$name;

      if($type=="!--short") //!-- comment
      {
       $this->tag[$open]["type"]="!--block";
       $this->tag[$this->tags]["type"]="--close";
      }
      else if($type=="!short") //!comment
      {
       $this->tag[$open]["type"]="!block";
       $this->tag[$this->tags]["type"]="!close";
      }
      else //normal tag
      {
       $this->tag[$open]["type"]="block";
       $this->tag[$this->tags]["type"]="close";
      }

      $this->tag[$open]["close"]=$this->tags; //add closing counter part to open tag

      //update open tag meta data
      $this->tag[$open]["inner"]["start"]=$this->tag[$open]["outer"]["end"];
      $this->tag[$open]["inner"]["end"]=$tags[0][$c][1];
      $this->tag[$open]["inner"]["length"]=$this->tag[$open]["inner"]["end"]-$this->tag[$open]["inner"]["start"];

      $this->tag[$open]["outer"]["end"]=$tags[0][$c][1]+strlen($tags[0][$c][0]);
      $this->tag[$open]["outer"]["length"]=$this->tag[$open]["outer"]["end"]-$this->tag[$open]["outer"]["start"];

      $this->tags++;
     }
    }
   }


   //get all #markers# in xml
   if(preg_match_all("!#([^#<>]*)#!si", $xml, $markers, PREG_OFFSET_CAPTURE))
   {
    $l1=sizeof($markers[0]);
    for($x=0;$x<$l1;$x++)
    {
     if(!$markers[1][$x][0]) //empty marker (##)
      continue; //skip

     //store market meta data
     $tag=Array();

     $marker=$markers[1][$x][0];

     $tag["name"]=preg_replace("/^([^\s]+).*$/", "\\1", $marker);
     $tag["type"]="marker";
     $tag["attributes"]=Array();

     $tag["outer"]=Array();
     $tag["outer"]["start"]=$markers[0][$x][1];
     $tag["outer"]["end"]=$markers[0][$x][1]+strlen($markers[0][$x][0]);
     $tag["outer"]["length"]=$tag["outer"]["end"]-$tag["outer"]["start"];

     //store tag attributes
     if(preg_match_all("!([^\s]+)=\"([^\"]*)\"!", $marker, $attributes)) //get all key=value pairs
     {
      $l2=sizeof($attributes[0]);
      for($x=0;$x<$l2;$x++)
       $tag["attributes"][$attributes[1][$x]]=$attributes[2][$x];
     }
     else if($marker=trim($marker)) //text/fake attributes 
     {
      $tag["attributes"]=explode(" ", $marker);
      array_shift($tag["attributes"]); //remove name from attributes
     }

     //find parent
     $parent=false;
     for($c=$this->tags-1;$c>=0;$c--) //backwards: first tag that value falls in is the parent
     {
      if(isset($this->tag[$c]["inner"])) //block tags
      {
       if($tag["outer"]["start"]>=$this->tag[$c]["inner"]["start"]&&$tag["outer"]["end"]<=$this->tag[$c]["inner"]["end"]) //value completely within tag
       {
        $parent=$c; 
        break;
       }
      }
     }

     //find new entry for values to be inserted 
     $entry=false;
     for($c=($parent!==false?($parent+1):0);$c<$this->tags;$c++)
     {
      if(!isset($this->tag[$c]["outer"])||$tag["outer"]["start"]<$this->tag[$c]["outer"]["start"]) //close tag or new value start after tag starts: insert new tags here
      {
       $entry=$c;
       break;
      }
     }

     if($entry===false)
      $entry=$this->tags;

     //prepare close references
     for($c=0;$c<$this->tags;$c++)
      if(isset($this->tag[$c]["close"])&&$this->tag[$c]["close"]>=$entry) //close beyond current entry
       $this->tag[$c]["close"]++; //increase close with number of new entries

     array_splice($this->tag, $entry, 0, Array($tag)); //insert new entry in tags
     $this->tags++;
    }
   }


   //add parent meta data according to new tag array
   $pc=false;
   $parents=Array();
   for($c=0;$c<$this->tags;$c++)
   {
    if(strstr($this->tag[$c]["type"], "block")) //block tag
    {
     $this->tag[$c]["parent"]=$pc;
     $parents[]=$c;
     $pc=$c;
    }
    else if(strstr($this->tag[$c]["type"], "close")) //close tag
    { 
     array_pop($parents);

     if(($tmp=sizeof($parents))>0)
      $pc=$parents[$tmp-1];
     else
      $pc=false;
    }
    else if(strstr($this->tag[$c]["type"], "marker")) //lookup marker parents
    {
     //marker inside a tag?
     for($x=0;$x<$this->tags;$x++) 
     {
      if(strstr($this->tag[$x]["type"], "short")||strstr($this->tag[$x]["type"], "marker"))
      {
       if($this->tag[$c]["outer"]["start"]>$this->tag[$x]["outer"]["start"]&&$this->tag[$c]["outer"]["end"]<$this->tag[$x]["outer"]["end"])
       {
        $this->tag[$c]["parent"]=$x;
        break;
       }
      }
     }

     if(isset($this->tag[$c]["parent"])) //found
      continue;

     //marker inside a block?
     $length=strlen($this->xml);
     for($x=0;$x<$this->tags;$x++) 
     {
      if(strstr($this->tag[$x]["type"], "block"))
      {
       if($this->tag[$c]["outer"]["start"]>=$this->tag[$x]["inner"]["start"]&&$this->tag[$c]["outer"]["end"]<=$this->tag[$x]["inner"]["end"])
       {
        if($this->tag[$x]["inner"]["length"]<$length)
        {
         $this->tag[$c]["parent"]=$x;
         $length=$this->tag[$x]["inner"]["length"];
        }
       }
      }

      if(!isset($this->tag[$c]["parent"]))
       $this->tag[$c]["parent"]=false;
     }
    }
    else //short tag
     $this->tag[$c]["parent"]=$pc;
   }

   return true;
  }


/*****************
 Manipulation 
 *****************/

  function set($path=false, $value, $replaceOriginal, $replaceTag)
  {
   //check if path exists and get proper key
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   //multiple values handling: try to set all values in the inner of the path and use the new inner as value 
   if(is_array($value))
   {
    if(array_type($value)=="indexed") //indexed, multiple entries: set all entries in array in inner 
    {
     $tmp=$this->getOuterXML($key);
     $subPath=preg_replace("!\[\d+\]$!", "", $tmp->getPath(0));

     $l=sizeof($value);
     for($c=0;$c<$l;$c++)
      $tmp->set($subPath, $value[$c], false, true); //set values for entry, don't replace the original value yet: needed for every iteration

     $tmp->set($subPath, "", true, true);
     $value=$tmp->output();
    }
    else //associative array: replace tags/marks that match keys with values
    {
     $tmp=new XML($this->onlyCommentsAndMarkers, false);

     if(!$tmp->parse($this->getInner($key))) //no xml tags/marker
     {
      $this->e->add("Unable to set array values in text tag: \"".$path."\"", "error");
      return false;
     }
     else //xml tags in innner
     {
      foreach($value as $k => $v) //walk through array to replace array based values
      {
       if(is_array($v))
       {
        if(!$v)
         $tmp->set($k, "", true, false); //replace original but leave tags 
        else
         $tmp->set($k, $v, true, false); //replace original but leave tags 
       }
      }

      foreach($value as $k => $v) //walk through array to replace normal values
      {
       if(!is_array($v))
       {
        //to ensure newly inserted tags aren't replace (i.e. endless loop) only existing entries will be replaced
        $entries=0; //find number of entries with $k key
        while($tmp->getKey($k."[".$entries."]")!==false)
         $entries++;

        for($c=$entries-1;$c>=0;$c--) //replace number of entries backwards
         $tmp->set($k."[".$c."]", $v, true, true); //replace original and tags
       }
      }

      $value=$tmp->output();
     }
    }
   }

   if($replaceTag||!strstr($this->tag[$key]["type"], "block")) //replace tag, always replace entire tag for non block tags
   {
    if($replaceOriginal)
    {
     $start=$this->tag[$key]["outer"]["start"];
     $length=$this->tag[$key]["outer"]["length"];

     if(!$this->_set($start, $length, $value)) //insert value
      return false;
    }
    else //keep original value
    {
     $start=$this->tag[$key]["outer"]["start"];

     if(!$this->_set($start, 0, $value)) //insert value + original tags
      return false;
    }
   }
   else
   {
    if($replaceOriginal)
    {
     $start=$this->tag[$key]["inner"]["start"];
     $length=$this->tag[$key]["inner"]["length"];

     if(!$this->_set($start, $length, $value)) //insert value
      return false;
    }
    else //keep original value
    {
     $tmp=$this->getOuterXML($key);
     $tmp->set(false, $value, true, false); //set value in tag

     $start=$this->tag[$key]["outer"]["start"];

     if(!$this->_set($start, 0, $tmp->output())) //insert value + original tags
      return false;
    }
   }

   return true;
  }


  function _set($start, $length, $value)
  {
   if(is_array($value)||is_object($value))
   {
    $this->e->add("_Setting array/object values is not supported", "error");
    return false;
   }

   if($start<0)
   {
    $this->e->add("_Set start may not be negative", "error");
    return false;
   }

   if($length<0)
   {
    $this->e->add("_Set length may not be negative", "error");
    return false;
   }

   if($start+$length>strlen($this->xml))
   {
    $this->e->add("_Set replace end is beyond end of XML", "error");
    return false;
   }

   $diff=strlen($value)-$length; //get difference in old and new length

   $xmlBefore=substr($this->xml, 0, $start);
   $xmlAfter=substr($this->xml, $start+$length);

   $this->xml=$xmlBefore.$value.$xmlAfter; //added value in in xml

   //update all positions of current tags
   for($c=0;$c<$this->tags;$c++)
   {
    if(isset($this->tag[$c]["outer"])) //all except close 
    {
     if($start<=$this->tag[$c]["outer"]["start"]) //tag starts after new value
     {
      if($start+$length>$this->tag[$c]["outer"]["start"])//within replacement: delete
      {
       if(strstr($this->tag[$c]["type"], "block"))
       {
        $closeTag=$this->tag[$c]["close"];
        $this->tag[$closeTag]["type"]="deleted"; 
       }

       $this->tag[$c]["type"]="deleted"; 
       continue;
      }

      $this->tag[$c]["outer"]["start"]+=$diff; //update start position
     }

     if($start<$this->tag[$c]["outer"]["end"]) //tag ends after bew vakye
      $this->tag[$c]["outer"]["end"]+=$diff; //update end position

     //update outer length if value was inserted within 
     $this->tag[$c]["outer"]["length"]=$this->tag[$c]["outer"]["end"]-$this->tag[$c]["outer"]["start"];
    }

    if(isset($this->tag[$c]["inner"])) //block tags
    {
     if($start<$this->tag[$c]["inner"]["start"]) //inner start after new value
      $this->tag[$c]["inner"]["start"]+=$diff; //update start position

     if($start<=$this->tag[$c]["inner"]["end"]) //inner ends after new value 
      $this->tag[$c]["inner"]["end"]+=$diff; //update position

     //update inner length if value was inserted within 
     $this->tag[$c]["inner"]["length"]=$this->tag[$c]["inner"]["end"]-$this->tag[$c]["inner"]["start"];
    }
   }

   //remove deleted tags and update parents and close references
   $l=$this->tags;
   for($c=0;$c<$l;$c++)
   {
    if($this->tag[$c]["type"]=="deleted") 
    {
     //prepare parents
     for($x=$c+1;$x<$l;$x++)
      if(isset($this->tag[$x]["parent"])&&$this->tag[$x]["parent"]>$c) //parent beyond current entry
       $this->tag[$x]["parent"]--; //reduce parent
     
     //prepare close tags
     for($x=0;$x<$l;$x++)
      if(isset($this->tag[$x]["close"])&&$this->tag[$x]["close"]>$c) //close beyond current entry
       $this->tag[$x]["close"]--; //reduce close

     array_splice($this->tag, $c, 1); //remove entry from tags

     $c--; //redo current entry since it became the "next" entry
     $l--; //reduce total tags;
    }
   }

   $this->tags=sizeof($this->tag); //update tags counter

   //new tags/markers inserted
   if(preg_match("/(<|#)/", $value)) 
   {
    $length=strlen($value);
    $end=$start+$length;

    //find parent
    $parent=false;
    for($c=$this->tags-1;$c>=0;$c--) //backwards: first tag that value falls in is the parent
    {
     if(isset($this->tag[$c]["inner"])) //block tags
     {
      if($start>=$this->tag[$c]["inner"]["start"]&&$end<=$this->tag[$c]["inner"]["end"]) //value completely within tag
      {
       $parent=$c; 
       break;
      }
     }
    }

    //find new entry for values to be inserted 
    $entry=false;
    for($c=($parent!==false?($parent+1):0);$c<$this->tags;$c++)
    {
     if(strstr($this->tag[$c]["type"], "close")) //close tag, look up end of referencing tag
     {
      for($x=0;$x<$this->tags;$x++)
      {
       if(isset($this->tag[$x]["close"])&&$this->tag[$x]["close"]==$c) //referencing tag
       {
        if($start<=$this->tag[$x]["inner"]["end"]) //new value start before close tag starts: insert new tags here
        {
         $entry=$c;
         break;
        }
       }
      }
     }
     else if($start<$this->tag[$c]["outer"]["start"]) //new value start after tag starts: insert new tags here
     {
      $entry=$c;
      break;
     }
    }

    if($entry===false)
     $entry=$this->tags;

    $xml=new XML($this->onlyCommentsAndMarkers);
    $xml->parse($value);

    //prepare new entries
    for($c=0;$c<$xml->tags;$c++)
    {
     if(isset($xml->tag[$c]["parent"])) //set new parents
     {
      if($xml->tag[$c]["parent"]===false) //directly under parent
       $xml->tag[$c]["parent"]=$parent;
      else //under new entry
       $xml->tag[$c]["parent"]+=$entry; //increase parent with number of entries
     }

     if(isset($xml->tag[$c]["close"])) //set new close references
     {
      if($xml->tag[$c]["close"]===false) //directly under close
       $xml->tag[$c]["close"]=$close;
      else //under new entry
       $xml->tag[$c]["close"]+=$entry; //increase close with number of entries
     }

     if(isset($xml->tag[$c]["outer"]))  //set outer positions
     {
      $xml->tag[$c]["outer"]{"start"}+=$start;
      $xml->tag[$c]["outer"]{"end"}+=$start;
     }

     if(isset($xml->tag[$c]["inner"]))  //set inner positions
     {
      $xml->tag[$c]["inner"]{"start"}+=$start;
      $xml->tag[$c]["inner"]{"end"}+=$start;
     }
    }

    $newEntries=$xml->tags;

    //prepare parents
    for($c=$entry+1;$c<$this->tags;$c++)
     if(isset($this->tag[$c]["parent"])&&$this->tag[$c]["parent"]!==false&&$this->tag[$c]["parent"]>=$entry) //parent beyond current entry
      $this->tag[$c]["parent"]+=$newEntries; //increase parent with number of new entries

    //prepare close references
    for($c=0;$c<$this->tags;$c++)
     if(isset($this->tag[$c]["close"])&&$this->tag[$c]["close"]>=$entry) //close beyond current entry
      $this->tag[$c]["close"]+=$newEntries; //increase close with number of new entries

    array_splice($this->tag, $entry, 0, $xml->tag); //insert new entry in tags
    $this->tags=sizeof($this->tag); //update tags counter
   }

   return true;
  }


  function addBefore($path=false, $value)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   $this->_set($this->tag[$key]["outer"]["start"], 0, $value);

   return true;
  }


  function addAfter($path=false, $value)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   $this->_set($this->tag[$key]["outer"]["end"], 0, $value);

   return true;
  }


  function addInsideAtBegin($path=false, $value)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   if(!strstr($this->tag[$key]["type"], "block"))
   {
    if($this->filename)
     $this->e->add("Can't add inside non block tag: \"".$path."\" in template: \"".$this->filename."\"", "warning");
    else
     $this->e->add("Can't add inside non block tag: \"".$path."\"", "warning");

    return false;
   }

   $this->_set($this->tag[$key]["inner"]["start"], 0, $value);

   return true;
  }


  function addInsideAtEnd($path=false, $value)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   if(!strstr($this->tag[$key]["type"], "block"))
   {
    if($this->filename)
     $this->e->add("Can't add inside non block tag: \"".$path."\" in template: \"".$this->filename."\"", "warning");
    else
     $this->e->add("Can't add inside non block tag: \"".$path."\"", "warning");

    return false;
   }

   $this->_set($this->tag[$key]["inner"]["end"], 0, $value);

   return true;
  }

 
  function setAttribute($path=false, $attribute, $value=false)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   if(
    !strstr($this->tag[$key]["type"], "block")&&
    !strstr($this->tag[$key]["type"], "short")&&
    !strstr($this->tag[$key]["type"], "marker")
   )
   {
    $this->e->add("Setting attributes is not supported for \"".$this->tag[$key]["type"]."\" tags", "error");
    return false;
   }

   if($value!==false)
    $value=htmlentities($value);

   if($this->tag[$key]["attributes"])
    $hasAttributes=true;
   else
    $hasAttributes=true;

   $found=false;
   $attributes="";
   foreach($this->tag[$key]["attributes"] as $k => $v)
   {
    if($k==$attribute)
    {
     if($value!==false) //edit
     {
      $attributes.=" ".$attribute."=\"".$value."\"";
      $this->tag[$key]["attributes"][$attribute]=$value;
     }
     else //delete
      unset($this->tag[$key]["attributes"][$attribute]);

     $found=true;
    }
    else
     $attributes.=" ".$k."=\"".$v."\"";
   }

   if(!$found&&$value!==false)
   {
    $attributes.=" ".$attribute."=\"".$value."\"";
    $this->tag[$key]["attributes"][$attribute]=$value;
   }

   if(strstr($this->tag[$key]["type"], "!--"))
   {
    $start=($this->tag[$key]["outer"]["start"]+4+strlen($this->tag[$key]["name"]))+1;
    $length=($this->tag[$key]["outer"]["end"]-$start)-4;
   }
   else if(strstr($this->tag[$key]["type"], "!"))
   {
    $start=($this->tag[$key]["outer"]["start"]+strlen($this->tag[$key]["name"]))+2;
    $length=($this->tag[$key]["outer"]["end"]-$start)-1;
   }
   else if(strstr($this->tag[$key]["type"], "block"))
   {
    $start=($this->tag[$key]["outer"]["start"]+strlen($this->tag[$key]["name"]))+1;
    $length=($this->tag[$key]["inner"]["start"]-$start)-1;
   }
   else if(strstr($this->tag[$key]["type"], "short"))
   {
    $start=($this->tag[$key]["outer"]["start"]+strlen($this->tag[$key]["name"]))+1;
    $length=($this->tag[$key]["outer"]["end"]-$start)-1;

    if(($outer=$this->getOuter($key)))
    {
     if(substr($outer, -3)==" />"||substr($outer, -3)==" ?>")
      $length-=2;
     else if(substr($outer, -2)=="/>"||substr($outer, -2)=="?>")
      $length--;
    }
   }
   else if(strstr($this->tag[$key]["type"], "marker"))
   {
    $start=($this->tag[$key]["outer"]["start"]+strlen($this->tag[$key]["name"]))+1;
    $length=($this->tag[$key]["outer"]["end"]-$start)-1;
   }
   else
   {
    $this->e->add("Setting attributes is not supported for \"".$this->tag[$key]["type"]."\" tags", "error");
    return false;
   }

   $this->_set($start, $length, $attributes);

   return true;
  }


  function rename($path=false, $name)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   if(
    !strstr($this->tag[$key]["type"], "block")&&
    !strstr($this->tag[$key]["type"], "short")&&
    !strstr($this->tag[$key]["type"], "marker")
   )
   {
    $this->e->add("Renaming tag is not supported for \"".$this->tag[$key]["type"]."\" tags", "error");
    return false;
   }

   if(strstr($this->tag[$key]["type"], "!--"))
    $startOffset=5;
   else if(strstr($this->tag[$key]["type"], "!"))
    $startOffset=2;
   else //block/short
    $startOffset=1;

   $start=$this->tag[$key]["outer"]["start"]+$startOffset;
   $length=strlen($this->tag[$key]["name"]);

   $this->_set($start, $length, $name); //update xml
   $this->tag[$key]["name"]=$name; //change name in tag array

   if(strstr($this->tag[$key]["type"], "block")) //update close tag for block tags
   {
    $start=$this->tag[$key]["inner"]["end"]+$startOffset+1;

    $this->_set($start, $length, $name); //update xml
    $this->tag[$this->tag[$key]["close"]]["name"]=$name; //change name in tag array for close tag
   }

   return true;
  }


  function delete($path=false)
  {
   if(!($tmp=$this->getPathAndKey($path)))
    return false;

   list($path, $key)=$tmp;

   $this->set($key, "", true, true);

   return true;
  }



/*****************
 Info
 *****************/

  function getPathAndKey($path)
  {
   if($path===false)
   {
    $key=$this->current; 
    $path=$this->getPath($key);
   }
   else if(is_numeric($path))
   {
    if(!isset($this->tag[$path]))
    {
     if($this->filename)
      $this->e->add("Unknown tag key: \"".$path."\" in template: \"".$this->filename."\"", "warning");
     else
      $this->e->add("Unknown tag key: \"".$path."\"", "warning");

     return false;
    }
    
    $key=$path; 
    $path=$this->getPath($key);
   }
   else if(($key=$this->getKey($path))===false)
   {
    if($this->filename)
     $this->e->add("Unknown tag: \"".$path."\" in template: \"".$this->filename."\"", "warning");
    else
     $this->e->add("Unknown tag: \"".$path."\"", "warning");

    return false;
   }

   return Array($path, $key);
  }


  function getKey($path, $parent=false)
  {
   if(!is_array($path))
    $path=explode("->", $path);

   $name=array_shift($path);
   $tmp=explode("#", $name);

   //find ID: html->body->div#id
   if(isset($tmp[1])) 
   {
    $name=$tmp[0];
    $tagID=$tmp[1];
    $tagNR=false;
   }
   else
   {
    $tagID=false;

    //find tag number: 
    //html->body->div[0]: first div in body, same as html->body->div
    //html->body->div[1]: second div in body
    //html->body->div[2]: third...
    if(preg_match("/(.+)\[(\d+)\]/", $name, $tmp))
    {
     $name=$tmp[1];
     $tagNR=$tmp[2];
    }
    else
     $tagNR=false;
   }

   $key=false;
   $tagMatches=0;
   for($c=0;$c<$this->tags;$c++)
   {
    if(!strstr($this->tag[$c]["type"], "close"))
    {
     if(((!$this->exactPath&&$parent===false)||$this->tag[$c]["parent"]===$parent)&&$this->tag[$c]["name"]===$name)
     {
      if($tagID) //check tag id
      {
       if(!isset($this->tag[$c]["attributes"]["id"])||$this->tag[$c]["attributes"]["id"]!=$tagID) //tag id doesn't match
        continue; //try next
      }
      else if($tagNR) //check tag number, 0 can be ignored here
      {
       if($tagNR!=$tagMatches) //not the same tag nr
       {
        $tagMatches++; //next match
        continue; //try next
       }
      }

      //found
      if(sizeof($path))
       $key=$this->getKey($path, $c);
      else
       $key=$c;

      break;
     }
    }
   }

   return $key;
  }


  function getName($key)
  {
   if(!isset($this->tag[$key]))
   {
    if($this->filename)
     $this->e->add("Unknown tag key: \"".$key."\" in template: \"".$this->filename."\"", "warning");
    else
     $this->e->add("Unknown tag key: \"".$key."\"", "warning");

    return false;
   }

   return $this->tag[$key]["name"];
  }


  function getPath($key)
  {
   if(!isset($this->tag[$key]))
   {
    if($this->filename)
     $this->e->add("Unknown tag key: \"".$key."\" in template: \"".$this->filename."\"", "warning");
    else
     $this->e->add("Unknown tag key: \"".$key."\"", "warning");

    return false;
   }

   if(strstr($this->tag[$key]["type"], "close"))
   {
    $this->e->add("Can't get path for close tag: \"".$key."\"", "warning");
    return false;
   }

   $entry=0;
   for($c=0;$c<$key;$c++)
   {
    if(isset($this->tag[$c]["parent"])&&$this->tag[$c]["parent"]==$this->tag[$key]["parent"]) //same parent
     if($this->tag[$c]["name"]==$this->tag[$key]["name"])
      $entry++;
   }

   $path=$this->tag[$key]["name"]."[".$entry."]";

   if(!strstr($this->tag[$key]["type"], "close"))
   {
    if($this->tag[$key]["parent"]!==false)
    {
     $entry=0;
     for($c=0;$c<$key;$c++)
     {
      if(isset($this->tag[$c]["parent"])&&$this->tag[$c]["parent"]==$this->tag[$key]["parent"]) //same parent
       if($this->tag[$c]["name"]==$this->tag[$key]["name"])
        $entry++;
     }

     $path=$this->getPath($this->tag[$key]["parent"])."->".$path;
    }
   }

   return $path;
  }


  function getInner($path=false)
  {
   if(is_numeric($path)&&isset($this->tag[$path]))
    $key=$path;
   else
   {
    if(!($tmp=$this->getPathAndKey($path)))
     return false;
 
    list($path, $key)=$tmp;
   }

   if(strstr($this->tag[$key]["type"], "block"))
    return substr($this->xml, $this->tag[$key]["inner"]["start"], $this->tag[$key]["inner"]["length"]);

   return false;
  }


  function getInnerXML($path=false)
  {
   if(!($tmp=$this->getInner($path)))
    return false;

   $xml=new XML($this->onlyCommentsAndMarkers);

   if(!$xml->parse($tmp))
    return false;

   return $xml;
  }


  function getOuter($path=false)
  {
   if(is_numeric($path)&&isset($this->tag[$path]))
    $key=$path;
   else
   {
    if(!($tmp=$this->getPathAndKey($path)))
     return false;
 
    list($path, $key)=$tmp;
   }

   if(
    strstr($this->tag[$key]["type"], "block")||
    strstr($this->tag[$key]["type"], "short")||
    strstr($this->tag[$key]["type"], "marker")
   )
    return substr($this->xml, $this->tag[$key]["outer"]["start"], $this->tag[$key]["outer"]["length"]);
   
   return false;
  }


  function getOuterXML($path=false)
  {
   if(!($tmp=$this->getOuter($path)))
    return false;

   $xml=new XML($this->onlyCommentsAndMarkers);

   if(!$xml->parse($tmp))
    return false;

   return $xml;
  }


  function getAttributes($path=false)
  {
   if(is_numeric($path)&&isset($this->tag[$path]))
    $key=$path;
   else
   {
    if(!($tmp=$this->getPathAndKey($path)))
     return false;
 
    list($path, $key)=$tmp;
   }
 
   if(!isset($this->tag[$key]["attributes"]))
    return false;

   $attributes=Array();
   foreach($this->tag[$key]["attributes"] as $k => $v)
    $attributes[$k]=html_entity_decode($v);

   return $attributes;
  }


  function getAttribute($path=false, $attribute)
  {
   if(is_numeric($path)&&isset($this->tag[$path]))
    $key=$path;
   else
   {
    if(!($tmp=$this->getPathAndKey($path)))
     return false;
 
    list($path, $key)=$tmp;
   }

   if(isset($this->tag[$key]["attributes"][$attribute]))
    return html_entity_decode($this->tag[$key]["attributes"][$attribute]);
   else
    return false;
  }


  function getChildren($path=false)
  {
   if(is_numeric($path)&&isset($this->tag[$path]))
    $key=$path;
   else
   {
    if(!($tmp=$this->getPathAndKey($path)))
     return false;
 
    list($path, $key)=$tmp;
   }

   $childs=Array();
   for($c=0;$c<$this->tags;$c++)
    if(isset($this->tag[$c]["parent"]))
     if($this->tag[$c]["parent"]===$key)
      $childs[]=$c;

   return $childs;
  }


  function getType($path=false)
  {
   if(is_numeric($path)&&isset($this->tag[$path]))
    $key=$path;
   else
   {
    if(!($tmp=$this->getPathAndKey($path)))
     return false;
 
    list($path, $key)=$tmp;
   }

   return $this->tag[$key]["type"];
  }


/*****************
 Search 
 *****************/

  function current($path)
  {
   if(($key=$this->getKey($path))===false)
   {
    if($this->filename)
     $this->e->add("Unknown tag: \"".$path."\" in template: \"".$this->filename."\"", "warning");
    else
     $this->e->add("Unknown tag: \"".$path."\"", "warning");

    return false;
   }

   $this->current=$key;
   return true;
  }


  function next($comment=true, $marker=true, $close=false)
  {
   if($this->current>=$this->tags)
   {
    $this->current=0;
    return false;
   }

   for($this->current++;$this->current<$this->tags;$this->current++)
   {
    if(!$comment&&preg_match("/^(\!|\-\-)/", $this->tag[$this->current]["type"])) //skip comment tags + comment close tags
     continue;

    if(!$marker&&strstr($this->tag[$this->current]["type"], "marker")) //skip marker tags
     continue;

    if(!$close&&strstr($this->tag[$this->current]["type"], "close")) //skip close tags
     continue;

    return $this->current;
   }

   $this->current=0;
   return false;
  }


  function search($tag, $comment=true, $marker=true, $close=false)
  {
   $tag=strtolower($tag);

   while(($key=$this->next($comment, $marker, $close))!==false)
    if(strtolower($this->tag[$this->current]["name"])==$tag) //we've got a match
     return $key;

   return false;
  }


  function reset()
  {
   $this->current=0;
  }


/*****************
 Output
 *****************/

  function output()
  {
   return $this->xml;
  }

  
  function display()
  {
   $output=htmlentities($this->xml);

   $output=preg_replace("/^(.+)$/m", "<li>\\1</li>", $output);
   $output=preg_replace("/^$/m", "<li>&nbsp; </li>", $output);
   $output=preg_replace_callback("/^<li>(\s+)/m", create_function('$m', 'return "<li>".str_repeat("&nbsp;", strlen($m[1]));'), $output);
   $output=preg_replace("/(&gt;)(.*)(&lt;)/Uims", "\\1<font color='#222222'>\\2</font>\\3", $output);
   $output=preg_replace("/(&lt;!--)(.*)(--&gt;)/Uims", "\\1<font color='#CCCC88'>\\2</font>\\3", $output);
   $output=preg_replace("/(&lt;)(.+)(&gt;)/Um", "<font color='#FF8888'>\\1</font><font color='#4444AA'>\\2</font><font color='#FF8888'>\\3</font>", $output);
   $output=preg_replace("/([^\s]+)(=&quot;)(.*)(&quot;)/U", "<font color='#5999A4'>\\1</font>\\2<font color='#666666'>\\3</font>\\4", $output);

   $output="<ul class=\"list decimal\">\n".$output."</ul>\n";

   return $output;
  }


 }

?>
