<?

 /*****************************************************************************
 * Filename: include/xmli.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."http.php");
 require_once($_path["include"]."xml.php");

 class XMLi extends XML
 {
  function XMLi($API=true)
  {
   global $_path;

   XML::XML();

   $this->xmlVersion="1.0";
   $this->xmlEncoding="ISO-8859-1";
   $this->xmlJasVersion="1.0";

   $this->templates=$_path["include"]."xmli/";
  }


  function parseTemplate($template)
  {
   if(!file_exists($this->templates.$this->xmlJasVersion."/".$template.".xml"))
   {
    $this->e->add("XML template: \"".$template."\" version \"".$this->xmlJasVersion."\" not found", "error");
    return false;
   }

   if(!$this->parseFile($this->templates.$this->xmlJasVersion."/".$template.".xml"))
   {
    $this->e->add("XML template: \"".$template."\" version \"".$this->xmlJasVersion."\" not found", "error");
    return false;
   }

   if($this->getAttribute("?xml", "version")!=$this->xmlVersion)
    $this->setAttribute("?xml", "version", $this->xmlVersion);

   if($this->getAttribute("?xml", "encoding")!=$this->xmlEncoding)
    $this->setAttribute("?xml", "encoding", $this->xmlEncoding);

   if($this->getAttribute("jas", "version")!=$this->xmlJasVersion)
    $this->setAttribute("jas", "version", $this->xmlJasVersion);

   return true;
  }
 }


 class XMLiServer extends XMLi
 {
  function XMLiServer($API=true, $config=false)
  {
   XMLi::XMLi();

   //get API config
   $config=new Config();
   $config->merge("API", $config);
   $this->config=$config->config;
   unset($config);

   if($API) //API mode, either make sure you have configAPIs set up properly or set API to false in your code if you abuse XMLi for other things
   {
    if(!$this->config["xml"])  //API disabled
    {
     $this->e->add("XML API disabled");

     $this->actions=Array(
      "view" => Array(
       "item" => false,
       "list" => false,
       "all" => false,
      ),

      "tree" => Array(
       "up" => false,
       "down" => false,
      ),

      "search" => Array(
       "title" => false,
       "field" => false,
       "show" => false,
       "full" => false,
      ),

      "add" => Array("item" => false),
      "edit" => Array("item" => false),
      "move" => Array("item" => false),
      "delete" => Array("item" => false),

      "user" => Array(
       "login" => false,
       "logout" => false,
       "password" => false,
      ),

      "database" => Array("info" => false),
     );

     $this->modules=false;
     $this->projects=false;
     $this->language=false;

     return false;
    }
    else //API is enabled
    {
     $this->actions=Array(
      "view" => Array( //true, modules Array, false
       "item" => ($this->config["xmlViewItem"]?($this->config["ViewItemAllModules"]?true:$this->config["ViewItemModules"]):false),
       "list" => ($this->config["xmlViewList"]?($this->config["ViewListAllModules"]?true:$this->config["ViewListModules"]):false),
       "all" => ($this->config["xmlViewAll"]?($this->config["ViewAllAllModules"]?true:$this->config["ViewAllModules"]):false),
      ),

      "tree" => Array(
       "up" => ($this->config["xmlTreeUp"]?($this->config["TreeUpAllModules"]?true:$this->config["TreeUpModules"]):false),
       "down" => ($this->config["xmlTreeDown"]?($this->config["TreeDownAllModules"]?true:$this->config["TreeDownModules"]):false),
      ),

      "search" => Array(
       "title" => ($this->config["xmlSearch"]?($this->config["SearchAllModules"]?true:$this->config["SearchModules"]):false),
       "field" => ($this->config["xmlSearch"]?($this->config["SearchAllModules"]?true:$this->config["SearchModules"]):false),
       "show" => ($this->config["xmlSearch"]?($this->config["SearchAllModules"]?true:$this->config["SearchModules"]):false),
       "full" => ($this->config["xmlSearch"]?($this->config["SearchAllModules"]?true:$this->config["SearchModules"]):false),
      ),

      "add" => Array("item" => ($this->config["xmlAdd"]?($this->config["AddAllModules"]?true:$this->config["AddModules"]):false)),
      "edit" => Array("item" => ($this->config["xmlEdit"]?($this->config["EditAllModules"]?true:$this->config["EditModules"]):false)),
      "move" => Array("item" => ($this->config["xmlMove"]?($this->config["MoveAllModules"]?true:$this->config["MoveModules"]):false)),
      "delete" => Array("item" => ($this->config["xmlDelete"]?($this->config["DeleteAllModules"]?true:$this->config["DeleteModules"]):false)),

      "user" => Array(
       "login" => $this->config["xmlUserLoginLogout"],
       "logout" => $this->config["xmlUserLoginLogout"],
       "password" => $this->config["xmlUserPassword"],
      ),
      "database" => Array("info" => ($this->config["xmlDatabaseInfo"]?($this->config["DatabaseInfoAllModules"]?true:$this->config["DatabaseInfoModules"]):false)),
     );

     $this->modules=$this->config["xmlModules"];
     $this->projects=$this->config["xmlProjects"];
     $this->language=$this->config["xmlLanguage"];
    }
   }
   else
   {
    $this->actions=Array(
     "view" => Array(
      "item" => true,
      "list" => true,
      "all" => true,
     ),

      "tree" => Array(
       "up" => true,
       "down" => true,
      ),

     "search" => Array(
      "title" => true,
      "field" => true,
      "show" => true,
      "full" => true,
     ),

     "add" => Array("item" => true),
     "edit" => Array("item" => true),
     "move" => Array("item" => true),
     "delete" => Array("item" => true),

     "user" => Array(
      "login" => true,
      "logout" => true,
      "password" => true,
     ),

     "database" => Array("info" => true),
    );

    $this->modules=true;
    $this->projects=true;
    $this->language=true;
   }
  }


  function output($headers=true)
  {
   $xml=parent::output(false);

   if($headers)
   {
    header('Content-type: text/xml');
    header('Content-length: '.strlen($xml));
   }

   return $xml;
  }


  function error($error=false)
  {
   if($error)
    $this->e->add($error);

   if($this->parseTemplate("errors"))
   {
    $sub=$this->getOuterXML("jas->errors->error");
    $this->set("jas->errors", "\n", true, false);

    for($c=0;$c<$this->e->errors();$c++)
    {
     $sub->setAttribute("error", "level", $this->e->getLevel($c));
     $sub->setAttribute("error", "text", $this->e->getError($c));
     $sub->setAttribute("error", "info", $this->e->getExtra($c));
     $this->addInsideAtEnd("jas->errors", "  ".$sub->output(false)."\n");
     $sub->reinit();
    }
   }
   else
   {
    for($c=0;$c<$this->e->errors();$c++)
    {
     print $this->e->getLevel($c).": ";
     print $this->e->getError($c)."\n";
    }
   }

   return true;
  }


  function handle($request)
  {
   if($this->e->any())
   {
    $this->error();
    return false;
   }

   $inXML=new XML();

   if(!$inXML->parse($request))
   {
    $this->e->add($inXML->e);
    $this->error();
    return false;
   }

   if($inXML->getKey("jas")===false)
   {
    $this->error("Incorrect XML format: \"jas\" tag not found");
    return false;
   }

   if(!($childs=$inXML->getChildren("jas")))
   {
    $this->error("Incorrect XML format: \"jas\" tag empty");
    return false;
   }

   if(!($version=$inXML->getAttribute("jas", "version")))
    $version=$this->xmlJasVersion;

   for($c=0;$c<sizeof($childs);$c++)
   {
    $type=$inXML->getName($childs[$c]);

    switch($version)
    {
     case "1.0":
     {
      $this->xmlJasVersion=$version;

      $func="handle_".$type."_".strtr($version, ".", "_");
      switch($type)
      {
       case "data":
       case "modules":
       case "projects":
       case "language":
       {
        if(!$this->$func($inXML->getOuterXML($childs[$c])))
         return false;

        break;
       }
       default:
       {
        $this->error("Incorrect XML format: unsupported type \"".$type."\" for version: \"".$version."\"");
        return false;
       }
      }

      break;
     }
     default:
     {
      $this->error("Incorrect XML format: unsupported version \"".$version."\"");
      return false;
     }
    }
   }

   return true;
  }

 /*
  * HANDLING 1.0
  */


  function handle_data_1_0(&$subXML)
  {
   if(!$this->parseTemplate("data"))
   {
    $this->error();
    return false;
   }

   if(!($module=$subXML->getAttribute("data", "module")))
   {
    $this->error("No module set for data request");
    return false;
   }

   if(!($action=$subXML->getAttribute("data", "action")))
   {
    $this->error("No action set for data request");
    return false;
   }

   if(!($extra=$subXML->getAttribute("data", "extra")))
   {
    $this->error("No extra action set for data request");
    return false;
   }

   $options=Array();
   if(($format=$subXML->getAttribute("data", "format")))
    $options["format"]=$format;

   if(($db=$subXML->getAttribute("data", "db")))
    $options["db"]=$db;

   if(($count=$subXML->getAttribute("data", "count"))&&strtolower($count)=="true")
    $count=true;
   else
    $count=false;

   $module=explode(":", $module);
   $d=Data::cache($module[0], $action, $extra, $options);

   if(isset($module[1])) //*:* referenced module
   {
    if(!isset($d->reference[$module[1]]))
    {
     $this->error("Unknown referenced module: \"".$module[1]."\"");
     return false;
    }

    if($d->reference[$module[1]]["type"]!="*:*")
    {
     $this->error("Referenced module: \"".$module[1]."\" isn't \"*:*\" type");
     return false;
    }

    $d=&$d->reference[$module[1]]["data"];
    $d->setAction($action);
    $d->setExtra($extra);
   }

   if($d->e->any("fatal"))
   {
    $this->error($d->e);
    return false;
   }


   if(!isset($this->actions[$d->action]))
   {
    $this->error("XML API ".$d->action." not supported");
    return false;
   }

   if(!isset($this->actions[$d->action][$d->extra]))
   {
    $this->error("XML API ".$d->action." ".$d->extra." not supported");
    return false;
   }

   if($this->actions[$d->action][$d->extra]===false)
   {
    $this->error("XML API ".$d->action." ".$d->extra." disabled");
    return false;
   }
   else //enabled
   {
    if(is_array($this->actions[$d->action][$d->extra])&&!in_array($d->module, $this->actions[$d->action][$d->extra]))
    {
     $this->error("XML API ".$d->module." ".$d->action." ".$d->extra." disabled");
     return false;
    }
    // else //all modules allowed
   }

   if(($action=="view"||$action=="tree"||$action=="search"))
    $submit=false;
   else if(($action=="add"||$action=="edit"||$action=="move"||$action=="delete"))
    $submit=true;
   else if(($action=="user"&&($extra=="login"||$extra=="logout"||$extra=="password")))
    $submit=true;
   else if(($action=="database"&&($extra=="info")))
    $submit=false;
   else
   {
    $this->error("Unsupported action: \"".$action."->".$extra."\"");
    return false;
   }

   $row=0;
   $show=Array();
   $sort=Array();

   foreach($subXML->getChildren("data") as $child)
   {
    switch($subXML->getName($child))
    {
     case "row":
     {
      $lastField=false;

      foreach($subXML->getChildren($child) as $child2)
      {
       if($subXML->getName($child2)!="field")
       {
        $this->error("Incorrect XML format: unknown tag: \"jas->data->row->".$subXML->getName($child2)."\"");
        return false;
       }

       if(!($field=$subXML->getAttribute($child2, "name")))
       {
        $this->error("Incorrect XML format: name attribute not set for tag: \"jas->data->row->".$field."\"");
        return false;
       }
       if(!$d->setValue($field, $subXML->getAttribute($child2, "value"), $row))
       {
        $this->error("Unable to set value for field: \"".$field."\"");
        return false;
       }

       $lastField=$field;
      }

      if($lastField&&!$d->setOperant($lastField, "OR", $row))
      {
       $this->error("Unable to set operant for field: \"".$field."\"");
       return false;
      }

      $row++;
      break;
     }
     case "show":
     {
      foreach($subXML->getChildren($child) as $child2)
      {
       if($subXML->getName($child2)!="field")
       {
        $this->error("Incorrect XML format: unknown tag: \"jas->data->show->".$subXML->getName($child2)."\"");
        return false;
       }

       if(!($field=$subXML->getAttribute($child2, "name")))
       {
        $this->error("Incorrect XML format: name attribute not set for tag: \"jas->data->show->".$field."\"");
        return false;
       }

       $show[]=$field;
      }

      break;
     }
     case "sort":
     {
      foreach($subXML->getChildren($child) as $child2)
      {
       if($subXML->getName($child2)!="field")
       {
        $this->error("Incorrect XML format: unknown tag: \"jas->data->sort->".$subXML->getName($child2)."\"");
        return false;
       }

       if(!($field=$subXML->getAttribute($child2, "name")))
       {
        $this->error("Incorrect XML format: name attribute not set for tag: \"jas->data->sort->".$field."\"");
        return false;
       }

       $sort[]=$field;
      }

      break;
     }
     default:
     {
      $this->error("Incorrect XML format: unknown tag: \"jas->data->".$subXML->getName($child)."\"");
      return false;
     }
    }
   }

   if($show)
    $d->setShowFields($show);
   if($sort)
    $d->sortOrder($sort);

   if($submit)
   {
    if(!$d->submit())
    {
     $this->error($d->e);
     return false;
    }
    else
//XXX: should return insert ID
    {
     $this->error($d->e);
     return true;
    }
   }

   if($d->action=="database"&&$d->extra=="info"&&$d->check("valuetype", "set", "info_updated")&&$d->getValue("info_updated")) //poll table for updates
   {
    session_write_close(); //don't write session data after this, this will open up the server for other requests instead of queuing them after this potentially long request

    $updates=false;
    $previousUpdate=DT::dateConvert($d->getValue("info_updated"), $d->field["info_updated"]["format"][$d->dataformat]["datetimeformat"]); //get previous update time from XML

    $timeout=ini_get("max_execution_time"); //polling timeout after max_execution_time
    $stoptime=$timeout+time();

    //poll till timeout occurs
    while(true)
    {
     if(!$d->get()) //get info
     {
      $this->error($d->e);
      return false;
     }

     $updated=DT::dateConvert($d->getValue("info_updated"), $d->field["info_updated"]["format"][$d->dataformat]["datetimeformat"]); //get current update time

     if($updated>$previousUpdate) //updated after previous update time
     {
      $updates=true; //set updated, returns new update time
      break;
     }

     if($stoptime<=time()) //timeout happend
      break; //stop polling

     usleep(0.1*1000000); //sleep .1 second before next poll
    }

    if(!$updates)
    {
     $this->e->add("No updates for: \"".$d->module."\"", "note", "Timeout: ".$timeout." second(s)");
     $this->error();
     return false;
    }
   }
   else
   {
    if(!$d->get(0, $count))
    {
     $this->error($d->e);
     return false;
    }
   }

   if($action=="user") //login / logout
   {
    $this->error($d->e);
    return true;
   }

   //main attributes
   $this->setAttribute("jas->data", "module", implode(":", $module));
   $this->setAttribute("jas->data", "action", $d->action);
   $this->setAttribute("jas->data", "extra", $d->extra);
   $this->setAttribute("jas->data", "format", $d->dataformat);

   if($count)
    $this->setAttribute("jas->data", "count", "true");
   else
    $this->setAttribute("jas->data", "count");


   //show fields
   $show=Array();
   if($d->action=="database"&&$d->extra=="info")
   {
    foreach($d->check("use", "info") as  $field)
     $show[]=$field;
   }
   else
   {
    foreach(array_keys($d->field) as $field)
     if($d->check("use", "show", $field)||$d->check("properties", "identifier", $field))
      if(!$d->check("properties", "control", $field))
       $show[]=$field;
   }

   if($show)
   {
    $tmp="\n";
    $sub=$this->getOuterXML("jas->data->show->field");
    foreach($show as $field)
    {
     $sub->setAttribute("field", "name", $field);
     $tmp.=$sub->output(false)."\n";
    }

    $this->set("jas->data->show", $tmp, true, false);
   }
   else
    $this->delete("jas->data->show");


   //sorting
   if($d->extra!="info"&&$d->extra!="item")
    $sort=$d->sortOrder;
   else
    $sort=Array();

   if($sort)
   {
    $tmp="\n";
    $sub=$this->getOuterXML("jas->data->sort->field");
    foreach($sort as $field)
    {
     $sub->setAttribute("field", "name", $field);
     $tmp.=$sub->output(false)."\n";
    }

    $this->set("jas->data->sort", $tmp, true, false);
   }
   else
    $this->delete("jas->data->sort");


   //data
   $tmp="\n";
   $row=$this->getOuterXML("jas->data->row");
   $sub=$row->getOuterXML("row->field");
   while($d->next())
   {
    $row->set("row", "\n", true, false);
    foreach($show as $field)
    {
     $sub->setAttribute("field", "name", $field);

     if($d->field[$field]["type"]=="filedata")
      $sub->setAttribute("field", "value", base64_encode($d->v[$field]));
     else if(is_array($d->v[$field]))
      $sub->setAttribute("field", "value", join(";", $d->v[$field]));
     else
      $sub->setAttribute("field", "value", $d->v[$field]);

     $row->addInsideAtEnd("row", $sub->output(false)."\n");
    }

    if($count&&$d->vc==1)
    {
     $sub->setAttribute("field", "name", "show_count");
     $sub->setAttribute("field", "value", $d->getValue("show_count"));
     $row->addInsideAtEnd("row", $sub->output(false)."\n");
    }

    $tmp.=$row->output(false);
   }

   $this->set("jas->data->row", $tmp, true, true); //replace original row tag

   return true;
  }

  function handle_modules_1_0(&$subXML)
  {
   if(!$this->modules)
   {
    $this->error("XML API modules list disabled");
    return false;
   }

   if(!$this->parseTemplate("modules"))
   {
    $this->error();
    return false;
   }

   $modules=Array();
   $allModules=modules();
   if($childs=$subXML->getChildren("modules"))
   {
    foreach($childs as $child) //filtering
    {
     if($subXML->getName($child)=="module")
     {
      $module=explode(":", $subXML->getAttribute($child, "name"));

      if(!in_array($module[0], $allModules))
      {
       $this->error("Module: \"".$module[0]."\" doesn't exist");
       return false;
      }

      $module=implode(":", $module);

      if(!in_array($module, $modules))
       $modules[]=$module;
     }
     else
     {
      $this->error("Incorrect XML format: unknown tag: \"jas->modules->".$subXML->getName($child)."\"");
      return false;
     }
    }
   }

   $tmp="\n";
   if(!$modules) //no modules requested, just list modules
   {
    $sub=new XML();
    $sub->parse("  <module name=\"\"/>"); //short tags

    foreach($allModules as $module)
    {
     $d=Data::cache($module, "view", "item", Array("skipCheckTables" => true));
     $sub->setAttribute("module", "name", $d->module);
     $tmp.=$sub->output(false)."\n";

     foreach(array_keys($d->reference) as $ref)
     {
      if($d->reference[$ref]["type"]=="*:*")
      {
       $sub->setAttribute("module", "name", $d->module.":".$d->reference[$ref]["data"]->module);
       $tmp.=$sub->output(false)."\n";
      }
     }
    }
   }
   else //get module definition for all request modules
   {
    $sub=$this->getOuterXML("jas->modules->module");

    foreach($modules as $module)
    {
     $sub->setAttribute("module", "name", $module);

     $module=explode(":", $module);
     $d=Data::cache($module[0], "view", "item", Array("skipCheckTables" => true));

     if(isset($module[1])) //*:* referenced module
     {
      if(!isset($d->reference[$module[1]]))
      {
       $this->error("Unknown referenced module: \"".$module[1]."\"");
       return false;
      }

      if($d->reference[$module[1]]["type"]!="*:*")
      {
       $this->error("Referenced module: \"".$module[1]."\" isn't \"*:*\" type");
       return false;
      }

      $d=&$d->reference[$module[1]]["data"];
     }

     $sub->setAttribute("module", "title", $d->title);

     $sub2=$sub->getOuterXML("module->fields->field");
     $sub->set("module->fields", "\n", true, false);

     foreach($d->field as $k => $v)
     {
      if(!$d->check("properties", "control", $k))
      {
       $sub2->setAttribute("field", "name", $k);
       $sub2->setAttribute("field", "type", $v["type"]);
       $sub2->setAttribute("field", "use", $v["use"]);
       $sub2->setAttribute("field", "properties", $v["properties"]);
       $sub2->setAttribute("field", "min", $v["min"]);
       $sub2->setAttribute("field", "max", $v["max"]);

       if(is_array($v["defaultvalue"]))
        $sub2->setAttribute("field", "default", implode(";", $v["defaultvalue"]));
       else
        $sub2->setAttribute("field", "default", $v["defaultvalue"]);

       $sub2->setAttribute("field", "title", $v["title"]);
       $sub2->setAttribute("field", "text", $v["text"]);
       $sub->addInsideAtEnd("module->fields", "    ".$sub2->output(false)."\n");
       $sub2->reinit();
      }
     }

     $sub2=$sub->getOuterXML("module->references->reference");
     $sub->set("module->references", "\n", true, false);

     foreach($d->reference as $k => $v)
     {
      $sub2->setAttribute("reference", "name", $k);
      $sub2->setAttribute("reference", "module", $v["module"]);
      $sub2->setAttribute("reference", "from", $v["from"]);
      $sub2->setAttribute("reference", "to", $v["to"]);
      $sub2->setAttribute("reference", "type", $v["type"]);
      $sub->addInsideAtEnd("module->references", "    ".$sub2->output(false)."\n");
      $sub2->reinit();
     }

     $tmp.=$sub->output(false)."\n";
     $sub->reinit();
    }
   }

   $this->set("jas->modules->module", $tmp, true, true); //replace original module tag

   return true;
  }

  function handle_projects_1_0(&$subXML)
  {
   global $_jses, $_jseskey, $_config, $_path;

   if(!$this->projects)
   {
    $this->error("XML API projects list disabled");
    return false;
   }

   if(!$this->parseTemplate("projects"))
   {
    $this->error();
    return false;
   }

   $allProjects=projects();
   if($childs=$subXML->getChildren("projects"))
   {
    foreach($childs as $child) //filtering
    {
     if($subXML->getName($child)=="project")
     {
      $project=$subXML->getAttribute($child, "name");
      if(in_array($project, $allProjects))
      {
       $_jses=Array();
       $_jses["project"]=$project;
       unset($_path["project"]);
       require($_path["root"]."include.php");
      }
      else
      {
       $this->error("Project: \"".$project."\" doesn't exist");
       return false;
      }
     }
     else
     {
      $this->error("Incorrect XML format: unknown tag: \"jas->projects->".$subXML->getName($child)."\"");
      return false;
     }
    }
   }

   $tmp="\n";
   $sub=$this->getOuterXML("jas->projects->project");
   foreach(projects() as $project)
   {
    $sub->setAttribute("project", "name", $project);

    if($project==$_jses["project"])
     $sub->setAttribute("project", "current", "yes");
    else
     $sub->setAttribute("project", "current");

    $tmp.=$sub->output(false)."\n";
    $sub->reinit();
   }

   $this->set("jas->projects->project", $tmp, true, true); //replace original project tag

   return true;
  }

  function handle_language_1_0(&$subXML)
  {
   global $_config;

   if(!$this->language)
   {
    $this->error("XML API language disabled");
    return false;
   }

   if(!$this->parseTemplate("jas")) //use empty template, contents will come from language xml files
   {
    $this->error();
    return false;
   }

   if(!($set=$subXML->getAttribute(0, "set")))
   {
    $this->error("Incorrect XML format: set attribute not set for tag: \"jas->language\"");
    return false;
   }

   if(!($language=$subXML->getAttribute(0, "language")))
    $language=$_config["language"];

   $l=new Language();

   if($language==$l->default)
   {
    $this->e->add("Not getting set \"".$set."\" for default language \"".$language."\"", "note");
    $this->error();
    return false;
   }

   if(!($file=$l->getFilename($set, $language)))
   {
    $this->error("Unable to get language file for set \"".$set."\" and language \"".$language."\"");
    return false;
   }

   $xml=new XML();
   if(!$xml->parseFile($file))
   {
    $this->error("Unable to read language XML for set \"".$set."\" and language \"".$language."\"");
    return false;
   }

   while($key=$xml->search("language"))
   {
    if($xml->getAttribute($key, "translated")==$language)
    {
     $xml->setAttribute($key, "set", $set);
     $this->addInsideAtEnd("jas", $xml->getOuter($key)."\n");
    }
   }

   return true;
  }

 }


 class XMLiClient extends XMLi
 {
  function XMLiClient($url, $session=false)
  {
   XMLi::XMLi(false);
   //$this->clear(); //cleared by XML::XML();

   $this->url($url);
   $this->session=$session;
  }


  function clear()
  {
   parent::clear();

   $this->url=false;

   $this->session=false;
   $this->sessionVar="PHPSESSID";
  }



  function url($url)
  {
   if(!preg_match("!^(http|https)://([a-z0-9\-\.]+)/?(.*)!", $url, $m))
   {
    $this->e->add("Incorrect URL: \"".$url."\"", "error");
    return false;
   }

   if(!preg_match("![a-z0-9]+\.[a-z0-9+]+!", $m[3])) //no file in url
    $url.="xml.php"; //add default file

   $this->url=$url;
  }


  function request()
  {
   if(!$this->url)
   {
    $this->e->add("XMLi Request failed: no URL set", "error");
    return true;
   }

   $this->e->clear(); //clear errors

   $http=new HTTP();
   $http->e->logging=$this->e->logging; //overwrite logging

   $http->addVar("xml", $this->xml);

   if($this->session)
    $http->addCookie($this->sessionVar, $this->session);

   if(!$http->request($this->url, "POST"))
   {
    $this->e->add($http->e);
    return false;
   }

   if(!$this->session)
   {
    if(!($cookies=$http->searchHeader("Set-Cookie")))
    {
     $this->e->add("XMLi Request failed: cookies not found", "error", "XMLi URL: \"". $this->url."\"");
     return true;
    }

    if(!preg_match("/".$this->sessionVar."=([^;]+)/", $cookies, $m))
    {
     $this->e->add("XMLi Request failed: session cookie not found", "error", "XMLi URL: \"". $this->url."\"");
     return true;
    }

    $this->session=$m[1];
   }

   if(!$this->parse($http->body))
    return false;

   if($this->getKey("jas")!==false&&$this->getKey("jas->errors")!==false) //errors found
   {
    $c=0;
    while($this->getKey("jas->errors->error[".$c."]")!==false) //store all in this->e
    {
     $this->e->add($this->getAttribute("jas->errors->error[".$c."]", "text"), $this->getAttribute("jas->errors->error[".$c."]", "level"), "XMLi URL: \"". $this->url."\"");
     $c++;
    }

    if($this->e->any("fatal")) //return false on fatal errors
     return false;
   }

   return true;
  }


  function data(&$d, $count=false)
  {
   $func="request_data_".strtr($this->xmlJasVersion, ".", "_");

   if(!$this->$func($d, $count))
    return false;

   return true;
  }

  function request_data_1_0(&$d, $count)
  {
   if(!$this->parseTemplate("data"))
    return false;

   //main attributes
   if($d->initializingModule)
    $this->setAttribute("jas->data", "module", $d->initializingModule.":".$d->module);
   else
    $this->setAttribute("jas->data", "module", $d->module);

   $this->setAttribute("jas->data", "action", $d->action);
   $this->setAttribute("jas->data", "extra", $d->extra);
   $this->setAttribute("jas->data", "format", "sql");

   if($count)
    $this->setAttribute("jas->data", "count", "true");
   else
    $this->setAttribute("jas->data", "count");


   //show fields
   $show=Array();
   if(
    $d->action!="add"&&
    $d->action!="edit"&&
    $d->action!="delete"&&
    (!($d->action=="database"&&$d->extra=="info"))
   )
   {
    foreach(array_keys($d->field) as $field)
     if($d->check("use", "show", $field)||$d->check("properties", "identifier", $field))
      if(!$d->check("properties", "control", $field))
       $show[]=$field;
   }

   if($show)
   {
    $tmp="\n";
    $sub=$this->getOuterXML("jas->data->show->field");
    foreach($show as $field)
    {
     $sub->setAttribute("field", "name", $field);
     $tmp.=$sub->output(false)."\n";
    }

    $this->set("jas->data->show", $tmp, true, false);
   }
   else
    $this->delete("jas->data->show");


   //sorting
   if($d->extra!="info"&&$d->extra!="item")
    $sort=$d->sortOrder;
   else
    $sort=Array();

   if($sort)
   {
    $tmp="\n";
    $sub=$this->getOuterXML("jas->data->sort->field");
    foreach($sort as $field)
    {
     $sub->setAttribute("field", "name", $field);
     $tmp.=$sub->output(false)."\n";
    }

    $this->set("jas->data->sort", $tmp, true, false);
   }
   else
    $this->delete("jas->data->sort");


   //data
   $fields=Array();
   switch($d->action)
   {
    case "view":
    {
     switch($d->extra)
     {
      case "list":
      {
       if(($refer=$d->getRightsReference())&&($field=$d->reference[$refer]["from"]))
        $fields[]=$field;

       break;
      }

      case "all":
      {
       foreach($d->check("valuetype", "set") as $field)
        $fields[]=$field;

       break;
      }

      case "item":
      {
       foreach($d->check("properties", "identifier") as $field)
         if($d->check("valuetype", "set", $field)||$d->check("valuetype", "request", $field)||$d->check("valuetype", "get", $field))
          $fields[]=$field;

       break;
      }

      default: break;
     }

     break;
    }

    case "search":
    {
     switch($d->extra)
     {
      case "title":
      case "field":
      case "show":
      case "full":
      {
       $fields[]="search";

       if($d->extra=="field")
        $fields[]="searchFields";

       foreach($d->check("valuetype", "set") as $field)
        if(strpos($field, "aggregation_")===0)
         $fields[]=$field;

       break;
      }
      default: break;
     }

     break;
    }

    case "tree":
    {
     switch($d->extra)
     {
      case "up":
      case "down":
      {
       if(($refer=$d->getRightsReference())&&($field=$d->reference[$refer]["from"]))
        $fields[]=$field;

       if($d->field["treedepth"]["valuetype"]=="set")
        $fields[]="treedepth";

       break;
      }
     }

     break;
    }


    case "add":
    {
     foreach($d->check("use", "sql") as $field)
      if(!$d->check("properties", "identifier", $field)||$d->dataformat=="sql"||$d->extends) //allow id
       if($d->field[$field]["module"]==$d->module)
        if($field!="r")
         $fields[]=$field;

     break;
    }
    case "edit":
    {
     foreach($d->check("use", "sql") as $field)
      if($d->field[$field]["module"]==$d->module)
       if(!$d->check("valuetype", "none", $field))
        if($field!="r")
         $fields[]=$field;

     break;
    }
    case "delete":
    {
     foreach($d->check("properties", "identifier") as $field)
      if($d->check("use", "sql", $field))
       $fields[]=$field;

     break;
    }

    case "user":
    {
     switch($d->extra)
     {
      case "login":
      {
       foreach($d->check("valuetype", "set") as $field)
        if($d->check("use", "sql", $field))
         $fields[]=$field;

       break;
      }
      case "logout":
      {
       break;
      }

      default: break;
     }

     break;
    }

    default: break;
   }


   if(!$count&&$d->action!="tree")
   {
    //add show from and total (i.e. limit) fields if needed
    if($d->getValue("show_from"))
     $fields[]="show_from";

    if($d->getValue("show_total"))
     $fields[]="show_total";
   }

   $tmp="\n";
   if($fields)
   {
    $row=$this->getOuterXML("jas->data->row");
    $sub=$row->getOuterXML("row->field");
    for($c=0;$c<$d->vmax;$c++)
    {
     $row->set("row", "\n", true, false);
     foreach($fields as $field)
     {
      $sub->setAttribute("field", "name", $field);
      $value=$d->sqlValue($field, $c);

      if(is_array($value))
       $sub->setAttribute("field", "value", join(";", $value));
      else
       $sub->setAttribute("field", "value", $value);

      $row->addInsideAtEnd("row", $sub->output(false)."\n");
     }

     $tmp.=$row->output(false);
    }
   }

   $this->set("jas->data->row", $tmp, true, true); //replace original row tag

   if(!$this->request())
    return false;

   //set data
   $c=0;
   $d->clearValues();
   $d->dataformat="sql"; //SQL values
   while($key=$this->search("row"))
   {
    $sub=$this->getOuterXML($key);

    while($key2=$sub->search("field"))
    {
     if($d->field[$sub->getAttribute($key2, "name")]["type"]=="filedata")
      $d->field[$sub->getAttribute($key2, "name")]["value"][$c]=base64_decode($sub->getAttribute($key2, "value"));
     else
      $d->field[$sub->getAttribute($key2, "name")]["value"][$c]=$sub->getAttribute($key2, "value");

     $d->field[$sub->getAttribute($key2, "name")]["valuetype"]="get";
    }

    $c++;
   }

   $d->vmax=$c;

   return true;
  }


  function setProject($project)
  {
   $func="request_project_".strtr($this->xmlJasVersion, ".", "_");

   if(!$this->$func($project))
    return false;

   return true;
  }

  function request_project_1_0($project)
  {
   if(!$this->parseTemplate("projects"))
    return false;

   $this->setAttribute("jas->projects->project", "name", $project); //set name
   $this->setAttribute("jas->projects->project", "current"); //remove current

   if(!$this->request())
    return false;

   //validate if project is actually set to current
   while($key=$this->search("project"))
   {
    if($this->getAttribute($key, "name")==$project)
    {
     if($this->getAttribute($key, "current")!="yes")
     {
      $this->e->add("Unable to set project to: \"".$project."\"", "error", "XMLi URL: \"". $this->url."\"");
      return false;
     }

     break;
    }
   }

   return true;
  }


  function getModule($module)
  {
   $func="request_module_".strtr($this->xmlJasVersion, ".", "_");

   if(!$this->$func($module))
    return false;

   return true;
  }

  function request_module_1_0($module)
  {
   if(!$this->parseTemplate("modules"))
    return false;

   $this->setAttribute("jas->modules->module", "name", $module); //set name
   $this->setAttribute("jas->modules->module", "title"); //remove title
   $this->delete("jas->modules->module->fields"); //remove fields
   $this->delete("jas->modules->module->references"); //remove fields

   if(!$this->request())
    return false;

   //validate if module definition is the same
   $success=true;
   $remoteFields=Array();
   $remoteReferences=Array();

   $module=explode(":", $module);
   $d=Data::cache($module[0], "view", "all", Array("skipCheckTables" => true));

   if(isset($module[1])) //*:* referenced module
   {
    if(!isset($d->reference[$module[1]]))
    {
     $this->e->add("Unknown referenced module: \"".$module[1]."\"", "error");
     return false;
    }

    if($d->reference[$module[1]]["type"]!="*:*")
    {
     $this->e->add("Referenced module: \"".$module[1]."\" isn't \"*:*\" type", "error");
     return false;
    }

    $d=&$d->reference[$module[1]]["data"];
   }

   //check fields
   while($key=$this->search("field"))
   {
    $field=$this->getAttribute($key, "name");
    $remoteFields[]=$field;

    if(!isset($d->field[$field]))
    {
     $this->e->add("Module: \"".$d->module."\" definitions mismatch: local field: \"".$field."\" not found", "error", "XMLi URL: \"". $this->url."\"");
     $success=false;
    }
    else
    {
     if($d->field[$field]["type"]!=$this->getAttribute($key, "type"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: field: \"".$field."\" local type: \"".$d->field[$field]["type"]."\" doesn't match \"".$this->getAttribute($key, "type")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->field[$field]["use"]!=$this->getAttribute($key, "use"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: field: \"".$field."\" local use: \"".$d->field[$field]["use"]."\" doesn't match \"".$this->getAttribute($key, "use")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->field[$field]["properties"]!=$this->getAttribute($key, "properties"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: field: \"".$field."\" local properties: \"".$d->field[$field]["properties"]."\" doesn't match \"".$this->getAttribute($key, "properties")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->field[$field]["min"]!=$this->getAttribute($key, "min"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: field: \"".$field."\" local minimum: \"".$d->field[$field]["min"]."\" doesn't match \"".$this->getAttribute($key, "min")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->field[$field]["max"]!=$this->getAttribute($key, "max"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: field: \"".$field."\" local maximum: \"".$d->field[$field]["max"]."\" doesn't match \"".$this->getAttribute($key, "max")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     // Skip default value check because of potential dynamic values
     //if($d->field[$field]["defaultvalue"]!=$this->getAttribute($key, "default"))
     //{
     // $this->e->add("Module: \"".$d->module."\" definitions mismatch: field: \"".$field."\" local default value: \"".$d->field[$field]["defaultvalue"]."\" doesn't match \"".$this->getAttribute($key, "default")."\"", "error", "XMLi URL: \"". $this->url."\"");
     // $success=false;
     //}
    }
   }

   foreach(array_keys($d->field) as $field)
   {
    if(!$d->check("properties", "control", $field))
    {
     if(!in_array($field, $remoteFields))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: remote field: \"".$field."\" not found", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
    }
   }

   //check references
   while($key=$this->search("reference"))
   {
    $reference=$this->getAttribute($key, "name");
    $remoteReferences[]=$reference;

    if(!isset($d->reference[$reference]))
    {
     $this->e->add("Module: \"".$d->module."\" definitions mismatch: local reference: \"".$reference."\" not found", "error", "XMLi URL: \"". $this->url."\"");
     $success=false;
    }
    else
    {
     if($d->reference[$reference]["from"]!=$this->getAttribute($key, "from"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: reference: \"".$reference."\" local from: \"".$d->reference[$reference]["from"]."\" doesn't match \"".$this->getAttribute($key, "from")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->reference[$reference]["module"]!=$this->getAttribute($key, "module"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: reference: \"".$reference."\" local module: \"".$d->reference[$reference]["module"]."\" doesn't match \"".$this->getAttribute($key, "module")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->reference[$reference]["module"]!=$this->getAttribute($key, "module"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: reference: \"".$reference."\" local module: \"".$d->reference[$reference]["module"]."\" doesn't match \"".$this->getAttribute($key, "module")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->reference[$reference]["to"]!=$this->getAttribute($key, "to"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: reference: \"".$reference."\" local to: \"".$d->reference[$reference]["to"]."\" doesn't match \"".$this->getAttribute($key, "to")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
     if($d->reference[$reference]["type"]!=$this->getAttribute($key, "type"))
     {
      $this->e->add("Module: \"".$d->module."\" definitions mismatch: reference: \"".$reference."\" local type: \"".$d->reference[$reference]["type"]."\" doesn't match \"".$this->getAttribute($key, "type")."\"", "error", "XMLi URL: \"". $this->url."\"");
      $success=false;
     }
    }
   }

   foreach(array_keys($d->reference) as $reference)
   {
    if(!in_array($reference, $remoteFields))
    {
     $this->e->add("Module: \"".$d->module."\" definitions mismatch: remote reference: \"".$reference."\" not found", "error", "XMLi URL: \"". $this->url."\"");
     $success=false;
    }
   }

   return $success;
  }

 }
?>
