<?

 /*****************************************************************************
 * Filename: project/jas/include/modules/categories.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["modules"]."categories.php");

 class JAS_Module_categories extends Module_categories
 {
  function init(&$data)
  {
   parent::init($data);

   $this->data->addField("page", "regexp", "show", false, false, 25, false, "Page");
   $this->data->regexps("page", "/^([a-z0-9_]+)$/");

   $this->data->addField("content", "enum", "all", "reload", false, false, "None", "Content");
   $this->data->enumOptions("content", "None", "Directory", "Plugin", "URL");

   $this->data->addField("directory", "localdirectory", "all", "recursive", false, 255, false, "Directory", "");

   $this->data->addField("url", "url", "all", "", false, 255, "", "URL");
   $this->data->addField("urlrel", "text", "all", "", false, 25, "", "URL rel");
   $this->data->enumOptions("urlrel", "", "external", "popup", "popup 800x600", "popup 1024x768", "popup 1280x1024", "popup 1920x1080");


   $plugins=Array("");
   foreach(plugins() as $plugin)
   {
    $plugin=new Plugin($plugin);
    if($plugin->getStatus())
     $plugins[]=$plugin->name;
   }
   unset($plugin);

   $this->data->addField("plugin", "plugin", "all", "reload", false, 25, false, "Plugin");
   $this->data->listOptions("plugin", $plugins);

   $this->data->addField("pluginDisplay", "enum", "all", "reload", false, false, "Frontend", "Display plugin");
   $this->data->enumOptions("pluginDisplay", "Frontend", "Backend");

   $this->data->addField("pluginConfigControl", "nr", "all", "", 0, false, false, "Control configuration");
   $this->data->addField("pluginConfigMain", "nr", "all", "", 0, false, false, "Main configuration");


   $this->data->group("Modules", "module", "moduleLock");
   $this->data->group("Content", "content", "directory", "plugin", "pluginDisplay", "pluginConfigControl", "pluginConfigMain", "url", "urlrel");
  }


  function treePreGetErrorCheck(){return $this->setPageFiltering();}
  function viewPreGetErrorCheck(){return $this->setPageFiltering();}

  function treePostGet(){return $this->setPageValues();}
  function viewPostGet(){return $this->setPageValues();}

  function postInit()
  {
   global $_path;

   $this->data->setFileDir("directory", $_path["project"]["include"]."pages/");

   if(!parent::postInit())
    return false;

   return $this->customizeFields();
  }
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   switch($this->data->getValue("content"))
   {
    case "None":
    {
     $this->data->field["directory"]["properties"].=";hidden";

     $this->data->field["plugin"]["properties"].=";hidden";
     $this->data->field["pluginDisplay"]["properties"].=";hidden";
     $this->data->field["pluginConfigControl"]["properties"].=";hidden";
     $this->data->field["pluginConfigMain"]["properties"].=";hidden";

     $this->data->field["url"]["properties"].=";hidden";
     $this->data->field["urlrel"]["properties"].=";hidden";

     break;
    }
    case "Directory":
    {
     $this->data->resetField("directory");
     $this->data->field["directory"]["min"]=1;

     $this->data->field["plugin"]["properties"].=";hidden";
     $this->data->field["pluginDisplay"]["properties"].=";hidden";
     $this->data->field["pluginConfigControl"]["properties"].=";hidden";
     $this->data->field["pluginConfigMain"]["properties"].=";hidden";

     $this->data->field["url"]["properties"].=";hidden";
     $this->data->field["urlrel"]["properties"].=";hidden";
     break;
    }
    case "Plugin":
    {
     $this->data->resetField("plugin");
     $this->data->field["plugin"]["min"]=1;
     $this->data->resetField("pluginDisplay");

     $this->data->field["directory"]["properties"].=";hidden";

     $this->data->field["pluginConfigControl"]["properties"].=";hidden";
     $this->data->field["pluginConfigMain"]["properties"].=";hidden";

     $this->data->field["url"]["properties"].=";hidden";
     $this->data->field["urlrel"]["properties"].=";hidden";

     if($this->data->getValue("pluginDisplay")=="Frontend"&&$this->data->checkField("plugin")&&($plugin=$this->data->getValue("plugin")))
     {
      if(in_array($plugin."_pluginConfigs", modules("plugins", $plugin)))
      {
       $this->data->resetField("pluginConfigControl");
       $this->data->resetField("pluginConfigMain");

       $this->data->addReference("pluginConfigControl", $plugin."_pluginConfigs", "nr", "*:0");
       $this->data->addReference("pluginConfigMain", $plugin."_pluginConfigs", "nr", "*:0");
      }
     }

     break;
    }
    case "URL":
    {
     $this->data->resetField("url");
     $this->data->field["url"]["min"]=1;
     $this->data->resetField("urlrel");

     $this->data->field["directory"]["properties"].=";hidden";

     $this->data->field["plugin"]["properties"].=";hidden";
     $this->data->field["pluginDisplay"]["properties"].=";hidden";
     $this->data->field["pluginConfigControl"]["properties"].=";hidden";
     $this->data->field["pluginConfigMain"]["properties"].=";hidden";

     break;
    }
    default: break;
   }

   return true;
  }

  function setPageFiltering()
  {
   if($this->data->check("use", "show", "page")&&!$this->data->check("use", "show", "name")) //also get name field if page is show field
    $this->data->field["name"]["use"].=";show";

   if($this->data->check("valuetype", "set", "page")&&!$this->data->check("valuetype", "set", "name")) //use page field for filtering on name
   {
    for($c=0;$c<$this->data->vmax;$c++)
    {
     if($name=$this->data->getValue("page", $c))
     {
      if(!is_numeric($name))
      {
       $name=str_replace("_", " ", $name);
       $this->data->setValue("name", $name,  $c);
      }
      else
       $this->data->setValue("nr", $name,  $c);
     }
    }
   }

   return true;
  }

  function setPageValues()
  {
   if($this->data->check("use", "show", "page"))
   {
    for($c=0;$c<$this->data->vmax;$c++)
    {
     if($name=$this->data->getValue("name", $c))
     {
      $name=strtolower(str_replace(" ", "_", $name));
      $this->data->setValue("page", $name,  $c);
     }
     else
      $this->data->setValue("page", $this->data->getValue("nr", $c),  $c);
    }
   }

   return true;
  }
 }
?>
