

 /*****************************************************************************
 * Filename: script/passwords.js
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


function Passwords()
{
 this.error=error;

 this.setLength=setLength;
 this.setLength(10);

 this.setCharacters=setCharacters;
 this.setCharacters("abcdefghijklmnopqrstuvwxyz", "ABCDEFGHIJKLMNOPQRSTUVWXYZ", "0123456789", "~!@#$%^&*()_+`-=[];',./{}:\"<>?\\|");

 this.setDistribution=setDistribution;
 this.setDistribution(50, 20, 20, 10);

 this.setMimimum=setMimimum;
 this.setMimimum(4, 3, 2, 1);

 this.generate=generate;

//generate password
 function generate()
 {
  var length=this.length;

  //detect whether any characters types are empty but required in distribution
  for(var key in this.characters)
  {
   if(!this.characters[key].length&&this.distribution[key])
   {
    this.error("Unable to generate password, no "+key+" characters defined");
    return false;
   }
  }

  //determine number of characters to use for each type
  var total=0;
  var keys=Array();
  var distribution={};
  if(this.distributionType=="percentage")
  {
   for(var key in this.distribution)
   {
    if(this.distribution[key])
    {
     distribution[key]=Math.round(this.distribution[key]*length);
     total+=distribution[key];
     keys.push(key);
    }
   }

   if(total<length) //too little characters in distribution
    for(var c=total;c<length;c++) //add characters evenly to distribution
     distribution[keys[c%keys.length]]++;
  }
  else //distribution type is minimum
  {
   for(var key in this.distribution)
   {
    if(this.distribution[key])
    {
     distribution[key]=this.distribution[key];
     total+=distribution[key];
     keys.push(key);
    }
   }

   if(total<length) //too little characters in distribution
   {
    for(var c=total;c<length;c++) //add characters to lower case
     distribution[keys[0]]++;
   }
   else if(total>length) //length smaller than minimum requirements
    length=total;
  }

  //generate password
  var password="";
  for(var c=0;c<length;c++)
  {
   //determine key
   var key="";
   while(true)
   {
    //get "weighted" random key
    var band=0;
    var current=Math.random()*length; //random entry
    for(var key2 in distribution) //determine in which distribution band the current value falls
    {
     band+=distribution[key2];

     if(current<band) //within distribution band
     {
      key=key2;
      break;
     }
    }

    if(distribution[key]) //characters left in distribution for current key
    {
     distribution[key]--; //decrease characters left in distribution
     break; //use current key
    }
   }

   var chr=Math.floor(Math.random()*this.characters[key].length); //get random character from current type
   password+=this.characters[key].charAt(chr); //add character to password
  }

  return password;
 }

//set password length
 function setLength(length)
 {
  this.length=length;
  return true;
 }

//set allowed characters for each type
 function setCharacters(lower, upper, number, special)
 {
  this.characters={
   lower: lower,
   upper: upper,
   number: number,
   special: special
  };

  return true;
 }

//set characters distribution for each type, must be 100%
 function setDistribution(lower, upper, number, special)
 {
  if(lower+upper+number+special>100)
  {
   this.error("Total distribution exceeds 100%");
   return false;
  }
  else if(lower+upper+number+special<100)
  {
   this.error("Total distribution is less than 100%");
   return false;
  }

  //ensure percentage distribution type is used
  this.distributionType="percentage";

  //convert to float (/100)
  this.distribution={
   lower: lower/100,
   upper: upper/100,
   number: number/100,
   special: special/100
  };

  return true;
 }

//set minimum characters for each type, must be less or equal to length
 function setMimimum(lower, upper, number, special)
 {
  if(lower+upper+number+special<1)
  {
   this.error("Total minimum password length is 0");
   return false;
  }

  //ensure minimum distribution type is used
  this.distributionType="minimum";

  //set minimum
  this.distribution={
   lower: lower,
   upper: upper,
   number: number,
   special: special
  };

  return true;
 }

//error
 function error(error)
 {
  alert(error);
  return true;
 }
}
