<?

 /*****************************************************************************
 * Filename: include/data.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2013 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/storage.php");
 require_once($_path["include"]."error.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."data.base.php");
 require_once($_path["include"]."language.php");
 require_once($_path["include"]."captcha.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."fs.php");

 require_once($_path["include"]."modules/empty.php");

/******************************************************************
 Content Management System Class

 functions:
 + Data:             Contructor
 + addField:         Adds a new option for the form and SQL query
 + submit:           Executes SQL function
 + form:             Returns Data form (HTML)
 + categorySelector: Returns category selecter pulldown (HTML)

******************************************************************/

 $_modules=Array();

 class Data extends BaseData
 {

/******************************************************************
 cache
 Get a cached data object

 - see Data constructor for arguments
 - register: which register to use, if set the returned object is the actual object instead of a copy

 returns: cached (copy) of the data object
******************************************************************/

  function cache($module=false, $action=false, $extra=false, $options=false, $register=false)
  {
   global $_modules, $_jses, $_config;

   if(isset($_config["cache"])&&isset($_config["cache"]["memory"])&&$_config["cache"]["memory"]&&$_config["cache"]["memoryCacheData"]) //memory cache enabled
   {
    if(!isset($_modules["_memory"])) //memory cache object not initialized
    {
     $_modules["_memory"]=new Cache("memory"); //load cache object

     if($_modules["_memory"]->e->any("fatal")) //fatal problem
      $_modules["_memory"]=false; //no memory cache
    }
   }

   if(!isset($_modules[$_jses["project"]]))
    $_modules[$_jses["project"]]=Array();

   $cache=&$_modules[$_jses["project"]];

   if($options===false)
    $options=Array();

   if(!$module)
   {
    if(isset($_REQUEST["m"])&&$_REQUEST["m"]!="")
     $module=$_REQUEST["m"];
    else
     $module="categories";
   }

   if($register) //use specific register
   {
    if(!isset($cache["_register"]))
     $cache["_register"]=Array();

    if(!isset($cache["_register"][$register]))
     $cache["_register"][$register]=Array();

    if(!isset($cache["_register"][$register][$module]))
     return $cache["_register"][$register][$module]=new Data($module, $action, $extra, $options);

    $d=&$cache["_register"][$register][$module];
   }
   else if($action=="reference") //don't cache references
    return new Data($module, $action, $extra, $options);
   else //cached
   {
    if(!isset($cache[$module])) //not cached
    {
     if(
      isset($_config["cache"])&&isset($_config["cache"]["memory"])&&$_config["cache"]["memory"]&&$_config["cache"]["memoryCacheData"]&&$_modules["_memory"]&& //memory cache enabled
      ($_modules["_memory"]->exists("DATA::CACHE(".$_jses["project"].",".$module.",".(isset($options["db"])?$options["db"]:"default").")", false))&& //cached in memory
      ($d=$_modules["_memory"]->read("DATA::CACHE(".$_jses["project"].",".$module.",".(isset($options["db"])?$options["db"]:"default").")")) //get from memory
     )
     {
      $cache[$module]=&$d; //cache persistent object
      unset($d);
     }
     else //no copy in memory
     {
      $tmpOptions=array_merge($options, Array("skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true)); //skip checkTables+setRequest+postInit (done below)

      if(isset($tmpOptions["request"])) //init cached version uses default requestVar
       unset($tmpOptions["request"]);

      $cache[$module]=new Data($module, $action, $extra, $tmpOptions); //cache new object
      unset($tmpOptions);

      if(
       isset($_config["cache"])&&isset($_config["cache"]["memory"])&&$_config["cache"]["memory"]&&$_config["cache"]["memoryCacheData"]&& //memory cache enabled
       $_modules["_memory"]&& //cache object loaded
       ($module!="log")
      )
       $_modules["_memory"]->save("DATA::CACHE(".$_jses["project"].",".$module.",".(isset($options["db"])?$options["db"]:"default").")", $cache[$module]); //save to cache
     }
    }

    $d=$cache[$module]->copy(); //make copy of cached object
   }

   //set options
   $d->setOptions($options);

   //set action/extra
   $d->setAction($action);
   $d->setExtra($extra);

   //check database tables if skipCheckTables isn't set
   if(!isset($options["skipCheckTables"])||!$options["skipCheckTables"])
    $d->checkTables(true, true); //also check extended modules and (initialized) references

   //set request values if skipSetRequest isn't set
   if(!isset($options["skipSetRequest"])||!$options["skipSetRequest"])
    $d->setRequestValues(true, true); //also set extended modules and (initialized) references

   //call post init if skipPostInit isn't set
   if(!isset($options["skipPostInit"])||!$options["skipPostInit"])
    $d->postInit(true, true); //also run post init for extended modules and (initialized) references

   return $d;
  }


/******************************************************************
 Data
 Create new data object

 - module: module to initialize
 - action: action
 - extra: extra action option
 - options: additional options:
   - format: data format to use: html, form, text, sql
   - request: request variable to use for this instance
   - db: database to use, can be a file link (xml)
   - skipCheckTables: skip check database tables
   - skipSetRequest: skip setting request values
   - skipPostInit: skip postInit call

 returns: data object
******************************************************************/

  function Data($module=false, $action=false, $extra=false, $options=false)
  {
   global $_config;

   $this->init(); //initializes a bunch of stuff

   //defaults set in setOptions
   $this->dataformat=false; //set to sql for non-conversion
   $this->requestVar=false;
   $this->db=false;

   if($options===false)
    $options=Array();

   $this->setOptions($options);

   if($action=="reference"&&$extra=="check") //pre-set action+extra for reference checking (required for addReference)
   {
    $this->action=$action;
    $this->extra=$extra;
   }

   if(!$this->setModule($module))
    return;

   if(!$this->setAction($action))
    return;

   if(!$this->setExtra($extra))
    return;

   //check database tables if skipCheckTables isn't set
   if(!isset($options["skipCheckTables"])||!$options["skipCheckTables"])
    $this->checkTables(true, true); //also check extended modules and (initialized) references

   //call set request values if skipSetRequest isn't set
   if(!isset($options["skipSetRequest"])||!$options["skipSetRequest"])
    $this->setRequestValues(true, true); //also set extended modules and (initialized) references

   //call post init if skipPostInit isn't set
   if(!isset($options["skipPostInit"])||!$options["skipPostInit"])
    $this->postInit(true, true); //also run post init for extended modules and (initialized) references
  }

  function clear()
  {
   $this->title=false;

   $this->field=Array();
   $this->fieldGroups=Array();

   $this->reference=Array();
   $this->referenceLevel=0;
   $this->initializingModule=false;

   $this->index=Array();
   $this->unique=Array();
   $this->fulltextIndex=Array();
   $this->regexp=Array();

   $this->sortOrder=Array();
   $this->listOptions=Array();

   $this->module=false;
   $this->moduleObject=false;
   $this->moduleClassName=false;

   $this->plugin=false;

   $this->vc=0;
   $this->vmax=0;

   $this->t=Array(); //field titles
   $this->v=Array(); //current value
   $this->r=$this->rights2bin(0); //current rights
  }


  function setOptions($options)
  {
   //set defaults
   if(!isset($options["format"])) //no special data format
    if($this->dataformat!="html") //initial object has special data format
     $this->dataformat="html"; //set to default data format

   if(!isset($options["request"])) //no special requestVar
    if($this->requestVar) //initial object has special requestVar
     $this->requestVar=false; //set to no requestVar

   if(!isset($options["db"])) //no special db
    if($this->db!="default") //initial object has special db
     $this->db="default"; //set to default db

   //Set options
   if(!is_array($options))
   {
    $this->e->add("Options argument should be an array", "system");
    return false;
   }
   else
   {
    foreach($options as $key => $value)
    {
     switch($key)
     {
      case "format":
      {
       if(!in_array($value, Array("html", "text", "form", "sql")))
        $this->e->add("Unknown data format: \"".$value."\"", "system");
       else
        $this->dataformat=$value;

       break;
      }
      case "request":
      {
       $this->requestVar=$value;
       break;
      }
      case "db":
      {
       $this->db=$value; //which database to use, can be overwritten in modules which can only be done with module without a category reference due to limitation of mysql
       break;
      }
      case "skipCheckTables":
      {
       //skip checkTables call, for cache to call after
       break;
      }
      case "skipSetRequest":
      {
       //skip setRequestValue call, for cache to call after
       break;
      }
      case "skipPostInit":
      {
       //skip postInit call, for cache to call after
       break;
      }
      default:
      {
       $this->e->add("Unknown option: \"".$key."\"", "system");
       break;
      }
     }
    }
   }

   return true;
  }


  function setModule($module)
  {
   global $_path, $_config, $_jses;

   //Clean up previous module
   $this->clear();

   if(!$module)
   {
    if(isset($_REQUEST["m"])&&$_REQUEST["m"]!="")
     $module=$_REQUEST["m"];
    else
     $module="categories";
   }

   if(!preg_match(REGEXP_DATA_PLUGIN_MODULE, $module)&&!preg_match(REGEXP_DATA_MODULE, $module))
   {
    $this->e->add("Illegal module name", "hack");
    return false;
   }

   if(!in_array($module, modules()))
   {
    $this->e->add("Module: \"".$module."\" doesn't exist", "hack");
    return false;
   }

   //Basics are ok
   $this->module=$module;

   //Global fields
   $this->addField("m", "module", "form", "hidden;control", 1, 25, $this->module);
   $this->addField("a", "action", "form", "hidden;control", 1, 25, $this->action);
   $this->addField("e", "extra", "form", "hidden;control", 1, 25, $this->extra);

   //add rights field
   if(isset($_config["debug"])&&$_config["debug"]) //debug mode, show rights field
    $this->addField("r", "rights", "sql;show", "control", 0, false, "", "Rights");
   else
    $this->addField("r", "rights", "sql", "control", 0, false, "", "Rights");

   $this->addField("reload_trigger", "text", "form", "hidden;control", 0, 25, ""); //set when javascript reload property is triggered

   $this->addField("show_from", "nr", "form", "hidden;control", 0, false, 0); //start showing item from here
   $this->addField("show_total", "nr", "form", "hidden;control", 0, 100, 0); //number of items per page, 0 to show all
   $this->addField("show_count", "nr", "form", "hidden;control", 0, 100, 0); //number of items

   $this->addField("aggregation_field", "text", "form", "hidden;control", 0, 25, "", "Aggregation field");
   $this->addField("aggregation_type", "enum", "form", "hidden;control", false, false, "", "Aggregation type");
   $this->enumOptions("aggregation_type", "", "average", "min", "max", "sum", "count");
   $this->addField("aggregation_method", "enum", "form", "hidden;control", false, false, "around", "Aggregation method");
   $this->enumOptions("aggregation_method", "before", "around", "after");
   $this->addField("aggregation_interval", "nr", "form", "hidden;control", 0, false, 0, "Aggregation interval");


   //Get module specific and global module translations
   $this->l->read("modules");
   $this->l->read("module_".$this->module);

   //Set module specific formatting
   if(isset($_config["modules"][$this->module])&&isset($_config["modules"][$this->module]["module"])) //configModule, not only configField
   {
    foreach(Array("timezone", "dateformat", "timeformat", "datetimeformat", "decimalseparator", "thousandsseparator") as $format)
    {
     $this->format["html"][$format]=$_config["modules"][$this->module][$format];
     $this->format["text"][$format]=$_config["modules"][$this->module][$format];
     $this->format["form"][$format]=$_config["modules"][$this->module]["form".$format];
    }
   }

   //init module
   if(preg_match(REGEXP_DATA_PLUGIN_MODULE, $this->module, $tmp)) //plugin module
   {
    $this->moduleObject=false;
    $this->moduleClassName=false;

    $plugin=$tmp[1];
    $module=$tmp[2];

    if(!in_array($plugin, plugins())&&$plugin!=".tmp")
    {
     $this->e->add("Plugin: \"".$plugin."\" doesn't exist", "plugin");
     return false;
    }

    $this->plugin=new Plugin($plugin);

    if(!$this->plugin->isEnabled())
    {
     $this->e->add("Plugin: \"".$plugin."\" is disabled", "plugin");
     return false;
    }

    if(!file_exists($this->plugin->path["modules"].$module.".php"))
    {
     $this->e->add("Module: \"".$module."\" doesn't exist for plugin: \"".$plugin."\"", "warning");
     return false;
    }

    require_once($this->plugin->path["modules"].$module.".php");

    $this->moduleClassName=$plugin."_".$module;
    $this->moduleObject=new $this->moduleClassName;
    $this->moduleObject->init($this);
   }
   else //normal module
   {
    $modulePaths=Array();

    if(isset($_path["project"]["modules"]))
     $modulePaths[]=$_path["project"]["modules"];

    if(isset($_path["modules"]))
     $modulePaths[]=$_path["modules"];

    foreach($modulePaths as $path)
    {
     if(file_exists($path.$this->module.".php"))
     {
      require_once($path.$this->module.".php");
      break;
     }
    }

    if($_jses["project"]=="jas"&&class_exists("JAS_Module_".$this->module))
    {
     $this->moduleClassName="JAS_Module_".$this->module;
     $this->moduleObject=new $this->moduleClassName;
     $this->moduleObject->init($this);
    }
    else if(class_exists("Project_Module_".$this->module)) //special names because of: Cannot redeclare class module_categories
    {
     $this->moduleClassName="Project_Module_".$this->module;
     $this->moduleObject=new $this->moduleClassName;
     $this->moduleObject->init($this);
    }
    else if(class_exists("Module_".$this->module))
    {
     $this->moduleClassName="Module_".$this->module;
     $this->moduleObject=new $this->moduleClassName;
     $this->moduleObject->init($this);
    }
    else //backwards compatibility, old modules didn't use class
    {
     $this->moduleObject=false;
     $this->moduleClassName=false;

     if(!$this->title) //second time the module is used on this page, include_once above didn't work
     {
      foreach($modulePaths as $path)
      {
       if(file_exists($path.$this->module.".php"))
       {
        require($path.$this->module.".php");
        break;
       }
      }
     }

     $tmp=$this->e->logging;
     $this->e->logging="none";
     $this->e->add("Module: \"".$this->module."\" uses old module definition", "debug");
     $this->e->logging=$tmp;
    }
   }

   //Add additional actions
   if(count($this->check("properties", "order"))) //ordering
    $this->addActions("order");
   else
    $this->removeActions("order");

   if(count($this->check("reference", "categories"))) //Move and Lost&found option for category referenced modules
   {
    $this->addExtra("view", "list");
    $this->addActions("move", "lost");
   }
   else
   {
    $this->removeExtra("view", "list");
    $this->removeActions("move", "lost");
   }

   if(count($this->check("properties", "deleted"))) //deleted and undelete for modules with "deleted" field
   {
    $this->addActions("deleted");
    $this->addActions("undelete");
   }
   else
   {
    $this->removeActions("deleted");
    $this->removeActions("undelete");
   }

   if($this->fulltextIndex) //full text index exists
    $this->addExtra("search", "index"); //add index based search

   if(isset($_config["usernr"])&&isset($_config["rootnr"])&&$_config["usernr"]==$_config["rootnr"]) //database manipulation (ignore if config isn't read)
   {
    $this->addExtra("database", "create");
    $this->addExtra("database", "alter");

    if(!($storage=$this->storageObject()))
     return false;

    if(method_exists($storage, "maintenance"))
     $this->addExtra("database", "maintenance");
   }
   else
   {
    $this->removeExtra("database", "create");
    $this->removeExtra("database", "alter");

    $this->removeExtra("database", "maintenance");
   }

   if(count($this->check("properties", "owner"))) //ownage option
    $this->addActions("owner");
   else
    $this->removeActions("owner");

   return true;
  }


  function setAction($action)
  {
   global $_config;

   if(!$action)
   {
    if(isset($_REQUEST["a"])&&$_REQUEST["a"]!="")
     $action=$_REQUEST["a"];
    else
     $action="view";
   }

   if(!preg_match(REGEXP_DATA_ACTION, $action))
   {
    $this->e->add("Illegal action", "hack");
    return false;
   }

   if(!$this->validAction($action))
   {
    $this->e->add("Invalid action: \"".$action."\"", "hack");
    return false;
   }

   //Basics are ok
   $succes=true;
   if($this->extends)
   {
    foreach(array_keys($this->extends) as $module)
    {
     if(!$this->extends[$module]->setAction($action))
     {
      $this->e->add($this->extends[$module]->e);
      $succes=false;
     }
    }
   }

   if(!$succes)
    return false;

   //Basics are ok
   $this->action=$action;

   if($this->action=="search")
   {
    if(!isset($this->field["search"]))
    {
     $this->addField("search", "search", "form", "control", 2, 128, false, "Search");
     $this->addField("searchFields", "multilist", "form", "control", 2, 128, false, "Search fields");

     //force search limit
     $this->field["show_total"]["min"]=1;
     $this->field["show_total"]["max"]=100;
     $this->field["show_total"]["defaultvalue"]=10;

     if($this->field["show_total"]["valuetype"]=="default")
      $this->setDefaultValue("show_total");
    }
   }
   else //remove search fields
   {
    if(isset($this->field["search"]))
    {
     unset($this->field["search"]);
     unset($this->field["searchFields"]);

     $this->field["show_total"]["min"]=$this->field["show_total"]["original"]["min"];
     $this->field["show_total"]["max"]=$this->field["show_total"]["original"]["max"];
     $this->field["show_total"]["defaultvalue"]=$this->field["show_total"]["original"]["defaultvalue"];

     if($this->field["show_total"]["valuetype"]=="default")
      $this->setDefaultValue("show_total");
    }
   }

   if($this->action=="move")
   {
    if(!isset($this->field["newcatnr"]))
    {
     $this->addField("newcatnr", "nr", "form", "global;default", 1, false, 1, "Move to new category");
     $this->addReference("newcatnr", "categories", "nr", "*:1");

     //link up extended modules fields
     if($this->extends)
     {
      foreach(array_keys($this->extends) as $module)
      {
       $this->extends[$module]->field["newcatnr"]=&$this->field["newcatnr"];
       $this->extends[$module]->reference["newcatnr"]=&$this->reference["newcatnr"];
      }
     }
    }
   }
   else //remove move fields
   {
    if(isset($this->field["newcatnr"]))
    {
     unset($this->field["newcatnr"]);
     unset($this->reference["newcatnr"]);
    }
   }

   if(isset($this->field["a"]))
   {
    $this->field["a"]["defaultvalue"]=$this->action;

    if(isset($this->field["a"]["valuetype"])&&$this->field["a"]["valuetype"]=="default")
     $this->setDefaultValue("a");
   }

   return true;
  }


  function setExtra($extra)
  {
   global $_config;

   if(!$extra)
   {
    if(isset($_REQUEST["e"])&&$_REQUEST["e"]!="")
     $extra=$_REQUEST["e"];
    else if(isset($this->extraList[$this->action]))
     $extra=$this->extraList[$this->action][0];
    else
     $extra="";
   }

   if(!preg_match(REGEXP_DATA_EXTRA, $extra))
   {
    $this->e->add("Illegal extra option", "hack");
    return false;
   }

   if(!$this->validExtra($extra))
   {
    $this->e->add("Invalid extra option: \"".$extra."\"", "hack");
    return false;
   }

   $succes=true;
   if($this->extends)
   {
    foreach(array_keys($this->extends) as $module)
    {
     if(!$this->extends[$module]->setExtra($extra))
     {
      $this->e->add($this->extends[$module]->e);
      $succes=false;
     }
    }
   }

   if(!$succes)
    return false;

   $this->extra=$extra;

   if(isset($this->field["e"]))
   {
    $this->field["e"]["defaultvalue"]=$this->extra;

    if(isset($this->field["e"]["valuetype"])&&$this->field["e"]["valuetype"]=="default")
     $this->setDefaultValue("e");
   }

   //table info
   if($this->action=="database"&&$this->extra=="info")
   {
    if($_config["usernr"]==$_config["rootnr"]&&!isset($this->field["info_type"]))
    {
     $this->addField("info_type", "text", "info", false, 0, false, false, "Type");

     $this->addField("info_rows", "nr", "info", false, 0, false, false, "Number of rows");
     $this->addField("info_insertID", "nr", "info", false, 0, false, false, "Next insert ID");

     $this->addField("info_created", "datetime", "info", "", false, false, false, "Created");
    }

    if(!isset($this->field["info_updated"]))
     $this->addField("info_updated", "datetime", "info", "", false, false, false, "Last updated");
   }

   return true;
  }


  function postInit($extendedModules=true, $initializedReferences=true)
  {
   global $_config;

   if(!$this->module)
    return false;

   $success=true;

   //call postInit for referenced modules
   if($extendedModules)
   {
    foreach(array_keys($this->extends) as $module)
    {
     if(!$this->extends[$module]->postInit($extendedModules))
     {
      $this->e->add($this->extends[$module]->e);
      $success=false;
     }
    }
   }

   //map values (now, usernr, etc)
   $this->mapValues();

   //map default category when adding
   if($this->action=="add")
   {
    //get default category (module) config setting
    if(isset($_config["modules"][$this->module])&&isset($_config["modules"][$this->module]["defaultcategory"])) //configModule
     $defaultcategory=$_config["modules"][$this->module]["defaultcategory"];
    else if(isset($_config["defaultcategory"])) //config
     $defaultcategory=$_config["defaultcategory"];
    else //none
     $defaultcategory="Default";

    if($defaultcategory!="Default"&&($rightsReference=$this->getRightsReference())) //get default category for rights reference
    {
     $from=$this->reference[$rightsReference]["from"];

     if($this->field[$from]["valuetype"]=="default") //only get if value type is default
     {
      if($defaultcategory=="First"||$defaultcategory=="Last") //global first/last category
      {
       $d=Data::cache("categories", "view", "all", Array("format" => "sql", "db" => $this->db));

       if($defaultcategory=="First")
        $d->sortOrder("nr+");
       else //Last
        $d->sortOrder("nr-");
      }
      else //Last used
      {
       $d=Data::cache($this->module, "view", "all", Array("format" => "sql", "db" => $this->db));

       //sort most recent or first entry
       $d->sortOrder("updated-");
      }

      //show 1 entry, only get from field
      $d->clearValues();
      $d->setValue("show_total", 1);
      $d->setShowFields($from);

      if(!$d->get())
       $this->e->add($d->e);

      if($d->next())
      {
       //use nr instead of catnr for categories
       if($d->module=="categories")
        $this->setValue($from, $d->v["nr"]);
       else
        $this->setValue($from, $d->v[$from]);

       //set default as value type
       $this->field[$from]["valuetype"]="default";
      }
     }
    }
   }

   //call postInit for current module
   if(!$this->moduleSpecificFunction("postInit", false, false)) //don't run post init for extended modules, done above
    $success=false;

   //call postInit for referenced *:* modules
   if($initializedReferences)
   {
    foreach(array_keys($this->reference) as $name)
    {
     if($this->reference[$name]["type"]=="*:*"&&isset($this->reference[$name]["data"]))
     {
      if(!$this->reference[$name]["data"]->postInit($extendedModules, $initializedReferences))
      {
       $this->e->add($this->reference[$name]["data"]->e);
       $success=false;
      }
     }
    }
   }

   return $success;
  }


  function checkTables($extendedModules=true, $initializedReferences=true)
  {
   if(!$this->module)
    return false;

   $success=true;

   //check tables for referenced modules
   if($extendedModules)
   {
    foreach(array_keys($this->extends) as $module)
    {
     if(!$this->extends[$module]->checkTables($extendedModules, $initializedReferences))
     {
      $this->e->add($this->extends[$module]->e);
      $success=false;
     }
    }
   }

   //check tables for current module
   if($this->module!="empty") //empty module doesn't have a table
   {
    if(!$this->tableExists())
    {
     $this->e->add("Table: \"".$this->module."\" doesn't exist, module not usuable", "sql");
     $success=false;
    }
   }

   //check tables for referenced *:* modules
   if($initializedReferences)
   {
    foreach(array_keys($this->reference) as $name)
    {
     if($this->reference[$name]["type"]=="*:*"&&isset($this->reference[$name]["data"]))
     {
      if(!$this->reference[$name]["data"]->checkTables($extendedModules, $initializedReferences))
      {
       $this->e->add($this->reference[$name]["data"]->e);
       $success=false;
      }
     }
    }
   }

   return $success;
  }

  function tableExists()
  {
   global $_jses;

   if(!$this->module)
    return false;

   //prepare session cache
   if(!isset($_jses["tableExists"]))
    $_jses["tableExists"]=Array();

   if(!isset($_jses["tableExists"][$_jses["project"]]))
    $_jses["tableExists"][$_jses["project"]]=Array();

   if(!isset($_jses["tableExists"][$_jses["project"]][$this->db]))
    $_jses["tableExists"][$_jses["project"]][$this->db]=Array();

   //check session check
   if(!isset($_jses["tableExists"][$_jses["project"]][$this->db][$this->module])) //not yet cached
   {
    if(!($storage=$this->storageObject()))
     return false;

    //check if table exists
    $_jses["tableExists"][$_jses["project"]][$this->db][$this->module]=$storage->table(); //store cache
   }

   if(!$_jses["tableExists"][$_jses["project"]][$this->db][$this->module]) //check table exist state from cache
    return false;

   return true;
  }


  function copy()
  {
   if(substr(phpversion(), 0, 1)>=5)
   {
    eval('$d=clone($this);');

    //clone internal objects
    eval('$d->e=clone($this->e);');
    eval('$d->l=clone($this->l);');

    if(isset($d->moduleObject)&&$d->moduleObject&&is_string($d->moduleObject))
     $d->moduleObject=unserialize($d->moduleObject);
    else if($this->moduleObject)
     eval('$d->moduleObject=clone($this->moduleObject);');

    if($this->plugin)
     $d->plugin=$this->plugin->copy();

    //copy field array (extending modules (below) must refer to the copy)
    foreach(array_keys($this->field) as $field)
    {
     unset($d->field[$field]);

     foreach($this->field[$field] as $k=> $v)
      $d->field[$field][$k]=$v;
    }
   }
   else
    $d=$this;

   //set moduleObject back reference to new object
   if($d->moduleObject)
   {
    unset($d->moduleObject->data);
    $d->moduleObject->data=&$d;
   }

   //set request var
   unset($d->requestVar);
   $d->requestVar=$this->requestVar;

   //copy references
   foreach(array_keys($d->reference) as $name)
   {
    if($d->reference[$name]["type"]=="*:*") //copy m2m data object
     $d->reference[$name]["data"]=$this->_copyRecurse($this->reference[$name]["data"]);
    else if($d->reference[$name]["type"]=="*:1"||$d->reference[$name]["type"]=="*:0") //set reference references to new object
    {
     $from=$d->reference[$name]["from"];
     if($from!=$name)
     {
      $d->field[$from]["valuetype"]=&$d->field[$name]["valuetype"];
      $d->field[$from]["value"]=&$d->field[$name]["value"];
     }
    }
   }

   //set extending modules
   $d->extends=Array();
   $d->extendedBy=Array();

   if($this->extends)
    foreach(array_keys($this->extends) as $module)
     $d->moduleObject->extend($module);

   return $d;
  }

  function _copyRecurse($var)
  {
   if(is_array($var))
   {
    $tmp=Array();
    foreach(array_keys($var) as $k)
    {
     if(is_array($var[$k])||is_object($var[$k]))
      $tmp[$k]=$this->_copyRecurse($var[$k]);
     else
      $tmp[$k]=$var[$k];
    }
   }
   else if(is_object($var))
    eval('$tmp=clone($var);');
   else
    $tmp=$var;

   return $tmp;
  }

  function group($title)
  {
   $fields=func_get_args();
   array_shift($fields); //skip title

   if(isset($fields[0])&&is_array($fields[0]))
    $fields=$fields[0];

   if(($group=array_search($title, $this->fieldGroups))===false)
   {
    $group=sizeof($this->fieldGroups);
    if($group==0)
    {
     $this->fieldGroups[]=$this->title;
     $group++;
    }
    $this->fieldGroups[]=$this->l->t($title);
   }

   foreach($fields as $field)
   {
    if(isset($this->field[$field]))
     $this->field[$field]["group"]=$group;
   }
  }

  function getGroups()
  {
   return $this->fieldGroups;
  }

  function getGroupFields($group)
  {
   $fields=Array();
   foreach(array_keys($this->field) as $field)
   {
    if(isset($this->field[$field]["group"])&&$this->field[$field]["group"]===$group)
     $fields[]=$field;
   }

   return $fields;
  }

  function deleteGroup($group)
  {
   $tmp=Array();
   $l=sizeof($this->fieldGroups);

   for($c=0;$c<$l;$c++)
    if($c!=$group)
     $tmp[]=$this->fieldGroups[$c];

   $this->fieldGroups=$tmp;

   foreach(array_keys($this->field) as $field)
   {
    if($this->field[$field]["group"]==$group)
      $this->field[$field]["group"]=0;
    else if($this->field[$field]["group"]>$group)
     $this->field[$field]["group"]--;
   }
  }

  function getGroupID($group)
  {
   $l=sizeof($this->fieldGroups);

   for($c=0;$c<$l;$c++)
    if($group==$this->l->t($this->fieldGroups[$c]))
     return $c;

   return false;
  }

/******************************************************************
******************************************************************/


  function defaultFields()
  {
   global $_config;

   $this->addField("owner", "nr", "all", "owner;userReference;default", 1, false, "usernr", "By");
   $this->addReference("owner", $_config["usermodule"], "nr", "*:1");

   $this->addField("updated", "datetime", "sql;show", "forceUpdate;default", false, false, "now", "Last update");

   $this->addField("nr", "nr", "sql;form", "identifier;hidden;global;default", 1, false);
   $this->addField("catnr", "nr", "sql;form", "rightsReference;global;default", 1, false, 1, "Category");
   $this->addReference("catnr", "categories", "nr", "*:1");
  }



/******************************************************************
 addField

 - name: field name
 - type: field type
   - title: only characters
   - text: all characters allowed except tags, [B]bold[/B] tags can be used instead
     - if max is more then 256 characters a textarea is used
   - id: hidden form field, database id
   - nr: number form field (if usernr/rootnr: the current user/root nr is assumed)
   - date: DD-MM-YYYY (if "now": today is assumed)
   - time: HH:MM:SS (if "now": current time is assumed)
   - datetime: DD-MM-YYYY HH:MM:SS (if "now": go figure :))
   - hidden: hidden form field, can't be used for sql

 - use: where to use the data
   - form: only use in HTML form
   - sql: only use in SQL query
   - form;sql: use in HTML form and SQL qiery
   - both: same as form;sql

 - properties: extra type properties
   - identifier
   - unique
   - global
   - default
   - control
   - reload

 - min: Minimal number or characters depending on type
 - max: Maximal number or characters depending on type

 - value: default value for field, if not set, $_REQUEST[$name] is used

******************************************************************/

  function addField($name, $type, $use="form;sql", $properties=false, $min=false, $max=false, $value=false, $title=false, $text=false)
  {
   global $_config;

   if(!preg_match(REGEXP_DATA_FIELDNAME, $name))
   {
    $this->e->add("Invalid fieldname: \"".$name."\"", "module");
    return false;
   }

   if(isset($this->field[$name]))
   {
    $this->e->add("Field: \"".$name."\" already exists", "module");
    return false;
   }

   if(!in_array($type, $this->fieldTypes))
   {
    $this->e->add("Unknown field type: \"".$type."\" for field: \"".$name."\"", "module");
    return false;
   }

   if(preg_match("/identifier/", $properties)&&($id=$this->getID()))
   {
    $this->e->add("Removing identifier property for field: \"".$name."\", module already has an identifier field: \"".$id."\"", "warning");
    $properties=preg_replace("/identifier;/", "", $properties);
   }

   if(($type=="submit"||$type=="label")&&(stristr($use, "sql")||stristr($use, "all")))
   {
    $this->e->add("Field type: \"".$type."\" for field: \"".$name."\" may not be used as a SQL field", "module");
    return false;
   }

   $this->field[$name]=Array();
   $this->field[$name]["type"]=$type;
   $this->field[$name]["module"]=$this->module;
   $this->field[$name]["properties"]=$properties;
   $this->field[$name]["operant"]=Array($this->defaultOperant);
   $this->field[$name]["mutilation"]=Array();

   if(!$this->checkProperties($name, $properties))
   {
    unset($this->field[$name]);
    return false;
   }


   $use=preg_replace("/all/", "form;sql;show", $use);
   $this->field[$name]["use"]=$use;

   $this->field[$name]["min"]=$min;
   $this->field[$name]["max"]=$max;

   $this->field[$name]["group"]=0;
   $this->field[$name]["plugin"]=false;

   if($this->field[$name]["type"]=="nr"&&$this->check("properties", "dropdown", $name)&&$this->field[$name]["max"]<=0)
    $this->e->add("Dropdown property needs a max value of at least 1 for field: \"".$name."\"", "module");

   if($type=="date"||$type=="time"||$type=="datetime")
    $formats=Array("timezone", "dateformat", "timeformat", "datetimeformat");
   else if($type=="float"||$type=="price")
    $formats=Array("decimals", "decimalseparator", "thousandsseparator");
   else
    $formats=Array();

   $this->field[$name]["format"]=Array();
   if(isset($_config["modules"][$this->module])&&isset($_config["modules"][$this->module]["field"][$name])) //configField
   {
    foreach($formats as $format)
    {
     $this->field[$name]["format"]["html"][$format]=$_config["modules"][$this->module]["field"][$name][$format];
     $this->field[$name]["format"]["text"][$format]=$_config["modules"][$this->module]["field"][$name][$format];
     $this->field[$name]["format"]["form"][$format]=$_config["modules"][$this->module]["field"][$name]["form".$format];
     $this->field[$name]["format"]["sql"][$format]=$this->format["sql"][$format];
    }
   }
   else
   {
    foreach($formats as $format)
    {
     $this->field[$name]["format"]["html"][$format]=$this->format["html"][$format];
     $this->field[$name]["format"]["text"][$format]=$this->format["html"][$format];
     $this->field[$name]["format"]["form"][$format]=$this->format["form"][$format];
     $this->field[$name]["format"]["sql"][$format]=$this->format["sql"][$format];
    }
   }

   if($value&&($type=="date"||$type=="time"||$type=="datetime"))
   {
    if(preg_match("/^now([\-\+]?)(\d*)$/", $value, $m))
     $this->field[$name]["defaultvalue"]=$value;
    else //time set
    {
     $difference=$value-time();

     if($difference>0) //later
      $this->field[$name]["defaultvalue"]="now+".$difference;
     else if($difference<0) //before
      $this->field[$name]["defaultvalue"]="now".$difference;
     else
      $this->field[$name]["defaultvalue"]="now";
    }
   }
   else if($value&&($type=="float"||$type=="price"))
    $this->field[$name]["defaultvalue"]=number_format($value, $this->field[$name]["format"][$this->dataformat]["decimals"], $this->field[$name]["format"][$this->dataformat]["decimalseparator"], $this->field[$name]["format"][$this->dataformat]["thousandsseparator"]);
   else
    $this->field[$name]["defaultvalue"]=$value;

   if($type=="localfile"||$type=="localfiles"||$type=="localdirectory")
   {
    $this->field[$name]["localpath"]=Array();
    $this->field[$name]["localpath"]["directory"]="./";
    $this->field[$name]["localpath"]["extensions"]=Array();
   }

   if($type=="image"||$type=="file")
   {
    $fileUse="";

    if($this->check("use", "sql", $name))
     $fileUse.="sql";

    if($this->check("use", "show", $name))
    {
     if($fileUse)
      $fileUse.=";show";
     else
      $fileUse.="show";
    }

    if($type=="image")
    {
     $this->addField($name."ImageWidth", "nr", $fileUse, false, 0, false, false, $title." width");
     $this->addField($name."ImageHeight", "nr", $fileUse, false, 0, false, false, $title." height");
     $this->addField($name."PHPImageType", "nr", $fileUse, false, 0, false, false, $title." image type");
    }

    $this->addField($name."Type", "text", $fileUse, false, 0, 100, false, $title." type");
    $this->addField($name."Size", "nr", $fileUse, false, 0, false, false, $title." size");

    if($this->check("use", "sql", $name))
     $this->addField($name."Data", "filedata", "sql", false, $min, $max, false, $title." data");
    else
     $this->addField($name."Data", "filedata", "", false, $min, $max, false, $title." data");

    $this->addField($name."Array", "array", "", false, false, false, false, $title." array");
   }

   $this->field[$name]["value"]=Array();
   $this->field[$name]["valuetype"]="";
   $this->field[$name]["operator"]=Array();

   //use default
   if($this->field[$name]["defaultvalue"]!==false||$type=="boolean")
   {
    $this->field[$name]["valuetype"]="default";
    $this->field[$name]["value"][0]=$this->field[$name]["defaultvalue"];
   }
   else //no default
   {
    $this->field[$name]["valuetype"]="none";
    if($type=="nr"||$type=="float"||$type=="price")
     $this->field[$name]["value"][0]=0;
    else
     $this->field[$name]["value"][0]="";
   }

   //Set field title
   if($title)
    $this->field[$name]["title"]=$this->l->t($title);
   else if($title===false)
    $this->field[$name]["title"]=$this->l->t($name);
   else
    $this->field[$name]["title"]="";

   $this->t[$name]=&$this->field[$name]["title"];
   $this->field[$name]["text"]=$this->l->t($text);

   $this->field[$name]["original"]=$this->field[$name];

   //add a confirmation password field for form
   if($this->check("properties", "password", $name)&&!preg_match("/^verify/", $name))
    return $this->addField("verify".$name, $type, "form", $properties, $min, $max, $value, "Verify ".$name);

   if($type=="version")
   {
    if($this->check("properties", "singleField", $name))
    {
     $this->addField($name."_major", "nr", "", false, $min, $max, false, $name." major");
     $this->addField($name."_minor", "nr", "", false, 0, false, false, $name." minor");
     $this->addField($name."_maintenance", "nr", "", false, 0, false, false, $name." maintenance");
    }
    else //multi fields
    {
     $this->addField($name."_major", "nr", "sql", false, $min, $max, false, $name." major");
     $this->addField($name."_minor", "nr", "sql", false, 0, false, false, $name." minor");
     $this->addField($name."_maintenance", "nr", "sql", false, 0, false, false, $name." maintenance");
     $this->field[$name]["use"]=str_replace("sql", "", $this->field[$name]["use"]);
    }
   }

   switch($type)
   {
    case "list":
    case "multilist":
    {
     $this->listOptions($name); //init empty list options
     break;
    }
    case "module":
    case "modules":
    {
     if($type=="module")
      $modules=Array(""); //add first empty entry
     else //modules
      $modules=Array();

     if(!$this->check("properties", "exists", $name))
      $modules=array_merge($modules, modules()); //get all modules
     else //only existing modules
     {
      $d=$this->copy(); //copy data object
      foreach(modules() as $module)
      {
       $d->module=$module; //dirty hack to speed up

       if($d->tableExists()) //table exists...
        $modules[]=$module; //add module to module list
      }

      unset($d, $module);
     }

     $this->listOptions($name, $modules); //init modules list options
     unset($modules);
     break;
    }
    case "plugin":
    {
     $this->listOptions($name, array_merge(Array(""), plugins())); //init plugins list options
     break;
    }
    case "project":
    {
     $this->listOptions($name, array_merge(Array(""), projects())); //init projects list options
     break;
    }

    default: break;
   }

   return true;
  }

  function setRequestValues($extendedModules=true, $initializedReferences=true)
  {
   if(!$this->module)
    return false;

   $success=true;

   //set request values in extended modules
   if($extendedModules)
   {
    foreach(array_keys($this->extends) as $module)
    {
     if(!$this->extends[$module]->setRequestValues($extendedModules, $initializedReferences))
     {
      $this->e->add($this->extends[$module]->e);
      $success=false;
     }
    }
   }

   //set request values in current module
   foreach(array_keys($this->field) as $field)
    if(!$this->setRequestValue($field))
     $success=false;

   //set request values in referenced *:* modules
   if($initializedReferences)
   {
    foreach(array_keys($this->reference) as $name)
    {
     if($this->reference[$name]["type"]=="*:*"&&isset($this->reference[$name]["data"]))
     {
      if(!$this->reference[$name]["data"]->setRequestValues($extendedModules, $initializedReferences))
      {
       $this->e->add($this->reference[$name]["data"]->e);
       $success=false;
      }
     }
    }
   }

   return $success;
  }

  function setRequestValue($name)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't set request values for non-existing field: \"".$name."\"", "module");
    return false;
   }

   if($this->field[$name]["type"]=="image"||$this->field[$name]["type"]=="file")
   {
    if($this->requestVar)
    {
     $_req=Array();
     if(isset($_FILES[$this->requestVar])&&isset($_FILES[$this->requestVar]["name"][$name])) //different array structure: requestVar[tmp_name][$name]
      foreach(array_keys($_FILES[$this->requestVar]) as $key) //reorganize in accepted format
       $_req[$name][$key]=$_FILES[$this->requestVar][$key][$name];
    }
    else
     $_req=$_FILES;

    if(isset($_req[$name])&&$_req[$name])
    {
     $fileupload="single";

     if(FS::size($_req[$name]["tmp_name"])!=$_req[$name]["size"])
     {
      $this->e->add("File size doesn't match for uploaded file: \"".$_req[$name]["name"]."\"", "error");
      return false;
     }

     if($this->field[$name]["type"]=="image"&&$_req[$name]["error"]==UPLOAD_ERR_OK)
     {
      $imageinfo=getimagesize($_req[$name]["tmp_name"]);
      $this->setValue($name."ImageWidth", $imageinfo[0]);
      $this->setValue($name."ImageHeight", $imageinfo[1]);
      $this->setValue($name."PHPImageType", $imageinfo[2]);
     }

     $this->setValue($name."Array", $_req[$name]);
     $this->setValue($name."Type", $_req[$name]["type"]);
     $this->setValue($name."Size", $_req[$name]["size"]);

     if($_req[$name]["error"]==UPLOAD_ERR_OK)
      $this->setValue($name."Data", FS::read($_req[$name]["tmp_name"]));

     $tmp=$_req[$name]["name"];
     unset($_req);

     $_req=Array();
     $_req[$name]=$tmp;
     unset($tmp);
    }
   }
   else if($this->requestVar)//special request var: subarray
   {
    if(isset($_REQUEST[$this->requestVar]))
     $_req=$_REQUEST[$this->requestVar];
    else
     $_req=Array();
   }
   else
    $_req=$_REQUEST;//default

   if($this->check("use", "form", $name))//check all input fields
   {
    if(isset($_req[$name]))//has a value?
    {
     $this->field[$name]["valuetype"]="request";

     if(is_array($_req[$name])&&!isset($fileupload)||(isset($fileupload)&&$fileupload=="array")) //multiple values even
     {
      $this->field[$name]["value"]=array_values($_req[$name]);

      if($this->vmax<sizeof($this->field[$name]["value"])) //use largest amount of entries
       $this->vmax=sizeof($this->field[$name]["value"]);

      for($c=0;$c<$this->vmax;$c++)
       $this->field[$name]["operant"][$c]=$this->defaultOperant;
     }
     else
     {
      $this->field[$name]["value"][0]=$_req[$name];//single value
      $this->field[$name]["operant"][0]=$this->defaultOperant;

      if($this->vmax==0)
       $this->vmax=1;
     }
    }
   }

   //Make booleans
   if($this->field[$name]["type"]=="boolean"&&$this->field[$name]["valuetype"]=="request")
   {
    foreach(array_keys($this->field[$name]["value"]) as $k)
    {
     if($this->field[$name]["value"][$k]===1||$this->field[$name]["value"][$k]==="1"||$this->field[$name]["value"][$k]==="true")
      $this->field[$name]["value"][$k]=true;
     else
      $this->field[$name]["value"][$k]=false;
    }
   }

   return true;
  }

  function resetField($name)
  {
   if(!isset($this->field[$name]))
   {
    $this->e->add("Can't reset non-existing field: \"".$name."\"", "module");
    return false;
   }

   $keys=Array("type", "use", "properties", "min", "max", "defaultvalue");

   foreach($keys as $key)
    $this->field[$name][$key]=$this->field[$name]["original"][$key];

   return true;
  }


/******************************************************************
 check
 check field properties or get fields with matching properties

 - check: what to check: reference, show, properties
 - type:  setting to check: reference module, show items, properties
 - name:  field name to check, leave empty to get all matching fields

 returns: matching fields or true/false if field name is given
******************************************************************/
  function check($check, $type, $name=false)
  {
   if($name)
   {
    if($check=="reference")
    {
     if(isset($this->reference[$name])&&$this->reference[$name]["module"]==$type)
      return true;
     else
      return false;
    }
    else
    {
     if(isset($this->field[$name])&&isset($this->field[$name][$check])&&strpos($this->field[$name][$check], $type)!==false)
     //if(isset($this->field[$name])&&isset($this->field[$name][$check])&&preg_match("/(^|;)".$type."(;|$)/i", $this->field[$name][$check]))
      return true;
     else
      return false;
    }
   }

   $results=Array();
   if($check=="reference")
   {
    $references=array_keys($this->reference);
    $l=sizeof($references);

    for($c=0;$c<$l;$c++)
     if($this->reference[$references[$c]]["module"]==$type)
      $results[]=$references[$c];
   }
   else
   {
    $fields=array_keys($this->field);
    $l=sizeof($fields);

    for($c=0;$c<$l;$c++)
     if(isset($this->field[$fields[$c]][$check])&&strpos($this->field[$fields[$c]][$check], $type)!==false)
//     if(isset($this->field[$fields[$c]][$check])&&preg_match("/(^|;)".$type."(;|$)/i", $this->field[$fields[$c]][$check]))
      $results[]=$fields[$c];
   }

   return $results;
  }

/******************************************************************
 regexps
 set field regexps

 - field: field to check with regexps
 - regexp1: regular expression to check first
 - regexp2: regular expression to check second
 - etc: additional regexps

 returns: true/false for succes
******************************************************************/
  function regexps($field)
  {
   $regexps=func_get_args();
   array_shift($regexps); //skip field

   if(isset($regexps[0])&&is_array($regexps[0]))
    $regexps=$regexps[0];

   if(!isset($this->field[$field]))
   {
    $this->e->add("Regexp field: \"".$field."\" doesn't exist", "module");
    return false;
   }

   foreach($regexps as $regexp)
   {
    if((@preg_match($regexp, ""))===false)
    {
     $this->e->add("Regexp: \"".$regexp."\" for field: \"".$field."\" is incorrect", "module");
     return false;
    }
   }

   $this->regexp[$field]=$regexps;

   return true;
  }

/******************************************************************
 getID
 get module identifier

 returns: field name identifier or false
******************************************************************/
  function getID()
  {
   if($this->ID)
    return $this->ID;

   foreach($this->check("properties", "identifier") as $field)
    return $this->ID=$field;

   return false;
  }

/******************************************************************
 getRightsReference
 get rightsReference categories field

 returns: reference name of matching rights+categories reference or false
******************************************************************/
  function getRightsReference()
  {
   if($this->rightsReference)
    return $this->rightsReference;

   foreach($this->check("reference", "categories") as $refer)
    if($this->check("properties", "rightsReference", $this->reference[$refer]["from"]))
     return $this->rightsReference=$refer;

   return false;
  }


/******************************************************************
 getSelfReference
 get self reference field

 returns: reference name of matching reference to it self or false
******************************************************************/
  function getSelfReference()
  {
   if($this->selfReference)
    return $this->selfReference;

   foreach($this->check("reference", $this->module) as $refer) //get reference
    if($this->check("use", "sql", $this->reference[$refer]["from"])) //sql field
     if($this->reference[$refer]["type"]!="*:*") //many to many self reference not supported
      return $this->selfReference=$refer;

   if($this->extends)
    foreach(array_keys($this->extends) as $module)
     foreach($this->check("reference", $module) as $refer) //get reference
      if($this->check("use", "sql", $this->reference[$refer]["from"])) //sql field
       if($this->reference[$refer]["type"]!="*:*") //many to many self reference not supported
        return $this->selfReference=$refer;

   return false;
  }



/******************************************************************
 addReference
 adds a reference to the module

 - name: reference name
 - module: reference to module
 - field: reference to field of module
 - type: reference type
   - 1:1
   - *:0
   - *:1
   - 0:*
   - 1:*
   - *:*
 - from: field name, $name used if empty

 returns: true/false
******************************************************************/

  function addReference($name, $module, $to, $type, $from=false)
  {
   if($this->action=="reference"&&$this->extra=="check")
    return false;

   if(!in_array($module, modules()))
   {
    $this->e->add("Unknown referenced module: \"".$module."\"", "module");
    return false;
   }

   if(!in_array($type, $this->referenceTypes))
   {
    $this->e->add("Unknown reference type: \"".$type."\"", "module");
    return false;
   }

   if(!$from)
   {
    if($type=="*:1"||$type=="*:0")
     $from=$name;
    else
     $this->e->add("Reference from field should be specified for type: \"".$type."\" for reference: \"".$name."\"", "module");
   }

   if(!isset($this->field[$from]))
    $this->e->add("Reference from field: \"".$from."\" doesn't exist for reference: \"".$name."\"", "module");

   if(isset($this->field[$name])&&($type!="*:1"&&$type!="*:0"))
    $this->e->add("Reference field: \"".$name."\" already exists", "module");

   if($rightsReference=$this->getRightsReference())
   {
    if($module=="categories"&&$this->check("properties", "rightsReference", $from))
     $this->e->add("Module already has a rights reference: \"".$rightsReference."\"", "module");
    else if($this->extends)
     foreach(array_keys($this->extends) as $extendedModules)
      if($extendedModules=="categories"&&$this->check("properties", "rightsReference", $from))
       $this->e->add("Module already has a rights reference: \"".$rightsReference."\"", "module");
   }
/*
   if($selfReference=$this->getSelfReference()&&$this->check("use", "sql", $from)) //allow 1 sql selfReferences
   {
    if($module==$this->module)
     $this->e->add("Module already has a self reference: \"".$selfReference."\"", "module");
    else if($this->extends)
     foreach(array_keys($this->extends) as $extendedModules)
      if($module==$extendedModules)
       $this->e->add("Module already has a self reference: \"".$selfReference."\"", "module");
   }
*/
   $d=Data::cache($module, "reference", "check", Array("format" => $this->dataformat, "db" => $this->db, "skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true), "_referenceCheck");
   $d->clearValues();

   if($type=="*:*")
   {
    if(in_array($name, modules()))
     $this->e->add("Reference field: \"".$name."\" already exists as a module for: \"".$type."\" reference", "module");
    else
    {
     $frommodule=$this->module;
     $tomodule=$d->module;

     $m2m=Data::cache("empty", $this->action, $this->extra, Array("format" => $this->dataformat, "db" => $this->db, "skipCheckTables" => true, "skipSetRequest" => true, "skipPostInit" => true));

     $m2m->module=$name;
     $m2m->initializingModule=$frommodule;
     $m2m->title=$frommodule.":".$tomodule;

     $m2m->addField("userrights", "rights", "sql", "allusers", 0, false, 3840); //view rights for all

     if($frommodule==$tomodule)
     {
      $m2m->addField($frommodule."_1", $this->field[$from]["type"], "all;from", "", 0, $this->field[$from]["max"], $this->field[$from]["defaultvalue"], ucfirst($name), "Reference to: ".$frommodule);
      $m2m->addReference($frommodule."_1", $frommodule, $from, "*:0", $frommodule."_1");

      $m2m->addField($tomodule."_2", $d->field[$to]["type"], "all;to", "", 0, $d->field[$to]["max"], $d->field[$to]["defaultvalue"], ucfirst($name), "Reference to: ".$tomodule);
      $m2m->addReference($tomodule."_2", $tomodule, $to, "*:0", $tomodule."_2");

      $m2m->addUnique("unique", $frommodule."_1", $tomodule."_2");
     }
     else
     {
      $m2m->addField($frommodule, $this->field[$from]["type"], "all;from", "", 0, $this->field[$from]["max"], $this->field[$from]["defaultvalue"], ucfirst($name), "Reference to: ".$frommodule);
      $m2m->addReference($frommodule, $frommodule, $from, "*:0", $frommodule);

      $m2m->addField($tomodule, $d->field[$to]["type"], "all;to", "", 0, $d->field[$to]["max"], $d->field[$to]["defaultvalue"], ucfirst($name), "Reference to: ".$tomodule);
      $m2m->addReference($tomodule, $tomodule, $to, "*:0", $tomodule);

      $m2m->addUnique("unique", $frommodule, $tomodule);
     }

     $m2m->clearValues();
    }
   }

   if(!isset($d->field[$to]))
   {
    $this->e->add("Reference to field: \"".$to."\" doesn't exist in module: \"".$module."\" for reference: \"".$name."\"", "module");
    return false;
   }

   if($this->field[$from]["type"]!=$d->field[$to]["type"])
    $this->e->add("Reference to field: \"".$to."\" in module: \"".$module."\" isn't the same type: \"".$this->field[$from]["type"]."\" for reference: \"".$name."\"", "module");

   switch($type)
   {
    case "1:1":
    {
     if($name==$from)
      $this->e->add("Reference from field: \"".$from."\" can't be the same as the reference name for reference: \"".$name."\"", "module");

     if(!$this->check("properties", "identifier", $from))
      $this->e->add("Reference from field: \"".$from."\" isn't an identifier field for reference: \"".$name."\"", "module");

     if(!$d->check("properties", "identifier", $to))
      $this->e->add("Reference to field: \"".$to."\" in module: \"".$module."\" isn't an identifier field for reference: \"".$name."\"", "module");

     if($this->getReferences($from, $type))
      $this->e->add("Reference from field: \"".$from."\" already has a: \"".$type."\" reference defined for reference: \"".$name."\"", "module");

     break;
    }
    case "*:0":
    case "*:1":
    {
//     if($this->check("properties", "identifier", $from))
//      $this->e->add("Reference from field: \"".$from."\" is an identifier field for reference: \"".$name."\"", "module");

     if(!$d->check("properties", "identifier", $to))
      $this->e->add("Reference to field: \"".$to."\" in module: \"".$module."\" isn't an identifier field for reference: \"".$name."\"", "module");

     if($this->getReferences($from))
      $this->e->add("Reference from field: \"".$from."\" already has a reference defined for reference: \"".$name."\"", "module");

     if($this->field[$from]["min"]===0&&$type=="*:1")
      $this->e->add("Reference from field: \"".$from."\" isn't required, while relation type is: \"".$type."\" for reference: \"".$name."\"", "module");

     if($this->field[$from]["min"]>0&&$type=="*:0")
      $this->e->add("Reference from field: \"".$from."\" is required, while relation type is: \"".$type."\" for reference: \"".$name."\"", "module");

     break;
    }
    case "0:*":
    case "1:*":
    {
     if($name==$from)
      $this->e->add("Reference from field: \"".$from."\" can't be the same as the reference name for reference: \"".$name."\"", "module");

     if(!$this->check("properties", "identifier", $from))
      $this->e->add("Reference from field: \"".$from."\" isn't an identifier field for reference: \"".$name."\"", "module");

//     if($d->check("properties", "identifier", $to))
//      $this->e->add("Reference to field: \"".$to."\" in module: \"".$module."\" is an identifier field for reference: \"".$name."\"", "module");

     if($d->field[$to]["min"]===0&&$type=="1:*")
      $this->e->add("Reference to field: \"".$to."\" isn't required, while relation type is: \"".$type."\" for reference: \"".$name."\"", "module");

     if($d->field[$to]["min"]>0&&$type=="0:*")
      $this->e->add("Reference to field: \"".$to."\" is required, while relation type is: \"".$type."\" for reference: \"".$name."\"", "module");

     break;
    }

    case "*:*":
    {
     if($name==$from)
      $this->e->add("Reference from field: \"".$from."\" can't be the same as the reference name for reference: \"".$name."\"", "module");

     if(!$this->check("properties", "identifier", $from))
      $this->e->add("Reference from field: \"".$from."\" isn't an identifier field for reference: \"".$name."\"", "module");

     if(!$d->check("properties", "identifier", $to))
      $this->e->add("Reference to field: \"".$to."\" in module: \"".$module."\" isn't an identifier field for reference: \"".$name."\"", "module");

     break;
    }
   }

   unset($d);

   if($this->e->any("module"))
    return false;

   $this->reference[$name]=Array();
   $this->reference[$name]["from"]=$from;
   $this->reference[$name]["module"]=$module;
   $this->reference[$name]["to"]=$to;
   $this->reference[$name]["type"]=$type;

   if($type=="*:*") //set data object, m2m already proper initialized
   {
    $this->reference[$name]["data"]=&$m2m;

    $fromRef=$this->reference[$name]["data"]->check("use", "from");
    unset($this->reference[$name]["data"]->reference[$fromRef[0]]); //remove return reference
   }

   if(!isset($this->field[$name]))
    $this->addField($name, $this->field[$from]["type"], "form;show", "", $this->field[$from]["min"], $this->field[$from]["max"], $this->field[$from]["defaultvalue"], ucfirst($name), "Reference to: ".$module);

   if($type=="0:*"||$type=="1:*"||$type=="*:*") //properly reset field
   {
    if($this->field[$name]["valuetype"]=="none")
    {
     $this->field[$name]["value"][0]=Array();
     $this->field[$name]["min"]=0;
    }
   }
   else if(($type=="*:1"||$type=="*:0")) //overwrite database values (from) with form / request values (name)
   {
    if($from!=$name)
    {
     $this->field[$from]["valuetype"]=&$this->field[$name]["valuetype"];
     $this->field[$from]["value"]=&$this->field[$name]["value"];
    }
   }

   if($module==$this->module||in_array($module, array_keys($this->extends))) //self reference
   {
    if(!isset($this->field["treedepth"])) //only 1 self reference allow, so duh
     $this->addField("treedepth", "nr", "show", "", 0, false, 0, "Tree depth");

    if(!in_array("tree", $this->actionList)) //add tree view
     $this->addActions("tree");
   }

   return true;
  }


/******************************************************************
 initReferenceData
 initializes the data object of the reference
 - name: reference name, leave empty to initialize all

 returns: true/false
******************************************************************/

  function initReferenceData($name=false)
  {
   if(!$name)
   {
    foreach(array_keys($this->reference) as $name)
     if(!isset($this->reference[$name]["data"]))
      if(!$this->initReferenceData($name))
       return false;

    return true;
   }

   if(!isset($this->reference[$name]))
   {
    $this->e->add("Reference: \"".$name."\" doesn't exist", "system");
    return false;
   }

   if(isset($this->reference[$name]["data"]))
   {
    $this->e->add("Reference: \"".$name."\" data object already initialized", "system");
    return false;
   }

   $this->reference[$name]["data"]=Data::cache($this->reference[$name]["module"], "view", "all", Array("format" => $this->dataformat, "db" => $this->db));
   return true;
  }


/******************************************************************
 getReferenceData
 gets the data object of the reference and initializes if necessary
 - name: reference name

 returns: data object/false
******************************************************************/

  function getReferenceData($name)
  {
   if(!isset($this->reference[$name]))
   {
    $this->e->add("Reference: \"".$name."\" doesn't exist", "system");
    return false;
   }

   if(!isset($this->reference[$name]["data"]))
    if(!$this->initReferenceData($name))
     return false;

   return $this->reference[$name]["data"];
  }


/******************************************************************
 addIndex
 adds an index to a table

 - name: index name
 - field: first index field
 - field: second index field
 - etc...
******************************************************************/

  function addIndex($name)
  {
   if(isset($this->index[$name]))
   {
    $this->e->add("Index: \"".$name."\" already exists", "module");
    return false;
   }

   if(isset($this->unique[$name]))
   {
    $this->e->add("Index: \"".$name."\" already exists as unique index", "module");
    return false;
   }

   if(isset($this->fulltextIndex[$name]))
   {
    $this->e->add("Full text index: \"".$name."\" already exists as full text index", "module");
    return false;
   }


   $tmp=func_get_args();
   array_shift($tmp); //skip name

   $fields=Array();
   foreach($tmp as $field)
   {
    if(isset($this->field[$field]["type"])&&$this->field[$field]["type"]=="version"&&!$this->check("properties", "singleField", $field)) //multi field version
    {
     $fields[]=$field."_major";
     $fields[]=$field."_minor";
     $fields[]=$field."_maintenance";
    }
    else
     $fields[]=$field;
   }
   unset($tmp);

   if(($l=sizeof($fields))==0)
   {
    $this->e->add("Index: \"".$name."\" doesn't have any fields", "module");
    return false;
   }


   if($fields!=array_unique($fields))
   {
    $this->e->add("Index: \"".$name."\" has duplicate fields", "module");
    $error=true;
   }
   else
    $error=false;


   foreach(array_keys($this->index) as $key)
   {
    if($this->index[$key]==$fields)
    {
     $this->e->add("Index: \"".$name."\" has same fields as index: \"".$key."\"", "module");
     $error=true;
    }
   }

   foreach(array_keys($this->unique) as $key)
   {
    if($this->unique[$key]==$fields)
    {
     $this->e->add("Index: \"".$name."\" has same fields as unique index: \"".$key."\"", "module");
     $error=true;
    }
   }

   foreach(array_keys($this->fulltextIndex) as $key)
   {
    if($this->fulltextIndex[$key]==$fields)
    {
     $this->e->add("Full text index: \"".$name."\" has same fields as full text index: \"".$key."\"", "module");
     $error=true;
    }
   }


   foreach($fields as $key => $k)
   {
    if(!isset($this->field[$k]))
    {
     $this->e->add("Index field: \"".$k."\" doesn't exist for index: \"".$name."\"", "module");
     $error=true;
    }
    else if(!$this->check("use", "sql", $k))
    {
     $this->e->add("Index field: \"".$k."\" isn't a SQL field for index: \"".$name."\"", "module");
     $error=true;
    }
   }


   if($error)
    return false;

   $this->index[$name]=$fields;

   return true;
  }

/******************************************************************
 addUnique
 adds an unique index to a table

 - name: unique index name
 - field: first unique field
 - field: second unique field
 - etc...
******************************************************************/

  function addUnique($name)
  {
   if(isset($this->unique[$name]))
   {
    $this->e->add("Unique index: \"".$name."\" already exists", "module");
    return false;
   }

   if(isset($this->index[$name]))
   {
    $this->e->add("Unique index: \"".$name."\" already exists as index", "module");
    return false;
   }

   if(isset($this->fulltextIndex[$name]))
   {
    $this->e->add("Full text index: \"".$name."\" already exists as full text index", "module");
    return false;
   }


   $tmp=func_get_args();
   array_shift($tmp); //skip name

   $fields=Array();
   foreach($tmp as $field)
   {
    if(isset($this->field[$field]["type"])&&$this->field[$field]["type"]=="version"&&!$this->check("properties", "singleField", $field)) //multi field version
    {
     $fields[]=$field."_major";
     $fields[]=$field."_minor";
     $fields[]=$field."_maintenance";
    }
    else
     $fields[]=$field;
   }
   unset($tmp);

   if(($l=sizeof($fields))==0)
   {
    $this->e->add("Unique index: \"".$name."\" doesn't have any fields", "module");
    return false;
   }


   if($fields!=array_unique($fields))
   {
    $this->e->add("Unique index: \"".$name."\" has duplicate fields", "module");
    $error=true;
   }
   else
    $error=false;


   foreach(array_keys($this->index) as $key)
   {
    if($this->index[$key]==$fields)
    {
     $this->e->add("Unique index: \"".$name."\" has same fields as index: \"".$key."\"", "module");
     $error=true;
    }
   }

   foreach(array_keys($this->unique) as $key)
   {
    if($this->unique[$key]==$fields)
    {
     $this->e->add("Unique index: \"".$name."\" has same fields as unique index: \"".$key."\"", "module");
     $error=true;
    }
   }

   foreach(array_keys($this->fulltextIndex) as $key)
   {
    if($this->fulltextIndex[$key]==$fields)
    {
     $this->e->add("Full text index: \"".$name."\" has same fields as full text index: \"".$key."\"", "module");
     $error=true;
    }
   }


   foreach($fields as $key => $k)
   {
    if(!isset($this->field[$k]))
    {
     $this->e->add("Unique index field: \"".$k."\" doesn't exist for index: \"".$name."\"", "module");
     $error=true;
    }
    else if(!$this->check("use", "sql", $k))
    {
     $this->e->add("Unique index field: \"".$k."\" isn't a SQL field for index: \"".$name."\"", "module");
     $error=true;
    }
   }


   if($error)
    return false;

   $this->unique[$name]=$fields;

   return true;
  }

/******************************************************************
 addFullTextIndex
 adds a fulltext index to a table

 - name: index name
 - field: first index field
 - field: second index field
 - etc...
******************************************************************/

  function addFullTextIndex($name)
  {
   if(isset($this->fulltextIndex[$name]))
   {
    $this->e->add("Full text index: \"".$name."\" already exists", "module");
    return false;
   }

   if(isset($this->index[$name]))
   {
    $this->e->add("Unique index: \"".$name."\" already exists as index", "module");
    return false;
   }

   if(isset($this->unique[$name]))
   {
    $this->e->add("Full text index: \"".$name."\" already exists as unique index", "module");
    return false;
   }


   $tmp=func_get_args();
   array_shift($tmp); //skip name

   $fields=Array();
   foreach($tmp as $field)
   {
    if(isset($this->field[$field]["type"])&&$this->field[$field]["type"]=="version"&&!$this->check("properties", "singleField", $field)) //multi field version
    {
     $fields[]=$field."_major";
     $fields[]=$field."_minor";
     $fields[]=$field."_maintenance";
    }
    else
     $fields[]=$field;
   }
   unset($tmp);

   if(($l=sizeof($fields))==0)
   {
    $this->e->add("Full text index: \"".$name."\" doesn't have any fields", "module");
    return false;
   }


   if($fields!=array_unique($fields))
   {
    $this->e->add("Full text index: \"".$name."\" has duplicate fields", "module");
    $error=true;
   }
   else
    $error=false;


   foreach(array_keys($this->index) as $key)
   {
    if($this->index[$key]==$fields)
    {
     $this->e->add("Full text index: \"".$name."\" has same fields as index: \"".$key."\"", "module");
     $error=true;
    }
   }

   foreach(array_keys($this->unique) as $key)
   {
    if($this->unique[$key]==$fields)
    {
     $this->e->add("Full text index: \"".$name."\" has same fields as unique index: \"".$key."\"", "module");
     $error=true;
    }
   }

   foreach(array_keys($this->fulltextIndex) as $key)
   {
    if($this->fulltextIndex[$key]==$fields)
    {
     $this->e->add("Full text index: \"".$name."\" has same fields as full text index: \"".$key."\"", "module");
     $error=true;
    }
   }


   foreach($fields as $key => $k)
   {
    if(!isset($this->field[$k]))
    {
     $this->e->add("Full text index field: \"".$k."\" doesn't exist for index: \"".$name."\"", "module");
     $error=true;
    }
    else
    {
     if(!$this->check("use", "sql", $k))
     {
      $this->e->add("Full text index field: \"".$k."\" isn't a SQL field for index: \"".$name."\"", "module");
      $error=true;
     }
     if(!in_array($this->field[$k]["type"], $this->searchableFieldTypes))
     {
      $this->e->add("Full text index field: \"".$k."\" isn't a searchable field for index: \"".$name."\"", "module");
      $error=true;
     }
    }
   }


   if(isset($this->field[$name]))
   {
    $this->e->add("Full text index: \"".$name."\" already exists as a field", "module");
    $error=true;
   }


   if($error)
    return false;

   $this->fulltextIndex[$name]=$fields;

   $this->addField($name, "float", "show"); //add $name field to hold relevance percentage

   return true;
  }

/******************************************************************
 sortOrder
 sets sort order for fields

 - field: first sort field, allows field+/field- for ascending/descendig, default: ascending
 - field: second sort field
 - etc...
******************************************************************/

  function sortOrder()
  {
   unset($this->sortOrder);
   $this->sortOrder=Array();

   $fields=func_get_args();
   if(isset($fields[0])&&is_array($fields[0]))
    $fields=$fields[0];

   foreach($fields as $k)
   {
    $last=substr($k, -1);
    if($last=="-"||$last=="+")
     $k=substr($k, 0, -1);
    else
     $last="+";

    if(preg_match(REGEXP_DATA_FIELDNAME, $k)&&(isset($this->field[$k])||isset($this->reference[$k])))
     $this->sortOrder[]=$k.$last;
   }
  }

  function resort($fields=false, $from=0, $to=false)
  {
   if($fields===false)
    $fields=$this->sortOrder;

   if(!$fields)
    return;

   if($to===false)
    $to=$this->vmax;

   if(!preg_match("/^([^\-\+]+)(\-|\+)?$/", $fields[0], $field))
    return;

   if(isset($field[2])&&$field[2]=="-")
    $reverse=true;
   else
    $reverse=false;

   $field=$field[1];

   $this->sort($field, $reverse, $from, $to); //sort field for given range
   array_shift($fields); //unset current field from sort array (make sure we won't end up in an endless loop)

   if(sizeof($fields)==0) //no deeper sorting fields, so let's stop
    return;

   $start=$from;
   $value=$this->field[$field]["value"][$from]; //get first value
   for($c=$from+1;$c<=$to;$c++)
   {
    if($c==$to||$this->field[$field]["value"][$c]!=$value) //check difference or end of range
    {
     //difference found, check sub fields in this range
     if(($c-$start)>1)//only for atleast 2 values
      $this->resort($fields, $start, $c);

     if($c==$to)
      break;

     $value=$this->field[$field]["value"][$c];
     $start=$c; //set next starting point for next run
    }
   }
  }

  function sort($field, $reverse=false, $from=false, $to=false)
  {
   if($from===false)
    $from=0;
   if($to===false)
    $to=$this->vmax;

   $tmp=Array();
   if(in_array($this->field[$field]["type"], Array("int", "nr")))
    for($c=$from;$c<$to;$c++) //correct int sorting
     $tmp[$c]=intval($this->field[$field]["value"][$c]);

   else if(in_array($this->field[$field]["type"], Array("float", "price")))
    for($c=$from;$c<$to;$c++) //remove crap from floats
     $tmp[$c]=floatval(str_replace(",", ".", str_replace(" ", "", $this->field[$field]["value"][$c])));

   else if($this->field[$field]["type"]=="date")
    for($c=$from;$c<$to;$c++) //convert dates to sortable
     $tmp[$c]=intval(DT::dateConvert($this->field[$field]["value"][$c], $this->field[$field]["format"][$this->dataformat]["dateformat"]));

   else if($this->field[$field]["type"]=="time")
    for($c=$from;$c<$to;$c++) //convert dates to sortable
     $tmp[$c]=intval(DT::dateConvert($this->field[$field]["value"][$c], $this->field[$field]["format"][$this->dataformat]["timeformat"]));

   else if($this->field[$field]["type"]=="datetime")
    for($c=$from;$c<$to;$c++) //convert dates to sortable
     $tmp[$c]=intval(DT::dateConvert($this->field[$field]["value"][$c], $this->field[$field]["format"][$this->dataformat]["datetimeformat"]));

   else
    for($c=$from;$c<$to;$c++) //ensure case insensitive sort
     $tmp[$c]=strtolower($this->field[$field]["value"][$c]);

   if($reverse)
    arsort($tmp); //sort on field, keeping keys
   else
    asort($tmp); //sort on field, keeping keys

   $keys=array_keys($tmp);

   foreach(array_keys($this->field) as $name) //sort all data fields based on sort order in keys
   {
    if($this->check("use", "show", $name)||$this->check("use", "sql", $name))
    {
     $newValues=Array();
     for($c=0;$c<$this->vmax;$c++)
     {
      if($from<=$c&&$c<$to)
      {
       if(isset($this->field[$name]["value"][$keys[$c-$from]]))
        $newValues[$c]=$this->field[$name]["value"][$keys[$c-$from]];
      }
      else
      {
       if(isset($this->field[$name]["value"][$c]))
        $newValues[$c]=$this->field[$name]["value"][$c];
      }
     }
     $this->field[$name]["value"]=$newValues;
    }
   }
  }

/******************************************************************
 Show/Form fields

 setUseFields
 sets "use" parameter for given fields

 - type: use type: show/form
 - fields: array of fields or array with array as first entry
 - add: set use argument to fields that aren't set yet and leave the others
 - recursive: set use argument for all (active) referenced module

 setShowFields
 sets "use" parameter to "show" for given fields

 - fields: array of fields or array with array as first entry

 setFormFields
 sets "use" parameter to "form" for given fields

 - fields: array of fields or array with array as first entry

******************************************************************/
  function setUseFields($use, $fields=false, $add=false, $recursive=false)
  {
   if($use!="show"&&$use!="form")
   {
    $this->e->add("_SetUseFields use argument should be: show or form", "error");
    return false;
   }

   if(!$fields) //no fields
    $fields=Array();
   else if(!is_array($fields)) //single field
    $fields=Array($fields);
   else if(isset($fields[0])&&is_array($fields[0])) //first array entry contains fields array
    $fields=$fields[0];

   foreach($fields as $name)
    if(isset($this->reference[$name])&&$name!=$this->reference[$name]["from"])
     $fields[]=$this->reference[$name]["from"];

   foreach(array_keys($this->field) as $name)
   {
    if(in_array($name, $fields)) //show it
    {
     if(!$this->check("use", $use, $name))//not show yet
      $this->field[$name]["use"].=";".$use; //show it
    }
    else if(!$add) //not add mode, remove use argument from all other fields
    {
     if($this->check("use", $use, $name)&&!$this->check("properties", "control", $name)) //unset all non control field
      $this->field[$name]["use"]=preg_replace("/".$use."/i", "", $this->field[$name]["use"]);
    }
   }

   if($recursive)
   {
    foreach(array_keys($this->reference) as $name)
    {
     if(isset($this->reference[$name])&&isset($this->reference[$name]["data"]))
     {
      if($this->reference[$name]["type"]=="*:*")
      {
       $d=&$this->reference[$name]["data"]; //get *:* module

       $refto=$d->check("use", "to");
       $refto=$refto[0];

       if(!isset($d->reference[$refto]["data"])) //not active, skip
        continue;

       $d=&$d->reference[$refto]["data"]; //get target module (*:* always skipped)
      }
      else
       $d=&$this->reference[$name]["data"];

      if(!$d->setUseFields($use, $fields, $add, $recursive))
      {
       $this->e->add($d->e);
       return false;
      }
     }
    }
   }

   return true;
  }

  function setShowFields()
  {
   return $this->setUseFields("show", func_get_args(), false);
  }

  function setFormFields()
  {
   return $this->setUseFields("form", func_get_args(), false);
  }


/******************************************************************
******************************************************************/

  function enumOptions($field)
  {
   $options=func_get_args();
   array_shift($options); //skip field

   if(isset($options[0])&&is_array($options[0])) //array provided
    $options=$options[0];

   return $this->listOptions($field, $options);
  }

  function listOptions($field)
  {
   $options=func_get_args();
   array_shift($options); //skip field

   if(isset($options[0])&&is_array($options[0])) //array provided
    $options=$options[0];

   if(!isset($this->field[$field]))
   {
    $this->e->add("List options field: \"".$field."\" doesn't exist", "module");
    return false;
   }

   if(!in_array($this->field[$field]["type"], $this->fieldOptionsFieldTypes))
   {
    $this->e->add("List options not support for field: \"".$field."\"", "module");
    return false;
   }

   $this->listOptions[$field]=Array();

   if($this->field[$field]["type"]=="enum")
   {
    foreach($options as $k) //always threat as indexed array
    {
     if(!preg_match(REGEXP_DATA_ENUMOPTION, $k))
     {
      $this->e->add("Invalid (enum)list option: \"".$k."\"", "error");
      return false;
     }
    }
   }
   else if(array_type($options)=="indexed")
   {
    foreach($options as $k)
    {
     if(!preg_match(REGEXP_DATA_LISTOPTION, $k))
     {
      $this->e->add("Invalid list option: \"".$k."\"", "error");
      return false;
     }
    }
   }
   else //associative
   {
    foreach($options as $k => $v)
    {
     if(!preg_match(REGEXP_DATA_LISTOPTION, $k))
     {
      $this->e->add("Invalid list option: \"".$k."\"", "error");
      return false;
     }
    }
   }

   $this->listOptions[$field]=$options;
   return true;
  }


/******************************************************************
******************************************************************/

 function setFileDir($field, $directory)
 {
  if(!$this->check("type", "localfile", $field)&&!$this->check("type", "localfiles", $field)&&!$this->check("type", "localdirectory", $field))
  {
   $this->e->add("Not a local file/directory field: \"".$field."\"", "module");
   return false;
  }

  if(!is_dir($directory))
  {
   $this->e->add("Invalid local file directory: \"".$directory."\"", "module");
   return false;
  }

  if($directory[strlen($directory)-1]!="/")
   $directory.="/";

  $this->field[$field]["localpath"]["directory"]=$directory;

  //update files
  if(!($options=$this->getFiles($field)))
   $options=Array();

  if(!$this->listOptions($field, $options))
   return false;


  return true;
 }

 function setFileExtensions($field)
 {
  $extensions=func_get_args();
  array_shift($extensions); //skip field

  if(!$this->check("type", "localfile", $field)&&!$this->check("type", "localfiles", $field))
  {
   $this->e->add("Not a local file field: \"".$field."\"", "module");
   return false;
  }

  $this->field[$field]["localpath"]["extensions"]=Array();
  foreach($extensions as $ext)
  {
   if(!preg_match("/^[a-z0-9]+$/i", $ext))
    $this->e->add("Invalid local file extensions: \"".$ext."\"", "module");
   else
    $this->field[$field]["localpath"]["extensions"][]=$ext;
  }

  //update files
  if(!($options=$this->getFiles($field)))
   $options=Array();

  if(!$this->listOptions($field, $options))
   return false;

  return true;
 }

 function getFiles($field)
 {
  if(!$this->check("type", "localfile", $field)&&!$this->check("type", "localfiles", $field)&&!$this->check("type", "localdirectory", $field))
  {
   $this->e->add("Not a local file/directory field: \"".$field."\"", "module");
   return false;
  }

  $options=Array();
  if($this->field[$field]["type"]=="localdirectory")
  {
   foreach(FS::files($this->field[$field]["localpath"]["directory"], $this->check("properties", "recursive", $field), true) as $file)
    if(is_dir($this->field[$field]["localpath"]["directory"].$file))
     $options[]=$file."/";
  }
  else //localfile(s)
  {
   if($this->field[$field]["localpath"]["extensions"]) //extension filtering
   {
    foreach(FS::files($this->field[$field]["localpath"]["directory"], $this->check("properties", "recursive", $field), true) as $file)
    {
     if(!is_file($this->field[$field]["localpath"]["directory"].$file))
      continue;

     if(!preg_match("/\.([a-z0-9]+)$/i", $file, $tmp))
      continue;

     if(!in_array($tmp[1], $this->field[$field]["localpath"]["extensions"]))
      continue;

     $options[]=$file;
    }
   }
   else //all files
   {
    foreach(FS::files($this->field[$field]["localpath"]["directory"], $this->check("properties", "recursive", $field), true) as $file)
     if(is_file($this->field[$field]["localpath"]["directory"].$file))
      $options[]=$file;
   }
  }

  sort($options);

  return $options;
 }


/******************************************************************
******************************************************************/
  function sqlValue($name, $entry=false)
  {
   global $_config;

   if($entry===false)
    $entry=$this->vc;

   if(!array_key_exists($entry, $this->field[$name]["value"])||$this->field[$name]["value"][$entry]===NULL)
    return NULL;

   if(isset($this->field[$name]["operator"][$entry])&&strstr($this->field[$name]["operator"][$entry], "like"))
   {
    if(is_array($this->field[$name]["value"][$entry]))
     return addslashes(implode(";",  $this->field[$name]["value"][$entry]));
    else
     return addslashes($this->field[$name]["value"][$entry]);
   }

   switch($this->dataformat)
   {
    case "sql": //no converting
    {
     switch($this->field[$name]["type"])
     {
      case "price":
      case "float":
       return floatval($this->field[$name]["value"][$entry]);

      case "nr":
       if($this->check("properties", "big", $name))
        return floatval($this->field[$name]["value"][$entry]);
       else
        return intval($this->field[$name]["value"][$entry]);

      case "rights":
       return $this->bin2rights($this->field[$name]["value"][$entry]);

      case "filedata":
       if($v)
       {
        if(($storage=$this->storageObject())&&$storage->type=="mysql")
         return base64_encode($v);
        else
         return $v;
       }
       else
        return false;

      default:
       return addslashes($this->field[$name]["value"][$entry]);
     }
    }
    case "html":
    case "text":
    case "form":
    {
     $v=&$this->field[$name]["value"][$entry];

     if($this->check("properties", "trim", $name)) //trim value
      $v=Mutilate::trim($v);

     switch($this->field[$name]["type"])
     {
      case "html":
      {
       $v=preg_replace("!\<script.*\/script\>!ims", "", $v); //remove script tags
       return addslashes($v);
      }

      case "text":
       return addslashes(preg_replace("/&nbsp; /","  ",  preg_replace("/\<br\/\>\\n/", "\n", html_entity_decode($v))));

      case "password":
      {
       if($this->action=="edit"&&strlen($v)==0)
        return "";
       else if($this->check("properties", "nocrypt", $name)) //module enforced plain text
        return addslashes($v);
       else if($this->check("properties", "md5", $name)) //module enforced md5
        return md5($v);
       else
       {
        //normal password field
        switch($_config["passwordcryptmethod"])
        {
         case "Blowfish":
         {
          //allowed characters in salt
          $chars="./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

          //generate salt
          $salt="$2y$".Mutilate::leadingZeros($_config["passwordcryptcost"], 2)."$";
          for($c=0;$c<21;$c++)
           $salt.=$chars[rand(0, strlen($chars)-1)];

          return addslashes(crypt($v, $salt));
         }
         case "MD5":
         {
          //allowed characters in salt
          $chars="!\"#%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~";

          //generate salt
          $salt="$1$";
          for($c=0;$c<8;$c++)
           $salt.=$chars[rand(0, strlen($chars)-1)];

          $salt.="$";

          return addslashes(crypt($v, $salt));
         }
         case "SHA256":
         {
          //allowed characters in salt
          $chars="!\"#%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~";

          //generate salt
          $salt="$5\$rounds=".$_config["passwordcryptrounds"]."$";
          for($c=0;$c<16;$c++)
           $salt.=$chars[rand(0, strlen($chars)-1)];

          return addslashes(crypt($v, $salt));
         }
         case "SHA512":
         {
          //allowed characters in salt
          $chars="!\"#%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~";

          //generate salt
          $salt="$6\$rounds=".$_config["passwordcryptrounds"]."$";
          for($c=0;$c<16;$c++)
           $salt.=$chars[rand(0, strlen($chars)-1)];

          return addslashes(crypt($v, $salt));
         }
         default: //MD5 Unsalted
          return md5($v);
        }
       }
      }

      case "date": //convertion from dutch format
       if($v=="")
        return "0000-00-00";
       else
        return DT::dateConvert($v, $this->field[$name]["format"][$this->dataformat]["dateformat"], $this->format["sql"]["dateformat"]);

      case "time": //convertion from dutch format
       if($v=="")
        return "00:00:00";
       else
        return DT::dateConvert($v, $this->field[$name]["format"][$this->dataformat]["timeformat"], $this->format["sql"]["timeformat"]);

      case "datetime": //convertion from dutch format
       if($v=="")
        return "0000-00-00 00:00:00";
       else
       {
        if($this->field[$name]["format"][$this->dataformat]["timezone"]!=$this->format["sql"]["timezone"])
         $tmp=DT::timezoneConvert($v, $this->field[$name]["format"][$this->dataformat]["timezone"], $this->format["sql"]["timezone"], $this->field[$name]["format"][$this->dataformat]["datetimeformat"]);
        else
         $tmp=&$v;

        return DT::dateConvert($tmp, $this->field[$name]["format"][$this->dataformat]["datetimeformat"], $this->format["sql"]["datetimeformat"]);
       }

      case "rights":
       if(preg_match("/^([0-1]+)$/", $v)) //binary string
        return $this->bin2rights($v);
       else if(is_numeric($v))
        return $v;
       else
        return 0;

      case "email":
       if(isset($this->field[$name]["mutilation"]["scrambleEmail"])||isset($_config["scrambleemail"])&&$_config["scrambleemail"]) //unscramble if potentially scrabbled
        return Mutilate::unscrambleEmail($v);
       else
        return $v;

      case "ip":
      case "ipv6":
       return $v;

      case "ipv4":
       return ip2long($v);

      case "file":
      case "image":
       return $v;

      case "filedata":
       if($v)
       {
        if(($storage=$this->storageObject())&&$storage->type=="mysql")
         return base64_encode($v);
        else
         return $v;
       }
       else
        return false;

      case "price":
      case "float":
       return floatval(str_replace($this->field[$name]["format"][$this->dataformat]["decimalseparator"], $this->format["sql"]["decimalseparator"], str_replace($this->field[$name]["format"][$this->dataformat]["thousandsseparator"], $this->format["sql"]["thousandsseparator"], $v)));

      case "nr":
       if($this->check("properties", "big", $name))
        return floatval($v);
       else
        return intval($v);

      case "boolean":
       if($v)
        return "true";
       else
        return "false";

      case "multilist":
       if($v)
       {
        if(is_array($v))
         return implode(";", $v);
        else
         return $v;
       }
       else
        return "";

      case "modules":
       if($v)
       {
        if(is_array($v))
         return implode(";", $v);
        else
         return $v;
       }
       else
        return "";

      case "localfiles":
       if($v)
        return implode(";", $v);
       else
        return "";

      default: return $v;
     }
    }
   }
   return false;
  }


/******************************************************************
******************************************************************/


  function setValues(&$storage)
  {
   global $_path, $_config;

   if($this->action=="database"&&$this->extra=="info")
    $k=$this->check("use", "info");
   else
   {
    $k=$this->check("use", "sql");

    if($this->action=="search"&&$this->extra=="index") //add full text index fields for index search
    {
     foreach(array_keys($this->fulltextIndex) as $index)
      if($this->check("use", "show", $index))
       $k[]=$index;
    }
   }



   $s=sizeof($k);

   $this->vmax=0;
   $this->vc=0;

   if($this->action!="user") //don't reset values when logging in, input needed for form
    foreach(array_keys($this->field) as $field)
     if(!$this->check("properties", "control", $field))
      $this->clearValues($field);

   switch($this->dataformat)
   {
//no conversion, direct data access... speeeeeeeeds up so bad
    case "sql":
    {
     while($storage->next())
     {
      for($c=0;$c<$s;$c++)
      {
       if($this->vmax==0)
        $this->field[$k[$c]]["valuetype"]="get";

       if(!isset($storage->res[$k[$c]]))
        $this->field[$k[$c]]["value"][$this->vmax]=NULL;
       else
       {
        switch($this->field[$k[$c]]["type"])
        {
         case "rights":
          $this->field[$k[$c]]["value"][$this->vmax]=$this->rights2bin($storage->res[$k[$c]]);
          break;

         case "filedata":
          if($storage->type=="mysql")
           $this->field[$k[$c]]["value"][$this->vmax]=base64_decode($storage->res[$k[$c]]);
          else
           $this->field[$k[$c]]["value"][$this->vmax]=&$storage->res[$k[$c]];
         break;

         default:
          $this->field[$k[$c]]["value"][$this->vmax]=&$storage->res[$k[$c]];
         break;
        }
       }
      }
      $this->vmax++;
     }

     break;
    }


    case "html":
    case "text":
    case "form":
    {
     while($storage->next())
     {
      for($c=0;$c<$s;$c++)
      {
       if($this->vmax==0)
        $this->field[$k[$c]]["valuetype"]="get";

       if(!isset($storage->res[$k[$c]]))
        $this->field[$k[$c]]["value"][$this->vmax]=NULL;
       else
       {
        switch($this->field[$k[$c]]["type"])
        {
         case "html": //inline html
          $this->field[$k[$c]]["value"][$this->vmax]=stripslashes($storage->res[$k[$c]]);
          $this->field[$k[$c]]["value"][$this->vmax]=preg_replace("!(\"|')[^\"']*(image|file)\.php!", "\\1".$_path["HTML"]["root"]."\\2.php", $this->field[$k[$c]]["value"][$this->vmax]);
         break;

         case "text":
          if($this->dataformat=="html")
          {
           $this->field[$k[$c]]["value"][$this->vmax]=preg_replace("/  /", "&nbsp; ", preg_replace("/\n/", "<br/>\n", stripslashes(htmlentities($storage->res[$k[$c]], ENT_NOQUOTES|ENT_XHTML, "ISO-8859-15"))));
           $this->field[$k[$c]]["value"][$this->vmax]=preg_replace("/\[link=(.*?)( title=(.*?))?\](.*?)\[\/link\]/", "<a href=\"\\1\" title=\"\\3\">\\4</a>", $this->field[$k[$c]]["value"][$this->vmax]);
           $this->field[$k[$c]]["value"][$this->vmax]=preg_replace("/\[file=(.*?)( title=(.*?))?\](.*?)\[\/file\]/", "<a href=\"".$_path["HTML"]["root"]."file.php?\\1\" title=\"\\3\">\\4</a>", $this->field[$k[$c]]["value"][$this->vmax]);
           $this->field[$k[$c]]["value"][$this->vmax]=preg_replace("/\[image=(.*?)( align=(\w+))?( style=(.*?))?( title=(.*?))?\]/", "<img src=\"".$_path["HTML"]["root"]."image.php?\\1\" style=\"float: \\3; \\5\" alt=\"\\7\"/>", $this->field[$k[$c]]["value"][$this->vmax]);
          }
          else //text
           $this->field[$k[$c]]["value"][$this->vmax]=$storage->res[$k[$c]];
         break;

         case "date":
          if($storage->res[$k[$c]]=="0000-00-00")
           $this->field[$k[$c]]["value"][$this->vmax]="";
          else //convert to field datetime format, used when applying mutilations below
           $this->field[$k[$c]]["value"][$this->vmax]=Mutilate::dateConvert($storage->res[$k[$c]], $this->format["sql"]["dateformat"], $this->field[$k[$c]]["format"][$this->dataformat]["dateformat"]);
         break;

         case "time":
          if($storage->res[$k[$c]]=="00:00:00")
           $this->field[$k[$c]]["value"][$this->vmax]="";
          else //convert to field datetime format, used when applying mutilations below
           $this->field[$k[$c]]["value"][$this->vmax]=Mutilate::dateConvert($storage->res[$k[$c]], $this->format["sql"]["timeformat"], $this->field[$k[$c]]["format"][$this->dataformat]["timeformat"]);
         break;

         case "datetime":
          if($storage->res[$k[$c]]=="0000-00-00 00:00:00")
           $this->field[$k[$c]]["value"][$this->vmax]="";
          else
          {
           //convert to field timezone, used when applying mutilations below
           if($this->field[$k[$c]]["format"][$this->dataformat]["timezone"]!=$this->format["sql"]["timezone"])
            $this->field[$k[$c]]["value"][$this->vmax]=Mutilate::timezoneConvert($storage->res[$k[$c]], $this->format["sql"]["timezone"], $this->field[$k[$c]]["format"][$this->dataformat]["timezone"], $this->format["sql"]["datetimeformat"]);
           else
            $this->field[$k[$c]]["value"][$this->vmax]=$storage->res[$k[$c]];

           //convert to field datetime format, used when applying mutilations below
           $this->field[$k[$c]]["value"][$this->vmax]=Mutilate::dateConvert($this->field[$k[$c]]["value"][$this->vmax], $this->format["sql"]["datetimeformat"], $this->field[$k[$c]]["format"][$this->dataformat]["datetimeformat"]);
          }
         break;

         case "rights":
          $this->field[$k[$c]]["value"][$this->vmax]=$this->rights2bin($storage->res[$k[$c]]);
         break;

         case "ipv4":
          $this->field[$k[$c]]["value"][$this->vmax]=long2ip($storage->res[$k[$c]]);
         break;

         case "boolean":
          if($storage->res[$k[$c]]=="true")
           $this->field[$k[$c]]["value"][$this->vmax]=true;
          else
           $this->field[$k[$c]]["value"][$this->vmax]=false;
         break;

         case "password":
          if($this->action=="user"&&$this->extra=="login") //show hash for user login
           $this->field[$k[$c]]["value"][$this->vmax]=$storage->res[$k[$c]];
          else //don't show hash
           $this->field[$k[$c]]["value"][$this->vmax]="";
         break;

         case "filedata":
          if($storage->type=="mysql")
           $this->field[$k[$c]]["value"][$this->vmax]=base64_decode($storage->res[$k[$c]]);
          else
           $this->field[$k[$c]]["value"][$this->vmax]=$storage->res[$k[$c]];
         break;

         case "modules":
         case "multilist":
         case "localfiles":
          if($storage->res[$k[$c]])
           $this->field[$k[$c]]["value"][$this->vmax]=explode(";", $storage->res[$k[$c]]);
          else
           $this->field[$k[$c]]["value"][$this->vmax]=Array();
         break;

         default:
          $this->field[$k[$c]]["value"][$this->vmax]=$storage->res[$k[$c]];
         break;
        }

        if($this->field[$k[$c]]["mutilation"])
         $this->field[$k[$c]]["value"][$this->vmax]=$this->applyMutilations($k[$c], $this->field[$k[$c]]["value"][$this->vmax]);
       }
      }
      $this->vmax++;
     }

     break;
    }
    default: break;
   }

   if($this->vmax==0)
    $this->e->add("No data gathered from database", "warning");

   return true;
  }



/******************************************************************
 get
 Sets DB data to value fields

******************************************************************/
  function get($referDepth=0, $count=false)
  {
   global $_config;

   if($this->e->any("fatal"))
    return false;

   foreach(array_keys($this->extends) as $module)
    if(!$this->extends[$module]->moduleSpecificFunction("PreGetErrorCheck"))
     return false;

   if(!$this->moduleSpecificFunction("PreGetErrorCheck"))
    return false;

   $unshowFields=Array();

   //prepare multi field version fields
   foreach($this->check("type", "version") as $field)
   {
    if(!$this->check("properties", "singleField", $field))
    {
     if($this->check("use", "show", $field)) //set multi fields as show fields
     {
      if(!$this->check("use", "show", $field."_major"))
       $unshowFields[]=$field."_major";
      if(!$this->check("use", "show", $field."_minor"))
       $unshowFields[]=$field."_minor";
      if(!$this->check("use", "show", $field."_maintenance"))
       $unshowFields[]=$field."_maintenance";

      $this->setShowFields(array_merge($this->check("use", "show"), Array($field."_major", $field."_minor", $field."_maintenance")));
     }

     if(!$this->check("valuetype", "none", $field)) //value type not none
     {
      $maintenance=false;
      for($c=0;$c<$this->vmax;$c++)
      {
       if(preg_match("/^(\d+)\.(\d+)\.?(\d*)$/", $this->getValue($field, $c), $m)) //split version value
       {
        //set multi fields values and operators, operant defaults to AND
        $this->setValue($field."_major", $m[1], $c);
        $this->setOperator($field."_major", $this->getOperator($field, $c), $c);
        $this->setValue($field."_minor", $m[2], $c);
        $this->setOperator($field."_minor", $this->getOperator($field, $c), $c);

        if($m[3])
        {
         $maintenance=true;
         $this->setValue($field."_maintenance", $m[3], $c);
         $this->setOperator($field."_maintenance", $this->getOperator($field, $c), $c);
        }
       }

       unset($m);
      }

      //copy value type
      $this->field[$field."_major"]["valuetype"]=$this->field[$field]["valuetype"];
      $this->field[$field."_minor"]["valuetype"]=$this->field[$field]["valuetype"];

      if($maintenance)
       $this->field[$field."_maintenance"]["valuetype"]=$this->field[$field]["valuetype"];
      else
       $this->field[$field."_maintenance"]["valuetype"]="none";

      unset($maintenance);
     }
    }
   }


   //set show from if show total is set
   if($this->check("valuetype", "none", "show_from")&&($this->check("valuetype", "set", "show_total")||$this->check("valuetype", "request", "show_total")))
    $this->setDefaultValue("show_from");

   //get rights reference when coping item
   if($this->action=="copy"&&$refer=$this->getRightsReference())
    $this->setShowFields(array_merge($this->check("use", "show"), Array($this->reference[$refer]["from"])));

   unset($refer);

   //check sort fields for non sql fields
   $resort=Array();
   if($this->action!="database"||$this->extra!="info")
   {
    //replace multi field version fields with sql fields
    $l=sizeof($this->sortOrder);
    for($c=0;$c<$l;$c++)
    {
     if(preg_match("/^(.*)(\-|\+)$/", $this->sortOrder[$c], $field))
     {
      if(isset($this->field[$field[1]]["type"])&&$this->field[$field[1]]["type"]=="version"&&!$this->check("properties", "singleField", $field[1])) //multi field version
      {
       $fields=Array();
       $fields[]=$field[1]."_major".$field[2];
       $fields[]=$field[1]."_minor".$field[2];
       $fields[]=$field[1]."_maintenance".$field[2];

       //replace original version field with new sql fields
       $this->sortOrder=array_merge(array_slice($this->sortOrder, 0, $c), $fields, array_slice($this->sortOrder, $c+1));

       $l+=2;
       $c+=2;
      }
     }
    }

    //find non-sql fields
    foreach($this->sortOrder as $k)
    {
     if(preg_match("/^(.*)(\-|\+)$/", $k, $field)&&!$this->check("use", "sql", $field[1])&&!isset($this->fulltextIndex[$field[1]])&&!isset($this->reference[$field[1]]))
      $resort[]=$field[1];
    }

    //get non-sql fields for resorting
    if($resort)
    {
     foreach($resort as $field)
      if(!$this->check("use", "show", $field))
       $unshowFields[]=$field;

     $this->setShowFields(array_merge($this->check("use", "show"), $resort));
    }
   }


   if(!$this->errorCheck())
    return false;


   foreach(array_keys($this->extends) as $module)
    if(!$this->extends[$module]->moduleSpecificFunction("PostGetErrorCheck"))
     return false;

   if(!$this->moduleSpecificFunction("PostGetErrorCheck"))
    return false;


   foreach(array_keys($this->extends) as $module)
    if(!$this->extends[$module]->moduleSpecificFunction("PreGet"))
     return false;

   if(!$this->moduleSpecificFunction("PreGet"))
    return false;


   //set search value to searchable  fields
   if($this->action=="search")
   {
    switch($this->extra)
    {
     case "title":
     {
      foreach($this->check("properties", "title") as $field)
      {
       if(in_array($this->field[$field]["type"], $this->searchableFieldTypes))
       {
        $this->field[$field]["type"]="search"; //overwrite field type so sqlValue doesn't mess up the value in the query
        $this->field[$field]["valuetype"]="set";
        foreach($this->field["search"]["value"] as $key => $value)
        {
         $this->field[$field]["value"][$key]=$value;
         $this->field[$field]["operator"][$key]="like";
         $this->field[$field]["operant"][$key]="OR";
        }
       }
      }

      $this->field[$field]["operant"][$key]="AND"; //reset last field operant to AND
      break;
     }
     case "field":
     {
      foreach(explode(";", $this->sqlValue("searchFields", 0)) as $field)
      {
       if($this->check("use", "sql", $field))
       {
        if(in_array($this->field[$field]["type"], $this->searchableFieldTypes))
        {
         $this->field[$field]["type"]="search"; //overwrite field type so sqlValue doesn't mess up the value in the query
         $this->field[$field]["valuetype"]="set";
         foreach($this->field["search"]["value"] as $key => $value)
         {
          $this->field[$field]["value"][$key]=$value;
          $this->field[$field]["operator"][$key]="like";
          $this->field[$field]["operant"][$key]="OR";
         }
        }
       }
      }

      $this->field[$field]["operant"][$key]="AND"; //reset last field operant to AND
      break;
     }
     case "show":
     {
      foreach($this->check("use", "show") as $field)
      {
       if($this->check("use", "sql", $field))
       {
        if(in_array($this->field[$field]["type"], $this->searchableFieldTypes))
        {
         $this->field[$field]["type"]="search"; //overwrite field type so sqlValue doesn't mess up the value in the query
         $this->field[$field]["valuetype"]="set";
         foreach($this->field["search"]["value"] as $key => $value)
         {
          $this->field[$field]["value"][$key]=$value;
          $this->field[$field]["operator"][$key]="like";
          $this->field[$field]["operant"][$key]="OR";
         }
        }
       }
      }

      $this->field[$field]["operant"][$key]="AND"; //reset last field operant to AND
      break;
     }
     case "full":
     {
      foreach($this->check("use", "sql") as $field)
      {
       if(in_array($this->field[$field]["type"], $this->searchableFieldTypes))
       {
        $this->field[$field]["type"]="search"; //overwrite field type so sqlValue doesn't mess up the value in the query
        $this->field[$field]["valuetype"]="set";
        foreach($this->field["search"]["value"] as $key => $value)
        {
         $this->field[$field]["value"][$key]=$value;
         $this->field[$field]["operator"][$key]="like";
         $this->field[$field]["operant"][$key]="OR";
        }
       }
      }

      $this->field[$field]["operant"][$key]="AND"; //reset last field operant to AND
      break;
     }
     case "index":
     {
      foreach(array_keys($this->fulltextIndex) as $field)
      {
       $this->field[$field]["type"]="search"; //overwrite field type so sqlValue doesn't mess up the value in the query
       $this->field[$field]["valuetype"]="set";
       foreach($this->field["search"]["value"] as $key => $value)
       {
        $this->field[$field]["value"][$key]=$value;
        $this->field[$field]["operator"][$key]="like";
        $this->field[$field]["operant"][$key]="OR";
       }
      }

      $this->field[$field]["operant"][$key]="AND"; //reset last field operant to AND
      break;
     }
     default: break;
    }
   }


   if(!$storage=$this->storageObject())
    return false;


   if($count)
   {
    switch($this->extra)
    {
     //view
     case "all":
     case "list":
     case "item":
     //search
     case "title":
     case "field":
     case "show":
     case "full":
     case "index":
     {
      if($storage->get(true)&&$storage->next())
       $this->setValue("show_count", $storage->res["count"], 0);
      else
       $this->setValue("show_count", 0, 0);

      break;
     }
     //tree
     case "up":
     {
      if($storage->treeUp(true)&&$storage->next())
       $this->setValue("show_count", $storage->res["count"], 0);
      else
       $this->setValue("show_count", 0, 0);

      break;
     }
     case "down":
     {
      if($storage->treeDown(0, true)&&$storage->next())
       $this->setValue("show_count", $storage->res["count"], 0);
      else
       $this->setValue("show_count", 0, 0);

      break;
     }
    }
   }
   else
    $this->setValue("show_count", 0, 0);

   if($referDepth===false)
   {
    if($this->action=="search")
    {
     //restore field types
     foreach(array_keys($this->field) as $field)
      if($this->field[$field]["type"]!=$this->field[$field]["original"]["type"])
       $this->field[$field]["type"]=$this->field[$field]["original"]["type"];
    }

    return true;
   }

   switch($this->action)
   {
    case "view":
      $succes=$storage->get();
    break;

    case "tree":
    {
     $field=$this->getSelfReference();
     if(!$this->check("use", "show", $field))
     {
      $unshowFields[]=$field;
      $this->setShowFields(array_merge($this->check("use", "show"), Array($field)));
     }

     switch($this->extra)
     {
      case "up":
       $succes=$storage->treeUp();
      break;

      case "down":
       $succes=$storage->treeDown();
      break;
     }
     break;
    }

    case "move":
    {
     $field=$this->getRightsReference();
     if(!$this->check("use", "show", $field))
     {
      $unshowFields[]=$field;
      $this->setShowFields(array_merge($this->check("use", "show"), Array($field)));
     }

     $succes=$storage->get();
     break;
    }

    case "add":
    case "copy":
    case "edit":
    case "users":
    case "delete":
    case "deleted":
    case "undelete":
    case "rights":
    case "owner":
     $succes=$storage->get();
    break;

    case "search":
     $succes=$storage->get();

     //restore field types
     foreach(array_keys($this->field) as $field)
      if($this->field[$field]["type"]!=$this->field[$field]["original"]["type"])
       $this->field[$field]["type"]=$this->field[$field]["original"]["type"];

    break;

    case "lost":
     $succes=$storage->lost();
    break;

    case "database":
     switch($this->extra)
     {
      case "info":
       $succes=$storage->info();
      break;

      case "create":
       $succes=$storage->create();
       $this->e->add($storage->e);
      return $succes;

      case "alter":
       $succes=$storage->alter();
       $this->e->add($storage->e);
      return $succes;

      default:
       $this->e->add("Extra: \"".$this->extra."\" not defined for action: \"".$this->action."\" in get function", "error");
      return false;
     }
    break;

    default:
     //it shouldn't come to this, so errorCheck is broken
     $this->e->add("Action: \"".$this->action."\" not defined in get function", "error");
     return false;
    break;
   }

   $this->e->add($storage->e);

   if(!$succes)
    return false;

   unset($succes);

   if(!$this->setValues($storage))
    return false;

   foreach($this->check("type", "version") as $field)
   {
    if($this->check("properties", "singleField", $field)) //set version field values to multi field version fields
    {
     if($this->check("use", "show", $field)) //show version field
     {
      for($c=0;$c<$this->vmax;$c++)
      {
       if(preg_match("/^(\d+)\.(\d+)\.?(\d*)$/", $this->getValue($field, $c), $m)) //split version value
       {
        //set multi fields values and operators, operant defaults to AND
        $this->setValue($field."_major", (int)$m[1], $c);
        $this->setValue($field."_minor", (int)$m[2], $c);

        if($m[3])
         $this->setValue($field."_maintenance", (int)$m[3], $c);
        else
         $this->setValue($field."_maintenance", 0, $c);
       }

       unset($m);
      }

      //set proper value type (get)
      $this->field[$field."_major"]["valuetype"]=$this->field[$field]["valuetype"];
      $this->field[$field."_minor"]["valuetype"]=$this->field[$field]["valuetype"];
      $this->field[$field."_maintenance"]["valuetype"]=$this->field[$field]["valuetype"];
     }
    }
    else //set multi field version fields values to version field
    {
     if($this->check("use", "show", $field)) //show version field
     {
      for($c=0;$c<$this->vmax;$c++)
      {
       //merge multi fields values
       $m=$this->getValue($field."_major", $c).".".$this->getValue($field."_minor", $c).".".$this->getValue($field."_maintenance", $c);
       $this->setValue($field, $m, $c);

       //set proper value type (get)
       $this->field[$field]["valuetype"]=$this->field[$field."_major"]["valuetype"];
       unset($m);
      }
     }
    }
   }

   //determine tree depth value
   if($this->action=="tree"&&$this->check("use", "show", "treedepth"))
   {
    $treedepth=-1;
    $prevs=Array();
    $refer=$this->getSelfReference();
    $refer=$this->reference[$refer]["from"];

    while($this->next())
    {
     if(!in_array($this->v[$refer], $prevs)) //new: level deeper
     {
      $treedepth++;
      $prevs[$treedepth]=$this->v[$refer];
     }
     else
     {
      while($this->v[$refer]!=$prevs[$treedepth]) //not current: upper level
      {
       unset($prevs[$treedepth]);
       $treedepth--;
      }
      //else stay on current level
     }

     $this->setValue("treedepth", $treedepth, $this->vc-1);

     $prev=$this->v[$refer];
    }

    $this->v=Array(); //reset values array to half values to be set
   }

   if(!$this->referenceLevel)
    $this->referenceLevel=$referDepth;


   foreach(array_keys($this->extends) as $module)
    if(!$this->extends[$module]->moduleSpecificFunction("PostGet"))
     return false;

   if(!$this->moduleSpecificFunction("PostGet"))
    return false;


   //resort for non-sql fields
   if($resort)
    $this->resort();


   //restore show status for fields that were orignially not shown (version, resort, selfReference, etc)
   if($unshowFields)
   {
    $showFields=$this->check("use", "show");

    $this->setShowFields(array_diff($showFields, $unshowFields)); //restore old show fields
    unset($showFields, $unshowFields);
   }


   foreach(array_keys($this->extends) as $module)
    if(!$this->extends[$module]->moduleSpecificFunction("PreGetReferences"))
     return false;

   if(!$this->moduleSpecificFunction("PreGetReferences"))
    return false;


   //get reference information
   foreach(array_keys($this->reference) as $name)
   {
    if(!$this->check("use", "show", $name))
     continue;

    $ref=&$this->reference[$name];

    if($this->action=="database"&&$this->extra=="info")
    {
     if($referDepth>0||$ref["type"]=="*:*")
     {
      if(!isset($ref["data"]))
      {
       $this->initReferenceData($name);
       $ref["data"]->setShowFields(); //don't show anything
      }

      $ref["data"]->setAction("database");
      $ref["data"]->setExtra("info");

      if($ref["type"]=="*:*")
      {
       if(!$ref["data"]->get(($referDepth)))
       {
        $this->e->add($ref["data"]->e);
        return false;
       }
      }
      else
      {
       if(!$ref["data"]->get(($referDepth-1)))
       {
        $this->e->add($ref["data"]->e);
        return false;
       }
      }
     }
    }
    //get values (no view->info)
    else if($ref["type"]=="*:*")
    {
     //don't bother creating $ref["data"], already done in addReference for *:*

     //get all id's from m2m table
     $ref["data"]->setAction("view");
     $ref["data"]->setExtra("all");
     $ref["data"]->clearValues();

     //get from/to fields from m2m module
     $reffrom=$ref["data"]->check("use", "from");
     $reffrom=$reffrom[0];
     $refto=$ref["data"]->check("use", "to");
     $refto=$refto[0];

     $c=0;
     foreach(array_keys($this->field[$ref["from"]]["value"]) as $k)
     {
      if($v=$this->sqlValue($ref["from"], $k))
      {
       if($ref["data"]->field[$reffrom]["valuetype"]=="none"||!in_array($v, $ref["data"]->field[$reffrom]["value"]))
       {
        $ref["data"]->setValue($reffrom, $v, $c);
        $ref["data"]->setOperant($reffrom, "OR", $c);
        $c++;
       }
      }
     }

     //ensure both reference fields are gathered, back reference (reffrom) already removed (in addReference) so no chance of duplicate data
     $ref["data"]->setShowFields($reffrom, $refto);

     //get referenced data
     if($c&&!$ref["data"]->get(($referDepth)))
     {
      $this->e->add($ref["data"]->e);
      return false;
     }

     //set id's to reference field directly for no depth
     foreach(array_keys($this->field[$ref["from"]]["value"]) as $k)
     {
      $this->field[$name]["value"][$k]=Array();
      for($c=0;$c<$ref["data"]->vmax;$c++)
       if($ref["data"]->field[$reffrom]["value"][$c]==$this->field[$ref["from"]]["value"][$k])
        $this->field[$name]["value"][$k][]=$ref["data"]->field[$refto]["value"][$c];
     }
    }
    else if(($ref["type"]=="0:*"||$ref["type"]=="1:*"))
    {
     if(!isset($ref["data"]))
      $this->initReferenceData($name);

     //get all id's from referenced table
     $ref["data"]->setAction("view");
     $ref["data"]->setExtra("all");
     $ref["data"]->clearValues();

     $showFields=$ref["data"]->check("use", "show");

     if($referDepth==0) //only get id's for no depth
      $ref["data"]->setShowFields($ref["to"]);
     else //get normal showFields and to field
      $ref["data"]->setShowFields(array_merge(Array($ref["to"]), $showFields));

     //ensure back reference doesn't gather duplicate data
     foreach(array_keys($ref["data"]->reference) as $reference) //look up counter part
     {
      if(
       ($this->module==$ref["data"]->reference[$reference]["module"])&& //module must match
       ($ref["from"]==$ref["data"]->reference[$reference]["to"])&& //from must match reference to
       ($ref["to"]==$ref["data"]->reference[$reference]["from"])&& //to must match reference from
       (($ref["type"]=="0:*"&&$ref["data"]->reference[$reference]["type"]=="*:0")||($ref["type"]=="1:*"&&$ref["data"]->reference[$reference]["type"]=="*:1")) //reference type should match its opposite
      )
      {
       $ref["data"]->getReferenceData($reference)->setShowFields(); //no show fields for back reference
       break; //done searching
      }
     }

     $c=0;
     foreach(array_keys($this->field[$ref["from"]]["value"]) as $k)
     {
      if($v=$this->sqlValue($ref["from"], $k))
      {
       if(!in_array($v, $ref["data"]->field[$ref["to"]]["value"]))
       {
        $ref["data"]->setValue($ref["to"], $v, $c);
        $ref["data"]->setOperant($ref["to"], "OR", $c);
        $c++;
       }
      }
     }

     //get referenced data
     if($c&&!$ref["data"]->get(($referDepth-1)))
     {
      $this->e->add($ref["data"]->e);
      return false;
     }

     //set id's as field values
     $refID=$ref["data"]->getID();
     foreach(array_keys($this->field[$ref["from"]]["value"]) as $k)
     {
      $this->field[$name]["value"][$k]=Array();
      for($c=0;$c<$ref["data"]->vmax;$c++)
       if($ref["data"]->field[$ref["to"]]["value"][$c]==$this->field[$ref["from"]]["value"][$k])
        $this->field[$name]["value"][$k][]=$ref["data"]->field[$refID]["value"][$c];
     }

     //restore showFields
     $ref["data"]->setShowFields($showFields);
     unset($showFields);
    }
    else  //*:0, *:1 and 1:1
    {
     if(!isset($ref["data"]))
      $this->initReferenceData($name);

     $ref["data"]->clearValues(); //clear values, regardless if information is gathered

     if($referDepth==0&&$ref["type"]=="1:1") //set reference id (same as current)
      $this->field[$name]["value"]=$this->field[$ref["from"]]["value"];
     else if($referDepth>0)
     {
      if(!$ref["data"]->check("use", "show")||$ref["data"]->check("use", "show")==Array("r")) //no additional information is gathered (ID is already available in local table)
       continue;

      //get all id's from m2m table
      $ref["data"]->setAction("view");
      $ref["data"]->setExtra("all");

      $c=0;
      $ref["data"]->unsetValue($ref["to"]);
      foreach(array_keys($this->field[$ref["from"]]["value"]) as $k)
      {
       if($v=$this->sqlValue($ref["from"], $k))
       {
        if(!in_array($v, $ref["data"]->field[$ref["to"]]["value"]))
        {
         $ref["data"]->setValue($ref["to"], $v, $c);
         $ref["data"]->setOperant($ref["to"], "OR", $c);
         $c++;
        }
       }
      }

      //get referenced data
      if($c&&!$ref["data"]->get(($referDepth-1)))
      {
       $this->e->add($ref["data"]->e);
       return false;
      }
     }
    }
   }


   foreach(array_keys($this->extends) as $module)
    if(!$this->extends[$module]->moduleSpecificFunction("PostGetReferences"))
     return false;

   if(!$this->moduleSpecificFunction("PostGetReferences"))
    return false;


   return true;
  }



 /******************************************************************
 submit
 Submits data to correct SQL function
 Returns errors

******************************************************************/
  function submit()
  {
   global $_config;

   if($this->e->any("fatal"))
    return false;

   $succes=true;

   if(!$this->moduleSpecificFunction("PreSubmitErrorCheck"))
    return false;

   //prepare multi field version fields
   foreach($this->check("type", "version") as $field)
   {
    if(!$this->check("properties", "singleField", $field))
    {
     if(!$this->check("valuetype", "none", $field)) //value type not none
     {
      for($c=0;$c<$this->vmax;$c++)
      {
       if(preg_match("/^(\d+)\.(\d+)\.?(\d*)$/", $this->getValue($field, $c), $m)) //split version value
       {
        //set multi fields values
        $this->setValue($field."_major", $m[1], $c);
        $this->setValue($field."_minor", $m[2], $c);

        if($m[3])
         $this->setValue($field."_maintenance", $m[3], $c);
        else
         $this->setValue($field."_maintenance", 0, $c);
       }

       unset($m);
      }

      //copy value type
      $this->field[$field."_major"]["valuetype"]=$this->field[$field]["valuetype"];
      $this->field[$field."_minor"]["valuetype"]=$this->field[$field]["valuetype"];
      $this->field[$field."_maintenance"]["valuetype"]=$this->field[$field]["valuetype"];
     }
    }
   }

   if($this->action=="add"||$this->action=="copy"||$this->action=="edit"||$this->action=="move"||$this->action=="order"||$this->action=="delete"||$this->action=="deleted"||$this->action=="undelete"||$this->action=="rights"||$this->action=="owner"||$this->action=="user")
   {
    if($this->vmax==0)
    {
     $this->e->add("No values in form", "error");
     return false;
    }

    if($this->action=="add"||$this->action=="copy")
    {
     if($orderfields=$this->check("properties", "order"))
     {
      //set order nr
      if(count($this->check("reference", "categories")))
       $d=Data::cache($this->module, "view", "list", Array("format" => $this->dataformat, "db" => $this->db));
      else
       $d=Data::cache($this->module, "view", "all", Array("format" => $this->dataformat, "db" => $this->db));

      foreach($orderfields as $orderfield)
      {
       if($this->getValue($orderfield)==0)
       {
        $d->clearValues();

        if($d->extra=="list")
         $d->setValue("catnr", $this->getValue("catnr"));

        $d->setShowFields("nr", $orderfield);
        $d->sortOrder($orderfield."-");
        $d->setValue("show_total", 1);

        $ordernr=1;
        if($d->get()&&$d->next())
         $ordernr=$d->v[$orderfield]+1;

        for($c=0;$c<$this->vmax;$c++)
         $this->setValue($orderfield, $ordernr++, $c);
       }
      }
      unset($d, $orderfields, $orderfield, $ordernr);
     }

     //set default values
     foreach($this->check("valuetype", "none") as $name)
      for($c=0;$c<$this->vmax;$c++)
       $this->setDefaultValue($name, $c);
    }
    else // update/delete
    {
     if($this->action=="user"&&$this->extra=="password")
     {
      foreach($this->check("properties", "owner") as $name)
       for($c=0;$c<$this->vmax;$c++)
        $this->setValue($name, $_config["usernr"], $c);
     }

     //unset values for non set fields
     foreach($this->check("valuetype", "default") as $name)
      for($c=0;$c<$this->vmax;$c++)
       $this->unsetValue($name, $c);

     if($this->dataformat!="sql")
     {
      //don't update noUpdate fields
      foreach($this->check("properties", "noUpdate") as $name)
       for($c=0;$c<$this->vmax;$c++)
        $this->unsetValue($name, $c);

      //overwrite values in forceUpdate fields with default
      foreach($this->check("properties", "forceUpdate") as $name)
       for($c=0;$c<$this->vmax;$c++)
        $this->setDefaultValue($name, $c);
     }
    }

    //unset all "deleted" field values, only set in case of delete/undelete
    if($this->validAction("undelete"))
     foreach($this->check("properties", "deleted") as $field)
      $this->field[$field]["valuetype"]="none";
   }

   if(!$storage=$this->storageObject())
    return false;

   //check rights
   switch($this->action)
   {
    case "add":
    case "copy":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("add"))
       $succes=false;

     break;
    }
    case "edit":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("edit"))
       $succes=false;

     break;
    }
    case "move":
    {
     //check new category for add rights
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      $this->setValue("catnr", $this->getValue("newcatnr", $this->vc), $this->vc);

      if(!$storage->checkRights("add")||!$storage->checkRights("delete"))
       $succes=false;
     }

     break;
    }
    case "delete":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("delete"))
       $succes=false;

     break;
    }
    case "undelete":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("undelete"))
       $succes=false;

     break;
    }
    case "owner":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("owner"))
       $succes=false;

     break;
    }
    case "rights":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("rights"))
       $succes=false;

     break;
    }
    case "user":
    {
     //login: no rights check, done in get query
     //logout: whatever
     //password: owner (the actual user) may change, owner force at begin of this function
     break;
    }
    case "order":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("edit"))
       $succes=false;

     break;
    }
    case "database":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      if(!$storage->checkRights("database"))
       $succes=false;

     break;
    }
    default:
    {
     $this->e->add("Action: \"".$this->action."\" not defined in submit function", "error");
     return false;
    }
   }

   if(!$succes)
   {
    $this->e->add($storage->e);
    return false;
   }

   if(!$this->errorCheck(true))
    $succes=false;

   if(!$this->moduleSpecificFunction("PostSubmitErrorCheck"))
    $succes=false;

   if(!$succes) //error checking failed
    return false;

   if(!$this->moduleSpecificFunction("PreSubmit"))
    return false;

   //submit extends before current module
   if(
    $this->action=="add"||
    $this->action=="copy"||
    $this->action=="edit"||
    $this->action=="move"||
    //delete extends done after this module is done
    $this->action=="owner"||
    ($this->action=="user"&&$this->extra=="password"))
   {
    if($this->extends)
    {
     foreach(array_keys($this->extends) as $module)
     {
      if(!$this->extends[$module]->submit())
      {
       $this->e->add($this->extends[$module]->e);
       $succes=false;
       break;
      }

      if($this->action=="add"||$this->action=="copy")
      {
       if(!$this->extends[$module]->extends) //top level, get insert id's
       {
        $id=$this->getID();
        $id2=$this->extends[$module]->getID();

        for($c=0;$c<$this->extends[$module]->vmax;$c++)
         $this->setValue($id, $this->extends[$module]->getValue($id2, $c), $c);
       }
      }
     }
    }
   }

   if(!$succes)
    return false;

   switch($this->action)
   {
    case "add":
    case "copy":
    {
     foreach(array_keys($this->reference) as $name)
     {
      $ref=&$this->reference[$name];
      switch($ref["type"])
      {
       case "1:1":
       {
        $d=Data::cache($ref["module"], $this->action, $this->extra, Array("format" => $this->dataformat, "db" => $this->db), md5($name));
        $d->clearValues();

        for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
         foreach($this->check("properties", "default") as $field)
          $d->setValue($field, $this->getValue($field, $this->vc), $this->vc);

        $d->vmax=$this->vmax;

        if(!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
         $this->setValue($name, $d->getValue($ref["to"], $this->vc), $this->vc);

        unset($d);
        break;
       }
       default: break;
      }
     }

     if(!$succes)
      break;

     for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //add each value
      if(!$storage->add())
       $succes=false;

     if(!$succes) //failed to insert, cleanup extended modules as well
     {
      foreach(array_keys($this->extends) as $module)
      {
       $d=$this->extends[$module]->copy();
       $d->clearValues();
       $d->setAction("delete");
       $d->setExtra("item");

       $id=$this->getID();
       $id2=$d->getID();

       for($c=0;$c<$this->extends[$module]->vmax;$c++)
        $d->setValue($id2, $this->getValue($id, $c), $c);

       $d->submit();

       unset($d);
      }

      break;
     }

     foreach(array_keys($this->reference) as $name)
     {
      $ref=&$this->reference[$name];

      if($this->module!=$this->field[$name]["module"]) //skip references defined in extended modules
       continue;

      switch($ref["type"])
      {
       case "0:*":
       case "1:*":
       {
        $d=Data::cache($ref["module"], "view", "all", Array("format" => $this->dataformat, "db" => $this->db));
        $d->clearValues();

        $id=$this->getID();
        $refid=$d->getID();

        $d->unsetValue($refid);

        $add=Array();
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         $add[$k]=Array();
         if(is_array($this->field[$name]["value"][$k]))
         {
          foreach($this->field[$name]["value"][$k] as $v)
           if(!in_array($v, $d->field[$refid]["value"]))
           $add[$k][]=$v;
         }
        }

        $d->clearValues();
        $d->action="edit";
        $d->extra="item";

        $c=0;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         foreach($add[$k] as $v)
         {
          $d->setValue($refid, $v, $c);
          $d->setValue($ref["to"], $this->sqlValue($id, $k), $c);
          $c++;
         }
        }

        if($c&&!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        unset($d);
        break;
       }
       case "*:*":
       {
        $d=$ref["data"]->copy();
        $d->action="view";
        $d->extra="all";

        $id=$this->getID();

        //get from/to fields from m2m module
        $reffrom=$ref["data"]->check("use", "from");
        $reffrom=$reffrom[0];
        $refto=$ref["data"]->check("use", "to");
        $refto=$refto[0];

        $d->clearValues();

        $add=Array();
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         $add[$k]=Array();

         if(is_array($this->field[$name]["value"][$k]))
         {
          foreach($this->field[$name]["value"][$k] as $v)
           if(!in_array($v, $d->field[$refto]["value"]))
            $add[$k][]=$v;
         }
        }

        $d->e->clear();
        $d->clearValues();
        $d->action="add";
        $d->extra="item";

        $c=0;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         foreach($add[$k] as $v)
         {
          $d->setValue($refto, $v, $c);
          $d->setValue($reffrom, $this->sqlValue($id, $k), $c);
          $d->setDefaultValue("userrights", $c);
          $c++;
         }
        }

        if($c&&!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        unset($d);
        break;
       }
       default: break;
      }
     }

     if($succes)
     {
      if($this->action=="copy")
       $this->e->add($this->title." copied successfully", "note", $this->vmax." items");
      else //add
       $this->e->add($this->title." added successfully", "note", $this->vmax." items");
     }

     break;
    }

    case "edit":
    {
     foreach(array_keys($this->reference) as $name)
     {
      $ref=&$this->reference[$name];
      switch($ref["type"])
      {
       case "1:1":
       {
        $d=Data::cache($ref["module"], $this->action, $this->extra, Array("format" => $this->dataformat, "db" => $this->db), md5($name));
        $d->clearValues();

        for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
         foreach($this->check("properties", "default") as $field)
          $d->setValue($field, $this->getValue($field, $this->vc), $this->vc);

        $d->vmax=$this->vmax;

        if(!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        unset($d);
        break;
       }
       default: break;
      }
     }

     if(!$succes)
      break;

     if($this->dataformat!="sql")
     {
      foreach($this->check("properties", "owner") as $field) //don't update owner
       $this->field[$field]["valuetype"]="none";

      foreach($this->check("properties", "rightsReference") as $field) //don't update rightsReference
       $this->field[$field]["valuetype"]="none";
     }

     for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //edit each value
     {
      foreach($this->check("use", "sql") as $field) //don't update password if empty
      {
       if($this->check("properties", "password", $field))
       {
        if(!$this->getValue($field, $this->vc))
         $this->field[$field]["valuetype"]="none";
        else
         $this->field[$field]["valuetype"]="set";
       }
      }

      if(!$storage->edit())
       $succes=false;
     }

     if(!$succes)
      break;

     foreach(array_keys($this->reference) as $name)
     {
      $ref=&$this->reference[$name];

      if($this->check("valuetype", "none", $name)) //reference not set
       continue;

      if($this->module!=$this->field[$name]["module"]) //skip references defined in extended modules
       continue;

      switch($ref["type"])
      {
       case "0:*":
       case "1:*":
       {
        $d=Data::cache($ref["module"], "view", "all", Array("format" => $this->dataformat, "db" => $this->db));
        $d->clearValues();

        $id=$this->getID();
        $refid=$d->getID();

        $c=0;
        $get=false;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         if($v=$this->sqlValue($id, $k))
         {
          if(!in_array($v, $d->field[$ref["to"]]["value"]))
          {
           $d->setValue($ref["to"], $v, $c);
           $d->setOperant($ref["to"], "OR", $c);
           $get=true;
           $c++;
          }
         }
        }

        $d->setShowFields($refid);

        if($get&&!$d->get())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        $add=Array();
        $delete=Array();
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         $add[$k]=Array();
         $delete[$k]=Array();
         while($d->next())
         {
          if(!in_array($d->v[$refid], $this->field[$name]["value"][$k]))
           $delete[$k][]=$d->v[$refid];
         }

         foreach($this->field[$name]["value"][$k] as $v)
         {
          if(!in_array($v, $d->field[$refid]["value"]))
           $add[$k][]=$v;
         }
        }

        $d->clearValues();
        $d->action="edit";
        $d->extra="item";

        $c=0;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         foreach($add[$k] as $v)
         {
          $d->setValue($refid, $v, $c);
          $d->setValue($ref["to"], $this->sqlValue($id, $k), $c);
          $c++;
         }
         foreach($delete[$k] as $v)
         {
          $d->setValue($refid, $v, $c);
          $d->setValue($ref["to"], 0, $c);
          $c++;
         }
        }

        if($c&&!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        unset($d);
        break;
       }
       case "*:*":
       {
        $d=$ref["data"]->copy();
        $d->action="view";
        $d->extra="all";

        $id=$this->getID();

        //get from/to fields from m2m module
        $reffrom=$ref["data"]->check("use", "from");
        $reffrom=$reffrom[0];
        $refto=$ref["data"]->check("use", "to");
        $refto=$refto[0];

        $d->clearValues();

        $c=0;
        $get=false;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         if($v=$this->sqlValue($id, $k))
         {
          if(!in_array($v, $d->field[$reffrom]["value"]))
          {
           $d->setValue($reffrom, $v, $c);
           $d->setOperant($reffrom, "OR", $c);
           $get=true;
           $c++;
          }
         }
        }

        $d->setShowFields($refto);
        $d->getReferenceData($refto)->setShowFields();

        if($get&&!$d->get())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        $add=Array();
        $delete=Array();
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         $add[$k]=Array();
         $delete[$k]=Array();

         while($d->next())
         {
          if(!is_array($this->field[$name]["value"][$k])||!in_array($d->v[$refto], $this->field[$name]["value"][$k]))
           $delete[$k][]=$d->v[$refto];
         }

         if(is_array($this->field[$name]["value"][$k]))
         {
          foreach($this->field[$name]["value"][$k] as $v)
          {
           if(!in_array($v, $d->field[$refto]["value"]))
            $add[$k][]=$v;
          }
         }
        }

        $d->e->clear();
        $d->clearValues();
        $d->action="add";
        $d->extra="item";

        $c=0;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         foreach($add[$k] as $v)
         {
          $d->setValue($refto, $v, $c);
          $d->setValue($reffrom, $this->sqlValue($id, $k), $c);
          $d->setDefaultValue("userrights", $c);
          $c++;
         }
        }

        if($c&&!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        $d->e->clear();
        $d->clearValues();
        $d->action="delete";
        $d->extra="item";

        $d->field[$reffrom]["properties"]="identifier";
        $d->field[$refto]["properties"]="identifier";

        $c=0;
        foreach(array_keys($this->field[$name]["value"]) as $k)
        {
         foreach($delete[$k] as $v)
         {
          $d->setValue($refto, $v, $c);
          $d->setValue($reffrom, $this->sqlValue($id, $k), $c);
          $d->setDefaultValue("userrights", $c);
          $c++;
         }
        }

        if($c&&!$d->submit())
        {
         $this->e->add($d->e);
         $succes=false;
        }

        $d->field[$reffrom]["properties"]="";
        $d->field[$refto]["properties"]="";

        unset($d);
        break;
       }
       default: break;
      }
     }

     if($succes)
      $this->e->add($this->title." updated successfully", "note", $this->vmax." items");

     break;
    }

    case "move":
    {
     $d2=$this->copy();
     $d2->setShowFields("catnr");
     $d2->action="view";
     $d2->extra="item";

     if($d2->get())
      $catnr=$d2->getValue("catnr");
     else
      $catnr=false;

     $refer=$this->getRightsReference();
     $rightsReference=$this->reference[$refer]["from"];

     foreach($this->check("use", "sql") as $field) //only rightsReference+forceUpdate
      if($field!=$rightsReference&&!$this->check("properties", "forceUpdate", $field))
       $this->field[$field]["valuetype"]="none";

     for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //move each category
     {
      if(!($newcatnr=$this->getValue("newcatnr", $this->vc))) //allow per entry and just 1 newcatnr
       $newcatnr=$this->getValue("newcatnr");

      $this->setValue($rightsReference, $newcatnr, $this->vc);

      if(!$storage->edit())
       $succes=false;
     }

     if(!$succes)
      break;

     //remove module from non-locked categories if no more items
     if($this->module!="categories"&&$this->module!="userRights"&&$this->module!="groupRights"&&count($this->check("properties", "rightsReference"))>0)
     {
      $d=Data::cache("categories", "view", "item", Array("format" => $this->dataformat, "db" => $this->db));

      //only check if category has been found and field type is modules
      if($catnr&&$d->field["module"]["type"]=="modules")
      {
       $d->clearValues();
       $d->setShowFields("module", "moduleLock");
       $d->setValue("nr", $catnr);

       if($d->get()&&$d->next())
       {
        if(isset($d->v["moduleLock"])&&!$d->v["moduleLock"]) //category modules locked?
        {
         $d2->clearValues();
         $d2->extra="list";
         $d2->setValue("catnr", $catnr);

         if($d2->get(false, true))
         {
          if($d2->getValue("show_count")==0) //last item has been removed
          {
           $modules=Array();
           foreach($d->v["module"] as $module)
            if($module!=$this->module)
             $modules[]=$module;

           $d->clearValues();
           $d->setAction("edit");

           $d->setValue("nr", $catnr);
           $d->setValue("module", $modules);

           unset($module, $modules);

           if(!$d->submit())
            $this->e->add($d->e);
          }
         }
        }
       }
      }

      unset($d);
     }

     if($succes)
      $this->e->add($this->title." moved successfully", "note", $this->vmax." items");

     unset($d2, $catnr);
     break;
    }

    case "delete":
    {
     if($this->validAction("undelete")) //"deleted" field, don't actually delete entry
     {
      foreach($this->check("use", "sql") as $field) //unset all values
       $this->field[$field]["valuetype"]="none";

      foreach($this->check("properties", "deleted") as $field) //set deleted to false
       for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //delete for each value
        $this->setValue($field, true, $this->vc);

      for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //"delete" for each value
       if(!$storage->edit())
        $succes=false;
     }
     else
     {
      $d2=$this->copy();
      $d2->setShowFields("catnr");
      $d2->action="view";
      $d2->extra="item";

      if($d2->get())
       $catnr=$d2->getValue("catnr");
      else
       $catnr=false;

      for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //delete each value
       if(!$storage->delete())
        $succes=false;

      if(!$succes)
       break;

      //remove module from non-locked categories if no more items
      if($this->module!="categories"&&$this->module!="userRights"&&$this->module!="groupRights"&&count($this->check("properties", "rightsReference"))>0)
      {
       $d=Data::cache("categories", "view", "item", Array("format" => $this->dataformat, "db" => $this->db));

       //only check if category has been found and field type is modules
       if($catnr&&$d->field["module"]["type"]=="modules")
       {
        $d->clearValues();
        $d->setShowFields("module", "moduleLock");
        $d->setValue("nr", $catnr);

        if($d->get()&&$d->next())
        {
         if(isset($d->v["moduleLock"])&&!$d->v["moduleLock"]) //category modules locked?
         {
          $d2->clearValues();
          $d2->extra="list";
          $d2->setValue("catnr", $catnr);

          if($d2->get(false, true))
          {
           if($d2->getValue("show_count")==0) //last item has been removed
           {
            $modules=Array();
            foreach($d->v["module"] as $module)
             if($module!=$this->module)
              $modules[]=$module;

            $d->clearValues();
            $d->setAction("edit");

            $d->setValue("nr", $catnr);
            $d->setValue("module", $modules);

            unset($module, $modules);

            if(!$d->submit())
             $this->e->add($d->e);
           }
          }
         }
        }
       }

       unset($d);
      }

      unset($d2, $catnr);

      foreach(array_keys($this->reference) as $name)
      {
       $ref=&$this->reference[$name];
       switch($ref["type"])
       {
        case "1:1":
        {
         $d=Data::cache($ref["module"], $this->action, $this->extra, Array("format" => $this->dataformat, "db" => $this->db), md5($name));
         $d->clearValues();

         for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
          foreach($this->check("properties", "default") as $field)
           $d->setValue($field, $this->getValue($field, $this->vc), $this->vc);

         if(!$d->submit())
         {
          $this->e->add($d->e);
          $succes=false;
         }

         unset($d);
         break;
        }
        case "0:*":
        case "1:*":
        {
         $d=Data::cache($ref["module"], "view", "all", Array("format" => $this->dataformat, "db" => $this->db));
         $d->clearValues();

         $id=$this->getID();
         $refid=$d->getID();

         $d->unsetValue($refid);

         $c=0;
         $get=false;
         foreach(array_keys($this->field[$id]["value"]) as $k)
         {
          if($v=$this->sqlValue($id, $k))
          {
           if(!in_array($v, $d->field[$ref["to"]]["value"]))
           {
            $d->setValue($ref["to"], $v, $c);
            $d->setOperant($ref["to"], "OR", $c);
            $get=true;
            $c++;
           }
          }
         }

         $d->setShowFields($refid);

         if($get&&!$d->get())
         {
          $this->e->add($d->e);
          $succes=false;
         }

         $delete=Array();
         foreach(array_keys($this->field[$name]["value"]) as $k)
         {
          $delete[$k]=Array();
          while($d->next())
          {
           $delete[$k][]=$d->v[$refid];
          }
         }

         $d->clearValues();
         $d->action="edit";
         $d->extra="item";

         $c=0;
         foreach(array_keys($this->field[$name]["value"]) as $k)
         {
          foreach($delete[$k] as $v)
          {
           $d->setValue($refid, $v, $c);
           $d->setValue($ref["to"], 0, $c);
           $c++;
          }
         }

         if($c&&!$d->submit())
         {
          $this->e->add($d->e);
          $succes=false;
         }

         unset($d);
         break;
        }
        case "*:*":
        {
         $d=$ref["data"]->copy();
         $d->action="view";
         $d->extra="all";

         $id=$this->getID();

         //get from/to fields from m2m module
         $reffrom=$ref["data"]->check("use", "from");
         $reffrom=$reffrom[0];
         $refto=$ref["data"]->check("use", "to");
         $refto=$refto[0];

         $d->clearValues();

         $c=0;
         $get=false;
         foreach(array_keys($this->field[$name]["value"]) as $k)
         {
          if($v=$this->sqlValue($id, $k))
          {
           if(!in_array($v, $d->field[$reffrom]["value"]))
           {
            $d->setValue($reffrom, $v, $c);
            $d->setOperant($reffrom, "OR", $c);
            $get=true;
            $c++;
           }
          }
         }

         $d->setShowFields($refto);
         $d->getReferenceData($refto)->setShowFields();

         if($get&&!$d->get())
         {
          $this->e->add($d->e);
          $succes=false;
         }

         $delete=Array();
         foreach(array_keys($this->field[$name]["value"]) as $k)
         {
          $delete[$k]=Array();
          while($d->next())
          {
           $delete[$k][]=$d->v[$refto];
          }
         }

         $d->e->clear();
         $d->clearValues();
         $d->action="delete";
         $d->extra="item";

         $d->field[$reffrom]["properties"]="identifier";
         $d->field[$refto]["properties"]="identifier";

         $c=0;
         foreach(array_keys($this->field[$name]["value"]) as $k)
         {
          foreach($delete[$k] as $v)
          {
           $d->setValue($refto, $v, $c);
           $d->setValue($reffrom, $this->sqlValue($id, $k), $c);
           $d->setDefaultValue("userrights", $c);
           $c++;
          }
         }

         if($c&&!$d->submit())
         {
          $this->e->add($d->e);
          $succes=false;
         }

         $d->field[$reffrom]["properties"]="";
         $d->field[$refto]["properties"]="";

         unset($d);
         break;
        }
        default: break;
       }
      }
     }

     if($this->extends)
     {
      foreach(array_keys($this->extends) as $module)
      {
       if(!$this->extends[$module]->submit())
       {
        $this->e->add($this->extends[$module]->e);
        $succes=false;
        break;
       }
      }
     }

     if($succes)
      $this->e->add($this->title." deleted successfully", "note", $this->vmax." items");

     break;
    }

    case "undelete":
    {
     foreach($this->check("use", "sql") as $field) //unset all values
      $this->field[$field]["valuetype"]="none";

     foreach($this->check("properties", "deleted") as $field) //set deleted to false
      for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //undelete for each value
       $this->setValue($field, false, $this->vc);

     for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //undelete for each value
      if(!$storage->edit())
       $succes=false;

     if($succes)
      $this->e->add($this->title." undeleted successfully", "note", $this->vmax." items");

     break;
    }

    case "rights":
    {
     //done in categories module
     break;
    }

    case "owner":
    {
     foreach($this->check("use", "sql") as $field) //only owner+forceUpdate
      if(!$this->check("properties", "owner", $field)&&!$this->check("properties", "forceUpdate", $field))
       $this->field[$field]["valuetype"]="none";

     for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //change owner for each value
      if(!$storage->edit())
       $succes=false;

     if($succes)
      $this->e->add($this->title." owner changed successfully", "note", $this->vmax." items");

     break;
    }

    case "user":
    {
     switch($this->extra)
     {
      case "login":
      {
       //find password field
       $passwordField=false;
       foreach($this->check("use", "login") as $k)
       {
        if($this->check("properties", "password", $k))
        {
         $passwordField=$k;
         break;
        }
       }

       if($passwordField) //password field found
       {
        //add to show field
        $showFields=$this->check("use", "show");
        if(!in_array($passwordField, $showFields))
         $this->setShowFields(array_merge($showFields, Array($passwordField)));

        unset($showFields);

        //get submitted password
        $password=$this->getValue($passwordField);
       }

       //look up user based on username field(s)
       foreach($this->check("use", "login") as $k)
        if($this->check("use", "sql", $k))
         if($this->check("properties", "username", $k))
          $this->setValue($k, $this->getValue($k));

       if(!$storage->login()||!$storage->get()||!$this->setValues($storage)) //set values for post submit check
       {
        $this->e->add("Unable to retrieve user information", "debug"); //user probably doesn't exist
        $this->e->add("Login failed", "error");
        return false;
       }

       if($passwordField) //validate password
       {
        if($this->check("properties", "nocrypt", $passwordField)) //plain text
        {
         if($this->getValue($passwordField)!==$password) //plain text password
         {
          $this->e->add("Password doesn't match", "debug");
          $this->e->add("Login failed", "error");
          return false;
         }
        }
        else //crypted password
        {
         if(!($hash=$this->getValue($passwordField))) //no password hash
         {
          $this->e->add("No password hash returned from database", "debug");
          $this->e->add("Login failed", "error");
          return false;
         }

         if($hash[0]=="$") //$ separated hash
         {
          //get salt
          switch(substr($hash, 1, strpos($hash, "$", 1)-1))
          {
           case "2a":
           case "2x":
           case "2y": //Blowfish
           {
            if($hash!==crypt($password, substr($hash, 0, 28))) //$2$2$22hash
            {
             $this->e->add("Blowfish password hash doesn't match", "debug");
             $this->e->add("Login failed", "error");
             return false;
            }

            break;
           }
           case "1": //MD5 (Salted)
           {
            if($hash!==crypt($password, substr($hash, 0, 12))) //$1$8$hash
            {
             $this->e->add("Blowfish password hash doesn't match", "debug");
             $this->e->add("Login failed", "error");
             return false;
            }

            break;
           }
           case "5": //SHA256
           case "6": //SHA512
           {
            if($hash!==crypt($password, substr($hash, 0, strrpos($hash, "$")))) //$6$round=5000$hash
            {
             $this->e->add("SHA password hash doesn't match", "debug");
             $this->e->add("Login failed", "error");
             return false;
            }

            break;
           }
           default:
           {
            $this->e->add("Unknown password prefix hash: \"".substr($hash, 1, strpos($hash, "$", 1)-1)."\"", "system");
            return false;
           }
          }
         }
         else //MD5 Unsalted
         {
          if($hash!==md5($password)) //old style password hashing
          {
           $this->e->add("MD5 password hash doesn't match", "debug");
           $this->e->add("Login failed", "error");
           return false;
          }
         }

         unset($hash);
        }

        unset($password);
       }

       $this->unsetValue($passwordField);
       unset($passwordField);

       break;
      }

      case "logout":
      {
       if(!$storage->logout())
       {
        $this->e->add("Logout failed", "error");
        return false;
       }

       break;
      }

      case "password":
      {
       foreach($this->check("use", "sql") as $field) //only rightsReference+forceUpdate
       {
        if($this->check("properties", "owner", $field))
         $this->field[$field]["valuetype"]="set"; //used for loose filtering
        else if($this->check("properties", "password", $field)||$this->check("properties", "forceUpdate", $field))
         $this->field[$field]["valuetype"]="request"; //make sure they don't popup in loose filtering
        else
         $this->field[$field]["valuetype"]="none";
       }

       for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //move each category
        if(!$storage->edit(true)) //loose filtering, identifier not per se the identifier
         $succes=false;

       if($succes)
        $this->e->add($this->title." password changed successfully", "note", $this->vmax." items");

       break;
      }
     }
     break;
    }

    case "database":
    {
     switch($this->extra)
     {
      case "create":
      {
       if(!$storage->create(true))
        $succes=false;

       //update tableExists cache
       if($succes)
       {
        global $_jses;
        $_jses["tableExists"][$_jses["project"]][$this->db][$this->module]=true; //store cache

        $this->e->add($this->title." table created successfully", "note");
       }
       break;
      }
      case "alter":
      {
       if(!$storage->alter(true))
        $succes=false;
       else
        $this->e->add($this->title." table altered successfully", "note");

       break;
      }
      case "maintenance":
      {
       if(!$storage->maintenance())
        $succes=false;
       else
        $this->e->add($this->title." table maintained successfully", "note");

       break;
      }
      default:
      {
       $this->e->add("Extra: \"".$this->extra."\" not defined for action: \"".$this->action."\" in submit function", "error");
       return false;
      }
     }
     break;
    }

    case "order":
    {
     $order=$this->extra;
     $this->action="edit";
     $this->extra="item";
     $this->get();
     $this->action="order";
     $this->extra=$order;

     if($orderfields=$this->check("properties", "order"))
     {
      if(count($this->check("reference", "categories")))
      {
       //get catnr of current item
       $d=Data::cache($this->module, "view", "item", Array("format" => $this->dataformat, "db" => $this->db));
       $d->clearValues();
       $d->setValue("nr", $this->getValue("nr"));
       $d->setShowFields("catnr");
       $d->get();

       $catnr=$d->getValue("catnr");

       //prepare $d for category based ordernr manipulation
       $d->setExtra("list");
       $d->clearValues();
       $d->setValue("catnr", $catnr);
      }
      else
      {
       //prepare $d for normal ordernr manipulation
       $d=Data::cache($this->module, "view", "all", Array("format" => $this->dataformat, "db" => $this->db));
       $d->clearValues();
      }

      $d->setShowFields("nr", $orderfields[0]);
      $d->sortOrder($this->sortOrder);
      $d->get();

      $previous=false;
      while($d->next())
      {
       if($d->v["nr"]==$this->getValue("nr"))
       {
        if($this->extra=="up")
        {
         if($previous===false)
         {
          $this->e->add("Already the first item", "error");
          $succes=false;
         }
         else if($previous[$orderfields[0]]==$d->v[$orderfields[0]])
         {
          $entry=0;
          $ordernr=$d->v[$orderfields[0]]+1;
          $this->setValue("nr", $previous["nr"], $entry);
          $this->setValue($orderfields[0], $ordernr, $entry);
          $entry++;

          while($d->next())//increase order of next entries
          {
           if($d->v[$orderfields[0]]==$ordernr) //also the next orders
            $ordernr++;

           if($d->v[$orderfields[0]]==($ordernr-1))
           {
            $this->setValue("nr", $d->v["nr"], $entry);
            $this->setValue($orderfields[0], $ordernr, $entry);
            $entry++;
           }
           else
            break;
          }

          $succes=true;
         }
         else
         {
          $this->setValue($orderfields[0], $previous[$orderfields[0]], 0);
          $this->setValue("nr", $previous["nr"], 1);
          $this->setValue($orderfields[0], $d->v[$orderfields[0]], 1);
          $succes=true;
         }

         break;
        }
        else if($this->extra=="down")
        {
         $current=$d->v;
         if(!$d->next())
         {
          $this->e->add("Already the last item", "error");
          $succes=false;
         }
         else if($current[$orderfields[0]]==$d->v[$orderfields[0]])
         {
          $entry=0;
          $ordernr=$d->v[$orderfields[0]]+1;
          $this->setValue("nr", $current["nr"], $entry);
          $this->setValue($orderfields[0], $ordernr, $entry);
          $entry++;

          while($d->next())//increase order of next entries
          {
           if($d->v[$orderfields[0]]==$ordernr) //also the next orders
            $ordernr++;

           if($d->v[$orderfields[0]]==($ordernr-1))
           {
            $this->setValue("nr", $d->v["nr"], $entry);
            $this->setValue($orderfields[0], $ordernr, $entry);
            $entry++;
           }
           else
            break;
          }
          $succes=true;
         }
         else
         {
          $this->setValue($orderfields[0], $d->v[$orderfields[0]], 0);
          $this->setValue("nr", $d->v["nr"], 1);
          $this->setValue($orderfields[0], $current[$orderfields[0]], 1);
          $succes=true;
         }

         break;
        }

       }

       $previous=$d->v;
      }

      if($succes)
      {
       foreach($this->check("use", "sql") as $field)
        if($field!="nr"&&$field!=$orderfields[0])
         for($c=0;$c<$this->vmax;$c++)
          $this->unsetValue($field, $c);

       $this->action="edit";
       if($succes=$this->submit())
       {
        $this->e=new Error();
        $this->e->add("Item order ".($this->extra=="up"?"increased":"decreased"), "note");
       }
       $this->action="order";
      }
     }
     break;
    }
    default:
    {
     $this->e->add("Action: \"".$this->action."\" not defined in submit function", "error");
     return false;
    }
   }

   if(!$succes)
   {
    $this->e->add($storage->e);
    return false;
   }

   if(!$this->moduleSpecificFunction("PostSubmit"))
    return false;

   //always succes
   if($this->check("type", "captcha")&&$succes) //new captcha, if captcha has been submitted in form
   {
    $captcha=new Captcha();
    $captcha->random();
    unset($captcha);
   }

   return $succes;
  }


  function moduleSpecificFunction($function, $actionSpecific=true, $extendedModules=false)
  {
   $success=true;

   if($extendedModules)
   {
    foreach(array_keys($this->extends) as $module)
     if(!$this->extends[$module]->moduleSpecificFunction($function, $actionSpecific, $extendedModules))
      $success=false;
   }

   if($actionSpecific)
    $moduleFunction=$this->action.$function;
   else
    $moduleFunction=$function;

   if(substr(phpversion(), 0, 1)<5)
    $moduleFunction=strtolower($moduleFunction);

   if(isset($this->moduleObject)&&$this->moduleObject)
   {
    if(method_exists($this->moduleObject, $moduleFunction))
    {
     $return=$this->moduleObject->$moduleFunction();

     if($return===NULL) //pretty much succes
      $this->e->add("Module function: \"".$this->module."->".$moduleFunction."\" doesn't have a return value", "debug");
     else if(!$return)
      $success=false;
    }
   }

   return $success;
  }
/******************************************************************
******************************************************************/

  function errorCheck($submit=false)
  {
   global $_config;

   if($this->e->any("fatal"))
    return false;

   if(count($this->check("reference", "categories"))==0&&count($this->check("properties", "owner"))==0&&count($this->check("type", "rights"))==0&&$_config["usernr"]!=$_config["rootnr"])
   {
    $this->e->add("You must be root to use module \"".$this->title."\"", "hack");
    return false;
   }

   if($submit)
   {
    switch($this->action) //manipulation can only be done by id, get should be called first
    {
     case "edit":
     case "move":
     case "delete":
     {
      $c=0;
      foreach($this->check("properties", "identifier") as $k) //only 1 supported
      {
       if($this->check("use", "sql", $k))
       {
        if(!$this->check("valuetype", "none", $k))
        {
         $this->checkField($k);
         $c++;
        }
       }
      }

      if($c==0)
      {
       $this->e->add("The identifier must be set", "error");
       return false;
      }

      break;
     }
    }

    //data consistency check
    switch($this->action)
    {
     case "add":
     case "copy":
     {
      if($this->dataformat=="sql")
      {
       foreach($this->check("use", "sql") as $k)
        if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
         $this->checkField($k);
      }
      else
      {
       foreach(array_keys($this->field) as $k)
        if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
         if($this->check("use", "sql", $k)||$this->check("use", "form", $k))
          $this->checkField($k);
      }

      if($this->unique)
      {
       if(!$storage=$this->storageObject())
        return false;

       foreach(array_keys($this->unique) as $unique)
       {
        for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
        {
         if(!$storage->unique($this->unique[$unique]))
         {
          $error=Array();
          foreach($this->unique[$unique] as $field)
           $error[]=$this->t[$field]." '".$this->getValue($field)."'";

          $this->e->add("Unique index \"".$unique."\" violated: ".join(" + ", $error)." is already in use", "error", "Entry: ".($this->vc+1));
          return false;
         }
        }
       }
       unset($storage);
      }

      if($this->module!="categories"&&$this->module!="userRights"&&$this->module!="groupRights"&&count($this->check("properties", "rightsReference"))>0)
      {
       //check if category is of correct type
       $d=Data::cache("categories", "view", "item", Array("format" => "form", "db" => $this->db));
       $d->clearValues();
       $d->setShowFields("module", "moduleLock");
       $d->setValue("nr", $this->getValue("catnr"));

       if(!$d->get())
       {
        $this->e->add("Can't check category module type", "error");
        $this->e->add($d->e);
        return false;
       }

       while($d->next())
       {
        switch($d->field["module"]["type"])
        {
         case "module": //old style: only 1 module allowed
         {
          if($d->v["module"]!=$this->module)
          {
           $this->e->add("Not allowed to insert \"".$this->title."\" items in this category", "error");
           return false;
          }
          break;
         }
         case "modules":
         {
          if(!in_array($this->module, $d->v["module"]))
          {
           if(!isset($d->v["moduleLock"])||$d->v["moduleLock"]) //category modules locked?
           {
            $this->e->add("Not allowed to insert \"".$this->title."\" items in this category", "error");
            return false;
           }

           $nr=$d->v["nr"];
           $modules=$d->v["module"];
           $modules[]=$this->module;

           $d->clearValues();
           $d->setAction("edit");

           $d->setValue("nr", $nr);
           $d->setValue("module", $modules);

           unset($nr, $modules);

           if(!$d->submit())
           {
            $this->e->add($d->e);
            $this->e->add("Not allowed to insert \"".$this->title."\" items in this category", "error");
            return false;
           }
          }
          break;
         }
         default:
         {
          $this->e->add("Unknown module type \"".$d->field["module"]["type"]."\" for \"".$d->title."\"", "error");
          return false;
         }
        }
       }

       unset($d);
      }

      break;
     }

     case "edit":
     {
      if($this->dataformat=="sql")
       $fields=$this->check("use", "sql");
      else
       $fields=array_keys($this->field);

      foreach($fields as $k)
       if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
        if(!$this->check("valuetype", "none", $k)||$this->check("type", "captcha", $k))
         $this->checkField($k);

      unset($fields);

      if($this->unique)
      {
       if(!$storage=$this->storageObject())
        return false;

       if($refer=$this->getRightsReference())
        $rightsReference=$this->reference[$refer]["from"];
       else
        $rightsReference=false;

       $d=$this->copy();
       $d->setAction("view");
       $d->setExtra("item");
       $d->clearValues();

       $fields=Array("nr", "catnr"); //get all fields needed for unique check
       foreach(array_keys($this->unique) as $unique)
        foreach($this->unique[$unique] as $field)
         if(!in_array($field, $fields))
          $fields[]=$field;

       $d->setShowFields($fields);

       for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
        $d->setValue("nr", $this->getValue("nr", $this->vc), $this->vc);

       $d->get();

       foreach(array_keys($this->unique) as $unique)
       {
        for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
        {
         $tmp=Array(); //store current values

         $checkUnique=false; //check if value of unique fields are about to change
         foreach($this->unique[$unique] as $field)
          if(!$this->check("valuetype", "none", $field))
           $checkUnique=true;

         if($checkUnique) //change: get values to properly check
         {
          $d->reset(); //get real values from the database
          while($d->next())
          {
           if($d->v["nr"]==$this->getValue("nr", $this->vc)) //find correct entry
           {
            foreach($this->unique[$unique] as $field) //what happens if 1 of multiple fields is set?
            {
             if($this->check("valuetype", "none", $field)) //not set in the first place, use database valuetype
             {
              $tmp[$field]=NULL;
              $this->setValue($field, $d->v[$field], $this->vc);
             }
             else if($field==$rightsReference)//store submitted value, use database value
             {
              $tmp[$field]=$this->getValue($field, $this->vc);
              $this->setValue($field, $d->v[$field], $this->vc);
             }
             //else //use submitted value
            }
           }
          }

          if(!$storage->unique($this->unique[$unique]))
          {
           $error=Array();
           foreach($this->unique[$unique] as $field)
            $error[]=$this->t[$field]." '".$this->getValue($field)."'";

           $this->e->add("Unique index \"".$unique."\" violated: ".join(" + ", $error)." is already in use", "error", "Entry: ".($this->vc+1));
           return false;
          }
         }

         foreach($tmp as $field => $value) //restore values
         {
          if($value===NULL)
           $this->unsetValue($field, $this->vc);
          else
           $this->setValue($field, $value, $this->vc);
         }

         unset($tmp);
        }
       }
       unset($storage);
      }
      break;
     }

     case "move":
     {
      //Category reference
      if(!($refer=$this->getRightsReference()))
      {
       $this->e->add("No category reference for module: \"".$this->title."\", needed for moving", "hack");
       return false;
      }

      $rightsReference=$this->reference[$refer]["from"];

      $this->checkField("newcatnr");

      //check if new category is of correct type
      $d=Data::cache("categories", "view", "item", Array("format" => $this->dataformat, "db" => $this->db));
      $d2=$this->copy();
      $d2->action="view";
      $d2->extra="item";

      for($this->vc=0;$this->vc<$this->vmax;$this->vc++) //check rights for eacht value
      {
       if(!($newcatnr=$this->getValue("newcatnr", $this->vc))) //allow per entry and just 1 newcatnr
        $newcatnr=$this->getValue("newcatnr");

       $d2->clearValues();
       $d2->setShowFields();
       $d2->setValue("nr", $this->getValue("nr", $this->vc));

       if(!$d2->get())
       {
        $this->e->add("Can't check old category", "error", "Entry: ".($this->vc+1));
        $this->e->add($d2->e);
        return false;
       }

       if($d2->getValue($rightsReference)==$newcatnr)
       {
        $this->e->add("Refusing to move the data to it's old location", "error", "Entry: ".($this->vc+1));
        return false;
       }

       if($this->module!="categories"&&$this->module!="userRights"&&$this->module!="groupRights"&&count($this->check("properties", "rightsReference"))>0)
       {
        $d->clearValues();
        $d->setShowFields("module", "moduleLock");
        $d->setValue("nr", $newcatnr);

        if(!$d->get())
        {
         $this->e->add("Can't check category module type", "error", "Entry: ".($this->vc+1));
         $this->e->add($d->e);
         return false;
        }

        while($d->next())
        {
         switch($d->field["module"]["type"])
         {
          case "module": //old style: only 1 module allowed
          {
           if($d->v["module"]!=$this->module)
           {
            $this->e->add("Unable to move \"".$this->title."\" to a non \"".$this->title."\" category", "error", "Entry: ".($this->vc+1));
            return false;
           }
           break;
          }
          case "modules":
          {
           if(!in_array($this->module, $d->v["module"]))
           {
            if(!isset($d->v["moduleLock"])||$d->v["moduleLock"]) //category modules locked?
            {
             $this->e->add("Unable to move \"".$this->title."\" to a non \"".$this->title."\" category", "error", "Entry: ".($this->vc+1));
             return false;
            }

            $nr=$d->v["nr"];
            $modules=$d->v["module"];
            $modules[]=$this->module;

            $d->clearValues();
            $d->setAction("edit");

            $d->setValue("nr", $nr);
            $d->setValue("module", $modules);

            unset($nr, $modules);

            if(!$d->submit())
            {
             $this->e->add($d->e);
             $this->e->add("Unable to move \"".$this->title."\" to a non \"".$this->title."\" category", "error", "Entry: ".($this->vc+1));
             return false;
            }
           }
           break;
          }
          default:
          {
           $this->e->add("Unknown module type \"".$d->field["module"]["type"]."\" for \"".$d->title."\"", "error");
           return false;
          }
         }
        }
       }
      }

      unset($d, $d2);

      if($this->unique)
      {
       if(!$storage=$this->storageObject())
        return false;

       $refer=$this->getRightsReference();
       $rightsReference=$this->reference[$refer]["from"];

       $d=$this->copy();
       $d->setAction("view");
       $d->setExtra("item");
       $d->clearValues();

       $fields=Array("nr", "catnr"); //get all fields needed for unique check
       foreach(array_keys($this->unique) as $unique)
        foreach($this->unique[$unique] as $field)
         if(!in_array($field, $fields))
          $fields[]=$field;

       $d->setShowFields($fields);

       for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
        $d->setValue("nr", $this->getValue("nr", $this->vc), $this->vc);

       $d->get();

       foreach(array_keys($this->unique) as $unique)
       {
        for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
        {
         $tmp=Array(); //store current values

         if(in_array($rightsReference, $this->unique[$unique])) //check all unique indexes with rights reference
          $checkUnique=true;
         else
          $checkUnique=false;

         if($checkUnique) //change: get values to properly check
         {
          if(!($newcatnr=$this->getValue("newcatnr", $this->vc))) //allow per entry and just 1 newcatnr
           $newcatnr=$this->getValue("newcatnr");

          $d->reset(); //get real values from the database
          while($d->next())
          {
           if($d->v["nr"]==$this->getValue("nr", $this->vc)) //find correct entry
           {
            foreach($this->unique[$unique] as $field) //what happens if 1 of multiple fields is set?
            {
             if($field==$rightsReference)//store submitted value, use newcatnr
             {
              $tmp[$field]=$this->getValue($field, $this->vc);
              $this->setValue($field, $newcatnr, $this->vc);
             }
             else if($this->check("valuetype", "none", $field)) //not set in the first place, use database valuetype
             {
              $tmp[$field]=NULL;
              $this->setValue($field, $d->v[$field], $this->vc);
             }
             else //store submitted value, use database value
             {
              $tmp[$field]=$this->getValue($field, $this->vc);
              $this->setValue($field, $d->v[$field], $this->vc);
             }
            }
           }
          }

          if(!$storage->unique($this->unique[$unique]))
          {
           $error=Array();
           foreach($this->unique[$unique] as $field)
            $error[]=$this->t[$field]." '".$this->getValue($field)."'";

           $this->e->add("Unique index \"".$unique."\" violated: ".join(" + ", $error)." is already in use", "error", "Entry: ".($this->vc+1));
           return false;
          }
         }

         foreach($tmp as $field => $value) //restore values
         {
          if($value===NULL)
           $this->unsetValue($field, $this->vc);
          else
           $this->setValue($field, $value, $this->vc);
         }
         unset($tmp);
        }
       }
       unset($storage);
      }

      break;
     }

     case "delete":
     case "undelete":
     {
      break;
     }

     case "owner":
     {
      foreach($this->check("properties", "owner") as $k)
       if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
        $this->checkField($k);

      break;
     }

     case "rights":
     {
      //done in categories module
      break;
     }

     case "user":
     {
      switch($this->extra)
      {
       case "login":
       {
        foreach($this->check("use", "login") as $k)
         if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
          $this->checkField($k);

        break;
       }
       case "logout":
       {
        break;
       }
       case "password":
       {
        foreach($this->check("properties", "password") as $k)
         if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
          $this->checkField($k);

        foreach($this->check("properties", "owner") as $k)
        {
         if($this->field[$k]["module"]==$this->module) //only check fields for current module, extended modules done elsewhere
         {
          $this->checkField($k);

          for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
           if($this->getValue($k, $this->vc)==$_config["anonnr"])
            $this->e->add("Can't change password of \"anonymous\" user", "hack", "Entry: ".($this->vc+1));
         }
        }

        break;
       }
      }

      break;
     }

     case "database":
     {
      if($this->extra!="info"&&$_config["usernr"]!=$_config["rootnr"])
       $this->e->add("Only root may alter database structure", "hack");

      break;
     }

     case "order":
     {
      break;
     }

     default:
     {
      $this->e->add("Can't do submit checks for action: ".$this->action, "hack");
      break;
     }
    }
   }
   else //get data
   {
//Check nr of total values requested
    $this->checkField("show_from");
    $this->checkField("show_total");

//Check aggregation fields
    if($this->check("valuetype", "set", "aggregation_field"))
    {
     $this->checkField("aggregation_field");
     $this->checkField("aggregation_type");
     $this->checkField("aggregation_method");
     $this->checkField("aggregation_interval");
    }

    //data consistency check
    switch($this->action)
    {
     case "view":
     case "add":
     case "copy":
     case "edit":
     case "move":
     case "delete":
     case "deleted":
     case "undelete":
     case "rights":
     case "owner":
     {
      switch($this->extra)
      {
       case "item";
       {
        $c=0;
        foreach($this->check("properties", "identifier") as $k) //only 1 supported
        {
         if($this->check("use", "sql", $k))
         {
          if(!$this->check("valuetype", "none", $k))
          {
           $this->checkField($k);
           $c++;
          }
         }
        }

        if($c==0)
        {
         $this->e->add("The identifier must be set", "error");
         return false;
        }

        break;
       }

       case "list":
       {
        $found=false;
        $refer=$this->getRightsReference();
        $referField=$this->reference[$refer]["from"];

        if($this->check("use", "sql", $referField)&&!$this->check("valuetype", "none", $referField))
        {
         $tmp=$this->field[$referField]["min"];
         $this->field[$referField]["min"]=0;
         $this->checkField($referField);
         $this->field[$referField]["min"]=$tmp;
        }
        else
        {
         $this->e->add("The category reference must be set", "error");
         return false;
        }

        break;
       }

       case "all":
       {
        foreach($this->check("valuetype", "set") as $k)
         $this->checkField($k);

        break;
       }

       default: break;
      }

      break;
     }

     case "tree":
     {
      $found=false;
      $refer=$this->getSelfReference();
      $referField=$this->reference[$refer]["from"];

      if($this->check("use", "sql", $referField)&&!$this->check("valuetype", "none", $referField))
      {
       $tmp=$this->field[$referField]["min"];
       $this->field[$referField]["min"]=0;
       $this->checkField($referField);
       $this->field[$referField]["min"]=$tmp;
      }
      else if($this->extra=="down") //tree down also allows ID to be set for inclusive (id field) results
      {
       $id=$this->getID();

       if($this->check("use", "sql", $id)&&!$this->check("valuetype", "none", $id))
        $this->checkField($id);
       else
       {
        $this->e->add("The self reference (".$referField.") or identifier (".$id.") must be set", "error");
        return false;
       }
      }
      else
      {
       $this->e->add("The self reference (".$referField.") must be set", "error");
       return false;
      }

      if($this->extra=="down")
       $this->checkField("treedepth");

      break;
     }

     case "lost":
     {
      //Check category reference
      if(!$this->getRightsReference())
       $this->e->add("No category rights reference for module: \"".$this->title."\", needed for lost and found search", "hack");

      //Gotta be root
      if($_config["usernr"]!=$_config["rootnr"])
       $this->e->add("Only root may search for lost and found", "hack");

      break;
     }

     case "search":
     {
      //Only check search string
      $this->checkField("search");

      switch($this->extra)
      {
       case "title":
       {
        if(count($this->check("properties", "title"))==0)
         $this->e->add("Can't do title search, no title field", "error");

        break;
       }
       case "full":
       {
        break;
       }
       case "field":
       {
        $searchFields=false;
        foreach(explode(";", $this->sqlValue("searchFields", 0)) as $field)
        {
         if($field)
         {
          if($this->check("use", "sql", $field))
           $searchFields=true;
          else
           $this->e->add("No search field: \"".$field."\"", "error");
         }
        }

        if(!$searchFields)
         $this->e->add("No search fields selected", "error");

        break;
       }

       default: break;
      }

      break;
     }

     case "database":
     {
      if($this->extra!="info"&&$_config["usernr"]!=$_config["rootnr"])
       $this->e->add("Only root may alter database structure", "hack");

      break;
     }

     default:
     {
      $this->e->add("Can't do get checks for action: ".$this->action, "hack");
      break;
     }
    }
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }


/******************************************************************
******************************************************************/

  function checkField($name)
  {
   global $_config;

   if(!isset($this->field[$name]))
   {
    $this->e->add("Field: \"".$name."\" doesn't exist");
    return false;
   }

   if($name=="m"||$name=="a"||$name=="e") //skip
    return true;

   $f=&$this->field[$name];

   if($f["type"]=="array") //skip, internal use (images/files)
    return true;

   for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
   {
    if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like"))
    {
     if(($v=$this->sqlValue($name))===NULL)
      continue;

     if(strstr($v, "%")){$this->e->add($f["title"]." \"".$v."\" may not contain a \"%\" for \"".$f["operator"][$this->vc]."\" operator", "error", "Entry: ".($this->vc+1)); return false;}

     if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]."", "error", "Entry: ".($this->vc+1)); return false;}
     if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" can't be larger then ".$f["max"]."", "error", "Entry: ".($this->vc+1)); return false;}
    }
   }


   //Check if input is valid
   switch($f["type"])
   {
    case "boolean":
    {
     //No check needed, forced in addField
     break;
    }

    case "nr":
    {
     if(!(preg_match("/identifier/i", $f["properties"])&&($this->action=="add"||$this->action=="copy")))
     {
      for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      {
       if(($v=$this->sqlValue($name))===NULL)
        continue;

       if($this->check("properties", "big", $name))
       {
        if(!is_float($v)){$this->e->add($f["title"]." \"".$v."\" isn't a number", "error", "Entry: ".($this->vc+1)); return false;}
       }
       else
       {
        if(!is_int($v)){$this->e->add($f["title"]." \"".$v."\" isn't a number", "error", "Entry: ".($this->vc+1)); return false;}
       }

       if($f["min"]!==false&&$v<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]."", "error", "Entry: ".($this->vc+1)); return false;}
       if($f["max"]!==false&&$v>$f["max"]){$this->e->add($f["title"]." \"".$v."\" can't be larger then ".$f["max"]."", "error", "Entry: ".($this->vc+1)); return false;}
      }
     }
     break;
    }


    case "ip":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!preg_match("/^((\d{1,3}\.\d{1,3}.\d{1,3}.\d{1,3})|(([a-f0-9]{1,4}:{1,2}){1,7})([a-f0-9]{1,4}))$/i", $v))
       {$this->e->add($f["title"]." \"".$v."\" is an illegal IP address, eg: 192.168.1.1 or 2001:7b8:390:1:201:80ff:fe60:6717", "hack", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }


    case "ipv4":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!is_int($v)){$this->e->add($f["title"]." \"".$v."\" is an illegal IPV4 address, eg: 192.168.1.1", "hack", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "ipv6":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!preg_match("/^(([a-f0-9]{1,4}:{1,2}){1,7})([a-f0-9]{1,4})$/i", $v))
       {$this->e->add($f["title"]." \"".$option."\" is an illegal IPV6 address, eg: 2001:7b8:390:1:201:80ff:fe60:6717", "error", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }


    case "float":
    case "price":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!is_float($v)){$this->e->add($f["title"]." \"".$v."\" isn't a float number", "error", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&$v<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]."", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&$v>$f["max"]){$this->e->add($f["title"]." \"".$v."\" can't be larger then ".$f["max"]."", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }


    case "password":
    {
     if(preg_match("/^verify(.+)$/", $name, $m)) //verification field
     {
      if($this->extra!="login")
      {
       for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
       {
        if($this->getValue($name, $this->vc)!==$this->getValue($m[1], $this->vc))
        {
         $this->e->add($this->field[$name]["title"]." must be the same", "error", "Entry: ".($this->vc+1));
         return false;
        }
       }
      }
     }
     else //normal password field
     {
      if($this->extra!="login")
      {
       for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
       {
        $v=$this->getValue($name, $this->vc);

        //don't check password when editting other data
        if(!($this->action=="edit"&&strlen($v)==0))
        {
         if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
         if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

         $passwordErrors="";
         if($_config["passwordlower"]&&strlen(preg_replace("/[^a-z]/", "", $v))<$_config["passwordlower"])
          $passwordErrors.=" should at least contain ".$_config["passwordlower"]." lower case character(s)";

         if($_config["passwordupper"]&&strlen(preg_replace("/[^A-Z]/", "", $v))<$_config["passwordupper"])
          $passwordErrors.=($passwordErrors?" and ":" should at least contain ").$_config["passwordupper"]." upper case character(s)";

         if($_config["passwordnumbers"]&&strlen(preg_replace("/[^0-9]/", "", $v))<$_config["passwordnumbers"])
          $passwordErrors.=($passwordErrors?" and ":" should at least contain ").$_config["passwordnumbers"]." number(s)";

         if($_config["passwordspecials"]&&strlen(preg_replace("/[^~\!@#\$%\^&\*\(\)\-_\=\+\[\{\]\}\\\|;\:,\<\.\>\/\?]/", "", $v))<$_config["passwordspecials"])
          $passwordErrors.=($passwordErrors?" and ":" should at least contain ").$_config["passwordspecials"]." special character(s): ~!@#$%^&*()-_=+[{]}\|;:,<.>/?";

         if($passwordErrors){$this->e->add($f["title"].$passwordErrors, "error", "Entry: ".($this->vc+1)); return false;}
         unset($passwordErrors);
        }
       }
      }
     }

     break;
    }


    case "multilist":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($v)
      {
       foreach(explode(";", $v) as $option)
       {
        if(!preg_match(REGEXP_DATA_LISTOPTION, $option)){$this->e->add($f["title"]." \"".$option."\" is an illegal option", "hack", "Entry: ".($this->vc+1)); return false;}
        if($option!="")
        {
         if(array_type($this->listOptions[$name])=="indexed"&&!in_array($option, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$option."\" isn't a valid option", "error", "Entry: ".($this->vc+1)); return false;}
         if(array_type($this->listOptions[$name])=="associative"&&!in_array($option, array_keys($this->listOptions[$name]))){$this->e->add($f["title"]." \"".$option."\" isn't a valid option", "error", "Entry: ".($this->vc+1)); return false;}
        }
       }
      }

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

     }
     break;
    }

    case "modules":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($v)
      {
       foreach(explode(";", $v) as $option)
       {
        if(!preg_match(REGEXP_DATA_PLUGIN_MODULE, $option)&&!preg_match(REGEXP_DATA_MODULE, $option)){$this->e->add($f["title"]." \"".$option."\" is an illegal module", "hack", "Entry: ".($this->vc+1)); return false;}
        if($option!="")
        {
         if(array_type($this->listOptions[$name])=="indexed"&&!in_array($option, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$option."\" isn't an existing module", "error", "Entry: ".($this->vc+1)); return false;}
         if(array_type($this->listOptions[$name])=="associative"&&!in_array($option, array_keys($this->listOptions[$name]))){$this->e->add($f["title"]." \"".$option."\" isn't an existing module", "error", "Entry: ".($this->vc+1)); return false;}
        }
       }
      }

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "captcha":
    {
     $captcha=new Captcha();
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!$captcha->check($v)){$this->e->add($f["title"]." \"".$v."\" doesn't match image", "error", "Entry: ".($this->vc+1)); return false;}
     }
     unset($captcha);

     break;
    }

    case "list":
    case "extra":
    case "action":
    case "module":
    case "plugin":
    case "project":

    case "html":
    case "text":
    case "title":
    case "submit":
    case "search":
    case "filename":
    case "alphabetic":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v))
      {
       switch($f["type"])
       {
        case "title":
        {
         if(!preg_match("/^[a-z0-9\ ]+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" may only contain characters, numbers and spaces", "error", "Entry: ".($this->vc+1)); return false;}
         if(preg_match("/^\s+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" may not only contain spaces", "error", "Entry: ".($this->vc+1)); return false;}
         break;
        }
        case "search":
        {
         if(!preg_match("/^[a-z0-9\ \-_\.]+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" may only contain characters, numbers and spaces", "error", "Entry: ".($this->vc+1)); return false;}
         if(preg_match("/^\s+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" may not only contain spaces", "error", "Entry: ".($this->vc+1)); return false;}
         break;
        }
        case "filename":
        {
         if(!preg_match("/^[a-z0-9\-\.]+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" may only contain characters, numbers and a few other characters", "error", "Entry: ".($this->vc+1)); return false;}
         break;
        }
        case "alphabetic":
        {
         if(!preg_match("/^[a-z]+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" may only contain characters", "error", "Entry: ".($this->vc+1)); return false;}
         break;
        }
        case "list":
        {
         if(!preg_match(REGEXP_DATA_LISTOPTION, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal option", "hack", "Entry: ".($this->vc+1)); return false;}
         if($v!="")
         {
          if(array_type($this->listOptions[$name])=="indexed"&&!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't a valid option", "error", "Entry: ".($this->vc+1)); return false;}
          if(array_type($this->listOptions[$name])=="associative"&&!in_array($v, array_keys($this->listOptions[$name]))){$this->e->add($f["title"]." \"".$v."\" isn't a valid option", "error", "Entry: ".($this->vc+1)); return false;}
         }
         break;
        }
        case "module":
        {
         if(!preg_match(REGEXP_DATA_PLUGIN_MODULE, $v)&&!preg_match(REGEXP_DATA_MODULE, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal module", "hack", "Entry: ".($this->vc+1)); return false;}
         if($v!="")
         {
          if(array_type($this->listOptions[$name])=="indexed"&&!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't an existing module", "error", "Entry: ".($this->vc+1)); return false;}
          if(array_type($this->listOptions[$name])=="associative"&&!in_array($v, array_keys($this->listOptions[$name]))){$this->e->add($f["title"]." \"".$v."\" isn't an existing module", "error", "Entry: ".($this->vc+1)); return false;}
         }
         break;
        }
        case "plugin":
        {
         if(!preg_match(REGEXP_DATA_PLUGIN, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal plugin", "hack", "Entry: ".($this->vc+1)); return false;}
         if($v!="")
         {
          if(array_type($this->listOptions[$name])=="indexed"&&!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't an existing plugin", "error", "Entry: ".($this->vc+1)); return false;}
          if(array_type($this->listOptions[$name])=="associative"&&!in_array($v, array_keys($this->listOptions[$name]))){$this->e->add($f["title"]." \"".$v."\" isn't an existing plugin", "error", "Entry: ".($this->vc+1)); return false;}
         }
         break;
        }
        case "project":
        {
         if(!preg_match(REGEXP_DATA_PROJECT, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal project", "hack", "Entry: ".($this->vc+1)); return false;}
         if($v!="")
         {
          if(array_type($this->listOptions[$name])=="indexed"&&!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't an existing project", "error", "Entry: ".($this->vc+1)); return false;}
          if(array_type($this->listOptions[$name])=="associative"&&!in_array($v, array_keys($this->listOptions[$name]))){$this->e->add($f["title"]." \"".$v."\" isn't an existing project", "error", "Entry: ".($this->vc+1)); return false;}
         }
         break;
        }
        case "action":
        {
         if(!preg_match(REGEXP_DATA_ACTION, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal action", "hack", "Entry: ".($this->vc+1)); return false;}
         if(!$this->validAction($v)){$this->e->add($f["title"]." \"".$v."\" isn't a valid action", "error", "Entry: ".($this->vc+1)); return false;}
         break;
        }
        case "extra":
        {
         if(!preg_match(REGEXP_DATA_EXTRA, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal extra option", "hack", "Entry: ".($this->vc+1)); return false;}
         if(!$this->validExtra($v)){$this->e->add($f["title"]." \"".$v."\" isn't a valid extra option", "error", "Entry: ".($this->vc+1)); return false;}
         break;
        }
        default: break;
       }
      }
     }
     break;
    }

    case "url":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v)>0&&!preg_match(REGEXP_DATA_URL, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal URL, eg: http://www.junerules.com/", "hack", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "hostname":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v)>0&&!preg_match(REGEXP_DATA_HOSTNAME, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal hostname name, eg: www.example.com", "hack", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "domain":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v)>0&&!preg_match(REGEXP_DATA_DOMAIN, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal domain name, eg: www.example.com", "hack", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "email":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v)>0&&!preg_match(REGEXP_DATA_EMAIL, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal e-mail address, eg: bill@microgates.com", "hack", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "regexp":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v)>0)
      {
       if(!isset($this->regexp[$name])||sizeof($this->regexp[$name])==0)
       {
        $this->e->add($f["title"].": no regular expression set", "module");
        return false;
       }

       foreach($this->regexp[$name] as $regexp)
        if(!preg_match($regexp, $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal value", "hack", "Entry: ".($this->vc+1)); return false;}
      }
     }
     break;
    }

    case "phone":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" may not be more then ".$f["max"]." character(s)", "error", "Entry: ".($this->vc+1)); return false;}

      if(strlen($v)>0)
       if(!preg_match("/^[0-9\-\s\(\)\+]+$/i", $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal phone number, eg: 0123-456789", "hack", "Entry: ".($this->vc+1)); return false;}
     }

     break;
    }


    case "date":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($v=="0000-00-00")
       continue;

      if(!preg_match("/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/i", $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal date format, eg: ".date($this->field[$name]["format"][$this->dataformat]["dateformat"]), "hack", "Entry: ".($this->vc+1)); return false;}
      if(!checkdate(substr($v, 5, 2), substr($v, 8, 2), substr($v, 0, 4))){$this->e->add($f["title"]." \"".$v."\" is an incorrect date", "error", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&DT::dateConvert($v, $this->format["sql"]["dateformat"])<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must be later then ".date($this->field[$name]["format"][$this->dataformat]["dateformat"], $f["min"])."", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&DT::dateConvert($v, $this->format["sql"]["dateformat"])>$f["max"]){$this->e->add($f["title"]." \"".$v."\" must be before ".date($this->field[$name]["format"][$this->dataformat]["dateformat"], $f["max"])."", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "time":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($v=="00:00:00")
       continue;

      if(!preg_match("/^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$/i", $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal time format, eg: ".date($this->field[$name]["format"][$this->dataformat]["timeformat"]), "hack", "Entry: ".($this->vc+1)); return false;}
      if(substr($v, 0, 2)>24||substr($v, 3, 2)>60||substr($v, 6, 2)>60){$this->e->add($f["title"]." \"".$v."\" is an incorrect time", "error", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&DT::dateConvert($v, $this->format["sql"]["timeformat"])+mktime(0,0,0)<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must be later then ".date($this->field[$name]["format"][$this->dataformat]["timeformat"], $f["min"])."", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&DT::dateConvert($v, $this->format["sql"]["timeformat"])+mktime(0,0,0)>$f["max"]){$this->e->add($f["title"]." \"".$v."\" must be before ".date($this->field[$name]["format"][$this->dataformat]["timeformat"], $f["max"])."", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }

    case "datetime":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($v=="0000-00-00 00:00:00")
       continue;

      if(!preg_match("/^[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}$/i", $v)){$this->e->add($f["title"]." \"".$v."\" is an illegal date time format, eg: ".date($this->field[$name]["format"][$this->dataformat]["datetimeformat"]), "hack", "Entry: ".($this->vc+1)); return false;}
      if(!checkdate(substr($v, 5, 2), substr($v, 8, 2), substr($v, 0, 4))){$this->e->add($f["title"]." \"".$v."\" is an incorrect date", "error", "Entry: ".($this->vc+1)); return false;}
      if(substr($v, 11, 2)>24||substr($v, 14, 2)>60||substr($v, 17, 2)>60){$this->e->add($f["title"]." \"".$v."\" is an incorrect time", "error", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&DT::dateConvert($v, $this->format["sql"]["datetimeformat"])<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must be later then ".date($this->field[$name]["format"][$this->dataformat]["datetimeformat"], $f["min"])."", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&DT::dateConvert($v, $this->format["sql"]["datetimeformat"])>$f["max"]){$this->e->add($f["title"]." \"".$v."\" must be before ".date($this->field[$name]["format"][$this->dataformat]["datetimeformat"], $f["max"])."", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }


    case "localfile":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!$v&&$f["min"]!==false){$this->e->add($f["title"]." \"".$v."\" not selected", "error", "Entry: ".($this->vc+1)); return false;}
      if($v)
      {
       if(!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't a local file", "error", "Entry: ".($this->vc+1)); return false;}
      }
     }

     break;
    }

    case "localfiles":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      foreach(explode(";", $v) as $file)
      {
       if(!$file&&$f["min"]!==false){$this->e->add($f["title"]." \"".$v."\" not selected", "error", "Entry: ".($this->vc+1)); return false;}
       if($file)
       {
        if(!in_array($file, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't a local file", "error", "Entry: ".($this->vc+1)); return false;}
       }
      }
     }

     break;
    }


    case "localdirectory":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!$v&&$f["min"]!==false){$this->e->add($f["title"]." \"".$v."\" not selected", "error", "Entry: ".($this->vc+1)); return false;}
      if($v)
      {
       if(!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't a local directory", "error", "Entry: ".($this->vc+1)); return false;}
      }
     }

     break;
    }


    case "image":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name."Array"))!==NULL)
      {
       if($v["error"]!=UPLOAD_ERR_OK)
       {
        if($v["error"]==UPLOAD_ERR_INI_SIZE||$v["error"]==UPLOAD_ERR_FORM_SIZE)
        {
         $this->e->add($f["title"]." too large", "error", "Entry: ".($this->vc+1));
         return false;
        }
        else if($v["error"]==UPLOAD_ERR_PARTIAL)
        {
         $this->e->add($f["title"]." partially uploaded", "error", "Entry: ".($this->vc+1));
         return false;
        }
        else if($v["error"]==UPLOAD_ERR_NO_FILE)
        {
         if($f["min"]==0||$this->action=="edit")
         {
          $this->unsetValue($name, $this->vc);

          $this->unsetValue($name."ImageWidth", $this->vc);
          $this->unsetValue($name."ImageHeight", $this->vc);
          $this->unsetValue($name."PHPImageType", $this->vc);

          $this->unsetValue($name."Type", $this->vc);
          $this->unsetValue($name."Size", $this->vc);
          $this->unsetValue($name."Data", $this->vc);
          $this->unsetValue($name."Array", $this->vc);

          $this->e->add($f["title"]." not uploaded", "note", "Entry: ".($this->vc+1));
          return false; //no image ? that's ok
         }
         else
         {
          $this->e->add($f["title"]." not uploaded", "error", "Entry: ".($this->vc+1));
          return false;
         }
        }
        else
        {
         $this->e->add($f["title"]." upload failed", "error", "Entry: ".($this->vc+1));
         return false;
        }
       }
      }


      if(($v=$this->sqlValue($name."Type"))!==NULL)
      {
       if(!preg_match("/(gif|png|jpg|jpeg)/i", $v, $mime)){$this->e->add($f["title"]." \"".$v."\" isn't a valid image mime type", "error", "Entry: ".($this->vc+1)); return false;}

       if($mime[1]=="jpeg")
        $mime="jpg";
       else
        $mime=$mime[1];
      }
      else
       $mime=false;


      if(($v=$this->sqlValue($name))!==NULL)
      {
       if(!preg_match("/^.*\.(gif|png|jpg)$/i", $v, $ext)){$this->e->add($f["title"]." \"".$v."\" isn't a valid image extension", "error", "Entry: ".($this->vc+1)); return false;}

       $ext=strtolower($ext[1]);
      }
      else
       $ext=false;


      if(($v=$this->sqlValue($name."PHPImageType"))!==NULL)
      {
       switch($v)
       {
        case 1: $type="gif"; break;
        case 2: $type="jpg"; break;
        case 3: $type="png"; break;

        default:
         $this->e->add($f["title"]." \"".$v."\" is an unkown image, only GIF, JPG and PNG are supported", "error", "Entry: ".($this->vc+1));
        return false;
       }
      }
      else
       $type=false;

      if($mime!=$type){$this->e->add($f["title"]." mime type (".$mime.") doesn't correspond with image type (".$type.")", "error", "Entry: ".($this->vc+1)); return false;}
      if($ext!=$type){$this->e->add($f["title"]." file extension (".$ext.") doesn't correspond with image type (".$type.")", "error", "Entry: ".($this->vc+1)); return false;}
     }

     break;
    }

    case "file":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
       continue;

      if(($v=$this->sqlValue($name."Array"))!==NULL)
      {
       if($v["error"]!=UPLOAD_ERR_OK)
       {
        if($v["error"]==UPLOAD_ERR_INI_SIZE||$v["error"]==UPLOAD_ERR_FORM_SIZE)
        {
         $this->e->add($f["title"]." too large", "error", "Entry: ".($this->vc+1));
         return false;
        }
        else if($v["error"]==UPLOAD_ERR_PARTIAL)
        {
         $this->e->add($f["title"]." partially uploaded", "error", "Entry: ".($this->vc+1));
         return false;
        }
        else if($v["error"]==UPLOAD_ERR_NO_FILE)
        {
         if($f["min"]==0||$this->action=="edit")
         {
          $this->unsetValue($name, $this->vc);

          $this->unsetValue($name."Type", $this->vc);
          $this->unsetValue($name."Size", $this->vc);
          $this->unsetValue($name."Data", $this->vc);
          $this->unsetValue($name."Array", $this->vc);

          $this->e->add($f["title"]." not uploaded", "note", "Entry: ".($this->vc+1));
          return false;
         }
         else
         {
          $this->e->add($f["title"]." not uploaded", "error", "Entry: ".($this->vc+1));
          return false;
         }
        }
        else
        {
         $this->e->add($f["title"]." upload failed", "error", "Entry: ".($this->vc+1));
         return false;
        }
       }
      }
     }

     break;
    }

    case "filedata":
    {
     if($f["valuetype"]=="set")
     {
      for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
      {
       if(isset($f["operator"][$this->vc])&&strstr($f["operator"][$this->vc], "like")) //don't validate for like operators, done above
        continue;

       if(($v=$this->sqlValue($name))===NULL)
        continue;

       if($f["min"]!==false&&strlen($v)<$f["min"]){$this->e->add($f["title"]." must at least be ".$f["min"]." bytes", "error", "Entry: ".($this->vc+1)); return false;}
       if($f["max"]!==false&&strlen($v)>$f["max"]){$this->e->add($f["title"]." may not be more then ".$f["max"]." bytes", "error", "Entry: ".($this->vc+1)); return false;}
      }
     }

     break;
    }


    case "rights":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if(!is_int($v)){$this->e->add($f["title"]." \"".$v."\" isn't a number"."", "error", "Entry: ".($this->vc+1)); return false;}

      if($f["min"]!==false&&$v<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]."", "error", "Entry: ".($this->vc+1)); return false;}
      if($f["max"]!==false&&$v>$f["max"]){$this->e->add($f["title"]." \"".$v."\" can't be larger then ".$f["max"]."", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }


    case "enum":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if((!$v||$v=="")&&in_array("false", $this->listOptions[$name]))
      {
       $this->setValue($name, "false");
       $v="false";
      }
      if(!in_array($v, $this->listOptions[$name])){$this->e->add($f["title"]." \"".$v."\" isn't a valid option", "error", "Entry: ".($this->vc+1)); return false;}
     }
     break;
    }


    case "version":
    {
     for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
     {
      if(($v=$this->sqlValue($name))===NULL)
       continue;

      if($f["min"]!==false||$f["max"]!==false)
      {
       if(!preg_match("/^(\d+)\.(\d+)\.?(\d*)$/", $v, $m)){$this->e->add($f["title"]." \"".$v."\" is an illegal version number, eg: 1.0.3", "hack", "Entry: ".($this->vc+1)); return false;}

       $v=(float)$m[1].".".$m[2];
       unset($m);

       if($f["min"]!==false&&$v<$f["min"]){$this->e->add($f["title"]." \"".$v."\" must at least be ".$f["min"]."", "error", "Entry: ".($this->vc+1)); return false;}
       if($f["max"]!==false&&$v>$f["max"]){$this->e->add($f["title"]." \"".$v."\" can't be larger then ".$f["max"]."", "error", "Entry: ".($this->vc+1)); return false;}
      }
     }

     break;
    }


    //CMS fields, don't check: won't be inserted in the database
    case "label":
    case "submit":
    {
     break;
    }

    default:
    {
     $this->e->add("Illegal type: \"".$f["type"]."\" for field: \"".$f["title"]."\"");
     return false;
    }
   }


   $this->vc=0;

   if(isset($this->reference[$name]))
   {
    $ref=&$this->reference[$name];
    switch($ref["type"])
    {
     case "1:1":
     {
      if($this->action=="add"||$this->action=="copy"||$this->action=="edit")
      {
       $d=Data::cache($ref["module"], $this->action, $this->extra, Array("format" => $this->dataformat, "db" => $this->db), md5($name));
       $d->clearValues();

       for($this->vc=0;$this->vc<$this->vmax;$this->vc++)
        foreach($this->check("properties", "default") as $field)
         $d->setValue($field, $this->getValue($field, $this->vc), $this->vc);

       $d->vmax=$this->vmax;

       $d->errorCheck(true);
       $this->e->add($d->e);

       unset($d);
      }
      break;
     }
     case "0:*":
     case "1:*":
     case "*:*":
     {
      $d=Data::cache($ref["module"], "view", "all", Array("format" => $this->dataformat, "db" => $this->db));
      $d->clearValues();

      $id=$this->getID();
      $refid=$d->getID();

      $d->unsetValue($refid);

      $c=0;
      $get=false;
      foreach(array_keys($this->field[$name]["value"]) as $k)
      {
       if(is_array($this->field[$name]["value"][$k]))
       {
        foreach($this->field[$name]["value"][$k] as $v)
        {
         if(!in_array($v, $d->field[$refid]["value"]))
         {
          $d->setValue($refid, $v, $c);
          $d->setOperant($refid, "OR", $c);
          $get=true;
          $c++;
         }
        }
       }
      }

      if($get&&!$d->get())
      {
       $this->e->add($d->e);
       return false;
      }

      foreach(array_keys($this->field[$name]["value"]) as $k)
      {
       if(is_array($this->field[$name]["value"][$k]))
       {
        foreach($this->field[$name]["value"][$k] as $v)
        {
         if(!in_array($v, $d->field[$refid]["value"]))
          $this->e->add($f["title"]." \"".$v."\" value \"".$v."\" doesn't exist in \"".$d->title."\"", "error", "Entry: ".($this->vc+1));

         if($ref["type"]!="*:*")
          while($d->next())
           if($d->v[$refid]==$v)
            if($d->v[$ref["to"]]>0&&$d->v[$ref["to"]]!=$this->sqlValue($id, $k))
             $this->e->add($f["title"]." \"".$v."\" value \"".$d->v("title")."\" is already in use in ".$d->module, "error", "Entry: ".($this->vc+1));
        }
       }
      }

      unset($d);
      break;
     }
     default: break;
    }
   }

   return true;
  }




/******************************************************************
******************************************************************/

  function next()
  {
   if($this->e->any("fatal"))
    return false;

//End of results
   if($this->vc>=$this->vmax)
   {
//Restart at the next call
    $this->vc=0;
    return false;
   }

   if($this->action=="database"&&$this->extra=="info")
   {
    foreach($this->check("use", "info") as $k)
     $this->v[$k]=$this->field[$k]["value"][$this->vc];

    foreach(array_keys($this->reference) as $k)
    {
     if($this->check("use", "show", $k))
     {
      if(isset($this->reference[$k])&&isset($this->reference[$k]["data"])) //reference field values
      {
       $this->reference[$k]["data"]->reset();
       if($this->reference[$k]["data"]->next())
       {
        $this->v[$k]=$this->reference[$k]["data"]->v;
       }
      }
     }
    }

    $this->vc++;
    return true;
   }


 //set new value's
 //from database
   foreach(array_keys($this->field) as $k)
   {
    if($k=="r"&&$this->field[$k]["type"]=="rights")
    {
     if(isset($this->field[$k]["value"][$this->vc]))
      $this->r=$this->field[$k]["value"][$this->vc];
    }
    else if($this->check("use", "show", $k)||$this->check("properties", "identifier", $k))
    {
     if(isset($this->reference[$k])&&isset($this->reference[$k]["data"])) //reference field values
     {
      $ref=&$this->reference[$k];

      if(!$ref["data"]->vmax) //no values gather for reference, skip lookup below
      {
       if($ref["type"]=="*:*"||$ref["type"]=="0:*"||$ref["type"]=="1:*"||($this->referenceLevel>0&&($ref["data"]->check("use", "show")&&$ref["data"]->check("use", "show")!=Array("r")))) //reference information gathered
        $this->v[$k]=Array(); //reset value
       else if(isset($this->field[$k]["value"][$this->vc])) //normal field value
        $this->v[$k]=$this->field[$k]["value"][$this->vc]; //set value
       else
        $this->v[$k]=NULL; //reset

       continue;
      }

      //if new value is different
      if(
       (
        !isset($this->v[$k])||
        !is_array($this->v[$k])||
        !isset($this->v[$k][$ref["to"]])||
        $this->v[$k][$ref["to"]]!=$this->field[$ref["from"]]["value"][$this->vc]
       )
      )
      {
       if($ref["type"]=="*:*")
       {
        //get from/to fields from m2m module
        $reffrom=$ref["data"]->check("use", "from");
        $reffrom=$reffrom[0];
        $refto=$ref["data"]->check("use", "to");
        $refto=$refto[0];
       }
       else if($ref["type"]=="0:*"||$ref["type"]=="1:*")
       {
        $showFields=$ref["data"]->check("use", "show");

        if(in_array($ref["to"], $showFields)) //ref to field already in show fields
         unset($showFields); //don't change show fields
        else
         $ref["data"]->setShowFields(array_merge(Array($ref["to"]), $showFields)); //use fake showFields to get ref to fields in ref[data]->v array
       }
       else  //*:0, *:1 and 1:1
       {
        if(!$ref["data"]->check("use", "show")) //no additional information, set IDs directly
        {
         if(isset($this->field[$k]["value"][$this->vc])) //normal field value
          $this->v[$k]=$this->field[$k]["value"][$this->vc];
         else
          $this->v[$k]=NULL;

         continue; //skip lookup below
        }
       }

       //check all possibilities
       $this->v[$k]=Array();//reset value
       $ref["data"]->reset();//restart search
       while($ref["data"]->next())
       {
        //and set the matches
        if($ref["type"]=="*:*")
        {
         if($ref["data"]->v[$reffrom]==$this->field[$ref["from"]]["value"][$this->vc])
          $this->v[$k][]=$ref["data"]->v[$refto];
        }
        else if($ref["type"]=="0:*"||$ref["type"]=="1:*")
        {
         if(in_array($ref["data"]->v("id"), $this->field[$k]["value"][$this->vc]))
         {
          if($ref["data"]->referenceLevel>=0) //multi level, set full array
          {
           if(isset($showFields)) //ref to field not in real show fields
            unset($ref["data"]->v[$ref["to"]]); //remove from array

           $this->v[$k][]=$ref["data"]->v;
          }
          else //just getting id's
           $this->v[$k][]=$ref["data"]->v("id");
         }
        }
        else if($this->referenceLevel>0)//*:0, *:1 and 1:1
        {
         if($ref["data"]->v[$ref["to"]]==$this->field[$ref["from"]]["value"][$this->vc])
         {
          $this->v[$k]=$ref["data"]->v;
          break; //stop after first entry
         }
        }
        else if(isset($this->field[$k]["value"][$this->vc])) //normal field value
         $this->v[$k]=$this->field[$k]["value"][$this->vc];
        else
         $this->v[$k]=NULL;
       }

       if(isset($showFields)) //reset 0:* and 1:* showFields
       {
        $ref["data"]->setShowFields($showFields);
        unset($showFields);
       }
      }
     }
     else if(isset($this->field[$k]["value"][$this->vc])) //normal field value
      $this->v[$k]=$this->field[$k]["value"][$this->vc];
     else
      $this->v[$k]=NULL;
    }
   }

   $this->vc++;
   return true;
  }



/******************************************************************
******************************************************************/

  function search($name, $value)
  {
   if(!isset($this->field[$name]))
    return false;

   //search if the value exists for the field
   $l=sizeof($this->field[$name]["value"]);
   for($c=$this->vc;$c<$l;$c++)
   {
    if($this->field[$name]["value"][$c]==$value)
    {
     //set the key to the value counter
     $this->vc=$c;

     //set all values to the v variable
     return $this->next();
    }
   }

   //not found, reset value counter
   $this->vc=0;

   return false;
  }

/******************************************************************
******************************************************************/

  function reset()
  {
   $this->vc=0;
  }

/******************************************************************
******************************************************************/
  function v($type)
  {
   $r="";

   if($type=="id")
    $type="identifier";

   foreach($this->check("properties", $type) as $k)
   {
    if(!isset($this->v[$k]))
     continue;

    if(is_array($this->v[$k]))
    {
     foreach($this->reference[$k]["data"]->check("properties", "title") as $k2)
     {
      if(isset($this->v[$k][$k2]))
       $r.=$this->v[$k][$k2]." ";
     }
    }
    else
    {
     $r.=$this->v[$k]." ";
    }
   }

   $r=rtrim($r);//remove trailing space

   return $r;
  }



  function storageObject()
  {
   global $_path, $_jses;

/*
   //XXX: disabled recursive modules claim storage->data reference and don't give it back
   global $_storage;

   if(!isset($_storage[$_jses["project"]]))
    $_storage[$_jses["project"]]=Array();
   if(!isset($_storage[$_jses["project"]][$this->module]))
    $_storage[$_jses["project"]][($this->plugin?$this->plugin->name."_":"").$this->module]=Array();

   $cache=&$_storage[$_jses["project"]][($this->plugin?$this->plugin->name."_":"").$this->module];

   if(isset($cache[$this->db]))
   {
    $cache[$this->db]->data=&$this;
    $cache[$this->db]->type=$cache[$this->db]->realType;
    $cache[$this->db]->clear();
    $cache[$this->db]->e->clear();
    return $cache[$this->db];
   }
*/
   $logging=$this->e->logging;
   $this->e->logging="none";

   if(preg_match("!^([a-z0-9]+)://!i", $this->db, $m)&&file_exists($_path["include"]."storage/".strtolower($m[1]).".php"))
   {
    $type=strtolower($m[1]);
    require_once($_path["include"]."storage/".$type.".php");

    $obj=$type."Storage";
    $obj=new $obj($this);

    if($obj->e->any("system"))
    {
     $this->e->add($obj->e);
     $this->e->logging=$logging;
     return false;
    }

    $this->e->logging=$logging;

    return $obj;
    /*
    $cache[$this->db]=&$obj;
    return $cache[$this->db];
    */
   }
   else
   {
    $obj=new Storage($this, false);
    $db=$obj->getConfig();

    switch($db["type"])
    {
     case "mysql":
     {
      $file=$db["type"].".php";
      $className="MySQLStorage";
      break;
     }
     default:
     {
      $file=$db["type"].".php";
      $className=$db["type"]."Storage";
      break;
     }
    }

    if(file_exists($_path["include"]."storage/".$file))
     require_once($_path["include"]."storage/".$file);
    else //plugin based
    {
     $found=false;
     foreach(plugins("project") as $plugin)
     {
      if(file_exists($_path["project"]["plugins"].$plugin."/storage/".$file))
      {
       require_once($_path["project"]["plugins"].$plugin."/storage/".$file);
       $found=true;
       break;
      }
     }

     if(!$found)
     {
      foreach(plugins("system") as $plugin)
      {
       if(file_exists($_path["plugins"].$plugin."/storage/".$file))
       {
        require_once($_path["plugins"].$plugin."/storage/".$file);
        $found=true;
        break;
       }
      }
     }

     if(!$found)
     {
      $this->e->add("Storage definition file: \"".$file."\" not found", "system");
      $this->e->logging=$logging;
      return false;
     }
    }

    if(!class_exists($className))
    {
     $this->e->add("Storage class: \"".$className."\" not found", "system");
     $this->e->logging=$logging;
     return false;
    }

    $obj=new $className($this);

    if($obj->e->any("system"))
    {
     $this->e->add($obj->e);
     $this->e->logging=$logging;
     return false;
    }

    $this->e->logging=$logging;

    return $obj;
    /*
    $cache[$this->db]=&$obj;
    return $cache[$this->db];
    */
   }

   $this->e->add("Unknown error in: Data::storageObject()", "system");
   $this->e->logging=$logging;
   return false;
  }

  function storageSwitch($database)
  {
   global $_path;

   if(!preg_match("!^([a-z0-9]+)://!i", $this->db, $m)||!file_exists($_path["include"]."storage/".strtolower($m[1]).".php")) //custom config?
   {
    $obj=new Storage($this, false);

    if(!($obj->getConfig())) //read from config file
    {
     $this->e->add($obj->e);
     return false;
    }
   }

   $this->db=$database;
   return true;
  }



  function sync(&$source, &$target)
  {
   global $_config;

   $this->vmax=0;

   if($this->action!="sync")
   {
    $this->e->add("Use \"sync\" action for synchronization");
    return false;
   }

   $id=$source->check("properties", "identifier");
   $idS=sizeof($id);

   $time=Array();
   foreach($source->check("type", "datetime") as $k)
   {
    $time=Array($k);
    if($source->check("properties", "forceUpdate", $k))//updated field, use it... else use the last datetime field
     break;
   }

   $timeS=sizeof($time);

   if($idS==0)
   {
    $this->e->add("No \"identifier\" fields");
    return false;
   }

   for($sourceC=0;$sourceC<$source->vmax;$sourceC++)//loop through source object and find changes
   {
    $foundID=false;

     for($targetC=0;$targetC<$target->vmax;$targetC++) //match with target object
     {
      $found=true;//match found

      for($c=0;$c<$idS;$c++) //check other ID's
       if(!isset($source->field[$id[$c]]["value"][$sourceC])||$source->field[$id[$c]]["value"][$sourceC]!=$target->field[$id[$c]]["value"][$targetC])
        $found=false; //doesn't match other ID's

      if(!$found)
       continue;
      else
       $foundID=true;

      if($foundID)
      {
       if($timeS==0) //not updated field, check all fields
       {
        $this->field["nr"]["value"][$this->vmax]=Array();
        for($c=0;$c<$idS;$c++)
         $this->field["nr"]["value"][$this->vmax][$id[$c]]=$source->field[$id[$c]]["value"][$sourceC]; //set identifiers

        $this->field["sync_status"]["value"][$this->vmax]="unchanged"; //status
        foreach(array_keys($source->field) as $key)
        {
         if(isset($source->field[$key]["value"][$sourceC])&&isset($target->field[$key]["value"][$targetC])&&$source->field[$key]["value"][$sourceC]!=$target->field[$key]["value"][$targetC])
         {
          $this->field["sync_status"]["value"][$this->vmax]="changed"; //status
          break;
         }
        }

        //default action if not set already (from POST)
        if(!isset($this->field["sync_action"]["value"][$this->vmax])||!$this->field["sync_action"]["value"][$this->vmax])
        {
         if(request("default_edit")==="0") //1 and false do default action
          $this->field["sync_action"]["value"][$this->vmax]="nothing";
         else
         $this->field["sync_action"]["value"][$this->vmax]=$this->syncOptions[$this->extra][$this->field["sync_status"]["value"][$this->vmax]][0];
        }

        $this->vmax++;
       }
       else
       {
        $this->field["nr"]["value"][$this->vmax]=Array();
        for($c=0;$c<$idS;$c++)
         $this->field["nr"]["value"][$this->vmax][$id[$c]]=$source->field[$id[$c]]["value"][$sourceC]; //set identifiers

        for($k=0;$k<$timeS;$k++) //check time fields (updated)
        {
         if($source->field[$time[$k]]["value"][$sourceC]==$target->field[$time[$k]]["value"][$targetC])  //matches, meaning unchanged
         {
          $this->field["sync_status"]["value"][$this->vmax]="unchanged"; //status

          //default action if not set already (from POST)
          if(!isset($this->field["sync_action"]["value"][$this->vmax])||!$this->field["sync_action"]["value"][$this->vmax])
           $this->field["sync_action"]["value"][$this->vmax]=$this->syncOptions[$this->extra][$this->field["sync_status"]["value"][$this->vmax]][0];

          $this->vmax++;
         }
         else //check newer/older
         {
          if(DT::dateConvert($source->field[$time[$k]]["value"][$sourceC], $source->format[$source->dataformat]["datetimeformat"])<DT::dateConvert($target->field[$time[$k]]["value"][$targetC], $target->format[$target->dataformat]["datetimeformat"]))
          {
           $this->field["sync_status"]["value"][$this->vmax]="target is newer"; //status

           //default action if not set already (from POST)
           if(!isset($this->field["sync_action"]["value"][$this->vmax])||!$this->field["sync_action"]["value"][$this->vmax])
           {
            if(request("default_edit")==="0") //1 and false do default action
             $this->field["sync_action"]["value"][$this->vmax]="nothing";
            else
             $this->field["sync_action"]["value"][$this->vmax]=$this->syncOptions[$this->extra][$this->field["sync_status"]["value"][$this->vmax]][0];
           }

           $this->vmax++;
          }
          else
          {
           $this->field["sync_status"]["value"][$this->vmax]="source is newer"; //status

           //default action if not set already (from POST)
           if(!isset($this->field["sync_action"]["value"][$this->vmax])||!$this->field["sync_action"]["value"][$this->vmax])
           {
            if(request("default_edit")==="0") //1 and false do default action
             $this->field["sync_action"]["value"][$this->vmax]="nothing";
            else
             $this->field["sync_action"]["value"][$this->vmax]=$this->syncOptions[$this->extra][$this->field["sync_status"]["value"][$this->vmax]][0];
           }

           $this->vmax++;
          }
         }
        }
       }
      }
     }

     if(!$foundID) //id not found in target so it doesn't exist
     {
      $this->field["nr"]["value"][$this->vmax]=Array();
      for($c=0;$c<$idS;$c++)
       $this->field["nr"]["value"][$this->vmax][$id[$c]]=$source->field[$id[$c]]["value"][$sourceC]; //set identifiers

      $this->field["sync_status"]["value"][$this->vmax]="not in target"; //status

      //default action if not set already (from POST)
      if(!isset($this->field["sync_action"]["value"][$this->vmax])||!$this->field["sync_action"]["value"][$this->vmax])
      {
       if(($this->extra=="source"&&request("default_delete")==="0")||($this->extra=="target"&&request("default_add")==="0")) //1 and false do default action
        $this->field["sync_action"]["value"][$this->vmax]="nothing";
       else
        $this->field["sync_action"]["value"][$this->vmax]=$this->syncOptions[$this->extra][$this->field["sync_status"]["value"][$this->vmax]][0];
      }

      $this->vmax++;
     }
   }

   //check the target for new items
   for($targetC=0;$targetC<$target->vmax;$targetC++) //loop through target results
   {
    $foundID=false;

     for($sourceC=0;$sourceC<$source->vmax;$sourceC++)  //check all source results
     {
      $c=0;
      if(isset($source->field[$id[$c]]["value"][$sourceC])&&$source->field[$id[$c]]["value"][$sourceC]==$target->field[$id[$c]]["value"][$targetC])
      {
       $foundID=true;
       for(;$c<$idS;$c++) //check other ID's
        if(!isset($source->field[$id[$c]]["value"][$sourceC])||$source->field[$id[$c]]["value"][$sourceC]!=$target->field[$id[$c]]["value"][$targetC])
         $foundID=false; //doesn't match other ID's

       if(!$foundID)
        continue;

       break;
      }
     }

     if(!$foundID) //id not found in source so it doesn't exist
     {
      $this->field["nr"]["value"][$this->vmax]=Array();
      for($c=0;$c<$idS;$c++)
       $this->field["nr"]["value"][$this->vmax][$id[$c]]=$target->field[$id[$c]]["value"][$targetC]; //set identifiers

      $this->field["sync_status"]["value"][$this->vmax]="not in source"; //status

      //default action if not set already (from POST)
      if(!isset($this->field["sync_action"]["value"][$this->vmax])||!$this->field["sync_action"]["value"][$this->vmax])
      {
       if(($this->extra=="source"&&request("default_add")==="0")||($this->extra=="target"&&request("default_delete")==="0")) //1 and false do default action
        $this->field["sync_action"]["value"][$this->vmax]="nothing";
       else
        $this->field["sync_action"]["value"][$this->vmax]=$this->syncOptions[$this->extra][$this->field["sync_status"]["value"][$this->vmax]][0];
      }

      $this->vmax++;
     }
   }

   return true;
  }


 }

?>
